(() => {
    'use strict';

    angular.module('noc').factory('Messageable', ($timeout, $rootScope, $state, $log) => {
        return function () {
            const scope = this;

            scope.alertMessages = {};

            scope.alertMessages.successes = [];
            scope.alertMessages.inProgress = [];
            scope.alertMessages.errors = [];

            scope.errorsMap = {};

            function pushMsg(list,msg){
                list.push(msg);
                $rootScope.$broadcast("header-change-height");
            }

            scope.goToTimeline = function () {
                $state.go("main.timeline"); //Assuming the base path is always "main".
            };

            scope.pushTimedMessage = function (message, queueName) {
                pushMsg(scope.alertMessages[queueName], message);

                message.duration = message.duration || 6000;
                $timeout(function () {
                    scope.alertMessages[queueName] = _.without(scope.alertMessages[queueName], message);
                    $rootScope.$broadcast("header-change-height");
                }, message.duration);
            };

            scope.pushSuccess = function (message) {
                scope.pushTimedMessage({message: message}, "successes");
            };

            scope.pushInProgress = function (message, seeTimeline, duration) {
                scope.pushTimedMessage({message: message, seeTimeline: seeTimeline, duration: duration}, "inProgress");
            };

            scope.pushError = function(message, errorResponse, callerKey, callerType){
                $log.error(message);

                if(errorResponse && errorResponse.status === "NETWORK_FAILURE"){
                    callerKey = "NETWORK";
                    callerType = "NETWORK";
                    message = errorResponse.message;
                }

                var caller = scope.errorsMap[callerKey];
                var error = null;
                if (caller) {
                    error = caller[callerType];
                } else {
                    caller = scope.errorsMap[callerKey] = {};
                }

                if (error) {
                    if (!error.dismissed) error.message = message;
                } else {
                    error = {
                        dismissed: false,
                        message: message,
                        dismiss: (force) => {
                            if (force) delete caller[callerType];
                            else caller[callerType].dismissed = true;
                            scope.alertMessages.errors = _.without(scope.alertMessages.errors, error);
                            $rootScope.$broadcast('header-change-height');
                            if (scope.alertMessages.errors.length <= 1) scope.alertMessages.showErrors = false;
                        }
                    };
                    caller[callerType] = error;

                    pushMsg(scope.alertMessages.errors, error);
                }
            };

            scope.alertMessages.dismissAllErrors = function(){
                _.forEach(scope.alertMessages.errors, function(error){
                    error.dismiss();
                });
                scope.alertMessages.showErrors = false;
            };

            scope.clearMessages = function(callerKey, callerType){
                var caller = scope.errorsMap[callerKey];
                if(caller){
                    if(callerType){
                        var error = caller[callerType];
                        if(error) error.dismiss(true);
                    }else{
                        _.forOwn(caller, function(error){
                            error.dismiss(true); //force the deletion of the entry
                        });
                    }
                }

                if(callerType){
                    //Always attempt to clear network related alertMessages if there was a callerType, since callerTypes are passed in on success.
                    scope.clearMessages("NETWORK");
                }
            };
        };
    });
})();
