(function () {
    "use strict";

    function updateBarChart(data, chart){
        addPatternToCurrentMonth(data);

        chart.series[0].setData(data.storageValues, false);
        chart.series[1].setData(data.computeValues, false);
        chart.xAxis[0].setCategories(data.categories, false);
        chart.reflow();
        chart.redraw();
    }

    function updatePieChart(data, chart){
        chart.series[0].setData(data.data, false);
        chart.redraw();
    }

    function addPatternToCurrentMonth(data){
        if(data.storageValues.length > 0){
            _.last(data.storageValues).color = "url(#patternBlue)";
            _.last(data.computeValues).color = "url(#patternOrange)";
        }
    }

    function constructBarChart(data, numberFormatter){
        addPatternToCurrentMonth(data);

        return new Highcharts.Chart({
            chart: {
                type: 'column',
                backgroundColor: 'transparent',
                marginBottom: 85,
                spacingBottom: 0,
                marginTop: 70,
                renderTo: "costBarHighchart",
                style:{
                    fontFamily: '"Open Sans", sans-serif'
                }
            },
            colors: ['#226db6', '#f2642d'],
            credits: false,
            title: {text: ''},
            xAxis: {
                gridLineColor: 'transparent',
                categories: data.categories,
                labels: {
                    style: {
                        color: "#5a5a5a",
                        'font-weight': 'normal',
                        'font-size': "14px"
                    },
                    y: 27
                },
                tickWidth: 0,
                lineColor: "#e7e7e8",
                lineWidth: 4,
                offset: 17
            },
            yAxis: {
                min: 0,
                gridLineColor: 'transparent',
                title: {text: null},
                labels: {
                    enabled: false
                },
                stackLabels: {
                    enabled: true,
                    y: -36,
                    formatter: function() {
                        var isToDate = this.x === data.storageValues.length - 1;
                        var toDateText = isToDate ? "<div class='to-date'> to date</div>" : "";
                        var toDateClass = isToDate ? "to-date-box" : '';
                        var storageSubDiv = "<div class='storage sub-total'><div class='circle'></div>$" + numberFormatter(this.points['0,' + this.x + ',0'][1] - this.points['0,' + this.x + ',0'][0], 2) + "</div>";
                        var computeSubDiv = "<div class='compute sub-total'><div class='circle'></div>$" + numberFormatter(this.points['1,' + this.x + ',0'][1] - this.points['1,' + this.x + ',0'][0], 2) + "</div>";
                        return "<div class='stack-label " + toDateClass +"'><div class='inner'><div class='total'>$"+ numberFormatter(this.total, 2)+"</div>"+toDateText+ "<div>" + storageSubDiv + computeSubDiv + "</div></div><div class='chupchik'></div></div>";
                    },
                    useHTML: true
                }
            },
            legend: {
                align: 'center',
                verticalAlign: 'bottom',
                y: -5,
                floating: true,
                borderColor: null,
                borderWidth: 0,
                shadow: false,
                symbolRadius: 5,
                symbolHeight: 10,
                symbolWidth: 10,
                itemStyle: {
                    color: "#747474",
                    'font-size': "13px",
                    'font-weight': "normal",
                    cursor: 'default'
                },
                itemHoverStyle: {
                    color: '#747474'
                }
            },
            tooltip: {
                enabled: false
            },
            plotOptions: {
                column: {
                    stacking: 'normal',
                    borderWidth: 0,
                    pointWidth: 60,
                    dataLabels: {
                        enabled: false
                    },
                    events: {
                        legendItemClick: function(e){
                            return !e.target.visible || _.reduce(this.chart.series, function(sum, serie) {
                                return sum + (serie.visible ? 1 : 0);
                            }, 0) > 1;
                        }
                    },
                    point: {
                        events: {
                            mouseOver: function(){
                                $(this.series.yAxis.stacks.column[this.x].label.element).find(".stack-label").addClass("expanded");
                            },
                            mouseOut: function(){
                                $(this.series.yAxis.stacks.column[this.x].label.element).find(".stack-label").removeClass("expanded");
                            }
                        }
                    }
                }
            },
            series: [
                {
                    name: 'Storage',
                    data: data.storageValues,
                    legendIndex: 0
                },{
                    name: 'Compute',
                    data: data.computeValues,
                    legendIndex: 1
                }
            ]
        });
    }

    function constructPieChart(data){
        return new Highcharts.Chart({
            chart: {
                type: 'pie',
                backgroundColor: 'transparent',
                renderTo: "costPieHighchart"
            },
            credits: false,
            title: {text: ''},
            plotOptions: {
                pie: {
                    borderColor: '#ececec',
                    shadow: false,
                    center: ['50%', '50%'],
                    startAngle: 0,
                    dataLabels: {
                        enabled: false
                    }
                },
                series:{
                    states: {
                        hover: {
                            halo: {
                                attributes: {
                                    fill: '#fbfbfb'
                                }
                            }
                        }
                    }
                }
            },
            tooltip: {
                enabled: false
            },
            series: [{
                name: 'Capacity',
                data: data.data,
                size: '75%',
                innerSize: '63%'
            }]
        });

    }

    angular.module('noc').component('costLegacyComponent', {
        bindings: {
            costResponse: '='
        },
        templateUrl: 'assets/templates/components/cost/cost-legacy-component.html',
        controller: function (workingEnvironmentResource, $timeout, $filter, nocDialog) {
            this.showCalculation = function(){
                nocDialog.dialog("", "assets/templates/dialogs/cost-calculation.html", {costResponse: this.costResponse, savePercentage: this.savePercentage});
            };

            this.constructPieData = function(costResponse){
                var ebsPercent = costResponse.storage.ebsSize.size / costResponse.storage.requestedSize.size;
                var data = [
                    {y: costResponse.storage.ebsSize.size, color: 'url(#img1)', name: "EBS Capacity"},
                    {y: costResponse.storage.requestedSize.size - costResponse.storage.ebsSize.size, color: '#226db6', name: "Required Capacity"}
                ];

                return {ebsPercent: ebsPercent, data: data};
            };

            this.constructBarData = function(costResponse){
                var categories = _.map(costResponse.costs, function(cost){
                    return moment({year: cost.year, month: cost.month - 1}).format('MMMM YYYY');
                });

                var storageValues = _.map(costResponse.costs, function(cost){
                    return {y: cost.storageCost.amount};
                });

                var computeValues = _.map(costResponse.costs, function(cost){
                    return {y: cost.computeCost.amount};
                });

                return {categories: categories, storageValues: storageValues, computeValues: computeValues};
            };


            this.handleOldCostCharts = function() {
                //put a mocked cost in case the cost is empty and show it dimmed
                if (this.costResponse.costs.length === 0) {
                    var year = new Date().getFullYear();
                    this.costResponse.costs = [{
                        "month": 1,
                        "year": year,
                        "lastModified": 1400134804000,
                        "computeCost": {"amount": 250.0, "currency": "USD"},
                        "storageCost": {"amount": 175.0, "currency": "USD"}
                    },
                        {
                            "month": 2,
                            "year": year,
                            "lastModified": 1400134804000,
                            "computeCost": {"amount": 300.0, "currency": "USD"},
                            "storageCost": {"amount": 250.0, "currency": "USD"}
                        },
                        {
                            "month": 3,
                            "year": year,
                            "lastModified": 1400134804000,
                            "computeCost": {"amount": 400.0, "currency": "USD"},
                            "storageCost": {"amount": 400.0, "currency": "USD"}
                        },
                        {
                            "month": 4,
                            "year": year,
                            "lastModified": 1400134804000,
                            "computeCost": {"amount": 700.0, "currency": "USD"},
                            "storageCost": {"amount": 550.0, "currency": "USD"}
                        },
                        {
                            "month": 5,
                            "year": year,
                            "lastModified": 1400134804000,
                            "computeCost": {"amount": 900.0, "currency": "USD"},
                            "storageCost": {"amount": 700.0, "currency": "USD"}
                        },
                        {
                            "month": 6,
                            "year": year,
                            "lastModified": 1400134804000,
                            "computeCost": {"amount": 1000.0, "currency": "USD"},
                            "storageCost": {"amount": 850.0, "currency": "USD"}
                        }];
                    this.costResponse.empty = true;
                }

                this.costResponse.costs = _.takeRight(this.costResponse.costs, 6);

                this.thisMonth = _.last(this.costResponse.costs);
                this.weDontSaveShit = this.costResponse.storage.requestedSize ? this.costResponse.storage.ebsSize.size >= this.costResponse.storage.requestedSize.size : true;
                if (!this.weDontSaveShit) {
                    this.savePercentage = 100 - (this.costResponse.storage.ebsSize.size / this.costResponse.storage.requestedSize.size) * 100;

                    var pieData = this.constructPieData(this.costResponse);
                    this.ebsPercent = pieData.ebsPercent;
                    if (!this.pieChart) {
                        $timeout(function () {
                            this.pieChart = constructPieChart(pieData);
                        });
                    } else {
                        $timeout(function () {
                            updatePieChart(pieData, this.pieChart);
                        });
                    }
                }

                var barData = this.constructBarData(this.costResponse);

                if (!this.barChart) {
                    $timeout(function () {
                        this.barChart = constructBarChart(barData, $filter('number'));
                    }.bind(this));
                } else {
                    $timeout(function () {
                        updateBarChart(barData, this.barChart);
                    }.bind(this));
                }
            };

            this.$onInit = function () {
                this.savePercentage = undefined;
                this.handleOldCostCharts();
            };
        }
    });
}());
