angular.module('noc').component('costMonthlyBreakdownComponent', {
    bindings: {
        annualData: '<'
    },
    templateUrl: 'assets/templates/components/cost/cost-monthly-breakdown-component.html',
    controllerAs: 'ctrlCostMonthlyBreakdown',
    controller: function ($filter) {

        this.$onInit = () => {
            this.selectedIndex = 0;
            this.selectOptions = getOptionsForSelect(this.annualData);
            this.selectedOption = this.selectOptions[0];
            this.onChangeMonthSelection();
        };

        //Order is for the pie chart
        const AwsCategories = [{
            name: 'EC2',
            label: 'Compute',
            order: '1'
        }, {
            name: 'EBS',
            label: 'EBS Storage',
            order: '2'
        }, {
            name: 'S3',
            label: 'S3 Cost',
            order: '3'
        }, {
            name: 'DataTransfer',
            label: 'Data Transfer',
            order: '4'
        }];

        // CostByService is the raw data as received from the server
        // This function builds data as needed by the pie-chart - AwsCategories specifies the order and the labels
        const constractCostDataForChart = (costByService) => {
            const costDataForChart = [];
            _.forEach(AwsCategories, (category) => {
                let elem = _.find(costByService, {'name': category.name});
                if (elem === undefined) elem = {total: {amount: 0}};
                costDataForChart.push({name: category.label, y: elem.total.amount});
            });

            return costDataForChart;
        };

        // Sort the EBS disk types so they'll always be rendered in the same order: Alphabetically, while "Snapshots" and "Other" always last
        const sortEbsCostByUsage = (costByService) => {
            costByService.costByUsage = _.sortBy(costByService.costByUsage, (type) => {
                return _.toLower(type.name);
            });
            const typeSnapshots = _.remove(costByService.costByUsage, {name: 'Snapshots'});
            costByService.costByUsage = _.concat(costByService.costByUsage, typeSnapshots);
            const typeOther = _.remove(costByService.costByUsage, {name: 'Other'});
            costByService.costByUsage = _.concat(costByService.costByUsage, typeOther);
        };

        const getDaysCountInMonth = (selectedIndex) => {
            const startDate = moment(this.annualData[selectedIndex].start, 'YYYY-MM-DD');
            const endDate = moment(this.annualData[selectedIndex].end, 'YYYY-MM-DD');

            return endDate.diff(startDate, 'days');
        };

        //The select droplist only shows the months that are not empty from the annualData array, sorted descending
        const getOptionsForSelect = (data) => {
            const nonEmptyStartDates = _.reduce(data, (accumulator, monthData) => {
                if (monthData.total.amount > 0) {
                    accumulator.push(monthData.start);
                }

                return accumulator;
            }, []);

            return _.map(nonEmptyStartDates.sort().reverse(), (start) => getOptionFromStartDate(start));
        };

        const getOptionFromStartDate = (startDate) => {
            const monthName = moment(startDate, 'YYYY-MM-DD').format('MMMM');
            const year = moment(startDate, 'YYYY-MM-DD').format('YYYY');

            return {value: startDate, label: `${monthName} ${year}`};
        };

        this.getCategoryIndex = (categoryName) => {
            return _.findIndex(this.monthData.costByService, {'name': categoryName});
        };

        const getSelectedMonthIndex = (selectedOptionValue) => {
            return _.findIndex(this.annualData, {'start': selectedOptionValue});
        };

        this.onChangeMonthSelection = () => {
            if (this.annualData && this.annualData.length > 0) {
                const startDate = this.selectedOption.value;
                this.selectedMonthName = moment(startDate, 'YYYY-MM-DD').format('MMMM');
                const selectedMonthIndex = getSelectedMonthIndex(this.selectedOption.value); // SelectedMonthIndex is the index of the selected month in the annualData array
                this.daysInMonth = getDaysCountInMonth(selectedMonthIndex);
                this.monthData = this.annualData[selectedMonthIndex];
                this.formattedTotalAmount = `$ ${$filter('number')(this.monthData.total.amount || 0, 0)}`;
                this.costDataForChart = constractCostDataForChart(this.monthData.costByService);
                if (this.monthData.costByService[this.getCategoryIndex('EBS')]) {
                    sortEbsCostByUsage(this.monthData.costByService[this.getCategoryIndex('EBS')]);
                }
            }
        };
    }
});
