angular.module('noc.components').directive('nocPopover', ($document, $timeout, $rootScope, $compile) => {
    'use strict';

    $rootScope.popoverHelper = {};
    let popover = null;
    let popoverParent = null;
    let existingPopoverSrc = null;
    let existingPopoverElement = null;

    const removePopover = () => {
        delete $rootScope.popoverHelper.key;
        if (popover) {
            const deadPopoverRef = popover;
            deadPopoverRef.addClass('exit');
            $timeout(() => {
                deadPopoverRef.remove();
            }, 300);
        }
        popover = null;
        existingPopoverElement = null;
        existingPopoverSrc = null;
        $rootScope.popoverHelper = {
            popoverApi: {
                remove: removePopover
            }
        };
    };

    const handleOverflow = (offset) => {
        popover.removeClass('right');

        const tooCloseThreshold = 45;
        const endOfScreen = $(window).height();
        const popoverContent = popover.find('div').first();
        if (popoverContent.length < 1) return;
        const bottomOfPopover = offset.top + popoverContent.height();
        if (endOfScreen - bottomOfPopover < tooCloseThreshold) {
            popover.addClass('right');
        }
    };

    const popoverHelperDiv = $('.popover-helper');

    return {
        restrict: 'A',
        scope: {
            'nocPopover': '='
        },
        link: function link(scope, element, attributes) {
            const popoverSrc = scope.nocPopover;
            const attachToParent = attributes.popoverContainer === 'parent';
            const positionBy = attributes.positionBy === 'parent';
            const avoidPosition = attributes.avoidPosition === 'true';
            const clickable = attachToParent ? element.parent().parent() : element;
            const showOnHover = attributes.showOnHover;

            scope.$parent.test = () => {
                $rootScope.popoverHelper.loaded = true;
            };

            function showPopover(event) {
                scope.$apply(() => {
                    if (popoverSrc === existingPopoverSrc && element === existingPopoverElement) {
                        return;
                    }
                    event.stopPropagation();
                    removePopover();
                    existingPopoverSrc = popoverSrc;
                    existingPopoverElement = element;

                    $rootScope.popoverHelper.continue = () => {
                        $timeout(() => {
                            if (positionBy || attachToParent) {
                                element.parent().append(popover);
                            }

                            if (!attachToParent) {
                                const offset = positionBy ? popover.parent().offset() : popoverParent.offset();
                                handleOverflow(offset);
                                popover.appendTo(popoverHelperDiv);
                                popover.css('top', offset.top).css('left', offset.left);
                            }

                            $timeout(() => {
                                popover.removeClass('enter');
                            });
                            const body = $document.find('body');

                            const generalSlaughter = (doNotApply) => {
                                if (doNotApply) removePopover();
                                else scope.$apply(removePopover);

                                $document.off('keydown', keyPressKiller);
                                body.off('click', clickKiller);
                                if (!attachToParent) {
                                    $(window).off('resize', generalSlaughter);
                                }
                            };

                            $rootScope.generalSlaughter = generalSlaughter;

                            const keyPressKiller = (event) => {
                                if (event.keyCode !== 8) return;
                                generalSlaughter();
                            };

                            const clickKiller = (event) => {
                                if (event.target.className === 'noc-close') return;
                                if ($(event.target).closest('.popover').length > 0) return;
                                if ($(event.target).closest('.popover2').length > 0) return;
                                if ($(event.target).parents('div.info').length > 0) return;
                                generalSlaughter();
                            };

                            $document.on('keydown', keyPressKiller);
                            body.on('click', clickKiller);

                            scope.$on('$destroy', () => {
                                generalSlaughter(true);
                            });

                            if (!attachToParent) {
                                $(window).on('resize', generalSlaughter);
                            }
                        });
                    };

                    popover = $compile('<div class="noc-popover"><ng-include onload="$root.popoverHelper.continue()" src="\'' + popoverSrc + '\'"></ng-include></div>')(scope.$parent);
                    if (!avoidPosition) {
                        popover
                            .css('position', 'absolute')
                            .css('top', 0)
                            .css('left', 0)
                            .addClass('enter');
                    }

                    popoverParent = element.parent();
                });
            }

            clickable.click(showPopover);
            if (showOnHover) clickable.hover(showPopover, removePopover);
        }
    };
});
