angular.module('noc').component('portalSetupComponent', {
    bindings: {
        name: '<',
        email: '<',
        ic: '<',
        finishSetup: '&'
    },
    controller: function controller(nocUIMain, nocDialog, tenancyResource, Patterns, $q) {
        this.$onInit = () => {
            this.accounts = tenancyResource.getAccounts();
            this.emailDomain = _.startCase(this.email.substring(this.email.indexOf('@') + 1, this.email.indexOf('.')));
            this.noAccounts = !this.accounts || this.accounts.length === 0;
            this.selectedAccount = this.selectedAccount || ((this.accounts && this.accounts[0]) ? this.accounts[0] : {accountName: this.emailDomain});
            this.newAccount = {name: (this.noAccounts ? this.selectedAccount.accountName : '')};
            this.origianlAccount = this.selectedAccount;
            this.needToCreateNewAccount = this.noAccounts;
            this.selectedAccountName = this.selectedAccount.accountName;
        };

        this.createAccountSetup = (accountName) => {
            const unloading = nocUIMain.loading();

            return tenancyResource.createNewAccount(accountName)
                .then((account) => {
                    this.selectedAccount.accountPublicId = account.data.accountPublicId;
                    this.selectedAccount.accountName = accountName;
                    unloading();

                    return this.selectedAccount.accountPublicId;
                });
        };

        this.switchAccount = () => {
            this.noAccountsError = false;

            nocDialog.confirm('', 'assets/templates/dialogs/setup-switch-account.html', {
                accounts: this.accounts,
                dialogSelectedAccount: {account: this.selectedAccount},
                createNewAccount: {value: this.needToCreateNewAccount},
                newAccount: this.newAccount,
                noAccounts: this.noAccounts,
                originalAccount: this.origianlAccount,
                width: 510,
                height: 330
            }).then((dialogScope) => {

                this.origianlAccount = this.selectedAccount;
                if (dialogScope.createNewAccount.value) {
                    this.newAccount.name = dialogScope.newAccount.name;
                    this.needToCreateNewAccount = true;
                    this.selectedAccountName = dialogScope.newAccount.name;
                } else {
                    this.needToCreateNewAccount = false;
                    this.newAccount.name = '';
                    this.selectedAccount = dialogScope.dialogSelectedAccount.account;
                    this.selectedAccountName = this.selectedAccount.accountName;
                }
                this.needToCreateNewAccount = dialogScope.createNewAccount.value;
            })
                .catch(() => {
                    if (!this.noAccounts) {
                        this.newAccount.name = '';
                    }

                });
        };

        this.isSiteNameValid = () => {
            if (!this.site) return false;
            const emailRegex = new RegExp(Patterns.siteName.regex);

            return this.site.length > 0 && emailRegex.test(this.site);
        };

        this.finishPortalSetup = () => {
            if (this.signupForm.$valid) {
                let createAccountP = !this.needToCreateNewAccount ? () => $q.resolve(this.selectedAccount.accountPublicId) : () => this.createAccountSetup(this.newAccount.name);
                createAccountP().finally(() => {
                    this.finishSetup({site: this.site, accountId: this.selectedAccount.accountPublicId});
                });

            } else {
                nocUIMain.handleFormValidity();
            }
        };
    },
    templateUrl: 'assets/templates/components/portal-setup-component.html'
});
