angular.module('noc.components').component('tieringPolicyComponent', {
    bindings: {
        disabled: '<',
        objectStorageByProvider: '<',
        policies: '<',
        selectedPolicy: '=',
        tierLevel: '<',
        tierLevelDisabled: '<'
    },

    controller: function () {
        this.tieringPolicyMessages = {
            'auto': 'Tiers cold Snapshot copies and cold user data from the active file system to object storage.',
            'snapshot_only': 'Tiers cold Snapshot copies to object storage. ',
            'none': 'Data tiering is disabled.'
        };

        this.tieringPolicyName = {
            'auto': 'Auto',
            'snapshot_only': 'Snapshot Only',
            'none': 'None'
        };

        this.$onInit = () => {
            this.isOpen = false;
            this.objectStorage = this.objectStorageByProvider;
            this.isAuto = _.includes(this.policies, 'auto');

            if (!this.selectedPolicy || !_.includes(this.policies, this.selectedPolicy.type)) {
                this.selectedPolicy = {
                    type: _.includes(this.policies, 'auto') ? 'auto' : (_.includes(this.policies, 'snapshot_only') ? 'snapshot_only' : 'none')
                };
            }

            this.moreInfoParameters = {tieringLevel: this.tierLevel};
        };

        this.toggle = () => {
            this.showAuto = _.includes(this.policies, 'auto');
            this.showSnapshot = _.includes(this.policies, 'snapshot_only');
            this.showNone = _.includes(this.policies, 'none');
            if (this.disabled) return;
            this.isOpen = !this.isOpen;
        };

        this.shortInfo = () => {
            return this.disabled ? 'Disabled - ' + this.disabled : this.tieringPolicyName[this.selectedPolicy.type] + ' - ' + this.tieringPolicyMessages[this.selectedPolicy.type];
        };

        this.tierLevelFunction = () => {
            if (this.tierLevel === 'normal' || !this.tierLevel) {
                return _.includes(this.objectStorage, 'S3') ? 'Standard' : 'Hot';
            } else {
                if(this.tierLevel === 'ia') {
                    return 'Standard-Infrequent Access';
                } else if(this.tierLevel === 'ia-single') {
                    return 'One Zone-Infrequent Access';
                } else if(this.tierLevel === 'intelligent') {
                    return 'Intelligent-Tiering';
                } else {
                    return _.capitalize(this.tierLevel);
                }
            }
        };

        this.disabledFunction = () => {
            if (this.disabled) {
                this.isOpen = false;
            }
            return this.disabled;
        };

    },
    controllerAs: 'ctrlTiering',
    templateUrl: 'assets/templates/components/tiering-policy-component.html',
    transclude: true
});
