(() => {
    'use strict';

    angular.module("noc.components").directive('form', function() {
        return {
            restrict: 'E',
            link: function(scope, element) {
                element.attr("novalidate", "novalidate");
            }
        };
    });

    const validationClassToMessageTypes = ['password-verify', 'email', 'pattern', 'required', 'rangeVerify', 'url',
        'password-is-required', 'username-is-required', 'unique', 'noc-tags-validation', 'noc-tags-reserved-keys',
        'unique-volume-name-validation', 'eval-serial-number-validation'];

    const extractMessageForElement = (element, Messages, Patterns, scope) => {
        let message = '';

        const type = _.find(validationClassToMessageTypes, (type) => {
            return element.hasClass('ng-invalid-' + type) && element.attr('data-error-message-' + type);
        });
        let key = '';
        if (type) {
            key = element.attr('data-error-message-' + type);
        } else {
            key = element.attr('data-error-message-any');
        }

        if (key) message = Patterns[key] ? Patterns[key].message : Messages.errors[key];

        if (_.isFunction(message)) {
            const params = scope.$eval(element.attr('data-error-params'));
            message = message(params);
        }

        if (!message) {
            _.forEach(validationClassToMessageTypes, (type) => {
                if (element.hasClass('ng-invalid-' + type)) {
                    message = Messages.errors[type];
                    if (message) return false;
                }

                return true;
            });
        }

        return message || Messages.errors.invalid;
    };

    var validationDirective = function (Messages, $timeout, Patterns) {
        return {
            restrict: 'EA',
            require: '^?ngModel',
            link: function (scope, element, attr, ctrl) {
                if(!element.attr("ng-model") && !element.attr("data-ng-model") && !element.is("div")) return;
                if (element.siblings(".form-control-feedback").length) return; //Don't add it twice.

                var popover;

                var repositionPopover;

                var addPopover = function() {
                    //NOC-10293 Remove tooltip "this field is required"
                    if (element.hasClass("ng-invalid-required")) return;
                    if (!popover) {
                        popover = $('<div class="popover error"><div class="arrow"></div><div class="popover-content">' + extractMessageForElement(element, Messages, Patterns, scope) + ' </div></div>');
                        popover.css("width", "250px");

                        repositionPopover = function(){
                            var body = $("body");
                            var docWidth = body.width();
                            var position;
                            if (element.hasClass('popover-bottom')) {
                                position = 'bottom';
                            } else if (element.offset().left + element.outerWidth(true) + 265 <= docWidth) {
                                position = "right";
                            } else if (element.offset().left - 265 > 0) {
                                position = "left";
                            } else {
                                position = 'bottom';
                            }

                            popover.removeClass("bottom right left").addClass(position);

                            element.parent().append(popover);
                            popover.css("top", "");
                            popover.css("left", "");

                            var topOffset = (popover.parents(".form-horizontal").length === 0 && popover.parents(".no-label").length === 0) ? 37 : 15;
                            topOffset += popover.parents('.virgin-layout').length === 0 ? 0 : 5;
                            topOffset += position === 'bottom' ? popover.parents().innerHeight() - 10 : 0;
                            popover.css("top", (topOffset - popover.outerHeight(true) / 2));
                            var addLeft = (popover.parents(".settings-input").length === 0 ? 0 : 40);
                            addLeft += position === 'bottom' ? (popover.parents().outerWidth() - 140) : 0;

                            $timeout(function(){
                                if(!popover) return; //Depending on timing, in certain cases the popover could have been removed before the timeout is triggered, so just ignore
                                var offset = popover.offset();
                                popover.appendTo("body");
                                popover.css("top", offset.top);
                                popover.css("left", offset.left + addLeft);
                            });
                        };
                        repositionPopover();

                        $(window).on("resize", repositionPopover);
                    }
                };

                var removePopover = function() {
                    if (popover){
                        popover.remove();
                        popover = null;
                        $(window).off("resize", repositionPopover);
                    }
                };

                var div = element.parent();
                div.css("position", "relative");

                if (element.siblings('label').length === 0) {
                    element.parent().addClass('no-label');
                }

                if (element.is('select')) {
                    element.parent().addClass('on-select');
                }

                element.blur(() => {
                    div.removeClass('focus');
                    removePopover();
                    //Bug fix  NOC-2816 Fields validation error briefly appears when Save is clicked, add the class visited only after a timeout, so click on cancel won't show the error for a second
                    $timeout(() => {
                        div.addClass('visited');
                    }, 300);
                });

                element.focus(function(){
                    div.addClass("focus");
                    if(element.hasClass('ng-invalid') && element.parents(".visited").length > 0){
                        addPopover();
                    }
                });

                div.append('<span class="form-control-feedback"><i class="fa fa-exclamation-circle"></i></span>');

                div.find(".form-control-feedback i").on("click", function(){
                    element.focus();
                });

                scope.$watch(
                    () => _.get(ctrl, '$invalid'),
                    (value) => {
                        if (value) {
                            div.addClass('noc-error');
                            if (div.hasClass('focus') && element.parents('.visited').length > 0) {
                                addPopover();
                            }
                        } else {
                            div.removeClass('noc-error');
                            removePopover();
                        }
                    });

                scope.$on('$destroy', function() {
                    removePopover();
                });
            }
        };
    };

    angular.module("noc.components").directive('input', validationDirective);
    angular.module("noc.components").directive('select', validationDirective);
    angular.module("noc.components").directive('nocDivVerify', validationDirective);

    angular.module("noc.components").directive("nocValidation", function ($compile, Patterns) {
        return {
            priority: 1001, // compiles first
            terminal: true, // prevent lower priority directives to compile after it
            compile: function (element) {
                var key = element.attr("noc-validation");
                element.removeAttr('noc-validation'); // necessary to avoid infinite compile loop
                if (key && Patterns[key]) {
                    element.attr("ng-pattern", new RegExp(Patterns[key].regex));
                    element.attr("data-error-message-pattern", key);
                } else {
                    throw new Error("Wrong or missing key for data-error-message-pattern");
                }

                return function (scope, element) {
                    $compile(element)(scope);
                };
            }
        };
    });

    angular.module('noc.components').directive('nocDisableValidation', () => {
        return {
            require: '^form',
            restrict: 'A',
            link: function link(scope, element, attributes, form) {
                if (!element.attr('name')) throw new Error('Form control must have name attribute in order to use nocDisableValidation');
                let control;
                scope.$watch(() => {
                    return scope.$eval(attributes.nocDisableValidation);
                }, (value) => {
                    control = control || form[element.attr('name')];
                    if (value) {
                        form.$removeControl(control);
                        control.$setViewValue('', true);
                    } else {
                        form.$addControl(control);
                    }
                });
            }
        };
    });

    //required when working with multi select on select 2, must be combined with the nocDivVerify (see ports in app\templates\tabs\connections\connections-new-edit.html)
    angular.module("noc.components").directive('nocRequiredMultiple', function() {
        return {
            require: '?ngModel',
            link: function(scope, elm, attr, ctrl) {
                if (!ctrl) return;
                attr.required = true; // force truthy in case we are on non input element

                var validator = function(value) {
                    if (attr.required && (_.isEmpty(value) || value === false)) {
                        ctrl.$setValidity('required', false);
                    } else {
                        ctrl.$setValidity('required', true);
                        return value;
                    }
                };

                ctrl.$formatters.push(validator);
                ctrl.$parsers.unshift(validator);

                attr.$observe('required', function() {
                    validator(ctrl.$viewValue);
                });
            }
        };
    });
})();

angular.module('noc.components').directive('nocTagsValidation', (nocUtil) => {
    'use strict';

    return {
        require: '?ngModel',
        scope: {
            nocTagsValidation: '='
        },
        link(scope, element, attrs, ctrl) {
            const same = () => {
                return _.filter(scope.nocTagsValidation.tags, {tagKey: ctrl.$viewValue}).length > 1;
            };

            const containsInvalidKey = () => {
                if (scope.nocTagsValidation.provider === 'AWS') {
                    const forbiddenTagKeys = _.invokeMap(nocUtil.getSystemTags(), 'toLowerCase');

                    return _.includes(forbiddenTagKeys, _.toLower(ctrl.$viewValue));
                } else {
                    return false;
                }
            };

            const check = () => {
                if (_.isEmpty(ctrl.$viewValue) || _.isUndefined(ctrl.$viewValue) || _.isEmpty(scope.nocTagsValidation.tags)) {
                    ctrl.$setValidity('nocTagsValidation', true);
                    ctrl.$setValidity('nocTagsReservedKeys', true);
                } else if (same()) {
                    ctrl.$setValidity('nocTagsValidation', false);
                } else {
                    ctrl.$setValidity('nocTagsValidation', true);
                    ctrl.$setValidity('nocTagsReservedKeys', true);
                }

                if (containsInvalidKey()) {
                    ctrl.$setValidity('nocTagsReservedKeys', false);
                } else {
                    ctrl.$setValidity('nocAwsTagsInvalidKeys', true);
                }
            };

            scope.$watch('nocTagsValidation.tags', check, true);

            scope.$watch(() => {
                return ctrl.$viewValue;
            }, check);
        }
    };
});
