(() => {
    'use strict';

    angular.module('noc').controller('nocNgFormController', function controller(ValidationKeys) {
        var NOC_UNIQUE_VALIDATION_KEY = ValidationKeys.NOC_UNIQUE_VALIDATION_KEY;
        var NOC_AT_LEAST_ONE_SELECTED_VALIDATION_KEY = ValidationKeys.NOC_AT_LEAST_ONE_SELECTED_VALIDATION_KEY;

        var validateUniqueness = function (descriptor, inputNewValue) {
            var inputs = validationData[NOC_UNIQUE_VALIDATION_KEY].inputs;
            // look for all inputs that had the old value
            var sameValueInputs = _.filter(inputs, function (input) {
                return descriptor !== input && descriptor.oldValue === input.modelCtrl.$viewValue;
            });

            // update old value
            descriptor.oldValue = inputNewValue;

            if (sameValueInputs.length === 1) {
                sameValueInputs[0].modelCtrl.$setValidity(NOC_UNIQUE_VALIDATION_KEY, true);
            }

            // check itself for uniqueness
            if (isBlank(inputNewValue)) return descriptor.modelCtrl.$setValidity(NOC_UNIQUE_VALIDATION_KEY, true);

            var foundSameValue = _.some(inputs, function (input) {
                return input !== descriptor && !isBlank(input.modelCtrl.$viewValue) && input.modelCtrl.$viewValue === inputNewValue && !_.startsWith(input.modelCtrl.$viewValue, 'Eval-');
            });

            return descriptor.modelCtrl.$setValidity(NOC_UNIQUE_VALIDATION_KEY, !foundSameValue);
        };

        var validateAtLeastOneSelected = function (descriptor, newValue) {
            var inputs = _.filter(validationData[NOC_AT_LEAST_ONE_SELECTED_VALIDATION_KEY].inputs, "shouldValidate");

            if (newValue && descriptor.shouldValidate) { // should use this input for validation and is true
                return descriptor.formCtrl.$setValidity(NOC_AT_LEAST_ONE_SELECTED_VALIDATION_KEY, true);
            }

            if (inputs.length > 0) {
                descriptor.formCtrl.$setValidity(NOC_AT_LEAST_ONE_SELECTED_VALIDATION_KEY, _.some(inputs, function (input) {
                    return descriptor !== input && input.modelCtrl.$viewValue;
                }));
            } else { // couldn't find an invalid input
                descriptor.formCtrl.$setValidity(NOC_AT_LEAST_ONE_SELECTED_VALIDATION_KEY, true);
            }
        };
        var validationData = {};
        validationData[NOC_UNIQUE_VALIDATION_KEY] = {inputs: [], validator: validateUniqueness};
        validationData[NOC_AT_LEAST_ONE_SELECTED_VALIDATION_KEY] = {inputs: [], validator: validateAtLeastOneSelected};

        var isBlank = function (value) {
            return _.isEmpty(value) || _.isUndefined(value);
        };

        this.addInput = function (descriptor, key) {
            validationData[key].inputs.push(descriptor);
        };

        this.removeInput = function (descriptor, key) {
            _.remove(validationData[key].inputs, descriptor);
        };

        this.validate = function (descritor, newValue, key) {
            validationData[key].validator(descritor, newValue);
        };
    });

    angular.module("noc.components").directive("ngForm", function () {
        return {
            restrict: 'E',
            controller: "nocNgFormController"
        };
    });
})();
