angular.module('noc.components').directive('nocVirtualScroll', ($compile, $http, $templateCache, $timeout, $window, $log) => {
    'use strict';

    const MAX_ITEMS_PER_ROW = 30,
        BUFFER_ROWS = 2,
        VS_COLLECTION_NAME = 'vsCollection';

    return {
        scope: true,
        link: function link(scope, element, attributes) {
            const logger = (message) => {
                $log.debug('VirtualScroll ' + attributes.nocVsRepeaterName + ': ' + message);
            };
            let canvasElement, contentElement;
            scope.templateLoaded = false;
            scope.initialized = false;

            const init = (template) => {
                canvasElement = angular.element("<div class='noc-virtual-scroll'/>");
                contentElement = angular.element("<div class='noc-vs-content' />");
                template = angular.element(template);
                template.attr('ng-repeat', repeaterModelName + ' in ' + VS_COLLECTION_NAME + ' track by $index');
                element.append(canvasElement);
                canvasElement.append(contentElement);
                contentElement.append($compile(template)(scope));
                scope.templateLoaded = true;
            };

            const fillThenCalc = (callback) => {
                if (scope.isInitializing) return;
                if (collection.length === 0) return;
                scope.isInitializing = true;
                const item = collection[0];
                contentElement.css({visibility: 'hidden'});
                scope[VS_COLLECTION_NAME] = [];
                for (let i = 0; i < MAX_ITEMS_PER_ROW; i++) {
                    scope[VS_COLLECTION_NAME].push(item);
                }
                $timeout(() => {
                    if (scope[VS_COLLECTION_NAME].length === 0) return;
                    scope.calcDimensions();
                    if (_.isFunction(callback)) callback();
                    scope.initialized = true;
                    scope.isInitializing = false;
                });
            };

            let rowHeight = 1,
                itemsPerRow = 1,
                canvasHeight,
                rowsInContainer;
            scope.calcDimensions = () => {
                //Items per row, row height, canvas height
                const elements = contentElement.children();
                if (elements.length < 2) return;
                let top = elements.get(0) && elements.get(0).offsetTop,
                    top2 = elements.get(1) && elements.get(1).offsetTop;

                for (let i = 2; i < MAX_ITEMS_PER_ROW; i++) {
                    if (top !== top2) {
                        itemsPerRow = i - 1;
                        rowHeight = top2 - top;
                        break;
                    }
                    top = top2;
                    top2 = elements.get(i) && elements.get(i).offsetTop;
                }
                setCanvasHeight();
            };

            const setCanvasHeight = () => {
                canvasHeight = Math.ceil(collection.length / itemsPerRow) * rowHeight;
                logger('Calculated ' + itemsPerRow + ' items per row');
                //Exporting itemsPerRow to allow special CSS for last items per row
                scope.itemsPerRow = itemsPerRow;
                canvasElement.css({'height': canvasHeight});
                logger('Canvas height calculated to ' + canvasHeight);
                rowsInContainer = Math.ceil(element[0].clientHeight / rowHeight);
            };

            let currentFirstIndex;
            scope.render = (firstIndex) => {
                firstIndex = firstIndex || 0;
                currentFirstIndex = firstIndex;
                //Last item to render including buffer
                const endIndex = Math.min(collection.length, firstIndex + rowsInContainer * itemsPerRow + BUFFER_ROWS * itemsPerRow);
                //First item to render including buffer
                firstIndex = Math.max(0, firstIndex - BUFFER_ROWS * itemsPerRow);
                scope[VS_COLLECTION_NAME] = _.slice(collection, firstIndex, endIndex);
                logger('Rendering started index ' + firstIndex + ' to ' + endIndex + ' (' + (endIndex - firstIndex + 1) + ' items)');
                const top = Math.floor(firstIndex / itemsPerRow) * rowHeight;
                contentElement.css({top});
                contentElement.css({visibility: 'visible'});
                logger('render container top calculated to ' + top + ' scroll top is ' + element.prop('scrollTop'));
            };

            const scrollPosToIndex = (scrollTop) => {
                return Math.floor(scrollTop / rowHeight * itemsPerRow);
            };

            const indexToScrollPos = (firstIndex) => {
                return Math.floor((firstIndex - (firstIndex % itemsPerRow)) / itemsPerRow) * rowHeight;
            };

            const updateView = () => {
                setCanvasHeight();
                const scrollTop = element.prop('scrollTop'),
                    firstIndex = scrollPosToIndex(scrollTop);
                scope.render(firstIndex);
                if (!scope.$$phase) scope.$apply();
            };

            const resize = () => {
                fillThenCalc(() => {
                    //We want to show the same element as before the resize and the element scroll moves
                    logger('resizing to first index ' + currentFirstIndex);
                    const newScrollTop = indexToScrollPos(currentFirstIndex);
                    element.prop('scrollTop', newScrollTop);
                    scope.render(currentFirstIndex);
                });
                if (!scope.$$phase) scope.$apply();
            };

            const throttledResize = _.throttle(resize, 500, {trailing: true});

            element[0].addEventListener('scroll', updateView);
            $window.addEventListener('resize', throttledResize);

            const domTemplate = attributes.nocVsRepeaterTemplate;
            $http.get(domTemplate, {cache: $templateCache})
                .then((response) => {
                    const template = response.data;
                    scope.$watch(collectionName, () => {
                        collection = scope[collectionName];
                        if (scope.initialized) updateView();
                        else if (!scope.templateLoaded) {
                            init(template);
                            fillThenCalc(scope.render);
                        } else {
                            fillThenCalc(scope.render);
                        }
                    });
                })
                .catch(() => {
                    throw new Error('Could not get template ' + domTemplate);
                });

            const repeaterModelName = attributes.nocVsRepeaterName;
            if (!repeaterModelName) throw new Error('noc-virtual-scroll nocVsRepeaterName not defined');

            const collectionName = attributes.nocVirtualScroll;
            let collection = scope[collectionName];

            scope.$on('$destroy', () => {
                element[0].removeEventListener('scroll', updateView);
                $window.removeEventListener('resize', resize);
            });
        }
    };
});
