(() => {
    'use strict';

    angular.module('noc').controller('AddAzureAccountController', ($scope, nocDialog, CloudProviderAccountResource,
                                                                   nocHttp, nocUI, $state) => {

        $scope.initialize = () => {
            $scope.azureAccountRequest = {};
            $scope.editMode = Boolean($state.params.accountPublicId);
            if ($scope.editMode) {
                CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                    .then((accounts) => {
                        $scope.selectedAccount = _.find(accounts, {publicId: $state.params.accountPublicId});
                        $scope.azureAccountRequest.accountName = $scope.selectedAccount.accountName;
                        $scope.azureAccountRequest.providerKeys = {
                            applicationId: $scope.selectedAccount.applicationId,
                            tenantId: $scope.selectedAccount.tenantId
                        };
                    });
            }
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/azure-learn-more.html', {
                width: 890,
                fromTop: '20%'
            });
        };

        $scope.goBack = () => {
            if ($scope.editMode) {
                $scope.goToCloudProviderAccountSettings();
            } else {
                $scope.goToAddCloudProviderAccount();
            }
        };

        const onError = (error) => {
            if (_.endsWith(error.message, 'Account name is already in use.')) {
                $scope.nameErrorMessage = 'Account name already exists';
            } else {
                $scope.errorMessage = error.message;
            }
        };

        $scope.createOrUpdateAzureAccount = () => {
            $scope.errorMessage = '';
            $scope.nameErrorMessage = '';
            if ($scope.addAzureAccountForm.$valid) {
                if ($scope.editMode) {
                    return CloudProviderAccountResource.editAccount(nocHttp, 'azure', $scope.selectedAccount.publicId, $scope.azureAccountRequest.accountName, $scope.azureAccountRequest.providerKeys)
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                } else {
                    return CloudProviderAccountResource.createAccount(nocHttp, 'azure', $scope.azureAccountRequest.accountName, $scope.azureAccountRequest.providerKeys, null, true)
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                }
            } else {
                nocUI.handleFormValidity();
            }
        };

        $scope.initialize();
    });
})();
