(() => {
    'use strict';

    angular.module('noc').controller('AddGcpAccountController', ($scope, nocDialog, CloudProviderAccountResource,
                                                                 nocHttp, nocUI, $state) => {
        $scope.initialize = () => {
            $scope.gcpAccountRequest = {};
            $scope.editMode = Boolean($state.params.accountPublicId);
            if ($scope.editMode) {
                CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                    .then((accounts) => {
                        $scope.selectedAccount = _.find(accounts, {publicId: $state.params.accountPublicId});
                        $scope.gcpAccountRequest.accountName = $scope.selectedAccount.accountName;
                        $scope.gcpAccountRequest.providerKeys = {accessKey: $scope.selectedAccount.accessKey};
                    });
            }
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/gcp-learn-more.html', {
                width: 920,
                fromTop: '20%'
            });
        };

        $scope.goBack = () => {
            if ($scope.editMode) {
                $scope.goToCloudProviderAccountSettings();
            } else {
                $scope.goToAddCloudProviderAccount();
            }
        };

        const onError = (error) => {
            if (_.endsWith(error.message, 'Account name is already in use.')) {
                $scope.nameErrorMessage = 'Account name already exists';
            } else {
                $scope.errorMessage = error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length);
            }
        };

        $scope.createOrUpdateGcpAccount = () => {
            $scope.errorMessage = '';
            $scope.nameErrorMessage = '';
            if ($scope.addGcpAccountForm.$valid) {
                if ($scope.editMode) {
                    return CloudProviderAccountResource.editAccount(nocHttp, 'gcp', $scope.selectedAccount.publicId, $scope.gcpAccountRequest.accountName, $scope.gcpAccountRequest.providerKeys)
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                } else {
                    return CloudProviderAccountResource.createAccount(nocHttp, 'gcp', $scope.gcpAccountRequest.accountName, $scope.gcpAccountRequest.providerKeys, null, true)
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                }
            } else {
                nocUI.handleFormValidity();
            }
        };

        $scope.initialize();
    });
})();
