(() => {
    'use strict';

    angular.module('noc').controller('AddNssAccountController', ($scope, nocDialog, CloudProviderAccountResource,
                                                                 nocHttp, nocUI, $state) => {
        $scope.initialize = () => {
            $scope.nssAccountRequest = {};
            $scope.editMode = Boolean($state.params.accountPublicId);
            $scope.registeredVsa = [];
            $scope.unregisteredVsa = [];
            getInitialUnregisteredVsa();
            if ($scope.editMode) {
                CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                    .then((accounts) => {
                        $scope.selectedAccount = _.find(accounts, {publicId: $state.params.accountPublicId});
                        $scope.nssAccountRequest.accountName = $scope.selectedAccount.accountName;
                        $scope.nssAccountRequest.providerKeys = {
                            nssUserName: $scope.selectedAccount.nssUserName
                        };
                        $scope.registeredVsa = $scope.selectedAccount.vsaList;
                    });
            }
        };

        const getInitialUnregisteredVsa = () => {
            CloudProviderAccountResource.getUnregisteredVsas(nocHttp)
                .then((unregisteredVsas) => {
                    _.forEach(unregisteredVsas, (vsa) => {
                        vsa._checked = true;
                    });
                    $scope.unregisteredVsa = unregisteredVsas;
                });
        };

        const getCheckedVsaForRegister = () => {
            return _.map(_.filter($scope.unregisteredVsa, '_checked'), 'publicId');
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/nss-learn-more.html', {
                width: 900,
                fromTop: '20%'
            });
        };

        $scope.openVsaDialog = (dialogModeValue) => {
            nocDialog.confirm('', 'assets/templates/dialogs/select-vsa-to-nss-register-dialog.html', {
                unregisteredVsa: _.cloneDeep($scope.unregisteredVsa),
                registeredVsa: $scope.registeredVsa,
                dialogMode: {value: dialogModeValue},
                width: 500,
                fromTop: '30%'
            })
                .then((dialogScope) => {
                    $scope.unregisteredVsa = dialogScope.unregisteredVsa;
                });
        };

        $scope.goBack = () => {
            if ($scope.editMode) {
                $scope.goToCloudProviderAccountSettings();
            } else {
                $scope.goToAddCloudProviderAccount();
            }
        };

        const onError = (error) => {
            if (_.endsWith(error.message, 'Account name is already in use.')) {
                $scope.nameErrorMessage = 'Account name already exists';
            } else {
                $scope.errorMessage = error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length);
            }
        };

        $scope.createOrUpdateNssAccount = () => {
            $scope.errorMessage = '';
            $scope.nameErrorMessage = '';
            if ($scope.addNssAccountForm.$valid) {
                if ($scope.editMode) {
                    return CloudProviderAccountResource.editAccount(nocHttp, 'nss', $scope.selectedAccount.publicId, $scope.nssAccountRequest.accountName, $scope.nssAccountRequest.providerKeys, getCheckedVsaForRegister())
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                } else {
                    return CloudProviderAccountResource.createAccount(nocHttp, 'nss', $scope.nssAccountRequest.accountName, $scope.nssAccountRequest.providerKeys, getCheckedVsaForRegister(), true)
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                }
            } else {
                nocUI.handleFormValidity();
            }
        };

        $scope.initialize();
    });
})();
