(function () {
    "use strict";

    angular.module('noc').factory("BaseCreateVolumeOnExistingWorkingEnvironment", function (usageProfiles, CifsResource) {
        return function (pidgeonService, nocHttp) {
            var scope = this;
            var pidgeon = pidgeonService;

            scope.checkCifsAvailability = function checkCifsAvailability(publicId, we, svmName) {
                var cifsResource = new CifsResource(publicId, we, svmName, nocHttp);
                return cifsResource.getCifsConfigurations()
                    .then(function (response) {
                        return response.length;
                    });
            };

            scope.getDefaultExportPolicyIps = function () {
                switch (scope.selectedWorkingEnvironment._typeAndProvider) {
                    case 'VSA_AWS':
                        return [scope.selectedWorkingEnvironment.awsProperties.vpc.cidrBlock];
                    case 'VSA_Azure':
                        return [scope.selectedWorkingEnvironment.providerProperties.vnetCidr];
                    case 'VSA_GCP':
                        return [scope.selectedWorkingEnvironment.providerProperties.subnetCidr];
                    default: return [];
                }
            };

            scope.volumeProtocolChanged = function volumeProtocolChanged() {
                scope.wizardController.setWizardStep(scope.wizardController.path['details']);
                if (scope.volumeProtocol.type !== "CIFS") {
                    scope.needCifsConfiguration = false;
                    scope.initSyncToS3Status();
                    return;
                }

                scope.syncToS3Status = scope.syncToS3Status !== 'NOT_AVAILABLE' ? 'CIFS' : 'NOT_AVAILABLE';
                scope.volumeRequest.syncToS3 = false;

                scope.volumeRequest.shareInfo.shareName = scope.volumeRequest.name ? scope.volumeRequest.name + "_share" : "";
                if (scope.volumeRequest.shareInfo.shareName.length > 80) scope.volumeRequest.shareInfo.shareName = "";
                var pubId = scope.selectedWorkingEnvironment.publicId;
                var type = scope.selectedWorkingEnvironment.workingEnvironmentType;
                var svmName = type === "ON_PREM" ? scope.volumeRequest.svmName : undefined;
                scope.checkCifsAvailability(pubId, scope.selectedWorkingEnvironment, svmName)
                    .then(function (response) {
                        scope.needCifsConfiguration = !response;
                        if (scope.needCifsConfiguration && scope.selectedWorkingEnvironment.workingEnvironmentType !== "VSA") {
                            scope.volumeProtocol.type = "NFS";
                            pidgeon.error("CIFS is not configured. You can configure it in System Manager.", "", "cifs");
                            scope.needCifsConfiguration = false;
                        }
                    });
            };

            scope.createVolumeRequest = function() {
                // init
                var ips = scope.getDefaultExportPolicyIps();
                return {
                    workingEnvironmentId: scope.selectedWorkingEnvironment.publicId,
                    svmName: scope.selectedWorkingEnvironment.svmName,
                    exportPolicyInfo:{
                        policyType: scope.selectedWorkingEnvironment.workingEnvironmentType === "VSA" ? "custom" : "none",
                        ips: ips,
                        _ips: ips.join(", ")
                    },
                    shareInfo:{
                        accessControl: {
                            _users: "Everyone;",
                            users: ["Everyone"],
                            permission: "full_control"
                        },
                        shareName: ""
                    },
                    usageProfile: usageProfiles.default
                };

            };
        };
    });
})();
