(() => {
    'use strict';

    angular.module('noc').controller('ReplicationStatusController', ($scope, nocHttp, nocUI, pidgeon, nocDialog, replicationTableColumnsService,
                                                                     nocSchedulesService, $timeout, nocUtil, Constants, columnPickerService,
                                                                     uiGridFilterService, nocPoliciesService, replicationResource, uiGridConstants, nocSelectedWorkspaceOrTenantService) => {
        $scope.moment = moment;
        $scope.search = {term: ''};

        $scope.deleteRelationship = (relationship) => {
            nocDialog
                .confirm('Are you sure you want to delete the relationship between "' + relationship.source.volumeName + '" and "' + relationship.destination.volumeName + '"?')
                .then(() => {
                    nocHttp.delete(
                        'replication/' + relationship.destination.workingEnvironmentId + '/' + relationship.destination.svmName + '/' + relationship.destination.volumeName,
                        'deleting relationship between ' + relationship.source.volumeName + ' and ' + relationship.destination.volumeName,
                        nocHttp.BLOCK)
                        .then(() => {
                            pidgeon.inProgress('Relationship delete in progress.', true);
                        });
                });
        };

        $scope.updateSnapMirror = function(relationship) {
            $scope.sendPostRequest(relationship, "update", "updating", false);
        };

        $scope.initializeRelationship = function(relationship){
            $scope.sendPostRequest(relationship, "initialize", "initializing", true);
        };

        $scope.breakRelationship = function(relationship) {
            $scope.sendPostRequest(relationship, "break", "breaking", true);
        };

        $scope.resyncRelationship = function(relationship) {
            nocDialog.confirm("Are you sure you want to resync the relationship?","assets/templates/dialogs/resync-relationship-confirm-content.html", {relationship : relationship})
                .then(function() {
                    $scope.sendPostRequest(relationship, "resync", "resynchronizing", false);
                });
        };

        $scope.reverseResyncRelationship = function(relationship) {
            nocDialog.confirm("Approve Reverse Relationship", "assets/templates/dialogs/reverse-relationship-confirm-content.html", {relationship : relationship, width : 650})
                .then(function() {
                    $scope.sendPostRequest(relationship, "reverse", "reversing", false, {
                        sourceWorkingEnvironmentId: relationship.source.workingEnvironmentId,
                        sourceSvmName: relationship.source.svmName,
                        sourceVolumeName: relationship.source.volumeName
                    });
                });
        };

        $scope.editMaxTransferRateConfirm = function (relationship) {
            var avgTransferRate = relationship._totalTransferSize.sortValue / relationship._totalTransferTime.sortValue;
            var avgTransferRateMBps = avgTransferRate / 1024 / 1024;
            nocDialog.confirm("Edit Max Transfer Rate", "assets/templates/dialogs/edit-max-transfer-rate-dialog.html", {
                width: 620,
                wizardController: {
                    limitMaxTransferRate: relationship.maxTransferRate.size !== 0,
                    maxTransferRateMBps: (relationship.maxTransferRate.size / 1024) || undefined,
                    avgTransferRateMBps: avgTransferRateMBps
                },
                maxInt32: Constants.maxInt32
            })
                .then(function (dialogScope) {
                    $scope.editMaxTransferRate(relationship, dialogScope.wizardController.limitMaxTransferRate, dialogScope.wizardController.maxTransferRateMBps * 1024);
                });
        };

        $scope.editMaxTransferRate = function (relationship, limitMaxTransferRate, maxTransferRate) {
            if (maxTransferRate !== relationship.maxTransferRate.size || (!limitMaxTransferRate && relationship.maxTransferRate.size)) {
                replicationResource.updateRelationship(nocHttp, relationship.destination,
                    {maxTransferRate: limitMaxTransferRate ? maxTransferRate : 0}, "updating max transfer rate");
            }
        };

        $scope.editSchedule = function (relationship) {
            nocSchedulesService.getSchedules(nocHttp, relationship.destination.workingEnvironmentId)
                .then(function (schedules) {
                    var selectedSchedule = _.find(schedules, {name: relationship.schedule});
                    var originalSchedule = selectedSchedule ? _.cloneDeep(selectedSchedule) : {name: Constants.emptyReplicationScheduleDescription};
                    return nocDialog.confirm("Select a replication schedule", "assets/templates/dialogs/select-schedule-dialog.html", {
                        schedules: schedules,
                        width: 965,
                        fromTop: "10%",
                        setSelected: function (newVal) {
                            selectedSchedule = newVal;
                        },
                        getSelected: function () {
                            return selectedSchedule;
                        }
                    })
                        .then(function () {
                            if (originalSchedule.name !== selectedSchedule.name) {
                                return replicationResource.updateRelationship(nocHttp, relationship.destination,
                                    {schedule: selectedSchedule.name}, "updating schedule information");
                            }
                        });
                });
        };

        $scope.policyInfo = function (relationship) {
            nocPoliciesService.getPolicies(nocHttp, relationship.source.workingEnvironmentId, relationship.destination.workingEnvironmentId)
                .then(function (policies) {
                    $scope.policy = _.find(policies, {name: relationship.policy});
                    return nocPoliciesService.getSnapshotLabels(nocHttp, [$scope.policy], relationship.source.workingEnvironmentId, relationship.source.svmName, relationship.source.volumeName)
                        .then(function () {
                            return nocDialog.dialog("Policy Information", "assets/templates/dialogs/policy-info.html", {
                                policy: $scope.policy,
                                width: 536
                            });
                        });
                });
        };

        $scope.sendPostRequest = (relationship, type, action, needConfirm, body) => {
            body = body || {};
            const params = {
                url: 'replication/' + type + '/' + relationship.destination.workingEnvironmentId + '/' + relationship.destination.svmName + '/' + relationship.destination.volumeName,
                data: body
            };

            function sendPostFunction() {
                nocHttp.post(params,
                    action + ' the relationship between ' + relationship.source.volumeName + ' and ' + relationship.destination.volumeName,
                    nocHttp.BLOCK);
            }

            if (needConfirm) {
                nocDialog
                    .confirm('Are you sure you want to ' + type + ' the relationship between "' + relationship.source.volumeName + '" and "' + relationship.destination.volumeName + '"?')
                    .then(sendPostFunction);
            } else {
                sendPostFunction();
            }
        };

        var replicationStatusUnloading = nocUI.loading();

        var onGet = function(replicationStatusResponse) {
            pidgeon.clear("replicationStatus");
            $scope.replicationStatusResponse = replicationStatusResponse;
            $scope.gridOptions.data = $scope.replicationStatusResponse;
            $scope.calculateTotalVolumesUsedSize();
            replicationStatusUnloading();
        };

        $scope.isRefreshOngoing = false;
        $scope.refresh = function(){
            if ($scope.isRefreshOngoing) return;
            $scope.isRefreshOngoing = true;
            replicationResource.getReplicationStatus(nocHttp, nocSelectedWorkspaceOrTenantService.getPublicId(), $scope.selectedWorkingEnvironment)
                .then(onGet)
                .finally(function () {
                    $scope.isRefreshOngoing = false;
                });
        };

        $scope.$on(replicationResource.REPLICATION_STATUS_GET_ALL_EVENT, function(event, payload){
            onGet(payload.replicationStatusResponse);
            replicationStatusUnloading();
        });
        $scope.$on(replicationResource.REPLICATION_STATUS_GET_EVENT, function(event, payload){
            onGet(payload.replicationStatusResponse);
            replicationStatusUnloading();
        });

        $scope.$on(replicationResource.REPLICATION_STATUS_GET_ERROR_EVENT, function(event, payload){
            pidgeon.error(pidgeon.serverErrorPrefix + "retrieving replication status for working environment " + $scope.selectedWorkingEnvironment.name, payload.error, "replicationStatus");
            replicationStatusUnloading();
        });
        $scope.$on(replicationResource.REPLICATION_STATUS_GET_ALL_ERROR_EVENT, function(event, payload){
            pidgeon.error(pidgeon.serverErrorPrefix + "retrieving replication status", payload.error, "replicationStatus");
            replicationStatusUnloading();
        });

        var rowTemplate = "<div ng-mouseenter='grid.appScope.over(row.entity)' ng-mouseleave='grid.appScope.leave(row.entity)' ng-repeat=\"(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name\" class=\"ui-grid-cell\" ng-class=\"{ 'ui-grid-row-header-cell': col.isRowHeader, 'hovered': grid.appScope.hoveredRow === row.entity }\" ui-grid-cell></div>";
        $scope.hoveredRow = null;

        $scope.over = function(entity) {
            $scope.hoveredRow = entity;
        };

        $scope.leave = function(entity) {
            if ($scope.hoveredRow === entity){
                $scope.hoveredRow = null;
            }
        };

        function setGridHeight() {
            const uiGrid = $('[ui-grid]'),
                top = uiGrid.offset().top,
                mainFooterTop = $('.main-footer').offset().top;
            uiGrid.removeClass('visible');
            uiGrid.height(mainFooterTop - top - 20);
            $timeout(() => {
                uiGrid.addClass('visible');
            });
        }

        var onScroll = function(scrollEvent) {
            if (scrollEvent.x) $scope.gridIsScrolled = scrollEvent.newScrollLeft > 0;
        };

        $scope.gridIsScrolled = false;
        $scope.gridOptions = {
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            useExternalFiltering: true,
            data: $scope.filteredRelationships,
            columnVirtualizationThreshold: 50,
            virtualizationThreshold: 30,

            rowTemplate: rowTemplate,

            columnDefs: replicationTableColumnsService.getColumnsDef(),

            onRegisterApi: function (gridAPI) {
                $scope.gridAPI = gridAPI;
                uiGridFilterService.installFilter(gridAPI, $scope);
                $scope.gridAPI.core.addRowHeaderColumn({
                    name: 'rowHeaderCol',
                    displayName: '',
                    width: 60,
                    enableColumnResizing: false,
                    cellTemplate: 'assets/templates/common/replication-status-popup-btn.html'
                });
                $scope.gridAPI.colMovable.on.columnPositionChanged($scope, () => {
                    replicationTableColumnsService.saveGridState($scope.gridAPI, $scope);
                });
                $timeout(() => {
                    replicationTableColumnsService.restoreGridState($scope.gridAPI, $scope);
                    $timeout(() => {
                        $(window).trigger('resize');
                    });
                });
                $scope.$watch('gridAPI.grid.isScrollingVertically', (isScrolling) => {
                    if (isScrolling && $scope.popoverHelper.popoverApi) $scope.popoverHelper.popoverApi.remove();
                });
                $scope.gridAPI.core.on.scrollBegin($scope, onScroll);
                $scope.gridAPI.core.on.scrollEnd($scope, onScroll);
            }
        };

        $(window).on("resize", setGridHeight);

        $scope.startColumnPicker = function() {
            columnPickerService.startColumnPicker($scope.gridOptions, $scope.gridAPI, replicationTableColumnsService, "Replication Status Table Columns");
        };

        $scope.$watch('replicationStatusResponse', $scope.doFiltering);

        $scope.getNumOfVolumesTransferring = function() {
            return _.filter($scope.replicationStatusResponse, {relationshipStatus : "transferring"}).length;
        };

        $scope.getNumOfHealthyRelationships = function() {
            return _.filter($scope.replicationStatusResponse, {_health : "Healthy"}).length;
        };

        $scope.getNumOfFailedRelationships = function() {
            return $scope.replicationStatusResponse.length - $scope.getNumOfHealthyRelationships();
        };

        $scope.calculateTotalVolumesUsedSize = function() {
            $scope.totalVolumesUsedSize = {size:0, unit:"Byte"};
            _.forEach($scope.replicationStatusResponse, function (replication) {
                $scope.totalVolumesUsedSize.size += nocUtil.convertTo(replication.volumeUsedSize, 'Byte').size;
            });
            $scope.totalVolumesUsedSize = nocUtil.normalizeCapacity($scope.totalVolumesUsedSize);
        };

        $scope.$on('$destroy', () => {
            replicationTableColumnsService.saveGridState($scope.gridAPI);
        });

        $scope.getActionsAllowed = function(replication) {
            return replication._allowedRelationshipOperations['break-relationship'] ||
                replication._allowedRelationshipOperations['resync-relationship'] ||
                replication._allowedRelationshipOperations['edit-relationship'] ||
                replication._allowedRelationshipOperations['delete-relationship'] ||
                replication._allowedRelationshipOperations['policy-info'] ||
                replication._allowedRelationshipOperations['update-snapmirror'];
        };

        $scope.getMenuTitle = function(relationship) {
            return $scope.getActionsAllowed(relationship)? "" : "Actions on a relationship are allowed only when both working environments are in 'ON' state";
        };

        $scope.shouldDisable = function(relationship) {
            return  !relationship._allowedRelationshipOperations['break-relationship'] &&
                    !relationship._allowedRelationshipOperations['resync-relationship'] &&
                    !relationship._allowedRelationshipOperations['edit-relationship'] &&
                    !relationship._allowedRelationshipOperations['delete-relationship'] &&
                    !relationship._allowedRelationshipOperations['policy-info'] &&
                    !relationship._allowedRelationshipOperations['update-snapmirror'];
        };

        $scope.$on('$destroy', function() {
            $(window).off('resize', setGridHeight);
        });
    });
})();
