(() => {
    'use strict';

    angular.module('noc').controller('NoServerConnectionDialogController', ($scope, nocUI, configService, nocHttp, pidgeon,
                                                                            nocUIMain, PortalSetupResource, nocCurrentUserService) => {
        $scope.model = {};
        $scope.values = {};
        $scope.proxyShowAuthenticationData = false;
        $scope.notAuthenticated = !nocCurrentUserService.isCurrentUserExist(); //This is used to identify if there is no authenticated user, so different API should be used to update proxy
        // (E.g. when in Portal mode and proxy went down). User is unauthenticated in case auth0Information is null (missing clientID to perform PortalAuthentication)

        $scope.initialize = () => {
            if ($scope.notAuthenticated) {
                $scope.values = {};
                $scope.originalValues = {};
            } else {
                configService.getConfig(nocHttp, ['proxyUri', 'proxyUserName', 'proxyPassword', 'proxyDomain'])
                    .then((configResponse) => {
                        $scope.values = configResponse;
                        $scope.originalValues = _.cloneDeep($scope.values);
                        if ($scope.values.proxyPassword && $scope.values.proxyUserName) {
                            $scope.proxyShowAuthenticationData = true;
                        }
                    });
            }
        };

        $scope.saveChanges = () => {
            if (!$scope.connectivityForm.$valid) {
                nocUI.handleFormValidity($scope.connectivityForm, $scope);
            } else {
                if (_.isString($scope.values.proxyUri)) {
                    $scope.values.proxyUri = _.trim($scope.values.proxyUri);
                }
                if ($scope.notAuthenticated) {
                    PortalSetupResource.savePortalProxyDetails({
                        uri: $scope.values.proxyUri,
                        proxyUserName: $scope.values.proxyUserName,
                        proxyPassword: $scope.values.proxyPassword,
                        proxyDomain: $scope.values.proxyDomain
                    }, nocHttp)
                        .then($scope.handleSuccessSave);
                } else {
                    configService.updateConfig(nocHttp, ['proxyUri', 'proxyUserName', 'proxyPassword', 'proxyDomain'],
                        $scope.values, $scope.originalValues)
                        .then($scope.handleSuccessSave);
                }
            }
        };

        $scope.handleSuccessSave = () => {
            pidgeon.success('Setting changes saved.');
            $scope.originalValues = _.cloneDeep($scope.values);
            $scope.connectivityForm.$setPristine();
            nocUIMain.reloadSilently(false);
        };

        $scope.expandProxyAuthenticationData = () => {
            $scope.proxyShowAuthenticationData = !$scope.proxyShowAuthenticationData;
        };

        $scope.initialize();
    });
})();
