(() => {
    'use strict';

    angular.module('noc').controller('ADSetupController', ($scope, $rootScope, nocHttp, pidgeon, nocUIMain) => {

        $scope.initialize = () => {
            $scope.isInitialized = false;
            $scope.editMode = true;
            $scope.portalMode = _.get($rootScope.supportServices, 'portalService.usePortalAuthentication');
            $scope.getAD()
                .then((ad) => {
                    if (ad) {
                        ad.successfullyConnectedTo = {host: ad.host, port: ad.port};
                        $scope.ad = _.assign($scope.ad, ad);
                    }
                    $scope.editMode = !ad;
                })
                .finally(() => {
                    $scope.adForm.$setPristine();
                    $scope.isInitialized = true;
                });
        };

        $scope.getAD = function () {
            return nocHttp.get({url: '/active-directory'}, 'retrieving active directory', nocHttp.LOADING)
                .then(function (ads) {
                    if (!_.isEmpty(ads)) {
                        return ads[0];
                    }
                    return undefined;
                });
        };

        $scope.editAD = function () {
            $scope.editMode = true;
        };

        $scope.needsConnectivityTest = function needsConnectivityTest(ad) {
            return !ad || //If didn't fill AD details
                !ad.successfullyConnectedTo || //Or didn't test connectivity yet
                ad.host !== ad.successfullyConnectedTo.host || ad.port !== ad.successfullyConnectedTo.port; //Or tested connectivity successfully then changed host or port
        };

        $scope.saveADError = function (adForm, ad) {
            if (adForm) { // make sure form was loaded
                if (!adForm.$pristine && adForm.$invalid) {
                    return 'AD details are invalid';
                }
                if (ad && adForm.$pristine) {
                    return 'AD is already stored';
                }
            }
            if ($scope.needsConnectivityTest(ad)) {
                return 'Connectivity to AD must be confirmed before saving';
            }
            return '';
        };

        $scope.saveAD = function (ad) {
            var saveADData = _.pick(ad, ['host', 'port']);
            saveADData.name = saveADData.name || 'default';
            var progressMessage = 'saving active directory';
            var showSuccessAndRefreshAD = function () {
                pidgeon.success('Active directory saved');
                $scope.initialize(); // need to get public id of AD in order to update it on the next save
                $scope.deleteADTestCredentials();
            };

            if (ad.publicId) { // has a public id to update
                // update AD
                nocHttp.put({
                    url: '/active-directory/' + ad.publicId,
                    data: saveADData
                }, progressMessage, nocHttp.BLOCK).then(showSuccessAndRefreshAD);
            }
            else {
                // create AD
                nocHttp.post({
                    url: '/active-directory',
                    data: saveADData
                }, progressMessage, nocHttp.BLOCK).then(showSuccessAndRefreshAD);
            }
        };

        $scope.testAD = function (ad) {
            var testADData = _.pick(ad, ['host', 'port', 'userName', 'password']); // no need for AD name when testing connectivity
            nocHttp.post({
                url: '/active-directory/check-connection',
                data: testADData
            }, 'testing active directory', nocHttp.BLOCK)
                .then(function () {
                    ad.successfullyConnectedTo = {host: ad.host, port: ad.port};
                    pidgeon.success('Connection is successful');
                });
        };

        $scope.$on('$destroy', function () {
            $scope.deleteADTestCredentials();
        });

        $scope.deleteADTestCredentials = function () {
            if ($scope.ad) {
                delete $scope.ad.userName;
                delete $scope.ad.password;
            }
            nocUIMain.clearFormValidity();
        };

        $scope.initialize();
    });
})();
