(() => {
    'use strict';

    angular.module('noc').controller('AsupController', ($scope, pidgeon, intercomNotifier, nocHttp, configService, $q, systemResource) => {
        $scope.values = {};
        $scope.asupTriggerSelect = {
            model: {},
            availableOptions: [],
            selectAll: false,
            empty: true
        };

        $scope.initialize = () => {
            $scope.daysInWeek = [
                {value: 1, name: 'Monday'},
                {value: 2, name: 'Tuesday'},
                {value: 3, name: 'Wednesday'},
                {value: 4, name: 'Thursday'},
                {value: 5, name: 'Friday'},
                {value: 6, name: 'Saturday'},
                {value: 7, name: 'Sunday'}
            ];

            $scope.hoursInDay = [
                {value: 0, name: '00:00'},
                {value: 1, name: '01:00'},
                {value: 2, name: '02:00'},
                {value: 3, name: '03:00'},
                {value: 4, name: '04:00'},
                {value: 5, name: '05:00'},
                {value: 6, name: '06:00'},
                {value: 7, name: '07:00'},
                {value: 8, name: '08:00'},
                {value: 9, name: '09:00'},
                {value: 10, name: '10:00'},
                {value: 11, name: '11:00'},
                {value: 12, name: '12:00'},
                {value: 13, name: '13:00'},
                {value: 14, name: '14:00'},
                {value: 15, name: '15:00'},
                {value: 16, name: '16:00'},
                {value: 17, name: '17:00'},
                {value: 18, name: '18:00'},
                {value: 19, name: '19:00'},
                {value: 20, name: '20:00'},
                {value: 21, name: '21:00'},
                {value: 22, name: '22:00'},
                {value: 23, name: '23:00'}
            ];

                $q.all([
                nocHttp.get('/occm/asup', 'retrieving AutoSupport configuration', nocHttp.LOADING),
                configService.getConfig(nocHttp, ['debugLogLevel', 'cotRollback']),
                $scope.fetchVsas()
            ])
                .then(_.spread((configuration, values) => {
                    configuration.schedule.dayOfWeek = configuration.schedule.dayOfWeek.toString();
                    configuration.schedule.hourOfDay = configuration.schedule.hourOfDay.toString();
                    $scope.configuration = configuration;
                    $scope.dayOfWeekObject = {day: {value: configuration.schedule.dayOfWeek, name: $scope.dayOfWeek(configuration.schedule.dayOfWeek)}};
                    $scope.hourOfDayObject = {hour: {value: configuration.schedule.hourOfDay, name: $scope.hourOfDay(configuration.schedule.hourOfDay)}};
                    $scope.originalConfiguration = _.cloneDeep(configuration);
                    $scope.values = values;
                    $scope.values.protocolLoggingEnabled = $scope.values.debugLogLevel === 'debug' || $scope.values.debugLogLevel === 'verbose';
                    $scope.values.rollbackEnabled = $scope.values.cotRollback;
                    $scope.originalValues = _.cloneDeep(values);
                }));
        };

        $scope.onDayChange = () => {
            $scope.configuration.schedule.dayOfWeek = $scope.dayOfWeekObject.day.value.toString();
        };

        $scope.onHourChange = () => {
            $scope.configuration.schedule.hourOfDay = $scope.hourOfDayObject.hour.value.toString();
        };

        $scope.isPristine = () => {
            return _.isEqual($scope.values, $scope.originalValues) && _.isEqual($scope.configuration, $scope.originalConfiguration);
        };

        $scope.sendNow = () => {
            nocHttp.post('/occm/asup/send', 'sending AutoSupport request', nocHttp.BLOCK)
                .then(() => {
                    pidgeon.inProgress('Sending AutoSupport request.', true);
                });
        };

        $scope.sendCotNow = (sendToOccm) => {
            let pidMessage = 'Sending VSA AutoSupport request';
            if (sendToOccm) {
                pidMessage = 'Sending VSA AutoSupport request, please wait few minutes before downloading.';
            }
            nocHttp.post(`/occm/asup/send-Cot-Asup?sendToOccm=${sendToOccm}`, 'sending VSA AutoSupport request', nocHttp.BLOCK)
                .then(() => {
                    pidgeon.inProgress(pidMessage, true);
                });
        };

        $scope.sendCotPerformanceNow = () => {
            nocHttp.post({
                url: '/occm/asup/send-Cot-performance-Asup/' + $scope.asupTriggerSelect.model.publicId
            }, 'sending VSA archive performance request', nocHttp.BLOCK)
                .then(() => {
                    pidgeon.inProgress('Sending VSA archive performance request.', true);
                });
        };

        $scope.sendCotPerformanceNowDisabledFunction = () => {
            return $scope.asupTriggerSelect.empty || ($scope.asupTriggerSelect.model.status.status !== 'ON' && $scope.asupTriggerSelect.model.status.status !== 'DEGRADED');
        };

        $scope.downloadCotNow = () => {
            $scope.download('/occm/asup/download-ontap-cloud-asups');
        };

        $scope.updateIntercomAvailability = () => {
            //When turning off ASUP scheduler, also intercom is not available.
            return systemResource(nocHttp).getSupportServices(nocHttp.LOADING)
                .then((supportServices) => {
                    //Will only do something if value changed
                    if (supportServices.intercomEnabled) {
                        intercomNotifier.boot();
                    } else {
                        intercomNotifier.logout();
                    }
                });
        };

        $scope.save = () => {
            const actions = [];
            if (!_.isEqual($scope.values, $scope.originalValues)) {
                $scope.values.debugLogLevel = $scope.values.protocolLoggingEnabled ? 'debug' : 'off';
                $scope.values.cotRollback = $scope.values.rollbackEnabled;
                actions.push(configService.updateConfig(nocHttp, ['debugLogLevel', 'cotRollback'], $scope.values, $scope.originalValues)
                    .then(() => {
                        $scope.originalValues = _.cloneDeep($scope.values);
                    }));
            }
            if (!_.isEqual($scope.configuration, $scope.originalConfiguration)) {
                actions.push(nocHttp.put({
                    url: '/occm/asup',
                    data: $scope.configuration
                }, 'saving support configuration', nocHttp.BLOCK)
                    .then(() => {
                        $scope.updateIntercomAvailability();
                        $scope.originalConfiguration = _.cloneDeep($scope.configuration);
                    }));
            }
            $q.all(actions)
                .then(() => {
                    pidgeon.success('Support configuration successfully saved');
                });
            $scope.currentOpenSection = '';
        };

        $scope.download = (url) => {
            return nocHttp.download(url, 'downloading AutoSupport', nocHttp.BLOCK)
                .then(() => {
                    pidgeon.success('AutoSupport was successfully downloaded');
                });
        };

        $scope.downloadNow = () => {
            $scope.download('/occm/asup/download');
        };

        $scope.fetchVsas = () => {
            const opts = {
                url: '/working-environments?fields=status',
                params: {}
            };

            return nocHttp.get(opts, 'retrieving working environments', nocHttp.LOADING)
                .then((data) => {
                    $scope.awsWes = _.map(data.vsaWorkingEnvironments, (we) => {
                        return {
                            name: we.name,
                            publicId: we.publicId,
                            status: we.status
                        };
                    });
                    $scope.azureWes = _.map(data.azureVsaWorkingEnvironments, (we) => {
                        return {
                            name: we.name,
                            publicId: we.publicId,
                            status: we.status
                        };
                    });
                    $scope.gcWes = _.map(data.gcpVsaWorkingEnvironments, (we) => {
                        return {
                            name: we.name,
                            publicId: we.publicId,
                            status: we.status
                        };
                    });

                    $scope.allWElist = [...$scope.awsWes, ...$scope.azureWes, ...$scope.gcWes];
                    $scope.asupTriggerSelect.availableOptions = $scope.allWElist.sort((firstWe, secondWe) => {
                        return firstWe.name.localeCompare(secondWe.name);
                    });
                    $scope.asupTriggerSelect.model = $scope.asupTriggerSelect.availableOptions[0];
                    $scope.asupTriggerSelect.empty = $scope.asupTriggerSelect.availableOptions.length === 0;
                });
        };

        $scope.tooltipWhenDisabled = function tooltipWhenDisabled() {
            if ($scope.asupTriggerSelect.empty) return 'There are no Working Environments available';
            if ($scope.asupTriggerSelect.model.status.status !== 'ON' && $scope.asupTriggerSelect.model.status.status !== 'DEGRADED') return 'This action is only available in ON or DEGRADED status';

            return 'Performance archive ASUP message sending';
        };

        $scope.toggleAsupRowInformation = (section) => {
            $scope.values = _.cloneDeep($scope.originalValues);
            $scope.configuration = _.cloneDeep($scope.originalConfiguration);
            if ($scope.currentOpenSection === section) {
                $scope.currentOpenSection = '';
            } else {
                $scope.currentOpenSection = section;
            }
        };

        $scope.dayOfWeek = (dayOfWeek) => {
            switch (dayOfWeek) {
                case '1':
                    return 'Monday';
                case '2':
                    return 'Tuesday';
                case '3':
                    return 'Wednesday';
                case '4':
                    return 'Thursday';
                case '5':
                    return 'Friday';
                case '6':
                    return 'Saturday';
                case '7':
                    return 'Sunday';
                default:
                    return '';
            }
        };

        $scope.hourOfDay = (hourOfDay) => {
            if (hourOfDay.length === 1) {
                hourOfDay = '0' + hourOfDay;
            }

            return hourOfDay + ':00';
        };

        $scope.shortInfoAutoSupportSchedule = () => {
            if ($scope.configuration) {
                return $scope.configuration.enabled ? 'Active: ' + $scope.dayOfWeek($scope.configuration.schedule.dayOfWeek) + ', ' + $scope.hourOfDay($scope.configuration.schedule.hourOfDay) : 'Not Active';
            } else {
                return '';
            }
        };

        $scope.shortInfoONTAPCloudInstance = () => {
            return $scope.asupTriggerSelect.model ? $scope.asupTriggerSelect.model.name : '';
        };

        $scope.shortInfoFullAPILogging = () => {
            return $scope.values.protocolLoggingEnabled ? 'ON' : 'OFF';
        };

        $scope.shortInfoAutomaticRollback = () => {
            return $scope.values.rollbackEnabled ? 'ON' : 'OFF';
        };

        $scope.initialize();
    });
})();
