(() => {
    'use strict';

    angular.module('noc').controller('SetProxyAfterSetupController', ($scope, PortalSetupResource, nocUI, systemResource,
                                                                      ServerUpgradeService, OCCMResource, nocHttp, $q) => {
        $scope.finishSetup = function finishSetup() {
            return PortalSetupResource.completeConnectivityPendingSetup()
                .then((response) => {
                    if (response.upgradeToVersion) {
                        //Doing auto upgrade, display nice loader while upgrading.
                        $scope.upgradeToVersion = response.upgradeToVersion;
                        $scope.controllerState = 'auto-upgrade';
                        const callbackAfterUpgrade = () => {
                            nocUI.reloadSilently(true);
                        };

                        return ServerUpgradeService.pollForServerUpgrade(callbackAfterUpgrade);
                    } else {
                        //No need for auto upgrade, reload, app should work.
                        nocUI.reloadSilently(true);
                    }
                })
                .catch((error) => {
                    $scope.errorMessage = error.data.message;
                });
        };

        $scope.saveProxyPortalSetup = function saveProxyPortalSetup(uri, username, password, domain) {
            $scope.detectingConnectivity = true;
            let ensureConnectivity;

            if (uri) {
                //User entered proxy:
                const proxyRequest = {
                    uri,
                    proxyUserName: username,
                    proxyPassword: password,
                    proxyDomain: domain
                };

                ensureConnectivity = PortalSetupResource.savePortalProxyDetails(proxyRequest)
                    .catch((error) => {
                        $scope.errorMessage = error.data.message;

                        return $q.reject();
                    });
            } else {
                //User allegedly fixed the connectivity issues without setting proxy:
                ensureConnectivity = OCCMResource.validatePortalConnectivity(nocHttp)
                    .catch((error) => {
                        $scope.errorMessage = error.message;

                        return $q.reject();
                    });
            }

            return ensureConnectivity
                .then(systemResource().getSupportServices) //Need to reload supported services in order to get the portal data
                .then($scope.finishSetup)
                .finally(() => {
                    $scope.detectingConnectivity = false;
                });
        };

        $scope.initialize = () => {
            $scope.controllerState = 'ensure-connectivity';
            $scope.detectingConnectivity = true;
            OCCMResource.validatePortalConnectivity(nocHttp)
                .then($scope.finishSetup)
                .finally(() => {
                    $scope.detectingConnectivity = false;
                });
        };

        $scope.initialize();
    });
})();
