(() => {
    'use strict';
    angular.module('noc').controller('SettingsController', ($scope, nocHttp, nocUI, pidgeon, nocNavigateAway, configService) => {
        $scope.values = {};
        $scope.proxyShowAuthenticationData = false;

        $scope.initialize = () => {
            $scope.automaticUpdateMoreInfoText = 'Cloud Manager automatically upgrades to the latest release <br> at 12:00 midnight if no operations are running.';
            $scope.automaticOntapUpdateMoreInfoText = 'This update occurs when you deploy a system and a newer Release Candidate, General Availability,\n' +
                '                                                                or patch release is available for the selected version. For example, if you select Cloud Volumes ONTAP\n' +
                '                                                                9.4 RC1 and Cloud Volumes ONTAP 9.4 GA is available, then Cloud Manager updates the system to that\n' +
                '                                                                version when creating the working environment. The update does not occur from one release to\n' +
                '                                                                another—for example, from 9.3 to 9.4.';
            $scope.capacityManagementModeMoreInfoText = 'If the Cloud Manager Admin set the Capacity Management Mode to automatic, Cloud Manager automatically purchases new disks for ' +
                                                        'Cloud Volumes ONTAP instances when more capacity is needed, deletes unused collections of disks (aggregates), and moves volumes between ' +
                                                        'aggregates, as needed.';
            configService.getConfig(nocHttp, ['maxAggregateFreeSpacePercentage', 's3EbsRatio', 'maxVolumeGrowSizePercentage',
                'proxyUri', 'autoVsaCapacityManagement', 'proxyUserName', 'proxyPassword', 'proxyDomain', 'autoUpgrade', 'autoOntapUpgrade'])
                .then((configResponse) => {
                    $scope.values = configResponse;
                    $scope.values.autoVsaCapacityManagementViewValue = $scope.values.autoVsaCapacityManagement ? 'auto' : 'manual';
                    $scope.originalValues = _.cloneDeep($scope.values);
                    if ($scope.values.proxyPassword && $scope.values.proxyUserName) {
                        $scope.proxyShowAuthenticationData = true;
                    }
                });
            //Set focus to "focus-holder" div to define the beginning of keyboard navigation (HTML's "autofocus" doesn't work)
            $scope.resetFocus();
        };

        $scope.saveChanges = () => {
            if (!$scope.settingsForm.$valid) {
                nocUI.handleFormValidity($scope.settingsForm, $scope);
            } else {
                $scope.values.autoVsaCapacityManagement = $scope.values.autoVsaCapacityManagementViewValue === 'auto';
                if (_.isString($scope.values.proxyUri)) {
                    $scope.values.proxyUri = _.trim($scope.values.proxyUri);
                }
                configService.updateConfig(nocHttp, ['maxAggregateFreeSpacePercentage',
                    's3EbsRatio', 'maxVolumeGrowSizePercentage', 'proxyUri',
                    'autoVsaCapacityManagement', 'proxyUserName', 'proxyPassword', 'proxyDomain', 'autoUpgrade', 'autoOntapUpgrade'], $scope.values, $scope.originalValues)
                    .then(() => {
                        pidgeon.success('Setting changes saved.');
                        $scope.originalValues = _.cloneDeep($scope.values);
                        $scope.settingsForm.$setPristine();
                    })
                    .catch(() => {
                        $scope.values = _.cloneDeep($scope.originalValues);
                    });
                $scope.currentOpenSection = '';
            }
        };

        $scope.expandProxyAuthenticationData = () => {
            $scope.proxyPasswordOrUsernameIsRequired();
            $scope.proxyShowAuthenticationData = !$scope.proxyShowAuthenticationData;
        };

        $scope.toggleSettingsRowInformation = (sectionName) => {
            $scope.values = _.cloneDeep($scope.originalValues);
            if ($scope.currentOpenSection === sectionName) {
                $scope.currentOpenSection = '';
            } else {
                $scope.currentOpenSection = sectionName;
            }
        };

        $scope.isDirty = () => {
            return !_.isEqual($scope.values, $scope.originalValues);
        };

        $scope.proxyPasswordOrUsernameIsRequired = () => {
            if (!$scope.settingsForm.proxy_password.$viewValue && !$scope.settingsForm.proxy_username.$viewValue) {
                $scope.settingsForm.proxy_password.$setValidity('password-is-required', true);
                $scope.settingsForm.proxy_username.$setValidity('username-is-required', true);
            }

            if ($scope.settingsForm.proxy_username.$viewValue && $scope.settingsForm.proxy_username.$viewValue.length > 0) {
                $scope.settingsForm.proxy_username.$setValidity('username-is-required', true);
                if (!$scope.settingsForm.proxy_password.$viewValue || $scope.settingsForm.proxy_password.$viewValue.length === 0) {
                    $scope.settingsForm.proxy_password.$setValidity('password-is-required', false);
                }
            }

            if ($scope.settingsForm.proxy_password.$viewValue && $scope.settingsForm.proxy_password.$viewValue.length > 0) {
                $scope.settingsForm.proxy_password.$setValidity('password-is-required', true);
                if (!$scope.settingsForm.proxy_username.$viewValue || $scope.settingsForm.proxy_username.$viewValue.length === 0) {
                    $scope.settingsForm.proxy_username.$setValidity('username-is-required', false);
                }
            }
        };

        $scope.isPristine = () => {
            return _.isEqual($scope.values, $scope.originalValues) && _.isEqual($scope.configuration, $scope.originalConfiguration);
        };

        $scope.resetFocus = () => {
            const focusHolderElement = $('.focus-holder')[0];
            if (focusHolderElement) {
                focusHolderElement.focus();
            }
        };

        $scope.initialize();

        nocNavigateAway.promptOnLeaveBeforeDiscardingChanges($scope);
    });
})();
