(() => {
    'use strict';

    angular.module('noc').controller('uploadNlfComponentController', function (nocUploader) {
        const isolatedScope = this;

        this.$onInit = () => {
            nocUploader.call(this, this.nocHttp);

            isolatedScope.licenseFile = {};
            isolatedScope.failedToLoad = {};
            isolatedScope.errorMessage = {};
            isolatedScope.serverError = {};
            isolatedScope.uploadProblem = {};
            isolatedScope.licenseFileName = {};
            isolatedScope.expirationDate = {};
            isolatedScope.licenseSerialNumber = {};

            isolatedScope.nodesIndexes = isolatedScope.isHa ? [1, 2] : [1];
        };

        this.onUpload = (id) => {
            const node = (id.substr(16)); //HiddenInputNode1 --> 1
            this.uploadProblem[node] = false;
            const inputId = '#' + id;
            $(inputId).click();
        };

        this.uploadLicenseFile = (hiddenInput, retryNode) => {
            let node, file;
            if (hiddenInput) {
                node = (hiddenInput.id.substr(16)); //HiddenInputNode1 --> 1
                file = hiddenInput.files[0];
                isolatedScope.licenseFile[node] = file;
                angular.element(hiddenInput).val(null);
            } else {
                node = retryNode;
                file = isolatedScope.licenseFile[node];
            }

            const descriptors = [{
                fieldName: 'filename',
                file,
                uploadFormField: 'license'
            }];
            const uploadUrl = this.pathPrefix + 'working-environments/parse-license-file';
            const ascii = /^[ -~]+$/;

            if (file) {
                this.upload('', descriptors, uploadUrl)
                    .then((response) => {
                        isolatedScope.licenseFileName[node] = file.name;
                        isolatedScope.licenseSerialNumber[node] = response.serialNumber;
                        isolatedScope.expirationDate[node] = moment(response.contractEndDate).format('ll');
                        if (isolatedScope.workingEnvironmentType === 'VSA') {
                            isolatedScope.providedLicense = response.license;
                        } else if (isolatedScope.workingEnvironmentType === 'HA') {
                            isolatedScope['providedLicenseNode' + node] = response.license;
                        } else {
                            throw new Error('Unknown provider ' + isolatedScope.workingEnvironmentType);
                        }
                    })
                    .catch((error) => {
                        isolatedScope.licenseFileName[node] = file.name;
                        isolatedScope.uploadProblem[node] = true;
                        isolatedScope.serverError[node] = error.message;
                        if (error.status === 400 || error.status === 401 || !ascii.test(file.name)) {
                            isolatedScope.failedToLoad[node] = false;
                            isolatedScope.errorMessage[node] = 'The file format is invalid';
                            if (!ascii.test(file.name)) {
                                isolatedScope.serverError[node] = 'The filename must not contain special characters.';
                            }
                        } else {
                            isolatedScope.failedToLoad[node] = true;
                            isolatedScope.errorMessage[node] = 'Failed to load the file';
                        }
                    });
            }
        };

        this.removeLicense = (node) => {
            this.licenseFileName[node] = '';
            this.licenseSerialNumber[node] = '';
            this.expirationDate[node] = '';
            this.errorMessage[node] = '';
            isolatedScope.failedToLoad[node] = false;
            isolatedScope.uploadProblem[node] = false;
        };

        this.isLicenseUnique = (node) => {
            return node === 2 && isolatedScope.licenseSerialNumber[2] && (isolatedScope.licenseSerialNumber[1] === isolatedScope.licenseSerialNumber[2]);
        };
    });
})();
