(() => {
    'use strict';

    function baseAuditControllerFn($scope, $location, $rootScope, $state, auditResource, nocUI, pidgeon,
                                   nocDialog, nocSelectedWorkspaceOrTenantService, $filter) {
        $scope.isAgent = nocSelectedWorkspaceOrTenantService.getIsAgentMode();

        var locationChangedOffFnc = _.noop;

        var fromWe = $state.params.fromWe;
        var currentWeId = null;

        if(fromWe) {
            $scope.hideWEandTenant = true;
            currentWeId = $state.params.weId;
        }

        function buildFilterItem(param, filterParam) {
            if(param !== 'all') {
                $scope.filteredColumns[filterParam] = {};
                $scope.filteredColumns[filterParam].name = param;
            } else {
                delete $scope.filteredColumns[filterParam];
            }
        }

        function updateFilters() {
            var current = $state.params;
            var params = $location.search();
            buildFilterItem(params.action || 'all', 'filteredAction');
            buildFilterItem(params.tenant || 'all', 'filteredTenant');
            buildFilterItem(params.user || 'all', 'filteredUser');
            buildFilterItem(params.we || current.we || 'all', 'filteredWe');
            buildFilterItem(params.status || 'all', 'filteredStatus');

            $scope.filterSummaries();
            $scope.createColumnLists();
        }

        locationChangedOffFnc = $rootScope.$on('$locationChangeSuccess', updateFilters);

        $scope.initialize = function(){
            //keeps track of open rows, so we can restore after update
            $scope.openedNodes = auditResource.openedNodes;
            $scope.summaries = auditResource.auditItems;
            $scope.rawSummaries = [];
            $scope.hideSystem = {};
            $scope.filteredColumns = {};

            $scope.statuses = {
                "Success":{icon:"fa-check",name:"Success"},
                "Received":{icon:"fa-refresh fa-spin",name:"In Progress"},
                "Failed":{icon:"fa-times",name:"Failed"}
            };

            $scope.selectedTimeframe = {name: "Live", displayName: "Time: Live", polling: true, payload: {limit: 25}};

            $scope.timeframes = [
                $scope.selectedTimeframe,
                {name: "Today", displayName: "Today", payload: {onlyGroup:true, after: moment().startOf('day').valueOf()}},
                {name: "Last 7 Days", displayName: "Last 7 Days", payload: {onlyGroup:true, after: moment().startOf('day').subtract(7, 'days').valueOf()}},
                {name: "All", displayName: "Time: All", payload: {onlyGroup:true}}
            ];

            $scope.changeTimeframe($scope.selectedTimeframe);
            updateFilters();
        };

        var unloading = _.noop;
        var pollStopper = _.noop;

        var reset = function () {
            unloading();
            pollStopper();
            auditResource.reset();
        };

        $scope.changeTimeframe = function (timeframe) {
            reset();
            unloading = nocUI.loading();
            if(timeframe.polling) {
                pollStopper = auditResource.startPolling(timeframe.payload, currentWeId);
            } else {
                auditResource.get(timeframe.payload, currentWeId);
            }
        };

        $scope.$on(auditResource.AUDIT_GET_EVENT, () => {
            $scope.filterSummaries();
            $scope.createColumnLists();
            pidgeon.clear('audit');
            unloading();
        });

        $scope.$on(auditResource.AUDIT_GET_ERROR_EVENT, function (event, data) {
            pidgeon.error(pidgeon.serverErrorPrefix + "retrieving list of audit records", data.error, "audit");
            unloading();
        });

        $scope.createColumnLists = function createColumnLists() {
            function mapSortUnique(arr, field) {
                const columnList = _(arr)
                    .map(field)
                    .uniq()
                    .map((name) => {
                        return {name: name || '(None)'};
                    })
                    .sortBy((obj) => _.toLower(obj.name))
                    .value();

                return columnList.length > 1 ? columnList : [];
            }

            $scope.columnLists = {};
            $scope.columnLists.tenantList = mapSortUnique($scope.filteredSummaries, 'tenantName');
            $scope.columnLists.actionList = mapSortUnique($scope.filteredSummaries, 'actionName');
            $scope.columnLists.weList = mapSortUnique($scope.filteredSummaries, 'workingEnvironmentName');
            $scope.columnLists.userList = mapSortUnique($scope.filteredSummaries, 'userName');
            $scope.columnLists.statusList = [];

            _.forEach($scope.filteredSummaries, (summary) => {
                if (summary.Failed) {
                    $scope.columnLists.statusList.push({name: 'Failed'});
                }
                if (summary.Success) {
                    $scope.columnLists.statusList.push({name: 'Success'});
                }
                if (summary['In Progress']) {
                    $scope.columnLists.statusList.push({name: 'In Progress'});
                }
            });
            $scope.columnLists.statusList = _.uniqBy($scope.columnLists.statusList, 'name');
            $scope.columnLists.statusList = $scope.columnLists.statusList.length > 1 ? _.sortBy($scope.columnLists.statusList, 'name') : [];

            if (_.find($scope.columnLists.userList, {name: 'System'}) && $scope.columnLists.userList.length > 2) {
                $scope.columnLists.userList.unshift({name: '(All without system)'});
            }

            $scope.columnLists.enableSearch = {
                tenant: $scope.columnLists.tenantList.length > 4,
                action: $scope.columnLists.actionList.length > 4,
                we: $scope.columnLists.weList.length > 4,
                user: $scope.columnLists.userList.length > 4
            };
        };

        $scope.filterSummaries = () => {
            _.forEach($scope.summaries, (summary) => {
                if (summary.status === 'Failed' || summary.containsFailedRecords) {
                    summary['Failed'] = true;
                }
                if (summary.status === 'Success') {
                    summary['Success'] = true;
                }
                if (summary.status === 'Received') {
                    summary['In Progress'] = true;
                }
            });

            $scope.filteredSummaries = $filter('timelineFilter')($scope.summaries, $scope.filteredColumns);
        };

        $scope.toggleOpen = function(node){
            node._open = !node._open;
            auditResource.openedNodes[node.id] = node._open;
            //keep a list of the opened nodes. Needs to be reconstructed during refresh
            if(node._open){
                auditResource.expandGroup(node);
            }

        };

        $scope.auditProperties = function (audit) {
            return _.transform(audit, function (result, value, key) {
                //ignore private properties
                if (/^_.+|id/.test(key)) return;

                //handled differently
                if (/records|actionParameters|parameters/.test(key)) return;

                //no value
                if (!value) return;

                //convert to date
                if (/startDate|endDate|date/.test(key)) {
                    value = $filter('dateAndTime')(value);
                }

                result[key] = value;
            });
        };

        $scope.showAuditProperties = function (audit) {
            nocDialog.dialog(audit.actionName, "assets/templates/dialogs/audit-details.html", {audit:audit, auditProperties: $scope.auditProperties});
        };

        $scope.toggleShowParameters = function(node){
            node._open = node._open === "showParameters" ? true : "showParameters";
            auditResource.openedNodes[node.id] = node._open;
            _.forEach(node._records, function(child){
                child._open = node._open;
                auditResource.openedNodes[child.id] = child._open;
            });
        };

        $scope.$on('$destroy', function () {
            reset();
            locationChangedOffFnc();
        });

        $scope.$watch('filteredColumns', function(){

            $location.search("action", $scope.filteredColumns.filteredAction  ? $scope.filteredColumns.filteredAction.name : 'all');
            $location.search("tenant", $scope.filteredColumns.filteredTenant  ? $scope.filteredColumns.filteredTenant.name : 'all');
            $location.search("user", $scope.filteredColumns.filteredUser  ? $scope.filteredColumns.filteredUser.name : 'all');
            $location.search("we", $scope.filteredColumns.filteredWe  ? $scope.filteredColumns.filteredWe.name : 'all');
            $location.search( "status", $scope.filteredColumns.filteredStatus  ? $scope.filteredColumns.filteredStatus.name : 'all');
        }, true);

        $scope.initialize();
    }

    angular.module("noc").controller("AuditController", baseAuditControllerFn);

    angular.module("noc").controller("WorkingEnvironmentAuditController", function ($scope, $injector, nocUI, pidgeon) {
        nocUI.loading = _.constant(_.noop);

        $scope.workingEnvironmentRestrict = true;
        $scope.hideWEandTenant = true;

        $injector.invoke(baseAuditControllerFn, this, {
            $scope: $scope,
            nocUI: nocUI,
            pidgeon: pidgeon
        });

    });

    angular.module("noc").controller("VolumeViewAuditController", function($scope, $injector, nocUI, pidgeon){
        $scope.workingEnvironmentRestrict = false;
        $scope.hideWEandTenant = true;

        $injector.invoke(baseAuditControllerFn, this, {
            $scope: $scope,
            nocUI: nocUI,
            pidgeon: pidgeon
        });
    });

    angular.module("noc").directive('nocAuditText', function ($compile,auditTemplateFinder) {
        var linker = function(scope, element) {
            var actionName = scope.data && scope.data.actionName;
            var version = scope.data && scope.data.version;
            scope.isNumber = _.isNumber;
            var template = auditTemplateFinder.find(actionName, version);
            var compiled = $compile(template)(scope);
            element.prepend(compiled);
        };

        return {
            restrict:"E",
            replace: false,
            link: linker,
            scope:{
                "data":"="
            }
        };
    });

    angular.module("noc").directive('nocAuditRecordText', function ($compile,auditTemplateFinder) {
        var linker = function(scope, element) {
            var actionName = scope.data && scope.data.actionName;
            var template = auditTemplateFinder.findRecord(actionName);
            var compiled = $compile(template)(scope);
            element.prepend(compiled);
        };

        return {
            restrict:"E",
            replace: false,
            link: linker,
            scope:{
                "data":"="
            }
        };
    });
})();
