(() => {
    'use strict';

    angular.module('noc').controller('KeyManagerController', ($scope, nocUI, pidgeon, nocHttp, nocDialog, $window, $timeout,
                                                              Constants, nocEncryptionResourcesService, uiGridConstants) => {
        $scope.initialize = function () {
            $scope.getKeyManagerPort();
            $scope.getKeyManagers();
            $scope.getKeyManagersCaCertificates();
            $scope.usernameFields = ["", "CommonName", "EmailAddress", "OrganizationalUnitName"];
        };

        $scope.getKeyManagerPort = function () {
            nocHttp.get('/occm/config','retrieving key manager port',nocHttp.LOADING)
                .then(function (config) {
                    $scope.keyManagerPort = config.keyManagerPort;

                    //Sanity, should never happen:
                    if (isNaN($scope.keyManagerPort) || $scope.keyManagerPort < 1 || $scope.keyManagerPort > 65535) {
                        pidgeon.error('There was a problem with the key manager port. Please reconfigure it.', null, 'keyManager');
                    }
                    $scope.originalKeyManagerPort = $scope.keyManagerPort;
                });
        };

        $scope.saveKeyManagerPort = function () {
            if ($scope.keyManagerPortForm.$valid) {
                nocHttp.put({url:'/occm/config', data:{
                    "keyManagerPort": $scope.keyManagerPort
                }}, 'updating key manager port', nocHttp.LOADING)
                    .then(function () {
                        pidgeon.success("Key manager port updated to " + $scope.keyManagerPort + ".");
                        $scope.originalKeyManagerPort = $scope.keyManagerPort;
                        $scope.portChangeMode = false;
                    });
            } else {
                nocUI.handleFormValidity($scope.keyManagerPortForm, $scope);
            }
        };

        $scope.getKeyManagers = function () {
            return nocHttp.get('/occm/encryption/key-managers', 'retrieving key managers', nocHttp.LOADING)
                .then(function (keyManagers) {
                    $scope.keyManagers = keyManagers;
                    _.forEach($scope.keyManagers,function (km) {
                        km._usernameString = km.usernameField ? (km.usernameValue + " (in field " + _.startCase(km.usernameField) + ")") : undefined;
                    });
                    $scope.gridOptionsKeyManagers.data = $scope.keyManagers;
                });
        };

        $scope.addOrEditKeyManager = function (keyManagerToEdit) {
            var params = {
                usernameFields: $scope.usernameFields
            };
            _.assignIn(params, keyManagerToEdit);
            nocDialog.dialog(
                (keyManagerToEdit ? "Edit" : "Add") + " Key Manager",
                "assets/templates/tabs/encryption/add-or-edit-key-manager.html",
                params
            )
                .then($scope.getKeyManagers);
        };

        $scope.deleteKeyManager = function (keyManager) {
            nocDialog
                .confirm("Delete Key Manager?", "assets/templates/tabs/encryption/delete-key-manager-dialog.html", {keyManagerName: keyManager.name})
                .then(function () {
                    return nocHttp.delete('/occm/encryption/key-managers/' + keyManager.publicId, 'deleting key manager', nocHttp.LOADING)
                        .then($scope.getKeyManagers);
                });
        };

        $scope.getKeyManagersCaCertificates = function () {
            nocEncryptionResourcesService.getKeyManagersCaCertificates(nocHttp)
                .then(function (results) {
                    $scope.keyManagersCaCertificates = results;
                    $scope.gridOptionsKeyManagersCaCertificate.data = $scope.keyManagersCaCertificates;
                });
        };

        $scope.addKeyManagerCaCertificate = function () {
            nocDialog.dialog("Add Key Manager CA certificate", "assets/templates/tabs/encryption/add-key-manager-ca-certificate.html", {
                width: Constants.dialogWidthForDisplayingCertificate
            })
                .then($scope.getKeyManagersCaCertificates);
        };

        $scope.showKeyManagerCaCertificates = function (keyManagerCaCertificate) {
            nocDialog.confirm("Key Manager CA Certificate", "assets/templates/tabs/encryption/show-certificate.html", {
                certificate: keyManagerCaCertificate.certificate,
                width: Constants.dialogWidthForDisplayingCertificate,
                callback: $scope.selectCertificateText
            });
        };

        $scope.deleteKeyManagerCaCertificates = function (keyManagerCaCertificate) {
            nocDialog
                .confirm("Delete Key Manager CA Certificate?", "assets/templates/tabs/encryption/delete-key-manager-ca-certificate-dialog.html", {subject: keyManagerCaCertificate.subjectName})
                .then(function () {
                    return nocHttp.delete('/occm/encryption/key-managers-ca-certificates/' + keyManagerCaCertificate.publicId,
                        'deleting key manager CA certificate',
                        nocHttp.LOADING)
                        .then($scope.getKeyManagersCaCertificates);
                });
        };

        $scope.initialize();

        var rowTemplate = "<div ng-mouseenter='grid.appScope.over(row.entity)' ng-mouseleave='grid.appScope.leave(row.entity)' ng-repeat=\"(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name\" class=\"ui-grid-cell\" ng-class=\"{ 'ui-grid-row-header-cell': col.isRowHeader, 'hovered': grid.appScope.hoveredRow === row.entity }\" ui-grid-cell></div>";
        $scope.hoveredRow = null;

        $scope.over = function(entity) {
            $scope.hoveredRow = entity;
        };

        $scope.leave = function(entity) {
            if ($scope.hoveredRow === entity){
                $scope.hoveredRow = null;
            }
        };

        function setGridHeight() {
            var uigrid = $('[ui-grid]'),
                top = uigrid.offset() && uigrid.offset().top;
            uigrid.removeClass('visible');
            var footerHeight = 30; // px
            uigrid.height($window.innerHeight - footerHeight - top - 20);
            $timeout(function () {
                uigrid.addClass('visible');
            });
        }

        $(window).on("resize", setGridHeight);

        const cellTemplateWithTitle = '<div class="ui-grid-cell-contents" title="{{COL_FIELD}}">{{COL_FIELD}}</div>';

        $scope.gridOptionsKeyManagers = {
            columnDefs: [
                {displayName: "Name", visible: true, field: "name", width: "150", cellTemplate: cellTemplateWithTitle},
                {displayName: "IP", visible: true, field: "address", width: "150", cellTemplate: cellTemplateWithTitle},
                {displayName: 'Client Authentication', visible: true, field: '_usernameString', width: '*', cellTemplate: cellTemplateWithTitle}
            ],
            enableColumnResizing: false,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            rowTemplate: rowTemplate,
            onRegisterApi: function(gridAPI) {
                $scope.gridAPI = gridAPI;
                $scope.gridAPI.core.addRowHeaderColumn({ name: 'rowHeaderCol', displayName: '', width: 60, enableColumnResizing: false, cellTemplate: "assets/templates/tabs/encryption/key-managers-popup-btn.html"});
                $timeout(setGridHeight);
                $scope.$on(Constants.HEADER_HEIGHT_CHANGED_EVENT, setGridHeight);
            }
        };

        $scope.gridOptionsKeyManagersCaCertificate = {
            columnDefs: [
                {
                    displayName: 'Expiration',
                    visible: true,
                    field: 'endDate',
                    width: 158,
                    cellTemplate: '<div class="ui-grid-cell-contents" ng-class="row.entity._expirationState.state" ng-attr-title="{{row.entity._expirationState.message}}">{{COL_FIELD | fullDate}}</div>'
                },
                {
                    displayName: 'Issuer',
                    visible: true,
                    field: 'directIssuerName',
                    width: 180,
                    cellTemplate: cellTemplateWithTitle
                },
                {
                    displayName: 'Subject',
                    visible: true,
                    field: 'subjectName',
                    width: '*',
                    cellTemplate: cellTemplateWithTitle
                }
            ],
            enableColumnResizing: false,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            rowTemplate,
            onRegisterApi: (gridAPI) => {
                $scope.gridAPI = gridAPI;
                $scope.gridAPI.core.addRowHeaderColumn({
                    name: 'rowHeaderCol',
                    displayName: '',
                    width: 60,
                    enableColumnResizing: false,
                    cellTemplate: 'assets/templates/tabs/encryption/key-managers-ca-certificates-popup-btn.html'
                });
                $timeout(setGridHeight);
                $scope.$on(Constants.HEADER_HEIGHT_CHANGED_EVENT, setGridHeight);
            }
        };

        $scope.$on("$destroy", function() {
            $(window).off("resize", setGridHeight);
        });
    });
})();
