(() => {
    'use strict';

    angular.module('noc').controller('HttpsCertificateController', ($scope, Constants, nocDialog, nocUI, nocHttp, nocEncryptionResourcesService,
                                                                    $location, nocUploader, $timeout) => {
        $scope.initialize = function () {
            $scope.alreadyGenerated = false;
            $scope.changeCsrCommonNameWarningGiven = false;
            $scope.commonName = "";
            $scope.selectedCertificateSetup = 'generate-csr';
            nocUploader.call($scope, nocHttp);
            nocHttp.get("/occm/key-store/signed-certificate", "retrieving certificate", nocHttp.LOADING)
                .then(function (certificate) {
                    if (certificate) {
                        $scope.switchToCertificateInstalledStep(certificate);
                        $scope.alreadyGenerated = true;
                        $scope.commonName = $scope.extractCommonNameFromCertificate(certificate);
                    } else {
                        $scope.step = 0;
                        nocHttp.get("/occm/key-store/self-certificate", "retrieving certificate", nocHttp.LOADING)
                            .then(function (certificate) {
                                if (certificate) {
                                    $scope.step = 1;
                                    $scope.commonName = $scope.extractCommonNameFromCertificate(certificate);
                                    $scope.alreadyGenerated = true;
                                } else {
                                    $scope.step = 0;
                                    $scope.alreadyGenerated = false;
                                }
                            });
                    }
                });
        };

        $scope.certificateMode = function certificateMode() {
            $scope.certificateFileName = null;
            $scope.privateKeyFileName = null;
            if ($scope.selectedCertificateSetup === 'install-certificate') {
                $scope.step = 2;
                $scope.externalCertificate = true;
            } else {
                $scope.step = 1;
                $scope.externalCertificate = false;
            }
        };

        $scope.isInstallBtnDisabled = function() {
            return ($scope.step === 1 && !$scope.certificateFileName) || ($scope.step === 2 && !($scope.privateKeyFileName && $scope.certificateFileName));
        };

        $scope.switchToCertificateInstalledStep = function(certificate){
            $scope.step = 3;
            nocEncryptionResourcesService.formatCertificate(certificate);
            $scope.certificate = certificate;
        };

        $scope.extractCommonNameFromCertificate = function(certificate) {
            // get common name
            var commonNameRegex = /CN=((\w|\*|\.|_|-)+)/; // allowed characters in common name: any alphanumeric, asterisk (*), period (.), underscore (_) and hyphen (-)
            var commonNameMatch = commonNameRegex.exec(certificate.subjectName);
            var commonName = commonNameMatch? commonNameMatch[1] : "";
            // add alternative names
            commonName += certificate.alternativeNames ? ";" + certificate.alternativeNames : "";

            return commonName;
        };

        $scope.regenerateCsr = function(commonName){
            nocUI
                .formValidate($scope.isCsrFormValid)
                .then(function(){
                    nocDialog.confirm($scope.certificate ? "The currently installed certificate will be uninstalled. Cloud Manager will use a self-signed certificate until you install the new CA-signed certificate." : "Regenerating a CSR will override the previously generated CSR.")
                        .then(function(){
                            $scope.step = 1;
                            $scope.generateCsr(commonName);
                        });
                });
        };

        $scope.generateCsr = function(commonName){
            nocUI
                .formValidate($scope.isCsrFormValid)
                .then(function(){
                    nocHttp.post({
                        url: '/occm/key-store',
                        data: {commonName: _.trim(commonName)}
                    }, 'generating CSR', nocHttp.BLOCK)
                        .then(function(csr){
                            $scope.step = 1;
                            $scope.viewCsr(csr.csr);
                            $scope.changeCsrCommonNameWarningGiven = false;
                            $scope.alreadyGenerated = true;
                            delete $scope.certificate;
                        });
                });
        };

        $scope.isCsrFormValid = function(){
            return $scope.csrForm.$valid;
        };

        $scope.onUploadCRT = function() {
            $("#hiddenCertificateInputButton").click();
        };

        $scope.onUploadPPK = function() {
            $("#hiddenPrivateKeyInputButton").click();
        };

        $scope.uploadCertificateFile = function(e) {
            $timeout(function() {
                if(e.files[0]) {
                    $scope.certificateFile = e.files[0];
                }
                $scope.certificateFileName = $scope.certificateFile.name;
                angular.element(e).val(null);
            }, 1); //timeout actives digest cycle so that file name appears instantly in input box
        };

        $scope.uploadPrivateKeyFile = function(e) {
            $timeout(function() {
                if(e.files[0]) {
                    $scope.privateKeyFile = e.files[0];
                }
                $scope.privateKeyFileName = $scope.privateKeyFile.name;
                angular.element(e).val(null);
            }, 1); //timeout actives digest cycle so that file name appears instantly in input box
        };

        $scope.uploadAndInstallCertificateFile = function uploadAndInstallCertificateFile() {
            var uploadFormFieldCRT, uploadFormFieldPPK, fileCRTFileNameFieldName, filePPKFileNameFieldName, uploadUrl;
            if ($scope.externalCertificate) {
                uploadFormFieldCRT = "certificate";
                uploadFormFieldPPK = "privateKey";
                fileCRTFileNameFieldName = "certificateFilename";
                filePPKFileNameFieldName = "privateKeyFilename";
                uploadUrl = '/occm/key-store/external-certificate-file';
            } else {
                fileCRTFileNameFieldName = "filename";
                uploadFormFieldCRT = "certificate";
                uploadUrl = '/occm/key-store/signed-certificate-file';
            }

            var descriptors = [{
                fieldName: fileCRTFileNameFieldName,
                file: $scope.certificateFile,
                uploadFormField: uploadFormFieldCRT
            }];

            if (uploadFormFieldPPK) {
                descriptors.push({
                    fieldName: filePPKFileNameFieldName,
                    file: $scope.privateKeyFile,
                    uploadFormField: uploadFormFieldPPK
                });
            }
            $scope.upload("uploading certificate file", descriptors, uploadUrl)
                .then(function (certificate) {
                    $scope.switchToCertificateInstalledStep(certificate);
                    removeRenwalFromSearch();
                    nocUI.reloadSilently();
                });
        };

        $scope.csrCommonNameChanged = () => {
            if (!$scope.changeCsrCommonNameWarningGiven && $scope.step === 2) {
                nocDialog.alert('Changing the CN name requires regenerating the CSR.');
            }
            $scope.changeCsrCommonNameWarningGiven = true;
        };

        var showCertificateDialog = function(title, text){
            nocDialog.confirm(title, "assets/templates/tabs/encryption/show-certificate.html", {
                certificate: text,
                width: Constants.dialogWidthForDisplayingCertificate,
                callback: function(){
                    $("textarea.show-certificate").select();
                }
            });
        };

        $scope.viewCsr = function(csr){
            showCertificateDialog("Copy this CSR and submit a certificate request to a CA", csr);
        };

        $scope.viewCertificate = function(){
            showCertificateDialog("Your Cloud Manager Certificate", $scope.certificate.certificate);
        };

        var lastUrl = "";
        $scope.changeCertificate = function(){
            if($scope.externalCertificate) {
                $scope.step = 2;
            } else {
                $scope.step = 1;
            }
            $scope.changeCsrCommonNameWarningGiven = false;
            $location.search('renewal', 'true');
            lastUrl = $location.absUrl();
        };

        var removeRenwalFromSearch = function() {
            var search = $location.search();
            delete search.renewal;
            $location.search(search);
        };

        $scope.$on("$locationChangeStart", function(event, newUrl) {
            if (newUrl === _(lastUrl).replace('?renewal=true', '').replace('&renewal=true', '')) {
                $scope.step = 3;
            }
        });

        $scope.$on('$destroy', removeRenwalFromSearch);

        $scope.initialize();
    });
})();
