(() => {
    'use strict';

    angular.module('noc').controller('TrustedCAController', ($scope, Constants, nocDialog, nocHttp, nocHttpsResourcesService,
                                                             Messages, uiGridConstants, pidgeon) => {
        var rowTemplate = "<div ng-mouseenter='grid.appScope.over(row.entity)' ng-mouseleave='grid.appScope.leave(row.entity)' ng-repeat=\"(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name\" class=\"ui-grid-cell\" ng-class=\"{ 'ui-grid-row-header-cell': col.isRowHeader, 'hovered': grid.appScope.hoveredRow === row.entity }\" ui-grid-cell></div>";
        var statusCellTemplate = "assets/templates/tabs/https-setup/trusted-status-cell-template.html";
        const viewCertificateCellTemplate = '<div class="ui-grid-cell-contents" title="{{COL_FIELD}}"><span ng-click="grid.appScope.viewTrustedCaCertificate(row.entity)" class="view-certificate"><i class="fa fa-info-circle"></i>View Certificate</div>';
        const cellTemplateWithTitle = '<div class="ui-grid-cell-contents" title="{{COL_FIELD}}">{{COL_FIELD}}</div>';

        var getTrustedCAs = function () {
            nocHttpsResourcesService.getTrustedCaCertificates(nocHttp)
                .then(function (results) {
                    $scope.trustedCaCertificates = results;
                    $scope.gridOptionsTrustedCA.data = $scope.trustedCaCertificates;
                });
        };

        $scope.initialize = function () {
            getTrustedCAs();
        };

        $scope.restartOCCM = function () {
            nocHttpsResourcesService.restartOCCM(nocHttp, pidgeon);
        };

        $scope.addTrustedCA = function () {
            nocDialog
                .confirm("Add a new CA-signed certificate", "assets/templates/dialogs/add-trusted-ca.html", {
                    width: 700
                })
                .then(getTrustedCAs);
        };

        $scope.deletedTrustedCaCertificate = function (certificate) {
            nocDialog
                .confirm(Messages.deleteTrustedCAConfirm)
                .then(function() {
                    nocHttp.delete('/occm/key-store/trusted-certificate/' + certificate.id,
                        "deleting the trusted CA certificate from Cloud Manager",
                        nocHttp.LOADING)
                        .then(getTrustedCAs);
                });
        };

        $scope.viewTrustedCaCertificate = function (trustedCaCertificate) {
            nocDialog.confirm("Trusted CA Certificate", "assets/templates/tabs/encryption/show-certificate.html", {
                certificate: trustedCaCertificate.certificate,
                width: Constants.dialogWidthForDisplayingCertificate,
                callback: $scope.selectCertificateText
            });
        };

        $scope.hoveredRow = null;

        $scope.over = function(entity) {
            $scope.hoveredRow = entity;
        };

        $scope.leave = function(entity) {
            if ($scope.hoveredRow === entity){
                $scope.hoveredRow = null;
            }
        };

        $scope.gridOptionsTrustedCA = {
            columnDefs: [
                {displayName: 'Expiration', visible: true, field: 'endDate', width: 158, cellTemplate: '<div class="ui-grid-cell-contents"  ng-attr-title="{{row.entity._expirationState.message}}">{{COL_FIELD | fullDate}}</div>'},
                {displayName: 'Issuer', visible: true, field: 'directIssuerName', width: '*', cellTemplate: cellTemplateWithTitle},
                {displayName: 'Subject', visible: true, field: 'subjectName', width: '*', cellTemplate: cellTemplateWithTitle},
                {displayName: 'Certificate', visible: true, field: 'viewCertificate', width: 180, cellTemplate: viewCertificateCellTemplate},
                {displayName: 'Status', visible: true, field: '_statusText', width: '*', cellTemplate: statusCellTemplate},
                {displayName: '', visible: true, width: 50, field: 'delete', cellTemplate: '<button ng-click="grid.appScope.deletedTrustedCaCertificate(row.entity)" ng-show="row.entity.status !== \'PENDING_DELETED\'" class="delete-certificate"><noc-icon name="trash"></noc-icon></button>'}

            ],
            enableColumnResizing: false,
            rowTemplate,
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            onRegisterApi: (gridAPI) => {
                $scope.gridAPI = gridAPI;
            }
        };

        $scope.initialize();
    });
})();
