(() => {
    'use strict';

    angular.module('noc').controller('TenantsNewEditController', ($scope, Messages, nocUI, nocHttp, nocNavigateAway, $log,
                                                                  nocDialog, pidgeon, tenantsMode, $state, $q, nocTenantsService, nocCurrentUserService) => {
        $scope.tenantsMode = tenantsMode;

        $scope.isAllowed = (action) => {
            return nocCurrentUserService.isAllowed(action);
        };

        $scope.resolveTenantByPublicId = (publicId, tenants) => {
            if (!tenants) {
                return nocHttp.get('/tenants/' + publicId + '?fields=workingEnvironmentsInformation', null, nocHttp.LOADING);
            } else {
                const tenant = _.find(tenants, {publicId});

                return tenant ? $q.when(tenant) : $q.reject();
            }
        };

        /*istanbul ignore next*/
        $scope.initializeEditMode = (tenant) => {
            $scope.originalSelectedTenant = tenant;
            $scope.selectedTenant = _.cloneDeep($scope.originalSelectedTenant);
        };

        $scope.initializeController = (tenantsMode) => {
            if (tenantsMode === 'edit') {
                $scope.resolveTenantByPublicId($state.params.tenantId, $scope.tenants)
                    .then($scope.initializeEditMode, $scope.backToGallery);
            } else {
                $scope.selectedTenant = {};
                $scope.originalSelectedTenant = _.cloneDeep($scope.selectedTenant);
            }
        };

        $scope.initializeController($scope.tenantsMode);

        $scope.isValid = () => {
            return $scope.tenantsForm.$valid;
        };

        $scope.isDirty = () => {
            return !_.isEqual($scope.originalSelectedTenant, $scope.selectedTenant);
        };

        $scope.deleteSelectedTenant = () => {
            nocTenantsService.reloadTenants();
            if ($scope.tenants.length < 2) {
                pidgeon.error('The last tenant cannot be deleted.');
            } else {
                nocDialog
                    .confirm(Messages.deleteConfirm({type: 'tenant', name: $scope.selectedTenant.name}))
                    .then(() => {
                        nocHttp.delete('/tenants/' + $scope.selectedTenant.publicId, 'deleting tenant', nocHttp.BLOCK)
                            .then(() => {
                                $scope.deleteTenant($scope.originalSelectedTenant);
                                $scope.updateOriginalTenant();
                                // If we are deleting the currently selected tenant
                                if ($state.params.slTenantId === $scope.selectedTenant.publicId) {
                                    $scope.backToGallery($scope.tenants[0] && $scope.tenants[0].publicId);
                                } else {
                                    $scope.backToGallery();
                                }
                            });
                    });
            }
        };

        $scope.saveSelectedTenant = () => {
            if ($scope.isValid()) {
                if ($scope.isDirty()) {
                    nocTenantsService.reloadTenants();
                    const clonedTenant = _.clone($scope.selectedTenant);

                    let url = '/tenants';
                    let method = 'post';
                    if ($scope.tenantsMode === 'edit') {
                        url = url + '/' + clonedTenant.publicId;
                        method = 'put';
                    }

                    delete clonedTenant.publicId;
                    delete clonedTenant.workingEnvironmentsInformation;

                    nocHttp[method]({
                        url,
                        data: clonedTenant
                    }, 'saving changes', nocHttp.BLOCK)
                        .then(() => {
                            nocTenantsService.reloadTenants();
                            $scope.updateOriginalTenant();
                            $scope.backToGallery();
                        })
                        .catch($log.error);
                } else {
                    $scope.backToGallery();
                }
            } else {
                nocUI.handleFormValidity();
            }
        };

        $scope.updateOriginalTenant = () => {
            const index = _.indexOf($scope.tenants, $scope.originalSelectedTenant);
            if (index > -1) {
                $scope.tenants[index] = $scope.selectedTenant;
                nocTenantsService.getTenants()[index] = $scope.selectedTenant;
            }
            $scope.originalSelectedTenant = $scope.selectedTenant;
        };

        nocNavigateAway.promptOnLeaveBeforeDiscardingChanges($scope);
    });
})();
