(() => {
    'use strict';

    angular.module('noc').controller('UsersController', ($scope, $rootScope, nocUtil, nocHttp, nocFriendlyStore, $q, nocTenantsService,
                                                         usersTableColumnsService, $timeout, $window, Messages, nocDialog, uiGridFilterService, Constants,
                                                         workingEnvironmentResource, usersResource, uiGridConstants, nocCurrentUserService) => {
        $scope.currentOpenSection = '';
        const instancePlacement = $rootScope.instancePlacement;
        $scope.hasInstanceProfile = !!(instancePlacement && instancePlacement.role && instancePlacement.providerRoleType === 'AWS');
        $scope.instanceProfileName = $scope.hasInstanceProfile ? instancePlacement.role : null;
        $scope.hasManagedServiceIdentity = !!(instancePlacement && instancePlacement.role && instancePlacement.providerRoleType === 'Azure');
        $scope.portalMode = _.get($rootScope.supportServices, 'portalService.usePortalAuthentication');
        $scope.search = {term: ''};
        $scope.$on('$stateChangeStart', (event, toState, toParams) => {
            if (toState.name === 'main.users.edit') {
                const user = _.find($scope.users, {'publicId': toParams.userId});
                if (!user || (user.role.publicId === 'Role-2' && nocCurrentUserService.getCurrentUserRolePublicId() === 'Role-3')) {
                    event.preventDefault();
                }
            }
        });

        $scope.editGivenUser = (user) => {
            $scope.checkAndSetState('^.edit', {userId: user.publicId});
        };

        $scope.goToUserSettings = (user) => {
            $scope.checkAndSetState('^.settings', {userId: user.publicId});
        };

        $scope.deleteUser = (user) => {
            nocDialog
                .confirm(Messages.deleteConfirm({type: 'user', name: user.name}))
                .then(() => {
                    nocHttp.delete('/users/' + user.publicId, 'deleting user', nocHttp.BLOCK)
                        .then(() => {
                            $scope.users = _.without($scope.users, user);
                            $scope.gridOptions.data = _.without($scope.gridOptions.data, user);
                        });
                });
        };

        $scope.removeUser = (user) => {
            nocDialog
                // .confirm(Messages.deleteConfirm({type: 'user', name: user.name}))
                .confirm('Remove User',
                    'assets/templates/dialogs/users-remove.html', {
                        userName: user.name,
                        dialogOptions: {
                            confirmBtnText: 'Remove',
                            cancelBtnText: 'Cancel'
                        }
                    })
                .then(() => {
                    usersResource.removeUser(nocHttp, user.publicId)
                        .then(() => {
                            $scope.users = _.without($scope.users, user);
                            $scope.gridOptions.data = _.without($scope.gridOptions.data, user);
                        });
                });
        };

        $scope.addUser = () => {
            $scope.checkAndSetState('^.add');
        };

        $scope.newUser = () => {
            $scope.checkAndSetState('^.new');
        };

        $scope.allowedRolesToCurrentRole = {
            'Role-1': {
                'Role-1': true,
                'Role-2': true,
                'Role-3': true
            },
            'Role-2': {
                'Role-2': true,
                'Role-3': true
            },
            'Role-3': {
                'Role-3': true
            }
        };

        $scope.roles = nocCurrentUserService.getRoles();
        $scope.showCostInfo = false;

        $scope.initializeUsers = () => {
            return $q.all([
                workingEnvironmentResource.getWorkingEnvironments(nocHttp),
                usersResource.getUsers(nocHttp)
            ])
                .then(_.spread((workingEnvironments, users) => {
                    $scope.currentUserRolePublicId = nocCurrentUserService.getCurrentUserRolePublicId();
                    $scope.workingEnvironments = workingEnvironments;
                    $scope.users = users;
                    const tenants = nocTenantsService.getTenants();

                    _.forEach($scope.users, (user) => {
                        user.currentUser = user.publicId === nocCurrentUserService.getCurrentUserRolePublicId();
                        user.name = user.firstName + ' ' + user.lastName;
                        // This the assigned working environments full objects
                        user.workingEnvironments = nocUtil.fillObjectsByKeys(user.assignedWorkingEnvironments, $scope.workingEnvironments);
                        user.role = _.find($scope.roles, {publicId: user.roleId});
                        if (user.tenantId) {
                            user.tenant = _.find(tenants, {publicId: user.tenantId});
                        }

                        if ($scope.currentUserRolePublicId === 'Role-1') {
                            user._association = (user.tenant && user.tenant.name) || '--';
                        } else if (user.role.publicId === 'Role-2') {
                            user._association = 'All';
                        } else if (user.workingEnvironments.length > 0) {
                            user._association = _.map(user.workingEnvironments, 'name').join(', ');
                        } else {
                            user._association = '--';
                        }

                        user._accountType = user.isLdap ? 'Active Directory' : 'Cloud Manager';
                    });

                    $scope.gridOptions.data = $scope.users;
                }));
        };

        $scope.generateGrantedWorkingEnvironmentsMap = (user) => {
            const map = {};
            _.forEach(user.workingEnvironments, (workingEnvironment) => {
                map[workingEnvironment.publicId] = true;
            });

            return map;
        };

        const rowTemplate = `<div ng-mouseenter="grid.appScope.over(row.entity)"
                                  ng-mouseleave="grid.appScope.leave(row.entity)"
                                  ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name"
                                  class="ui-grid-cell"
                                  ng-class="{'ui-grid-row-header-cell': col.isRowHeader, 'hovered': grid.appScope.hoveredRow === row.entity, 'disabled': currentUserRolePublicId === 'Role-3' && row.entity.role.publicId === 'Role-2' }"
                                  ui-grid-cell></div>`;
        $scope.hoveredRow = null;

        $scope.over = (entity) => {
            if (nocCurrentUserService.getCurrentUserRolePublicId() === 'Role-3' && entity.role.publicId === 'Role-2') return;
            $scope.hoveredRow = entity;
        };

        $scope.leave = (entity) => {
            if ($scope.hoveredRow === entity) {
                $scope.hoveredRow = null;
            }
        };

        $scope.shouldDisableRow = (user) => {
            return nocCurrentUserService.getCurrentUserRolePublicId() === 'Role-3' && user.role === 'Role-2';
        };

        function setGridHeight() {
            const uiGrid = $('[ui-grid]'),
                top = uiGrid.offset() && uiGrid.offset().top;
            uiGrid.removeClass('visible');
            const footerHeight = 30; //In px
            uiGrid.height($window.innerHeight - footerHeight - top - 20);
            $timeout(() => {
                uiGrid.addClass('visible');
            });
        }

        $(window).on('resize', setGridHeight);

        $scope.gridOptions = {
            columnDefs: usersTableColumnsService.getColumnsDef(),

            rowHeight: 58,
            enableColumnResizing: false,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,

            rowTemplate,

            onRegisterApi: (gridAPI) => {
                uiGridFilterService.installFilter(gridAPI, $scope);
                $timeout(setGridHeight);
                $scope.$on(Constants.HEADER_HEIGHT_CHANGED_EVENT, setGridHeight);
            }
        };

        $scope.$on('$destroy', () => {
            $(window).off('resize', setGridHeight);
        });
    });
})();
