(() => {
    'use strict';

    angular.module('noc').controller('UsersNewEditController', ($scope, $rootScope, nocDialog, nocUI, nocHttp, nocNavigateAway,
                                                                $stateParams, nocTenantsService, usersResource, volViewGeneralResource,
                                                                $state, $log, nocCurrentUserService) => {

        $scope.selectedTenantWE = [];
        $scope.tenants = nocTenantsService.getTenants();
        $scope.allowNotifications = _.get($rootScope, 'supportServices.kinesisEnabled');
        $scope.columnClass = 'col-md-3';
        $scope.usePortalAuthentication = _.get($rootScope, 'supportServices.portalService.usePortalAuthentication');

        $scope.initializeEditor = (userId) => {
            const currentUserRolePublicId = nocCurrentUserService.getCurrentUserRolePublicId();
            if (userId) {
                const user = _.find($scope.users, {publicId: userId});
                $scope.usersMode = 'edit';
                $scope.originalSelectedUser = user;
                user.grantedMap = $scope.generateGrantedWorkingEnvironmentsMap(user);

                $scope.selectedUser = _.cloneDeep($scope.originalSelectedUser);
                $scope.selectedRolePublicId = $scope.selectedUser.roleId;

                if ($scope.selectedUser.tenant) {
                    $scope.selectedUserTenantId = $scope.selectedUser.tenant.publicId;
                }

                $scope.selectedUser.authenticationMode = $scope.selectedUser.isLdap ? 'Active Directory' : 'Cloud Manager';

                usersResource.getUser(nocHttp, userId)
                    .then((user) => {
                        $scope.selectedUser.accountId = user.accountId;
                    });
            } else {
                $scope.usersMode = 'new';
                const userRole = _.find($scope.roles, {publicId: _.keys($scope.allowedRolesToCurrentRole[currentUserRolePublicId])[0]});
                $scope.selectedRolePublicId = userRole ? userRole.publicId : '';
                $scope.selectedUser = {grantedMap: {}, role: userRole, authenticationMode: 'Cloud Manager', notificationsOptIn: true};

                //If the current user creating this new user, is not an admin, set the right tenant for the newly created user
                //NOC-1776 Creating WE admin does not enable assigning WE
                if (currentUserRolePublicId !== 'Role-1') {
                    $scope.selectedUser.tenant = $scope.selectedTenant;
                    $scope.selectedUserTenantId = _.get($scope.selectedTenant, 'publicId');
                }
                $scope.originalSelectedUser = _.cloneDeep($scope.selectedUser);

                activeDirectory();
            }
        };

        $scope.isAllowed = (action) => {
            return nocCurrentUserService.isAllowed(action);
        };

        function activeDirectory() {
            nocHttp.get({url: '/active-directory'}, 'checking if active directory is configured', nocHttp.LOADING)
                .then((activeDirectories) => {
                    $scope.authenticationTypes = ['Cloud Manager'];
                    if (!_.isEmpty(activeDirectories)) $scope.authenticationTypes.push('Active Directory');
                });
        }

        $scope.openWEsList = () => {
            nocDialog.dialog('Working environments created by ' + $scope.selectedUser.name, 'assets/templates/dialogs/working-environments-list.html', {wes: _.cloneDeep($scope.selectedUser.createdWorkingEnvironments)});
        };

        $scope.updateOriginalUser = () => {
            //To avoid warning on navigate away
            $scope.originalSelectedUser = $scope.selectedUser;
        };

        $scope.isValid = () => $scope.usersForm.$valid;

        $scope.filterCreateRolesForCurrentUser = (role) => {
            if ($scope.usersMode === 'edit') return true;
            return nocCurrentUserService.isCurrentUserExist() && !_.isUndefined($scope.allowedRolesToCurrentRole[nocCurrentUserService.getCurrentUserRolePublicId()][role.publicId]);
        };

        $scope.isDirty = () => {
            return !_.isEqual($scope.originalSelectedUser, $scope.selectedUser) && $scope.usersForm.$dirty;
        };

        $scope.emptyGrantedMapFromGivenUser = (user) => {
            user.grantedMap = {};
        };

        $scope.deleteTenantFromGivenUser = (user) => {
            //Delete tenant only in case the current user is an oncloud admin, otherwise the tenant is always fixed hence
            // no need to delete it.
            if ($scope.selectedUser.role.publicId === 'Role-1') {
                delete $scope.selectedUser.tenant;
                delete user.tenant;
            }
        };

        $scope.backToTable = () => {
            $scope.checkAndSetState('^.index');
        };

        $scope.resetPasswordOfSelectedUser = () => {
            nocDialog.dialog('Reset Password', 'assets/templates/tabs/users/reset-password.html', {selectedUser: $scope.selectedUser});
        };

        $scope.changePasswordOfCurrentUser = () => {
            nocDialog.dialog('Change Password', 'assets/templates/tabs/users/current-user-change-password.html', {selectedUser: $scope.selectedUser});
        };

        $scope.volumeViewChangeUserPassword = () => {
            nocDialog.dialog('Change Password', 'assets/templates/tabs/volume_view/dialogs/volume-view-change-password.html',
                {
                    selectedUser: $scope.selectedUser,
                    width: 650
                });
        };

        $scope.buildGrantPermission = (clonedUser, originalSelectedUser) => {
            const grantPermission = {};

            const selectedWorkingEnvironments = _.keys(_.pickBy(clonedUser.grantedMap));
            const originalWorkingEnvironments = _.map(originalSelectedUser.workingEnvironments, 'publicId');
            const intersect = _.intersection(originalWorkingEnvironments, selectedWorkingEnvironments);

            grantPermission.addWorkingEnvironments = _.difference(selectedWorkingEnvironments, intersect);
            grantPermission.removeWorkingEnvironments = _.difference(originalWorkingEnvironments, intersect);

            return grantPermission;
        };

        $scope.prepareClonedUser = (clonedUser) => {
            clonedUser.roleId = clonedUser.role.publicId;
            delete clonedUser.role;

            if (clonedUser.tenant) {
                clonedUser.tenantId = clonedUser.tenant.publicId;
                delete clonedUser.tenant;
            }

            delete clonedUser.workingEnvironments;
            delete clonedUser.retypePassword;
            delete clonedUser.publicId;
            delete clonedUser.grantedMap;
            delete clonedUser.name;

            if (clonedUser.authenticationMode === 'Active Directory') {
                clonedUser.password = 'LDAP123456'; //Mandatory field
                clonedUser.isLdap = true;
            } else {
                delete clonedUser.ldapUserName; //Optional field
                clonedUser.isLdap = false;
            }

            delete clonedUser.authenticationMode;
        };

        $scope.saveSelectedUser = () => {
            if ($scope.isValid()) {
                if ($scope.isDirty()) {
                    const clonedUser = _.clone($scope.selectedUser);

                    if (_.get(clonedUser, 'role.publicId') === 'Role-1') {
                        delete clonedUser.tenant;
                    }

                    const grantPermission = $scope.buildGrantPermission(clonedUser, $scope.originalSelectedUser);

                    let url = '/users';
                    let method = 'post';
                    if ($scope.usersMode === 'edit') {
                        url = url + '/' + clonedUser.publicId;
                        method = 'put';
                    }

                    $scope.prepareClonedUser(clonedUser);

                    const grantFunction = (publicId) => {
                        if (!_.isEmpty(grantPermission.addWorkingEnvironments) || !_.isEmpty(grantPermission.removeWorkingEnvironments)) {
                            nocHttp.post({
                                url: '/users/' + publicId + '/working-environments-grant-revoke',
                                data: grantPermission
                            }, 'changing working environment permissions', nocHttp.BLOCK)
                                .then(() => {
                                    $scope.updateOriginalUser();
                                    $scope.backToTable();
                                });
                        } else {
                            //So is dirty is false, to avoid confirm on navigate after save
                            $scope.updateOriginalUser();
                            $scope.backToTable();
                        }
                    };

                    /*NOC-3502 WE admin role can't change his own e-mail
                     allow updating the current user, or any other user which is not we admin*/
                    if ($scope.selectedUser.currentUser || nocCurrentUserService.getCurrentUserRolePublicId() !== 'Role-3') {
                        nocHttp[method]({url, data: clonedUser},
                            $scope.usersMode === 'edit' ? 'updating user' : 'creating user',
                            nocHttp.BLOCK)
                            .then((user) => {
                                grantFunction(user.publicId);
                            });
                    } else {
                        grantFunction($scope.selectedUser.publicId);
                    }

                    //If updating the current user, update the name of scope.currentUser, to update the display.
                    if ($scope.selectedUser.currentUser) {
                        $scope.currentUser.firstName = $scope.selectedUser.firstName;
                        $scope.currentUser.lastName = $scope.selectedUser.lastName;
                    }
                } else {
                    $scope.backToTable();
                }
            } else {
                nocUI.handleFormValidity($scope.usersForm, $scope);
            }
        };

        $scope.volViewSaveUserDetails = () => {
            if ($scope.isValid()) {
                volViewGeneralResource.volViewUpdateUserDetails(nocHttp, $scope.selectedUser.firstName, $scope.selectedUser.lastName, $scope.selectedUser.email, $scope.selectedUser.ldapUserName);

                //Update the name of scope.currentUser to update the display
                $scope.currentUser.firstName = $scope.selectedUser.firstName;
                $scope.currentUser.lastName = $scope.selectedUser.lastName;

                $state.go('volume-view.volumes');
            } else {
                nocUI.handleFormValidity($scope.usersForm, $scope);
            }
        };

        if (!$scope.users) {
            $scope.initializeUsers()
                .then(() => {
                    $scope.initializeEditor($stateParams.userId);
                })
                .catch($log.error);
        } else {
            $scope.initializeEditor($stateParams.userId);
        }

        $scope.$watch('selectedUser.role', (role) => {
            if (role) {
                $scope.selectedUser.tenant = _.find($scope.tenants, {publicId: $scope.selectedUserTenantId});
            }
        });

        $scope.$watch('selectedUser.tenant', (tenant) => {
            $scope.selectedTenantWE = tenant ? _.filter($scope.workingEnvironments, {tenantId: tenant.publicId}) : [];
        });

        nocNavigateAway.promptOnLeaveBeforeDiscardingChanges($scope);
    });
})();
