(() => {
    'use strict';

    angular.module('noc').controller('VolumeViewActionRequiredController', ($scope, volViewInstancesResource, volViewGeneralResource,
                                                                            nocHttp, $filter, nocUtil, pidgeon) => {
        const normalizeCapacityFilter = $filter('normalizedCapacity');
        const actionIntoWords = $filter('actionIntoWords');

        $scope.sumCapacityForDisks = nocUtil.sumCapacityForDisks;

        function summarizeActionRequired(originalAction) {
            var workingEnvironment = {name: !_.isEmpty(originalAction.actionRequired.parameters.instances) ? _.split(originalAction.actionRequired.parameters.instances[0].instanceName, '-')[0] : ''};
            switch (originalAction.actionRequired.actionType) {
                case 'EmptyWorkingEnvironment': {
                    var resourcesToDelete = originalAction.actionRequired.parameters.resourcesToDelete;
                    var pluralizeText = function (instanceNameAndType, savedCapacity) {
                        var instancesNames = _.map(instanceNameAndType, "instanceName").join(', ');
                        var savedCapacityString = normalizeCapacityFilter($scope.sumCapacityForDisks(savedCapacity), 2, true);
                        var suffix = " of AWS capacity that is not being utilized. The instance can be deleted.";
                        return (instanceNameAndType.length > 1) ?
                            ("Instances " + instancesNames + " have " + savedCapacityString + suffix) :
                            ("Instance " + instancesNames + " has " + savedCapacityString + suffix);
                    };
                    const parseDiskInfo = function parseDiskInfo(item) {
                        return String(item.numberOfDisks) + ' x ' + normalizeCapacityFilter(item.diskSize) + ' ' + item.diskType;
                    };
                    var pluralizeText2 = function (instanceNameAndType, numberOfDisksAndDiskSize) {
                        var str = (instanceNameAndType.length > 1) ? "Instances types: " : "Instance type: ";
                        return str + _.map(instanceNameAndType, "instanceType").join(', ') + " | Attached EBS volumes: " + _.map(numberOfDisksAndDiskSize, parseDiskInfo).join(', ');
                    };
                    return {
                        title: "Saving Opportunity Identified",
                        text: pluralizeText(resourcesToDelete.instanceNameAndType, resourcesToDelete.numberOfDisksAndDiskSize),
                        text2: pluralizeText2(resourcesToDelete.instanceNameAndType, resourcesToDelete.numberOfDisksAndDiskSize),
                        buttonCaption: "Delete",
                        volumesList: null,
                        severity: originalAction.actionRequired.severity,
                        fixIt: function (nocHttp) {
                            return volViewInstancesResource.deleteWorkingEnvironment(nocHttp, originalAction.actionRequired.parameters.workingEnvironmentId);
                        }
                    };
                }
                case 'MoreDisksNeededButVsaLimitExceeded': {
                    if (originalAction.actionRequired.parameters.licenseParameters) {
                        var instanceStr = (originalAction.actionRequired.parameters.instances.length > 1) ? 'instances' : 'instance';
                        var instancesNames = _.map(originalAction.actionRequired.parameters.instances, function (instanceObject) {
                            return instanceObject.instanceName + ' (' + instanceObject.instanceId + ')';
                        }).join(', ');
                        const isHA = originalAction.actionRequired.parameters.instances.length === 2;
                        var text2Prefix = "This action takes ";
                        var text2Suffix = isHA ? "approximately 90 minutes, during which I/O to the connected volumes will not be interrupted." : "the " + instanceStr + " offline for approximately 25 minutes, during which I/O to the connected volumes will be interrupted.";
                        return {
                            title: "Capacity Threshold Exceeded",
                            text: 'Additional capacity is required for ' + instanceStr + ' ' + instancesNames + '. To add additional capacity, Cloud Manager must first change the ' + instanceStr + ' type to ' + originalAction.actionRequired.parameters.licenseParameters.newInstanceType + '.',
                            text2: text2Prefix + text2Suffix,
                            buttonCaption: "Change Instance Type",
                            volumesList: originalAction.affectedVolumes.join(', '),
                            severity: originalAction.actionRequired.severity,
                            fixIt: function () {
                                return volViewInstancesResource.changeInstanceType(nocHttp, originalAction.actionRequired.parameters,
                                    {instanceType: originalAction.actionRequired.parameters.licenseParameters.newInstanceType});
                            }
                        };
                    } else {
                        return {
                            title: "Capacity Threshold Exceeded",
                            text: "Underlying AWS resource have reached its max capacity.",
                            buttonCaption: null,
                            noButtonCaption: "Unable to add capacity",
                            volumesList: originalAction.affectedVolumes.join(', '),
                            severity: originalAction.actionRequired.severity,
                            fixIt: _.noop
                        };
                    }
                }
                case 'licenseGracePeriod': {
                    return {
                        title: 'Subscription Expired',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: null,
                        noButtonCaption: null,
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: _.noop
                    };
                }
                case 'licenseExpirationWarning': {
                    return {
                        title: 'Renew Your Subscription',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: null,
                        noButtonCaption: null,
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: _.noop
                    };
                }
                case 'licenseWasSuspended': {
                    return {
                        title: 'Subscription Expired',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: null,
                        noButtonCaption: null,
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: _.noop
                    };
                }
                case 'MoreDisksNeededButDiskLimitExceeded': {
                    return {
                        title: 'Disk Limit Exceeded',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: null,
                        noButtonCaption: "Unable to add capacity",
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: _.noop
                    };
                }
                case 'RestrictedAggregatesCanBeFixed': {
                    return {
                        title: 'Restricted Aggregates',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: null,
                        noButtonCaption: null,
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: _.noop
                    };
                }
                case 'RestrictedAggregatesCannotBeFixed': {
                    return {
                        title: 'Restricted Aggregates',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: null,
                        noButtonCaption: null,
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: _.noop
                    };
                }
                case 'LicenseUsedCapacityWarning': {
                    return {
                        title: 'Approaching license capacity limit',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: originalAction.actionRequired.parameters.licenseParameters ? "Change license" : null,
                        noButtonCaption: !originalAction.actionRequired.parameters.licenseParameters ? "Cannot change license" : null,
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: function () {
                            return volViewInstancesResource.changeInstanceType(nocHttp, originalAction.actionRequired.parameters,
                                {instanceType: originalAction.actionRequired.parameters.licenseParameters.newInstanceType});
                        }
                    };
                }
                case 'NonCompliantConfig': {
                    return {
                        title: 'Limited functionality: instance type change required',
                        text: actionIntoWords(originalAction.actionRequired, workingEnvironment),
                        buttonCaption: originalAction.actionRequired.parameters.licenseParameters ? 'Change license' : null,
                        noButtonCaption: !originalAction.actionRequired.parameters.licenseParameters ? 'Cannot change license' : null,
                        volumesList: originalAction.affectedVolumes.join(', '),
                        severity: originalAction.actionRequired.severity,
                        fixIt: function () {
                            return volViewInstancesResource.changeInstanceType(nocHttp, originalAction.actionRequired.parameters,
                                {instanceType: originalAction.actionRequired.parameters.licenseParameters.newInstanceType});
                        }
                    };
                }
                default: {
                    //Should never happen, list of actions required should be filtered in the server to contain these
                    //types only.
                    pidgeon.error('Unknown action required ' + originalAction.actionRequired.actionType);
                }
            }
        }

        $scope.getActionsRequired = function getActionsRequired() {
            const unloading = nocHttp.LOADING(); //eslint-disable-line new-cap
            volViewGeneralResource.getActionsRequired(nocHttp)
                .then((response) => {
                    $scope.actionsRequired = _(response).map(summarizeActionRequired)
                        .sortBy('severity') //'error' then 'warning'
                        .value();
                })
                .finally(unloading);
        };

        $scope.initialize = function () {
            $scope.getActionsRequired();
        };

        $scope.solve = function(action){
            action.fixIt(nocHttp);
            action.noButtonCaption = "Process Started";
            action.buttonCaption = null;
            $scope.showFootnote = true;
        };

        $scope.initialize();
    });
})();
