(function () {
    "use strict";

    angular.module("noc").controller("VolumeViewAddVolumeController", function ($scope, nocHttp, nocUI, providerDiskTypes, volViewVolumesResource, volViewGeneralResource, $q, nocDialog, pidgeon, systemResource, $rootScope) {
        $scope.initComplete = false;
        $scope.editMode = false;
        $scope.forceEditMode = false;
        _.set($scope.wizardController, 'volumeRequest.size.unit', 'GB');
        _.set($scope, 'createVolumeSelectObjects.diskType', _.find(providerDiskTypes['AWS'], {default: true}));
        $scope.passwordRequest = {};

        $scope.selectDefaultRegion = function selectDefaultRegion(){
            //select the OCCM region or the first
            var occmRegion = _.find($scope.regions, {code: $scope.instancePlacement.region});
            if(!occmRegion) {
                $scope.forceEditMode = true;
                $scope.editMode = true;
            }

            $scope.handleRegionChange(occmRegion || $scope.regions[0]);
        };

        function init() {
            $q.all([
                systemResource(nocHttp).getInstancePlacement(nocHttp.LOADING, true),
                volViewVolumesResource.getRegions(nocHttp),
                volViewGeneralResource.getOntapCloudPassword(nocHttp)
            ])
                .then(_.spread(function (instancePlacement, regions, ontapCloudPasswordExists) {
                    $scope.instancePlacement = instancePlacement;
                    $scope.regions = regions;
                    $scope.wizardController.ontapCloudPasswordExists = ontapCloudPasswordExists;

                    $scope.selectDefaultRegion();
                }));
        }

        function loadVPCs(region) {
            return volViewVolumesResource.getVPCs(nocHttp, region)
                .then(function (vpcs) {
                    $scope.vpcs = vpcs;
                });
        }

        $scope.handleRegionChange = (newRegion) => {
            $scope.createVolumeSelectObjects.region = newRegion;
            const region = $scope.createVolumeSelectObjects.region.code;
            if (region) {
                loadVPCs(region)
                    .then(() => {
                        //Select OCCM VPC if exist, else if only one exist select it, else select nothing.
                        $scope.createVolumeSelectObjects.vpc = _.find($scope.vpcs, {vpcId: $scope.instancePlacement.vpcId}) || ($scope.vpcs.length === 1 ? $scope.vpcs[0] : undefined);
                        $scope.handleVpcChange();
                    })
                    .finally(() => {
                        $scope.initComplete = true;
                    });
            }
        };

        $scope.handleVpcChange = function () {
            $scope.wizardController.availabilityZones = $scope.createVolumeSelectObjects.vpc ? _.union(_.map($scope.createVolumeSelectObjects.vpc.subnets, 'availabilityZone')) : undefined;
            $scope.wizardController.numberOfAvailabilityZones = $scope.wizardController.availabilityZones ? $scope.wizardController.availabilityZones.length : 0;

            if ($scope.createVolumeSelectObjects.vpc) {
                $scope.createVolumeSelectObjects.subnet = _.find($scope.createVolumeSelectObjects.vpc.subnets, {subnetId: $scope.instancePlacement.subnetId}) || _.get($scope,'createVolumeSelectObjects.vpc.subnets[0]');
            } else {
                $scope.createVolumeSelectObjects.subnet = undefined;
            }
            if ($scope.currentStepPath !== $scope.wizardController.path.details) {
                $scope.setWizardStep($scope.wizardController.path.details);
            }
        };


        $scope.backToVolumesPage = function () {
            $rootScope.showCreateFirstVolumeScreen = false;
            $scope.setPathToPreviousOrRoot();
        };

        $scope.finishStep = function () {
            $scope.wizardController.volumeRequest.region = $scope.createVolumeSelectObjects.region.code;
            $scope.wizardController.volumeRequest.vpcId = $scope.createVolumeSelectObjects.vpc.vpcId;
            $scope.wizardController.volumeRequest.subnetId = $scope.createVolumeSelectObjects.subnet.subnetId;
            $scope.wizardController.volumeRequest.diskType = $scope.createVolumeSelectObjects.diskType.name;
            if ($scope.wizardController.isHa) {
                $scope.next('.nodes-and-mediator');
            } else if (!$scope.isValid()) {
                nocUI.handleFormValidity($scope.volumeViewAddVolumeForm, $scope);
            } else {
                $scope.getPlacementInfo()
                    .then($scope.createVolume)
                    .then($scope.backToVolumesPage);
            }
        };

        $scope.makeSureOntapCloudPasswordExists = function makeSureOntapCloudPasswordExists() {
            if (!$scope.wizardController.ontapCloudPasswordExists) {
                return nocDialog.confirm("Cloud Volumes ONTAP Password", "assets/templates/tabs/volume_view/dialogs/ontap-cloud-password-dialog.html", {
                    //width must be at least 662px - NOC-11492
                    width: 662,
                    dialogClass: 'ontap-cloud-password-dialog',
                    fromTop: "15%",
                    passwordRequest: {},
                    customTags: [],
                    dialogOptions: {
                        confirmBtnText: "Save",
                        cancelBtnText: "Cancel"
                    }
                })
                    .then(function (dialogScope) {
                        $scope.wizardController.ontapCloudPasswordExists = true;
                        return $q.all([
                            volViewGeneralResource.setOntapCloudPassword(nocHttp, dialogScope.passwordRequest.ontapCloudPassword),
                            volViewGeneralResource.setAWSTags(nocHttp, dialogScope.customTags)
                        ]);
                    });
            } else {
                return $q.when();
            }
        };

        $scope.getPlacementInfo = function getPlacementInfo() {
            return volViewVolumesResource.possiblePlacement(nocHttp, $scope.wizardController.volumeRequest)
                .then(function (placementInfo) {
                    if (placementInfo && placementInfo.upgradeInfo) {
                        return nocDialog.confirm("Saving Opportunity Identified", "assets/templates/tabs/volume_view/dialogs/possible-upgrade.html", {
                            affectedVolumes: placementInfo.affectedVolumes,
                            isHa: $scope.wizardController.isHa,
                            width: 650,
                            fromTop: '10%'
                        })
                            .then(function (dialogScope) {
                                return dialogScope.doUpgrade ? placementInfo : null;
                            });
                    } else {
                        return placementInfo;
                    }
                });

        };

        /*
         * placementInfo is null when creating new instance
         */
        $scope.createVolume = function createVolume(placementInfo) {
            pidgeon.clear("add-volume");
            if (placementInfo) delete placementInfo.affectedVolumes;
            $scope.wizardController.volumeRequest.upgradeCreateRequest = (placementInfo && placementInfo.upgradeInfo) ? placementInfo : null;

            var ontapCloudPasswordPromise = placementInfo ? $q.when : $scope.makeSureOntapCloudPasswordExists;
            return ontapCloudPasswordPromise()
                .then(function () {
                    return volViewVolumesResource.createVolume(nocHttp, $scope.wizardController.volumeRequest)
                        .catch(function (error) {
                            if (error.failureInfo) {
                                nocDialog.dialog("Subscribe in Marketplace", "assets/templates/dialogs/include-linky-dialog.html", {bindText: error.failureInfo});
                            } else {
                                pidgeon.error(pidgeon.serverErrorPrefix + "creating volume", error, "add-volume");
                            }
                            return $q.reject();
                        });
                });
        };

        $scope.switchEditMode = function () {
            if ($scope.editMode) {
                $scope.selectDefaultRegion();
            }
            $scope.editMode = !$scope.editMode;
            if ($scope.currentStepPath !== $scope.wizardController.path.details) {
                $scope.setWizardStep($scope.wizardController.path.details);
            }
        };

        init();
    });
}());
