(function () {
    "use strict";

    angular.module("noc").controller("VolumeViewResourcesController", function ($scope, volumeViewColumnsService, uiGridFilterService, $timeout, uiGridConstants, volViewInstancesResource, nocDialog, nocHttp, nocUI, $rootScope, $q) {
        $scope.search = {term: ''};
        $scope.unloadingCount = 0;
        var unloading = nocUI.loading();
        var resourcesUnloading = function(){
            $scope.loaded = true;
            unloading();
        };
        $scope.instances = [];

        $scope.removeDeletedInstancesFromInstancesList = function removeDeletedInstancesFromInstancesList(polledWorkingEnvironments) {
            var newIds = _.keyBy(polledWorkingEnvironments, 'workingEnvironmentId');

            //remove deleted instances:
            $scope.instances = _.filter($scope.instances, function (instance) {
                return newIds[instance.workingEnvironmentId];
            });
            $scope.gridOptions.data = $scope.instances;
        };

        $scope.updateInstance = function updateInstance(instance) {
            return volViewInstancesResource.getInstancesById(nocHttp, instance.workingEnvironmentId)
                .then(function (instanceResponse) {
                    $scope.instances = _($scope.instances)
                        //Remove all previous instances of this working environment id
                        .reject({workingEnvironmentId: instance.workingEnvironmentId})
                        //Add updated instances for this working environment id from API response (except if are not found or deleting or degraded)
                        .concat(_.reject(instanceResponse, function (instance) {
                            return instance.status === 'not found' || instance.status === 'failed to create' || instance.status === 'deleting' || instance.status === 'degraded';
                        })).value();
                    $scope.gridOptions.data = $scope.instances;
                    //Keep the loader until last API returns
                    if (--$scope.unloadingCount === 0) resourcesUnloading();
                });
        };

        var resourcePollingSuccess = function (polledWorkingEnvironments) {
            $scope.removeDeletedInstancesFromInstancesList(polledWorkingEnvironments);
            $scope.unloadingCount = polledWorkingEnvironments.length;
            _.forEach(polledWorkingEnvironments, $scope.updateInstance);
            if (!polledWorkingEnvironments.length) resourcesUnloading();
        };

        var pollStopper = volViewInstancesResource.startResourcesPolling(resourcePollingSuccess);

        $scope.gridOptions = {
            data: [],
            useExternalFiltering: true,
            columnVirtualizationThreshold: 50,
            virtualizationThreshold: 30,
            saveSort: true,
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            columnDefs: volumeViewColumnsService.getResourcesColumnsDef(),
            onRegisterApi : function(gridAPI) {
                $scope.gridAPI = gridAPI;
                uiGridFilterService.installFilter(gridAPI, $scope);
                $timeout(() => {
                    volumeViewColumnsService.restoreGridState(gridAPI, $scope, 'VOLUME_VIEW_RESOURCES_GRID');
                    $timeout(() => {
                        $(window).trigger('resize');
                    });
                });
            }
        };

        $scope.changeInstanceType = function (instance) {
            volViewInstancesResource.getTargetInstanceTypes(nocHttp, instance)
                .then(function (targetInstanceTypes) {
                    nocDialog.confirm("Change instance type for instance " + instance._displayName,
                        "assets/templates/tabs/volume_view/dialogs/change-instance-type.html", {
                            currentInstanceType: instance.instanceType,
                            ontapCloudAwsPageUrl: instance._isHa ? $rootScope.externalConfig.ha_cot_aws_product_page : $rootScope.externalConfig.cot_aws_product_page,
                            instancesTypes: targetInstanceTypes,
                            selectedInstanceType: targetInstanceTypes[0],
                            isHa: instance._isHa,
                            dialogOptions: {
                                confirmBtnText: "Change instance type",
                                cancelBtnText: "Cancel"
                            }
                        })
                        .then(function (dialogScope) {
                            volViewInstancesResource.changeInstanceType(nocHttp, instance, dialogScope.selectedInstanceType)
                                .then(function () {
                                    //Update the instance to get 'Updating' status from server.
                                    $scope.updateInstance(instance);
                                });
                        });
                });
        };

        function turnInstanceOnOrOff(instance, onOrOff) {
            var confirmation = onOrOff === 'off' || instance._isHa ? nocDialog.confirm('Turn ' + onOrOff + (instance._isHa ? ' HA' : '') + ' instance',
                "assets/templates/tabs/volume_view/dialogs/turn-instance-on-or-off-dialog.html", {
                    onOrOff: onOrOff,
                    isHa: instance._isHa,
                    connectedInstances: instance._instancesInCluster,
                    dialogOptions: {
                        confirmBtnText: 'Turn ' + _.capitalize(onOrOff),
                        cancelBtnText: "Cancel"
                    }
                }) : $q.when();

            confirmation
                .then(function () {
                    var action = onOrOff === 'on' ? 'startWorkingEnvironment' : 'stopWorkingEnvironment';
                    volViewInstancesResource[action](nocHttp, instance.workingEnvironmentId)
                        .then(function () {
                            //Update the instance to get 'Starting'\'Stopping' status from server.
                            $scope.updateInstance(instance);
                        });
                });
        }

        $scope.turnOff = function turnOff(instance) {
            turnInstanceOnOrOff(instance, 'off');
        };

        $scope.turnOn = function turnOn(instance) {
            turnInstanceOnOrOff(instance, 'on');
        };

        $scope.$on('$destroy', function(){
            pollStopper();
            volumeViewColumnsService.saveGridState($scope.gridAPI, "VOLUME_VIEW_RESOURCES_GRID");
        });
    });
}());
