(function () {
    "use strict";

    angular.module("noc").controller("VolumeViewSupportRegistrationController", function ($scope, nocUI, volViewGeneralResource, nocHttp, $timeout) {

        $scope.initialize = function(){
            volViewGeneralResource.getNssUserName(nocHttp)
                .then(function (nssUserName) {
                    if (nssUserName) {
                        $scope.alreadyRegisteredNssUserName = nssUserName;
                        $scope.nssUserName = nssUserName;
                        $scope.nssEditingDisabled = true;
                    } else {
                        $scope.alreadyRegisteredNssUserName = false;
                        $scope.nssEditingDisabled = false;
                    }
                })
                .finally(function () {
                    $scope.nssLoaded = true;
                });
        };

        $scope.isValid = function () {
            return $scope.VolumeViewSupportRegistrationForm.$valid;
        };

        var setNssInputsValidity = function (isValid) {
            $scope.VolumeViewSupportRegistrationForm.user.$setValidity('nss-validity', isValid);
            $scope.VolumeViewSupportRegistrationForm.pass.$setValidity('nss-validity', isValid);
        };

        $scope.enableEdit = function () {
            $scope.nssEditingDisabled = false;
            nocUI.clearFormValidity();
        };

        $scope.register = function () {
            nocUI.handleFormValidity($scope.VolumeViewSupportRegistrationForm, $scope);
            volViewGeneralResource.registerVolumesToNss(nocHttp, $scope.nssUserName, $scope.nssPassword)
                .then(function () {
                    $scope.nssPassword = '';
                    $scope.alreadyRegisteredNssUserName = $scope.nssUserName;
                    $scope.nssEditingDisabled = true;
                    $scope.errorMessage = null;
                    setNssInputsValidity(true);
                    $scope.successMessage = "NSS Credentials has been saved successfully.";
                    $timeout(function(){
                        $scope.successMessage = null;
                    }, 5000);
                })
                .catch(function (error) {
                    setNssInputsValidity(false);
                    $scope.errorMessage = error.message;
                });
        };

        $scope.cancel = function(){
            $scope.nssEditingDisabled = true;
            setNssInputsValidity(true);
            $scope.errorMessage = "";
            $scope.nssUserName = $scope.alreadyRegisteredNssUserName;
            $scope.nssPassword = "";
        };

        $scope.initialize();
    });
}());
