(function () {
    "use strict";

    angular.module("noc").controller("VolumeViewController", function ($scope, volumeViewColumnsService, uiGridFilterService, nocUtil, volViewVolumesResource, volViewGeneralResource, nocDialog, nocHttp, nocNormalizerService, $timeout, uiGridConstants, $rootScope, nocUI, nocFriendlyStore) {
        $scope.search = {term: ''};
        var volumesUnloading = nocUI.loading();

        $scope.startCreateNewVolumeWizard = function () {
            nocFriendlyStore.haVolume = !!_.find($scope.volumes,'isHa');
            $scope.checkAndSetState("^.create-volume-wizard");
        };

        var accumulateVolSize = function (volumeSizes) {
            var accumulatedCapacity = _(volumeSizes)
                .map(function (capacity) {
                    return nocUtil.convertTo(capacity, "Byte");
                }).reduce(function (size, acc) {
                    return {size: size.size + acc.size, unit: "Byte"};
                }, {size: 0, unit: "Byte"});

            return nocUtil.convertTo(accumulatedCapacity, "GB");
        };

        var summarizeVolumes = function(volumes) {
            var result = {
                count: volumes.length,
                usedSize: accumulateVolSize(_.map(volumes, "usedSize")),
                size: accumulateVolSize(_.map(volumes, "size")),
                statuses: _(volumes).map("_displayStatus").countBy().map(function(value, key) {return {name: key, count: value};}).value()
            };

            result.percentage = nocUtil.convertTo(result.usedSize, "Byte").size / (nocUtil.convertTo(result.size, "Byte").size || 1);

            return result;
        };

        $scope.showMissingVolumesErrors = function (errors) {
            //Using customized title to override the default margin of the built in one.
            nocDialog.confirm('', "assets/templates/tabs/volume_view/dialogs/missing-volumes-dialog.html", {"errors": errors, width: 650});
        };

        function calculateActionRequiredHighestLevel(actionsRequired) {
            if (actionsRequired.length === 0) return null;
            return _.find(actionsRequired, ["actionRequired.severity", "error"]) ? "error" : "warning";
        }

        $scope.getActionsRequiredSummary = function () {
            return volViewGeneralResource.getActionsRequired(nocHttp)
                .then(function (response) {
                    $scope.actionsRequiredLevel = calculateActionRequiredHighestLevel(response);
                    $scope.actionsRequiredCount = response.length;
                });
        };

        var volumePollingSuccess = function(response) {
            $scope.volumes = response.volumes;
            $scope.gridOptions.data = $scope.volumes;
            $scope.errors = response.errors;
            $rootScope.firstVolume = $scope.errors.length === 0 && $scope.volumes.length === 0;
            $scope.summary = summarizeVolumes(response.volumes);
            $scope.getActionsRequiredSummary();

            if($scope.volumes.length === 0 && $scope.errors.length === 0 && $rootScope.showCreateFirstVolumeScreen){
                $scope.startCreateNewVolumeWizard();
            } else {
                $rootScope.showCreateFirstVolumeScreen = false;
            }
            volumesUnloading();
        };

        var pollStopper = volViewVolumesResource.startVolumesPolling(volumePollingSuccess);

        $scope.delete = function (volume) {
            nocDialog.confirm("Delete Volume " + volume.name, "assets/templates/tabs/volume_view/dialogs/delete-volume-dialog.html",
                {
                    volume: volume,
                    width: 666,
                    dialogOptions: {
                        confirmBtnText: "Delete",
                        cancelBtnText: "Cancel"
                    }
                })
                .then(function () {
                    volViewVolumesResource.deleteVolume(nocHttp, volume)
                        .then(function () {
                            //force the poller to poll once to get 'Deleting' status form server.
                            $rootScope.$broadcast('requestStatusChanged');
                        });
                });

        };

        $scope.clone = function(volume) {
            nocDialog.confirm("Clone Volume " + volume.name, "assets/templates/tabs/volume_view/dialogs/clone-volume-dialog.html",
                {
                    volume: volume,
                    targetName: volume.name + "_clone",
                    width: 650,
                    dialogOptions: {
                        confirmBtnText: "Clone",
                        cancelBtnText: "Cancel"
                    }
                }).then(function (dialogScope) {
                    volViewVolumesResource.cloneVolume(nocHttp, dialogScope.volume.id, dialogScope.targetName)
                        .then(function () {
                            //force the poller to poll once to get the new cloned volume from server
                            $rootScope.$broadcast('requestStatusChanged');
                        });
                });
        };

        $scope.mount = function(volume) {
            nocDialog.confirm("Mount Volume " + volume.name, "assets/templates/tabs/volume_view/dialogs/mount-volume-dialog.html",
                {
                    volume: volume,
                    width: 650,
                    dialogOptions: {
                        cancelBtnText: "Close"
                    }
                });
        };

        $scope.editVolumeAccessControl = function(volume){
            volume.exportPolicyInfo._ips = volume.exportPolicyInfo.ips.join(", ");
            var originalVol = _.cloneDeep(volume);
            nocDialog.confirm("Manage Access to Volume " + volume.name, "assets/templates/tabs/volume_view/dialogs/access-control-volume-dialog.html",
                {
                    volume: volume,
                    width: 650,
                    dialogOptions: {
                        confirmBtnText: "Save",
                        cancelBtnText: "Cancel"
                    }
                })
                .then(function(dialogScope){
                    if (!_.isEqual(originalVol, volume)) {
                        dialogScope.volume.exportPolicyInfo.ips = nocNormalizerService.normalizeCustomAccessControl(dialogScope.volume.exportPolicyInfo._ips);
                        dialogScope.volume.exportPolicyInfo.policyType = dialogScope.volume.exportPolicyInfo.ips.length > 0 ? "custom" : "none";
                        return volViewVolumesResource.updateVolume(nocHttp, dialogScope.volume);
                    }
                });
        };

        $scope.changeDiskType = function changeDiskType(volume) {
            var currentDiskTypeDisplay = volume._diskType;
            var supportedDiskTypes = volViewVolumesResource.getPossibleDiskTypes(volume.isHa);
            _.remove(supportedDiskTypes, {name: volume.diskType}); // don't show current disk type;
            var selectedDiskTypeName = _.get(_.head(supportedDiskTypes), 'name');

            nocDialog.confirm('Change Underlying Disk Type for Volume ' + volume.name,
                'assets/templates/tabs/volume_view/dialogs/change-disk-type-volume-dialog.html',
                {
                    currentDiskTypeDisplay: currentDiskTypeDisplay,
                    diskTypes : supportedDiskTypes,
                    selectedDiskTypeName: selectedDiskTypeName,
                    width: 650,
                    dialogOptions: {
                        confirmBtnText: "Change",
                        cancelBtnText: "Cancel"
                    }
                })
                .then(function(dialogScope){
                    $scope.quoteForDiskTypeChange(volume, dialogScope.selectedDiskTypeName)
                        .then(function(quote) {
                            return volViewVolumesResource.changeDiskType(nocHttp, volume, dialogScope.selectedDiskTypeName, quote);
                        })
                        .then(function() {
                            //force the poller to poll once to get the new volume type from server
                            $rootScope.$broadcast('requestStatusChanged');
                        });
                });
        };

        $scope.quoteForDiskTypeChange = function quoteForDiskTypeChange(volume, diskType) {
            return volViewVolumesResource.quoteForDiskTypeChange(nocHttp, volume, diskType)
                .then(function (quote) {
                    // if needs upgrade
                    if (quote && quote.upgradeInfo) {
                        return nocDialog.confirm("Upgrade Required", "assets/templates/tabs/volume_view/dialogs/possible-upgrade.html", {
                            mustUpgrade: true,
                            affectedVolumes: quote.affectedVolumes,
                            isHa: volume.isHa,
                            width: 650,
                            fromTop: '10%'
                        })
                            .then(function () {
                                return quote;
                            });
                    }
                    // else doesn't need upgrade
                    else {
                        return quote;
                    }
                });
        };

        $scope.gridOptions = {
            useExternalFiltering: true,
            columnVirtualizationThreshold: 50,
            virtualizationThreshold: 30,
            saveSort: true,
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            columnDefs: volumeViewColumnsService.getVolumesColumnsDef(),
            onRegisterApi : function(gridAPI) {
                $scope.gridAPI = gridAPI;
                uiGridFilterService.installFilter(gridAPI, $scope);
                $timeout(() => {
                    volumeViewColumnsService.restoreGridState(gridAPI, $scope, 'VOLUME_VIEW_VOLUMES_GRID');
                    $timeout(() => {
                        $(window).trigger('resize');
                    });
                });
            }
        };

        $scope.$on('$destroy', function(){
            pollStopper();
            volumeViewColumnsService.saveGridState($scope.gridAPI, "VOLUME_VIEW_VOLUMES_GRID");
        });
    });
}());
