(function () {
    'use strict';

    angular.module('noc').controller('VolumesLayoutController', function ($scope, nocHttp, configService, nocDialog, $timeout, $state, nocLocalStorage, $rootScope, occmUpdateService, nocCurrentUserService) {

        $scope.selectedUser = $scope.currentUser;
        $scope.initialize = function () {
            //1. Make sure capacity automation is enabled:
            configService.getConfig(nocHttp, ['autoVsaCapacityManagement'])
                .then(function (configResponse) {
                    $scope.currentUser = nocCurrentUserService.getCurrentUser();
                    if (!configResponse.autoVsaCapacityManagement) {
                        nocDialog.confirm('Capacity Management Mode',
                            'assets/templates/tabs/volume_view/dialogs/enable-capacity-automation.html', {
                                width: 650,
                                dialogOptions: {
                                    confirmBtnText: 'Enable Automatic Mode',
                                    cancelBtnText: 'Switch to Storage System View'
                                }
                            })
                            .then(function () {
                                configService.updateConfig(nocHttp, ['autoVsaCapacityManagement'], {autoVsaCapacityManagement: true}, configResponse);
                                //2. Check for OCCM update:
                                occmUpdateService.showUpdatesDialog('updatesAvailable', true);
                            })
                            .catch($scope.switchToSystemStorageView);
                    }
                });

            //3.
            $rootScope.showCreateFirstVolumeScreen = true;
            $scope.$state = $state;
        };

        $scope.volumesTabClicked = function volumesTabClicked() {
            $rootScope.showCreateFirstVolumeScreen = false;
        };

        $scope.switchToSystemStorageView = function () {
            nocLocalStorage.VOLUME_VIEW.save(false);
            //to bring back scroll bar
            $('body').removeClass('no-scroll-overflow-hidden');
            //to make sure the top menu will already know that we are not in managed mode
            $timeout(function () {
                if (!_.startsWith($state.current.name, 'main')) $state.go('main');
            });
            if (_.get($rootScope.supportServices, 'portalService.usePortalAuthentication')) portalWidget();
        };

        $scope.toggleMenu = function () {
            $('.volume-view-container').toggleClass('menu-open');
            $('body').toggleClass('no-scroll-overflow-hidden');
            if ($scope.menuOpen) {
                $timeout(function () {
                    $scope.menuOpen = false;
                }, 700);
            } else {
                $scope.menuOpen = !$scope.menuOpen;
            }

        };

        $scope.showVolumeViewUserSettings = function () {
            $state.go('volume-view.volume-view-user-settings', {userId: $scope.currentUser.publicId});
            $scope.toggleUserMenu();
        };

        var isUserMenuOpen = false;
        $scope.toggleUserMenu = function toggleUserMenu() {
            isUserMenuOpen = !isUserMenuOpen;
            let hovering = true;
            const leavingUserName = function leavingUserName() {
                hovering = false;
            };
            const openUserMenu = function openUserMenu() {
                if (hovering) $('.user-menu').toggleClass('user-menu-open');
            };
            $timeout(openUserMenu, 275);
            $('.vol-view-username').on('mouseleave', leavingUserName);
            const closeUserMenu = function closeUserMenu() {
                $('.user-menu').removeClass('user-menu-open');
            };
            if (isUserMenuOpen) {
                $('.vol-view-username').on('mouseleave', closeUserMenu);
            } else {
                $scope.$on('$destroy', () => {
                    $('.vol-view-username').off('mouseleave', closeUserMenu);
                });
            }
        };

        const portalWidget = () => {
            if (!$rootScope.isPortalWidgetAlreadyExist) {
                const portalWidgetElement = document.createElement('script');
                portalWidgetElement.src = $rootScope.about.environment === 'production' ? 'https://services.cloud.netapp.com/widget/loader.js' : 'https://staging.services.cloud.netapp.com/widget/loader.js';
                document.head.appendChild(portalWidgetElement);
                $rootScope.isPortalWidgetAlreadyExist = true;
            }
        };

        const removePortalWidget = () => {
            const portalWidgetElement = $('#portal-widget-frame-container');
            portalWidgetElement.remove();
            $rootScope.isPortalWidgetAlreadyExist = false;
        };

        $scope.showMenuItem = function (itemName) {
            $('.volume-view-container').addClass('menu-item-open');
            $scope.menuViewItem = itemName;
            removePortalWidget();
        };

        $scope.closeAll = function () {
            portalWidget();
            $('.volume-view-container').removeClass('menu-open').removeClass('menu-item-open');
            $('body').removeClass('no-scroll-overflow-hidden');

            //Using timeout to let the animation fade.
            $timeout(function () {
                $scope.menuViewItem = null;
                $scope.menuOpen = false;
            }, 1000);

        };

        $scope.backToVolumes = function () {
            $state.go('volume-view.volumes');
        };

        $scope.initialize();

    });
}());
