(() => {
    'use strict';
    angular.module('noc').controller('MountVolumeController', ($scope, $stateParams, nocUI) => {
        const unblock = nocUI.block();

        $scope.unhealthyNodeTitle = 'IP is unavailable because the node is not healthy.';
        $scope.title = 'Go to your Linux machine and enter this mount command';
        $scope.nfsMode = true;
        $scope.connectCommand = '';

        function initialize() {
            $scope.selectedVolume = _.find($scope.volumes, {name: $stateParams.selectedVolumeName || '', svmName: $stateParams.selectedVolumeSVM || ''});
            $scope.nfsMode = !!_.get($scope.selectedVolume, 'mountPoint');
            if (!$scope.nfsMode) {
                $scope.title = 'Go to your machine and enter this command';
                $scope.connectCommand = `${$scope.selectedVolume.cifsShareAccessPoint}`;
                $scope.connectPrimaryCommand = `${$scope.selectedVolume.primaryCifsShareAccessPoint}`;
                $scope.connectSecondaryCommand = `${$scope.selectedVolume.secondaryCifsShareAccessPoint}`;
            } else {
                $scope.connectCommand = `mount ${$scope.selectedVolume.mountPoint} <dest_dir>`;
                $scope.connectPrimaryCommand = `mount ${$scope.selectedVolume.primaryNoFailoverMountPoint} <dest_dir>`;
                $scope.connectSecondaryCommand = `mount ${$scope.selectedVolume.secondaryNoFailoverMountPoint} <dest_dir>`;
            }
        }

        $scope.$watch('volumes', (volumes) => {
            /* istanbul ignore else */
            if (volumes) {
                initialize();
                unblock();
            }
        });

        $scope.isNodeHealthy = (isPrimary) => {
            const command = isPrimary ? $scope.connectPrimaryCommand : $scope.connectSecondaryCommand;
            const node = _.find($scope.selectedWorkingEnvironment.ontapClusterProperties.nodes, (node) => {
                return _.find(node.lifs, (lif) => {
                    return _.includes(command, lif._ip);
                });
            });

            return _.get(node, 'healthy');
        };
    });
})();
