(() => {
    'use strict';
    angular.module('noc').controller('AggregateController', ($scope, pidgeon, nocDialog, aggregatesResource, nocUtil,
                                                             Messages, providerDiskTypes, $filter, nocHttp,
                                                             $state, workingEnvironmentResource, nocEncryptionResourcesService) => {
        const doFiltering = nocUtil.getNgFilterFunc($scope, 'aggregates', 'filteredAggregates', 'searchAggregates',
            ['name', '_displayCapacity', '_diskTypeDisplay', '_displayUsedCapacityTier', '_diskTypeLetters', '_displayUsedCapacity', '_numOfDisks', '_numOfVolumes']);

        $scope.diskTypes = providerDiskTypes[$scope.selectedWorkingEnvironment._cloudProviderName];

        //Moved to an outside method in order to unit test noc-9308
        $scope.buildCloudProviderCapacityData = function (aggregate) {
            return {
                key: 'Underlying ' + $scope.selectedWorkingEnvironment._cloudProviderName + ' Capacity:',
                text: $filter('normalizedCapacity')($scope.calculateRawCapacity(aggregate), 2, true)
            };
        };

        $scope.buildDataForInformationDialog = function (aggregate) {
            const data = [];
            data.push({key: ($scope.selectedWorkingEnvironment._isAWS ? 'EBS' : 'Disk') + ' Allocated Capacity:', text: aggregate._displayCapacity});
            data.push({
                key: ($scope.selectedWorkingEnvironment._isAWS ? 'EBS' : 'Disk') + ' Used Capacity:',
                text: $filter('normalizedCapacity')(aggregate.usedCapacity, 2, true)
            });
            if (aggregate.capacityTier) {
                data.push({
                    key: aggregate.capacityTier + ' Used Capacity:',
                    text: $filter('normalizedCapacity')(aggregate.capacityTierUsed, 2, true)
                });
            }
            const volumes = _.map(aggregate.volumes, (volume) => {
                return {value: volume.name + ' (' + $filter('normalizedCapacity')(volume.totalSize, 2, true) + ')'};
            });
            data.push({
                key: 'Volumes:',
                text: aggregate.volumes.length || 'None',
                linkDataList: volumes,
                listShow: false,
                trimmedText: true
            });
            const disks = _.map(aggregate.providerVolumes, (providerVolume) => {
                return {value: providerVolume.name, title: providerVolume.id};
            });
            data.push({
                key: $scope.selectedWorkingEnvironment._cloudProviderName + ' Disks:',
                text: aggregate.providerVolumes.length,
                linkDataList: disks,
                listShow: false,
                trimmedText: $scope.selectedWorkingEnvironment._cloudProviderName !== 'AWS'
            }); //Just in case of Azure add the id as title
            data.push({key: 'State:', text: aggregate.state});
            const underlyingTier = $scope.diskTypes[aggregate._providerVolumeType].display + (aggregate.capacityTier ? ' and S3' : '');
            data.push({
                key: 'Underlying ' + $scope.selectedWorkingEnvironment._cloudProviderName + ' Tier:',
                text: underlyingTier,
                trimmedText: true,
                title: underlyingTier
            });
            const diskSizeText = $filter('normalizedCapacity')(aggregate.providerVolumes[0].size);
            data.push({key: $scope.selectedWorkingEnvironment._cloudProviderName + ' Disk Size:', text: diskSizeText});
            data.push($scope.buildCloudProviderCapacityData(aggregate));
            const encryptionStrings = {
                'ontapEncrypted': 'ONTAP Managed Encryption',
                'awsEncrypted': 'AWS Managed Encryption',
                'notEncrypted': 'Not Encrypted',
                'na': 'N/A'
            };
            data.push({key: 'Encryption Type:', text: encryptionStrings[aggregate.encryptionType]});
            if (aggregate.encryptionType === 'ontapEncrypted') {
                data.push({key: 'Key ID:', text: aggregate.encryptionKeyId, trimmedText: true});
            }
            if (aggregate.encryptionType === 'na') {
                data.push({key: 'Key ID:', text: 'N/A'});
            }
            if (aggregate.homeNode) {
                data.push({key: 'Home Node:', text: aggregate.homeNode, trimmedText: true});
            }
            if (aggregate.providerVolumes[0].iops) {
                data.push({key: 'Provisioned IOPS:', text: aggregate.providerVolumes[0].iops});
            }

            return data;
        };

        $scope.showAggregateInfoBox = function (event, aggregate) {
            if (!$scope.selectedWorkingEnvironment._allowedOperations['aggregate-info']) return;
            $scope.lists = nocUtil.splitDataInto2Lists($scope.buildDataForInformationDialog(aggregate));

            nocDialog.dialog(aggregate.name, 'assets/templates/dialogs/general-info-dialog.html', {width: 866, fromTop: '15%', lists: $scope.lists})
                .then(() => {
                    event.stopPropagation();
                });
        };

        $scope.calculateRawCapacity = function calculateRawCapacity(aggregate) {
            return nocUtil.normalizeCapacity(nocUtil.sumCapacityUsingField(aggregate.providerVolumes, 'size'));
        };

        $scope.pollingSuccessCallback = function pollingSuccessCallback(aggregatesResponse) {
            _.forEach(aggregatesResponse, (aggregate) => {
                const providerVolume = $scope.diskTypes[_.get(aggregate, 'providerVolumes[0].diskType')];
                aggregate._diskTypeDisplay = providerVolume.display;
                aggregate._diskTypeLetters = providerVolume.shortName;
                aggregate._tier = aggregate._diskTypeLetters + (aggregate.capacityTier ? '+' + aggregate.capacityTier : '');
            });
            pidgeon.clear('aggregates-poll');
            const aggregates = aggregatesResponse;
            nocUtil.mergeExistingObject($scope.aggregates, aggregates, 'name');
            $scope.aggregates = aggregates;

            const isKvm = _.includes($scope.selectedWorkingEnvironment._instanceType, '5');
            const maxDisksProp = isKvm ? 'externalConfig.disksConfiguration.aws.maxDisksSingleNodeKvm' : 'externalConfig.disksConfiguration.aws.maxDisksSingleNode';
            const maxDisksHaProp = isKvm ? 'externalConfig.disksConfiguration.aws.maxDisksHaKvm' : 'externalConfig.disksConfiguration.aws.maxDisksHa';

            const maxDisksSingleNode = _.get($scope, maxDisksProp);
            const maxDisksHa = _.get($scope, maxDisksHaProp);
            const numReservedDisksForWorm = (_.get($scope.selectedWorkingEnvironment, 'ontapClusterProperties.wormEnabled') ? 1 : 0);
            const numReservedDisksHa = _.get($scope, 'externalConfig.disksConfiguration.aws.numReservedDisksHa') + numReservedDisksForWorm;
            const numReservedDisksSingleNode = _.get($scope, 'externalConfig.disksConfiguration.aws.numReservedDisksSingleNode') + numReservedDisksForWorm;
            const maxDisksAllowed = ($scope.selectedWorkingEnvironment.isHA ? maxDisksHa : maxDisksSingleNode) - ($scope.selectedWorkingEnvironment.isHA ? numReservedDisksHa : numReservedDisksSingleNode);

            $scope.maxDisksMessage = ($scope.selectedWorkingEnvironment.isHA ? 'In AWS, each Cloud Volumes ONTAP node in an HA pair ' : 'In AWS, a single-node Cloud Volumes ONTAP system ') +
                `has a limit of ${maxDisksAllowed} data disks. Take this limit into consideration when choosing a disk size to ensure that you can utilize the maximum capacity allowed by your Cloud Volumes ONTAP license.
                 Note that the data disk limit does not include the boot disk and root disk.`;

            $scope.disksCount = _.reduce($scope.aggregates, (sum, agg) => {
                return sum + agg.disks.length;
            }, 0);

            if ($scope.selectedWorkingEnvironment.isHA) {
                $scope.disksUtilizationMessage = `The data disk limit per node is ${maxDisksAllowed} (${maxDisksAllowed * 2} total)`;
            } else {
                $scope.disksUtilizationMessage = `The data disk limit is ${maxDisksAllowed}`;
            }
            const licenseTypeName = _.get($scope.selectedWorkingEnvironment, '_licenseType');
            const cloudProviderName = _.get($scope.selectedWorkingEnvironment, '_cloudProviderName');
            $scope.showMaxDisksMessage = cloudProviderName === 'AWS' && (licenseTypeName === 'Cloud Volumes ONTAP BYOL' || licenseTypeName === 'Cloud Volumes ONTAP Premium' || licenseTypeName === 'Cloud Volumes ONTAP Standard');

            doFiltering();
            getFilteredDisksCount();
        };

        $scope.pollingErrorCallback = function pollingErrorCallback(errorResponse) {
            pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving aggregates from server', errorResponse, 'aggregates-poll');
            $scope.aggregates = $scope.aggregates || [];
            doFiltering();
            getFilteredDisksCount();
        };

        $scope.capacityFeaturesPollingSuccessCallback = function capacityFeaturesPollingSuccessCallback(workingEnvironmentResponse) {
            pidgeon.clear('capacityFeatures-poll');
            $scope.selectedWorkingEnvironment.capacityFeatures = workingEnvironmentResponse.capacityFeatures;
            $scope.selectedWorkingEnvironment._providerVolumeTypes = workingEnvironmentResponse._providerVolumeTypes;
        };

        $scope.capacityFeaturesPollingErrorCallback = function capacityFeaturesPollingErrorCallback(errorResponse) {
            pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving capacityFeatures from server', errorResponse, 'capacityFeatures-poll');
        };

        let aggregatesPollingStopper = _.noop;
        let capacityFeaturesPollingStopper = _.noop;

        $scope.initializeController = function initializeController() {
            capacityFeaturesPollingStopper = workingEnvironmentResource.startCapacityFeaturesPolling($scope.selectedWorkingEnvironment,
                $scope.capacityFeaturesPollingSuccessCallback, $scope.capacityFeaturesPollingErrorCallback);

            aggregatesPollingStopper = aggregatesResource.startAggregatesPolling($scope.selectedWorkingEnvironment,
                $scope.pollingSuccessCallback, $scope.pollingErrorCallback);
        };

        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId, ['capacityFeatures'])
            .then($scope.initializeController, $scope.backToGallery);

        $scope.deleteAggregate = (aggregate) => {
            if (!$scope.selectedWorkingEnvironment._allowedOperations['aggregate-delete'] || !$scope.canDeleteAggregate(aggregate)) return;
            nocDialog
                .confirm(Messages.deleteAggregateConfirm({name: aggregate.name}), 'assets/templates/tabs/working-environments/delete-aggregate-dialog-content.html', {ontapEncrypted: aggregate.encryptionType === 'ontapEncrypted'})
                .then(() => {
                    aggregatesResource.deleteAggregate(nocHttp, $scope.selectedWorkingEnvironment, aggregate)
                        .then(() => {
                            pidgeon.inProgress('Aggregate delete in progress.', true);
                            delete $scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId];
                            aggregate._deleting = true;
                        });
                });
        };

        function getAwsEncriptionMissingKeyDisableMessage() {
            const isKeyFromAnotherAccount = nocEncryptionResourcesService.isKeyFromAnotherAccount($scope.selectedWorkingEnvironment._awsEncryptionKey);
            if (!isKeyFromAnotherAccount && $scope.selectedWorkingEnvironment._awsEncryptionKey && $scope.selectedWorkingEnvironment._awsEncryptionKey.status !== 'Enabled') {
                return 'Cannot allocate new AWS disks for this working environment since its AWS encryption master key is not enabled.';
            }
        }

        $scope.disableAddAggregateTootip = function disableAddAggregateTootip() {
            const awsEncriptionMissingKeyDisableMessage = getAwsEncriptionMissingKeyDisableMessage();
            if (awsEncriptionMissingKeyDisableMessage) return awsEncriptionMissingKeyDisableMessage;

            const volumePossibleTypes = _.values(_.cloneDeep($scope.selectedWorkingEnvironment._providerVolumeTypes));

            _.forEach(volumePossibleTypes, (type) => {
                type.supportedSizes = _.filter(type.supportedSizes, (size) => {
                    return size.maxDisksAllow.numOfDisks > 0;
                });
            });

            const capacityLimitReached = _.every(volumePossibleTypes, (type) => {
                return type.supportedSizes.length <= 0;
            });

            if (capacityLimitReached) return 'Cannot create new aggregate, capacity limit reached';
        };

        $scope.createVolumeDisableTooltip = function (workingEnvironment, aggregate) {
            if (aggregate.state === 'offline') {
                return 'Cannot create volume on offline aggregate';
            }

            if (!workingEnvironment._allowedOperations['aggregate-add-capacity']) {
                return 'Cannot create volume on workingEnvironment with status ' + workingEnvironment.status;
            }
        };

        $scope.addDisksToAggregateDisableTooltip = function addDisksToAggregateDisableTooltip(workingEnvironment, aggregate) {
            const awsEncriptionMissingKeyDisableMessage = getAwsEncriptionMissingKeyDisableMessage();
            if (awsEncriptionMissingKeyDisableMessage) return awsEncriptionMissingKeyDisableMessage;

            const curNumOfDisks = aggregate.disks.length;
            const maxNumOfDisksInAggregateLimit = workingEnvironment.capacityFeatures.maxDisksPerAggregate - (workingEnvironment.isHA ? curNumOfDisks / 2 : curNumOfDisks);

            if (maxNumOfDisksInAggregateLimit <= 0) {
                return 'Cannot add any disks to aggregate, maximum capacity of ' + workingEnvironment.capacityFeatures.maxDisksPerAggregate + ' disks per aggregate' + (workingEnvironment.isHA ? ' per node' : '') + ' reached';
            }

            const diskSize = aggregate.providerVolumes[0].size;
            const diskType = aggregate.providerVolumes[0].diskType;
            const supportedSizes = workingEnvironment._providerVolumeTypes[diskType].supportedSizes;
            const supportedSize = _.find(supportedSizes, {_identifier: $filter('normalizedCapacity')(diskSize, 2, true)});
            const maxDisksAllowedByServer = supportedSize.maxDisksAllow.numOfDisks;

            if (maxDisksAllowedByServer <= 0) {
                return 'Cannot add any disks to aggregate: ' + supportedSize.maxDisksAllow.reason;
            }

            if (aggregate.state === 'offline') {
                return 'Cannot add any disks to offline aggregate';
            }
        };

        $scope.showAddDisksAggregateForm = function (workingEnvironment, aggregate) {
            $scope.checkAndSetState('^.add-disks-aggregate', {
                workingEnvironmentId: workingEnvironment.publicId,
                aggregateName: aggregate.name
            });
        };

        $scope.startCreateNewAggregateWizard = function startCreateNewAggregateWizard(workingEnvironment) {
            $scope.checkAndSetState('create-new-aggregate-wizard', {
                workingEnvironmentId: workingEnvironment.publicId,
                type: workingEnvironment._workingEnvironmentTypeByLocation,
                provider: workingEnvironment._cloudProviderName
            });
        };

        $scope.$watch('searchAggregates', () => {
            doFiltering();
            getFilteredDisksCount();
        });

        const getFilteredDisksCount = () => {
            $scope.filteredDisksCount = _.reduce($scope.filteredAggregates, (sum, agg) => {
                return sum + agg.disks.length;
            }, 0);
        };

        $scope.$on('$destroy', () => {
            aggregatesPollingStopper();
            capacityFeaturesPollingStopper();
        });

        $scope.canDeleteAggregate = (aggregate) => {
            const volumes = aggregate.volumes || [];
            if (volumes.length === 0) {
                return true;
            }

            if (volumes.length > 1) {
                return false;
            }

            return _.find(volumes, 'rootVolume') && $scope.aggregates.length > 1;

        };

        $scope.cannotDeleteAggregateTooltip = (aggregate) => {
            if ($scope.canDeleteAggregate(aggregate)) {
                return '';
            } else {
                const volumes = aggregate.volumes || [];
                const hasRootVolume = _.find(volumes, 'rootVolume');
                if (!hasRootVolume && volumes.length > 0) {
                    return 'Aggregate with volumes cannot be deleted';
                }

                if (hasRootVolume && volumes.length > 1) {
                    return 'Aggregate with volumes cannot be deleted';
                }

                return 'Aggregate has root volume that cannot be moved';
            }
        };
    });
})();
