(function () {
    "use strict";
    angular.module("noc").controller("VpcController", function ($scope, workingEnvironmentResource, nocUtil, pidgeon, nocHttp) {
        var instancesPollingStopper = _.noop;

        $scope.loaded = function(){
            return !!($scope.vpc && $scope.instances);
        };

        workingEnvironmentResource.getVpcOfWorkingEnvironment(nocHttp, $scope.selectedWorkingEnvironment)
            .then(function (we) {
                $scope.vpc = we.awsProperties.vpc;
                $scope.selectedWorkingEnvironment.vpcCidrBlock = $scope.vpc.cidrBlock;
            });

        var doFiltering = nocUtil.getNgFilterFunc($scope, 'instances', 'filteredInstances', 'search', ['name', 'privateIpAddress']);

        $scope.$on(workingEnvironmentResource.INSTANCES_GET_EVENT, function(event, payload){
            pidgeon.clear("instances");

            if(payload.workingEnvironment.publicId !== $scope.selectedWorkingEnvironment.publicId) return;

            var instances = payload.instances;

            nocUtil.mergeExistingObject($scope.instances, instances, "privateIpAddress");

            $scope.instances = instances;

            _.forEach($scope.instances, function (instance) {
                instance.name = instance.name || "Unnamed";
            });

            doFiltering();
        });

        $scope.$watch('search', doFiltering);

        $scope.$on(workingEnvironmentResource.INSTANCES_GET_ERROR_EVENT, function(event, payload){
            if(payload.workingEnvironment.publicId !== $scope.selectedWorkingEnvironment.publicId) return;

            var error = payload.error;

            pidgeon.error(pidgeon.serverErrorPrefix + "retrieving VPC instances for VPC " + $scope.selectedWorkingEnvironment.vpcId, error, "instances");
        });

        instancesPollingStopper = workingEnvironmentResource.startInstancesPolling($scope.selectedWorkingEnvironment);

        $scope.$on("$destroy", instancesPollingStopper);
    });
}());