(() => {
    'use strict';
    angular.module('noc').controller('ChangeTierController', ($scope, nocHttp, $stateParams, $state, $log,
                                                              volumesResource, aggregatesResource, nocUI, pidgeon) => {
        const unloading = nocUI.loading();
        $scope.loaded = false;

        $scope.initializeController = _.once(() => {
            aggregatesResource.getAggregates(nocHttp, $scope.selectedWorkingEnvironment)
                .then((aggregates) => {
                    $scope.selectedVolume = _.find($scope.volumes, {
                        name: $stateParams.selectedVolumeName,
                        svmName: $stateParams.selectedVolumeSVM
                    });
                    $scope.currentTier = $scope.selectedWorkingEnvironment._providerVolumeTypes[$scope.selectedVolume.providerVolumeType];
                    $scope.selectedAggregate = _.find(aggregates, {
                        name: $scope.selectedVolume.aggregateName
                    });

                    $scope.volumeIops = _.get($scope.selectedAggregate, 'providerVolumes[0].iops');

                    //Change tier to io1 is currently disabled in UI, because we didn't have the time to develop UI for selecting
                    //iops. It's available by API.
                    $scope.possibleTiers = _.pickBy($scope.selectedWorkingEnvironment._providerVolumeTypes, (tier) => {
                        return tier.diskName !== 'io1';
                    });

                    $scope.wizardController = {}; //Tier selection component works with wizardController.

                    $scope.changeDiskQuoteRequest = {
                        workingEnvironmentId: $scope.selectedWorkingEnvironment.publicId,
                        svmName: $stateParams.selectedVolumeSVM,
                        name: $scope.selectedVolume.name,
                        verifyNameUniqueness: false,
                        size: $scope.selectedVolume.size,
                        initialSize: $scope.selectedVolume.usedSize,
                        enableThinProvisioning: $scope.selectedVolume.thinProvisioning,
                        iops: $scope.volumeIops
                    };
                    $scope.tierSelected($scope.currentTier);
                    $scope.wizardController.tieringEnabled = Boolean($scope.selectedVolume.capacityTier);
                    $scope.currentTier.tieringEnabled = Boolean($scope.selectedVolume.capacityTier);
                    $scope.loaded = true;

                    const availableTieringPoliciesForTier = _.find(_.get($scope.selectedWorkingEnvironment, 'capacityFeatures.supportedCapacityTiers.supportedCapacityTiersPerVolumeType'), {volumeType: _.get($scope.wizardController, 'selectedTier.name')});
                    $scope.availableTieringPoliciesForTier = availableTieringPoliciesForTier ? availableTieringPoliciesForTier.availableTieringPolicies : undefined;

                    $scope.wizardController.forceCompositeVersion = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.forceCompositeVersion;
                    $scope.wizardController.compositeSupported = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.compositeSupported;
                    if ($scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons) {
                        $scope.wizardController.capacityTiersDisableReasons = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons;
                    }

                    if ($scope.selectedVolume.tieringPolicy) {
                        $scope.wizardController.tieringPolicy = {
                            type: $scope.selectedVolume.tieringPolicy
                        };
                    } else {
                        $scope.wizardController.tieringPolicy = {type: !$scope.selectedVolume.capacityTier ? 'none' : (_.includes($scope.availableTieringPoliciesForTier, 'auto') ? 'auto' : 'snapshot_only')};
                    }
                    unloading();
                });
        });

        $scope.shouldPassCapacityTier = () => {
            if ($scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons &&
                $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons.length > 0) return false;

            if (!$scope.wizardController.compositeSupported || $scope.wizardController.encryptionType === 'ONTAP' ||
                (!_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes($scope.wizardController.selectedTier.supportedCapacityTiers, objectStorageType))) ||
                !$scope.selectedVolume.thinProvisioning) return false;

            return ($scope.wizardController.forceCompositeVersion || ($scope.wizardController.tieringPolicy && $scope.wizardController.tieringPolicy.type !== 'none'));
        };

        //This function is an indication NOT to send tiering policy eventhough capacity tier was sent
        $scope.shouldNotPassTieringPolicy = () => {
            return $scope.wizardController.forceCompositeVersion && ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0);
        };

        $scope.getObjectStorageName = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            return providerToObjectStorage[$scope.selectedWorkingEnvironment._cloudProviderName];
        };

        $scope.disabledTooltipForTier = function disabledTooltipForTier(tier) {
            if (tier && tier.disabledMessage) {
                return tier.disabledMessage;
            }
            if ($scope.selectedVolume && !$scope.selectedVolume.thinProvisioning) {
                return $scope.getObjectStorageName() + ' tiering is not available with "No Storage Efficiency" usage profile.';
            }
            if (!_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes($scope.wizardController.selectedTier.supportedCapacityTiers, objectStorageType))) {
                return $scope.getObjectStorageName() + ' tiering is not available for this disk type.';
            }
        };

        $scope.tierSelected = function tierSelected(tier) {
            $scope.wizardController.selectedTier = tier;

            const availableTieringPoliciesForTier = _.find(_.get($scope.selectedWorkingEnvironment, 'capacityFeatures.supportedCapacityTiers.supportedCapacityTiersPerVolumeType'), {volumeType: _.get($scope.wizardController, 'selectedTier.name')});
            $scope.availableTieringPoliciesForTier = availableTieringPoliciesForTier ? availableTieringPoliciesForTier.availableTieringPolicies : undefined;

            if ($scope.disabledTooltipForTier(tier)) {
                $scope.wizardController.tieringEnabled = false;
            }
        };

        const actionName = 'changing volume tier';

        $scope.changeTier = function changeTier(quoteResponse) {
            const volumeBaseUrl = $scope.selectedWorkingEnvironment._pathPrefix + 'volumes/' + $scope.selectedWorkingEnvironment.publicId + '/' + $scope.selectedVolume.svmName + '/' + $scope.selectedVolume.name;
            const changeTierRequest = _(quoteResponse)
                .pick(['aggregateName', 'numOfDisks', 'newAggregate'])
                .assignIn({
                    newDiskTypeName: $scope.changeDiskQuoteRequest.providerVolumeType,
                    newCapacityTier: $scope.changeDiskQuoteRequest.capacityTier,
                    newTieringPolicy: $scope.changeDiskQuoteRequest.tieringPolicy,
                    iops: $scope.changeDiskQuoteRequest.iops
                })
                .value();

            return nocHttp.post({
                url: volumeBaseUrl + '/change-tier',
                data: changeTierRequest
            }, actionName, nocHttp.BLOCK)
                .then(() => {
                    $scope.boxGoBack();
                    pidgeon.inProgress('Changing tier started.', true);
                });
        };

        $scope.isPristine = () => {
            return (_.get($scope.wizardController, 'selectedTier.name') === _.get($scope.currentTier, 'name')) && ((_.get($scope.wizardController, 'tieringPolicy.type') === 'none' && !_.get($scope.selectedVolume, 'tieringPolicy')) || (_.get($scope.wizardController, 'tieringPolicy.type') === _.get($scope.selectedVolume, 'tieringPolicy')));
        };

        $scope.startChangeTierProcess = function startChangeTierProcess() {
            if ($scope.wizardController.selectedTier.name !== $scope.selectedVolume.providerVolumeType) {
                delete $scope.changeDiskQuoteRequest.iops;
            }
            $scope.changeDiskQuoteRequest.providerVolumeType = $scope.wizardController.selectedTier.name;
            $scope.changeDiskQuoteRequest.capacityTier = $scope.shouldPassCapacityTier() ? ($scope.selectedWorkingEnvironment._isAWS ? 'S3' : ($scope.selectedWorkingEnvironment._isAzure ? 'Blob' : 'cloudStorage')) : undefined;
            $scope.changeDiskQuoteRequest.tieringPolicy = $scope.shouldPassCapacityTier() && !$scope.shouldNotPassTieringPolicy() ? _.get($scope.wizardController, 'tieringPolicy.type') : undefined;

            volumesResource.askForQuote(nocHttp,
                $scope.selectedWorkingEnvironment._pathPrefix,
                $scope.changeDiskQuoteRequest,
                actionName)
                .then((quoteResponse) => {
                    return volumesResource.getUserApprovalForNewVolumeIfNeeded(quoteResponse,
                        $scope.selectedWorkingEnvironment._cloudProviderName,
                        $scope.selectedWorkingEnvironment._providerVolumeTypes[$scope.changeDiskQuoteRequest.providerVolumeType].display,
                        $scope.selectedWorkingEnvironment.isHA,
                        $scope.selectedVolume.name);
                })
                .then($scope.changeTier)
                .catch($log.error);
        };

        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId, ['capacityFeatures'])
            .then(() => {
                $scope.$watch('volumes', (volumes) => {
                    /* istanbul ignore else */
                    if (volumes) {
                        $scope.initializeController();
                    }
                });
            });
    });
})();
