(() => {
    'use strict';

    angular.module('noc').controller('DataFabricController', ($scope, nocHttp, workingEnvironmentResource, pidgeon, actionRequiredResource,
                                                              nocUI, $state, Constants, nocDialog, $rootScope, workingEnvironmentStatuses,
                                                              nocLocalStorage, routeLogger, $timeout, nocCurrentUserService, nocSelectedWorkspaceOrTenantService) => {
        const localStorage = nocLocalStorage.GALLERY_VIEW_SHOWN(nocSelectedWorkspaceOrTenantService.getPublicId()); //eslint-disable-line new-cap

        var actionRequiredPollingStopper = _.noop;
        $scope.showReplicationAnimation = false;
        $scope.showTable = localStorage.restore() || false;

        $scope.isAllowed = (requiredPermissions) => {
            return nocCurrentUserService.isAllowed(requiredPermissions, nocSelectedWorkspaceOrTenantService.getSelectedWorkspace());
        };

        $scope.switchView = function(showTable) {
            $scope.showTable = showTable;
            localStorage.save($scope.showTable);
        };

        var computingCapacityMap = {};

        $scope.resetCapacityByTypeMap = function (workingEnvironments) {
            $scope.capacityByTypeMap = {};
            if (_.isEmpty(workingEnvironments)) return;

            var types = _(workingEnvironments).map("_workingEnvironmentTypeByLocation").union().value();

            _.forEach(types, function(type ){
                $scope.capacityByTypeMap[type] = {"_normalizedReservedSizeTB" : {size: 0, unit : "TB" }, count:0};
                $scope.capacityByTypeMap[type].type = type;
                $scope.capacityByTypeMap[type].description =  Constants.typeToLongNameMap[type];
            });
        };

        $scope.updateCapacityByTypeMap = function(workingEnvironment, workingEnvironmentPreviousState) {
            //in case of a dataONTAP working environment with location different than ON_PREM treat it as NPS for total size calculations
            var workingEnvironmentType = workingEnvironment._workingEnvironmentTypeByLocation;
            $scope.capacityByTypeMap[workingEnvironmentType]._normalizedReservedSizeTB.size += workingEnvironment._normalizedReservedSizeTB.size;

            if(workingEnvironmentPreviousState){
                $scope.capacityByTypeMap[workingEnvironmentType]._normalizedReservedSizeTB.size -= workingEnvironmentPreviousState._normalizedReservedSizeTB.size;
            }else{
                $scope.capacityByTypeMap[workingEnvironmentType].count++;
            }
        };

        $scope.shouldShowReplicationAction = function (workingEnvironment) {
            $scope.createReplicationMapByType($scope.workingEnvironments);
            //workingEnvironment can be either ON_PREM or VSA, otherwise it would have been filtered earlier by the allowed operations.
            //there should be at least one possible target
            var otherPossibleTargetType = workingEnvironment.workingEnvironmentType === 'VSA' ? 'ON_PREM' : 'VSA';
            return $scope.replicationMapByType[workingEnvironment.workingEnvironmentType] > 1 || $scope.replicationMapByType[otherPossibleTargetType] > 0;
        };


        $scope.updateShowReplicateAnimation = function() {
            $rootScope.activateTutorial("assets/templates/tabs/working-environments/replicate-tutorial.html");
        };

        $scope.actionsCalculated = false;

        $scope.setWorkingEnvironments(null);

        //Initialization
        $scope.clearActionsRequired();

        $scope.getWorkingEnvironmentById = function(publicId) {
            return _.find($scope.workingEnvironments, {publicId : publicId});
        };

        $scope.removeWE = function(we) {
            $scope.removeWorkingEnvironment(we, false)
                .then(function() {
                    _.remove($scope.workingEnvironments, {publicId: we.publicId});
                });
        };

        $scope.showFailureMessage = function(we) {
            nocDialog.dialog(we.name, "assets/templates/dialogs/include-html-dialog.html", {ngModel: we.status.message});
        };

        $scope.gotoCreateWE = function gotoCreateWE(startDiscoverWizard, publicId, pathPrefix) {
            const weTypes = {
                '/aws/ha/': 'HA_AWS',
                '/azure/ha/': 'HA_AZURE',
                '/azure/vsa/': 'VSA_AZURE',
                '/vsa/': 'VSA_AWS',
                '/gcp/vsa/': 'VSA_GCP'
            };
            const weType = weTypes[pathPrefix];

            $scope.checkAndSetState('wizard', {
                from: $state.current.name,
                id: publicId,
                type: weType,
                pathPrefix,
                startDiscoverWizard
            });
        };

        $scope.recreateWizard = function(we) {
            $scope.gotoCreateWE(null, we.publicId, we._pathPrefix);
        };

        $scope.goToTimeline = function(we) {
            $state.go("main.timeline", {we: we.name});
        };

        $scope.setComputingCapacityForWE = function(we, state) {
            computingCapacityMap[we.publicId] = state;
        };

        $scope.getComputingCapacityForWE = function(we) {
            return computingCapacityMap[we.publicId];
        };

        $scope.updateCredentials = function () {
            nocDialog.dialog("Update " + $scope.selectedWorkingEnvironment._typeLongName + " Working Environment Password",
                "assets/templates/tabs/working-environments/update-credentials.html",
                {workingEnvironment: $scope.selectedWorkingEnvironment});
        };

        var workingEnvironmentsUnloading = nocUI.loading();

        $scope.mergeWorkingEnvironmentsList = function(previousWorkingEnvironments, currentWorkingEnvironments){
            var previousWePublicIds = _.keyBy(previousWorkingEnvironments, 'publicId');
            var currentWePublicIds = _.keyBy(currentWorkingEnvironments, 'publicId');

            //newWeList is all of the previous working environments that are also in the currentWorkingEnvironmentList
            var newWeList = _.filter(previousWorkingEnvironments, function(we){
                return currentWePublicIds[we.publicId];
            });

            //add to newWeList all of the workingEnvironments in currentWorkingEnvironments that were not in the previous list
            _.forEach(currentWorkingEnvironments, function(we){
                if(!previousWePublicIds[we.publicId]) newWeList.push(we);
            });

            return _.sortBy(newWeList, ["workingEnvironmentType", "workingEnvironmentName"]);
        };

        $scope.mergeNewWorkingEnvironmentResults = function(oldWorkingEnvironmentsArray, newWorkingEnvironment, oldWorkingEnvironment){
            var index = _.findIndex(oldWorkingEnvironmentsArray, {publicId: newWorkingEnvironment.publicId});
            if(index > -1){
                oldWorkingEnvironmentsArray[index] = newWorkingEnvironment;
                $scope.updateCapacityByTypeMap(newWorkingEnvironment, oldWorkingEnvironment);
                /* istanbul ignore else */
                if(newWorkingEnvironment.workingEnvironmentType === "VSA"){
                    if (_.isEmpty(newWorkingEnvironment.activeActions) && !_.isEmpty(oldWorkingEnvironment.activeActions)) {
                        delete $scope.actionsRequired[oldWorkingEnvironment.publicId];
                        $scope.setComputingCapacityForWE(oldWorkingEnvironment, true);
                        nocHttp.get(newWorkingEnvironment._pathPrefix + "working-environments/" + newWorkingEnvironment.publicId + "?fields=status,actionsRequired")
                            .then(function (workingEnvironmentWithActionRequiredResponse) {
                                var map = {};
                                map[newWorkingEnvironment.publicId] = workingEnvironmentWithActionRequiredResponse.actionsRequired;
                                $scope.setActionRequiredFor(newWorkingEnvironment.publicId, false, map);
                            }).finally(function () {
                                $scope.setComputingCapacityForWE(oldWorkingEnvironment, false);
                            });
                    }
                }
            }
        };

        $scope.mergeReplicationProperties = function(oldWorkingEnvironmentsArray, workingEnvironmentWithReplication){
            var index = _.findIndex(oldWorkingEnvironmentsArray, {publicId: workingEnvironmentWithReplication.publicId});
            /* istanbul ignore else */
            if (index > -1 && workingEnvironmentWithReplication.replicationProperties) {
                var old = oldWorkingEnvironmentsArray[index];
                var newWe = _.cloneDeep(old);
                newWe.replicationProperties = workingEnvironmentWithReplication.replicationProperties;
                newWe.peers = newWe.replicationProperties.peers;
                newWe.replicationTargets = newWe.replicationProperties.replicationTargets;
                newWe._peers = workingEnvironmentWithReplication._peers;
                oldWorkingEnvironmentsArray[index] = newWe;
            }
        };

        $scope.$on(workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, function(event, payload) {
            workingEnvironmentsUnloading();
            if (nocSelectedWorkspaceOrTenantService.getPublicId() === payload.tenantId) {
                pidgeon.clear("list");
                var newWeList = $scope.mergeWorkingEnvironmentsList($scope.workingEnvironments || [], payload.workingEnvironments);

                $scope.resetCapacityByTypeMap(newWeList);

                $timeout(function(){
                    _.forEach(newWeList, function (workingEnvironment) {
                        $scope.updateCapacityByTypeMap(workingEnvironment);
                        if(!workingEnvironment._moreDataRequest){
                            var providerProperties = workingEnvironment._cloudProviderName === 'AWS' ? "awsProperties" : "providerProperties";
                            var haProperties = workingEnvironment.isHA? ",haProperties" : "";
                            workingEnvironment._moreDataRequest = workingEnvironmentResource
                                .getWorkingEnvironmentByPublicId(nocHttp, workingEnvironment.publicId,
                                    workingEnvironment._pathPrefix,
                                    'status,ontapClusterProperties.fields(upgradeVersions,nodes),encryptionProperties,' + providerProperties + ',reservedSize,schedules,cloudSyncProperties,k8sProperties' + haProperties)
                                .then((newWorkingEnvironmentResponse) => {
                                    $timeout(() => {
                                        //Make sure to update the selected working environment with the new status bug fix NOC-9940 Initializing message remains in right panel although WE status is on
                                        if ($scope.selectedWorkingEnvironment && $scope.selectedWorkingEnvironment.publicId === newWorkingEnvironmentResponse.publicId) {
                                            _.assignIn($scope.selectedWorkingEnvironment, newWorkingEnvironmentResponse);
                                        }
                                        $scope.mergeNewWorkingEnvironmentResults($scope.workingEnvironments, newWorkingEnvironmentResponse, workingEnvironment);
                                        workingEnvironment._moreDataRequest = null;

                                        workingEnvironmentResource
                                            .getWorkingEnvironmentByPublicId(nocHttp, workingEnvironment.publicId, workingEnvironment._pathPrefix, 'status,replicationProperties')
                                            .then((workingEnvironmentWithReplicationProperties) => {
                                                $scope.mergeReplicationProperties($scope.workingEnvironments, workingEnvironmentWithReplicationProperties);
                                                $scope.createReplicationMapByType(newWeList);
                                            });
                                    });
                                });
                        }
                    });
                });

                $scope.selectedWorkingEnvironment = $scope.selectedWorkingEnvironment ? _.find(newWeList, {publicId: $scope.selectedWorkingEnvironment.publicId}) : undefined;

                $scope.setWorkingEnvironments(newWeList);

                if (actionRequiredPollingStopper === _.noop && newWeList.length > 0) {
                    actionRequiredPollingStopper = actionRequiredResource.startGlobalActionRequiredPolling(nocSelectedWorkspaceOrTenantService.getPublicId(), _.noop);
                    $scope.setComputingCapacity(true);
                }
            }
        });

        $scope.$on(workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_ERROR_EVENT, function(event, payload){
            if(nocSelectedWorkspaceOrTenantService.getPublicId() !== payload.tenantId) return;

            pidgeon.error(pidgeon.serverErrorPrefix + "retrieving working environments", payload.error, "list");
        });

        $scope.$on(actionRequiredResource.GLOBAL_ACTION_REQUIRED_GET_EVENT, function(event, actionRequiredResponse) {
            pidgeon.clear("action-required");
            $scope.actionsCalculated = false;
            $scope.totalNumOfActionsRequired = 0;

            _.forEach($scope.workingEnvironments, function (workingEnvironment) {
                $scope.setActionRequiredFor(workingEnvironment.publicId, !_.isEmpty(workingEnvironment.activeActions), actionRequiredResponse);
                if ($scope.actionsRequired[workingEnvironment.publicId]) {
                    $scope.totalNumOfActionsRequired += $scope.actionsRequired[workingEnvironment.publicId].length;
                }
            });
            $scope.setComputingCapacity(false);
            $scope.actionsCalculated = true;
        });

        $scope.$on(actionRequiredResource.GLOBAL_ACTION_REQUIRED_ERROR_EVENT, function(event, error) {
            pidgeon.error(pidgeon.serverErrorPrefix + "retrieving required actions for capacity", error, "action-required");
            $scope.setComputingCapacity(false);
        });

        var galleryPollingStopper = workingEnvironmentResource.startWorkingEnvironmentsPolling();

        $scope.onClick = function(workingEnvironment) {
            $scope.$apply(function() {
                $scope.selectedWorkingEnvironment = _.find($scope.workingEnvironments, {publicId: workingEnvironment.publicId});
            });
        };

        $scope.onDrop = function(source, target) {
            if (workingEnvironmentStatuses.isReplicationAllowed(source, target)) {
                $scope.checkAndSetState('snap-mirror-wizard', {from: "main.working-environments.index", slTenantId: nocSelectedWorkspaceOrTenantService.getPublicId(),
                    sourceId: source.publicId, targetId: target.publicId, sourceType:source._workingEnvironmentTypeByLocation, sourceProvider: source._cloudProviderName, targetType:target._workingEnvironmentTypeByLocation, targetProvider: target._cloudProviderName});
            }
        };

        $scope.clearSelectedWorkingEnvironment = function() {
            $scope.selectedWorkingEnvironment = null;
        };

        $scope.selectWorkingEnvironment = function(workingEnvironment){
            $scope.selectedWorkingEnvironment = workingEnvironment;
        };

        $scope.dataFabricInformationFilter = (capacity) => capacity.count > 0;

        $scope.$on("$destroy", function(){
            galleryPollingStopper();
            actionRequiredPollingStopper();
        });

        $scope.notifyGalleryAction = (actionClicked) => {
            routeLogger.reportGalleryAction(actionClicked);
        };
    });
})();
