(function () {
    'use strict';

    angular.module('noc').controller('EditTagsController', ($scope, nocHttp, nocUI, workingEnvironmentResource,
                                                            nocDialog, $q, nocUtil, pidgeon, Patterns, MetadataResource) => {
        $scope.initialize = () => {
            _.set($scope, 'tags.customTags', []);
            _.set($scope, 'workingEnvironmentTypeAndProvider.providerName', $scope.selectedWorkingEnvironment._cloudProviderName);
            $scope.maxTags = 50;
            //Either AWS or Azure
            const isAws = $scope.selectedWorkingEnvironment.cloudProviderName === 'Amazon';
            $scope.keyNamePlaceHolder = $scope.selectedWorkingEnvironment.cloudProviderName === 'Amazon' ? 'Up to 127 characters' : 'Up to 512 characters';
            $scope.maxUserTags = isAws ? 20 : 15;

            const subscriptionIdParam = isAws ? '' : '?subscriptionId=' + _.get($scope, 'selectedWorkingEnvironment.providerProperties.subscriptionId');
            const cloudProviderIdParam = `cloudProviderAccountId=${$scope.selectedWorkingEnvironment._cloudProviderAccountId}`;
            const params = isAws ? `?${cloudProviderIdParam}` : `${subscriptionIdParam}&${cloudProviderIdParam}`;

            $q.all([
                workingEnvironmentResource.getTags(nocHttp, $scope.selectedWorkingEnvironment),
                MetadataResource.getTagKeys(nocHttp, $scope.selectedWorkingEnvironment._pathPrefix, params)
            ])
                .then(_.spread((tagsResponse, keysData) => {
                    $scope.tags.customTags = tagsResponse.tags;
                    $scope.originalTags = _($scope.tags.customTags).sortBy(['tagKey'])
                        .cloneDeep();

                    //Find existing tags for autocomplete
                    filterCurrentTags(keysData);
                    $scope.existingTagNames = _.map(keysData, 'key');
                    $scope.existingTags = {};
                    _.forEach(keysData, (k) => {
                        $scope.existingTags[k.key] = k.values;
                    });

                    //Get additional tags data
                    if ($scope.selectedWorkingEnvironment.cloudProviderName === 'Amazon') {
                        $scope.additionalTags();
                        $scope.numberOfAdditionalTags = $scope.selectedWorkingEnvironment.isHA ? $scope.additionalTagsData.data.length : $scope.additionalTagslists.leftList.length;
                    }

                    const regexTagKey = $scope.selectedWorkingEnvironment.cloudProviderName === 'Amazon' ? 'amazonTagKey' : 'azureTagKey';
                    $scope.weTagKeyRegex = new RegExp(Patterns[regexTagKey].regex);
                    $scope.workingEnvironmentTagKeyError = regexTagKey;
                    const regexTagValue = $scope.selectedWorkingEnvironment.cloudProviderName === 'Amazon' ? 'amazonTagValue' : 'azureTagValue';
                    $scope.weTagValueRegex = new RegExp(Patterns[regexTagValue].regex);
                    $scope.workingEnvironmentTagValueError = regexTagValue;

                    //Show page only when loaded
                    $scope.tagsLoaded = true;
                }));
        };

        var filterCurrentTags = function filterCurrentTags(keysData) {
            _.forEach($scope.tags.customTags, (tag) => {
                _.remove(keysData, {key: tag.tagKey});
            });
        };

        $scope.noChanges = function () {
            return _.isEqual($scope.originalTags, _.sortBy($scope.tags.customTags, ['tagKey']));
        };

        $scope.isValid = function () {
            return $scope.editTagsForm.$valid;
        };

        $scope.addCustomTag = function () {
            if ($scope.addAllowed()) {
                $scope.addTagsPressed = true;
                $scope.tags.customTags.push({tagKey: '', tagValue: ''});
            }
        };

        $scope.removeCustomTag = function (tag) {
            $scope.tags.customTags = _.without($scope.tags.customTags, tag);
        };

        $scope.customTagsCount = function () {
            return _.filter($scope.tags.customTags, (t) => {
                return t.tagKey.length > 0;
            }).length;
        };

        $scope.updateTags = function () {
            if (!$scope.isValid()) {
                nocUI.handleFormValidity($scope.editTagsForm, $scope);
            } else {
                workingEnvironmentResource.setTags(nocHttp, $scope.selectedWorkingEnvironment, $scope.tags.customTags)
                    .then(() => {
                        pidgeon.success('Working environment tags saved');
                        $scope.showMonitoring();
                    });
            }
        };

        $scope.tags = function (selectedWorkingEnvironment) {
            function weAwsTags(we) {
                if (we.isHA) return {};

                const instance = nocUtil.getOccmInstanceAws(selectedWorkingEnvironment);

                return _.get(instance, 'tags');
            }

            return selectedWorkingEnvironment._cloudProviderName === 'AWS' ?
                weAwsTags(selectedWorkingEnvironment) :
                _.get(selectedWorkingEnvironment, 'providerProperties.tags');
        };

        $scope.addAllowed = function () {
            if ($scope.selectedWorkingEnvironment._cloudProviderName === 'Azure') return true;
            const allowed = $scope.numberOfAdditionalTags + $scope.tags.customTags.length < $scope.maxTags;
            $scope.popoverText = allowed ? undefined : 'Unable to add a new tag because the number of "Working Environment Tags" and "Additional Tags" has reached the limit of ' + $scope.maxTags + ' tags.';

            return allowed;
        };

        $scope.hasScrollBar = function () {
            //Deducting 20 (about the button row height) to flow is smoother
            return $('body').get(0).scrollHeight - 20 > $('body').height();
        };

        $scope.additionalTags = function () {
            let tags = $scope.tags($scope.selectedWorkingEnvironment),
                nodes, instances,
                index = 1;
            $scope.additionalTagslists = {};
            $scope.additionalTagsData = {};

            //Filter out editable tags
            const editableTagKeys = _.map($scope.originalTags, 'tagKey');
            tags = _.omit(tags, editableTagKeys);

            if ($scope.selectedWorkingEnvironment._cloudProviderName === 'AWS' && $scope.selectedWorkingEnvironment.isHA) {
                instances = _.get($scope.selectedWorkingEnvironment, 'awsProperties.instances');
                const nodesCloudProviderIds = _.map($scope.selectedWorkingEnvironment.ontapClusterProperties.nodes, 'cloudProviderId');
                nodes = _(instances).filter((instance) => {
                    return _.includes(nodesCloudProviderIds, instance.id);
                })
                    .sortBy('name')
                    .value();
                $scope.additionalTagsData.data = [];

                _.forEach(nodes, (node) => {
                    _.forEach(node.tags, (value, key) => {
                        let entry = _.find($scope.additionalTagsData.data, {key});
                        if (!_.includes(editableTagKeys, key)) {
                            if (!entry) {
                                entry = {
                                    key,
                                    isSystemTag: _.includes(nocUtil.getSystemTags(), key)
                                };
                                entry['value' + index] = value;
                                $scope.additionalTagsData.data.push(entry);
                            } else {
                                entry['value' + index] = value;
                            }
                        }
                    });
                    index += 1;
                });

                $scope.additionalTagsData.data = _.sortBy($scope.additionalTagsData.data, 'key');
                if (nodes && nodes.length > 0) {
                    $scope.additionalTagsData.node1 = nodes[0].name;
                }

                if (nodes && nodes.length > 1) {
                    $scope.additionalTagsData.node2 = nodes[1].name;
                }
            } else {
                const list = _.sortBy(_.map(tags, (value, key) => {
                    return {text: value, key, isSystemTag: _.includes(nocUtil.getSystemTags(), key), trimmedText: true, trimmedKey: true};
                }), 'key');
                $scope.additionalTagslists.leftList = list;
                $scope.additionalTagslists.keysHeader = 'Key';
                $scope.additionalTagslists.valuesHeader = 'Value';
            }
        };

        $scope.showAdditionalTagsDialog = function () {
            const dialogText = 'The following tags are applied to some of the ' +
                $scope.selectedWorkingEnvironment._cloudProviderName + ' resources associated with this working environment. You can manage these tags from the ' +
                $scope.selectedWorkingEnvironment._cloudProviderName + ' console.';

            const warningText = 'Do not edit or delete system tags from the ' + $scope.selectedWorkingEnvironment._cloudProviderName + ' console. Doing so can cause permanent damage to the working environment.';

            if ($scope.selectedWorkingEnvironment.isHA) {
                nocDialog.dialog('Additional Tags [' + $scope.numberOfAdditionalTags + ']', 'assets/templates/tabs/working-environments/ha-tags.html', {
                    dialogText,
                    width: 1250,
                    tags: $scope.additionalTagsData,
                    hasWarning: _.filter($scope.additionalTagsData.data, {isSystemTag: true}).length > 0,
                    warningText,
                    limitedHeight: true
                });
            } else {
                nocDialog.dialog('Additional Tags [' + $scope.numberOfAdditionalTags + ']', 'assets/templates/dialogs/general-info-dialog.html',
                    {dialogText,
                        width: 800,
                        lists: $scope.additionalTagslists,
                        hasWarning: _.filter($scope.additionalTagslists.leftList, {isSystemTag: true}).length > 0,
                        warningText,
                        limitedHeight: true});
            }
        };

        $scope.initialize();
    });
}());
