(function () {
    "use strict";
    angular.module('noc').controller('HaEditRouteTablesDialogController', function ($scope, $q, MetadataResource, nocHttp, nocUI) {

        $scope.initialize = function () {
            $scope.wizardController = {};
            $scope.controllerData = { selectAll: false };
            $scope.initRouteTables();
        };

        $scope.initRouteTables = function() {
            const region = $scope.selectedWorkingEnvironment.awsProperties.regionName;
            const vpcId = $scope.selectedWorkingEnvironment.awsProperties.vpc.vpcId;
            const roleArn = _.get($scope, 'selectedWorkingEnvironment.awsProperties.roleArn');
            const cloudProviderAccountId = _.get($scope, 'selectedWorkingEnvironment.awsProperties.cloudProviderAccountId');

            const getWE = nocHttp.get({ url: "/aws/ha/working-environments/" + $scope.selectedWorkingEnvironment.publicId, params: {fields: 'haProperties'}}, "retrieving current route table ids", nocHttp.LOADING);
            const getVPCs = nocHttp.get({ url: "/aws/ha/metadata/vpcs", params: {region, roleArn, cloudProviderAccountId}}, "retrieving VPCs", nocHttp.LOADING);
            const getFullRouteTables = MetadataResource.getRouteTables(nocHttp, region, vpcId, cloudProviderAccountId);

            $q.all([getWE, getVPCs, getFullRouteTables])
                .then(_.spread(function (we, vpcs, fullRouteTables) {
                    var currentRouteTablesIds = we.haProperties.routeTables;
                    var currentVPC = _.find(vpcs, {vpcId: vpcId});
                    $scope.wizardController.routeTables = _(fullRouteTables)
                        .map(function (row) {
                            row._subnets = _(row.subnets).map(function (routeTableSubnetId) {
                                return currentVPC && _.find(currentVPC.subnets, {subnetId: routeTableSubnetId});
                            }).sortBy('cidr').value();
                            // select the current route tables
                            row.selected = _.includes(currentRouteTablesIds, row.id);
                            return row;
                        })
                        // show the current route tables first
                        .orderBy('selected','desc')
                        .value();

                    $scope.onSelectionChange();
                }));
        };

        $scope.onSelectionChange = function() {
            $scope.controllerData.selectAll = _.every($scope.wizardController.routeTables, {selected: true});
            $scope.selectedRouteTableIds = _($scope.wizardController.routeTables).filter('selected').map('id').value();
        };

        $scope.onSelectAll = function() {
            _.forEach($scope.wizardController.routeTables, function(row) {
                row.selected = $scope.controllerData.selectAll;
            });

            $scope.onSelectionChange();
        };

        $scope.expandRow = function (row) {
            row.expanded = !row.expanded;
            _.forEach($scope.wizardController.routeTables, function(rowIter) {
                if (row !== rowIter) rowIter.expanded = false;
            });
        };

        $scope.isValid = function isValid(){
            return $scope.routingTablesForm.$valid;
        };

        $scope.saveRouteTables = function saveRouteTables() {
            $scope.userClickedNext = true;
            if ($scope.isValid()) {
                var unblock = nocUI.block();
                // using the main screen's noc http will show the error on the main screen
                $scope.mainScreenNocHttp.put({
                    url: "/aws/ha/working-environments/" + $scope.selectedWorkingEnvironment.publicId + "/route-tables",
                    data: {routeTableIds: $scope.selectedRouteTableIds}
                }, "saving route tables", $scope.mainScreenNocHttp.LOADING)
                    .finally(function () {
                        unblock();
                        $scope.closeDialog();
                    });
            }
        };

        $scope.initialize();
    });
})();