(() => {
    'use strict';

    angular.module('noc').controller('LicenseController', ($filter, nocHttp, $scope, aggregatesResource, nocDialog, versionUtil, pidgeon) => {
        $scope.initialize = () => {
            $scope.localVsaResponse = $scope.vsaResponse || $scope.offVsaResponse;

            $scope.evalLicense = _.get($scope.selectedWorkingEnvironment, 'ontapClusterProperties.evaluation');

            if ($scope.localVsaResponse.cluster && $scope.localVsaResponse.cluster.licenseType) {
                $scope.isBYOL = _.includes($scope.localVsaResponse.cluster.licenseType.name, 'BYOL');
            }

            if (!$scope.localVsaResponse) {
                $scope.showMonitoring();
            } else {
                if ($scope.selectedWorkingEnvironment) {
                    if ($scope.selectedWorkingEnvironment.status.status === 'off') return; //No need to get aggregates in off state
                    $scope.getAggregates();
                    $scope.isVersionPost84 = versionUtil.compare($scope.selectedWorkingEnvironment._ontapVersion, 'ONTAP-8.4') >= 0;
                }

                if ($scope.isBYOL) {
                    $scope.getRegistrationDetails();
                }
            }
        };

        $scope.getAggregates = () => {
            aggregatesResource.getAggregates(nocHttp, $scope.selectedWorkingEnvironment)
                .then((aggregatesResponse) => {
                    $scope.licenseAggregates = aggregatesResponse;
                    $scope.attachedProviderCapacity = $filter('allocatedAggregatesCapacity')($scope.licenseAggregates);
                    if ($scope.selectedWorkingEnvironment.isHA) {
                        $scope.attachedProviderCapacity.size /= 2;
                    }
                });
        };

        $scope.getRegistrationDetails = () => {
            $scope.registrationDetails = {serialNumbers: [''], licenseExpiryDates: ['']};
            nocHttp.get($scope.selectedWorkingEnvironment._pathPrefix + 'working-environments/' + $scope.selectedWorkingEnvironment.publicId + '?fields=supportRegistrationInformation',
                'retrieving registration status from server')
                .then((vsaResponse) => {
                    const nodeNames = _.map($scope.localVsaResponse.cluster.nodes, 'name');
                    const nodesSupportInfo = vsaResponse.supportRegistrationInformation;
                    const serialNumbers = _.map(nodesSupportInfo, 'serialNumber');
                    let licenseExpiryDates = _.map(nodesSupportInfo, (supportInfo) => $filter('fullDate')(supportInfo.licenseExpiryDate));
                    if (nodeNames.length === 1) {
                        $scope.registrationDetails.serialNumbers = serialNumbers;
                        $scope.registrationDetails.licenseExpiryDates = licenseExpiryDates;
                    } else if (nodeNames.length > 1) { //Display each node name and its data
                        $scope.registrationDetails.serialNumbers = formatNodeNamesWithData(nodeNames, serialNumbers);
                        $scope.registrationDetails.licenseExpiryDates = formatNodeNamesWithData(nodeNames, licenseExpiryDates);
                    }

                    if ($scope.evalLicense && nodeNames.length > 1) {
                        licenseExpiryDates = [$scope.evalExpiration, $scope.evalExpiration];
                        $scope.formattedEvalExpiration = formatNodeNamesWithData(nodeNames, licenseExpiryDates);
                    } else if ($scope.evalLicense && nodeNames.length === 1) {
                        $scope.formattedEvalExpiration = [$scope.evalExpiration];
                    }
                });
        };

        const formatNodeNamesWithData = (nodeNames, dataForEachNode) => {
            return _.zipWith(nodeNames, dataForEachNode, (name, data) => name + ' - ' + data);
        };

        $scope.showUploadLicenseFileDialog = () => {
            nocDialog.confirm('Update License', 'assets/templates/dialogs/upload-license-file.html', {
                width: 650,
                fromTop: '30%',
                selectedWorkingEnvironment: $scope.selectedWorkingEnvironment,
                mainScreenNocHttp: nocHttp
            })
                .then(() => {
                    $scope.getRegistrationDetails(); //Update registration details
                    pidgeon.inProgress('Uploading a cloud license file started.', true);
                });
        };

        $scope.initialize();
    });
})();
