(() => {
    'use strict';

    angular.module('noc').controller('MonitoringController', ($scope, nocEncryptionResourcesService, $rootScope, nocHttp, Messages, workingEnvironmentResource,
                                                              pidgeon, nocUtil, nocDialog, $stateParams, $state, $log, ManifestMapper,
                                                              nocCloudSyncService, $q, volumesResource,
                                                              nocUI, $filter, MetadataResource) => {
        let statusPollingStopper = _.noop;
        $scope.initializeShutdownSchedules = () => {
            nocHttp.get($scope.selectedWorkingEnvironment._pathPrefix + 'working-environments/' + $scope.selectedWorkingEnvironment.publicId + '/schedules',
                'retrieving schedules',
                null)
                .then((schedules) => {
                    $scope.selectedWorkingEnvironment._originalSchedules = schedules;
                });
        };

        $scope.doesCurrentStateEndWith = (state) => {
            const currentState = $state.current.name;

            return _.endsWith(currentState, state);
        };

        $scope.getObjectStorageName = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            return providerToObjectStorage[$scope.selectedWorkingEnvironment._cloudProviderName];
        };

        $scope.getEnabledSchedules = () => {
            return _.map(_.filter($scope.selectedWorkingEnvironment._originalSchedules, {enabled: true}), 'name');
        };

        $scope.getScheduleDowntimeTitle = () => {
            const scheduledDowntime = $scope.getEnabledSchedules();

            return scheduledDowntime.length > 0 ? 'Scheduled Downtime: ' + scheduledDowntime.join(', ') : 'Schedule Downtime';
        };

        $scope.showEnabledSchedulesIcon = () => {
            return $scope.getEnabledSchedules().length > 0 && $scope.selectedWorkingEnvironment.status.status !== 'failed' && $scope.selectedWorkingEnvironment.status.status !== 'not-found';
        };

        $scope.getGadget = () => {
            return $state.params.gadget;
        };

        $scope.initializeController = function initializeController() {
            if (!$scope.selectedWorkingEnvironment._allowedOperations.resources) {
                $scope.backToGallery();
            }


            $scope.$watch($scope.getGadget, () => {
                $scope.gadget = $scope.getGadget();
            });
            $scope.workingEnvironmentType = $stateParams.type;

            if ($scope.selectedWorkingEnvironment.workingEnvironmentType !== 'ON_PREM') {
                statusPollingStopper = workingEnvironmentResource.startStatusPolling($scope.selectedWorkingEnvironment);
            }

            $scope.$on($scope.selectedWorkingEnvironment.publicId + workingEnvironmentResource.WE_STATUS_GET_EVENT, (event, payload) => {
                workingEnvironmentResource.updateWorkingEnvironmentStatus($scope.selectedWorkingEnvironment, payload);
            });

            $scope.$on($scope.selectedWorkingEnvironment.publicId + workingEnvironmentResource.WE_STATUS_ERROR_EVENT, (event, payload) => {
                if (payload.status === 404) {
                    workingEnvironmentResource.updateWorkingEnvironmentStatus($scope.selectedWorkingEnvironment, 'deleted');
                    pidgeon.success('Working environment successfully deleted.');
                    statusPollingStopper();
                }
            });

            if ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'VSA') {
                $scope.initializeShutdownSchedules();

                $scope.$watch('selectedWorkingEnvironment.haProperties.failoverMode', () => {
                    const hasHaDeploymentModel = _.get($scope.selectedWorkingEnvironment, 'haProperties.failoverMode');
                    $scope.haDeploymentModel = $scope.haDeploymentModel || (hasHaDeploymentModel === 'FloatingIP' ? 'Multiple AZs' : hasHaDeploymentModel === 'PrivateIP' ? 'Single Availability Zone' : '');
                });

                // First initialization
                if ($scope.selectedWorkingEnvironment && !$scope.vsaResponse) {
                    $scope.vsaResponse = workingEnvironmentResource.workingEnvironmentResponseToVsaResponse($scope.selectedWorkingEnvironment);
                }

                $scope.$on(workingEnvironmentResource.VSA_GET_EVENT, (event, payload) => {
                    payload.currentActiveActions = $scope.vsaResponse.activeActions;
                    $scope.vsaResponse = payload.vsaResponse;
                    // In order to keep the old fields that don't exist in the new object (if we asked for other fields in the past)
                    _.assignInWith($scope.selectedWorkingEnvironment, payload.workingEnvironment, nocUtil.dontOverrideObjectWithNull);

                    if ($scope.vsaResponse.vsaEncryptionResponse && $scope.vsaResponse.vsaEncryptionResponse.ontapEncryption) {
                        _.forEach($scope.vsaResponse.vsaEncryptionResponse.keyManagers, (keyManager) => {
                            keyManager._displayStatus = _.startCase(keyManager.status);
                        });
                        $scope.clientCertificate = _.find($scope.vsaResponse.vsaEncryptionResponse.ontapEncryptionCertificates, {'certificateType': 'client'});
                        $scope.serverCertificate = _.find($scope.vsaResponse.vsaEncryptionResponse.ontapEncryptionCertificates, {'certificateType': 'server_ca'});
                        nocEncryptionResourcesService.formatCertificate($scope.clientCertificate);
                        nocEncryptionResourcesService.formatCertificate($scope.serverCertificate);
                    }

                    workingEnvironmentResource.updateWorkingEnvironmentStatus($scope.selectedWorkingEnvironment, payload.vsaResponse.status);
                    $scope.isAllowdToOptimizeNetwork = _.includes(ManifestMapper.getNetworkUtilizationEnabledInstanceTypes($scope.selectedWorkingEnvironment._ontapVersion),
                        $scope.selectedWorkingEnvironment._instanceType);

                    $rootScope.$broadcast(workingEnvironmentResource.VSA_GET_EVENT2, payload);
                });

                $scope.$on(workingEnvironmentResource.VSA_GET_ERROR_EVENT, (event, payload) => {
                    if ($scope.selectedWorkingEnvironment.status.status !== 'deleted' && $scope.selectedWorkingEnvironment.status.status !== 'deleting') {
                        pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving Cloud Volumes ONTAP information for working environment: ' + $scope.selectedWorkingEnvironment.name,
                            payload.error, 'vsaInformation');
                    }
                });

                isAllowedToShowNetworkUtilization($scope.selectedWorkingEnvironment);
            }

            const evalExpirationTimestamp = moment($scope.selectedWorkingEnvironment.ontapClusterProperties.creationTime).add(31, 'days');
            $scope.evalExpiration = $filter('fullDate')(evalExpirationTimestamp);
            $scope.evalDaysLeft = evalExpirationTimestamp.diff(moment(), 'days');

            $scope.monitoringInitialized = true;
        };

        function isAllowedToShowNetworkUtilization(selectedWorkingEnvironment) {
            if (selectedWorkingEnvironment._typeAndProvider === 'VSA_AWS') {
                ManifestMapper.initialize(nocHttp, selectedWorkingEnvironment.isHA ? 'HA_AWS' : 'VSA_AWS').then(() => {
                    $scope.isAllowdToOptimizeNetwork = _.includes(ManifestMapper.getNetworkUtilizationEnabledInstanceTypes(selectedWorkingEnvironment._ontapVersion),
                        selectedWorkingEnvironment._instanceType);
                });
            }
        }

        $scope.isSuspended = () => {
            return !!_.find($scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId], {actionType: 'licenseWasSuspended'});
        };

        $scope.popupSvmNameEditDialog = () => {
            nocDialog.confirm('', 'assets/templates/dialogs/change-svm-name.html', {
                svmName: $scope.selectedWorkingEnvironment.svmName
            }).then((dialogScope) => {
                const selectedWE = $scope.selectedWorkingEnvironment;
                const newName = dialogScope.svmName;
                if (newName === selectedWE.svmName) return;
                nocHttp.put({
                    url: selectedWE._pathPrefix + 'working-environments/' + selectedWE.publicId + '/svm',
                    data: {svmName: newName}
                }, 'editing SVM name', nocHttp.BLOCK)
                    .then(() => {
                        selectedWE.svmName = newName;
                    });
            });
        };

        $scope.tieringName = (tierLevel) => {
            if (tierLevel === 'normal') {
                return 'Standard';
            } else if (tierLevel === 'ia') {
                return 'Standard-Infrequent Access';
            } else if (tierLevel === 'ia-single') {
                return 'One Zone-Infrequent Access';
            } else {
                return 'Intelligent-Tiering';
            }
        };

        $scope.updateSelectedWorkingEnvironment = function updateSelectedWorkingEnvironment(workingEnvironmentId, requiredFields) {
            $scope.selectedWorkingEnvironment = $scope.selectedWorkingEnvironment || _.find($scope.workingEnvironments, {publicId: workingEnvironmentId}) || {};

            // Make sure all required fields are part of the selected working environment. if there are no additional required fields continue as normal
            const foundAllRequiredFields = _.every(requiredFields, function (field) {
                return $scope.selectedWorkingEnvironment[field];
            });
            if (!_.isEmpty($scope.selectedWorkingEnvironment) && $scope.selectedWorkingEnvironment.status.status !== 'unknown' && foundAllRequiredFields) {
                return $q.when();
            }

            // If not found yet:
            const pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.type, $state.params.provider);
            const workingEnvironmentsUnloading = nocUI.loading();
            const providerProperties = $state.params.provider === 'AWS' ? 'awsProperties' : 'providerProperties';
            const moreFields = requiredFields ? (',' + requiredFields.join(',')) : '';

            return workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, workingEnvironmentId, pathPrefix, 'status,activeActions,' + providerProperties + ',encryptionProperties,ontapClusterProperties.fields(upgradeVersions),actionsRequired,capacityFeatures' + moreFields)
                .then((updatedWE) => {
                    _.assignInWith($scope.selectedWorkingEnvironment, updatedWE, nocUtil.dontOverrideObjectWithNull);
                })
                .finally(workingEnvironmentsUnloading);
        };

        const properties = $stateParams.type === "HA" ? ['svmName', 'cloudSyncProperties', 'haProperties', 'k8sProperties'] : ['svmName', 'cloudSyncProperties', 'k8sProperties'];
        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId, properties)
            .then($scope.initializeController, $scope.backToGallery);

        $scope.vsaComputingCapacity = false;

        $scope.setComputingCapacity = (state) => {
            $scope.vsaComputingCapacity = state;
        };

        $scope.setVsaPopoverMode = (mode) => {
            $scope.vsaPopoverMode = mode;
        };

        $scope.gotoUpgradeWorkingEnvironment = () => {
            $scope.$parent.gotoUpgradeWorkingEnvironment($scope.selectedWorkingEnvironment, () => {
                $scope.goToGadget('supportRegistration');
            });
        };

        $scope.gotoUpgradeMediator = () => {
            $scope.$parent.gotoUpgradeMediator($scope.selectedWorkingEnvironment);
        };

        $scope.updateExtraProperties = (workingEnvironment) => {
            if (!$scope.selectedWorkingEnvironment._allowedOperations['sync-to-s3']) return $q.when();

            return workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, workingEnvironment.publicId, workingEnvironment._pathPrefix, 'cloudSyncProperties,k8sProperties')
                .then((workingEnvironmentResponse) => {
                    $scope.selectedWorkingEnvironment.cloudSyncProperties = workingEnvironmentResponse.cloudSyncProperties;
                    $scope.selectedWorkingEnvironment.k8sProperties = workingEnvironmentResponse.k8sProperties;
                });
        };

        $scope.$on(volumesResource.VOLUMES_GET_EVENT, (event, payload) => {
            pidgeon.clear('volumes');

            if (payload.workingEnvironment.publicId !== $scope.selectedWorkingEnvironment.publicId) return;

            // Due to architecture constraints in the server, the info about volumes synced to S3 is returned on
            // get_working_environment API. Therefore we call it here once again with appropriate fields if needed.
            $scope.updateExtraProperties($scope.selectedWorkingEnvironment)
                .then(() => {
                    $scope.volumes = volumesResource.initVolumesForMonitoringDisplay($scope.selectedWorkingEnvironment.cloudSyncProperties, payload.volumes, $scope.selectedWorkingEnvironment._cloudProviderName);
                    $scope.filteredVolumes = nocUtil.filterVolumes($scope.volumes, $scope.search);
                    payload.unloading();
                })
                .catch($log.log);
        });

        $scope.$on(volumesResource.VOLUMES_GET_ERROR_EVENT, (event, payload) => {
            if (payload.workingEnvironment.publicId !== $scope.selectedWorkingEnvironment.publicId) return;

            const error = payload.error;
            const prefix = pidgeon.serverErrorPrefix + 'retrieving volumes for working environment ' + $scope.selectedWorkingEnvironment.name;

            // Workaround for NOC-9848, NOC-10420 and others.
            // error message that start with the following text is most probably caused by synchronization problem in client:
            // Client thinks the working environment status is ON or DEGRADED but actually it's something else in database.
            // The best workaround we could think of is not to display the error. By the time the volumes poller polls again
            // the status will be updated by status poller and everything will be OK.
            if (_.startsWith(error.message, 'Operation may only be performed on working environments whose status is')) {
                $log.error(prefix + ': ' + error.message + '. This error was not reported to user.');
            } else {
                pidgeon.error(prefix, error, 'volumes');
            }
        });

        $scope.launchSystemManager = function launchSystemManager() {
            const systemManagerUrl = $scope.selectedWorkingEnvironment.ontapClusterProperties.systemManagerUrl;

            if (systemManagerUrl) {
                nocDialog.dialog('Launch System Manager', 'assets/templates/dialogs/launch-system-manager-content.html', {
                    launchUrl: systemManagerUrl,
                    showWarning: $scope.selectedWorkingEnvironment.workingEnvironmentType
                });
            } else {
                nocDialog.confirm('Unable to launch System Manager, ONTAP version must be above 8.3');
            }
        };

        $scope.deleteVolume = (volume) => {
            nocDialog
                .confirm(Messages.deleteConfirm({type: 'volume', name: volume.name}))
                .then(() => {
                    nocHttp.delete($scope.selectedWorkingEnvironment._pathPrefix + 'volumes/' + $scope.selectedWorkingEnvironment.publicId + '/' + volume.svmName + '/' + volume.name,
                        'deleting volume',
                        nocHttp.BLOCK);
                });
        };

        $scope.goToMountVolume = function goToMountVolume(volume) {
            $scope.goToGadget('mountVolume', volume);
        };

        $scope.goToChangeTier = function goToChangeTier(volume) {
            $scope.goToGadget('changeTier', volume);
        };

        $scope.goToRestoreVolume = function goToRestoreVolume(volume) {
            volumesResource.getSnapshots(nocHttp, $scope.selectedWorkingEnvironment._pathPrefix, $scope.selectedWorkingEnvironment.publicId, volume.svmName, volume.name)
                .then((snapshots) => {
                    if (!snapshots.length) {
                        nocDialog.alert(Messages.noSnapshotsAlert({volume: volume.name}));
                    } else {
                        $scope.goToGadget('restoreVolume', volume);
                    }
                });
        };

        $scope.goToCreateASnapshot = function goToCreateASnapshot(volume) {
            $scope.goToGadget('snapshotVolume', volume);
        };

        $scope.goToEditVolume = function goToEditVolume(volume) {
            $scope.goToGadget('editVolume', volume);
        };

        $scope.syncVolumeToS3 = function syncVolumeToS3(volume) {
            return nocCloudSyncService.syncVolume(nocHttp, $scope.selectedWorkingEnvironment.publicId, volume.name, $state.params.type);
        };

        $scope.deleteSyncVolumeToS3 = function deleteSyncVolumeToS3(volume) {
            return nocCloudSyncService.deleteSyncVolume(nocHttp, $scope.selectedWorkingEnvironment.publicId, volume.name, $state.params.type);
        };

        $scope.goToActionRequired = () => {
            $scope.lastGadget = $scope.gadget;
            $scope.goToGadget('actionRequired');
        };

        $scope.boxGoBack = () => {
            if ($scope.lastGadget) {
                $scope.goToGadget($scope.lastGadget);
                delete $scope.lastGadget;
            } else {
                $scope.showMonitoring();
            }
        };

        $scope.calcUsedSpace = function calcUsedSpace(volumes) {
            return nocUtil.sumCapacityUsingField(volumes, 'usedSize');
        };

        $scope.calcTotalCapacityTierUsedSize = function calcTotalCapacityTierUsedSize(volumes) {
            return nocUtil.sumCapacityUsingField(volumes, 'capacityTierUsedSize');
        };

        $scope.calcTotalUsedSpace = function calcTotalUsedSpace(volumes) {
            const usedSizeBytes = nocUtil.convertTo(nocUtil.sumCapacityUsingField(volumes, 'usedSize'), 'Byte');
            const capacityTierUsedSizeBytes = nocUtil.convertTo(nocUtil.sumCapacityUsingField(volumes, 'capacityTierUsedSize'), 'Byte');

            return nocUtil.normalizeCapacity({size: usedSizeBytes.size + capacityTierUsedSizeBytes.size, unit: 'Byte'});
        };

        $scope.calcTotalAllocatedSpace = function calcTotalAllocatedSpace(volumes) {
            return nocUtil.sumCapacityUsingField(volumes, 'size');
        };

        $scope.getUsedDetails = () => {
            const normalizeCapacityFilter = $filter('normalizedCapacity');
            let usedSizeNormalized = normalizeCapacityFilter($scope.calcUsedSpace($scope.filteredVolumes), 2, true);
            let tierUsedSizeNormalized = normalizeCapacityFilter($scope.calcTotalCapacityTierUsedSize($scope.filteredVolumes), 2, true);
            usedSizeNormalized = (usedSizeNormalized === '0 Byte') ? '0 KB' : usedSizeNormalized;
            tierUsedSizeNormalized = (tierUsedSizeNormalized === '0 Byte') ? '0 KB' : tierUsedSizeNormalized;

            return `(${usedSizeNormalized} in ${$scope.selectedWorkingEnvironment._isAWS ? 'EBS' : 'Disk'}, ${tierUsedSizeNormalized} in ${$scope.selectedWorkingEnvironment._isAWS ? 'S3' : 'Blob'})`;
        };

        $scope.buildDataForInstanceInformationDialog = (instance) => {
            const data = [];
            data.push({key: 'Name:', text: instance.name, 'trimmedText': true, title: instance.name});
            data.push({key: 'Instance Type:', text: instance.instanceType});
            data.push({key: 'State:', text: instance.state});
            data.push({key: 'Public Ip Address:', text: instance.publicIpAddress ? instance.publicIpAddress : 'None'});
            data.push({
                key: 'Public Dns Name:',
                text: instance.publicDnsName ? instance.publicDnsName : 'None',
                trimmedText: true
            });
            data.push({key: 'Private Dns Name:', text: instance.privateDnsName, trimmedText: true});
            data.push({key: 'Image Id:', text: instance.imageId});

            return MetadataResource.getInstanceProfiles(nocHttp, $scope.selectedWorkingEnvironment._pathPrefix,
                _.get($scope, 'selectedWorkingEnvironment.awsProperties.cloudProviderAccountId'))
                .then((instanceProfiles) => {
                    const instanceProfile = _.find(instanceProfiles, {'instanceProfileId': instance.instanceProfileId});
                    if (instanceProfile) {
                        data.push({
                            key: 'IAM Role:',
                            text: instanceProfile.name,
                            trimmedText: true
                        });
                    }

                    return data;
                });
        };

        $scope.showInstanceInfoBox = (event, instance) => {
            $scope.buildDataForInstanceInformationDialog(instance)
                .then((list) => {
                    const lists = nocUtil.splitDataInto2Lists(list);

                    nocDialog.dialog(instance.privateIpAddress + ' (' + instance.id + ')',
                        'assets/templates/dialogs/general-info-dialog.html', {
                           width: 904,
                           lists
                        })
                        .then(() => {
                            event.stopPropagation();
                        });
                });
        };

        $scope.openAwsEncryptionPopover = () => {
            $scope.popoverHelper.awsEncryptionKey = $scope.selectedWorkingEnvironment._awsEncryptionKey;
        };

        $scope.instancesFilter = (instance) => {
            //Filtering by:
            //  * instance name
            //  * instance ip
            return nocUtil.genericItemFilter($scope.search, instance, ['name', 'privateIpAddress']);
        };

        $scope.aggregatesFilter = (aggr) => {
            //Filtering by:
            //  * instance name
            //  * displayed capacity
            return nocUtil.genericItemFilter($scope.search, aggr, ['name', '_displayCapacity']);
        };

        $scope.keyManagersFilter = (keyManager) => {
            //Filtering by:
            //  * IP address
            //  * displayed status
            return nocUtil.genericItemFilter($scope.search, keyManager, ['address', '_displayStatus']);
        };

        $scope.$on('$destroy', () => {
            statusPollingStopper();
        });

        $scope.updateCredentials = (we) => {
            nocDialog.dialog('Update ' + we._typeLongName + ' Working Environment Password',
                'assets/templates/tabs/working-environments/update-credentials.html',
                {workingEnvironment: we});
        };

        $scope.showUpdateEvalLicenseDialog = () => {
            nocDialog.confirm('', 'assets/templates/dialogs/update-eval-license.html', {
                width: 650,
                fromTop: '25%',
                selectedWorkingEnvironment: $scope.selectedWorkingEnvironment,
                evalExpiration: $scope.evalExpiration,
                evalDaysLeft: $scope.evalDaysLeft,
                mainScreenNocHttp: nocHttp
            })
                .then(() => {
                    $scope.showMonitoring();
                });
        };
    });
})();
