(() => {
    'use strict';
    angular.module('noc').controller('MonitoringSVMOffController', ($scope, nocHttp, pidgeon, workingEnvironmentResource, nocUtil) => {
        let lastStatus;

        $scope.getActionsRequired = () => {
            nocHttp.get($scope.selectedWorkingEnvironment._pathPrefix + 'working-environments/' + $scope.selectedWorkingEnvironment.publicId + '?fields=actionsRequired',
                'retrieving required actions for working environment ' + $scope.selectedWorkingEnvironment.name,
                nocHttp.LOADING)
                .then((workingEnvironment) => {
                    const map = {};
                    map[$scope.selectedWorkingEnvironment.publicId] = workingEnvironment.actionsRequired;
                    $scope.setActionRequiredFor($scope.selectedWorkingEnvironment.publicId, false, map);
                });
        };

        $scope.shouldShowTurnOnAction = () => {
            return workingEnvironmentResource.shouldShowTurnOnAction($scope);
        };

        $scope.vsaPollingStopper = _.noop;

        $scope.initialize = function initialize() {
            lastStatus = {};
            $scope.stateInfoSubTitle = $scope.selectedWorkingEnvironment.isHA && $scope.selectedWorkingEnvironment._stateInfo.haSubTitle ?
                $scope.selectedWorkingEnvironment._stateInfo.haSubTitle : $scope.selectedWorkingEnvironment._stateInfo.subTitle;

            if ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'VSA') {
                pidgeon.clear('vsaInformation');

                $scope.$on(workingEnvironmentResource.VSA_GET_EVENT2, (event, payload) => {
                    $scope.offVsaResponse = payload.vsaResponse;
                    //In order to keep the old fields that don't exist in the new object (if we asked for other fields in the past)
                    _.assignInWith($scope.selectedWorkingEnvironment, payload.workingEnvironment, nocUtil.dontOverrideObjectWithNull);
                });

                $scope.$on(workingEnvironmentResource.VSA_GET_ERROR_EVENT, (event, payload) => {
                    if ($scope.selectedWorkingEnvironment.status.status === 'deleted') {
                        $scope.vsaPollingStopper();
                    } else if ($scope.selectedWorkingEnvironment.status.status !== 'deleting') {
                        pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving Cloud Volumes ONTAP information for working environment ' + $scope.selectedWorkingEnvironment.name, payload.error, 'vsaInformation');
                    }
                });

                $scope.vsaPollingStopper = workingEnvironmentResource.startVsaPolling($scope.selectedWorkingEnvironment);

                if ($scope.selectedWorkingEnvironment.status.status === 'off') {
                    $scope.getActionsRequired();
                }
            }
        };

        $scope.$on(workingEnvironmentResource.WORKING_ENVIRONMENT_STATUS_UPDATE, (event, payload) => {
            if (lastStatus.status !== payload.workingEnvironment.status.status) {
                if (_.get(lastStatus, 'status.status') !== 'off' && payload.workingEnvironment.status.status === 'off') {
                    $scope.getActionsRequired();
                }

                lastStatus = payload.workingEnvironment.status;
            }
        });

        $scope.$on('$destroy', () => {
            $scope.vsaPollingStopper();
        });

        if ($scope.monitoringInitialized) {
            $scope.initialize();
        } else {
            $scope.$watch('monitoringInitialized', () => {
                if ($scope.monitoringInitialized) {
                    $scope.initialize();
                }
            });
        }
    });
})();
