(function () {
    "use strict";
    angular.module("noc").controller("UpdateCredentialsController", function ($scope, nocUI, workingEnvironmentResource, nocPoller, Patterns, nocHttp) {
        $scope.initialize = function initialize() {
            $scope.updateCredentials = {username: "admin"};
            $scope.workingEnvironment = $scope.workingEnvironment || $scope.selectedWorkingEnvironment;

            $scope.passwordRegexp = $scope.workingEnvironment.workingEnvironmentType === 'VSA' ?
                new RegExp(Patterns['vsaSvmPassword'].regex) :
                new RegExp(Patterns['string255'].regex);
        };

        $scope.isValid = function () {
            return $scope.updateCredentialsForm.$valid;
        };

        $scope.callRightAPI = function(updatedCredentials) {
            if ($scope.workingEnvironment.workingEnvironmentType === 'VSA') {
                return workingEnvironmentResource.setPassword(nocHttp, $scope.workingEnvironment, updatedCredentials.password, true);
            } else { //ON_PREM
                return workingEnvironmentResource.updateCredentials(nocHttp, $scope.workingEnvironment, updatedCredentials);
            }
        };

        $scope.saveUpdatedCredentials = function (updatedCredentials) {
            if (!$scope.isValid()) {
                nocUI.handleFormValidity($scope.updateCredentialsForm, $scope);
            } else {
                nocPoller.pause();
                $scope.callRightAPI(updatedCredentials)
                    .then(function () {
                        //pushes the success to the dataFabric controller. If used here regular "pidgeon.success" the message would appear on the dialog and disappear with it instantly as it closes.
                        $scope.$parent.$parent.pushSuccess("Successfully updated credentials for " + $scope.workingEnvironment._typeLongName + " working environment " + $scope.workingEnvironment.name + ".");
                        $scope.closeDialog();
                    })
                    .finally(nocPoller.unpause);
            }
        };

        $scope.$on("$destroy", function () {
            nocUI.clearFormValidity();
        });

        $scope.initialize();
    });
})();
