(() => {
    'use strict';

    angular.module('noc').controller('VirginChooseWizardController', ($scope, $rootScope, $state, BaseVirginStep, nocHttp,
                                                                      pidgeon, ManifestMapper, $timeout, nocUtil,
                                                                      proxyPopupService, CloudProviderAccountResource) => {
        $scope.initialize = () => {
            $scope.wizardController.failedRequestType = $state.params.type;
            if ($scope.currentStateName === 'init' && !$scope.wizardController.failedRequestType) {
                $scope.wizardController.next('.choose');
            }
            BaseVirginStep.call($scope);
            $scope.wizardController.clearAllStepData();
            $scope.tab = $state.params.startDiscoverWizard ? 'discover' : 'create';
            $scope.wizardController.workingEnvironmentTypeAndProvider = {};

            $scope.azureIsSupported = !$rootScope.runningInIC;

            if ($scope.wizardController.failedRequestType) {
                $scope.wizardController.recreateId = $state.params.id;
                $scope.getCreateRequestParameters();
            }

            CloudProviderAccountResource.getAccounts(nocHttp)
                .then((accounts) => {
                    $scope.wizardController.providerAccounts = accounts;
                });
            $scope.selectProvider();
            $scope.wizardController.selectedType = 'VSA';
        };

        $scope.selectProvider = (providerName) => {
            $scope.wizardController.selectedProvider = providerName || $rootScope.instancePlacement.installLocation;
            if ($scope.wizardController.selectedProvider === 'GCP') {
                $scope.wizardController.selectedType = 'VSA';
            }
        };

        $scope.selectType = (type) => {
            $scope.wizardController.selectedType = type;
        };

        /*eslint-disable key-spacing, no-multi-spaces*/
        $scope.chosenToWETypeAndProviderName = {
            'VSA_AWS': {type: 'VSA', providerName: 'AWS', combined: 'VSA_AWS'},
            'HA_AWS': {type: 'HA', providerName: 'AWS', combined: 'HA_AWS'},
            'VSA_AZURE': {type: 'VSA', providerName: 'Azure', combined: 'VSA_Azure'},
            'HA_AZURE': {type: 'HA', providerName: 'Azure', combined: 'HA_Azure'},
            'VSA_GCP': {type: 'VSA', providerName: 'GCP', combined: 'VSA_GCP'},
            'Recover': {type: 'VSA', providerName: 'AWS', combined: 'VSA_AWS'},
            'RecoverAzure': {type: 'VSA', providerName: 'Azure', combined: 'VSA_Azure'},
            'RecoverGCP': {type: 'VSA', providerName: 'GCP', combined: 'VSA_GCP'},
            'OnPrem': {type: 'ON_PREM', providerName: '', combined: 'ON_PREM'},
            'SoftLayer': {type: 'SOFTL', providerName: 'IBM', combined: 'IBM'}
        };
        /*eslint-enable key-spacing, no-multi-spaces*/

        $scope.setTab = (tabName) => {
            if ($scope.tab !== tabName) {
                $state.go('wizard.init.choose');
            }
            $scope.tab = tabName;
        };

        function noAccountsExist(accounts) {
            if (!accounts) return;
            const needAwsCredentials = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS' &&
                accounts.awsAccounts.length === 0;

            const needAzureCredentials = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure' &&
                accounts.azureAccounts.length === 0;

            return needAwsCredentials || needAzureCredentials;
        }

        $scope.startWizard = (choice) => {
            if (choice === 'Recover-CVO') {
                return $state.go('wizard.init.choose.cvo-discover');
            } else {
                const supportedChoices = ['VSA_AWS', 'VSA_AZURE', 'VSA_GCP', 'Recover', 'RecoverAzure', 'OnPrem', 'HA_AWS', 'HA_AZURE', 'SoftLayer', 'RecoverGCP'];
                if (!_.includes(supportedChoices, choice)) throw new Error('Wizard \'' + choice + '\' is not supported. Should be one of ' + supportedChoices.join(', '));

                $scope.wizardController.workingEnvironmentTypeAndProvider = $scope.chosenToWETypeAndProviderName[choice];
                $scope.wizardController.workingEnvironmentType = $scope.wizardController.workingEnvironmentTypeAndProvider.type;

                $scope.wizardController.pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix(
                    $scope.wizardController.workingEnvironmentTypeAndProvider.type,
                    $scope.wizardController.workingEnvironmentTypeAndProvider.providerName);

                if (choice === 'VSA_AWS' || choice === 'HA_AWS' || choice === 'VSA_AZURE' || choice === 'HA_AZURE' || choice === 'VSA_GCP') {
                    $scope.wizardController.title = $scope.wizardController.failedRequestType ? 'Re-create ' + $scope.wizardController.failedRequest.name + ' Working Environment' : 'Create a New Working Environment';
                    $scope.wizardController.isAzureHa = choice === 'HA_AZURE';

                    pidgeon.clear('manifest');

                    return ManifestMapper.initialize(nocHttp, choice)
                    // If manifest initialized successfully - go to next step
                        .then(() => {
                            if ($scope.wizardController.failedRequestType) {
                                return $state.go('wizard.init.create-detailsCredentials');
                            } else if (noAccountsExist($scope.wizardController.providerAccounts)) {
                                return $state.go('wizard.init.choose.no-accounts');
                            } else {
                                return $state.go('wizard.init.choose.create-detailsCredentials');
                            }
                        })
                        // Else - show error message from manifest initialization
                        .catch((error) => {
                            const connectionFailure = ' Verify that the Cloud Manager host has access to the internet';
                            if (_.includes(error.causeMessage, connectionFailure)) {
                                proxyPopupService.proxyFail('Failed to retrieve metadata file global.manifest.', 'Unable to contact server.');
                            } else {
                                pidgeon.error('An error occurred while retrieving manifests. Cannot create a new Cloud Volumes ONTAP. You can still discover an existing working environment', null, 'manifest');
                            }
                        });
                } else if (noAccountsExist($scope.wizardController.providerAccounts)) {
                    return $state.go('wizard.init.choose.cvo-discover.no-accounts');
                } else {
                    return $state.go('wizard.init.choose.' + (choice === 'Recover' || choice === 'RecoverAzure' || choice === 'RecoverGCP' ? 'cvo-discover.region' : 'cluster'));
                }
            }
        };

        $scope.startWizardWrapper = (choice) => {
            return $scope.startWizard(choice)
                .then(() => {
                    //If starting a new wizard we must clear all data, otherwise a wrong vpc can be chosen for the newFas or VSA
                    //the cleaning action should happen only after the child scopes gets destroyed, after startWizard() calls $state.go();
                    if ($scope.wizardController.choice !== choice) {
                        $timeout($scope.clearAllStepData);
                    }
                    $scope.wizardController.choice = choice;
                });
        };

        $scope.getCreateRequestParameters = function getCreateRequestParameters() {
            const pathPrefix = $state.params.pathPrefix;

            return nocHttp.get(pathPrefix + 'working-environments/' + $scope.wizardController.recreateId + '/create-request-parameters',
                'retrieving parameters used in create working environment request', nocHttp.LOADING)
                .then((response) => {
                    $scope.wizardController.failedRequest = response.parameters;
                    $scope.startWizardWrapper($scope.wizardController.failedRequestType);
                });
        };

        $scope.initialize();
    });
})();
