(() => {
    'use strict';

    angular.module('noc').controller('CloudOntapUpgradeNSSStepController', ($scope, $state, nocHttp, workingEnvironmentResource, BaseVirginStep, nocUtil,
                                                                            nocUI, CloudProviderAccountResource, nocDialog) => {
        BaseVirginStep.call($scope);

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/nss-learn-more.html', {
                width: 890,
                fromTop: '20%'
            });
        };

        $scope.nextNss = (versionOrEula) => {
            if ($scope.isValid()) {
                callRegisterWithNextStep($scope.workingEnvironment, versionOrEula);
            } else {
                nocUI.handleErrorsInWizard(true);
            }
        };

        const registerWithNssAccount = (nextStep) => {
            if ($scope.wizardController.isNssRegistered) {
                return $state.go(nextStep);
            }
            $scope.errorMessage = '';
            $scope.nameErrorMessage = '';
            const vsaList = [$scope.wizardController.workingEnvironment.publicId];
            if ($scope.wizardController.existingNssAccounts.length === 0) {
                CloudProviderAccountResource.createAccount(nocHttp, 'nss', $scope.wizardController.nssAccountRequest.accountName, $scope.wizardController.nssAccountRequest.providerKeys, vsaList, true)
                    .then((account) => {
                        $scope.wizardController.isNssRegistered = true;
                        $scope.wizardController.existingNssAccounts = [account];
                        $scope.wizardController.selectedNssAccount = account;
                        $state.go(nextStep);
                    })
                    .catch((error) => {
                        if (_.endsWith(error.message, 'Account name is already in use.')) {
                            $scope.nameErrorMessage = 'Account name already exists';
                        } else {
                            $scope.errorMessage = error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length);
                        }
                    });
            } else {
                const nssRegisterUrl = $scope.wizardController.workingEnvironment._pathPrefix + 'working-environments/' + $scope.wizardController.workingEnvironment.publicId +
                    '/support-registration?nssAccountId=' + $scope.wizardController.selectedNssAccount.publicId;
                nocHttp.post(nssRegisterUrl, '', nocHttp.BLOCK)
                    .then(() => {
                        $scope.wizardController.isNssRegistered = true;
                        $state.go(nextStep);
                    })
                    .catch((error) => {
                        $scope.errorMessage = error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length);
                    });
            }
        };

        const callRegisterWithNextStep = (workingEnvironment, versionOrEula) => {
            if (workingEnvironment.isHA) {
                workingEnvironmentResource.checkIfCifsLockExists(nocHttp, workingEnvironment)
                    .then((isCifsLockExists) => {
                        const nextStep = isCifsLockExists ? '.cifs' : versionOrEula;
                        registerWithNssAccount(nextStep);
                    });
            } else {
                registerWithNssAccount(versionOrEula);
            }
        };

        $scope.goToAccountsSettings = () => {
            $scope.checkAndSetState('cloud-provider-accounts.account-settings');
        };

        $scope.initialize = () => {
            const pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.type, $state.params.provider);
            workingEnvironmentResource.getVSAByPublicId(nocHttp, $state.params.workingEnvironmentId, pathPrefix, _.includes(pathPrefix, '/ha'))
                .then((workingEnvironment) => {
                    $scope.wizardController.versionsAvailable = workingEnvironment.ontapClusterProperties.upgradeVersions;
                    $scope.wizardController.workingEnvironment = workingEnvironment;
                    $scope.wizardController.workingEnvironment._pathPrefix = pathPrefix;
                    $scope.wizardController.selectedVersion = $scope.wizardController.versionsAvailable[0];
                    $scope.currentVersion = workingEnvironment.ontapClusterProperties.ontapVersion;
                    $scope.hasMediatorUpgradeOption = _.get(workingEnvironment.haProperties, 'mediatorVersionsToUpdate') && workingEnvironment.haProperties.mediatorVersionsToUpdate.length > 0;
                });
            CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                .then((accounts) => {
                    $scope.wizardController.existingNssAccounts = _.filter(accounts, {'_provider': 'nss'});
                    if ($scope.wizardController.existingNssAccounts.length > 0) {
                        $scope.wizardController.selectedNssAccount = $scope.wizardController.existingNssAccounts[0];
                    }
                });
        };

        $scope.initialize();
    });
})();
