(() => {
    'use strict';

    angular.module('noc').controller('CreateNewAggregateDetailsStepController', ($scope, $state, nocHttp, nocUI,
                                                                                 nocUtil, workingEnvironmentResource,
                                                                                 $filter, nocAdvancedAllocationService) => {
        $scope.initializeController = function initializeController() {
            $scope.wizardController.title += ' in ' + $scope.wizardController.selectedWorkingEnvironment.name;
            $scope.wizardController.addAggregateRequest = {
                workingEnvironmentId: $scope.selectedWorkingEnvironment.publicId
            };

            // If WE is HA with a valid nodes array - enable home node selection with first node as default
            if ($scope.wizardController.selectedWorkingEnvironment.isHA) {
                $scope.wizardController.addAggregateRequest.homeNode = $scope.wizardController.selectedWorkingEnvironment.ontapClusterProperties.nodes[0].name;
            }

            // If WE is Azure HA, select tier in this step because only Premium disk exists
            if ($scope.wizardController.selectedWorkingEnvironment.isHA && $scope.wizardController.selectedWorkingEnvironment._isAzure && !$scope.wizardController.selectedTier) {
                $scope.possibleTiers = nocAdvancedAllocationService.getPossibleTiers($scope.wizardController.selectedWorkingEnvironment);
                const defaultDiskType = $scope.wizardController.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.diskType;
                tierSelected($scope.possibleTiers[defaultDiskType]);
            }

            function tierSelected(tier) {
                $scope.wizardController.selectedTier = tier;
                const supportedSizes = $scope.wizardController.selectedTier.supportedSizes;
                const defaultProviderVolumeSize = $scope.wizardController.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.size;
                const defaultDiskSize = _.find(supportedSizes, {_identifier: $filter('normalizedCapacity')(defaultProviderVolumeSize, 2, true)});
                $scope.wizardController.providerVolumeSize = defaultDiskSize || supportedSizes[0];
            }
        };

        $scope.getSelectedWorkingEnvironment = function getSelectedWorkingEnvironment(workingEnvironmentId) {
            const pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.type, $state.params.provider);
            const workingEnvironmentsUnloading = nocUI.loading();
            const fields = 'ontapClusterProperties,capacityFeatures';

            return workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, workingEnvironmentId, pathPrefix,
                fields)
                .then((selectedWe) => {
                    $scope.wizardController.selectedWorkingEnvironment = selectedWe;
                })
                .finally(workingEnvironmentsUnloading);
        };

        $scope.getSelectedWorkingEnvironment($state.params.workingEnvironmentId)
            .then($scope.initializeController);
    });
})();
