(() => {
    'use strict';

    angular.module('noc').controller('PortalSetupWizardController', ($scope, nocUI, nocHttp, $rootScope, $http, $window, OCCMResource,
                                                                     nocAuth0Service, PortalSetupResource, nocRoutingResolverService,
                                                                     ServerUpgradeService, systemResource, intercomNotifier, tenancyResource) => {
        function loginAndGoToPortalSetupWizard(clientId) {
            nocAuth0Service.portalUserAuthentication(clientId)
                .then((authResult) => {
                    tenancyResource.reloadAccounts()
                        .then(() => {
                            const userMetadata = nocAuth0Service.extractUserMetadata(authResult);

                            const nameSepIdx = userMetadata.name.indexOf(' ');
                            const firstName = nameSepIdx === -1 ? userMetadata.name : userMetadata.name.substr(0, nameSepIdx);
                            $scope.portalSetup = {firstName, email: userMetadata.email};
                            $scope.setupRequest = {adminUser: {email: userMetadata.email}, company: userMetadata.company};
                            $scope.wizardStep = 'sign-up';
                            $scope.ic = $rootScope.instancePlacement.ic;
                        });
                })
                .catch(() => {
                    $rootScope.appState = 'failed-reach-auth0';
                    nocRoutingResolverService.resolveEntryPoint();
                });
        }

        $scope.finishPortalSetup = (request) => {
            const unloading = nocUI.loading();
            $scope.setupRequest.site = request.site;
            $scope.setupRequest.tenancyAccountId = request.accountId;
            PortalSetupResource.finish($scope.setupRequest)
                .then((response) => {
                    if (response.data.upgradeToVersion) {
                        //Doing auto upgrade, display nice loader while upgrading.
                        $scope.upgradeToVersion = response.data.upgradeToVersion;
                        $scope.wizardStep = 'auto-upgrade';
                        const callbackAfterUpgrade = () => {
                            nocUI.reloadSilently(true);
                        };

                        return ServerUpgradeService.pollForServerUpgrade(callbackAfterUpgrade);
                    } else {
                        //No need for auto upgrade, reload, app should work.
                        nocUI.reloadSilently(true);
                    }
                })
                .finally(unloading);
        };

        $scope.initialize = () => {
            $rootScope.externalConfig = {
                'intercom_prod_app_id': $rootScope.supportServices.intercomAppId
            };
            intercomNotifier.boot();
            $scope.setupRequest = {};
            $scope.wizardStep = 'proxy';
            $scope.detectingConnectivity = true;
            OCCMResource.validatePortalConnectivity(nocHttp)
                .then($scope.registerAndSetup)
                .catch(() => {
                    $scope.detectingConnectivity = false;
                });
        };


        $scope.refreshSetup = () => {
            $window.location.href = _($window.location.href)
                .replace(/#.*/, '')
                .replace(/\?.*/, '');
        };

        $scope.registerAndSetup = () => {
            $http.post('occm/setup-portal/register')
                .then((response) => {
                    loginAndGoToPortalSetupWizard(response.data.clientId);
                })
                .catch((error) => {
                    switch (error.status) {
                        case 409: //OCCM is already registered
                            loginAndGoToPortalSetupWizard($rootScope.supportServices.portalService.auth0Information.clientId);
                            break;
                        default: //No connection to portal
                            //removePortalWidget();
                            $rootScope.appState = 'setup-server-registration-error';
                            nocRoutingResolverService.resolveEntryPoint();
                    }
                });
        };

        $scope.saveProxyPortalSetup = (uri, username, password, domain) => {
            $scope.detectingConnectivity = true;
            const proxy = {
                uri,
                proxyUserName: username,
                proxyPassword: password,
                proxyDomain: domain
            };
            PortalSetupResource.savePortalProxyDetails(proxy)
                .then(() => {
                    //Need to reload supported services in order to get all the portal data
                    return systemResource().getSupportServices()
                        .then(() => {
                            $scope.proxyServerErrorMessage = null;
                            nocUI.clearFormValidity();

                            return $scope.registerAndSetup();
                        });
                })
                .catch((error) => {
                    $scope.proxyServerErrorMessage = error;
                })
                .finally(() => {
                    $scope.detectingConnectivity = false;
                });
        };

        $scope.initialize();
    });
})();
