(() => {
    'use strict';
    /* istanbul ignore next */
    angular.module('noc').controller('ChooseInstanceRoleStepController', ($scope, $rootScope, nocHttp, nocUI, MetadataResource, pidgeon) => {
        $scope.initialize = () => {
            $scope.selectedInstanceProfiles = {};

            $scope.instanceProfileDefaultRole = {
                name: 'Let Cloud Manager create the role (recommended)',
                instanceProfileId: null
            };

            MetadataResource.getInstanceProfiles(nocHttp, $scope.wizardController.pathPrefix, $scope.wizardController.cloudProviderAccountId)
                .then((instanceProfiles) => {
                    $scope.instanceProfiles = instanceProfiles;

                    if (!$rootScope.runningInIC) {
                        $scope.instanceProfiles.unshift($scope.instanceProfileDefaultRole);
                        $scope.selectedInstanceProfiles.selectedInstanceProfileNode = $scope.wizardController.instanceProfileName ? {name: $scope.wizardController.instanceProfileName} : $scope.instanceProfiles[0];
                        if ($scope.wizardController.choice === 'HA_AWS') {
                            $scope.selectedInstanceProfiles.selectedInstanceProfilesMediator = $scope.wizardController.haParameters.mediatorInstanceProfileName ? {name: $scope.wizardController.haParameters.mediatorInstanceProfileName} : $scope.instanceProfiles[0];
                        }
                    } else {
                        if (_.isEmpty(instanceProfiles)) {
                            pidgeon.error('Your AWS account does not contain any IAM roles. You must create at least one role to continue.');
                        }
                        $scope.selectedInstanceProfiles.selectedInstanceProfileNode = $scope.wizardController.instanceProfileName ? {name: $scope.wizardController.instanceProfileName} : null;
                        if ($scope.wizardController.choice === 'HA_AWS') {
                            $scope.selectedInstanceProfiles.selectedInstanceProfilesMediator = $scope.wizardController.haParameters.mediatorInstanceProfileName ? {name: $scope.wizardController.haParameters.mediatorInstanceProfileName} : null;
                        }
                    }

                    if ($scope.wizardController.failedRequestType) {
                        $scope.selectedInstanceProfiles.selectedInstanceProfileNode = {
                            name: $scope.wizardController.failedRequest.instanceProfileName || $scope.selectedInstanceProfiles.selectedInstanceProfileNode.name
                        };
                        if ($scope.wizardController.choice === 'HA_AWS') {
                            $scope.selectedInstanceProfiles.selectedInstanceProfilesMediator = {
                                name: $scope.wizardController.failedRequest.haParams.mediatorInstanceProfileName || $scope.selectedInstanceProfiles.selectedInstanceProfilesMediator.name
                            };
                        }
                    }
                });
        };

        $scope.next = () => {
            if ($scope.isValid()) {
                nocUI.clearFormValidity();
                if (!$rootScope.runningInIC && $scope.selectedInstanceProfiles.selectedInstanceProfileNode.name !== $scope.instanceProfileDefaultRole.name) {
                    $scope.wizardController.instanceProfileName = _.get($scope, 'selectedInstanceProfiles.selectedInstanceProfileNode.name');
                } else {
                    $scope.wizardController.instanceProfileName = null;
                }

                if ($scope.wizardController.choice === 'HA_AWS') {
                    if ($scope.selectedInstanceProfiles.selectedInstanceProfilesMediator.name !== $scope.instanceProfileDefaultRole.name) {
                        $scope.wizardController.haParameters.mediatorInstanceProfileName = $scope.selectedInstanceProfiles.selectedInstanceProfilesMediator.name;
                    } else {
                        $scope.wizardController.haParameters.mediatorInstanceProfileName = null;
                    }
                }

                if ($rootScope.runningInIC && !$scope.wizardController.selectedPackage.isCustom) {
                    $scope.wizardController.next('.volume');
                } else {
                    $scope.wizardController.next('.license');
                }
            } else {
                nocUI.handleErrorsInWizard(true);
            }
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('roles', {
                mediatorInstanceProfileName: _.get($scope.wizardController, 'haParameters.mediatorInstanceProfileName'),
                instanceProfileName: $scope.wizardController.instanceProfileName
            });
            if ($scope.wizardController.choice === 'HA_AWS') {
                $scope.wizardController.haParameters.mediatorInstanceProfileName = null;
            }
            $scope.wizardController.instanceProfileName = null;
        });

        $scope.restoreData = () => {
            const restoredData = $scope.wizardController.getStepData('roles');
            if (!restoredData) return;

            $scope.wizardController.instanceProfileName = restoredData.instanceProfileName;
            if ($scope.wizardController.choice === 'HA_AWS') {
                $scope.wizardController.haParameters.mediatorInstanceProfileName = restoredData.mediatorInstanceProfileName;
            }
        };

        $scope.restoreData();
        $scope.initialize();
    });
})();
