(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentAzureVnetStepController', ($scope, nocUI, nocHttp, $log,
                                                                                               MetadataResource, $rootScope) => {
        $scope.initialize = function initialize() {
            if ($scope.wizardController.vnetBeenHereBefore === $scope.wizardController.workingEnvironmentTypeAndProvider) return;
            $scope.wizardController.vnetBeenHereBefore = $scope.wizardController.workingEnvironmentTypeAndProvider;
            $scope.wizardController.useExistingSecurityGroup = false;

            $scope.clearAllModelValues();
            $scope.getNetworkRequirements()
                .then($scope.getAzureRegions)
                .catch($log.error);
        };

        $scope.pluralizeString = (length, string) => {
            switch (length) {
                case 0: return 'No' + string + 's';
                case 1: return length + string;
                default: return length + string + 's';
            }
        };

        $scope.getAzureRegions = () => {
            const subscriptionIdParam = 'subscriptionId=' + $scope.wizardController.selectedAzureSubscription.subscriptionId;

            return MetadataResource.getRegions(nocHttp, $scope.wizardController.pathPrefix, true, subscriptionIdParam, $scope.wizardController.cloudProviderAccountId)
                .then((regions) => {
                    $scope.regions = regions;
                    _.forEach(regions, (region) => {
                        region.vnets = $scope.reconstructVnets(region.vnets);
                    });
                    const regionWithSubnets = _.find(regions, (region) => !_.isEmpty(region.vnets));

                    if ($rootScope.instancePlacement) {
                        const currentRegion = _.find(regions, {name: $rootScope.instancePlacement.region});
                        if (currentRegion) {
                            $scope.handleRegionChange(currentRegion);
                        } else {
                            $scope.handleRegionChange(regionWithSubnets);
                        }
                    } else {
                        $scope.handleRegionChange(regionWithSubnets);
                    }

                    if ($scope.wizardController.failedRequestType) {
                        const regionObject = _.find($scope.regions, {name: $scope.wizardController.failedRequest.region});
                        $scope.handleRegionChange(regionObject);
                        const recreateVnet = _.find($scope.wizardController.selectedRegionObject.vnets, {id: $scope.wizardController.failedRequest.vnetId});
                        if (recreateVnet && recreateVnet.cidr.subnets.length > 0) {
                            $scope.wizardController.selectedVnet = recreateVnet;
                            $scope.selectSubnetAndSecurityGroup();
                            $scope.wizardController.selectedSubnet = _.find($scope.wizardController.selectedVnet.cidr.subnets, {id: $scope.wizardController.failedRequest.subnetId});
                            $scope.wizardController.selectedSecurityGroup = _.find($scope.wizardController.securityGroups, {id: $scope.wizardController.failedRequest.securityGroupId}) || $scope.wizardController.defaultSecurityGroup;
                            $scope.wizardController.useExistingSecurityGroup = $scope.wizardController.selectedSecurityGroup.id !== $scope.wizardController.defaultSecurityGroup.id;
                        }
                        $scope.wizardController.connectivityVerified = true;
                    }
                });
        };

        $scope.getNetworkRequirements = function getNetworkRequirements() {
            return MetadataResource.getNetworkRequirements(nocHttp, $scope.wizardController.pathPrefix, $scope.wizardController.workingEnvironmentTypeAndProvider.type)
                .then((networkRequirementsResponse) => {
                    $scope.wizardController.networkRequirements = networkRequirementsResponse;
                });
        };

        $scope.handleRegionChange = (newRegion) => {
            if (!newRegion) return;
            $scope.wizardController.selectedRegionObject = newRegion;
            $scope.wizardController.selectedRegion = newRegion.name;

            if ($scope.wizardController.selectedRegion !== $scope.lastSelectedRegion) {
                $scope.lastSelectedRegion = $scope.wizardController.selectedRegion;
                $scope.wizardController.selectedVnet = null;
                $scope.wizardController.selectedSubnet = null;
                $scope.wizardController.selectedSecurityGroup = null;
            }

            //Automatically select VNet if only one, and that one has subnets:
            if (newRegion.vnets && newRegion.vnets.length === 1 && newRegion.vnets[0].cidr.subnets.length) {
                $scope.wizardController.selectedVnet = newRegion.vnets[0];
                $scope.selectSubnetAndSecurityGroup();
            }
        };

        $scope.reconstructVnets = (vnets) => {
            if (vnets.length === 0) return;

            //The default security group must be defined on the wizard controller so in the review step we will be able to compare the selected with the default to see if the default was chosen
            $scope.wizardController.defaultSecurityGroup = {
                id: 'Use a generated security group',
                name: 'Use a generated security group',
                resourceGroup: ''
            };

            return _.flatMap(vnets.virtualNetworks, (vnet) => {
                return _.map(vnet.cidrs, (cidr) => {
                    return {
                        name: vnet.name,
                        identifier: vnet.name + '_' + cidr.cidr,
                        vnetCidr: cidr.cidr,
                        id: vnet.id,
                        resourceGroup: vnet.resourceGroup,
                        cidr,
                        subnets: cidr.subnets.length,
                        securityGroups: vnets.securityGroups
                    };
                });
            });
        };

        $scope.selectSubnetAndSecurityGroup = () => {
            //Sanity:
            if (!$scope.wizardController.selectedVnet && $scope.wizardController.selectedVnet.subnets.length === 0) return;

            nocUI.clearFormValidity();
            //Select by default only if there is 1 subnet, otherwise force the user to select. See NOC-2630
            $scope.wizardController.selectedSubnet = ($scope.wizardController.selectedVnet.cidr.subnets.length === 1 && $scope.wizardController.selectedVnet.cidr.subnets[0].availableIps >= $scope.wizardController.networkRequirements.node) ?
                $scope.wizardController.selectedVnet.cidr.subnets[0] :
                null;

            $scope.selectSecurityGroups();
        };

        $scope.selectSecurityGroups = () => {
            if ($scope.wizardController.selectedVnet) {
                $scope.wizardController.securityGroups = $scope.wizardController.selectedVnet.securityGroups;
                $scope.wizardController.selectedSecurityGroup = $scope.wizardController.useExistingSecurityGroup ? $scope.wizardController.securityGroups[0] : $scope.wizardController.defaultSecurityGroup;
            }
        };

        $scope.subnetTitle = (subnet) => {
            return subnet.availableIps < $scope.wizardController.networkRequirements.node ?
                'The working environment requires ' + $scope.pluralizeString($scope.wizardController.networkRequirements.node, ' IP') + ', but the subnet ' +
                (subnet.availableIps === 0 ? 'has none available.' : 'only has ' + subnet.availableIps + ' available.') :
                subnet.cidr;
        };

        $scope.clearAllModelValues = () => {
            $scope.wizardController.selectedRegionObject = null;
            $scope.wizardController.selectedRegion = null;
            $scope.wizardController.selectedVnet = null;
            $scope.wizardController.selectedSecurityGroup = null;
            $scope.wizardController.selectedSubnet = null;
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('vnet', {
                vnetBeenHereBefore: $scope.wizardController.vnetBeenHereBefore,
                selectedRegionObject: $scope.wizardController.selectedRegionObject,
                selectedVnet: $scope.wizardController.selectedVnet,
                selectedRegion: $scope.wizardController.selectedRegion,
                regions: $scope.regions,
                selectedSecurityGroup: $scope.wizardController.selectedSecurityGroup,
                selectedSubnet: $scope.wizardController.selectedSubnet
            });

            $scope.wizardController.vnetBeenHereBefore = null;
            $scope.clearAllModelValues();
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('vnet'));

        $scope.initialize();
    });
})();
