(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentDetailsStepAndCredentialsController', ($scope, nocUI, MetadataResource,
                                                                                                           nocHttp, pidgeon, $q,
                                                                                                           Patterns, nocDialog) => {
        $scope.weNameReadonly = false;

        $scope.initialize = () => {
            $scope.lastCloudProvider = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName;
            $scope.isAWS = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS';
            $scope.isAzure = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure';
            $scope.isGCP = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'GCP';
            $scope.regexName = $scope.isAWS ? 'workingEnvironmentName' : ($scope.isAzure ? 'azureWorkingEnvironmentName' : 'gcpWorkingEnvironmentName');
            $scope.weNameRegex = new RegExp(Patterns[$scope.regexName].regex);
            $scope.workingEnvironmentNameError = $scope.regexName;
            const regexTagKey = $scope.isAWS ? 'amazonTagKey' : ($scope.isAzure ? 'azureTagKey' : 'gcpLabelKey');
            $scope.weTagKeyRegex = new RegExp(Patterns[regexTagKey].regex);
            $scope.workingEnvironmentTagKeyError = regexTagKey;
            const regexTagValue = $scope.isAWS ? 'amazonTagValue' : ($scope.isAzure ? 'azureTagValue' : 'gcpLabelValue');
            $scope.weTagValueRegex = new RegExp(Patterns[regexTagValue].regex);

            $scope.workingEnvironmentTagValueError = regexTagValue;
            $scope.wizardController.customTags = $scope.wizardController.customTags || [];
            $scope.tags = $scope.wizardController;
            if ($scope.wizardController.detailsBeenHereBefore !== $scope.wizardController.workingEnvironmentTypeAndProvider.providerName) {
                $scope.wizardController.customTags = [];
            }
            $scope.wizardController.detailsBeenHereBefore = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName;

            $scope.errorMessagePatternTag = $scope.isAWS ? 'awsTagKeysMustBeUnique' : ($scope.isAzure ? 'azureTagKeysMustBeUnique' : 'gcpLabelKeysMustBeUnique');
            $scope.keyNamePlaceHolder = $scope.isAzure ? 'Up to 512 characters' : 'Up to 127 characters';

            if ($scope.isAWS) {
                $scope.originalAwsAccounts = $scope.wizardController.providerAccounts.awsAccounts;
                const accountFromRecreatedWe = _.find($scope.originalAwsAccounts, {publicId: $scope.wizardController.cloudProviderAccountId});
                $scope.wizardController.defaultAwsAccount = _.find($scope.originalAwsAccounts, {accountType: 'INSTANCE_PROFILE'}) || $scope.originalAwsAccounts[0];
                // If cloudProviderAccount was assigned from backup, no need to assign the default
                if (!$scope.wizardController.cloudProviderAccount || !_.find($scope.originalAwsAccounts, {publicId: $scope.wizardController.cloudProviderAccount.publicId})) {
                    $scope.wizardController.cloudProviderAccount = accountFromRecreatedWe || $scope.wizardController.defaultAwsAccount;
                }
                $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                selectAccountsForAwsDialog();
            }

            if ($scope.isAzure) {
                $scope.originalAzureAccounts = $scope.wizardController.providerAccounts.azureAccounts;
                const accountFromRecreatedWe = _.find($scope.originalAzureAccounts, {publicId: $scope.wizardController.cloudProviderAccountId});
                $scope.wizardController.defaultAzureAccount = _.find($scope.originalAzureAccounts, {accountType: 'SERVICE_IDENTITY'}) || $scope.originalAzureAccounts[0];
                // If cloudProviderAccount was assigned from backup, no need to assign the default
                if (!$scope.wizardController.cloudProviderAccount || !_.find($scope.originalAzureAccounts, {publicId: $scope.wizardController.cloudProviderAccount.publicId})) {
                    $scope.wizardController.cloudProviderAccount = accountFromRecreatedWe || $scope.wizardController.defaultAzureAccount;
                }
                $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                selectAccountsAndSubscriptionsForAzureDialog();

                if ($scope.wizardController.failedRequestType) {
                    $scope.wizardController.resourceGroupName = $scope.wizardController.failedRequest.resourceGroup;
                    $scope.wizardController.useDefaultResourceGroupName = ($scope.wizardController.resourceGroupName === $scope.wizardController.workingEnvironmentName + '-rg');
                    $scope.wizardController.selectedAzureSubscription = _.find($scope.wizardController.associatedSubscriptions, {subscriptionId: $scope.wizardController.failedRequest.subscriptionId});
                } else {
                    $scope.wizardController.useDefaultResourceGroupName = true;
                    $scope.wizardController.resourceGroupName = ($scope.wizardController.workingEnvironmentName || '[Working Environment Name]') + '-rg';
                }
            }
        };

        function selectAccountsForAwsDialog() {
            if ($scope.wizardController.failedRequestType) {
                $scope.selectedAWSAccount = _.find($scope.originalAwsAccounts, {publicId: _.get($scope.wizardController.failedRequest, 'cloudProviderAccount')});
            } else {
                $scope.selectedAWSAccount = $scope.wizardController.defaultAwsAccount;
            }
            $scope.wizardController.selectedAWSAccountId = _.get($scope.selectedAWSAccount, 'accountId');

            $scope.dialogScope = {
                awsAccounts: $scope.originalAwsAccounts,
                selectedAWSAccount: $scope.selectedAWSAccount
            };
            $scope.getTagKeysAndPutOnWizardController();
        }

        function setDialogScopeForAzure(subscriptionsResponse) {
            $scope.wizardController.associatedSubscriptions = subscriptionsResponse;
            if ($scope.wizardController.associatedSubscriptions) {
                $scope.wizardController.defaultAzureSubscription = _.find($scope.wizardController.associatedSubscriptions, {default: true}) || $scope.wizardController.associatedSubscriptions[0];
                if (!$scope.wizardController.selectedAzureSubscription) {
                    $scope.wizardController.selectedAzureSubscription = $scope.wizardController.defaultAzureSubscription;
                }
            } else {
                $scope.wizardController.defaultAzureSubscription = null;
                $scope.wizardController.selectedAzureSubscription = null;
            }

            if ($scope.wizardController.failedRequestType) {
                $scope.wizardController.selectedAzureSubscription = _.find($scope.wizardController.associatedSubscriptions, {subscriptionId: $scope.wizardController.failedRequest.subscriptionId}) || $scope.wizardController.selectedAzureSubscription;
            }

            $scope.dialogScope = {
                azureAccounts: $scope.originalAzureAccounts,
                selectedAzureAccount: $scope.selectedAzureAccount,
                associatedSubscriptions: $scope.wizardController.associatedSubscriptions,
                currentAzureSubscription: _.cloneDeep($scope.wizardController.selectedAzureSubscription),
                getTagKeysAndPutOnWizardController: $scope.getTagKeysAndPutOnWizardController
            };
        }

        function selectAccountsAndSubscriptionsForAzureDialog() {
            $scope.selectedAzureAccount = $scope.wizardController.defaultAzureAccount;
            if ($scope.wizardController.failedRequestType) {
                $scope.selectedAzureAccount = _.find($scope.originalAzureAccounts, {publicId: $scope.wizardController.failedRequest.cloudProviderAccount}) || $scope.wizardController.selectedAzureAccount;
            }

            return MetadataResource.getAssociatedSubscriptionsForAzure(nocHttp, pidgeon, $scope.selectedAzureAccount.publicId)
                .then((subscriptionsResponse) => {
                    pidgeon.clear();
                    setDialogScopeForAzure(subscriptionsResponse);
                    $scope.getTagKeysAndPutOnWizardController();
                })
                .catch(() => {
                    setDialogScopeForAzure();
                });
        }

        /*
         Need to watch the svm user name so that a change to the user name will trigger a validation check on the svm password
         -- because the svm password can not contain the username
         */
        $scope.$watch('wizardController.svmUserName', () => {
            $scope.svmPasswordRegex = new RegExp('^(?=.*[0-9])(?=.*[a-zA-Z])((?!' + $scope.wizardController.svmUserName + ').){8,}$');
        });

        $scope.addCustomTag = () => {
            $scope.addTagsPressed = true;
            $scope.wizardController.customTags.push({tagKey: '', tagValue: ''});
        };

        $scope.removeCustomTag = (tag) => {
            $scope.wizardController.customTags = _.without($scope.wizardController.customTags, tag);
        };

        $scope.handleWeNameChange = () => {
            if ($scope.wizardController.useDefaultResourceGroupName) {
                $scope.wizardController.resourceGroupName = ($scope.wizardController.workingEnvironmentName || '[Working Environment Name]') + '-rg';
            }
        };

        $scope.handleResourceGroupCheckboxChange = () => {
            if ($scope.wizardController.useDefaultResourceGroupName) {
                $scope.wizardController.resourceGroupName = ($scope.wizardController.workingEnvironmentName || '[Working Environment Name]') + '-rg';
            } else {
                $scope.wizardController.resourceGroupName = '';
            }
        };

        $scope.validateWorkingEnvironmentNameUniqueness = () => {
            //In recreate mode the user cannot change the WE name so there is no need to validate their uniqueness again.
            if ($scope.wizardController.failedRequestType) return $q.when();

            pidgeon.clear('weName');

            return nocHttp.get('/working-environments/exists/' + $scope.wizardController.workingEnvironmentName,
                'checking if the working environment name is already used', nocHttp.BLOCK)
                .then((weNameIsAlreadyUsed) => {
                    if (weNameIsAlreadyUsed) {
                        pidgeon.error('The name \'' + $scope.wizardController.workingEnvironmentName + '\' is already used by another working environment. Please use another one.', null, 'weName');

                        return $q.reject();
                    }
                });
        };

        $scope.validateResourceGroupNameUniqueness = () => {
            //In recreate mode the user cannot change the WE name or recourse group so there is no need to validate their uniqueness again.
            if ($scope.wizardController.failedRequestType) return $q.when();

            //Only relevant for Azure.
            if (!_.includes($scope.wizardController.choice, 'AZURE')) return $q.when();

            pidgeon.clear('resourceGroupName');

            const subscriptionIdParam = '?subscriptionId=' + $scope.wizardController.selectedAzureSubscription.subscriptionId;
            const cloudProviderAccountId = $scope.wizardController.cloudProviderAccountId ? (subscriptionIdParam ? '&' : '?') + 'cloudProviderAccountId=' + $scope.wizardController.cloudProviderAccountId : '';

            return nocHttp.get('/azure/vsa/working-environments/resource-group-exists/' + $scope.wizardController.resourceGroupName + subscriptionIdParam + cloudProviderAccountId,
                'checking if the resource group name is already used', nocHttp.BLOCK)
                .then((resourceGroupNameIsAlreadyUsed) => {
                    if (resourceGroupNameIsAlreadyUsed) {
                        pidgeon.error('The name \'' + $scope.wizardController.resourceGroupName + '\' is already used by another resource group. Please use another one.', null, 'resourceGroupName');

                        return $q.reject();
                    }
                });
        };

        $scope.openAwsAccountNameDialog = () => {
            return nocDialog.dialog('',
                'assets/templates/tabs/working-environments/aws-multi-account-dialog-content.html', {
                    dialogScope: $scope.dialogScope,
                    wizardController: $scope.wizardController,
                    width: 560
                }).then(() => {
                $scope.wizardController.cloudProviderAccount = $scope.dialogScope.selectedAWSAccount;
                $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                $scope.getTagKeysAndPutOnWizardController();
            });
        };

        $scope.openAzureSubscriptionDialog = () => {
            return nocDialog.dialog('',
                'assets/templates/dialogs/azure-subscription.html', {
                    width: 560,
                    dialogScope: $scope.dialogScope,
                    wizardController: $scope.wizardController
                })
                .then(() => {
                    $scope.wizardController.cloudProviderAccount = $scope.dialogScope.selectedAzureAccount;
                    $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                    $scope.wizardController.selectedAzureSubscription = $scope.dialogScope.currentAzureSubscription;
                    $scope.getTagKeysAndPutOnWizardController();
                    pidgeon.clear();
                });
        };

        $scope.getTagKeysAndPutOnWizardController = () => {
            const pathPrefix = $scope.isAWS ? '/vsa/' : '/azure/vsa/';
            const subscriptionIdParam = $scope.isAWS ? '' : '?subscriptionId=' + $scope.wizardController.selectedAzureSubscription.subscriptionId;
            const cloudProviderAccountId = $scope.wizardController.cloudProviderAccountId ? (subscriptionIdParam ? '&' : '?') + 'cloudProviderAccountId=' + $scope.wizardController.cloudProviderAccountId : '';

            MetadataResource.getTagKeys(nocHttp, pathPrefix, subscriptionIdParam + cloudProviderAccountId)
                .then((keysData) => {
                    $scope.wizardController.existingTagNames = _.map(keysData, 'key');
                    $scope.wizardController.existingTags = {};
                    _.forEach(keysData, (keyData) => {
                        $scope.wizardController.existingTags[keyData.key] = keyData.values;
                    });
                    $scope.existingTagNames = $scope.wizardController.existingTagNames;
                    $scope.existingTag = $scope.wizardController.existingTag;
                });
        };

        $scope.next = () => {
            if ($scope.isValid()) {
                $q.all([
                    $scope.validateWorkingEnvironmentNameUniqueness(),
                    $scope.validateResourceGroupNameUniqueness()
                ])
                    .then(() => {
                        $scope.wizardController.next(getNextStep());
                    });
            } else {
                nocUI.handleFormValidity($scope.VirginWizardForm);
            }
        };

        function getNextStep() {
            const nextStep = {
                'VSA_AWS': '.vpc',
                'HA_AWS': '.choose-ha-deployment-model',
                'VSA_Azure': '.vnet',
                'HA_Azure': '.vnet',
                'VSA_GCP': '.gcp-vpc'
            };

            return nextStep[$scope.wizardController.workingEnvironmentTypeAndProvider.combined];
        }

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('create-detailsCredentials', {
                workingEnvironmentName: $scope.wizardController.workingEnvironmentName,
                svmPassword: $scope.wizardController.svmPassword,
                retypeSVMPassword: $scope.wizardController.retypeSVMPassword,
                useDefaultResourceGroupName: $scope.wizardController.useDefaultResourceGroupName,
                resourceGroupName: $scope.wizardController.resourceGroupName,
                selectedAWSAccountId: $scope.wizardController.selectedAWSAccountId,
                selectedAzureSubscription: $scope.wizardController.selectedAzureSubscription,
                cloudProviderAccount: $scope.wizardController.cloudProviderAccount,
                detailsBeenHereBefore: $scope.wizardController.detailsBeenHereBefore
            });

            $scope.wizardController.workingEnvironmentName = '';
            $scope.wizardController.svmPassword = '';
            $scope.wizardController.retypeSVMPassword = '';
            $scope.wizardController.useDefaultResourceGroupName = '';
            $scope.wizardController.resourceGroupName = '';
            $scope.wizardController.selectedAWSAccountId = '';
            $scope.wizardController.selectedAzureSubscription = null;
            $scope.wizardController.cloudProviderAccount = null;
            $scope.wizardController.detailsBeenHereBefore = null;
        });

        if ($scope.wizardController.failedRequestType) {
            $scope.wizardController.workingEnvironmentName = $scope.wizardController.failedRequest.name;
            $scope.wizardController.customTags = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS' ?
                $scope.wizardController.failedRequest.awsTags : $scope.wizardController.failedRequest.azureTags;
            $scope.wizardController.useDefaultResourceGroupName = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure' ? $scope.wizardController.failedRequest.useDefaultResourceGroupName : null;
            $scope.wizardController.resourceGroupName = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure' ? $scope.wizardController.failedRequest.resourceGroupName : null;
            $scope.wizardController.selectedAWSAccountId = $scope.wizardController.failedRequest.selectedAWSAccountId;
            $scope.wizardController.selectedAzureSubscription = $scope.wizardController.failedRequest.selectedAzureSubscription;
            $scope.wizardController.cloudProviderAccountId = $scope.wizardController.failedRequest.cloudProviderAccount;
        } else {
            _.assignIn($scope.wizardController, $scope.wizardController.getStepData('create-detailsCredentials'));
        }

        $scope.$watch('wizardController.choice', () => {
            if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName !== $scope.lastCloudProvider) {
                $scope.initialize();
            }
        });
    });
})();
