(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentGoogleVpcStepController', ($scope, $log, MetadataResource,
                                                                                               nocHttp, $rootScope) => {
        $scope.initialize = () => {
            if ($scope.wizardController.gcpVpcBeenHereBefore === $scope.wizardController.workingEnvironmentTypeAndProvider) return;
            $scope.wizardController.gcpVpcBeenHereBefore = $scope.wizardController.workingEnvironmentTypeAndProvider;
            $scope.wizardController.useExistingFirewallPolicy = false;

            $scope.clearAllModelValues();

            $scope.getNetworkRequirements()
                .then($scope.getGoogleRegions)
                .catch($log.error);
        };

        $scope.getGoogleRegions = () => {
            return MetadataResource.getRegions(nocHttp, $scope.wizardController.pathPrefix)
                .then((regions) => {
                    $scope.regions = regions;
                    $scope.wizardController.instancePlacementDefaultForWizard = _.cloneDeep($rootScope.instancePlacement);
                    $scope.wizardController.instancePlacementDefaultForWizard.zone = $scope.wizardController.instancePlacementDefaultForWizard.region;
                    const occmRegionObject = _.find($scope.regions, (region) => {
                        return _.startsWith($scope.wizardController.instancePlacementDefaultForWizard.zone, region.name);
                    });
                    $scope.wizardController.instancePlacementDefaultForWizard.region = _.get(occmRegionObject, 'name');

                    $scope.wizardController.selectedRegionObject = occmRegionObject;
                    $scope.wizardController.selectedRegion = $scope.wizardController.instancePlacementDefaultForWizard.region;
                    $scope.wizardController.selectedZone = _.find(_.get($scope.wizardController.selectedRegionObject, 'zones'), {name: $scope.wizardController.instancePlacementDefaultForWizard.zone});
                    $scope.wizardController.selectedVpc = _.find(_.get($scope.wizardController.selectedRegionObject, 'vpcs'), {name: $scope.wizardController.instancePlacementDefaultForWizard.vpcId});
                    $scope.wizardController.selectedSubnet = _.find(_.get($scope.wizardController.selectedVpc, 'subnets'), {name: $scope.wizardController.instancePlacementDefaultForWizard.subnetId});
                });
        };

        $scope.handleRegionChange = (newRegion) => {
            if (!newRegion) return;
            $scope.wizardController.selectedRegionObject = newRegion;
            $scope.wizardController.selectedRegion = newRegion.name;

            if ($scope.wizardController.selectedRegion !== $scope.lastSelectedRegion) {
                $scope.lastSelectedRegion = $scope.wizardController.selectedRegion;
                $scope.wizardController.selectedZone = null;
                $scope.wizardController.selectedVpc = null;
                $scope.wizardController.selectedSubnet = null;

                if (newRegion.zones && newRegion.zones.length === 1) {
                    $scope.wizardController.selectedZone = newRegion.zones[0];
                }
                if (newRegion.vpcs && newRegion.vpcs.length === 1) {
                    $scope.wizardController.selectedVpc = newRegion.vpcs[0];
                }
                if ($scope.wizardController.selectedVpc) {
                    $scope.handleVpcChange();
                }
            }
        };

        $scope.handleVpcChange = () => {
            if (_.get($scope.wizardController, 'selectedVpc.subnets') && (_.get($scope.wizardController, 'selectedVpc.subnets').length === 1)) {
                $scope.wizardController.selectedSubnet = $scope.wizardController.selectedVpc.subnets[0];
            }
            $scope.wizardController.selectedSubnet = ((_.get($scope.wizardController, 'selectedVpc.subnets.length') === 1) &&
                $scope.wizardController.selectedVpc.subnets[0].availableIps >= $scope.wizardController.networkRequirements.node) ?
                $scope.wizardController.selectedVpc.subnets[0] :
                null;

            $scope.selectFirewallPolicy();
        };

        $scope.selectFirewallPolicy = () => {
            if ($scope.wizardController.selectedVpc) {
                $scope.wizardController.firewallPolicies = $scope.wizardController.selectedVpc.firewalls;
                $scope.wizardController.selectedFirewallPolicy = $scope.wizardController.useExistingFirewallPolicy ? $scope.wizardController.firewallPolicies[0] : null;
            }
        };

        $scope.pluralizeString = (length, string) => {
            switch (length) {
                case 0: return 'No' + string + 's';
                case 1: return length + string;
                default: return length + string + 's';
            }
        };

        $scope.getNetworkRequirements = function getNetworkRequirements() {
            return MetadataResource.getNetworkRequirements(nocHttp, $scope.wizardController.pathPrefix, $scope.wizardController.workingEnvironmentTypeAndProvider.type)
                .then((networkRequirementsResponse) => {
                    $scope.wizardController.networkRequirements = networkRequirementsResponse;
                });
        };

        $scope.subnetTitle = (subnet) => {
            return subnet.availableIps < $scope.wizardController.networkRequirements.node ?
                'The working environment requires ' + $scope.pluralizeString($scope.wizardController.networkRequirements.node, ' IP') + ', but the subnet ' +
                (subnet.availableIps === 0 ? 'has none available.' : 'only has ' + subnet.availableIps + ' available.') :
                subnet.ipCidrRange;
        };

        $scope.clearAllModelValues = () => {
            $scope.wizardController.selectedRegionObject = null;
            $scope.wizardController.selectedRegion = null;
            $scope.wizardController.selectedZone = null;
            $scope.wizardController.selectedFirewallPolicy = null;
            $scope.wizardController.selectedSubnet = null;
            $scope.wizardController.selectedVpc = null;
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('gcp-vpc', {
                gcpVpcBeenHereBefore: $scope.wizardController.gcpVpcBeenHereBefore,
                selectedRegionObject: $scope.wizardController.selectedRegionObject,
                selectedRegion: $scope.wizardController.selectedRegion,
                regions: $scope.regions,
                selectedZone: $scope.wizardController.selectedZone,
                selectedVpc: $scope.wizardController.selectedVpc,
                selectedSubnet: $scope.wizardController.selectedSubnet,
                selectedFirewallPolicy: $scope.wizardController.selectedFirewallPolicy
            });

            $scope.wizardController.gcpVpcBeenHereBefore = null;
            $scope.clearAllModelValues();
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('gcp-vpc'));

        $scope.initialize();
    });
})();
