(function () {
    "use strict";

    angular.module("noc").controller("VirginCreateWorkingEnvironmentHaRouteTablesStepController", function ($scope, nocUI, MetadataResource, nocHttp) {
        $scope.controllerData = { selectAll: false };

        $scope.onSelectionChange = function() {
            $scope.controllerData.selectAll = _.every($scope.wizardController.routeTables, {selected: true});
            $scope.setWizardStep($scope.wizardController.path['ha-route-tables']);
        };

        /*istanbul ignore next: trivial*/
        $scope.onSelectAll = function() {
            _.forEach($scope.wizardController.routeTables, function(row) {
                row.selected = $scope.controllerData.selectAll;
            });
        };

        /*istanbul ignore next: trivial*/
        $scope.expandRow = function (row) {
            row.expanded = !row.expanded;
            _.forEach($scope.wizardController.routeTables, function(rowIter) {
                if (row !== rowIter) rowIter.expanded = false;
            });
        };

        $scope.validateFloatingIPAddresses = function validateFloatingIPAddresses() {
            return MetadataResource.validateFloatingIPAddresses(
                nocHttp,
                $scope.wizardController.createVpcSelectObjects.region.code,
                $scope.wizardController.createVpcSelectObjects.vpc.vpcId,
                _($scope.wizardController.routeTables).filter("selected").map("id").value(),
                {
                    clusterFloatingIP: $scope.wizardController.haParameters.clusterFloatingIP,
                    dataFloatingIP: $scope.wizardController.haParameters.dataFloatingIP,
                    dataFloatingIP2: $scope.wizardController.haParameters.dataFloatingIP2,
                    svmFloatingIP: $scope.wizardController.haParameters.svmFloatingIP
                },
                $scope.wizardController.cloudProviderAccountId
            );
        };

        $scope.next = function next() {
            $scope.userClickedNext = true;
            nocUI.formValidate($scope.isValid)
                .then($scope.validateFloatingIPAddresses)
                .then(function () {
                    $scope.wizardController.next(".encryption");
                });
        };

        var recoverFailedRequestRouteTables = function(){
            _.forEach($scope.wizardController.routeTables, function(routeTable){
                routeTable.selected = _.includes($scope.wizardController.failedRequest.haParams.routeTableIds, routeTable.id);
            });
        };

        $scope.getRouteTables = function getRouteTables() {
            if (!$scope.wizardController.routeTables) {
                const region = $scope.wizardController.createVpcSelectObjects.region.code;
                const vpcId = $scope.wizardController.createVpcSelectObjects.vpc.vpcId;
                const node1SubnetId = $scope.wizardController.createVpcSelectObjects.subnet.subnetId;
                const node2SubnetId = $scope.wizardController.createVpcSelectObjects.node2Subnet.subnetId;
                const cloudProviderAccountId = $scope.wizardController.cloudProviderAccountId;

                MetadataResource.getRouteTables(nocHttp, region, vpcId, cloudProviderAccountId)
                    .then(function (response) {
                        $scope.wizardController.routeTables = _(response).map(function (row) {
                            row._subnets = _(row.subnets).map(function (subnetId) {
                                return _.find($scope.wizardController.createVpcSelectObjects.vpc.subnets, {subnetId: subnetId});
                            }).sortBy('cidr').value();
                            // select the main route table and tables that are associated with the nodes subnets
                            row.selected = row.main ||
                                _.some(row._subnets, function (subnet) {
                                    return subnet.subnetId === node1SubnetId || subnet.subnetId === node2SubnetId;
                                });
                            return row;
                        }).orderBy('selected', 'desc').value();

                        $scope.onSelectionChange();
                        if($scope.wizardController.failedRequestType) recoverFailedRequestRouteTables();
                    });
            }
        };

        $scope.getRouteTables();
    });
}());