(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentHaVpcStepController', ($scope, BaseCreateVpc, nocUI, nocHttp,
                                                                                           pidgeon, $state, $timeout) => {
        BaseCreateVpc.call($scope, pidgeon, nocUI, nocHttp);

        // defined on scope so it can be tested, this function validate the selection of subnets so there are enough available IP addresses
        $scope.validateSubnetSelection = function() {
            if (!$scope.wizardController.networkRequirements || !$scope.wizardController.createVpcSelectObjects.vpc) return false;

            var key = "subnet-validation";
            pidgeon.clear(key);
            var subnets = _.cloneDeep($scope.wizardController.createVpcSelectObjects.vpc.subnets);
            var firstNodeSubnet = _.find(subnets, {subnetId: $scope.wizardController.createVpcSelectObjects.subnet && $scope.wizardController.createVpcSelectObjects.subnet.subnetId});
            var secondNodeSubnet = _.find(subnets, {subnetId: $scope.wizardController.createVpcSelectObjects.node2Subnet && $scope.wizardController.createVpcSelectObjects.node2Subnet.subnetId});
            var mediatorSubnet = _.find(subnets, {subnetId: $scope.wizardController.createVpcSelectObjects.mediatorSubnet && $scope.wizardController.createVpcSelectObjects.mediatorSubnet.subnetId});

            if (firstNodeSubnet) firstNodeSubnet.availableIps -= $scope.wizardController.networkRequirements.node;
            if (secondNodeSubnet) secondNodeSubnet.availableIps -= $scope.wizardController.networkRequirements.node;
            if (mediatorSubnet) mediatorSubnet.availableIps -= $scope.wizardController.networkRequirements.mediator;

            var exceptions = _.filter([firstNodeSubnet, secondNodeSubnet, mediatorSubnet], function(subnet) {
                return subnet && subnet.availableIps < 0;
            });

            var nonValidSubnet = _.head(exceptions);
            if (nonValidSubnet) {
                pidgeon.error("Subnet " + nonValidSubnet.cidr + " cannot provide enough available IP addresses" ,undefined, key);
                return false;
            }
            return true;
        };

        $scope.$watchGroup(['wizardController.createVpcSelectObjects.subnet.subnetId', 'wizardController.createVpcSelectObjects.mediatorSubnet.subnetId', 'wizardController.createVpcSelectObjects.node2Subnet.subnetId'], function(){
            $scope.validateSubnetSelection();
        });

        /*$scope.$watchGroup(['wizardController.createVpcSelectObjects.region'], function(){
            $state.go($scope.wizardController.path['ha-vpc']);
        });*/

        $scope.getKeyPairs = _.noop;

        $scope.getAvailabilityZones = function () {
            $scope.wizardController.availabilityZones = _.union(_.map($scope.wizardController.createVpcSelectObjects.vpc.subnets, 'availabilityZone'));
        };

        $scope.clearAvailabilityZones = function () {
            $scope.wizardController.node1AvailabilityZone = null;
            $scope.wizardController.node2AvailabilityZone = null;
            $scope.wizardController.mediatorAvailabilityZone = null;
        };

        $scope.clearNode1Subnet = () => {
            $scope.wizardController.createVpcSelectObjects.subnet = undefined;
            $scope.showNode1Subnet = false; //Re-render the subnet (hack for NOC-10614)
            $timeout(() => {
                $scope.showNode1Subnet = true;
            });
            $state.go($scope.wizardController.path['ha-vpc']);
        };

        $scope.clearNode2Subnet = () => {
            $scope.wizardController.createVpcSelectObjects.node2Subnet = undefined;
            $scope.showNode2Subnet = false;
            $timeout(() => {
                $scope.showNode2Subnet = true;
            });
            $state.go($scope.wizardController.path['ha-vpc']);
        };

        $scope.clearMediatorSubnet = () => {
            $scope.wizardController.createVpcSelectObjects.mediatorSubnet = undefined;
            $scope.showMediatorSubnet = false;
            $timeout(() => {
                $scope.showMediatorSubnet = true;
            });
            $state.go($scope.wizardController.path['ha-vpc']);
        };

        $scope.next = function () {
            nocUI.formValidate($scope.validateSubnetSelection)
                .then(function(){
                    $scope.wizardController.next(".ha-public-key");
                });
        };

        var recoverFailedRequestSubnets = function(){
            if( !_.get($scope.wizardController, 'createVpcSelectObjects.vpc') ) return;
            $scope.wizardController.createVpcSelectObjects.node2Subnet = _.find($scope.wizardController.createVpcSelectObjects.vpc.subnets, {subnetId: $scope.wizardController.failedRequest.haParams.node2SubnetId});
            $scope.wizardController.createVpcSelectObjects.mediatorSubnet = _.find($scope.wizardController.createVpcSelectObjects.vpc.subnets, {subnetId: $scope.wizardController.failedRequest.haParams.mediatorSubnetId});
            $scope.wizardController.createVpcSelectObjects.node1AvailabilityZone = _.get($scope.wizardController, "createVpcSelectObjects.subnet.availabilityZone");
            $scope.wizardController.createVpcSelectObjects.node2AvailabilityZone = _.get($scope.wizardController, "createVpcSelectObjects.node2Subnet.availabilityZone");
            $scope.wizardController.createVpcSelectObjects.mediatorAvailabilityZone = _.get($scope.wizardController, "createVpcSelectObjects.mediatorSubnet.availabilityZone");
        };

        $scope.$on("$destroy", function () {
            $scope.wizardController.backupStepData("ha-vpc", {
                createVpcSelectObjects: {
                    node2Subnet: _.get($scope.wizardController, "createVpcSelectObjects.node2Subnet"),
                    mediatorSubnet: _.get($scope.wizardController, "createVpcSelectObjects.mediatorSubnet"),
                    node1AvailabilityZone: _.get($scope.wizardController, "createVpcSelectObjects.node1AvailabilityZone"),
                    node2AvailabilityZone: _.get($scope.wizardController, "createVpcSelectObjects.node2AvailabilityZone"),
                    mediatorAvailabilityZone: _.get($scope.wizardController, "createVpcSelectObjects.mediatorAvailabilityZone")
                },
                availabilityZones: $scope.wizardController.availabilityZones,
                vpcBeenHereBefore: $scope.wizardController.vpcBeenHereBefore
            });
            $scope.wizardController.vpcBeenHereBefore = null;
            $scope.wizardController.availabilityZones = null;
            $scope.wizardController.createVpcSelectObjects = undefined;
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData("ha-vpc"));

        $scope.initialize()
            .then(() => {
                if ($scope.wizardController.failedRequestType) recoverFailedRequestSubnets();
            });
        $scope.showNode1Subnet = $scope.showNode2Subnet = $scope.showMediatorSubnet = true;
    });
})();
