(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentLicenseStepController', ($scope, pidgeon, ManifestMapper,
                                                                                             nocUI, nocHttp, CreateWorkingEnvironmentWizardService,
                                                                                             MetadataResource, $log, nocDialog) => {
        CreateWorkingEnvironmentWizardService.call($scope, nocHttp, nocUI);

        const nameForInstanceByProvider = {
            AWS: 'instance',
            Azure: 'VM',
            GCP: 'machine'
        };
        $scope.wizardController.nameForInstance = nameForInstanceByProvider[$scope.wizardController.workingEnvironmentTypeAndProvider.providerName];
        $scope.licenseOptions = [];
        $scope.wizardController.ontapVersions = [];

        $scope.getOntapVersions = (regionCode, license) => {
            return ManifestMapper.getAvailableVersionsByLicense(regionCode, license);
        };

        function getIconNameForLicense(licenseName) {
            if (_.endsWith(licenseName, 'Explore')) return 'explore-license';
            if (_.endsWith(licenseName, 'Standard')) return 'standard-license';
            if (_.endsWith(licenseName, 'Premium')) return 'premium-license';
        }

        $scope.getLicenceOptions = function getLicenceOptions() {
            $scope.licenseOptions = _.reject(ManifestMapper.getAvailableLicenses($scope.wizardController.selectedRegion, $scope.wizardController.selectedOntapVersion), (license) => {
                return _.includes(license.type, 'byol');
            });
            _.forEach($scope.licenseOptions, (license) => {
                license._iconName = getIconNameForLicense(license.name);
            });
        };

        $scope.selectOntapVersion = () => {
            //If scrolled up to here from advanced step, bring the wizard back to the current step.
            if ($scope.wizardController.currentStateName !== 'license') {
                $scope.wizardController.setWizardStep($scope.wizardController.path.license);
            }
            $scope.wizardController.selectedOntapVersion = $scope.wizardController.selectedOntapVersion || $scope.ontapVersions[0];
            $scope.getLicenceOptions();
            $scope.selectLicense();

            //Refresh instance types according to the version
            $scope.wizardController.selectedLicense.availableInstances = ManifestMapper.getAvailableInstances($scope.wizardController.selectedRegion, $scope.wizardController.selectedOntapVersion, $scope.wizardController.selectedLicense.type);
            $scope.setInstanceType();
        };

        $scope.getLegacyInstances = function getLegacyInstances(providerType) {
            return MetadataResource.legacyInstanceTypes(nocHttp, $scope.wizardController.workingEnvironmentType !== 'HA', providerType !== 'AWS')
                .then((legacyInstanceTypes) => {
                    return _.map(legacyInstanceTypes, 'instanceType');
                });
        };

        $scope.openOntapVersionDialog = () => {
            return nocDialog.dialog('',
                'assets/templates/dialogs/ontap-version.html', {
                    width: 560,
                    dialogScope: {
                        ontapVersions: $scope.ontapVersions,
                        selectedOntapVersion: _.cloneDeep($scope.wizardController.selectedOntapVersion)
                    },
                    wizardController: $scope.wizardController
                })
                .then(() => {
                    $scope.selectOntapVersion();
                });
        };

        $scope.initialize = () => {
            pidgeon.clear('licenses');
            $scope.getLegacyInstances($scope.wizardController.workingEnvironmentTypeAndProvider.providerName)
                .then((legacyTypes) => {
                    $scope.disabledInstanceMessage = 'This ' + $scope.wizardController.nameForInstance + ' type does not support ONTAP Encryption';
                    $scope.legacyInstanceTypes = legacyTypes;
                    const lic = _.get($scope.wizardController, 'selectedLicense.type');
                    const isByolLic = lic ? _.includes(lic, 'byol') : false;
                    $scope.ontapVersions = $scope.getOntapVersions($scope.wizardController.selectedRegion, isByolLic ? 'byol' : 'paygo');
                    if (!$scope.wizardController.selectedOntapVersion) {
                        $scope.selectOntapVersion();
                    } else {
                        $scope.getLicenceOptions();
                    }
                    $scope.selectLicense(null, $scope.wizardController.selectedInstanceType);


                    if ((!$scope.licenseOptions || !$scope.licenseOptions.length) && !isByolLic) {
                        pidgeon.error('No licenses available for selected region', null, 'licenses');
                    }
                    if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS') {
                        const defaultTenanacy = {name: 'Shared', requestName: 'default'};
                        const dedicatedTenanacy = {name: 'Dedicated', requestName: 'dedicated'};
                        $scope.tenancies = [defaultTenanacy, dedicatedTenanacy];
                        $scope.disableTenancyChoice = _.get($scope.wizardController, 'createVpcSelectObjects.vpc.tenancy') === 'dedicated';
                        $scope.wizardController.tenancy = $scope.disableTenancyChoice ? dedicatedTenanacy :
                            ($scope.wizardController.tenancy || defaultTenanacy);
                    }

                    if ($scope.wizardController.failedRequestType) {
                        $scope.wizardController.selectedOntapVersion = $scope.wizardController.failedRequest.vsaMetadata.ontapVersion;
                        $scope.selectOntapVersion();
                        if (!($scope.wizardController.selectedLicense && _.includes($scope.wizardController.selectedLicense.type, 'byol'))) {
                            $scope.wizardController.selectedLicense = _.find($scope.licenseOptions, {type: $scope.wizardController.failedRequest.vsaMetadata.licenseType});
                        }
                        $scope.selectLicense();
                        $scope.setInstanceType($scope.wizardController.failedRequest.vsaMetadata.instanceType);
                        if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS') {
                            $scope.wizardController.tenancy = _.find($scope.tenancies, {requestName: $scope.wizardController.failedRequest.instanceTenancy});
                        }
                    }
                })
                .catch($log.error);
        };

        $scope.isLicenseDisabled = function isLicenseDisabled(license) {
            return $scope.wizardController.encryptionType === 'ONTAP' && !license._supportsOntapEncryption;
        };

        $scope.selectLicense = (license, instanceType) => {
            $scope.wizardController.selectedLicense =
                //Real options:
                license || $scope.wizardController.selectedLicense ||
                //Default options:
                _.find($scope.licenseOptions, {default: true}) || $scope.licenseOptions[0] || {name: 'No license available'};

            $scope.setInstanceType(instanceType);

            nocUI.clearFormValidity();
        };

        $scope.setInstanceType = (instanceType) => {
            $scope.wizardController.selectedInstanceType = instanceType ||
                ManifestMapper.getDefaultInstance($scope.wizardController.selectedRegion, $scope.wizardController.selectedOntapVersion, $scope.wizardController.selectedLicense.type) ||
                _.find($scope.wizardController.selectedLicense.availableInstances, (instanceType) => !$scope.isInstanceTypeDisabled(instanceType));
        };

        $scope.isInstanceTypeDisabled = (instanceType) => {
            return ($scope.wizardController.encryptionType === 'ONTAP' &&
            !_.includes(ManifestMapper.getEncryptionEnabledInstanceTypes($scope.wizardController.selectedOntapVersion), instanceType));
        };

        $scope.isInstanceLegacy = (selectedInstanceType, selectedLicenseType) => {
            return _.includes($scope.legacyInstanceTypes, selectedInstanceType) && selectedLicenseType !== 'Cloud Volumes ONTAP BYOL';
        };

        $scope.next = function next() {
            $scope.continueAfterLicenseOrPackagesStep('.tiering');
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('license', {
                selectedOntapVersion: $scope.wizardController.selectedOntapVersion,
                selectedLicense: $scope.wizardController.selectedLicense,
                selectedInstanceType: $scope.wizardController.selectedInstanceType,
                tenancy: $scope.wizardController.tenancy
            });

            $scope.wizardController.selectedOntapVersion = null;
            if (!_.includes(_.get($scope.wizardController, 'selectedLicense.type'), 'byol')) {
                //Only if it is paygo delete the license, as it was selected in prev step
                $scope.wizardController.selectedLicense = null;
            }
            $scope.wizardController.selectedInstanceType = null;
            $scope.wizardController.tenancy = null;
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('license'));
        $scope.initialize();
    });
})();
