(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentReviewStepController', ($scope, nocUI, nocHttp, pidgeon,
                                                                                            $timeout, nocDialog, nocNormalizerService,
                                                                                            nocRegionsService, nocSelectedWorkspaceOrTenantService) => {
        $scope.initialize = function initialize() {
            $scope.hasFloatingIps = $scope.wizardController.workingEnvironmentType === 'HA' && $scope.wizardController.failoverMode === 'FloatingIP';
            if ($scope.isAzure) {
                $scope.wizardController.encryptionType = 'AZURE';
            }
            if ($scope.isGCP) {
                $scope.wizardController.encryptionType = 'GCP';
            }
            $scope.selectedTab = 'Overview';
        };

        $scope.shouldPassCapacityTier = () => {
            //If we didnt show the component at all or thick volume or unsupported disk or fastest package or WORM enabled cannot be tiered or the user unchecked the tiering option
            if ($scope.wizardController.tieringDisabled || !$scope.wizardController.compositeSupported || $scope.wizardController.gcpAccountSkipped ||
                !_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes($scope.wizardController.selectedTier.supportedCapacityTiers, objectStorageType)) ||
                !_.get($scope.wizardController, 'volumeRequest.usageProfile.flags.enableThinProvisioning') ||
                _.get($scope.wizardController, 'worm.enabled')) return false;

            //Volume is skipped
            if ($scope.wizardController.volumeSkipped) return $scope.wizardController.forceCompositeVersion && !_.includes($scope.wizardController.selectedPackage.name, 'fastest');

            //Volume is not skipped
            return (!$scope.wizardController.volumeSkipped && ($scope.wizardController.forceCompositeVersion || ($scope.wizardController.tieringPolicy && $scope.wizardController.tieringPolicy.type !== 'none' && !($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0))));
        };

        //This function is an indication NOT to send tiering policy eventhough capacity tier was sent
        $scope.shouldNotPassTieringPolicy = () => {
            return $scope.wizardController.forceCompositeVersion && ($scope.wizardController.volumeSkipped || $scope.wizardController.gcpAccountSkipped || $scope.wizardController.capacityTiersDisableReasons.length > 0);
        };

        /*istanbul ignore next*/
        $scope.selectTab = (tabName) => {
            $scope.selectedTab = tabName;
        };

        $scope.haFailoverModeDisplay = (failoverMode) => {
            return failoverMode === 'FloatingIP' ? 'Multiple Availability Zones' : 'Single Availability Zone';
        };

        $scope.getKeyManagersDisplayList = () => {
            return _.map($scope.wizardController.ontapEncryption.keyManagers, 'name').join(', ');
        };

        $scope.buildBaseWorkingEnvironmentRequest = () => {
            const request = {
                name: $scope.wizardController.workingEnvironmentName,
                volume: $scope.wizardController.volumeSkipped ? undefined : _.cloneDeep($scope.wizardController.volumeRequest),
                cifsConfigurationRequest: $scope.wizardController.cifsRequest,
                tenantId: nocSelectedWorkspaceOrTenantService.getPublicId(),
                region: $scope.isGCP ? $scope.wizardController.selectedZone.name : $scope.wizardController.selectedRegion,
                packageName: $scope.wizardController.selectedPackage.name,
                dataEncryptionType: $scope.wizardController.encryptionType,
                capacityTier: $scope.shouldPassCapacityTier() ? ($scope.isAWS ? 'S3' : $scope.isAzure ? 'Blob' : 'cloudStorage') : undefined,
                vsaMetadata: {
                    ontapVersion: $scope.wizardController.selectedOntapVersion,
                    licenseType: $scope.wizardController.selectedLicense.type,
                    instanceType: $scope.wizardController.selectedInstanceType,
                    providedLicense: $scope.wizardController.providedLicense
                }
            };

            if ($scope.wizardController.selectedNSSAccount) {
                request.nssAccount = _.get($scope.wizardController, 'selectedNSSAccount.publicId');
            }

            return request;
        };

        $scope.buildVSAWorkingEnvironmentRequest = () => {
            const request = {
                subnetId: $scope.wizardController.createVpcSelectObjects.subnet.subnetId,
                svmPassword: $scope.wizardController.svmPassword,
                vpcId: $scope.wizardController.createVpcSelectObjects.vpc.vpcId,
                ontapEncryptionParameters: $scope.wizardController.ontapEncryption ? {
                    keyManagerIds: _.map($scope.wizardController.ontapEncryption.keyManagers, 'publicId'),
                    keyManagerCaCertificateId: $scope.wizardController.ontapEncryption.keyManagerCaCertificateId
                } : null,
                securityGroupId: $scope.wizardController.createVpcSelectObjects.securityGroup.securityGroupId === $scope.wizardController.defaultSecurityGroup.securityGroupId ?
                    undefined : $scope.wizardController.createVpcSelectObjects.securityGroup.securityGroupId,
                ebsVolumeType: $scope.wizardController.selectedTier.diskName,
                ebsVolumeSize: $scope.wizardController.providerVolumeSize,
                awsTags: $scope.wizardController.customTags || [],
                optimizedNetworkUtilization: $scope.wizardController.optimizedNetworkUtilization === 'optimized',
                instanceTenancy: $scope.wizardController.tenancy.requestName,
                iops: $scope.wizardController.selectedTier.diskName === 'io1' ? $scope.wizardController.iops : null,
                instanceProfileName: $scope.wizardController.instanceProfileName,
                cloudProviderAccount: $scope.wizardController.cloudProviderAccountId
            };

            if ($scope.wizardController.useKeysFromAnotherAccount || _.get($scope.wizardController, 'awsEncryptionKey.default') === false) {
                request.awsEncryptionParameters = $scope.wizardController.useKeysFromAnotherAccount ? {kmsKeyArn: $scope.wizardController.awsEncryptionKey.kmsKeyArn} : {kmsKeyId: $scope.wizardController.awsEncryptionKey.keyId};
            }

            if ($scope.wizardController.selectedLicense.platformLicenseRequired) {
                _.set(request, 'vsaMetadata.platformSerialNumber', $scope.wizardController.platformSerialNumber);
            }

            if ($scope.wizardController.createVpcSelectObjects.access.requestValue === 'key') {
                _.assignIn(request, {clusterKeyPairName: $scope.wizardController.createVpcSelectObjects.key.name});
            }

            return request;
        };

        $scope.buildHAVSAWorkingEnvironmentRequest = function buildHAVSAWorkingEnvironmentRequest() {
            const haparams = _.cloneDeep($scope.wizardController.haParameters);
            haparams.failoverMode = $scope.wizardController.failoverMode;
            haparams.node1SubnetId = $scope.wizardController.createVpcSelectObjects.subnet.subnetId;
            haparams.node2SubnetId = haparams.failoverMode === 'FloatingIP' ? $scope.wizardController.createVpcSelectObjects.node2Subnet.subnetId : haparams.node1SubnetId;
            haparams.mediatorSubnetId = haparams.failoverMode === 'FloatingIP' ? $scope.wizardController.createVpcSelectObjects.mediatorSubnet.subnetId : haparams.node1SubnetId;
            haparams.routeTableIds = _($scope.wizardController.routeTables).filter('selected')
                .map('id')
                .value();
            haparams.mediatorKeyPairName = $scope.wizardController.haKeySelectObjects.mediatorKey.name;

            //The entire haMediatorSecurityGroup object was copied as part of the cloneDeep
            // we only need the mediatorSecurityGroupId, so we copy it, and the object can be deleted
            haparams.mediatorSecurityGroupId = $scope.wizardController.haParameters.haMediatorSecurityGroup.securityGroupId === $scope.wizardController.defaultSecurityGroup.securityGroupId ?
                undefined : $scope.wizardController.haParameters.haMediatorSecurityGroup.securityGroupId;
            delete haparams.haMediatorSecurityGroup;

            if ($scope.wizardController.selectedLicense.platformLicenseRequired) {
                haparams.platformSerialNumberNode1 = $scope.wizardController.platformSerialNumberNode1;
                haparams.platformSerialNumberNode2 = $scope.wizardController.platformSerialNumberNode2;
                haparams.providedLicenseNode1 = $scope.wizardController.providedLicenseNode1;
                haparams.providedLicenseNode2 = $scope.wizardController.providedLicenseNode2;
            }

            if ($scope.wizardController.mediatorAccess.access.requestValue === 'proxy') {
                haparams.mediatorProxy = $scope.wizardController.mediatorProxy;
            }
            haparams.mediatorAssignPublicIP = $scope.wizardController.mediatorAccess.access.requestValue === 'public';

            const request = {
                svmPassword: $scope.wizardController.svmPassword,
                vpcId: $scope.wizardController.createVpcSelectObjects.vpc.vpcId,
                ontapEncryptionParameters: $scope.wizardController.ontapEncryption ? {
                    keyManagerIds: _.map($scope.wizardController.ontapEncryption.keyManagers, 'publicId'),
                    keyManagerCaCertificateId: $scope.wizardController.ontapEncryption.keyManagerCaCertificateId
                } : null,
                securityGroupId: $scope.wizardController.createVpcSelectObjects.securityGroup.securityGroupId === $scope.wizardController.defaultSecurityGroup.securityGroupId ? undefined : $scope.wizardController.createVpcSelectObjects.securityGroup.securityGroupId,
                ebsVolumeType: $scope.wizardController.selectedTier.diskName,
                ebsVolumeSize: $scope.wizardController.providerVolumeSize,
                haParams: haparams,
                instanceProfileName: $scope.wizardController.instanceProfileName,
                awsTags: $scope.wizardController.customTags || [],
                optimizedNetworkUtilization: $scope.wizardController.optimizedNetworkUtilization === 'optimized',
                instanceTenancy: $scope.wizardController.tenancy.requestName,
                iops: $scope.wizardController.selectedTier.diskName === 'io1' ? $scope.wizardController.iops : null,
                cloudProviderAccount: $scope.wizardController.cloudProviderAccountId
            };

            if ($scope.wizardController.haKeySelectObjects.access.requestValue === 'key') {
                _.assignIn(request, {clusterKeyPairName: $scope.wizardController.haKeySelectObjects.nodesKey.name});
            }

            if ($scope.wizardController.useKeysFromAnotherAccount || _.get($scope.wizardController, 'awsEncryptionKey.default') === false) {
                request.awsEncryptionParameters = $scope.wizardController.useKeysFromAnotherAccount ? {kmsKeyArn: $scope.wizardController.awsEncryptionKey.kmsKeyArn} : {kmsKeyId: $scope.wizardController.awsEncryptionKey.keyId};
            }

            return request;
        };

        $scope.buildAzureWorkingEnvironmentRequest = () => {
            const request = {
                subnetId: $scope.wizardController.selectedSubnet.id,
                svmPassword: $scope.wizardController.svmPassword,
                vnetId: $scope.wizardController.selectedVnet.id,
                cidr: $scope.wizardController.selectedVnet.cidr.cidr,
                ontapEncryptionParameters: $scope.wizardController.ontapEncryption ? {
                    keyManagerIds: _.map($scope.wizardController.ontapEncryption.keyManagers, 'publicId'),
                    keyManagerCaCertificateId: $scope.wizardController.ontapEncryption.keyManagerCaCertificateId
                } : null,
                securityGroupId: $scope.wizardController.selectedSecurityGroup.id === $scope.wizardController.defaultSecurityGroup.id ? null : $scope.wizardController.selectedSecurityGroup.id,
                skipSnapshots: false,
                diskSize: $scope.wizardController.providerVolumeSize.size,
                storageType: $scope.wizardController.selectedTier.name,
                azureTags: $scope.wizardController.customTags || [],
                resourceGroup: $scope.wizardController.resourceGroupName,
                serialNumber: $scope.wizardController.selectedLicense.platformLicenseRequired ?
                    $scope.wizardController.platformSerialNumber : undefined,
                subscriptionId: $scope.wizardController.selectedAzureSubscription.subscriptionId,
                cloudProviderAccount: $scope.wizardController.cloudProviderAccountId
            };

            if ($scope.wizardController.choice === 'HA_AZURE' && $scope.wizardController.selectedLicense.platformLicenseRequired) {
                request.haParams = {
                    platformSerialNumberNode1: $scope.wizardController.platformSerialNumberNode1,
                    platformSerialNumberNode2: $scope.wizardController.platformSerialNumberNode2
                };
            }

            return request;
        };

        $scope.buildGcpWorkingEnvironmentRequest = () => {
            const gcpLabels = _.map($scope.wizardController.customTags, ({tagKey: labelKey, tagValue: labelValue}) => ({labelKey, labelValue}));
            const request = {
                subnetId: $scope.wizardController.selectedSubnet.name,
                svmPassword: $scope.wizardController.svmPassword,
                vpcId: $scope.wizardController.selectedVpc.name,
                firewallRule: _.get($scope.wizardController, 'selectedFirewallPolicy.name'),
                gcpVolumeSize: $scope.wizardController.providerVolumeSize,
                gcpVolumeType: $scope.wizardController.selectedTier.name,
                serialNumber: $scope.wizardController.selectedLicense.platformLicenseRequired ?
                    $scope.wizardController.platformSerialNumber : undefined,
                gcpLabels: gcpLabels || []
            };

            if ($scope.wizardController.selectedGcpAccount) {
                _.set(request, 'gcpStorageAccount', $scope.wizardController.selectedGcpAccount.publicId);
            }

            return request;
        };
        $scope.createWERequest = () => {
            const data = $scope.buildBaseWorkingEnvironmentRequest();

            if ($scope.wizardController.writingSpeedState) {
                data.writingSpeedState = _.toUpper($scope.wizardController.writingSpeedState);
            }

            if ($scope.wizardController.worm && $scope.wizardController.worm.enabled) {
                data.wormRequest = {
                    activationCode: $scope.wizardController.worm.activationCode,
                    retentionPeriod: $scope.wizardController.worm.retentionPeriod
                };
            }

            const buildRequestByProvider = {
                AWS: $scope.wizardController.choice === 'HA_AWS' ? $scope.buildHAVSAWorkingEnvironmentRequest : $scope.buildVSAWorkingEnvironmentRequest,
                Azure: $scope.buildAzureWorkingEnvironmentRequest,
                GCP: $scope.buildGcpWorkingEnvironmentRequest
            };
            _.merge(data, buildRequestByProvider[$scope.wizardController.workingEnvironmentTypeAndProvider.providerName]());

            if (data.volume) {
                _.assignIn(data.volume, data.volume.usageProfile.flags);

                if ($scope.wizardController.volumeProtocol.type === 'CIFS') {
                    delete data.volume.exportPolicyInfo;
                    data.volume.shareInfo.accessControl.users = nocNormalizerService.normalizeUsersGroups(data.volume.shareInfo.accessControl._users);
                    delete data.volume.shareInfo.accessControl._users;
                } else { //NFS
                    delete data.volume.shareInfo;
                    delete data.cifsConfigurationRequest;
                    if (data.volume.exportPolicyInfo.policyType === 'none') {
                        data.volume.exportPolicyInfo.ips = []; // Server validates that ips should be an empty array for policy type none
                    } else {
                        data.volume.exportPolicyInfo.ips = nocNormalizerService.normalizeCustomAccessControl(data.volume.exportPolicyInfo._ips);
                    }
                    delete data.volume.exportPolicyInfo._ips;//Internal field that should not be sent to the server
                }

                data.volume.size = {size: data.volume.sizeGb, unit: 'GB'};
                data.volume.tieringPolicy = $scope.shouldPassCapacityTier() && !$scope.shouldNotPassTieringPolicy() ? _.get($scope.wizardController, 'tieringPolicy.type') : undefined;

                delete data.volume.sizeGb;
                delete data.volume.usageProfile;
            } else {
                delete data.cifsConfigurationRequest;
            }

            if ($scope.wizardController.failedRequestType) {
                data.failedToCreateWorkingEnvironmentId = $scope.wizardController.recreateId; // Send WE id to server so that the WE name can be reused
            }

            return data;
        };

        $scope.createWorkingEnvironment = () => {
            if ($scope.isValid()) {
                const data = $scope.createWERequest();
                pidgeon.clear('create');
                const urlPrefix = $scope.wizardController.workingEnvironmentType === 'HA' ? ($scope.isAWS ? '/aws/ha' : '/azure/ha') : ($scope.isAWS ? '/vsa' : $scope.isAzure ? 'azure/vsa' : 'gcp/vsa');
                nocHttp.post({
                    url: urlPrefix + '/working-environments',
                    data
                }, null, nocHttp.BLOCK)
                    .then($scope.setPathToPreviousOrRoot)
                    .catch((error) => {
                        if (error.failureInfo) {
                            nocDialog.dialog('Subscribe in Marketplace', 'assets/templates/dialogs/include-linky-dialog.html', {bindText: error.failureInfo});
                        } else {
                            pidgeon.error(pidgeon.serverErrorPrefix + 'creating working environment', error, 'create');
                        }
                    });
            } else { //Not valid
                nocUI.handleErrorsInWizard(true);
            }
        };

        $scope.getVPCName = () => {
            return $scope.wizardController.createVpcSelectObjects.vpc.vpcId + ' (' + $scope.wizardController.createVpcSelectObjects.vpc.cidrBlock + ')';
        };

        $scope.getGcpVpcName = () => {
            return $scope.wizardController.selectedVpc.name;
        };

        $scope.getObjectStorageName = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            return providerToObjectStorage[$scope.wizardController.workingEnvironmentTypeAndProvider.providerName];
        };

        $scope.getCidr = () => {
            if ($scope.isAWS) {
                return $scope.wizardController.createVpcSelectObjects.subnet.cidr;
            } else if ($scope.isAzure) {
                return $scope.wizardController.selectedSubnet.cidr;
            } else {
                return $scope.wizardController.selectedSubnet.ipCidrRange;
            }
        };

        $scope.getVnetName = () => {
            if (!$scope.wizardController.selectedVnet) return '';

            return $scope.wizardController.selectedVnet.name + ' | ' + $scope.wizardController.selectedVnet.resourceGroup + ' (' + $scope.wizardController.selectedVnet.cidr.cidr + ')';
        };

        $scope.getUnderlyingDisks = () => {
            return $scope.wizardController.selectedTier.display + ', ' + $scope.wizardController.providerVolumeSize._identifier;
        };

        $scope.encryptionTypeString = () => {
            if ($scope.wizardController.encryptionType === 'ONTAP') {
                return 'ONTAP Managed';
            } else if ($scope.wizardController.encryptionType === 'AWS') {
                return 'AWS Managed';
            } else if ($scope.wizardController.encryptionType === 'AZURE') {
                return 'Azure Managed';
            } else if ($scope.wizardController.encryptionType === 'GCP') {
                return 'Google Cloud Managed';
            } else {
                return 'None';
            }
        };

        $scope.pricePopup = () => {
            $timeout(() => {
                const title = $scope.wizardController.workingEnvironmentType === 'VSA' ?
                    'Cloud Manager allocates the following resources for each Cloud Volumes ONTAP working environment:' :
                    'Cloud Manager allocates the following resources for each HA Cloud Volumes ONTAP working environment:';
                nocDialog.dialog(title, 'assets/templates/dialogs/purchase-summary-dialog-content.html', {
                    wizardController: $scope.wizardController,
                    width: 635
                });
            });
        };

        $scope.registrationPopup = () => {
            nocDialog.dialog('NetApp Support Registration Information', 'assets/templates/dialogs/support-registration-manual-register-popup.html');
        };

        $scope.apiPopup = () => {
            nocDialog.dialog('API Request', 'assets/templates/dialogs/api-example-popup.html', {
                weRequest: $scope.createWERequest(),
                pathPrefix: $scope.wizardController.pathPrefix
            });
        };

        $scope.getRegionName = (regionName) => {
            return $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS' ? regionName : (nocRegionsService.azureRegions()[regionName] ? nocRegionsService.azureRegions()[regionName].name : regionName);
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('review', {
                agreeRegister: $scope.wizardController.agreeRegister,
                agreeResources: $scope.wizardController.agreeResources
            });
            $scope.wizardController.agreeRegister = false;
            $scope.wizardController.agreeResources = false;
        });

        $scope.initialize();

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('review'));
    });
})();
