(() => {
    'use strict';
    angular.module('noc').controller('VirginCreateWorkingEnvironmentTieringPolicyStepController', ($scope, BaseCreateVolume) => {
        BaseCreateVolume.call($scope);

        $scope.getObjectStorageName = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            return providerToObjectStorage[$scope.wizardController.workingEnvironmentTypeAndProvider.providerName];
        };

        $scope.disabledTooltipForTier = function disabledTooltipForTier(tier) {
            if ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0) {
                return $scope.wizardController.capacityTiersDisableReasons[0];
            }
            if (_.get($scope.wizardController, 'worm.enabled')) {
                return 'If you want to use storage tiers, you must disable WORM storage.';
            }
            if (!_.get($scope, 'wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning')) {
                return 'If you want to use storage tiers, you must enable the Storage Efficiency usage profile.';
            }
            if (tier.disabledMessage) {
                return tier.disabledMessage;
            }
            if (!_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes(_.get(tier, 'supportedCapacityTiers'), objectStorageType))) {
                tier.disabledMessage = $scope.getObjectStorageName() + ' tiering is not available for this disk type.';

                return tier.disabledMessage;
            }
            if ($scope.wizardController.tieringDisabled) {
                return 'If you want to use storage tiers, you must select enable in the storage tiering checkbox.';
            }
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('tiering-policy', {
                tieringPolicy: $scope.wizardController.tieringPolicy
            });
            $scope.wizardController.tieringPolicy = {};
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('tiering-policy'));

        if ($scope.wizardController.failedRequestType && $scope.wizardController.failedRequest.volume) {
            $scope.volumeRequest.tieringPolicy = $scope.wizardController.failedRequest.volume.tieringPolicy;
            $scope.wizardController.tieringPolicy = $scope.wizardController.failedRequest.volume.tieringPolicy;
            $scope.wizardController.capacityTier = $scope.wizardController.failedRequest.capacityTier;
            $scope.volumeRequest.usageProfile = _.find($scope.usageProfiles, (usage) => {
                return _.isMatch(usage.flags, {
                    enableCompression: $scope.wizardController.failedRequest.volume.enableCompression,
                    enableDeduplication: $scope.wizardController.failedRequest.volume.enableDeduplication,
                    enableThinProvisioning: $scope.wizardController.failedRequest.volume.enableThinProvisioning
                });
            });
        }

        $scope.$watch('wizardController.availableTieringPoliciesForTier', (availableTieringPoliciesForTier) => {
            $scope.wizardController.availableTieringPoliciesForTier = availableTieringPoliciesForTier;
        });
    });
})();
