(() => {
    'use strict';
    angular.module('noc').controller('VirginCreateWorkingEnvironmentVolumeStepController', ($scope, nocUI, pidgeon, BaseCreateVolume, usageProfiles, $state, nocHttp, TieringMetadataResource) => {
        BaseCreateVolume.call($scope);

        $scope.initialize = () => {
            //NOC-3249 remove un-skip
            $scope.wizardController.volumeSkipped = false;

            $scope.volumePossibleTypes = _.values($scope.possibleTiers);
            $scope.volumeRequest = $scope.volumeRequest || {};
            $scope.volumeProtocol = $scope.wizardController.volumeProtocol || {type: 'NFS'};
            $scope.needCifsConfiguration = $scope.volumeProtocol.type === 'CIFS';
            $scope.fetchSnapshotPolicy(null, $scope.wizardController.pathPrefix, nocHttp)
                .then(() => {
                    $scope.moreInfoParameters = {
                        schedules: _.find($scope.snapshotPolicies, {name: $scope.wizardController.volumeRequest.snapshotPolicyName}).schedules
                    };
                    $scope.initialized = true;
                });

            TieringMetadataResource.getSupportedCapacityTiers(
                nocHttp,
                $scope.wizardController.pathPrefix,
                $scope.wizardController.selectedRegion,
                $scope.wizardController.selectedOntapVersion,
                $scope.wizardController.encryptionType || 'NONE',
                $scope.wizardController.selectedLicense.type,
                $scope.wizardController.selectedInstanceType,
                $scope.wizardController.selectedAzureSubscription ? $scope.wizardController.selectedAzureSubscription.subscriptionId : null,
                $scope.wizardController.cloudProviderAccountId
            )
                .then((response) => {
                    $scope.wizardController.availableTieringPoliciesForTier = _.find(response.supportedCapacityTiersPerVolumeType, {volumeType: $scope.wizardController.selectedTier.name}).availableTieringPolicies;
                    $scope.wizardController.tieringPolicy = {
                        type: _.includes($scope.availableTieringPoliciesForTier, 'auto') ? 'auto' : (_.includes($scope.wizardController.availableTieringPoliciesForTier, 'snapshot_only') ? 'snapshot_only' : 'none')
                    };
                    $scope.wizardController.forceCompositeVersion = response.forceCompositeVersion;
                    $scope.wizardController.compositeSupported = response.compositeSupported;

                    if (response.capacityTiersDisableReasons) {
                        $scope.wizardController.capacityTiersDisableReasons = response.capacityTiersDisableReasons;
                    }

                    if (_.includes($scope.wizardController.selectedPackage.name, 'fastest')) {
                        $scope.volumeRequest.usageProfile = $scope.usageProfiles[1];
                    } else {
                        $scope.volumeRequest.usageProfile = $scope.usageProfiles[0];
                    }
                });
        };

        $scope.getDefaultExportPolicyIps = () => {
            switch ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName) {
                case 'AWS': return [$scope.wizardController.createVpcSelectObjects.vpc.cidrBlock];
                case 'Azure': return [$scope.wizardController.selectedVnet.cidr.cidr];
                case 'GCP': return [$scope.wizardController.selectedSubnet.ipCidrRange];
                default: return [];
            }
        };

        $scope.skip = function skip() {
            $scope.wizardController.volumeSkipped = true;
            $scope.volumeRequest.name = '';
            $scope.volumeRequest.sizeGb = '';
            const defaultPolicy = $scope.getSnapshotPolicyByName('default');
            $scope.volumeRequest.snapshotPolicyName = defaultPolicy ? defaultPolicy.name : $scope.snapshotPolicies[0].name;
            $scope.volumeRequest.usageProfile = usageProfiles.default;

            $scope.nextStep();
        };

        $scope.$watchCollection('volumeRequest', (newVolRequest) => {
            if ((newVolRequest.name || newVolRequest.sizeGb) && $scope.wizardController.volumeSkipped &&
                $scope.currentStepPath !== $scope.wizardController.path.volume) {
                $scope.setWizardStep($scope.wizardController.path.volume);
            }
        });

        $scope.userPressedEnter = () => {
            if (!$scope.volumeRequest.name && !$scope.volumeRequest.sizeGb) {
                return $scope.skip();
            }
            $scope.wizardController.volumeSkipped = false;
            $scope.nextStep();
        };

        $scope.next = function next() {
            $scope.wizardController.volumeSkipped = false;
            $scope.nextStep();
        };

        $scope.nextStep = () => {
            if ($scope.isValid() || $scope.wizardController.volumeSkipped) {
                pidgeon.clear();
                nocUI.clearFormValidity();
                $scope.wizardController.volumeProtocol = $scope.volumeProtocol;
                if ($scope.needCifsConfiguration && !$scope.wizardController.volumeSkipped) {
                    $state.go('.cifs');
                } else if (!$scope.wizardController.volumeSkipped) {
                    $state.go('.tiering-policy');
                } else if ($scope.isGCP) {
                    $state.go('.gcp-accounts');
                } else {
                    $state.go('.review');
                }
            } else {
                nocUI.handleErrorsInWizard(true);
            }
        };

        $scope.volumeProtocolChanged = () => {
            $scope.needCifsConfiguration = $scope.volumeProtocol.type === 'CIFS';
            $scope.volumeRequest.shareInfo.shareName = $scope.volumeRequest.name ? $scope.volumeRequest.name + '_share' : '';
            if ($scope.volumeRequest.shareInfo.shareName.length > 80) $scope.volumeRequest.shareInfo.shareName = '';
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('volume', {
                volumeSkipped: $scope.wizardController.volumeSkipped,
                volumeRequest: $scope.wizardController.volumeRequest,
                volumeProtocol: $scope.volumeProtocol
            });

            $scope.wizardController.volumeSkipped = false;
            $scope.wizardController.volumeRequest = {};
        });

        $scope.clearFormValidity = nocUI.clearFormValidity;

        const ips = $scope.getDefaultExportPolicyIps();
        $scope.wizardController.volumeRequest = {
            exportPolicyInfo: {
                policyType: 'custom',
                ips,
                _ips: ips.join(', ')
            },
            shareInfo: {
                accessControl: {
                    _users: 'Everyone;',
                    users: ['Everyone'],
                    permission: 'full_control'
                },
                shareName: ''
            },
            usageProfile: usageProfiles.default
        };

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('volume'));
        $scope.initialize();
        if ($scope.wizardController.failedRequestType && $scope.wizardController.failedRequest.volume) {
            $scope.volumeRequest.name = $scope.wizardController.failedRequest.volume.name;
            $scope.volumeRequest.sizeGb = $scope.wizardController.failedRequest.volume.size.size;
            $scope.volumeRequest.snapshotPolicyName = $scope.wizardController.failedRequest.volume.snapshotPolicyName;
            $scope.volumeProtocol.type = $scope.wizardController.failedRequest.volume.exportPolicyInfo ? 'NFS' : 'CIFS';
            $scope.volumeProtocolChanged();
            if ($scope.volumeProtocol.type === 'NFS') {
                $scope.volumeRequest.exportPolicyInfo.policyType = $scope.wizardController.failedRequest.volume.exportPolicyInfo.policyType;
                $scope.volumeRequest.exportPolicyInfo._ips = $scope.wizardController.failedRequest.volume.exportPolicyInfo.ips.toString();
            } else if ($scope.volumeProtocol.type === 'CIFS') {
                $scope.volumeRequest.shareInfo = $scope.wizardController.failedRequest.volume.shareInfo;
                $scope.volumeRequest.shareInfo.accessControl._users = $scope.wizardController.failedRequest.volume.shareInfo.accessControl.users.join(';');
            }
        }
    });
})();
