(() => {
    'use strict';

    angular.module('noc').controller('VirginCloudOntapCredentialsStepController', ($scope, nocUI, nocHttp, nocUtil,
                                                                                   CloudProviderAccountResource, nocDialog, pidgeon, nocSelectedWorkspaceOrTenantService) => {
        $scope.initialize = () => {
            $scope.wizardController.svmUser = 'admin';
            $scope.nssAccountRequest = {};
            CloudProviderAccountResource.getAccounts(nocHttp)
                .then((accounts) => {
                    $scope.wizardController.nssAccounts = accounts.nssAccounts;
                    if (!$scope.wizardController.selectedNSSAccount) {
                        $scope.wizardController.selectedNSSAccount = accounts.nssAccounts[0];
                    }
                });

            const isSingleAws = !_.includes($scope.wizardController.selectedWorkingEnvironment.type, 'HA') && $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS';
            $scope.unregistered = !_.get($scope.wizardController.selectedWorkingEnvironment, 'registered');
            $scope.byol = isSingleAws ? _.get($scope.wizardController.selectedWorkingEnvironment, 'isByol') : _.get($scope.wizardController.selectedWorkingEnvironment, 'byol');
        };

        const recover = () => {
            const basicData = {
                cloudProviderAccount: $scope.wizardController.cloudProviderAccountId,
                tenantId: $scope.selectedTenant ? $scope.selectedTenant.publicId : nocSelectedWorkspaceOrTenantService.getPublicId(),
                region: $scope.wizardController.selectedRegion,
                clusterCredentials: {userName: $scope.wizardController.svmUser, password: $scope.wizardController.svmPassword},
                nssAccountId: _.get($scope.wizardController, 'selectedNSSAccount.publicId')
            };
            const data = {
                AWS: _.merge({
                    stackId: $scope.wizardController.selectedWorkingEnvironment.stackId
                }, basicData),
                Azure: _.merge({
                    id: $scope.wizardController.selectedWorkingEnvironment.id,
                    resourceGroupName: $scope.wizardController.selectedWorkingEnvironment.resourceGroup,
                    instanceName: $scope.wizardController.selectedWorkingEnvironment.name
                }, basicData),
                GCP: _.merge({
                    zone: $scope.wizardController.selectedWorkingEnvironment.zone,
                    instanceName: $scope.wizardController.selectedWorkingEnvironment.name
                }, basicData)
            };

            if (_.get($scope, 'wizardController.selectedAzureSubscription.subscriptionId')) {
                data.Azure.providerAccount = $scope.wizardController.selectedAzureSubscription.subscriptionId;
            }

            const workingEnvironmentType = _.includes($scope.wizardController.selectedWorkingEnvironment.type, 'HA') ? 'HA' : $scope.wizardController.workingEnvironmentTypeAndProvider.type;
            const pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix(workingEnvironmentType, $scope.wizardController.workingEnvironmentTypeAndProvider.providerName);
            nocHttp.post({
                url: pathPrefix + (workingEnvironmentType === 'HA' ? 'discovery/recover' : 'recover'),
                data: data[$scope.wizardController.workingEnvironmentTypeAndProvider.providerName]
            }, 'discovering working environment ' + $scope.wizardController.selectedWorkingEnvironment.name, nocHttp.LOADING)
                .then($scope.setPathToPreviousOrRoot);
        };

        $scope.recover = () => {
            if ($scope.isValid()) {
                if (_.get($scope.nssAccountRequest, 'accountName')) {
                    CloudProviderAccountResource.createAccount(nocHttp, 'nss', $scope.nssAccountRequest.accountName, $scope.nssAccountRequest.providerKeys, [], true)
                        .then((account) => {
                            pidgeon.clear('createNssAccount');
                            $scope.wizardController.nssAccounts = [account];
                            $scope.wizardController.selectedNSSAccount = account;
                            recover();
                        })
                        .catch((error) => {
                            pidgeon.error(error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length), null, 'createNssAccount');
                        });
                } else {
                    recover();
                }
            } else {
                nocUI.handleErrorsInWizard(true);
            }
        };

        $scope.usingByolAndNotEval = () => {
            return $scope.byol;
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/nss-learn-more.html', {
                width: 890,
                fromTop: '20%'
            });
        };

        $scope.goToAccountSettings = () => {
            $scope.checkAndSetState('cloud-provider-accounts.account-settings');
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('credentials', {
                svmPassword: $scope.wizardController.svmPassword,
                svmUser: $scope.wizardController.svmUser
            });
            $scope.wizardController.svmPassword = '';
            $scope.wizardController.svmPassword = 'admin';
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('credentials'));

        $scope.initialize();
    });
})();
