(() => {
    'use strict';

    angular.module('noc').controller('VirginCloudOntapRegionStepController', ($scope, $rootScope, nocUI, $state, nocDialog,
                                                                              nocUtil, $timeout, nocHttp, MetadataResource,
                                                                              $q, uiGridConstants, pidgeon) => {
        $scope.initialize = function initialize() {
            $scope.wizardController.title = 'Discover an Existing Cloud Volumes ONTAP';
            const providerAccounts = $scope.wizardController.providerAccounts;
            if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS') {
                $scope.originalAwsAccounts = providerAccounts.awsAccounts;
                $scope.wizardController.defaultAwsAccount = _.find(providerAccounts.awsAccounts, {accountType: 'INSTANCE_PROFILE'}) || providerAccounts.awsAccounts[0];
                if (!$scope.wizardController.cloudProviderAccount || !_.find($scope.originalAwsAccounts, {publicId: $scope.wizardController.cloudProviderAccount.publicId})) {
                    $scope.wizardController.cloudProviderAccount = $scope.wizardController.defaultAwsAccount;
                }
                $scope.selectedAWSAccount = $scope.wizardController.defaultAwsAccount;
                $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                selectAccountsForAwsDialog();
            } else if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure') {
                $scope.originalAzureAccounts = providerAccounts.azureAccounts;
                $scope.wizardController.defaultAzureAccount = _.find(providerAccounts.azureAccounts, {accountType: 'SERVICE_IDENTITY'}) || providerAccounts.azureAccounts[0];
                if (!$scope.wizardController.cloudProviderAccount || !_.find($scope.originalAzureAccounts, {publicId: $scope.wizardController.cloudProviderAccount.publicId})) {
                    $scope.wizardController.cloudProviderAccount = $scope.wizardController.defaultAzureAccount;
                }
                $scope.selectedAzureAccount = $scope.wizardController.defaultAzureAccount;
                $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                selectAccountsAndSubscriptionsForAzureDialog();
            } else {
                $scope.getRegions();
            }
        };

        function selectAccountsForAwsDialog() {
            $scope.wizardController.selectedAWSAccountId = $scope.selectedAWSAccount.accountId;

            $scope.dialogScope = {
                awsAccounts: $scope.originalAwsAccounts,
                selectedAWSAccount: $scope.selectedAWSAccount,
                discover: true,
                handleRegionChange: $scope.handleRegionChange,
                selectedRegion: $scope.selectedRegionObject
            };
            $scope.getRegions();
        }

        function setDialogScopeForAzure(subscriptionsResponse) {
            $scope.wizardController.associatedSubscriptions = subscriptionsResponse;

            if (subscriptionsResponse) {
                $scope.wizardController.defaultAzureSubscription = _.find(subscriptionsResponse, {default: true}) || $scope.wizardController.associatedSubscriptions[0];
                if (!$scope.wizardController.selectedAzureSubscription) {
                    $scope.wizardController.selectedAzureSubscription = $scope.wizardController.defaultAzureSubscription;
                }
            } else {
                $scope.wizardController.defaultAzureSubscription = null;
                $scope.wizardController.selectedAzureSubscription = null;
            }

            $scope.dialogScope = {
                azureAccounts: $scope.originalAzureAccounts,
                selectedAzureAccount: $scope.selectedAzureAccount,
                associatedSubscriptions: $scope.wizardController.associatedSubscriptions,
                currentAzureSubscription: _.cloneDeep($scope.wizardController.selectedAzureSubscription),
                discover: true,
                handleRegionChange: $scope.handleRegionChange,
                selectedRegion: $scope.selectedRegionObject
            };
        }


        function selectAccountsAndSubscriptionsForAzureDialog() {
            return MetadataResource.getAssociatedSubscriptionsForAzure(nocHttp, pidgeon, $scope.wizardController.cloudProviderAccountId)
                .then((subscriptionsResponse) => {
                    setDialogScopeForAzure(subscriptionsResponse);
                    const subscriptionId = 'subscriptionId=' + $scope.wizardController.selectedAzureSubscription.subscriptionId;
                    $scope.getRegions(subscriptionId, null);
                })
                .catch(() => {
                    setDialogScopeForAzure();
                });
        }

        $scope.openAwsAccountNameDialog = () => {
            return nocDialog.dialog('',
                'assets/templates/tabs/working-environments/aws-multi-account-dialog-content.html', {
                    dialogScope: $scope.dialogScope,
                    wizardController: $scope.wizardController,
                    width: 560
                })
                .then(() => {
                    const accountChanged = $scope.selectedAWSAccount.accountId !== $scope.dialogScope.selectedAWSAccount.accountId;
                    $scope.selectedAWSAccount = $scope.dialogScope.selectedAWSAccount;
                    $scope.wizardController.cloudProviderAccount = $scope.selectedAWSAccount;
                    $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                    $scope.getRegions('', accountChanged);
                });
        };

        $scope.openAzureSubscriptionDialog = () => {
            return nocDialog.dialog('',
                'assets/templates/dialogs/azure-subscription.html', {
                    width: 560,
                    dialogScope: $scope.dialogScope,
                    wizardController: $scope.wizardController
                })
                .then(() => {
                    const accountChanged = $scope.wizardController.cloudProviderAccount !== $scope.dialogScope.selectedAzureAccount;
                    $scope.wizardController.cloudProviderAccount = $scope.dialogScope.selectedAzureAccount;
                    $scope.wizardController.cloudProviderAccountId = $scope.wizardController.cloudProviderAccount.publicId;
                    $scope.wizardController.selectedAzureSubscription = $scope.dialogScope.currentAzureSubscription;
                    const subscriptionId = 'subscriptionId=' + $scope.wizardController.selectedAzureSubscription.subscriptionId;
                    $scope.getRegions(subscriptionId, accountChanged || $scope.dialogScope.isAzureSubscriptionChanged);
                    pidgeon.clear();
                });
        };

        $scope.getRegions = function getRegions(providerAccountIdParam, accountChanged) {
            MetadataResource.getRegions(nocHttp, $scope.wizardController.pathPrefix, null, providerAccountIdParam, $scope.wizardController.cloudProviderAccountId)
                .then((regions) => {
                    $scope.regions = regions;
                    const occmRegionCode = _.get($rootScope, 'instancePlacement.region');
                    const region = _.find(regions, (region) => region.code === occmRegionCode || region.name === occmRegionCode) || regions[0];

                    $scope.handleRegionChange(region, accountChanged);
                });
        };

        $scope.handleRegionChange = (newRegion, accountChanged) => {
            if (!$scope.regions || !newRegion) return;
            $scope.selectedRegionObject = newRegion;
            $scope.wizardController.selectedRegion = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS' ? newRegion.code : newRegion.name;
            if (($scope.wizardController.selectedRegion === $scope.lastSelectedRegion) && !accountChanged) return;
            $scope.lastSelectedRegion = $scope.wizardController.selectedRegion;
            $scope.discoveredWorkingEnvironments = [];
            $scope.wizardController.selectedWorkingEnvironment = null;
            const pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix($scope.wizardController.workingEnvironmentTypeAndProvider.type, $scope.wizardController.workingEnvironmentTypeAndProvider.providerName);
            let providerParams = {};
            if (_.get($scope, 'wizardController.selectedAzureSubscription.subscriptionId')) {
                providerParams = {subscriptionId: _.get($scope, 'wizardController.selectedAzureSubscription.subscriptionId')};
            }
            providerParams.region = $scope.lastSelectedRegion;
            providerParams.cloudProviderAccountId = $scope.wizardController.cloudProviderAccountId;

            let haProviderPath = null;
            switch ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName) {
                case 'Azure':
                    haProviderPath = '/azure/ha/';
                    break;
                case 'AWS':
                    haProviderPath = '/aws/ha/';
                    break;
                default:
                    haProviderPath = null;
                    break;
            }

            const haDiscoveryFunc = haProviderPath ? () => {
                return nocHttp.get({
                    url: haProviderPath + 'discovery/discover',
                    params: providerParams
                }, 'discovering HA working environments', nocHttp.LOADING)
            } : () => $q.resolve([]);

            return $q.all([
                nocHttp.get({
                    url: pathPrefix + 'discover',
                    params: providerParams
                }, 'discovering working environments', nocHttp.LOADING),
                haDiscoveryFunc()
            ])
                .then((discoveryResponses) => {
                    const workingEnvironments = _.map(discoveryResponses[0], (we) => {
                        we.type = 'Cloud Volumes ONTAP';
                        we._availabilityZones = we.availabilityZone ? [we.availabilityZone] : [];
                        we._subnetIds = we.subnetId ? [we.subnetId] : [];

                        return we;
                    })
                        .concat(_.map(discoveryResponses[1], (we) => {
                            we.type = 'HA Cloud Volumes ONTAP';
                            we._availabilityZones = we.availabilityZones || [];
                            we._subnetIds = we.subnetIds || [];

                            return we;
                        }));

                    $scope.discoveredWorkingEnvironments = workingEnvironments;
                    $scope.gridOptions.data = workingEnvironments;
                    $scope.fetched = true;
                });
        };

        $scope.handleSelectWorkingEnvironment = (workingEnvironment) => {
            if (workingEnvironment && !workingEnvironment.recoverable) return;

            const wasChanged = workingEnvironment !== $scope.wizardController.selectedWorkingEnvironment;
            $scope.wizardController.selectedWorkingEnvironment = workingEnvironment;
            if (!$scope.isValid()) {
                nocUI.handleFormValidity();
            } else if (!$scope.wizardController.selectedWorkingEnvironment) {
                nocDialog.alert('Cloud Volumes ONTAP must be selected in order to proceed');
            } else {
                if (wasChanged) {
                    //Bug fix NOC-5124 Discovery - When choosing different VSA to discover wizard keep the previous user/password -- need to clear the user name and password when choosing another VSA
                    //must be handled manually since the controller of the next step is not actually destroyed*/
                    $scope.clearAllStepData();
                    $scope.wizardController.svmPassword = '';
                }
                $state.go('.credentials');
            }
        };

        $scope.getTableHeight = () => {
            return {height: $scope.gridOptions.data.length * 58 + 58};
        };

        const generateGridOptions = function generateGridOptions() {

            let colums = [];
            switch ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName) {
                case 'AWS':
                    colums = generateAwsColumnsDefs();
                    break;
                case 'Azure':
                    colums = generateAzureColumnsDefs();
                    break;
                case 'GCP':
                    colums = generateGoogleColumnsDefs();
                    break;
                default:
                    break;
            }

            return {
                noUnselect: true,
                enableRowSelection: true,
                enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
                rowTemplate: '<div title="{{row.entity.recoverable ? \'\' : \'Working environment cannot be discovered.\'}}" ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.uid" class="ui-grid-cell" ng-class="{ \'ui-grid-row-header-cell\': col.isRowHeader, \'disabled-row\': !row.entity.recoverable, \'selectable-row\': row.entity.discoverable}" ng-click="grid.appScope.onClick(row)" ui-grid-cell></div>',
                isRowSelectable: (row) => row.entity.recoverable,
                appScopeProvider: {
                    onClick: (row) => {
                        if (row.isSelected) {
                            if ($scope.wizardController.currentStateName !== 'region') {
                                $scope.wizardController.setWizardStep('wizard.init.choose.region');
                            }
                            $timeout(() => {
                                $scope.handleSelectWorkingEnvironment(row.entity);
                            });
                        }
                    }
                },
                columnDefs: colums
            };
        };

        function generateAwsColumnsDefs() {

            const cellTemplateVPC = '<div class="ui-grid-cell-contents">{{COL_FIELD ? COL_FIELD : row.entity.vpcId}}</div>';

            return [
                {field: 'name', visible: true},
                {field: 'vpcName', displayName: 'VPC Name', visible: true, cellTemplate: cellTemplateVPC},
                {
                    field: '_availabilityZones',
                    displayName: 'Availability Zone',
                    visible: true,
                    cellFilter: 'availabilityZonesFormat'
                },
                {field: '_subnetIds', displayName: 'Subnet Id', visible: true, cellFilter: 'subnetIdFormat'},
                {field: 'stackName', displayName: 'Cloud Formation Name', visible: true},
                {field: 'clusterAddress', displayName: 'Cluster Address', visible: true},
                {field: 'type', displayName: 'Type', visible: true}
            ];
        }

        function generateAzureColumnsDefs() {
            return [
                {field: 'name', visible: true},
                {field: 'vnetName', displayName: 'VNet Name', visible: true},
                {field: 'subnetName', displayName: 'Subnet Name', visible: true},
                {field: 'resourceGroup', displayName: 'Resource Group', visible: true},
                {field: 'clusterAddress', displayName: 'Cluster Address', visible: true},
                {field: 'type', displayName: 'Type', visible: true}
            ];
        }

        function generateGoogleColumnsDefs() {
            return [
                {field: 'name', visible: true},
                {field: 'projectName', displayName: 'Project', visible: true},
                {field: 'zone', displayName: 'Zone', visible: true},
                {field: 'vnetName', displayName: 'VPC Name', visible: true},
                {field: 'clusterAddress', displayName: 'Cluster Address', visible: true},
                {field: 'type', displayName: 'Type', visible: true}
            ];
        }

        $scope.gridOptions = generateGridOptions();

        $scope.$watch('wizardController.workingEnvironmentTypeAndProvider.providerName', () => {
            $scope.gridOptions = generateGridOptions();
        });

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('region', {
                selectedRegion: $scope.wizardController.selectedRegion,
                selectedWorkingEnvironment: $scope.wizardController.selectedWorkingEnvironment,
                selectedAzureSubscription: $scope.wizardController.selectedAzureSubscription,
                cloudProviderAccount: $scope.wizardController.cloudProviderAccount
            });
            $scope.wizardController.selectedRegion = '';
            $scope.wizardController.selectedWorkingEnvironment = null;
            $scope.wizardController.selectedAzureSubscription = null;
            $scope.wizardController.cloudProviderAccount = null;
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('region'));

        //We should initialize the controller if there is a change in the choice to rollback to this step, this is the only way to do it in the first step...
        $scope.$watch('wizardController.choice', () => {
            if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName !== $scope.lastCloudProvider) {
                $scope.initialize();
            }
        });
    });
})();
