(() => {
    'use strict';

    angular.module('noc').controller('SnapMirrorDestinationVolumeController', ($scope, nocHttp, pidgeon, nocUI, $state, $timeout,
                                                                               BaseCreateVolume, nocAdvancedAllocationService) => {
        BaseCreateVolume.call($scope);

        function autoSelectAggregateIfOnlyOne() {
            if ($scope.targetWorkingEnvironment.workingEnvironmentType === 'ON_PREM' && $scope.originalAggregates.length === 1) {
                $scope.wizardController.selectedDestinationAggregateName = $scope.originalAggregates[0].name;
            }
        }

        function autoSelectSvmIfOnlyOne() {
            if ($scope.destinationSvms.length === 1) {
                $scope.wizardController.destinationSvmName = $scope.destinationSvms[0].name;
            }
        }

        $scope.initialize = function initialize() {
            $scope.wizardController.selectedDestinationAggregateName = '';
            nocAdvancedAllocationService.getAggregates(nocHttp, pidgeon, $scope.wizardController.targetWorkingEnvironment)
                .then((aggregates) => {
                    $scope.originalAggregates = aggregates;
                    _.forEach($scope.originalAggregates, (aggregate) => {
                        aggregate.value = aggregate.name;
                        aggregate._tier = aggregate._providerVolumeType + (aggregate.capacityTier ? '+' + aggregate.capacityTier : '');
                    });
                    if ($scope.targetWorkingEnvironment.workingEnvironmentType === 'VSA') {
                        $scope.possibleTiers = _.pickBy($scope.targetWorkingEnvironment._providerVolumeTypes, (tier) => {
                            return tier.diskName !== 'io1';
                        });

                        const selectedDestinationDiskType = _.includes($scope.wizardController.targetWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.diskType, 'io1') ? 'gp2' : $scope.wizardController.targetWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.diskType;
                        $scope.wizardController.selectedTier = $scope.possibleTiers[selectedDestinationDiskType];

                        $scope.moreInfoParameters = {
                            provider: $scope.wizardController.targetWorkingEnvironment.cloudProviderName === 'Amazon' ? 'AWS' : $scope.wizardController.targetWorkingEnvironment.cloudProviderName,
                            objectStorage: $scope.wizardController.targetWorkingEnvironment._objectStorageDisplayName
                        };

                        $scope.wizardController.forceCompositeVersion = $scope.targetWorkingEnvironment.capacityFeatures.supportedCapacityTiers.forceCompositeVersion;
                        $scope.wizardController.compositeSupported = $scope.targetWorkingEnvironment.capacityFeatures.supportedCapacityTiers.compositeSupported;
                        $scope.wizardController.capacityTiersDisableReasons = _.get($scope.targetWorkingEnvironment, 'capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons');
                        $scope.wizardController.targetProviderCapacityTierName = $scope.wizardController.targetWorkingEnvironment._objectStorageDisplayName;

                        $scope.wizardController.tieringEnabled = $scope.selectCapacityTierRadioDefault();

                        $scope.initializeAggregatesForDisplay();
                    }
                    autoSelectAggregateIfOnlyOne();
                    $scope.initDestinationVolumeIcon();
                });
            if ($scope.targetWorkingEnvironment.workingEnvironmentType === 'ON_PREM') {
                nocAdvancedAllocationService.getSVMs(nocHttp, $scope.wizardController.targetWorkingEnvironment, $scope)
                    .then(autoSelectSvmIfOnlyOne);
            }
            function descriptionForCapacityTier(diskName) {
                return 'Reduces storage costs by storing the data in an S3 capacity tier. Data moves to the ' + diskName + ' performance tier if it is read.';
            }

            _.forEach($scope.targetWorkingEnvironment._providerVolumeTypes, (currentVolume) => {
                if (_.includes(currentVolume.supportedCapacityTiers, $scope.wizardController.targetProviderCapacityTierName)) {
                    const diskNameDisplay = currentVolume.display.substring(0, currentVolume.display.indexOf('(') - 1);
                    currentVolume.description = descriptionForCapacityTier(diskNameDisplay);
                }
            });
        };

        $scope.selectCapacityTierRadioDefault = () => {
            if (!$scope.wizardController.compositeSupported ||
                $scope.wizardController.encryptionType === 'ONTAP' ||
                !_.includes($scope.wizardController.selectedTier.supportedCapacityTiers, $scope.wizardController.targetProviderCapacityTierName) ||
                ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0)) return false;

            return $scope.wizardController.forceCompositeVersion;
        };

        $scope.initializeAggregatesForDisplay = function initializeAggregatesForDisplay() {
            const tieringEnabled = $scope.wizardController.tieringEnabled;
            $scope.tierName = $scope.wizardController.selectedTier.name + (tieringEnabled ? '+' + $scope.wizardController.targetProviderCapacityTierName : '');
            const noAggrWithSelectedDiskTypeExist = _.every($scope.originalAggregates, (aggregate) => {
                return !_.includes(aggregate._tier, $scope.tierName);
            });

            //This ugly code is the only way I found to make the aggregate selection update upon tier change:
            $scope.aggregatesWithStaticOption = null;
            $scope.wizardController.selectedDestinationAggregateName = null;
            /*istanbul ignore next*/
            $timeout(() => {
                //Add static option:
                $scope.wizardController.selectedDestinationAggregateName = '';
                $scope.aggregatesWithStaticOption = _.cloneDeep($scope.originalAggregates);
                $scope.aggregatesWithStaticOption.unshift(noAggrWithSelectedDiskTypeExist ? {
                    name: 'Create new aggregate of tier ' + $scope.tierName,
                    value: ''
                } : {
                    name: 'Automatically select the best aggregate',
                    value: ''
                });
            });
        };

        $scope.initDestinationVolumeIcon = function initDestinationVolumeIcon() {
            const selectedCapacityTier = _.get($scope.wizardController.selectedTier, 'shortName') + (($scope.wizardController.wizardController.tieringEnabled && $scope.targetWorkingEnvironment._isAWS) ? '+' + $scope.wizardController.targetProviderCapacityTierName : '');
            $scope.wizardController._destinationIconName = $scope.wizardController.selectedTier ? 'volume-' + _(selectedCapacityTier).toLower().replace('_', '-') : 'resource-volume'; //eslint-disable-line newline-per-chained-call
        };

        $scope.disableAggregateChoice = (aggregate) => {
            if ($scope.wizardController.selectedVolume.thinProvisioning && !$scope.wizardController.tieringEnabled) {
                //Despite disabling tiering, thin volumes can be created on composite aggregates with tiering policy 'none'.
                return aggregate.value && (aggregate.state !== 'online' ||
                    aggregate._providerVolumeType !== $scope.wizardController.selectedTier.name);
            } else {
                return aggregate.value && (aggregate.state !== 'online' ||
                    aggregate._tier !== $scope.wizardController.selectedTier.name + ($scope.wizardController.tieringEnabled ? ($scope.targetWorkingEnvironment._isAWS ? '+S3' : '+Blob') : ''));
            }
        };

        $scope.tieringEnabledChanged = () => {
            $scope.initializeAggregatesForDisplay();
            $scope.initDestinationVolumeIcon();
        };

        $scope.tierSelected = function tierSelected(tier) {
            $scope.wizardController.selectedTier = tier;
            if (!_.includes($scope.wizardController.selectedTier.supportedCapacityTiers, $scope.wizardController.targetProviderCapacityTierName)) {
                $scope.wizardController.tieringEnabled = false;
            }
            $scope.setWizardStep($scope.wizardController.path.destinationVolume);
            $scope.initializeAggregatesForDisplay();
            $scope.initDestinationVolumeIcon();
        };

        $scope.tieringCheckboxSelected = function tieringCheckboxSelected() {
            $scope.initializeAggregatesForDisplay();
            $scope.initDestinationVolumeIcon();
        };

        $scope.disabledTooltipForTier = function disabledTooltipForTier(tier) {
            if (tier && tier.disabledMessage) {
                return tier.disabledMessage;
            }
            if (!_.includes(_.get(tier, 'supportedCapacityTiers'), $scope.wizardController.targetProviderCapacityTierName)) {
                return $scope.wizardController.targetProviderCapacityTierName + ' tiering is not available for this disk type.';
            }
        };

        $scope.prepareRequest = function prepareRequest() {
            $scope.volumeRequest = {
                workingEnvironmentId: $scope.targetWorkingEnvironment.publicId,
                svmName: $scope.targetWorkingEnvironment.svmName,
                name: $scope.wizardController.targetVolumeName,
                size: $scope.wizardController.selectedVolume.size,
                initialSize: $scope.wizardController.selectedVolume.usedSize,
                snapshotPolicyName: 'none',
                providerVolumeType: $scope.wizardController.selectedTier.diskName,
                capacityTier: $scope.wizardController.tieringEnabled ? $scope.wizardController.targetProviderCapacityTierName : undefined,
                isReplicationFlow: true,
                exportPolicyInfo: {
                    policyType: 'none',
                    ips: []
                },
                usageProfile: {
                    flags: {
                        enableThinProvisioning: $scope.wizardController.tieringEnabled ? true : $scope.wizardController.selectedVolume.thinProvisioning,
                        enableDeduplication: false,
                        enableCompression: false
                    }
                }
            };
        };

        $scope.askForQuote = function askForQuote() {
            $scope.prepareRequest();
            if ($scope.wizardController.selectedDestinationAggregateName) {
                $scope.volumeRequest.aggregateName = $scope.wizardController.selectedDestinationAggregateName;
            }

            return nocHttp.post({
                url: $scope.targetWorkingEnvironment._pathPrefix + 'volumes/quote',
                data: $scope.generateRequestBody()
            }, 'creating destination volume', nocHttp.LOADING);
        };

        $scope.next = function next() {
            const go = () => {
                $state.go('.maxTransferRate');
            };

            if (!$scope.isValid()) {
                nocUI.handleErrorsInWizard(true); //Parameter 'true' makes the screen scroll to the first error.
            } else if ($scope.targetWorkingEnvironment.workingEnvironmentType === 'VSA') {
                $scope.askForQuote()
                    .then((quoteResponse) => {
                        $scope.wizardController.quoteResponse = quoteResponse;
                        $scope.wizardController.volumeRequest = $scope.volumeRequest;
                        go();
                    });
            } else { //Destination type is ON_PREM
                go();
            }
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.advancedMode = undefined;
            $scope.wizardController.selectedDestinationAggregateName = undefined;
            $scope.wizardController.destinationSvmName = undefined;
        });

        $scope.initialize();
    });
})();
