(() => {
    'use strict';

    angular.module('noc').controller('SnapMirrorReviewController', ($scope, nocUI, pidgeon, nocHttp, BaseVirginStep, nocDialog,
                                                                    nocApprovalMessageBuilder, Constants, providerDiskTypes) => {
        BaseVirginStep.call($scope);
        $scope.diskTypes = providerDiskTypes[$scope.targetWorkingEnvironment._cloudProviderName];

        $scope.showPopUp = function showPopUp() {
            const quoteResponseForView = _.assignIn(_.cloneDeep($scope.wizardController.quoteResponse), {
                provider: $scope.targetWorkingEnvironment._cloudProviderName,
                isHA: $scope.targetWorkingEnvironment.isHA,
                diskType: $scope.wizardController.selectedTier.display
            });
            if ($scope.targetWorkingEnvironment.isHA) quoteResponseForView.numOfDisks /= 2;
            nocDialog
                .confirm('', 'assets/templates/dialogs/add-volume-approval-content.html', {
                    html: nocApprovalMessageBuilder.buildTitle(quoteResponseForView),
                    error: quoteResponseForView,
                    volumeName: $scope.wizardController.volumeRequest.name,
                    hideHint: true,
                    show: true,
                    okButtonOnly: true
                });
        };

        $scope.buildSnapMirrorRequest = () => {
            const replicationVolume = {
                sourceSvmName: $scope.wizardController.selectedVolume.svmName,
                sourceVolumeName: $scope.wizardController.selectedVolume.name,
                destinationVolumeName: $scope.wizardController.targetVolumeName
            };

            if ($scope.targetWorkingEnvironment.workingEnvironmentType === 'VSA') {
                replicationVolume.destinationProviderVolumeType = $scope.wizardController.selectedTier.diskName;
                replicationVolume.destinationCapacityTier = $scope.wizardController.tieringEnabled ? $scope.wizardController.targetProviderCapacityTierName : undefined;
                replicationVolume.destinationAggregateName = $scope.wizardController.quoteResponse.aggregateName;
                replicationVolume.numOfDisksApprovedToAdd = $scope.wizardController.quoteResponse.numOfDisks;
                replicationVolume.advancedMode = $scope.wizardController.advancedMode && !_.isEmpty($scope.wizardController.selectedDestinationAggregateName);
            } else if ($scope.targetWorkingEnvironment.workingEnvironmentType === 'ON_PREM') {
                replicationVolume.destinationAggregateName = $scope.wizardController.selectedDestinationAggregateName;
                replicationVolume.destinationSvmName = $scope.wizardController.destinationSvmName;
            }

            return {
                'replicationRequest': {
                    sourceWorkingEnvironmentId: $scope.wizardController.sourceWorkingEnvironment.publicId,
                    destinationWorkingEnvironmentId: $scope.wizardController.targetWorkingEnvironment.publicId,
                    sourceInterclusterLifIps: $scope.wizardController.selectedLifsAddresses.sourceLifs,
                    destinationInterclusterLifIps: $scope.wizardController.selectedLifsAddresses.destinationLifs,
                    policyName: $scope.wizardController.policy.name,
                    scheduleName: $scope.wizardController.selectedSchedule.name === Constants.emptyReplicationScheduleDescription ? null : $scope.wizardController.selectedSchedule.name,
                    maxTransferRate: $scope.wizardController.maxTransferRate
                },
                replicationVolume
            };
        };

        $scope.approveAndSnapMirror = () => {
            const url = '/replication/' + ($scope.targetWorkingEnvironment.workingEnvironmentType === 'VSA' ? 'vsa' : 'onprem');
            if ($scope.isValid()) {
                nocHttp.post({
                    url,
                    data: $scope.buildSnapMirrorRequest()
                }, 'replicating', nocHttp.BLOCK)
                    .then(() => {
                        pidgeon.inProgress('Replication started.', true);
                        $scope.setPathToPreviousOrRoot();
                    });
            } else {
                nocUI.handleErrorsInWizard(true);
            }
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.agreeResources = null;
        });
    });
})();
