(() => {
    'use strict';

    angular.module('noc').factory('CifsResource', () => {
        const normalizeResponse = function normalizeResponse(response) {
            const normalize = function normalize(responseItem) {
                responseItem.ip1 = _.isArray(responseItem.ipAddresses) && responseItem.ipAddresses[0];
                responseItem.ip2 = _.isArray(responseItem.ipAddresses) && responseItem.ipAddresses[1];
                delete responseItem.ipAddresses;
            };

            _.forEach(response, normalize);

            return response;
        };

        function CifsResource(publicId, we, svmName, nocHttp) {
            if (!publicId) throw new Error('Invalid publicId');
            this.svmName = svmName;
            this.url = we._pathPrefix + 'working-environments/' + publicId + '/cifs';
            this.nocHttp = nocHttp;
            this.pathPrefix = we._pathPrefix + 'working-environments/' + publicId;
        }

        // GET return an array of configurations, the array is empty if nothing is configured
        CifsResource.prototype.getCifsConfigurations = function getCifsConfigurations() {
            const url = this.svmName ? this.url + '?vserver=' + this.svmName : this.url;

            return this.nocHttp.get(url, 'retrieving CIFS configurations', this.nocHttp.LOADING)
                .then(normalizeResponse);
        };

        CifsResource.prototype.createCifsConfigurations = function createCifsConfigurations(body) {
            return this.nocHttp.post({url: this.url, data: body}, 'creating CIFS configurations', this.nocHttp.BLOCK);
        };

        CifsResource.prototype.createNTPConfigurations = function createNTPConfigurations(body) {
            const u = this.pathPrefix + '/ntp';

            return this.nocHttp.post({url: u, data: body}, 'creating NTP configurations', this.nocHttp.BLOCK).catch((e) => {
                return e;
            });
        };

        return CifsResource;
    });
})();
