(() => {
    'use strict';

    angular.module('noc').factory('KubernetesResource', () => {
        const resource = {};

        resource.removeK8sConfiguration = (nocHttp, k8sClusterId) => {
            return nocHttp.delete({
                url: '/k8s/' + k8sClusterId
            }, 'removing Kubernetes configuration', nocHttp.BLOCK);
        };

        resource.connectWorkingEnvironment = (nocHttp, publicId, k8sClusterId, data) => {
            return nocHttp.post({
                url: 'k8s/connect/' + k8sClusterId + '/' + publicId,
                data
            }, 'connecting Cloud Volumes ONTAP to Kubernetes cluster', nocHttp.BLOCK);
        };

        resource.disconnectWorkingEnvironment = (nocHttp, publicId, k8sClusterId) => {
            return nocHttp.delete({
                url: 'k8s/disconnect/' + k8sClusterId + '/' + publicId
            }, 'disconnecting Cloud Volumes ONTAP from Kubernetes cluster', nocHttp.BLOCK);
        };

        resource.getK8sClusters = (nocHttp) => {
            return nocHttp.get('/k8s/clusters', 'retrieving Kubernetes clusters', nocHttp.BLOCK);
        };

        resource.getExternalClusters = (nocHttp, token) => {
            return nocHttp.get({
                url: '/k8s/external-clusters',
                headers: {'Api-token': token}
                }
                , 'retrieving external Kubernetes clusters', nocHttp.LOADING);
        };

        resource.saveK8sClusters = (nocHttp, data) => {
            return nocHttp.post({
                url: 'k8s/save/',
                data
            }, 'saving Kubernetes cluster', nocHttp.BLOCK);
        };

        return resource;
    });
})();
