(function () {
    "use strict";
    angular.module("noc").factory("volViewGeneralResource", function () {
        var resource = {},
            VOLUME_VIEW_URL = '/managed/aws/vsa/volumes',
            ONTAP_CLOUD_PASSWORD_URL = VOLUME_VIEW_URL + '/ontap-cloud-password',
            AWS_TAGS_URL =             VOLUME_VIEW_URL + "/aws-tags",
            NSS_KEYS_URL =             VOLUME_VIEW_URL + "/nss-keys";

        resource.getOntapCloudPassword = function (nocHttp) {
            return nocHttp.get(ONTAP_CLOUD_PASSWORD_URL, "retrieving Cloud Volumes ONTAP password", nocHttp.LOADING)
                .then(function (response) {
                    return response.exists;
                });
        };

        resource.setOntapCloudPassword = function (nocHttp, password) {
            return nocHttp.put({
                url: ONTAP_CLOUD_PASSWORD_URL,
                data: {"ontapCloudPassword": password}
            }, "setting Cloud Volumes ONTAP password", nocHttp.LOADING);
        };

        resource.getAWSTags = function(nocHttp) {
            return nocHttp.get(AWS_TAGS_URL, 'retrieving list of AWS tags', nocHttp.LOADING);
        };

        resource.setAWSTags = function(nocHttp, awsTags) {
            return nocHttp.put({
                url: AWS_TAGS_URL,
                data: {"tags": awsTags}
            }, "setting AWS tags", nocHttp.LOADING);
        };

        resource.volViewUpdateUserDetails = function (nocHttp, firstName, lastName, email, ldapUserName) {
            var updateUserData = {
                firstName: firstName,
                lastName: lastName,
                email: email
            };
            if (ldapUserName) {
                updateUserData.ldapUserName = ldapUserName;
            }
            return nocHttp.put({
                url: VOLUME_VIEW_URL + '/update-user',
                data: updateUserData
            }, 'updating user details', nocHttp.BLOCK);
        };

        resource.getNssUserName = function(nocHttp){
            return nocHttp.get(NSS_KEYS_URL, "retrieving NSS credentials", nocHttp.LOADING)
                .then(function(response){
                    return response.nssUserName;
                });
        };

        resource.registerVolumesToNss = function (nocHttp, nssUserName, nssPassword) {
            return nocHttp.put({
                url: NSS_KEYS_URL,
                data: {
                    "nssUserName": nssUserName,
                    "nssPassword": nssPassword
                }
            }, null, nocHttp.LOADING);
        };

        resource.getActionsRequired = function (nocHttp) {
            return nocHttp.get(VOLUME_VIEW_URL + '/actions-required', "retrieving actions required");
        };

        return resource;
    });
})();