(function () {
    "use strict";
    angular.module("noc").factory("volViewInstancesResource", function (nocPoller, nocUtil, nocRegionsService) {
        var resource = {},
            VOLUME_VIEW_INSTANCES_URL = '/managed/aws/vsa/instances/';

        resource.startResourcesPolling = function (successFn) {
            return nocPoller.start(VOLUME_VIEW_INSTANCES_URL, 30 * 1000, successFn, _.noop, _.noop, true);
        };

        resource.getInstancesById = function (nocHttp, id) {
            return nocHttp.get(VOLUME_VIEW_INSTANCES_URL + id, "retrieving instances")
                .then(function (instances) {
                    return _.map(instances, resource.initInstance);
                });
        };

        var statusColors = {
            on: "green",
            initializing: 'blue',
            starting: 'blue',
            stopping: 'red',
            updating: 'blue',
            off: 'red',
            failed: 'red',
            "On - In Takeover": 'green'
        };

        function instanceLocation(instance) {
            if (!instance.region || !instance.availabilityZone) return '';
            var region = nocRegionsService.awsRegions()[instance.region];
            if (!region) return '';
            var availabilityZone = _.split(instance.availabilityZone, '-').pop();
            return region.name + ' ' + availabilityZone + ' (' + region.location + ') | ' + instance.subnetCidr;
        }

        resource.initInstance = function (instance, index, allInstancesInWe) {
            instance._displayName = instance.name + ' (' + instance.instanceId + ')';
            instance._location = instanceLocation(instance);
            instance.status = _(instance.status).toLower().replace(/_/g, ' ');
            instance._displayStatus = instance.status === 'degraded in takeover' ? 'On - In Takeover' : _.replace(instance.status, 'degraded ', ''); //remove "degraded" prefix
            instance._statusColor = statusColors[instance._displayStatus];
            instance._ebsVolumes = _(instance.ebsVolumes).groupBy('volumeType').map(function (arr, key) {
                return {
                    ebsVolumeType: key,
                    quantity: _.sumBy(arr, 'numOfVolumes'),
                    diskSizes: _(arr).sortBy('sizeInBytes', function (ebsVolume) {
                        return nocUtil.convertTo(ebsVolume.size, "Byte").size;
                    }).map(function (n) {
                        return n.numOfVolumes + " x " + n.size.size + ' ' + n.size.unit;
                    }).join(" | ")
                };
            }).value();
            instance._ebsVolumeTotalCount = _.sumBy(instance._ebsVolumes, 'quantity');
            instance._awsTags = _.sortBy(_.map(instance.awsTags, function (value, key) {
                return {value: value, key: key};
            }), "key");
            instance._awsTagsCount = _.size(instance.awsTags);
            instance._isHa = allInstancesInWe && allInstancesInWe.length === 3;
            instance._isDegraded = _.startsWith(instance.status, 'degraded');
            instance._menuActive = instance.status === "on" || instance.status === "off" || instance._isDegraded;
            instance._instancesInCluster = _.map(allInstancesInWe, 'name');

            return instance;
        };

        resource.deleteWorkingEnvironment = function (nocHttp, weId) {
            return nocHttp.delete(VOLUME_VIEW_INSTANCES_URL + weId, 'deleting provider instances', nocHttp.LOADING);
        };

        resource.getTargetInstanceTypes = function (nocHttp, instance) {
            return nocHttp.get(VOLUME_VIEW_INSTANCES_URL + instance.workingEnvironmentId + "/target-instance-types",
                "retrieving target instance types", nocHttp.LOADING);
        };

        resource.changeInstanceType = function (nocHttp, instance, newInstanceType) {
            return nocHttp.put({
                url: VOLUME_VIEW_INSTANCES_URL + instance.workingEnvironmentId + "/instance-type",
                data: newInstanceType
            }, "changing instance type", nocHttp.LOADING);
        };

        resource.startWorkingEnvironment = function startWorkingEnvironment(nocHttp, workingEnvironmentId) {
            return nocHttp.post(VOLUME_VIEW_INSTANCES_URL + workingEnvironmentId + '/start', 'starting Cloud Volumes ONTAP', nocHttp.BLOCK);
        };

        resource.stopWorkingEnvironment = function stopWorkingEnvironment(nocHttp, workingEnvironmentId) {
            return nocHttp.post(VOLUME_VIEW_INSTANCES_URL + workingEnvironmentId + '/stop', 'stopping Cloud Volumes ONTAP', nocHttp.BLOCK);
        };

        return resource;
    });
})();