(() => {
    'use strict';

    angular.module('noc').factory('workingEnvironmentResource', (nocPoller, $rootScope, nocUtil, workingEnvironmentStatuses,
                                                                 Constants, $filter, $state, providerDiskTypes,
                                                                 nocEncryptionResourcesService, $q, nocSelectedWorkspaceOrTenantService) => {
        const resource = {};

        resource.moment = moment;
        //===========================================================================================================================
        /*
         Working Environments

         */
        resource.WORKING_ENVIRONMENTS_GET_EVENT = 'workingEnvironmentsGetSuccess';
        resource.WORKING_ENVIRONMENTS_GET_ERROR_EVENT = 'workingEnvironmentsGetError';
        resource.WORKING_ENVIRONMENT_STATUS_UPDATE = 'workingEnvironmentStatusUpdate';

        resource.workingEnvironments = null;

        resource.updateWorkingEnvironmentStatus = (workingEnvironment, newStatus) => {
            let updateStatus = newStatus;
            if (_.isString(newStatus)) updateStatus = {status: newStatus, message: null};

            workingEnvironment.status = updateStatus;
            resource.initWorkingEnvironment(workingEnvironment);
            $rootScope.$broadcast(resource.WORKING_ENVIRONMENT_STATUS_UPDATE, {workingEnvironment});
        };

        function instanceDataFromAwsProperties(key, we) {
            const filteredInstances = _.filter(we.awsProperties.instances, (i) => {
                return !_.includes(i.name, '-mediator');
            });

            const currInstance = _.find(filteredInstances, {tags: {WorkingEnvironmentId: we.publicId}, isVsaInstance: true});

            return currInstance ? currInstance[key] : null;
        }

        function initProviderVolumes(provider, providerVolumesType, supportedCapacityTiers) {
            const diskTypes = _.cloneDeep(providerDiskTypes[provider]);

            _.forEach(diskTypes, (type) => {
                type.supportedSizes = _(providerVolumesType)
                    .filter((diskSizeObject) => {
                        return _.includes(diskSizeObject.supportedVolumeTypes, type.diskName);
                    })
                    //Add string identifier to 'size' object and pick important fields only
                    .map((diskSizeObject) => {
                        diskSizeObject.size._identifier = diskSizeObject.size.size + ' ' + diskSizeObject.size.unit;
                        diskSizeObject.size.maxDisksAllow = diskSizeObject.maxDisksAllow;

                        return diskSizeObject.size;
                    })
                    .value();

                type.disabledMessage = supportedCapacityTiers.capacityTiersDisableReasons.join(', ');
            });

            return _.pickBy(diskTypes, (diskType) => {
                return !_.isEmpty(diskType.supportedSizes);
            });
        }

        function basicInitWorkingEnvironment(workingEnvironment) {
            const typeAndProvider = {
                Amazon: 'VSA_AWS',
                Azure: 'VSA_Azure',
                GCP: 'VSA_GCP',
                default: 'ON_PREM'
            };
            workingEnvironment._typeAndProvider = typeAndProvider[workingEnvironment.cloudProviderName] || typeAndProvider.default;

            //In case of a dataONTAP working environment with location different than ON_PREM treat it as NPS. VSA working environments shouldn't have location. their providerName will be used instead
            workingEnvironment._workingEnvironmentLocation = workingEnvironment.workingEnvironmentType === 'ON_PREM' ? workingEnvironment.location : undefined;
            workingEnvironment._workingEnvironmentTypeByLocation = workingEnvironment.isHA ? 'HA' : workingEnvironment.workingEnvironmentType === 'ON_PREM' && workingEnvironment.location !== 'ON_PREM' ? 'NPS' : workingEnvironment.workingEnvironmentType;
            workingEnvironment._typeLongName = Constants.typeToLongNameMap[workingEnvironment.isHA ? 'HA' : workingEnvironment._workingEnvironmentTypeByLocation] || workingEnvironment.workingEnvironmentType;
            workingEnvironment._pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix(workingEnvironment._workingEnvironmentTypeByLocation, workingEnvironment.cloudProviderName);

            workingEnvironment._normalizedReservedSizeTB = nocUtil.convertTo(workingEnvironment.reservedSize, 'TB');
            workingEnvironment._normalizedReservedSize = _.get(workingEnvironment, 'reservedSize.size') > 0 ? $filter('normalizedCapacity')(workingEnvironment.reservedSize, 2, true, 'TB') : undefined;

            workingEnvironment._cloudProviderName = Constants.dataONTAPLocations[workingEnvironment._workingEnvironmentLocation] ?
                Constants.dataONTAPLocations[workingEnvironment._workingEnvironmentLocation].providerName :
                (workingEnvironment.cloudProviderName === 'Amazon' ? 'AWS' : workingEnvironment.cloudProviderName);
            workingEnvironment._isAWS = workingEnvironment._cloudProviderName === 'AWS';
            workingEnvironment._isAzure = workingEnvironment._cloudProviderName === 'Azure';
            workingEnvironment._isGCP = workingEnvironment._cloudProviderName === 'GCP';
            workingEnvironment._nameForInstance = workingEnvironment._isAWS ? 'instance' : (workingEnvironment._isAzure ? 'VM' : 'machine');
            workingEnvironment._hasEnabledSchedules = _.some(workingEnvironment.schedules, 'enabled');

            const objectStorageByProvider = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage',
                default: ''
            };
            workingEnvironment._objectStorageDisplayName = objectStorageByProvider[workingEnvironment.cloudProviderName] || objectStorageByProvider.default;
        }

        function initStatusField(workingEnvironment) {
            if (workingEnvironment.status && workingEnvironment.status.status) {
                workingEnvironment.status.status = nocUtil.normalizeStatus(workingEnvironment.status.status);
            } else {
                workingEnvironment.status = {status: 'unknown', message: null};
            }

            workingEnvironment._stateInfo = _.cloneDeep(workingEnvironmentStatuses.stateInfoMap[workingEnvironment.status.status]) || {};
            workingEnvironment._allowedOperations = workingEnvironmentStatuses.getAllowedOperations(workingEnvironment._typeAndProvider, workingEnvironment.status.status);
            workingEnvironment._displayStatus = (workingEnvironment.status.status === 'off' || workingEnvironment.status.status === 'unknown') ?
                workingEnvironment._stateInfo.title :
                workingEnvironment.status.status;
        }

        function initProviderProperties(workingEnvironment) {
            if (workingEnvironment.awsProperties || workingEnvironment.providerProperties) {
                workingEnvironment.regionName = _.get(workingEnvironment, 'awsProperties.regionName') || _.get(workingEnvironment, 'providerProperties.regionName');
            }

            workingEnvironment._regionName = $filter('regionFormat')(workingEnvironment.regionName);

            if (workingEnvironment.awsProperties) {
                if (workingEnvironment.awsProperties.vpc) {
                    workingEnvironment.vpcId = workingEnvironment.awsProperties.vpc.vpcId;
                    workingEnvironment.cidrBlock = workingEnvironment.awsProperties.vpc.cidrBlock;
                }
                workingEnvironment.regionName = workingEnvironment.awsProperties.regionName;
                workingEnvironment.instances = workingEnvironment.awsProperties.instances;
                workingEnvironment.availabilityZones = workingEnvironment.awsProperties.availabilityZones;

                workingEnvironment._instanceType = instanceDataFromAwsProperties('instanceType', workingEnvironment);
            }

            if (workingEnvironment.providerProperties) {
                workingEnvironment._instanceType = workingEnvironment.providerProperties.instanceType;
            }
        }

        function initReplicationProperties(workingEnvironment) {
            if (workingEnvironment.replicationProperties) {
                workingEnvironment.peers = workingEnvironment.replicationProperties.peers;
                workingEnvironment.replicationTargets = workingEnvironment.replicationProperties.replicationTargets;
            }

            //Extract the peer names of this working environment
            const peersNames = _.map(workingEnvironment.peers, (workingEnvironmentPublicId) => {
                return _.result(_.find(resource.workingEnvironments, {publicId: workingEnvironmentPublicId}), 'name');
            });

            //Now push the working environments this working environment appears as a peer (for cases this working environment is down)
            if (workingEnvironment.status.status !== 'on') {
                _.forEach(resource.workingEnvironments, (otherWorkingEnvironment) => {
                    if (_.includes(otherWorkingEnvironment.peers, workingEnvironment.publicId)) {
                        peersNames.push(otherWorkingEnvironment.name);
                    }
                });
            }
            workingEnvironment._peers = peersNames.join(', ');
        }

        function initOntapClusterProperties(workingEnvironment) {
            if (workingEnvironment.ontapClusterProperties) {
                const nodes = workingEnvironment.ontapClusterProperties.nodes;

                workingEnvironment._ontapVersion = workingEnvironment.ontapClusterProperties.ontapVersion;
                workingEnvironment._licenseType = _.get(workingEnvironment, 'ontapClusterProperties.licenseType.name', '');
                workingEnvironment._serialNumber = nocUtil.concatArrayPropertyData(nodes, 'serialNumber');
                workingEnvironment._writingSpeedState = _.result(workingEnvironment, 'ontapClusterProperties.writingSpeedState.toLowerCase');
                workingEnvironment._clusterManagementIP = _.get(workingEnvironment, 'ontapClusterProperties.nodes[0].lifs') ? _.find(workingEnvironment.ontapClusterProperties.nodes[0].lifs, {'lifType': 'Cluster Management'}).ip : _.get(workingEnvironment, 'ontapClusterProperties.clusterAddress');
                if (workingEnvironment._cloudProviderName === 'AWS' && _.get(workingEnvironment, 'awsProperties.accountId')) {
                    workingEnvironment._accountOrSubscriptionName = 'Account ID: ' + _.get(workingEnvironment, 'awsProperties.accountId');
                    workingEnvironment._cloudProviderAccountId = _.get(workingEnvironment, 'awsProperties.cloudProviderAccountId');
                } else if (workingEnvironment._cloudProviderName === 'Azure' && _.get(workingEnvironment, 'providerProperties.subscriptionName')) {
                    workingEnvironment._accountOrSubscriptionName = 'Subscription Name: ' + _.get(workingEnvironment, 'providerProperties.subscriptionName');
                    workingEnvironment._cloudProviderAccountId = _.get(workingEnvironment, 'providerProperties.cloudProviderAccountId');
                } else {
                    workingEnvironment._accountOrSubscriptionName = '';
                }
            }
        }

        function initCloudSyncProperties(workingEnvironment) {
            if (workingEnvironment.cloudSyncProperties) {
                workingEnvironment.cloudSyncProperties._failedRelationships = _.filter(workingEnvironment.cloudSyncProperties.relationships, {relationshipStatus: 'FAILED'}).length;
                workingEnvironment.cloudSyncProperties._displayAsFailed = workingEnvironment.cloudSyncProperties.isSynced &&
                    (workingEnvironment.cloudSyncProperties.status.status === 'FAILED' || workingEnvironment.cloudSyncProperties._failedRelationships);
            }
        }

        function initCapacityFeatures(workingEnvironment) {
            if (workingEnvironment.capacityFeatures && workingEnvironment.capacityFeatures.supportedCapacityTiers) {
                const _supportedCapacityTiers = {
                    supportedCapacityTiersPerVolumeType: _(workingEnvironment.capacityFeatures.supportedCapacityTiers.supportedCapacityTiersPerVolumeType)
                        .keyBy('volumeType')
                        .mapValues('supportedCapacityTiers')
                        .value(),
                    capacityTiersDisableReasons: workingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons
                };
                workingEnvironment._providerVolumeTypes = initProviderVolumes(workingEnvironment._cloudProviderName,
                    workingEnvironment.capacityFeatures.providerVolumesType,
                    _supportedCapacityTiers);
            }
        }

        function initK8sProperties(workingEnvironment) {
            if (workingEnvironment.k8sProperties) {
                workingEnvironment._connectedToK8s = _.get(workingEnvironment.k8sProperties, 'connectedClusterIds.length') > 0;
            }
        }

        function initEncryptionProperties(workingEnvironment) {
            function _getWorkingEnvironmentEncryptionType(workingEnvironment) {
                if (workingEnvironment.status.status === 'on' || workingEnvironment.status.status === 'off' || workingEnvironment.status.status === 'degraded') {
                    if (workingEnvironment.encryptionProperties && workingEnvironment.encryptionProperties.awsVolumeEncryption) {
                        return 'AWS';
                    } else if (workingEnvironment.encryptionProperties && workingEnvironment.encryptionProperties.ontapEncryption) {
                        return 'Cloud Volumes ONTAP';
                    } else if (workingEnvironment.encryptionProperties && workingEnvironment.encryptionProperties.azureVolumeEncryption) {
                        return 'Azure';
                    } else if (workingEnvironment.encryptionProperties && workingEnvironment.encryptionProperties.gcpVolumeEncryption) {
                        return 'GCP';
                    } else {
                        return 'None';
                    }
                } else {
                    return 'N/A';
                }
            }

            workingEnvironment._encryptionType = _getWorkingEnvironmentEncryptionType(workingEnvironment);
            if (workingEnvironment.encryptionProperties && workingEnvironment.encryptionProperties.awsEncryptionKey) {
                nocEncryptionResourcesService.formatAwsEncryptionKeyExpiration(workingEnvironment.encryptionProperties.awsEncryptionKey);
                workingEnvironment.encryptionProperties.awsEncryptionKey.status = _.startCase(workingEnvironment.encryptionProperties.awsEncryptionKey.status);
                workingEnvironment.encryptionProperties.awsEncryptionKey._isFromAnotherAccount = nocEncryptionResourcesService.isKeyFromAnotherAccount(workingEnvironment.encryptionProperties.awsEncryptionKey);
                workingEnvironment._awsEncryptionKey = workingEnvironment.encryptionProperties.awsEncryptionKey;
            }
        }

        resource.initWorkingEnvironment = function (workingEnvironment) {
            basicInitWorkingEnvironment(workingEnvironment);
            initStatusField(workingEnvironment);
            initProviderProperties(workingEnvironment); //Also initializes awsProperties
            initReplicationProperties(workingEnvironment);
            initOntapClusterProperties(workingEnvironment);
            initCloudSyncProperties(workingEnvironment);
            initCapacityFeatures(workingEnvironment);
            initEncryptionProperties(workingEnvironment);
            initK8sProperties(workingEnvironment);
        };

        const workingEnvironmentsPollingSuccessFunction = resource.workingEnvironmentsPollingSuccessFunction = function (workingEnvironments, status, headers, config) {
            resource.workingEnvironments = resource.processAggregateWorkingEnvironmentResponse(workingEnvironments);

            $rootScope.$broadcast(resource.WORKING_ENVIRONMENTS_GET_EVENT, {workingEnvironments: resource.workingEnvironments, tenantId: config.params ? config.params.tenantId : undefined});
        };

        const workingEnvironmentsPollingErrorFunction = resource.workingEnvironmentsPollingErrorFunction = function (error, status, headers, config) {
            $rootScope.$broadcast(resource.WORKING_ENVIRONMENTS_GET_ERROR_EVENT, {error, tenantId: config.params ? config.params.tenantId : undefined});
        };

        let workingEnvironmentsListenerCount = 0;
        let workingEnvironmentsPollStopper = null;

        resource.processAggregateWorkingEnvironmentResponse = function (aggregate) {
            const wes = [];

            _.forEach(aggregate.onPremWorkingEnvironments, (we) => {
                we.workingEnvironmentType = 'ON_PREM';
                resource.initWorkingEnvironment(we);
                wes.push(we);
            });

            _.forEach(aggregate.vsaWorkingEnvironments, (we) => {
                we.workingEnvironmentType = 'VSA';
                resource.initWorkingEnvironment(we);
                wes.push(we);
            });

            _.forEach(aggregate.azureVsaWorkingEnvironments, (we) => {
                we.workingEnvironmentType = 'VSA';
                resource.initWorkingEnvironment(we);
                wes.push(we);
            });

            _.forEach(aggregate.gcpVsaWorkingEnvironments, (we) => {
                we.workingEnvironmentType = 'VSA';
                resource.initWorkingEnvironment(we);
                wes.push(we);
            });

            return wes;
        };

        resource.getWorkingEnvironmentByPublicId = function getWorkingEnvironmentByPublicId(nocHttp, publicId, pathPrefix, fields) {
            const opts = {
                url: pathPrefix + 'working-environments/' + publicId,
                params: {}
            };

            if (fields) opts.params.fields = fields;

            return nocHttp.get(opts)
                .then((data) => {
                    const workingEnvironment = data;
                    resource.initWorkingEnvironment(workingEnvironment);

                    return workingEnvironment;
                });
        };

        resource.getWorkingEnvironments = function getWorkingEnvironments(nocHttp, tenantId, fields, isAddUser) {
            const opts = {
                url: '/working-environments',
                params: {}
            };

            if (!fields) fields = 'status';
            //for backwards comparability - add user
            if (isAddUser) opts.params.tenantId = tenantId;
            if (tenantId !== null) opts.params.tenantId = nocSelectedWorkspaceOrTenantService.getPublicId();
            if (tenantId) opts.params.tenantId = tenantId;
            if (fields) opts.params.fields = fields;
            return nocHttp.get(opts, 'retrieving working environments', nocHttp.LOADING)
                .then((data) => {
                    return resource.processAggregateWorkingEnvironmentResponse(data);
                });
        };

        resource.changeSerial = function changeSerial(nocHttp, workingEnvironment, node1, node2) {
            return nocHttp.post({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/change-serial',
                data: {
                    nodeOne: node1,
                    nodeTwo: node2
                }
            },
            'updating serial number', nocHttp.BLOCK);
        };

        resource.uploadLicenseFile = function uploadLicenseFile(nocHttp, workingEnvironment, license1, license2) {
            return nocHttp.post({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/upload-licenses',
                data: {
                    licenseOne: license1,
                    licenseTwo: license2
                }
            },
            'uploading Cloud license file', nocHttp.BLOCK);
        };

        resource.deleteWorkingEnvironment = function deleteWorkingEnvironment(nocHttp, workingEnvironment, forceDelete) {
            return nocHttp.delete({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId,
                params: {
                    localDelete: _.get(workingEnvironment, 'status.status') === 'failed-to-create',
                    forceDelete
                }
            }, 'deleting working environment', nocHttp.BLOCK)
                .then(() => {
                    if (workingEnvironment.workingEnvironmentType === 'ON_PREM') {
                        // OnPrem environment is deleted immediately, so there is no need to set status to
                        // "deleting" and display "deleting..." animation to the user.
                        resource.updateWorkingEnvironmentStatus(workingEnvironment, 'deleted');
                    } else {
                        resource.updateWorkingEnvironmentStatus(workingEnvironment, 'deleting');
                    }
                });
        };

        resource.startWorkingEnvironmentsPolling = function (tenantId, fields) {
            workingEnvironmentsListenerCount++;
            if (workingEnvironmentsListenerCount === 1) {
                const opts = {method: 'GET', url: '/working-environments', params: {}};
                if (!fields) fields = 'status';
                opts.params.tenantId = tenantId || nocSelectedWorkspaceOrTenantService.getPublicId();
                if (fields) opts.params.fields = fields;

                workingEnvironmentsPollStopper = nocPoller.start(opts, 1 * 60 * 1000, workingEnvironmentsPollingSuccessFunction, workingEnvironmentsPollingErrorFunction, null, true);
            }

            return _.once(() => {
                workingEnvironmentsListenerCount--;
                if (workingEnvironmentsListenerCount === 0) workingEnvironmentsPollStopper();
            });
        };
        //===========================================================================================================================
        /*
         VPCS
         */
        resource.getVpcOfWorkingEnvironment = function getVpcOfWorkingEnvironment(nocHttp, workingEnvironment) {
            const providerProperties = workingEnvironment._cloudProviderName === 'AWS' ? 'awsProperties' : 'providerProperties';

            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '?fields=' + providerProperties,
                'retrieving VPC information for VPC ' + workingEnvironment.vpcId);
        };

        //===========================================================================================================================
        /*
         Instances

         */
        resource.INSTANCES_GET_EVENT = 'instancesGetSuccess';
        resource.INSTANCES_GET_ERROR_EVENT = 'instancesGetError';

        resource.sortInstances = function sortInstances(instances) {
            const sortedInstances = nocUtil.sortByIp(instances, 'privateIpAddress');

            const occmInstance = _.filter(sortedInstances, {isOCCMInstance: true});
            const vsaInstances = _.filter(sortedInstances, {isVsaInstance: true, _isDataBroker: false});
            const dataBrokerInstances = _.filter(sortedInstances, {_isDataBroker: true});
            const otherInstances = _.filter(sortedInstances, {isVsaInstance: false, isOCCMInstance: false, _isDataBroker: false});

            return occmInstance.concat(vsaInstances, dataBrokerInstances, otherInstances);
        };

        resource.initInstances = function initInstances(instances) {
            _.forEach(instances, (instance) => {
                instance._isMediator = _.includes(instance.name, '-mediator');
                instance._isDataBroker = _.includes(instance.name, 'dataBroker');
            });

            return resource.sortInstances(instances);
        };

        resource.startInstancesPolling = function (workingEnvironment) {
            const providerProperties = workingEnvironment._cloudProviderName === 'AWS' ? 'awsProperties' : 'providerProperties';

            return nocPoller.start(
                {
                    method: 'GET',
                    url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '?fields=supportedFeatures,k8sProperties,' + providerProperties
                },
                5 * 60 * 1000,
                (workingEnvironment) => {
                    const event = {
                        workingEnvironment,
                        instances: resource.initInstances(workingEnvironment.awsProperties.instances)
                    };
                    $rootScope.$broadcast(resource.INSTANCES_GET_EVENT, event);
                },
                (error) => {
                    $rootScope.$broadcast(resource.INSTANCES_GET_ERROR_EVENT, {workingEnvironment, error});
                }
            );
        };

        resource.WE_STATUS_GET_EVENT = '-statusGetSuccess';
        resource.WE_STATUS_ERROR_EVENT = '-statusGetError';

        resource.startStatusPolling = (workingEnvironment) => {
            return nocPoller.start({
                method: 'GET',
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId,
                params: {fields: 'status'}
            }, 0.5 * 60 * 1000, (workingEnvironment) => {
                $rootScope.$broadcast(workingEnvironment.publicId + resource.WE_STATUS_GET_EVENT, workingEnvironment.status);
            }, (error, status) => {
                $rootScope.$broadcast(workingEnvironment.publicId + resource.WE_STATUS_ERROR_EVENT, {error, status});
            }, null, true);
        };

        //===========================================================================================================================
        /*
         VSA

         */
        resource.VSA_GET_EVENT = 'vsaGetSuccess';
        resource.VSA_GET_EVENT2 = 'vsaGetSuccessContinueProcess';
        resource.VSA_GET_ERROR_EVENT = 'vsaGetError';

        resource.fillLifsData = (nodesData) => {
            //Adding prefix node-[index] so it will be eaier to know which lif belongs to each node
            if (nodesData && nodesData.length > 1) {
                _.forEach(nodesData, (node, index) => {
                    _.forEach(node.lifs, (lif) => {
                        lif._ip = lif.ip;
                        lif.ip = (lif.lifType === 'Cluster Management' || lif.lifType === 'SVM Management') ? lif.ip : '<span class="node-ip-container"><span class="node-index">Node ' + (index + 1) + ': </span>' + lif.ip + '</span>';
                    });
                });
            }

            let lifs = [],
                allLifs = _.flatMap(nodesData, 'lifs'),
                typeGroups = _.groupBy(allLifs, (lif) => {
                    return lif.lifType + (lif.privateIp ? ' Private' : ' Public');
                });
            _.forOwn(typeGroups, (types, keyType) => {
                const protocolGroups = _.groupBy(types, 'dataProtocols');
                _.forOwn(protocolGroups, (protocol, keyProtocol) => {
                    const lifsData = {
                        key: keyType + '-' + keyProtocol,
                        ips: (_.map(protocol, 'ip')).join('<span class="node-index-barrier">|</span>'),
                        lifType: keyType.substring(0, keyType.lastIndexOf(' ')),
                        dataProtocols: protocol[0].dataProtocols,
                        privateIp: protocol[0].privateIp
                    };
                    lifs.push(lifsData);
                });
            });

            const svmManagement = _.find(lifs, {key: 'SVM Management Private-'});
            const clusterManagement = _.find(lifs, {key: 'Cluster Management Private-'});
            if (svmManagement) svmManagement.priority = 1;
            if (clusterManagement) clusterManagement.priority = 1;

            return _.sortBy(lifs, ['priority', 'lifType', 'ip']);
        };

        resource.workingEnvironmentResponseToVsaResponse = function (workingEnvironmentResponse) {
            const resolveFrom = function (container, key) {
                return workingEnvironmentResponse[container] ? workingEnvironmentResponse[container][key] : null;
            };

            function resolveInstanceDataFromAwsProperties(key) {
                return instanceDataFromAwsProperties(key, workingEnvironmentResponse);
            }

            function calculateNodeState(node, instance) {
                if (node.healthy) {
                    if (node.inTakeover) {
                        return 'On - In Takeover';
                    }

                    return 'On';
                } else {
                    if (instance && instance.state === 'running') {
                        return 'Failed';
                    }

                    return 'Off';
                }
            }

            function nodesData() {
                return workingEnvironmentResponse.ontapClusterProperties ? workingEnvironmentResponse.ontapClusterProperties.nodes : [];
            }

            function awsNodesBuilder() {
                const instances = _.get(workingEnvironmentResponse, 'awsProperties.instances'),
                    filtered = _.filter(instances, {tags: {WorkingEnvironmentId: workingEnvironmentResponse.publicId}}),
                    nodes = [],
                    ontapNodes = nodesData();

                _.forEach(ontapNodes, (node) => {
                    const instance = _.find(filtered, {id: node.cloudProviderId});
                    nodes.push({
                        instance,
                        node,
                        state: calculateNodeState(node, instance)
                    });
                });

                return nodes;
            }

            function azureNodesBuilder() {
                const nodes = [],
                    ontapNodes = nodesData(),
                    haProperties = _.get(workingEnvironmentResponse, 'haProperties');

                _.forEach(ontapNodes, (node) => {
                    const nodeVMInfo = _.find(haProperties, {serialNumber: node.serialNumber});
                    nodes.push({
                        node,
                        state: calculateNodeState(node, nodeVMInfo)
                    });
                });

                return nodes;
            }

            function mediatorData() {
                function calculateMediatorState(mediator) {
                    const instance = mediator || {};

                    return instance.state === 'running' ? 'On' : 'Off';
                }

                let instances = _.get(workingEnvironmentResponse, 'awsProperties.instances'),
                    mediator = _.find(instances, (instance) => {
                        return instance.tags.WorkingEnvironment === workingEnvironmentResponse.name && _.includes(_.toLower(instance.name), '-mediator');
                    }),
                    name = mediator ? mediator.name : '';

                const mediatorStatus = workingEnvironmentResponse.haProperties ? workingEnvironmentResponse.haProperties.mediatorStatus : null;
                const mediatorHealthy = mediatorStatus ? mediatorStatus.statusCode === 0 : false;

                return {
                    name,
                    privateIpAddress: mediator ? mediator.privateIpAddress : undefined,
                    availabilityZone: mediator ? mediator.availabilityZone : undefined,
                    instance: mediator,
                    state: calculateMediatorState(mediator),
                    mediatorHealthy
                };
            }

            let vsaResponse = {
                id: workingEnvironmentResponse.publicId,
                status: workingEnvironmentResponse.status,
                activeActions: workingEnvironmentResponse.activeActions,
                vsaEncryptionResponse: {
                    awsVolumeEncryption: resolveFrom('encryptionProperties', 'awsVolumeEncryption'),
                    keyManagers: resolveFrom('encryptionProperties', 'keyManagers'),
                    ontapEncryption: resolveFrom('encryptionProperties', 'ontapEncryption'),
                    ontapEncryptionCertificates: resolveFrom('encryptionProperties', 'encryptionCertificates')
                }
            };

            vsaResponse.cluster = {
                clusterName: resolveFrom('ontapClusterProperties', 'clusterName'),
                ontapVersion: resolveFrom('ontapClusterProperties', 'ontapVersion'),
                systemManagerUrl: resolveFrom('ontapClusterProperties', 'systemManagerUrl'),
                creationTime: resolveFrom('ontapClusterProperties', 'creationTime'),
                licenseType: resolveFrom('ontapClusterProperties', 'licenseType'),
                writingSpeedState: _.result(workingEnvironmentResponse, 'ontapClusterProperties.writingSpeedState.toLowerCase'),
                lastModifiedOffbox: resolveFrom('ontapClusterProperties', 'lastModifiedOffbox') || 'N/A'
            };
            vsaResponse.cluster.nodes = nodesData();
            vsaResponse.cluster.lifs = resource.fillLifsData(nodesData());
            vsaResponse.cluster.serialNumbers = nocUtil.concatArrayPropertyData(nodesData(), 'serialNumber');
            vsaResponse.cluster.systemIds = nocUtil.concatArrayPropertyData(nodesData(), 'systemId');

            if (workingEnvironmentResponse._cloudProviderName === 'AWS') {
                vsaResponse = _.assignIn(vsaResponse, {
                    accountId: resolveFrom('awsProperties', 'accountId'),
                    keyPairName: resolveInstanceDataFromAwsProperties('keyPairName'),
                    tenancy: resolveInstanceDataFromAwsProperties('tenancy'),
                    productCode: resolveInstanceDataFromAwsProperties('productCode'),
                    amiId: resolveInstanceDataFromAwsProperties('imageId'),
                    instanceId: _.map(awsNodesBuilder(), 'instance.id').join(', '),
                    instanceType: resolveInstanceDataFromAwsProperties('instanceType'),
                    publicIp: resolveInstanceDataFromAwsProperties('publicIpAddress'),
                    publicDnsName: resolveInstanceDataFromAwsProperties('publicDnsName'),
                    placementGroup: resolveInstanceDataFromAwsProperties('placementGroup')
                });

                if (workingEnvironmentResponse.isHA) {
                    vsaResponse.cluster.extendedNodesData = awsNodesBuilder();
                    vsaResponse.cluster.mediator = mediatorData();
                }
            }

            if (workingEnvironmentResponse._cloudProviderName === 'Azure') {
                const resourceGroup = resolveFrom('providerProperties', 'resourceGroup');

                vsaResponse = _.assignIn(vsaResponse, {
                    subscriptionId: resolveFrom('providerProperties', 'subscriptionId'),
                    instanceType: resolveFrom('providerProperties', 'instanceType'),
                    resourceGroupName: resourceGroup ? resourceGroup.name : null,
                    deploymentId: resolveFrom('providerProperties', 'deploymentId'),
                    creationTime: resolveFrom('providerProperties', 'creationTime')
                });

                if (workingEnvironmentResponse.isHA) {
                    vsaResponse.cluster.extendedNodesData = azureNodesBuilder();
                }
            }

            if (workingEnvironmentResponse._cloudProviderName === 'GCP') {
                vsaResponse = _.assignIn(vsaResponse, {
                    projectName: resolveFrom('providerProperties', 'projectName'),
                    deploymentName: resolveFrom('providerProperties', 'deploymentName')
                });
                vsaResponse.cluster.capacityTierInfo = resolveFrom('ontapClusterProperties', 'capacityTierInfo');
            }

            return vsaResponse;
        };

        function getFields(workingEnvironment) {
            const providerProperties = workingEnvironment._cloudProviderName === 'AWS' ? 'awsProperties' : 'providerProperties';
            const haProperties = workingEnvironment.isHA ? ',haProperties' : '';

            return 'status,encryptionProperties,ontapClusterProperties.fields(upgradeVersions),activeActions,supportedFeatures,k8sProperties,fpolicyProperties,' + providerProperties + haProperties;
        }

        resource.refreshVsaData = function refreshVsaData(nocHttp, workingEnvironment) {
            return nocHttp.get({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '?fields=' + getFields(workingEnvironment),
                headers: {'X-Occm-Bypass-Cache': true}
            }, null, nocHttp.LOADING)
                .then((workingEnvironmentResponse) => {
                    resource.initWorkingEnvironment(workingEnvironmentResponse);
                    const vsaResponse = resource.workingEnvironmentResponseToVsaResponse(workingEnvironmentResponse);

                    $rootScope.$broadcast(resource.VSA_GET_EVENT, {workingEnvironment: workingEnvironmentResponse, vsaResponse});
                })
                .catch((error) => {
                    $rootScope.$broadcast(resource.VSA_GET_ERROR_EVENT, {workingEnvironment, error});
                });
        };

        resource.startVsaPolling = function (workingEnvironment) {
            return nocPoller.start({
                method: 'GET',
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '?fields=' + getFields(workingEnvironment)
            }, 2 * 60 * 1000, (workingEnvironmentResponse) => {
                resource.initWorkingEnvironment(workingEnvironmentResponse);
                const vsaResponse = resource.workingEnvironmentResponseToVsaResponse(workingEnvironmentResponse);

                $rootScope.$broadcast(resource.VSA_GET_EVENT, {workingEnvironment: workingEnvironmentResponse, vsaResponse});
            }, (error) => {
                $rootScope.$broadcast(resource.VSA_GET_ERROR_EVENT, {workingEnvironment, error});
            }, null, true);
        };

        resource.VSA_ACTIVE_ACTIONS_GET_EVENT = '-vsaActiveActionsGetSuccess';
        resource.VSA_ACTIVE_ACTIONS_ERROR_EVENT = '-vsaActiveActionsGetError';

        resource.startVsaActiveActionsPolling = function (workingEnvironment, frequencyInSeconds = 60) {
            return nocPoller.start({
                method: 'GET',
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '?fields=activeActions'
            }, frequencyInSeconds * 1000, (workingEnvironmentResponse) => {
                const activeActions = workingEnvironmentResponse && !_.isEmpty(workingEnvironmentResponse.activeActions) ? workingEnvironmentResponse.activeActions : null;
                $rootScope.$broadcast(workingEnvironment.publicId + resource.VSA_ACTIVE_ACTIONS_GET_EVENT, activeActions);
            },
            (errorResponse) => {
                $rootScope.$broadcast(resource.VSA_ACTIVE_ACTIONS_ERROR_EVENT, errorResponse);
            }, null, true);
        };

        resource.CAPACITY_FEATURES_GET_EVENT = '-capacityFeaturesGetSuccess';
        resource.CAPACITY_FEATURES_ERROR_EVENT = '-capacityFeaturesGetError';

        resource.startCapacityFeaturesPolling = function (workingEnvironment, successCallback, errorCallback) {
            return nocPoller.start({
                method: 'GET',
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '?fields=capacityFeatures'
            }, 2 * 60 * 1000, (workingEnvironmentResponse) => {
                resource.initWorkingEnvironment(workingEnvironmentResponse);
                successCallback(workingEnvironmentResponse);
            }, errorCallback, null, true);
        };

        resource.getVSAByPublicId = function getVSAByPublicId(nocHttp, publicId, pathPrefix, isHa) {
            pathPrefix = pathPrefix || '/vsa/';
            let fields = '?fields=status,ontapClusterProperties.fields(upgradeVersions),activeActions,supportedFeatures,k8sProperties';
            if (isHa) {
                fields += ',haProperties';
            }

            return nocHttp.get(pathPrefix + 'working-environments/' + publicId + fields,
                'finding Cloud Volumes ONTAP with publicId: ' + publicId,
                nocHttp.LOADING);
        };

        resource.getUrlWithFields = function (publicId, pathPrefix, fields) {
            pathPrefix = pathPrefix || '/vsa/';

            return pathPrefix + 'working-environments/' + publicId + '?fields=' + fields;
        };

        resource.stopVsa = function stopVsa(nocHttp, workingEnvironment, takeSnapshots) {
            return nocHttp.post({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/stop',
                params: {
                    takeSnapshots
                }
            }, 'stopping Cloud Volumes ONTAP', nocHttp.BLOCK);
        };

        resource.startVsa = function startVsa(nocHttp, workingEnvironment) {
            return nocHttp.post(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/start',
                'starting Cloud Volumes ONTAP',
                nocHttp.BLOCK);
        };

        resource.updateSchedules = function updateSchedules(nocHttp, workingEnvironment, schedules) {
            return nocHttp.put({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/schedules',
                data: {schedules}
            }, 'saving schedules', nocHttp.BLOCK);
        };

        resource.setPassword = function setPassword(nocHttp, workingEnvironment, password, occmOnly) {
            return nocHttp.put({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/set-password' + (occmOnly ? '?occmOnly=true' : ''),
                data: {password}
            }, 'setting working environment password', nocHttp.BLOCK);
        };

        resource.updateCredentials = function updateCredentials(nocHttp, workingEnvironment, updatedCredentials) {
            return nocHttp.put({
                url: '/onprem/working-environments/' + workingEnvironment.publicId + '/update-credentials',
                data: updatedCredentials
            }, 'updating working environment credentials', nocHttp.BLOCK);
        };

        resource.shouldShowTurnOnAction = function (scope, workingEnvironment) {
            const selectedWorkingEnvironment = scope.selectedWorkingEnvironment || workingEnvironment;
            return !scope.actionsRequired || !scope.actionsRequired[selectedWorkingEnvironment.publicId] || !_.find(scope.actionsRequired[selectedWorkingEnvironment.publicId], {'actionType': 'licenseWasSuspended'});
        };

        resource.configureFabricPool = function configureFabricPool(nocHttp, workingEnvironment, instanceProfileName, gcpStorageAccountId) {
            return nocHttp.post({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/enable-capacity-tiering',
                data: {instanceProfileName, gcpStorageAccountId }
            }, 'enabling capacity tiering', nocHttp.BLOCK);
        };

        resource.changeTierLevel = function changeTierLevel(nocHttp, workingEnvironment, tierLevel) {
            return nocHttp.post({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/change-tier-level',
                data: tierLevel
            }, 'changing tiering level', nocHttp.BLOCK);
        };

        //===========================================================================================================================
        /*
         Cost

         */

        resource.getCost = (nocHttp, workingEnvironment) => {
            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/cost-and-usage', 'retrieving cost', nocHttp.LOADING);
        };

        resource.getSavings = (nocHttp, workingEnvironment) => {
            return workingEnvironment.status.status === 'on' || workingEnvironment.status.status === 'degraded' ?
                nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/ontap-saving', 'retrieving savings', nocHttp.LOADING) :
                $q.resolve({isVsaAvailable: true});
        };
        //===========================================================================================================================
        /*
        Ransomware protection
         */
        resource.activateSnapshot = (nocHttp, workingEnvironment) => {
            return nocHttp.put(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/activate-snapshot-policy', 'activating snapshot policy');
        };

        resource.activateFpolicy = (nocHttp, workingEnvironment) => {
            return nocHttp.put(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/activate-fpolicy', 'activating fpolicy');
        };

        resource.disableFpolicy = (nocHttp, workingEnvironment) => {
            return nocHttp.put(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/disable-fpolicy', 'disabling fpolicy');
        };
        //===========================================================================================================================
        /*
         Change VSA image

         */

        resource.eligibilityTests = function eligibilityTests(nocHttp, workingEnvironment) {
            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/update-eligibility',
                'retrieving eligibility support status of working environment ' + workingEnvironment.name, nocHttp.BLOCK);
        };

        resource.changeOntapImage = function changeOntapImage(nocHttp, workingEnvironment, payload) {
            const pathPrefix = workingEnvironment._pathPrefix || nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.type, $state.params.provider);

            return nocHttp.post({
                url: pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/update-image',
                data: payload
            }, 'upgrading Cloud Volumes ONTAP image', nocHttp.BLOCK);
        };

        resource.getLocalImages = function getLocalImages(nocHttp, workingEnvironment) {
            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/ontap-available-images',
                'retrieving Cloud Volumes ONTAP local images', nocHttp.LOADING);
        };

        resource.changeMediatorVersion = function changeMediatorVersion(nocHttp, workingEnvironment, payload) {
            const pathPrefix = workingEnvironment._pathPrefix || nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.type, $state.params.provider);

            return nocHttp.post({
                url: pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/update-mediator',
                data: payload
            }, 'upgrading Mediator', nocHttp.BLOCK);
        };

        //=================================================================================================================================
        /*
         licenses and instance types
         */

        resource.getLicenseAndInstanceTypes = function getLicenseAndInstanceTypes(nocHttp, workingEnvironment) {
            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/license-instance-type',
                'retrieving license and ' + workingEnvironment._nameForInstance + ' types', nocHttp.BLOCK);
        };

        resource.changeLicenseAndInstanceType = function changeLicenseAndInstanceType(nocHttp, workingEnvironment, licenseType, instanceType) {
            const message = 'changing ' + (licenseType ? 'license ' : '') + (licenseType && instanceType ? 'and ' : '') + (instanceType ? workingEnvironment._nameForInstance + ' ' : '') + 'type';

            return nocHttp.put({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/license-instance-type',
                data: {
                    licenseType,
                    instanceType
                }
            }, message, nocHttp.BLOCK)
                .then((response) => {
                    if (_.toUpper(workingEnvironment.status.status) !== 'OFF') {
                        resource.updateWorkingEnvironmentStatus(workingEnvironment, 'updating');
                    }

                    return response;
                });
        };

        resource.checkIfCifsLockExists = function checkIfCifsLockExists(nocHttp, workingEnvironment) {
            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/cifs-locks-exist',
                'checking if cifs lock exists', nocHttp.LOADING);
        };

        resource.changeNetworkUtilization = function (nocHttp, workingEnvironment, optimize) {
            return nocHttp.put({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/networkOptimization',
                data: {
                    optimize
                }
            }, 'changing network optimization level', nocHttp.BLOCK);
        };

        resource.getNumberOfAvailableIpsInSubnet = function getNumberOfAvailableIpsInSubnet(nocHttp, workingEnvironment) {
            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/available-ips-in-subnet ',
                'retrieving number of available IPs in subnet', nocHttp.LOADING);
        };

        //=================================================================================================================================
        /*
         tags types
         */

        resource.getTags = function getTags(nocHttp, workingEnvironment) {
            return nocHttp.get(workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/user-tags',
                'retrieving user tags', nocHttp.BLOCK);
        };

        resource.setTags = function setTags(nocHttp, workingEnvironment, tags) {
            return nocHttp.put({
                url: workingEnvironment._pathPrefix + 'working-environments/' + workingEnvironment.publicId + '/user-tags',
                data: {tags}
            }, 'updating user tags', nocHttp.BLOCK);
        };

        return resource;
    });
})();
