(() => {
    'use strict';

    angular.module('noc').config(($stateProvider, virginWizardTreeBuilder) => {
        const occmWizardSteps = [
            {
                name: 'wizard',
                controller: 'VirginWizardController',
                templateUrl: 'assets/templates/wizards/virgin-wizard.html',
                url: '/wizard?slTenantId&from&nopoll&id&type&pathPrefix&startDiscoverWizard',
                resolve: {
                    layoutOptions: function layoutOptions() {
                        return {title: 'Add a Working Environment'};
                    },
                    exitOptions: /*@ngInject*/function exitOptions($stateParams) {
                        return {
                            exit: function exit() {
                                if ($stateParams.from) {
                                    this.checkAndSetState($stateParams.from);
                                } else {
                                    this.checkAndSetState('main');
                                }
                            }
                        };
                    },
                    initOptions: function initOptions() {
                        return {
                            hasInitStep: true,
                            firstStep: 'wizard.init'
                        };
                    }
                }

            },
            {
                name: 'init',
                controller: 'VirginChooseWizardController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/init.html'
            },
            {
                name: 'choose',
                controller: 'VirginChooseWizardController',
                wizardTemplateUrl: 'assets/templates/wizards/choose-wizard.html'
            },
            {
                name: 'no-accounts',
                controller: 'VirginAddFirstAccountController',
                wizardTemplateUrl: 'assets/templates/wizards/no-accounts.html'
            },
            //Discover OnPrem wizard
            {
                name: 'cluster',
                controller: 'VirginOnPremClusterStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-discover-on-prem/cluster.html'
            },
            {
                name: 'cvo-discover',
                controller: 'VirginChooseWizardController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-discover-cloud-ontap/select-provider.html'
            },
            {
                name: 'details',
                controller: 'VirginOnPremDetailsStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-discover-on-prem/details.html'
            },
            //Recover Cloud Volumes ONTAP wizard
            {
                name: 'region',
                controller: 'VirginCloudOntapRegionStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-discover-cloud-ontap/region.html'
            },
            {
                name: 'credentials',
                controller: 'VirginCloudOntapCredentialsStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-discover-cloud-ontap/credentials.html'
            },
            {
                name: 'choose-ha-deployment-model',
                controller: 'HaDeploymentModelController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/choose-ha-deployment-model.html'
            },
            {
                name: 'vpc',
                controller: 'VirginCreateWorkingEnvironmentVPCStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-vpc.html'
            },
            {
                name: 'ha-vpc',
                controller: 'VirginCreateWorkingEnvironmentHaVpcStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-ha-vpc.html'
            },
            {
                name: 'vnet',
                controller: 'VirginCreateWorkingEnvironmentAzureVnetStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-azure-vnet.html'
            },
            {
                name: 'gcp-vpc',
                controller: 'VirginCreateWorkingEnvironmentGoogleVpcStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-google-vpc.html'
            },
            {
                name: 'ha-public-key',
                controller: 'VirginCreateWorkingEnvironmentHAPublicKeyController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-ha-public-key.html'
            },
            {
                name: 'ha-floating-ip',
                controller: 'VirginCreateWorkingEnvironmentHAFloatingIpController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-ha-floating-ip.html'
            },
            {
                name: 'ha-route-tables',
                controller: 'VirginCreateWorkingEnvironmentHaRouteTablesStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-ha-routing-tables.html'
            },
            {
                name: 'encryption',
                controller: 'VirginCreateWorkingEnvironmentEncryptionStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-encryption.html'
            },
            {
                name: 'key-managers',
                controller: 'VirginCreateWorkingEnvironmentKeyManagersStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-key-managers.html'
            },
            {
                name: 'byol',
                controller: 'CreateWorkingEnvironmentByolStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-byol.html'
            },
            {
                name: 'packages',
                controller: 'CreateWorkingEnvironmentPackagesStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-packages.html'
            },
            {
                name: 'roles',
                controller: 'ChooseInstanceRoleStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/choose-instance-role.html'
            },
            {
                name: 'license',
                controller: 'VirginCreateWorkingEnvironmentLicenseStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-license.html'
            },
            {
                name: 'check-mp-registration',
                controller: 'VirginCreateCheckMarketPlaceRegistration',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-check-mp-registration.html'
            },
            {
                name: 'tiering',
                controller: 'VirginCreateWorkingEnvironmentTieringStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-tiering.html'
            },
            {
                name: 'writing-speed',
                controller: 'VirginCreateWorkingEnvironmentWritingSpeedStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-writing-speed.html'
            },
            {
                name: 'worm',
                controller: 'VirginCreateWorkingEnvironmentWormStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-worm.html'
            },
            {
                name: 'azure-preview-license',
                controller: 'VirginCreateWorkingEnvironmentAzurePreviewLicense',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/azure-preview-license.html'
            },
            {
                name: 'create-detailsCredentials',
                controller: 'VirginCreateWorkingEnvironmentDetailsStepAndCredentialsController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-details-credentials.html'
            },
            {
                name: 'volume',
                controller: 'VirginCreateWorkingEnvironmentVolumeStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-volume.html'
            },
            {
                name: 'cifs',
                controller: 'CreateCIFSStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-cifs.html'
            },
            {
                name: 'tiering-policy',
                controller: 'VirginCreateWorkingEnvironmentTieringPolicyStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-tiering-policy.html'
            },
            {
                name: 'gcp-accounts',
                controller: 'VirginCreateGcpAccountsController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-gcp-accounts.html'
            },
            {
                name: 'review',
                controller: 'VirginCreateWorkingEnvironmentReviewStepController',
                wizardTemplateUrl: 'assets/templates/wizards/virgin-create-working-environment/create-review.html'
            }
        ];

        const volumeEnding = {
            cifs: {
                'tiering-policy': {
                    'gcp-accounts': {
                        review: true
                    },
                    review: true
                }
            },
            'tiering-policy': {
                'gcp-accounts': {
                    review: true
                },
                review: true
            },
            'gcp-accounts': {
            review: true
            },
            review: true
        };

        const tieringEnding = {
            'writing-speed': {
                volume: volumeEnding
            },
            worm: { //Skip 'writing-speed' for HA
                volume: volumeEnding
            },
            volume: volumeEnding //Skip 'WORM' for HA & older versions
        };

        const licenseEnding = {
            'check-mp-registration': {
                'tiering': tieringEnding
            },
            'tiering': tieringEnding
        };

        const byolEnding = {
            packages: {
                license: licenseEnding,
                roles: {
                    license: licenseEnding,
                    volume: volumeEnding
                },
                'check-mp-registration': {
                    volume: volumeEnding
                },
                volume: volumeEnding
            }
        };

        const vpcEnding = {
            encryption: {
                byol: byolEnding,
                'key-managers': {
                    byol: byolEnding
                }
            },
            byol: byolEnding
        };

        const haVpcEnding = {
            'ha-public-key': {
                'ha-floating-ip': {
                    'ha-route-tables': vpcEnding
                }
            }
        };

        const haDeploymentModelEnding = {
            'ha-vpc': haVpcEnding,
            vpc: {
                'ha-public-key': vpcEnding
            }
        };

        const createNewVsaEnding = {
            credentials: true,
            vpc: vpcEnding,
            vnet: {
                byol: byolEnding
            },
            'gcp-vpc': {
                byol: byolEnding
            },
            'choose-ha-deployment-model': haDeploymentModelEnding
        };

        const noAccountsEnding = {
            cluster: { //Discover onPrem
                details: true
            },
            region: { //Discover vsa
                credentials: true
            },
            'create-detailsCredentials': createNewVsaEnding
        };

        const occmWizard = {
            wizard: {
                init: {
                    choose: {
                        'cvo-discover': {
                            'no-accounts': noAccountsEnding,
                            region: { //Discover vsa
                                credentials: true
                            },
                            cluster: { //Discover onPrem
                                details: true
                            }
                        },
                        cluster: { //Discover onPrem
                            details: true
                        },
                        region: { //Discover vsa
                            credentials: true
                        },
                        'no-accounts': noAccountsEnding,
                        'create-detailsCredentials': createNewVsaEnding
                    },
                    'create-detailsCredentials': createNewVsaEnding
                }
            }
        };

        virginWizardTreeBuilder($stateProvider, occmWizard, occmWizardSteps);
    });
})();
