(() => {
    'use strict';

    angular.module('noc').run(($rootScope, $http, $location, nocUIMain, nocNavigateAway, nocPoller, $timeout,
                               $state, Messageable, loggerService, routeLogger, nocTenantsService, $log,
                               intercomNotifier, $q, nocRoutingResolverService, systemResource, nocLocalStorage, nocDialog,
                               Constants, proxyPopupService, nocRegionsService, nocAuth0Service, nocCurrentUserService,
                               marketingPopupService, nocSelectedWorkspaceOrTenantService, nocWorkspacesService) => {
        let agentMode;

        $rootScope.appState = 'none';

        //eslint-disable-next-line id-length
        $rootScope._ = _; //Add lodash
        $rootScope.tutorial = {};
        $rootScope.tutorial.html = '';
        $rootScope.tutorial.show = false;
        $rootScope.tutorial.TUTORIAL_CLOSED_MESSAGE = 'TUTORIAL_CLOSED';

        $rootScope.activateTutorial = (url, name) => {
            $rootScope.tutorial.html = url;
            $rootScope.tutorial.name = name;
            $rootScope.tutorial.show = true;
        };

        $rootScope.closeTutorial = () => {
            $rootScope.tutorial.html = '';
            $rootScope.tutorial.show = false;
            $rootScope.$broadcast($rootScope.tutorial.TUTORIAL_CLOSED_MESSAGE, {name: $rootScope.tutorial.name});
        };

        loggerService.start();
        routeLogger.start();
        Messageable.call($rootScope);

        function accountWidget(accountWidgetUrl) {
            window.accountManagementWidget = window.accountManagementWidget || {};
            window.accountManagementWidget.sendMessage = window.accountManagementWidget.sendMessage || function () {
                (window.accountManagementWidget.sendMessage.q = window.accountManagementWidget.sendMessage.q || []).push(arguments[0]);
            };

            const elem = document.createElement('script');
            elem.src = accountWidgetUrl;
            document.head.appendChild(elem);
        }

        const loadInitialStates = function loadInitialStates(currentUser, userMetadataObj) {
            //eslint-disable-next-line noc/catch-or-return
            agentMode = !!(_.get($rootScope.supportServices, 'tenancyServiceInformation.url'));
            if (agentMode) {
                accountWidget(_.get($rootScope.supportServices, 'tenancyServiceInformation.accountWidgetUrl'));
            }

            $q.all([
                nocCurrentUserService.initializeCurrentUser(currentUser, $http, agentMode, userMetadataObj),
                agentMode ? nocWorkspacesService.reloadWorkspaces() : nocTenantsService.reloadTenants()
            ]).then(initialize)
                .catch((error) => {
                    $log.error(error);
                });


            nocPoller.start('/occm/config/simplicator', 1 * 60 * 1000,
                (simplicatorConfig) => {
                    $rootScope.simplicatorConfig = simplicatorConfig;
                },
                (error) => {
                    if (error.status === 'NETWORK_FAILURE') {
                        $rootScope.pushError(error.messageForDisplay(), error, 'NETWORK', 'NETWORK');
                    }

                    $rootScope.simplicatorConfig = $rootScope.simplicatorConfig ? {
                        redirected: $rootScope.simplicatorConfig.redirected,
                        uri: '???'
                    } : null;
                });
        };

        const initialize = () => {
            if (nocCurrentUserService.isAllowed('patch:*')) { //Notification should be displayed only for admin role
                systemResource().getAvailableVersions()
                    .then((updates) => {
                        if (updates.length > 0) {
                            $rootScope.newVersionAvailable = true;
                            $rootScope.updatesAvailable = updates;
                        }
                    })
                    .catch(() => {
                        proxyPopupService.proxyFail('Failed to get available update versions data from server', 'Unable to contact server.');
                    });
            }

            $rootScope.appState = 'loaded';
            nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl(cancelledState.params, agentMode);
            cancelledState.params.slTenantId = nocSelectedWorkspaceOrTenantService.getPublicId();
            $state.go(cancelledState.name, cancelledState.params, {location: false});
            marketingPopupService.getPopupData();
        };

        const loadAbout = function loadAbout() {
            return $http.get('/occm/system/about')
                .then((about) => {
                    $rootScope.about = about.data;
                    const newVersion = $rootScope.about.version + '.' + $rootScope.about.build;
                    const prevVersion = nocLocalStorage.LATEST_LOGIN_VERSION.restore();
                    if (prevVersion && newVersion !== prevVersion) {
                        $rootScope.versionUpdated = true;
                    }

                    nocLocalStorage.LATEST_LOGIN_VERSION.save(newVersion);
                });
        };

        const loadExternalConfig = function loadExternalConfig() {
            return $http.get('/occm/system/external-config')
                .then((response) => {
                    $rootScope.externalConfig = response.data;
                    intercomNotifier.boot();
                });
        };

        const loadCloudSyncUrl = () => {
            return $http.get('/cloudsync/service-url')
                .then((response) => {
                    $rootScope.cloudSyncUrl = response.data;
                });
        };

        const startTime = new Date().getTime();

        function loadInitialDataFlow(currentUser, userMetadataObj) {
            loadExternalConfig().then(() => {
                loadCloudSyncUrl().then(() => {
                    nocRegionsService.getRegionsFromServer().then(() => {
                        loadInitialStates(currentUser, userMetadataObj);
                    }).catch(/*Not suppose to happn*/);
                }).catch(/*Not suppose to happn*/);
            }).catch(/*Not suppose to happn*/);
        }

        const portalWidget = () => {
            if (!$rootScope.isPortalWidgetAlreadyExist) {
                const portalWidgetElement = document.createElement('script');
                portalWidgetElement.src = $rootScope.about.environment === 'production' ? 'https://services.cloud.netapp.com/widget/loader.js' : 'https://staging.services.cloud.netapp.com/widget/loader.js';
                document.head.appendChild(portalWidgetElement);
                $rootScope.isPortalWidgetAlreadyExist = true;
            }
        };

        const handlePortalLogin = () => {
            //The auth0Information is needed for portalUserAuthentication(), if missing it indicates of no connectivity
            if (!$rootScope.supportServices.portalService.auth0Information) {
                proxyPopupService.proxyFail('', 'Unable to contact server');
            } else {
                nocAuth0Service.portalUserAuthentication()
                    .then((authResult) => {
                        $http.get('/auth/current-user?rand=' + new Date().getTime())
                            .then((response) => {
                                nocLocalStorage.LOGINN_AFTER_MIGRATION.save(true);
                                loadInitialDataFlow(response.data, nocAuth0Service.extractUserMetadata(authResult));
                                portalWidget();
                            })
                            .catch((error) => {
                                if (error.status === 403) {
                                    //The routing to Unauthorized page is handled by Interceptor
                                    const userMetadata = nocAuth0Service.extractUserMetadata(authResult);

                                    const currentUserErrorObj = {
                                        firstName: userMetadata.name.substr(0, userMetadata.name.indexOf(' ')),
                                        lastName: userMetadata.name.substr(userMetadata.name.indexOf(' ') + 1, userMetadata.name.length),
                                        email: userMetadata.email,
                                        company: userMetadata.company,
                                        publicId: userMetadata.email,
                                        auth0Id: userMetadata.auth0Id
                                    };
                                    nocCurrentUserService.initializeCurrentUser(currentUserErrorObj, $http, agentMode, userMetadata.auth0Id);

                                    $rootScope.externalConfig = {
                                        'intercom_prod_app_id': $rootScope.supportServices.intercomAppId
                                    };
                                    intercomNotifier.boot();
                                } else {
                                    //Sanity: should not happen:
                                    $log.log(error.data);
                                }
                            });
                    })
                    .catch(/*Failure is already logged, nothing further to do*/);
            }
        };

        const loadCurrentUser = function loadCurrentUser() {
            return $http.get('/auth/current-user?rand=' + new Date().getTime()) // Add random query param as FF caches the response otherwise
                .then((response) => {
                    loadInitialDataFlow(response.data);
                })
                .catch((error) => {
                    switch (error.status) {
                        case 401: //Unauthorized
                            if (_.get($rootScope, 'supportServices.portalService.usePortalAuthentication')) {
                                const loggedInThePast = nocLocalStorage.LOGINN_AFTER_MIGRATION.restore(false);
                                if ($rootScope.about.migrationPerformed && !loggedInThePast) {
                                    nocLocalStorage.LOGINN_AFTER_MIGRATION.save(false);
                                    $rootScope.appState = 'after-migration';
                                    nocRoutingResolverService.resolveEntryPoint();
                                } else {
                                    handlePortalLogin();
                                }
                            } else {
                                $rootScope.appState = 'login';
                            }
                            break;
                        case 403: //No trusted CAs in IC
                            $rootScope.appState = 'failed';
                            nocDialog.confirm('Install Trusted CA Certificate', 'assets/templates/dialogs/add-trusted-ca.html', {
                                width: 700
                            });
                            break;
                        case 409: //Automatic setup after portal, user needs to configure proxy
                            //Display dialog to ask user for proxy, and call set proxy API.
                            $rootScope.appState = 'needs-proxy';
                            break;
                        case 420: //Setup wizard
                            $rootScope.externalConfig = {
                                'intercom_prod_app_id': $rootScope.supportServices.intercomAppId
                            };
                            intercomNotifier.boot();
                            if (_.get($rootScope, 'supportServices.portalService.usePortalAuthentication')) {
                                $rootScope.appState = 'portal-setup-wizard';
                            } else {
                                $rootScope.appState = 'setup-wizard';
                            }
                            nocRoutingResolverService.resolveEntryPoint();
                            break;
                        default:
                            if ((new Date().getTime() - startTime) < (2 * 60 * 1000)) {
                                $timeout(loadCurrentUser, 2500);
                            } else {
                                proxyPopupService.proxyFail('', 'Unable to contact server');
                            }
                    }
                });
        };

        const preserveSearch = ['slTenantId', 'nopoll', 'type', 'provider'];
        $rootScope.checkAndSetState = (toState, toParams, preventReload) => {
            toParams = toParams || {};
            _.forEach(preserveSearch, (key) => {
                if (!toParams[key]) toParams[key] = $state.params[key];
            });
            nocNavigateAway.sendNavigateAwayMessage(() => {
                nocUIMain.clearFormValidity();
                $state.go(toState, toParams, preventReload ? {notify: false} : undefined);
                $rootScope.$broadcast('header-change-height');
            });
        };

        let cancelledState;
        $rootScope.$on('$stateChangeStart', (event, stateTo, stateParams) => {
            if ($rootScope.appState === 'none' || $rootScope.appState === 'failed' || $rootScope.appState === 'login') {
                cancelledState = {
                    name: stateTo.name,
                    params: stateParams
                };
                event.preventDefault();
            } else if ($rootScope.appState === 'setup-wizard' && !_.includes(stateTo.name, 'anterior-setup-wizard')) {
                //Trying to go to somewhere else when needed to setup:
                //prevents the user from browsing around the app while app mode is 'setup-wizard'.
                //but allow to continue with the setup
                if (!_.includes(stateTo.name, 'setup-wizard')) {
                    $log.warn('Preventing navigation to ' + stateTo.url + ' while setup is not complete');
                    event.preventDefault();
                }
            } else if ($rootScope.appState !== 'setup-wizard' && _.includes(stateTo.name, 'anterior-setup-wizard')) {
                //Trying to go to setup wizard when already done setup
                nocUIMain.reloadSilently(true);
            } else if (stateTo.permission) {
                const isPermitted = nocCurrentUserService.isAllowed(stateTo.permission, nocSelectedWorkspaceOrTenantService.getSelectedWorkspace());
                if (!isPermitted) {
                    event.preventDefault();
                    $state.transitionTo('main');
                }
            }
        });

        $rootScope.$on('$stateChangeSuccess', (event, stateTo, stateParams) => {
            if ($rootScope.appState === 'loaded') {
                agentMode = !!(_.get($rootScope.supportServices, 'tenancyServiceInformation.url'));
                nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl(stateParams, agentMode);
            }

            const origAppView = $rootScope.appView;
            $rootScope.appView = _.startsWith(stateTo.name, 'volume-view.') || _.startsWith(stateTo.name, 'anterior') ? 'volume-view' : 'storage-system-view';

            if (_.get($rootScope, 'supportServices.portalService.usePortalAuthentication' && !_.includes($rootScope.appState, 'setup'))) {
                portalWidget();
            }

            if (origAppView !== $rootScope.appView) {
                intercomNotifier.event('using-' + $rootScope.appView);
            }
        });

        $rootScope.$on('$locationChangeStart', () => {
            if (!$rootScope.nextPreviousUrl) {
                $rootScope.nextPreviousUrl = $location.url();
            }
        });

        $rootScope.$on('$locationChangeSuccess', () => {
            $rootScope.previousUrl = $rootScope.nextPreviousUrl;
            $rootScope.nextPreviousUrl = $location.url();
        });

        $rootScope.$on('$stateChangeError', (event, toState, toParams, fromState, fromParams, error) => {
            $log.error(error);
        });

        $rootScope.$on(Constants.ASYNC_OPERATION_ERROR_EVENT, (event, errorMessage, pidgeonService) => {
            if ($rootScope.isDialogOpen) {
                pidgeonService.error('Async Operation Error', errorMessage, 'Async');
            } else {
                nocDialog.dialog('Async Operation Error', 'assets/templates/dialogs/async-operation-error.html', {
                    width: 650,
                    errorMessage
                });
            }
        });

        const loadInstancePlacement = function loadInstancePlacement() {
            return systemResource().getInstancePlacement()
                .then((instancePlacementResponse) => {
                    $rootScope.instancePlacement = instancePlacementResponse || {installLocation: 'OnPrem'};
                    $rootScope.runningInIC = instancePlacementResponse && instancePlacementResponse.ic;
                    $rootScope.runningInChina = instancePlacementResponse && instancePlacementResponse.china;
                    $rootScope.runningInGovCloud = instancePlacementResponse && instancePlacementResponse.govCloud;
                });
        };

        $rootScope.isAllowed = (action) => {
            return nocCurrentUserService.isAllowed(action, nocSelectedWorkspaceOrTenantService.getSelectedWorkspace());
        };

        /////////////////////
        //      Init
        /////////////////////

        $q.all([
            loadAbout(),
            loadInstancePlacement(),
            systemResource().getSupportServices()
        ])
            .then(loadCurrentUser)
            .catch((error) => {
                $log.error(error);
                proxyPopupService.proxyFail('', 'Unable to contact server: ' + error.data.messageForDisplay());
            });
    });
})();
