(() => {
    'use strict';

    angular.module('noc').factory('nocAdvancedAllocationService', ($q, aggregatesResource) => {
        const advancedAllocationService = {};

        advancedAllocationService.getAggregates = (nocHttp, pidgeon, targetWorkingEnvironment) => {
            return aggregatesResource.getAggregates(nocHttp, targetWorkingEnvironment)
                .then((aggregates) => {
                    _.forEach(aggregates, aggregatesResource.initializeAggregate);
                    if (aggregates.length) {
                        return _.sortBy(aggregates, 'name');
                    } else {
                        pidgeon.error('No aggregates other than root aggregate were found in working environment ' + targetWorkingEnvironment.name + '.', null, 'aggregates');

                        return $q.reject();
                    }
                });
        };

        advancedAllocationService.getSVMs = (nocHttp, targetWorkingEnvironment, scope) => {
            return nocHttp.get(targetWorkingEnvironment._pathPrefix + 'working-environments/' + targetWorkingEnvironment.publicId + '?fields=svms',
                'retrieving SVMs for working environment ' + targetWorkingEnvironment.name,
                nocHttp.LOADING)
                .then((workingEnvironmentResponse) => {
                    scope.destinationSvms = workingEnvironmentResponse.svms;

                    return workingEnvironmentResponse;
                });
        };

        advancedAllocationService.getPossibleTiers = (selectedWorkingEnvironment) => {
            return _(selectedWorkingEnvironment._providerVolumeTypes)
                .mapValues((type) => {
                    type.supportedSizes = _.filter(type.supportedSizes, (size) => {
                        return size.maxDisksAllow.numOfDisks > 0;
                    });

                    return type;
                })
                .pickBy((diskType) => {
                    return !_.isEmpty(diskType.supportedSizes);
                })
                .value();
        };

        return advancedAllocationService;
    });
})();
