(() => {
    'use strict';

    angular.module('noc').factory('nocAuth0Service', ($rootScope, $q, $injector, $log) => {
        const auth0Service = {};

        auth0Service.constructAuth = (clientId) => {
            // eslint-disable-next-line no-undef
            window.auth = new Auth({
                clientID: clientId || $rootScope.supportServices.portalService.auth0Information.clientId,
                audience: $rootScope.supportServices.portalService.auth0Information.audience,
                domain: $rootScope.supportServices.portalService.auth0Information.domain
            });
        };

        auth0Service.generateTokenForCustomAudience = (audience) => {
            // eslint-disable-next-line no-undef
            const auth = new Auth({
                clientID: $rootScope.supportServices.portalService.auth0Information.clientId,
                audience,
                domain: $rootScope.supportServices.portalService.auth0Information.domain
            });
            const uriPrefix = _.endsWith(window.location.origin, ':1337') ? window.location.origin : `${window.location.origin}/occmui`;
            const deferred = $q.defer(); //eslint-disable-line angular/deferred

            auth.refreshSso({redirectUri: `${uriPrefix}/silent-callback.html`})
                .then((authResult) => {
                    const token = authResult.accessToken;
                    deferred.resolve(token);
                })
                .catch((error) => {
                    $log.log(error);
                    deferred.reject(error);
                });

            return deferred.promise;
        };

        function refreshAuth(forgotMyPasswordState) {
            // Configure uri prefix to run with gulp proxy or localhost
            const uriPrefix = _.endsWith(window.location.origin, ':1337') ? window.location.origin : `${window.location.origin}/occmui`;
            const deferred = $q.defer(); //eslint-disable-line angular/deferred
            // eslint-disable-next-line noc/catch-or-return
            window.auth.refreshSso({redirectUri: `${uriPrefix}/silent-callback.html`})
                .then((authResult) => {
                    $rootScope.loginToken = `Bearer ${authResult.accessToken}`;
                    $rootScope.auth0Id = parseJwt(authResult.accessToken).sub;
                    const loginTokenExpiresIn = authResult.expiresIn;

                    //Fetching the $http service this way to avoid circular dependency with the interceptor
                    const $http = $injector.get('$http');

                    setTimeout(() => {
                        refreshAuth();
                    }, loginTokenExpiresIn * 1000 - 10000);

                    $http.defaults.headers.common.Authorization = $rootScope.loginToken;
                    deferred.resolve(authResult);
                }, (error) => {
                    if (error.needLogin) {
                        //No need to resolve promise because redirect to auth0 occurs when successful
                        const params = forgotMyPasswordState ? {service: 'ontap-cloud', initialScreen: 'forgotPassword'} : {service: 'ontap-cloud', disableSignup: false};
                        window.auth.loginRedirect({}, params);
                    } else {
                        $log.log(error);
                        deferred.reject(error);
                    }
                });

            return deferred.promise;
        }

        auth0Service.portalUserAuthentication = (clientId, forgotMyPasswordState) => {
            if (!clientId && !$rootScope.supportServices.portalService.auth0Information) {
                //Sanity, should not happen
                $log.error('Cannot construct window.auth without clientId or auth0Information');

                return $q.reject();
            }

            auth0Service.constructAuth(clientId);

            return refreshAuth(forgotMyPasswordState);
        };

        /* eslint-disable lodash/prefer-lodash-method */
        function parseJwt(token) {
            const base64Url = token.split('.')[1];
            const base64 = base64Url.replace('-', '+').replace('_', '/');

            return JSON.parse(window.atob(base64));
        }
        /* eslint-enable lodash/prefer-lodash-method*/

        auth0Service.extractUserMetadata = (authResult) => {
            const userInfo = parseJwt(authResult.idToken);
            const userMetadata = userInfo['http://cloud.netapp.com/user_metadata'];
            userMetadata.auth0Id = userInfo.sub;

            return userMetadata;
        };

        return auth0Service;
    });
})();
