(() => {
    'use strict';
    angular.module('noc').service('autoCapacityService', (nocLocalStorage, nocDialog, configService,
                                                          $log, $q, $rootScope, nocCurrentUserService) => {
        const service = {};

        const dialogWasAlreadySeen = () => {
            return nocLocalStorage.DONT_SHOW_AUTO_CAPACITY_DIALOG.restore();
        };

        const isCapacityManagementModeAutomatic = () => {
            return configService.getConfig(null, ['autoVsaCapacityManagement']);
        };

        const dontShowAgain = () => {
            nocLocalStorage.DONT_SHOW_AUTO_CAPACITY_DIALOG.save(true);
        };

        service.showAutoCapacityDialog = () => {
            if (dialogWasAlreadySeen() || !nocCurrentUserService.isAllowed('configuration:*')) return $q.when();

            return isCapacityManagementModeAutomatic()
                .then((configResponse) => {
                    if (configResponse.autoVsaCapacityManagement) return;

                    return nocDialog.confirm('System Update - Automatic Capacity Management Mode', 'assets/templates/dialogs/auto-capacity-dialog.html', {width: 650})
                        .finally(() => {
                            dontShowAgain();

                            return configService.updateConfig(null, ['autoVsaCapacityManagement'], {autoVsaCapacityManagement: true}, configResponse);
                        });
                })
                .catch($log.error);
        };

        return service;
    });
})();
