(function () {
    'use strict';

    angular.module('noc').factory('nocCloudSyncService', () => {
        const resource = {};

        function resolveUrlPrefixFromType(weType, postFix) {
            let url = '/cloudsync/' + postFix + '/';
            if (weType === 'ON_PREM' || weType === 'NPS') {
                url = url + 'onprem/';
            }

            return url;
        }

        resource.getKetPairsByWorkingEnvironment = (nocHttp, weId) => {
            return nocHttp.get({
                url: `/cloudsync/key-pairs-by-vsa/${weId}`
            }, 'retrieving Key Pairs', nocHttp.LOADING);
        };

        resource.geVcpsByWorkingEnvironment = (nocHttp, weId) => {
            return nocHttp.get({
                url: `/cloudsync/vpcs-by-vsa/${weId}`
            }, 'retrieving Key Pairs', nocHttp.LOADING);
        };

        resource.getBucketsByUserId = (nocHttp, userId)  => {
            //GET /cloudsync/buckets
            return nocHttp.get({
                url: '/cloudsync/buckets',
                params: {userId: userId}
            }, 'retrieving S3 buckets', nocHttp.LOADING);
        };

        resource.getBuckets = function getBuckets(nocHttp, weId) {
            //GET /cloudsync/buckets
            return nocHttp.get({
                url:  `/cloudsync/buckets/${weId}`
            }, 'retrieving S3 buckets', nocHttp.LOADING);
        };

        resource.sync = function sync(nocHttp, workingEnvironmentId, syncRequest, weType) {
            //POST /cloudsync/sync/{workingEnvironmentId}
            return nocHttp.post({
                url: resolveUrlPrefixFromType(weType, 'sync') + workingEnvironmentId,
                data: syncRequest
            }, 'creating sync with Cloud Sync', nocHttp.BLOCK);
        };

        resource.getBucketSubdirectoriesByUserId = (nocHttp, bucketName, userId, fullPath) => {
            //GET /cloudsync/buckets/{bucketName}/sub-directories
            return nocHttp.get({
                url: '/cloudsync/buckets/' + bucketName + '/sub-directories',
                params: {userId: userId, fullPath: fullPath}
            }, 'retrieving sub-directories for S3 bucket ' + bucketName, nocHttp.LOADING);
        };

        resource.getBucketSubdirectories = function getBucketSubdirectories(nocHttp, bucketName, weId, fullPath) {
            //GET /cloudsync/buckets/{bucketName}/sub-directories
            return nocHttp.get({
                url: `/cloudsync/buckets/${bucketName}/sub-directories/${weId}`,
                params: {fullPath}
            }, 'retrieving sub-directories for S3 bucket ' + bucketName, nocHttp.LOADING);
        };

        resource.deleteSync = function deleteSync(nocHttp, workingEnvironmentId, weType) {
            //DELETE /cloudsync/sync/{workingEnvironmentId}
            const url = resolveUrlPrefixFromType(weType, 'sync') + workingEnvironmentId;

            return nocHttp.delete(url, 'deleting Cloud Sync integration', nocHttp.BLOCK);
        };

        resource.syncVolume = function syncVolume(nocHttp, workingEnvironmentId, volumeName, weType) {
            //POST /cloudsync/relationship/{workingEnvironmentId}/{volumeName}
            const url = resolveUrlPrefixFromType(weType, 'relationship') + workingEnvironmentId + '/' + volumeName;

            return nocHttp.post(url, 'syncing volume ' + volumeName + ' to S3 with cloud sync', nocHttp.BLOCK);
        };

        resource.deleteSyncVolume = function deleteSyncVolume(nocHttp, workingEnvironmentId, volumeName, weType) {
            //DELETE /cloudsync/relationship/{workingEnvironmentId}/{volumeName}
            const url = resolveUrlPrefixFromType(weType, 'relationship') + workingEnvironmentId + '/' + volumeName;

            return nocHttp.delete(url, 'deleting Cloud Sync relationship for volume ' + volumeName, nocHttp.BLOCK);
        };

        return resource;
    });
}());
