(() => {
    'use strict';

    angular.module('noc').factory('CreateWorkingEnvironmentWizardService', ($rootScope, nocLocalStorage,
                                                                            $q, MetadataResource, $state, nocCurrentUserService) => {
        //Don't change to arrow function, will ruin the 'this' and cause a lot of pain
        return function (nocHttp, nocUI) {
            const scope = this;

            scope.isMarketPlaceRegistered = function isMarketPlaceRegistered() {
                const provider = scope.wizardController.workingEnvironmentTypeAndProvider.providerName;

                if (provider === 'GCP' || (provider === 'Azure' && nocLocalStorage.DISABLE_AZURE_MP_CHECK.restore(false))) {
                    return $q.when({subscribed: true});
                }

                const request = {
                    region: scope.wizardController.selectedRegion,
                    subnetId: provider === 'Azure' ? scope.wizardController.selectedSubnet.id : scope.wizardController.createVpcSelectObjects.subnet.subnetId,
                    vsaMetadata: {
                        ontapVersion: scope.wizardController.selectedOntapVersion,
                        licenseType: scope.wizardController.selectedLicense.type,
                        instanceType: scope.wizardController.selectedInstanceType,
                        providedLicense: scope.wizardController.providedLicense
                    }
                };

                if (provider === 'Azure'){
                    request.subscriptionId = scope.wizardController.selectedAzureSubscription.subscriptionId;
                }

                if (scope.wizardController.cloudProviderAccountId) {
                    request.cloudProviderAccount = scope.wizardController.cloudProviderAccountId;
                }

                return MetadataResource.isRegisteredToMarketPlace(nocHttp, request, scope.wizardController.workingEnvironmentType, provider);
            };

            scope.continueAfterLicenseOrPackagesStep = function continueAfterLicenseOrPackagesStep(finalStep) {
                if (scope.isValid()) {
                    nocUI.clearFormValidity();
                    if ($rootScope.runningInIC) {
                        $state.go(finalStep);
                    } else {
                        return scope.isMarketPlaceRegistered()
                            .then((registered) => {
                                if (registered.subscribed) {
                                    $state.go(finalStep);
                                } else {
                                    $state.go('.check-mp-registration');
                                }
                            });
                    }
                } else {
                    nocUI.handleErrorsInWizard(true);
                }
            };
        };
    });
})();
