(() => {
    'use strict';

    angular.module('noc').factory('nocCurrentUserService', () => {
        const currentUserService = {};
        let currentUser = {};
        let roles;
        let agentMode;

        currentUserService.initializeCurrentUser = (user, http, agent, userMetadataObj) => {
            currentUser = user;
            agentMode = agent;

            if (!roles && http) {
                currentUserService.initializeRoles(http);
            }

            if (userMetadataObj) {
                currentUser.auth0Id = userMetadataObj.auth0Id;
                currentUser.email = userMetadataObj.email;
                currentUser.firstName = userMetadataObj.name.substr(0, userMetadataObj.name.indexOf(' '));
                currentUser.lastName =  userMetadataObj.name.substr(userMetadataObj.name.indexOf(' ') + 1, userMetadataObj.name.length);
            }
        };

        currentUserService.initializeRoles = (http) => {
            return http.get('/roles').then((rolesResponse) => {
                roles = rolesResponse.data;
            });
        };

        currentUserService.isCurrentUserExist = () => {
            return currentUser !== null;
        };

        currentUserService.getRoles = () => {
            return roles;
        };

        currentUserService.getCurrentUserPermissions = () => {
            return _.get(currentUser, 'role.permissions');
        };

        //Backwards compatibility:  this getter is for tenants_service & intercom
        currentUserService.getCurrentUser = () => {
            return currentUser;
        };

        //Backwards compatibility:  this getter is for old pages, so there is defiantly role.publicId
        currentUserService.getCurrentUserRolePublicId = () => {
            return _.get(currentUser, 'role.publicId');
        };

        currentUserService.getCurrentUserPublicId = () => {
            return _.get(currentUser, 'publicId');
        };

        currentUserService.getCurrentUserAuth0Id = () => {
            return _.get(currentUser, 'auth0Id');
        };

        currentUserService.getCurrentUserCloudSyncLicenseInformation = () => {
            return _.get(currentUser, 'cloudSyncLicenseInformation');
        };

        currentUserService.getCurrentUserCloudSyncLicenseTypes = () => {
            return _.get(currentUser, 'cloudSyncLicenseInformation.cloudSyncLicenseTypes');
        };

        currentUserService.getCurrentUserCloudSyncLicenseExpirationDate = () => {
            return _.get(currentUser, 'cloudSyncLicenseInformation.licenseExpirationDate');
        };

        currentUserService.getCurrentUserCloudSyncLicenseExpired = () => {
            return _.get(currentUser, 'cloudSyncLicenseInformation._cloudSyncLicenseExpired');
        };


        const isMatch = (requiredPermission, userPermission) => {
            requiredPermission = _.split(requiredPermission, ':');
            userPermission = _.split(userPermission, ':');
            userPermission = _.take(userPermission, 2);
            if (userPermission[1] === '*') {
                return _.isEqual(userPermission[0], requiredPermission[0]);
            } else {
                return _.isEqual(userPermission, requiredPermission);
            }
        };


        currentUserService.isAllowed = (requiredPermissions, selectedWorkspace) => {
            if (!currentUser) return false;

            requiredPermissions = _.isString(requiredPermissions) && !_.isEmpty(requiredPermissions) ? [requiredPermissions] : _.toArray(requiredPermissions);
            const userPermissions = currentUser.permissions;

            if (agentMode && currentUser.roleId === 'Role-2') {
                const required = _.split(requiredPermissions, ':');
                if ((required[0] === 'we' || required[0] === 'onprem') && (required[1] === 'create' || required[1] === 'delete' || required[1] === 'discover')) {
                    return selectedWorkspace.associatedToAgent;
                }
            }

            let isFail = false;
            _.forEach(requiredPermissions, (requiredPermission) => {
                let hasMatch = false;
                _.forEach(userPermissions, (userPermission) => {
                    if (isMatch(requiredPermission, userPermission)) {
                        hasMatch = true;
                        return false;
                    }
                });
                if (!hasMatch) {
                    isFail = true;
                    return false;
                }
            });
            return !isFail;
        };

        return currentUserService;
    });
})();
