(function () {
    'use strict';

    angular.module('noc').service('nocDialog', function ($q, $document, Messageable, $compile, $rootScope, $http, $templateCache, nocUIMain, $timeout, Constants) {
        let reAllowNavigation;
        const createDialog = function (opts) {
            const unblock = nocUIMain.block('createDialog', true);
            const deferred = opts.deferred;

            $q.all([
                $http.get(opts.templateUrl, {cache: $templateCache}),
                $http.get('assets/templates/common/messages-container.html', {cache: $templateCache})
            ]).then((htmls) => {
                const scope = $rootScope.$new();
                scope.isDialogScope = true;
                $rootScope.isDialogOpen = true;
                if (opts.locals) {
                    _.assignIn(scope, opts.locals);
                }

                Messageable.call(scope);

                nocUIMain.ignoreFormValidity();

                //NOC-6902
                reAllowNavigation = $rootScope.$on('$locationChangeStart', (event) => {
                    event.preventDefault(); // Prevents the navigation. See http://www.markcampbell.me/tutorial/2013/10/08/preventing-navigation-in-an-angularjs-project.html
                }
                );

                const dialogContent = $(htmls[0].data);
                const messageContainer = $(htmls[1].data);
                dialogContent.append(messageContainer);
                const element = $compile(dialogContent)(scope);

                $document.find('body').append(element);
                opts.constructor(element, scope, opts, deferred);

                const dialog = $('.ui-dialog');

                $timeout(() => {
                    dialog.children().wrapAll('<div class="dialog-container"></div>');
                    messageContainer.prependTo(dialog);
                    if (scope.callback) scope.callback();
                });
            })
                .catch(() => {
                    throw new Error('Could not find dialog templates');
                })
                .finally(unblock);

            return {
                deferred
            };
        };

        const constructConfirm = function (element, scope, opts, deferred) {
            scope.modalController = scope;
            const at = scope.fromTop ? 'center top+' + scope.fromTop : 'center top+30%';
            let shouldResolve = false;
            scope.modalOk = function () {
                shouldResolve = true;
                $rootScope.$broadcast(Constants.broadcastEventNames.MODAL_OK_CLICKED, opts.id);
                $timeout(() => {
                    element.dialog('close');
                });
            };

            scope.modalCancel = function () {
                $timeout(() => {
                    element.dialog('close');
                });
            };

            scope.modalOkIfValid = function (formName) {
                if (!scope[formName] || scope[formName].$valid) {
                    scope.modalOk();
                } else {
                    nocUIMain.handleFormValidity();
                }
            };

            element.dialog({
                dialogClass: 'no-close confirm' + (opts.title ? '' : ' no-title-dialog') + ($rootScope.appView === 'volume-view' ? ' volume-view-confirm' : '') + ' ' + (scope.dialogClass || ''),
                modal: true,
                title: opts.title,
                minHeight: 0,
                width: scope.width ? scope.width : ($rootScope.appView === 'volume-view' ? 600 : 415),
                resizable: false,
                position: {my: 'center top', at, of: window},
                close() {
                    return destroyDialog(element, scope).then(() => {
                        if (shouldResolve) deferred.resolve(scope);
                        else deferred.reject();
                    });
                }
            });

            //Close dialog when clicking anywhere outside
            if (opts.locals.clickOutsideToClose) {
                $('.ui-widget-overlay').click(() => {
                    element.dialog('close');
                });
            }

            animateEntrance(element);
        };

        const constructDialog = function (element, scope, opts, deferred) {
            const at = scope.fromTop ? ('center top+' + scope.fromTop) : 'center top+30%';

            scope.modalController = scope;

            scope.closeDialog = function () {
                $timeout(() => {
                    element.dialog('close');
                });
            };

            element.dialog({
                dialogClass: 'no-close dialog' + (opts.title ? '' : ' no-title-dialog') + ($rootScope.appView === 'volume-view' ? ' volume-view-dialog' : ''),
                modal: true,
                title: opts.title,
                minHeight: 0,
                width: scope.width ? scope.width : 600,
                resizable: false,
                position: {my: 'center top', at, of: window},
                close() {
                    return destroyDialog(element, scope).then(() => {
                        deferred.resolve();
                    });
                }
            });

            animateEntrance(element);

            return element;
        };

        const constructAlert = function (element, scope, opts, deferred) {
            scope.modalController = scope;

            scope.closeDialog = function () {
                $timeout(() => {
                    element.dialog('close');
                });
            };

            element.dialog({
                dialogClass: 'no-close message' + (opts.title ? '' : ' no-title-dialog') + ($rootScope.appView === 'volume-view' ? ' volume-view-dialog' : ''),
                modal: true,
                title: opts.title,
                minHeight: 0,
                width: 364,
                resizable: false,
                position: {my: 'center top', at: 'center top+30%', of: window},
                close() {
                    return destroyDialog(element, scope)
                        .then(() => {
                            deferred.resolve();
                        });
                }
            });

            animateEntrance(element);

            return element;
        };

        const animateEntrance = function (element) {
            const activeElement = document.activeElement;
            const parent = element.parent();
            parent.css('opacity', 0);
            const overlay = parent.next();
            overlay.css('opacity', 0);

            $timeout(() => {
                parent.addClass('enter');
                overlay.addClass('enter');
                parent.css('opacity', 1);
                overlay.css('opacity', '');
                $timeout(() => {
                    parent.removeClass('enter');
                    overlay.removeClass('enter');
                    $timeout(() => {
                        let nocFocussed = parent.find('[noc-focus]');
                        if (nocFocussed.length < 1) nocFocussed = parent.find('[autofocus]');
                        if (nocFocussed.length < 1 || $(activeElement).is(nocFocussed.get(0))) {
                            $(activeElement).focus();
                        } else {
                            nocFocussed.get(0).focus();
                        }
                    });
                    nocUIMain.clearFormValidity();
                }, 500);
            });
        };

        const destroyDialog = function (element, scope) {
            const deferred = $q.defer(); //eslint-disable-line angular/deferred
            const parent = element.parent();
            parent.css('display', 'block');
            parent.removeClass('enter');
            parent.addClass('exit');
            parent.css('opacity', 0);
            parent.css('top', -1 * parent.outerHeight(true));
            nocUIMain.ignoreFormValidity();
            reAllowNavigation();

            $timeout(() => {
                element.dialog('destroy');
                element.remove();
                scope.$destroy();
                deferred.resolve();
                nocUIMain.clearFormValidity();
                $rootScope.isDialogOpen = false;
            }, 250);

            return deferred.promise;
        };

        // This uses promise chaining to be able to call two dialogs and get them in order.
        // NOC-10840  UI - Dialogs race conditions (two simulatanious dialogs)
        let lastPromise = $q.when();
        const registerDialogOpening = function (args) {
            args.deferred = $q.defer(); //eslint-disable-line angular/deferred

            lastPromise.finally(() => {
                createDialog(args);
            });

            lastPromise = args.deferred.promise;

            return lastPromise;
        };

        this.dialog = function (title, templateUrl, locals) {
            const args = {title, templateUrl, constructor: constructDialog, locals};

            return registerDialogOpening(args);
        };

        this.alert = function (title, locals) {
            const args = {
                title,
                templateUrl: 'assets/templates/dialogs/alert-content.html',
                locals,
                constructor: constructAlert
            };

            return registerDialogOpening(args);
        };

        /*
         TemplateUrl and Locals are both optional.
         If templateUrl is present but not a string then it is treated as locals
         */
        this.confirm = function (text, templateUrl, locals, id) {
            if (!_.isString(templateUrl)) {
                templateUrl = 'assets/templates/dialogs/default-confirm-content.html';

                if (templateUrl && !locals) locals = templateUrl;
            }

            const args = {title: text, templateUrl, constructor: constructConfirm, locals, id};

            return registerDialogOpening(args);
        };
    });
}());
