(() => {
    'use strict';

    angular.module('noc').service('galleryTableColumnsService', (uiGridConstants, nocLocalStorage) => {
        const service = {};

        const titledCellTemplate = '<div class="ui-grid-cell-contents vertical-aligned" title="{{COL_FIELD}}">{{COL_FIELD}}</div>';
        const titledCellTemplateWithRed = '<div class="ui-grid-cell-contents vertical-aligned" ng-class="{red: row.entity.status.status === \'failed\' || row.entity.status.status === \'not-found\' || row.entity.status.status === \'failed-to-create\', yellow: row.entity.status.status === \'degraded\'}" title="{{COL_FIELD}}">{{COL_FIELD}}</div>';
        const capitalizedTitledCellTemplateWithRed = _.replace(titledCellTemplateWithRed, 'title="{{COL_FIELD}}"', 'title="{{COL_FIELD | __capitalize}}"');

        const defaultColumnsDef = [
            {displayName: 'Name', visible: true, default: true, sort: {direction: uiGridConstants.DESC, priority: 0}, field: 'name', headerCellClass: 'havitush',
                disabled: true, pinnedLeft: true, minWidth: 120, maxWidth: 300,
                cellTemplate: titledCellTemplateWithRed},
            {displayName: 'Notifications', visible: true, default: true, field: '_notifications', minWidth: 70, maxWidth: 140, cellTemplate: 'assets/templates/tabs/working-environments/notifications-cell-template.html'},
            {displayName: 'Status', visible: true, default: true, cellClass: 'capitalize', field: '_displayStatus', minWidth: 125, cellTemplate: capitalizedTitledCellTemplateWithRed},
            {displayName: 'Type', visible: true, default: true, field: '_typeLongName', minWidth: 230, cellTemplate: titledCellTemplate},
            {displayName: 'Cloud Provider', visible: true, default: true, cellClass: 'capitalize', field: '_cloudProviderName', minWidth: 160, cellTemplate: titledCellTemplate},
            {displayName: 'Region', visible: true, default: true, field: '_regionName', minWidth: 160, cellTemplate: titledCellTemplate},
            {displayName: 'Encryption', visible: true, default: true, field: '_encryptionType', minWidth: 130, cellTemplate: titledCellTemplate},
            {displayName: 'Instance Type', visible: false, default: false, field: '_instanceType', minWidth: 160, cellTemplate: titledCellTemplate},
            {displayName: 'ONTAP Version', visible: false, default: false, field: '_ontapVersion', minWidth: 160, cellTemplate: titledCellTemplate},
            {displayName: 'License Type', visible: false, default: false, field: '_licenseType', minWidth: 200, cellTemplate: titledCellTemplate},
            {displayName: 'Allocated Capacity', visible: false, default: false, field: '_normalizedReservedSize', minWidth: 160, cellTemplate: titledCellTemplate},
            {displayName: 'Peers', visible: false, default: false, field: '_peers', minWidth: 160, cellTemplate: titledCellTemplate},
            {displayName: 'Serial Number', visible: false, default: false, field: '_serialNumber', minWidth: 360, cellTemplate: titledCellTemplate},
            {displayName: 'Write Speed', visible: false, default: false, cellClass: 'capitalize', field: '_writingSpeedState', minWidth: 120, cellTemplate: capitalizedTitledCellTemplateWithRed},
            {displayName: 'Account ID / Subscription Name', visible: true, default: false, field: '_accountOrSubscriptionName', minWidth: 240, cellTemplate: titledCellTemplate}
        ];

        _.forEach(defaultColumnsDef, (col) => {
            col.width = '*';
        });

        service.getColumnsDef = () => defaultColumnsDef;

        service.restoreGridState = (gridApi, scope) => {
            const restored = nocLocalStorage.GALLERY_GRID.restore(gridApi);
            if (restored) {
                gridApi.saveState.restore(scope, restored);
            }
        };

        service.saveGridState = (gridApi) => {
            if (gridApi) {
                const state = gridApi.saveState.save();
                nocLocalStorage.GALLERY_GRID.save(state);
            }
        };

        return service;
    });
})();
