(() => {
    'use strict';

    angular.module('noc').service('replicationTableColumnsService', (uiGridConstants, nocLocalStorage, uiGridUtilities) => {
        const service = {};
        const sortByField = (fieldName) => {
            return (a, b) => { //eslint-disable-line id-length
                return a[fieldName] <= b[fieldName] ? 1 : -1;
            };
        };
        const titledCellTemplate = uiGridUtilities.titledCellTemplate,
            policyCellTemplate = "<div class='ui-grid-cell-contents' title='{{COL_FIELD.tooltip}}'><div class='trimmed-text'>{{COL_FIELD.displayName}}</div></div>",
            sourceDestCellTemplate = "<div class='ui-grid-cell-contents'><div class='trimmed-text'><strong title='{{COL_FIELD.volumeName}}'>{{COL_FIELD.volumeName}}</strong></div>" +
                "<div class='trimmed-text' title='{{COL_FIELD.clusterName}} {{COL_FIELD.workingEnvironmentStatus !== \"ON\" ? \"(\" + COL_FIELD._stateInfo.title +\")\" : \"\"}}'>" +
                "<span ng-class='{red: COL_FIELD.workingEnvironmentStatus !== \"ON\"}'>{{COL_FIELD.clusterName}}</span></div></div>";

        const defaultColumnsDef = [
            {
                displayName: 'Source',
                visible: true,
                sort: {direction: uiGridConstants.DESC, priority: 0},
                field: 'source',
                disabled: true,
                pinnedLeft: true,
                default: true,
                minWidth: 160,
                maxWidth: 360,
                _filterBy: ['source.clusterName', 'source.volumeName', 'source.workingEnvironmentStatus'],
                cellTemplate: sourceDestCellTemplate,
                headerCellClass: 'havitush',
                sortingAlgorithm: sortByField('volumeName')
            },
            {displayName: 'Source SVM Name', visible: false, field: 'source.svmName', minWidth: 170},
            {displayName: 'Source AZ', visible: false, field: 'source.availabilityZone', minWidth: 170},
            {
                displayName: 'Target',
                visible: true,
                field: 'destination',
                disabled: true,
                pinnedLeft: true,
                default: true,
                minWidth: 160,
                maxWidth: 360,
                _filterBy: ['destination.clusterName', 'destination.volumeName', 'destination.workingEnvironmentStatus'],
                cellTemplate: sourceDestCellTemplate,
                headerCellClass: 'havitush havitush2',
                sortingAlgorithm: sortByField('volumeName')
            },
            {displayName: 'Target SVM Name', visible: false, field: 'destination.svmName', minWidth: 170},
            {displayName: 'Target AZ', visible: false, field: 'destination.availabilityZone', minWidth: 170},
            {
                displayName: 'Lag Duration', visible: true, field: '_lagDuration', default: true, minWidth: 120,
                _filterBy: ['_lagDuration.viewValue'],
                cellTemplate: '<div class=\'ui-grid-cell-contents\'>{{COL_FIELD.viewValue}}</div>',
                sortingAlgorithm: sortByField('sortValue')
            },
            {
                displayName: 'Relationship Health', visible: true, field: '_health', default: true, minWidth: 120,
                cellTemplate: '<div class="ui-grid-cell-contents" title="{{COL_FIELD}}">' +
                '<div class="trimmed-text"><i class="fa" ng-class="{\'fa-check\': COL_FIELD === \'Healthy\', \'fa-times\': COL_FIELD !== \'Healthy\'}"></i>  {{COL_FIELD === \'Healthy\' ? \'Healthy\' : \'Failed\'}}</div>' +
                '<div class="\'"trimmed-text"\'" ng-if="COL_FIELD !== \'Healthy\'">{{COL_FIELD}}</div>' +
                '</div>'
            },
            {
                displayName: 'Status',
                visible: true,
                field: 'relationshipStatus',
                default: true,
                minWidth: 70,
                _filterBy: ['relationshipStatus', '_relationshipProgress'],
                cellTemplate: '<div class=\'ui-grid-cell-contents\'><div class=\'trimmed-text\' title=\'{{row.entity.relationshipStatus}}\'>{{row.entity.relationshipStatus}}</div><div title=\'{{row.entity._relationshipProgress}}\'>{{row.entity._relationshipProgress}}</div></div>',
                sortingAlgorithm: sortByField('relationshipStatus')
            },
            {
                displayName: 'Mirror State',
                visible: true,
                disabled: true,
                field: 'mirrorState',
                default: true,
                minWidth: 150
            },
            {
                displayName: 'Last Successful Transfer',
                visible: true,
                field: 'lastTransferInfo',
                default: true,
                minWidth: 170,
                _filterBy: ['lastTransferInfo._transferEnded', 'lastTransferInfo._transferSize'],
                cellTemplate: '<div class=\'ui-grid-cell-contents\'><div class=\'trimmed-text\' title=\'{{COL_FIELD._transferEnded}}\'>{{COL_FIELD._transferEnded}}</div><div title=\'{{COL_FIELD._transferSize}}\'>{{COL_FIELD._transferSize}}</div></div>',
                sortingAlgorithm: sortByField('transferEnded')
            },
            {displayName: 'Relationship Type', visible: false, field: 'relationshipType', minWidth: 170},
            {displayName: 'Used Size', visible: false, field: '_volumeUsedSize', minWidth: 170},
            {displayName: 'Used Size On S3', visible: false, field: '_volumeCapacityTier', minWidth: 170},
            {
                displayName: 'Policy',
                visible: true,
                field: '_policyName',
                default: true,
                minWidth: 90,
                cellTemplate: policyCellTemplate,
                _filterBy: ['_policyName.displayName']
            },
            {displayName: 'Policy Type', visible: false, field: 'policyType', minWidth: 120},
            {displayName: 'Schedule', visible: true, field: '_schedule', default: true, minWidth: 100},
            {displayName: 'Max Transfer Rate', visible: false, field: '_maxTransferRate', minWidth: 170},
            {displayName: 'Network Compression Ratio', visible: false, field: 'networkCompressionRatio', minWidth: 150},
            {
                displayName: 'Newest Snapshot copy Created',
                visible: false,
                field: '_newestSnapshotCreated',
                minWidth: 170,
                cellTemplate: '<div class=\'ui-grid-cell-contents\'>{{COL_FIELD.viewValue}}</div>',
                _filterBy: ['_newestSnapshotCreated.viewValue'],
                sortingAlgorithm: sortByField('sortValue')
            },
            {displayName: 'Last Transfer Type', visible: false, field: 'lastTransferInfo.transferType', minWidth: 170},
            {
                displayName: 'Last Transfer Duration',
                visible: false,
                field: 'lastTransferInfo._transferDuration',
                minWidth: 170
            },
            {
                displayName: 'Last Transfer Error',
                visible: false,
                field: 'lastTransferInfo.transferError',
                minWidth: 170,
                cellTemplate: titledCellTemplate
            },
            {displayName: 'Transfer Type', visible: false, field: 'currentTransferInfo.transferType', minWidth: 170},
            {
                displayName: 'Transfer Priority',
                visible: false,
                field: 'currentTransferInfo.transferPriority',
                minWidth: 170
            },
            {
                displayName: 'Transfer Error',
                visible: false,
                field: 'currentTransferInfo.transferError',
                minWidth: 170,
                cellTemplate: titledCellTemplate
            },
            {
                displayName: 'Total Transfer Time', visible: false, field: '_totalTransferTime', minWidth: 170,
                _filterBy: ['_totalTransferTime.viewValue'],
                cellTemplate: '<div class=\'ui-grid-cell-contents\'>{{COL_FIELD.viewValue}}</div>',
                sortingAlgorithm: sortByField('sortValue')
            },

            {
                displayName: 'Total Transfer Size', visible: false, field: '_totalTransferSize', minWidth: 170,
                cellTemplate: '<div class=\'ui-grid-cell-contents\'>{{COL_FIELD.viewValue}}</div>',
                _filterBy: ['_totalTransferSize.viewValue'],
                sortingAlgorithm: sortByField('sortValue')
            }
        ];

        _.forEach(defaultColumnsDef, (col) => {
            col.width = '*';
        });

        service.getColumnsDef = () => defaultColumnsDef;

        service.restoreGridState = (gridApi, scope) => {
            const restored = nocLocalStorage.REPLICATION_STATUS_GRID.restore(gridApi);
            if (restored) {
                gridApi.saveState.restore(scope, restored);
            }
        };

        service.saveGridState = (gridApi) => {
            if (gridApi) {
                const state = gridApi.saveState.save();
                nocLocalStorage.REPLICATION_STATUS_GRID.save(state);
            }
        };

        return service;
    });
})();
