(function(){
    "use strict";

    var setDebugEnabled;

    angular.module("noc").config(function($provide,$logProvider) {
        setDebugEnabled = function(nocLocalStorage){
            var enabled = nocLocalStorage.DEBUG.restore();
            if(_.isNil(enabled)){
                enabled = false;

                //make sure the key appears in the localstore so it will be easy to change
                nocLocalStorage.DEBUG.save(false);
            }

            $logProvider.debugEnabled(enabled);
        };


        $provide.decorator('$log', function($delegate, logStore) {
            return _.mapValues($delegate, function(func, funcName){
                return function(){
                    logStore.log(funcName === "log" ? "info" : funcName, arguments[0]);
                    func.apply(this, arguments);
                };
            });
        });
    });

    angular.module("noc").factory("logStore", function($document, nocLocalStorage){
        var logStore = {};

        logStore.extractError = function(arg){
            return arg.stack && arg.message ? arg.message + "\n" + arg.stack : arg;
        };

        logStore.log = function(level, arg){
            var logs = nocLocalStorage.LOG_STORAGE.restore([]);
            logs.push({
                level: level,
                message: logStore.extractError(arg),
                time: moment().utc().format("YYYY-MM-DD HH:mm:ss")
            });

            nocLocalStorage.LOG_STORAGE.save(logs);
        };

        logStore.popAll = () => {
            const logs = nocLocalStorage.LOG_STORAGE.restore([]);
            nocLocalStorage.LOG_STORAGE.clear();

            return logs;
        };

        _.forEach($document.find('body').data('initialLogStore'), (error) => {
            logStore.log('error', error);
        });

        return logStore;
    });

    angular.module('noc').factory('loggerService', (logStore, nocPoller, nocLocalStorage) => {
        setDebugEnabled(nocLocalStorage);

        const loggerService = {};

        loggerService.paramsFn = () => {
            const currLogs = logStore.popAll();
            if (currLogs.length === 0) return null;
            else {
                return {
                    url: '/occm/client-logs',
                    method: 'POST',
                    data: currLogs
                };
            }
        };

        loggerService.start = _.once(() => {
            nocPoller.start(loggerService.paramsFn, 10 * 1000, _.noop);
        });

        return loggerService;
    });

    angular.module('noc').factory('routeLogger', ($rootScope, nocPoller) => {

        let batch = [],
            stayTime = new Date(),
            currentState;
        const loggerService = {};

        function init() {

            $rootScope.$on('$stateChangeSuccess', (event, toState, toParams) => {

                if (currentState) {
                    batch.push({state: currentState.name, mode: toParams.gadget || '', dwell: new Date() - stayTime, time: new Date().getTime()});
                }

                currentState = toState;
                stayTime = new Date();
            });


        }

        loggerService.paramsFn = () => {
            let dst = [];
            if (batch.length === 0) return null;
            else {
                angular.copy(batch, dst);
                batch = [];

                return {
                    url: '/occm/client-logs/html',
                    method: 'POST',
                    data: dst
                };
            }
        };

        loggerService.start = _.once(() => {
            nocPoller.start(loggerService.paramsFn, 10 * 1000, _.noop);
        });

        loggerService.reportGalleryAction = (actionName) => {
            batch.push({state: actionName, mode: 'gallery', dwell: 0});
        };

        init();

        return loggerService;


    });

}());