(function () {
    "use strict";

    angular.module("noc").service("nocLongPollingService", function ($q, $injector, NocErrorResponse,$log) {

        var clientId = Math.random().toString(36).substring(2, 16);

        var service = {};

        service.REQUEST_ID_REQUEST_HEADER = 'X-Occm-Use-Async';
        service.REQUEST_ID_RESPONSE_HEADER = 'X-Occm-Async-Request-Id';

        var lastRequestId = null;
        var lastPollRequest = null;
        var unhandledResponses = [];

        var startAskingIfNeeded = function() {
            if (!_.isEmpty(service.requestsPromiseMap) && !lastPollRequest) {
                lastPollRequest = $injector.get("$http").get('/async/' + clientId + '?last=' + lastRequestId)
                    .then(service.dispatchResponses)
                    .catch(function (error) {
                        $log.error("error polling async responses: "+ error.data);
                    }).finally(function() {
                        lastPollRequest = null;
                        startAskingIfNeeded();
                    });
            }
        };

        service.requestsPromiseMap = {};

        service.getAsyncResponse = function(response) {
            var deferred = $q.defer(); //eslint-disable-line angular/deferred
            var requestId = response.headers(service.REQUEST_ID_RESPONSE_HEADER);

            var matchedResponsesArray = _.remove(unhandledResponses, {requestId: requestId});
            if (matchedResponsesArray.length > 0) {
                service.handleResponse(response, deferred, matchedResponsesArray[0]);
            } else {
                service.requestsPromiseMap[requestId] = { r: response, d: deferred };
            }

            startAskingIfNeeded();
            return deferred.promise;
        };

        service.dispatchResponses = function(responses) {
            unhandledResponses = _.reject(unhandledResponses.concat(responses.data), service.handleResponseIfPossible);
        };

        service.handleResponseIfPossible = function(response) {
            lastRequestId = response.requestId;
            var data =  service.requestsPromiseMap[response.requestId];
            if (data) { // we skip processing responses for request id that we do no have a promise for
                delete service.requestsPromiseMap[response.requestId];
                service.handleResponse(data.r, data.d, response);
                return true;
            } else {
                return false;
            }
        };

        service.handleResponse = function(origResponse, deferred, asyncResponse) {
            origResponse.status = asyncResponse.statusCode;
            if (origResponse.status >= 200 && origResponse.status <= 300) { // Success
                origResponse.data = asyncResponse.response;
                deferred.resolve(origResponse);
            } else { // Error
                origResponse.data = new NocErrorResponse(asyncResponse.response);
                deferred.reject(origResponse);
            }
        };

        service.getClientId = function() {
            return clientId;
        };

        return service;
    });


}());