(() => {
    'use strict';

    angular.module('noc').factory('nocNavigateAway', ($timeout, Messages, nocDialog, $rootScope) => {
        const service = {};

        service.eventName = 'navigateAwayRequest';

        service.sendNavigateAwayMessage = (proceedFunction) => {
            const payload = {
                cancel() {
                    payload.cancelled = true;
                },
                proceedFunction() {
                    $timeout(proceedFunction);
                }
            };

            $rootScope.$broadcast(service.eventName, payload);

            if (!payload.cancelled) {
                payload.proceedFunction();
            }
        };

        service.handleNavigateAway = (payload, messageOrFunction) => {
            payload.cancel();
            if (_.isFunction(messageOrFunction)) {
                messageOrFunction(payload.proceedFunction);
            } else {
                nocDialog
                    .confirm(_.isUndefined(messageOrFunction) ? Messages.defaultConfirm : messageOrFunction)
                    .then(payload.proceedFunction);
            }
        };

        service.promptOnLeaveBeforeDiscardingChanges = (controllerScope) => {
            return controllerScope.$on(service.eventName, (event, payload) => {
                if (controllerScope.isDirty()) {
                    service.handleNavigateAway(payload);
                }
            });
        };

        return service;
    });
})();
