(() => {
    'use strict';

    angular.module('noc').factory('nocHttpFactory', ($http, $q, $rootScope, Constants) => {
        function spawn(pidgeon, nocUI) {
            function call(config, description, changeUIWhileCallingServer) {
                const localPidgeon = {
                    //The description is used as the callerType too
                    clear: () => {
                        if (description) pidgeon.clear(description);
                    },
                    error: (error) => {
                        if (description) pidgeon.error(pidgeon.serverErrorPrefix + description, error.data, description);
                    }
                };

                localPidgeon.clear();
                const backToNormal = changeUIWhileCallingServer();

                if (_.get($rootScope.supportServices, 'portalService.usePortalAuthentication')) {
                    $http.defaults.headers.common.Authorization = $rootScope.loginToken;
                }

                return $http(config)
                    .catch((error) => {
                        if (error.status === 409) {
                            $rootScope.$broadcast(Constants.ASYNC_OPERATION_ERROR_EVENT, error.data, pidgeon);
                        } else {
                            localPidgeon.error(error);
                        }
                        error.data.status = error.status;

                        return $q.reject(error.data);
                    })
                    .finally(backToNormal);
            }

            function shortcut(method) {
                return (urlOrConfig, description, changeUIWhileCallingServer) => {
                    const config = _.isString(urlOrConfig) ? {url: urlOrConfig} : urlOrConfig;
                    const configWithMethod = _.assign(config, {method});

                    return call(configWithMethod, description, changeUIWhileCallingServer || _.noop)
                        .then((response) => {
                            return response.data;
                        });
                };
            }

            //Based on blog post here: http://jaliyaudagedara.blogspot.com/2016/05/angularjs-download-files-by-sending.html
            function download(url, description, changeUIWhileCallingServer) {
                const requestConfig = {
                    method: 'GET',
                    url,
                    responseType: 'arraybuffer',
                    cache: 'false',
                    headers: {'Accept': undefined}
                };

                return call(requestConfig, description, changeUIWhileCallingServer || _.noop)
                    .then((response) => {
                        const data = response.data;
                        const headers = response.headers();

                        const filename = _.split(headers['content-disposition'], '=')[1];
                        const contentType = headers['content-type'];

                        const linkElement = document.createElement('a');
                        try {
                            const blob = new Blob([data], {type: contentType}, filename);
                            const url = window.URL.createObjectURL(blob);

                            linkElement.setAttribute('href', url);
                            linkElement.setAttribute('download', filename);

                            const clickEvent = new MouseEvent('click', {
                                'view': window,
                                'bubbles': true,
                                'cancelable': false
                            });
                            linkElement.dispatchEvent(clickEvent);
                        } catch (ex) {
                            return $q.reject(ex);
                        }
                    });
            }

            return {
                //Constants for changeUIWhileCallingServer
                BLOCK: _.bind(nocUI.block, nocUI),
                LOADING: _.bind(nocUI.loading, nocUI),

                //Generic call method
                call,

                //Shortcut methods
                get: shortcut('GET'),
                put: shortcut('PUT'),
                post: shortcut('POST'),
                'delete': shortcut('DELETE'),

                download
            };
        }

        return {spawn};
    });
})();
