(() => {
    'use strict';

    angular.module('noc').service('nocRoutingResolverService', (systemResource, $q, $log, $state, nocLocalStorage, $rootScope, nocCurrentUserService) => {
        // Returns a promise that resolves if and only if OCCM is running on AWS, nocHttp is optional - it defaults to use $http.
        const isOnAWS = (nocHttp) => {
            return systemResource(nocHttp).getInstancePlacement(nocHttp && nocHttp.LOADING)
                .then((response) => {
                    return response && response.installLocation === 'Azure' ? $q.reject(service.VOL_VIEW_NOT_ALLOWED) : $q.when();
                }, () => {
                    $log.log('Can not recognize OCCM\'s cloud provider');

                    return $q.reject(service.VOL_VIEW_NOT_ALLOWED);
                });
        };

        let service = {
            VOL_VIEW_NOT_ALLOWED: 'NOT_ALLOWED',
            // Use this when user tries to go to an unknown url.
            resolveEntryPoint(nocHttp) {
                // Check if user is logged in
                if ($rootScope.appState === 'unauthorized' || $rootScope.appState === 'login') return;

                if ($rootScope.appState === 'portal-setup-wizard') {
                    $state.go('auth0-setup-wizard');

                    return;
                }

                if ($rootScope.appState === 'setup-server-registration-error') {
                    $state.go('auth0-setup-failed-register');

                    return;
                }

                if ($rootScope.appState === 'failed-reach-auth0') {
                    $state.go('auth0-failed-reach');

                    return;
                }

                if ($rootScope.appState === 'after-migration') {
                    $state.go('after-migration');

                    return;
                }

                if ($rootScope.appState === 'setup-wizard') {
                    if (_.get($rootScope, 'supportServices.volumeViewEnabled')) {
                        $state.go('anterior-setup-wizard');
                    } else {
                        $rootScope.noAnteriorSetupWizard = true;
                        $state.go('setup-wizard.welcome');
                    }

                    return;
                }

                this.isVolumeViewAllowed($rootScope, nocHttp).then(() => {
                    if (nocLocalStorage.VOLUME_VIEW.isDefined()) {
                        const value = nocLocalStorage.VOLUME_VIEW.restore();
                        const gotoState = value ? 'volume-view.volumes' : 'main';

                        return $state.go(gotoState);
                    }

                    nocHttp.get('occm/config', 'retrieving configurations', nocHttp.LOADING).then((config) => {
                        const isVolViewDefault = config.useVolumeViewAsDefault;
                        nocLocalStorage.VOLUME_VIEW.save(isVolViewDefault);

                        return isVolViewDefault ? $state.go('volume-view.volumes') : $state.go('main');
                    });
                })
                    .catch(() => {
                        $state.go('main');
                    });
            },

            // Resolves if and only if the user is ALLOWED to go to volume view
            isVolumeViewAllowed($rootScope) {
                // Check for support services
                if (!$rootScope.supportServices.volumeViewEnabled) return $q.reject(service.VOL_VIEW_NOT_ALLOWED);
                // Check for user role, only admin is allowed to enter volume view
                //if (!nocCurrentUserService.isAllowed('volume_view:*'))

                //No more VV
                return $q.reject(service.VOL_VIEW_NOT_ALLOWED);

                //return isOnAWS(nocHttp);
            }
        };

        return service;
    });
})();
