(function () {
    "use strict";

    angular.module("noc").factory("nocSelect2Options", function (nocUtil) {
        function subnet(weType, minimumRequiredIps) {
            function subnetMatcher(searchString, itemText, element) {
                var subnet = element.data('data');
                return nocUtil.genericItemFilter(searchString, subnet, ["cidr", "subnetName", "availabilityZone"]);
            }

            function subnetFormatter(subnetItem) {
                var subnet = $(subnetItem.element).data('data');

                if (subnet) {
                    var inOCCMSubnet = $(subnetItem.element).data('in-occm-subnet');
                    var thirdLine = (subnet.subnetName || "") + " (" + subnet.availabilityZone + ")";
                    var fourthLine = weType === 'HA' ? (subnet.availableIps > 0 ? subnet.availableIps + " available IP addresses" : "No available IP addresses") : undefined;

                    var subnetString = subnet.cidr + (inOCCMSubnet ? ' (OCCM subnet)' : '');
                    var title = subnet.availableIps < minimumRequiredIps ? "The instance requires " + minimumRequiredIps + " IP addresses, but the subnet only has " + subnet.availableIps + " available." : (subnet.subnetName || subnetString);
                    return nocUtil.formatSelect2(
                        title,
                        subnetString,
                        undefined,
                        thirdLine,
                        fourthLine
                    );
                } else {
                    return subnetItem.text;
                }
            }

            return {
                placeholder: "Select a subnet",
                matcher: subnetMatcher,
                formatResult: subnetFormatter,
                formatSelection: subnetFormatter
            };

        }

        /*istanbul ignore next: no interesting logic*/
        function providerVolumeType(defaultTitle) {
            function formatDiskType(diskTypeItem) {
                var diskType = $(diskTypeItem.element).data('data');
                if (!diskType) return diskTypeItem.text;
                return nocUtil.formatSelect2(
                    defaultTitle || diskType.description,
                    diskType.display,
                    undefined,
                    diskType.description
                );
            }

            return {
                formatResult: formatDiskType,
                formatSelection: formatDiskType
            };
        }

        return {
            subnet,
            providerVolumeType
        };
    });
}());
