(() => {
    'use strict';

    angular.module('noc').factory('nocSelectedWorkspaceOrTenantService', ($log, nocLocalStorage, nocCurrentUserService, nocTenantsService, nocWorkspacesService) => {
        const selectedWorkspaceOrTenant = {};
        let selectedTenant, selectedWorkspace, isAgentMode;

        selectedWorkspaceOrTenant.resolveTenantOrWorkspaceFromCurrentUserOrUrl = (queryParams, agentMode) => {
            isAgentMode = agentMode;

            //If current user is admin, and there is no tenantId defined in the url, take the last saved tenant if exists, as defined in NOC-5540 Browser - Save the last tenant used by user to open its gallery on next login
            const tenantOrWorkspaceId = (nocCurrentUserService.getCurrentUser() ? nocCurrentUserService.getCurrentUser().tenantId : null) || queryParams.slTenantId || nocLocalStorage.SELECTED_TENANT.restore();
            const list = isAgentMode ? nocWorkspacesService.getWorkspaces() : nocTenantsService.getTenants();
            let tenantOrWorkspace = _.find(list, {publicId: tenantOrWorkspaceId}) || _.find(list, {workspacePublicId: tenantOrWorkspaceId});


            if (!tenantOrWorkspace) {
                //If no tenant was found until now, use the first tenant by default
                tenantOrWorkspace = list[0];

                //If the tenant id was specified, but not found - there's a problem
                if (tenantOrWorkspaceId) $log.warn('Could not find tenant with id \'' + tenantOrWorkspaceId + '\'. Showing data for \'' + tenantOrWorkspace.name + '\'');
            }
            selectedWorkspaceOrTenant.setSelected(tenantOrWorkspace);
            return tenantOrWorkspace;
        };

        selectedWorkspaceOrTenant.getIsAgentMode = () => {
            return isAgentMode;
        };

        selectedWorkspaceOrTenant.getPublicId = () => {
            return _.get(selectedTenant, 'publicId') || _.get(selectedWorkspace, 'workspacePublicId');
        };

        selectedWorkspaceOrTenant.getSelectedTenant = () => {
            return selectedTenant;
        };

        selectedWorkspaceOrTenant.getSelectedWorkspace = () => {
            return selectedWorkspace;
        };

        selectedWorkspaceOrTenant.setSelected = (tenantOrWorkspace) => {
            if (isAgentMode) {
                selectedWorkspace = tenantOrWorkspace;
                nocLocalStorage.SELECTED_TENANT.save(tenantOrWorkspace.workspacePublicId);
                selectedTenant = null;
            } else {
                selectedTenant = tenantOrWorkspace;
                selectedTenant._name = tenantOrWorkspace.name;
                nocLocalStorage.SELECTED_TENANT.save(tenantOrWorkspace.publicId);
                selectedWorkspace = null;
            }
        };

        return selectedWorkspaceOrTenant;
    });
})();
