(() => {
    'use strict';

    angular.module('noc').factory('nocTenantsService', ($rootScope, $http, $q) => {
        const tenantsService = {};

        let _tenants;

        tenantsService.setSelectedTenant = (tenant) => {
            $rootScope.selectedTenant = tenant;
        };

        tenantsService.getTenants = () => {
            return _tenants;
        };

        tenantsService.getTenantsFromServer = (nocHttp, fields) => {
            const url = '/tenants' + (fields.workingEnvironmentsInformation ? '?fields=workingEnvironmentsInformation' : '');

            return nocHttp.get(url, 'retrieving list of tenants', nocHttp.LOADING).then((tenants) => {
                _.forEach(tenants, (t) => {
                    t.key = t.publicId;
                });
                _tenants = tenants;

                return tenants;
            });
        };

        tenantsService.reloadTenants = () => {
            return $http.get('/tenants')
                .then((tenantsResponse) => {
                    _tenants = tenantsResponse.data;

                    _.forEach(_tenants, (tenant) => {
                        tenant._name = tenant.name;
                        tenant.key = tenant.publicId;
                    });
                })
                .catch((errorResponse) => {
                    if (!_tenants) {
                        $rootScope.failureMessage = 'Unable to retrieve list of tenants: ' + errorResponse.data.messageForDisplay();
                        $rootScope.appState = 'failed';
                    }
                });
        };

        /**
         * Retrieves nss data for a tenant
         * @param {object} nocHttp
         * @param {number} tenantId
         * @param {boolean} shouldValidateNss
         * @param {boolean} noLoading
         * @return {object} nss data: {nssUserName: string, isNssValid: boolean, nssInvalidReason: string}
         */
        tenantsService.getTenantNssData = (nocHttp, tenantId, shouldValidateNss, noLoading) => {
            // Initialize nss data result
            const nssData = {nssUserName: null, isNssValid: false, nssInvalidReason: null};
            // Build promises
            const promises = [];

            // Get tenant promise
            const tenantRetrievalRequestUrl = '/tenants/' + tenantId;
            const getTenantPromise = noLoading ? nocHttp.get(tenantRetrievalRequestUrl, 'retrieving NSS credentials') : nocHttp.get(tenantRetrievalRequestUrl, 'retrieving NSS credentials', nocHttp.BLOCK);
            promises.push(getTenantPromise);

            // Validate tenant promise
            if (shouldValidateNss) {
                const tenantValidationRequestUrl = tenantRetrievalRequestUrl + '/validate';
                const validateTenantPromise = nocHttp.post(tenantValidationRequestUrl, 'validating NSS credentials', nocHttp.LOADING);
                promises.push(validateTenantPromise);
            }

            // Wait for all promises
            return $q.all(promises)
            // Extract data from responses
                .then(_.spread((getTenantData, validateTenantData) => {
                    nssData.nssUserName = getTenantData.nssUserName;
                    if (shouldValidateNss) {
                        nssData.isNssValid = validateTenantData.isValid;
                        nssData.nssInvalidReason = validateTenantData.reason;
                    }

                    return nssData;
                }));
        };

        return tenantsService;
    });
})();
