(() => {
    'use strict';

    angular.module("noc").factory("columnPickerService", function($timeout, nocDialog, uiGridConstants){

        var columnPicker = {};

        // used inside the dialog, out for testability
        columnPicker.setShowFunction = function(oldState) {
            return function(which) {
                var f = function(col) {
                    if (which === "all") {
                        col.visible = true;
                    } else if (which === "none") {
                        col.visible = col.disabled || false;
                    } else {
                        col.visible = col.default || false;
                    }
                };
                _.forEach(oldState, f);
            };
        };

        // Used inside the dialog, out for testability
        columnPicker.columnPickerOnConfirm = function(newState) {
            _.forEach(columnPicker.gridOptions.columnDefs, function(oldCol) {
                var newCol = _.find(newState.columns, {displayName: oldCol.displayName});
                oldCol.visible = newCol && newCol.visible;
            });
            $timeout(function() {
                columnPicker.gridAPI.core.notifyDataChange(uiGridConstants.dataChange.COLUMN);
                $timeout(function() {
                    columnPicker.columnsService.saveGridState(columnPicker.gridAPI);
                });
            });
        };

        columnPicker.startColumnPicker = function(gridOptions, gridAPI, columnsService, dialogTitle) {
            columnPicker.gridOptions = gridOptions;
            columnPicker.columnsService = columnsService;
            columnPicker.gridAPI = gridAPI;

            var oldState = _.cloneDeep(columnPicker.gridOptions.columnDefs);
            nocDialog
                .confirm(dialogTitle, "assets/templates/dialogs/ui-grid-cols-picker-dialog.html", {
                    columns: oldState,
                    width: 500,
                    fromTop: "20%",
                    setShow: columnPicker.setShowFunction(oldState)
                })
                .then(columnPicker.columnPickerOnConfirm);
        };

        return columnPicker;
    });

    angular.module('noc').factory('uiGridFilterService', () => {
        const uiGridFilterService = {};

        uiGridFilterService.includes = (scope, row, term) => {
            const regexp = new RegExp(term, 'i');

            function match(key) {
                const value = scope.$eval(key, row);

                return value && regexp.test(value);
            }

            return !term || _.some(scope.gridOptions.columnDefs, (column) => {
                    return column.visible && _.some(column._filterBy || [column.field], match);
                });
        };

        /**
         * Installs a filter that checks the values of all the columns against the filter term ($scope.search.term)
         * Also refreshed the grid when doFiltering() is called.
         *
         * This function assumes that search.term exists in the $scope
         * It also places a  function called doFiltering() on the $scope which should be called in the ng-change of the search input field
         */
        uiGridFilterService.installFilter = (gridApi, scope) => {
            //On every rows update - go over all the rows and calc visibility
            //called by grid.refresh() or grid.queueGridRefresh()
            gridApi.grid.registerRowsProcessor(
                (rows) => {
                    let visibleRowsCount = 0;

                    _.forEach(rows, (row) => {
                        //eslint-disable-next-line lodash/prefer-lodash-method
                        row.visible = uiGridFilterService.includes(scope, row.entity, scope.search.term);
                        if (row.visible) visibleRowsCount++;
                    });

                    scope.visibleRowsCount = visibleRowsCount;

                    return rows;
                },
                200);

            //Committing the filter by refreshing the grid. This method is debounced for better behaviour and performance
            scope.doFiltering = _.debounce(() => {
                gridApi.grid.refresh();
            }, 200);
        };

        return uiGridFilterService;
    });

    angular.module('noc').factory('uiGridSortUtilities', (nocUtil) => {
        const sortUtilities = {};

        sortUtilities.sortCapacityObjects = (aCapacity, bCapacity) => {
            aCapacity = nocUtil.convertTo(aCapacity, 'Byte');
            bCapacity = nocUtil.convertTo(bCapacity, 'Byte');

            return (aCapacity.size > bCapacity.size) ? 1 : -1;
        };

        return sortUtilities;
    });

    angular.module('noc').factory('uiGridUtilities', () => {
        const utilities = {};

        utilities.titledCellTemplate = '<div class="ui-grid-cell-contents" title="{{COL_FIELD}}">{{COL_FIELD}}</div>';

        return utilities;
    });
})();
