(() => {
    'use strict';

    angular.module('noc').factory('nocUIMain', ($window, $document, $timeout, $injector, $log, intercomNotifier, $rootScope, nocAuth0Service) => {
        const nocUIMain = {};

        const blockDiv = $document.find('#block');
        const loadingDiv = $document.find('#loading');

        nocUIMain.reloadSilently = (backToRoot, startTime) => {
            if (startTime === undefined) {
                startTime = new Date().getTime();
            }
            //Can't inject nocPoller directly - Circular dependency found: $http <- nocPoller <- nocUIMain <- v8Interceptor <- $http
            $injector.get('nocPoller').pause();
            //Can't inject statusRequestPoller directly - Circular dependency found: $http <- statusRequestPoller <- nocUIMain <- v8Interceptor <- $http
            $injector.get('statusRequestPoller').pause();
            //Can't inject $http directly - Circular dependency found: nocUIMain <- v8Interceptor <- $http <- nocUIMain
            const pendingRequests = $injector.get('$http').pendingRequests;
            const timeDifference = (new Date().getTime() - startTime);
            if (pendingRequests > 0 && (timeDifference < 2000)) {
                //To analyze NOC-9746 Azure - Login to OCCM takes time or failed with error Unauthorized
                $log.log('nocUIMain reloadSilently, there are ' + pendingRequests + ' pending requests and the time difference since last reload is ' + timeDifference + ' calling reloadSilently again!!!');
                $timeout(() => {
                    nocUIMain.reloadSilently(backToRoot, startTime);
                }, 50);
            } else {
                $('body').css('display', 'none');
                $($window).off('beforeunload');

                if (backToRoot) {
                    $window.location.href = _($window.location.href)
                        .replace(/#.*/, '')
                        .replace(/\?.*/, '');
                } else {
                    $window.location.reload();
                }
            }
        };

        nocUIMain.logout = function(delayMillis) {
            $timeout(function () {
                if ($rootScope.supportServices.portalService && $rootScope.supportServices.portalService.usePortalAuthentication) {
                    nocAuth0Service.constructAuth();
                    window.auth.logout();
                } else {
                    //can't inject $http directly - Circular dependency found: nocUIMain <- v8Interceptor <- $http <- nocUIMain
                    $injector.get("$http").post("/auth/logout")
                        .finally(function () {
                            //let intercom know that the user logged out
                            intercomNotifier.logout();

                            nocUIMain.reloadSilently(true);
                        });
                }
            }, delayMillis);
        };

        var canUnload = false;
        var block = function(transparent){
            if (transparent){
                blockDiv.css("background-color", "transparent");
            }
            blockDiv.show();
            $document.on('mousedown mouseup keydown keypress keyup touchstart touchend touchmove', false);
            loading();
        };

        var unblock = function(){
            blockDiv.hide();
            blockDiv.css("background-color", "");
            $document.off('mousedown mouseup keydown keypress keyup touchstart touchend touchmove');
            //In case of block and loading, canUnload will be true and allow a short loader
            canUnload = true;
            unloading();
        };

        var waitingToUnload = false;
        var loading = function(){
            loadingDiv.show();
            canUnload = false;
            waitingToUnload = false;
            $timeout(function(){
                canUnload = true;
                if(waitingToUnload){
                    unloading();
                }
            }, 500);
        };

        const unloading = () => {
            waitingToUnload = false;
            if (canUnload) {
                loadingDiv.hide();
            } else {
                waitingToUnload = true;
            }
        };

        var blockCount = 0;
        nocUIMain.block = function(className, transparent){
            if(blockCount === 0) block(transparent);
            blockCount++;
            $log.debug("NocUIMain block was called for ", className, " block count (after adding)", blockCount);

            return _.once(function(){
                blockCount--;
                $log.debug("NocUIMain unblock was called for ", className, " block count (after subtracting)", blockCount);
                if(blockCount === 0) unblock();
            });
        };

        var loadingCount = 0;

        nocUIMain.isLoading = function isLoading() {
            return !!loadingCount;
        };

        nocUIMain.loading = function(className){
            if(loadingCount === 0) loading();
            loadingCount++;
            $log.debug("NocUIMain loading was called for ", className, " loading count (after adding)", loadingCount);

            return _.once(function(){
                loadingCount--;
                $log.debug("NocUIMain unloading was called for ", className, " loading count (after subtracting)", loadingCount);
                if(loadingCount === 0) unloading();
            });
        };

        nocUIMain.handleFormValidity = function(){
            $("body").addClass("visited");
        };

        var scrollToFirstErrorStep = function() {
            var firstInvalidInput = $(':input.ng-invalid').first(),
                topFillHeight = firstInvalidInput.parents(".layout-content-holder").children("[noc-match-header-height]").height(),
                theView = firstInvalidInput.closest("[ui-view]");
            if (firstInvalidInput.length) {
                var scrollAmount = theView.offset().top - topFillHeight - parseInt(theView.css('margin-top')) - (_.startsWith($injector.get("$state").current.name, "volume-view") ? 100 : 0);
                $("html, body").animate({ scrollTop: scrollAmount}, 600, "linear");
            }
        };

        nocUIMain.handleErrorsInWizard = function(withScrolling) {
            nocUIMain.handleFormValidity();
            if (withScrolling) $timeout(scrollToFirstErrorStep);
        };

        nocUIMain.clearFormValidity = function(){
            $(".visited").removeClass("visited");
            $("body").removeClass("ignore-validity");
        };

        nocUIMain.ignoreFormValidity = function(){
            $("body").addClass("ignore-validity");
        };

        return nocUIMain;
    });
})();
