(() => {
    'use strict';

    //Return array with major, minor and patch numbers
    //handling 8.3.1 8.3 and 8
    function parse(ver) {
        let results = (/.*?(\d+)\.(\d+)\.(\d+).*/).exec(ver);
        if (results) return [results[1], results[2], results[3]];

        results = (/.*?(\d+)\.(\d+).*/).exec(ver);
        if (results) return [results[1], results[2], 0];

        results = (/.*?(\d+).*/).exec(ver);
        if (results) return [results[1], 0, 0];

        return [0, 0, 0];
    }

    /**
     * returns 1, 0 or -1 is the other version is smaller, same or bigger than this
     */
    function compare(ver1,ver2){
        var parsed1 = parse(ver1);
        var parsed2 = parse(ver2);

        var diff =
            parsed1[0] - parsed2[0] || //return non zero result ignoring minor and patch values
            parsed1[1] - parsed2[1] || //return non zero result ignoring patch values
            parsed1[2] - parsed2[2];

        //simple implementation for Math.sign() which could not be used since PhantomJS does not support it
        return diff === 0 ? 0 : (diff > 0 ? 1 : -1);
    }

    //See unit tests for examples
    angular.module("noc").factory("versionUtil", function () {
        return {
            compare: compare,
            cotVersionDisplay: cotVersionDisplay
        };
    });

    function cotVersionDisplay(ver){
        //remove ending .T<number> and= .ha or .azure if exists
        //both can appear e.g. .T1.ha
        var regexp = /^(.+?)(\.T\d+)?(\.ha)?(\.azure)?(\.azureha)?$/;

        return ver && regexp.exec(ver)[1];
    }

    //eslint-disable-next-line lodash/prefer-lodash-method
    angular.module('noc').filter('cotVersion', _.constant(cotVersionDisplay));
})();
