var _ = require('lodash');

/**
 * Bases on eslint-plugin-promise/catch-or-return with the following changes:
 * removed support for allowThen which we don't need.
 * skip for some promise generator e.g. when nocHttp is used.
 *
 */
module.exports = function (context) {
    //identifiers that produce promises but take care of catching the errors
    const exceptions = context.options;

    function isMethod(node, name) {
        return node.type === 'CallExpression' &&
            node.callee.type === 'MemberExpression' &&
            (!name || node.callee.property.name === name);
    }

    function findRoot(node){
        return isMethod(node) ? findRoot(node.callee.object) : node;
    }

    function isOkRoot(expressionStatement) {
        const root = findRoot(expressionStatement.expression);

        return root.type === 'Identifier' && (_.includes(exceptions, root.name) || (/.+resource/i).test(root.name));
    }

    function endsWithThenMethodCall(expressionStatement){
        return isMethod(expressionStatement.expression,'then');
    }

    function doesNotEndWithCatch(expressionStatement){
        return isMethod(expressionStatement.expression) &&
            isMethod(expressionStatement.expression.callee.object,'then') &&
            expressionStatement.expression.callee.property.name !== 'catch';
    }

    return {
        //if it's an ExpressionStatement, it's not a ReturnStatement
        ExpressionStatement: function (es) {
            if( endsWithThenMethodCall(es) || doesNotEndWithCatch(es)){
                if (!isOkRoot(es)) {
                    context.report(es, 'Expected catch() or return');
                }
            }
        }
    };
};
