var _ = require('lodash');

/**
 * messages passed to nocHttp must:
 * 1. nocHttp message must start with lowercase
 * 2. nocHttp message must start with present progressive verb (-ing)
 * 3. nocHttp.get() message should start with 'retrieving', not 'fetching' or 'getting'
 *
 * Limitations:
 * 1. Currently the rules can't handle complex messages, for example:
 * nocHttp,get("url", "Retrieving data for " + resource1)
 *
 */
module.exports = function (context) {
    function isNocHttpCall(node){
        return node.type === 'CallExpression' &&
            node.callee.type === 'MemberExpression' &&
            node.callee.object.name === "nocHttp";
    }

    function isDescriptionStartsWithLowerCase(node){
        var firstChar = node.arguments[1].value[0];
        return firstChar === _.toLower(firstChar);
    }

    function isDescriptionIsInPresentProgressive(node){
        var firstWord = _.split(node.arguments[1].value, ' ')[0];
        return _.endsWith(firstWord, 'ing');
    }

    function isNocHttpGet(node){
        return node.callee.property.name === "get";
    }

    function isDescriptionStartsWithForbiddenWord(node){
        var firstWord = _.split(node.arguments[1].value, ' ')[0];
        return firstWord === "fetching" || firstWord === "getting";
    }

    return {
        CallExpression: function (es) {
            if (isNocHttpCall(es)) {
                if (!es.arguments[1] || !es.arguments[1].value) return;
                if (!isDescriptionStartsWithLowerCase(es)) {
                    context.report(es, 'nocHttp message must start with lowercase');
                }
                if (!isDescriptionIsInPresentProgressive(es)) {
                    context.report(es, 'nocHttp message must start with present progressive verb (-ing)');
                }
                if (isNocHttpGet(es) && isDescriptionStartsWithForbiddenWord(es)) {
                    context.report(es, "nocHttp.get() message should start with 'retrieving', not 'fetching' or 'getting'");
                }
            }
        }
    };
};
