const notifier = require('gulp-notify');
const _ = require('lodash');
const program = require('../program');

function notify(message, sound) {
    const data = {
        title: 'HTMLLint Violations',
        message,
        sound // See: https://github.com/mikaelbr/node-notifier#all-notification-options-with-their-defaults
    };
    notifier(data).write('');
}

module.exports = {
    report: (messageTemplate, errors, file) => {
        if (!_.isEmpty(errors)) {
            const summary = 'HTMLLint Violations: ' + file.relative;
            const messages = _.map(errors, _.template(messageTemplate));

            if (program.teamcity) {
                const output = [];
                output.push("##teamcity[testSuiteStarted name='HTMLLint Violations']");
                _.forEach(messages, (message) => {
                    output.push('##teamcity[testStarted name="' + summary + '"]');
                    output.push('##teamcity[testFailed name="' + summary + '" message="' + message + '"]');
                    output.push('##teamcity[testFinished name="' + summary + '"]');
                });
                output.push("##teamcity[testSuiteFinished name='HTMLLint Violations']");

                console.log(output.join('\n')); // eslint-disable-line no-console
            } else {
                notify(summary, 'Sosumi');
                //Cant use:
                //    _.forEach(messages, notify)
                // because index is passed as second parameter (sound), causing errors in case more than 1 error per file.
                _.forEach(messages, (message/*, index, collection*/) => {
                    notify(message);
                });
            }
        }
    }
};
