const lazypipe = require('lazypipe');
const through = require('through-gulp');
const _ = require('lodash');

const iconStore = require('../icon_store');
const reporter = require('./reporter');

module.exports = lazypipe()
    .pipe(through, null, (cb) => {
        const errors = _.compact(_.map([
            //The following icons are calculated in simple_volumes_columns_service.js and not referred directly in an html file
            'clone',
            'volume-stroke',
            'resource-volume',
            'volume-gp2',
            'volume-gp2+s3',
            'volume-prm',
            'volume-sc1',
            'volume-st1',
            'volume-st1+s3',
            'volume-io1',
            'volume-io1+s3',
            'volume-std',
            'volume-standard',
            'exclamation-round-yellow',
            'volume-move',
            'arrow-up-blue',
            'arrow-down-blue',
            'arrow-down-grey',
            'magnifying-glass-blue',
            'close-filter-blue',
            'close-filter',

            //The following icons are calculated in data_fabric_layout.js and not referred directly in an html file
            'sync-blue',
            'sync-red',

            //The following icons are calculated in metadata_resource.js and not referred directly in an html file
            'light-bulb',
            'low-cost',
            'rocket',
            'volume-star',
            'explore-license',
            'standard-license',
            'premium-license',

            //The following icons are calculated in cost-breakdown-category-component.js and not referred directly in an html file
            'cost-compute',
            'cost-data-transfer',
            'cost-ebs-storage',
            'cost-s3'
        ], (name) => {
            if (!iconStore.exists(name)) {
                return {name};
            }
        }));

        reporter.report(
            'noc-icon name attribute value "${name}" used in volumeViewColumnsService does not have a matching .svg file',
            errors,
            {relative: 'services/grid/volume_view_columns_service.js'});

        _.forEach(iconStore.listNotVisited(), (error) => {
            //At the end of the steam, assert that all the svg files are referred in the html files
            reporter.report('SVG icon ${path} is not used', [error], {relative: error.path});
        });

        cb();
    });
