var glob = require('glob');
var gulp = require('gulp');
var gulpif = require('gulp-if');
var size = require('gulp-size');
var debug = require('gulp-debug');
var notify = require('gulp-notify');
var plumber = require('gulp-plumber');
var _ = require('lodash');
var program = require('./program');

var errorHandler = notify.onError({
    //Teamcity fails the build when a message matching /(Gulp build error)(.*)/ appears
    title:'Error in <%=error.plugin%>',
    message:'<%=error.message%>'
});

_.forEach(glob.sync('gulp/tasks/*.js'), function (taskFile) {
    var task = require('../'+taskFile);

    if(_.isFunction(task.registerTask)){
        program.debug('loading task - ' + taskFile);
        task.registerTask();
    }else{
        program.debug(task.toString());

        var taskFunction = function() {
            return gulp.src(task.flatGlob)
                .pipe(gulpif(program.verbose,debug()))
                .pipe(plumber(errorHandler))
                .pipe(task.pipe())
                .pipe(gulpif(program.verbose,size({title: task.name})));
        };

        if(task.deps){
            gulp.task(task.name, task.deps, taskFunction);
        }else{
            gulp.task(task.name, taskFunction);
        }
    }
});
