const program = require('commander');
const log = require('fancy-log');
const _ = require('lodash');

program
    .option('-v, --verbose', 'show more data in build log: total size of files in tasks')
    .option('-t, --teamcity', 'Uses TeamCity reporter for tests and linting')
    .option('-s, --server <server address[:port]>', '(Dev) - delegate requests to server. Default value is localhost. Also completes demoXX and jh-tlv-XX to full address')
    .option('-l, --log-connect', '(Dev) - log the connect task requests and responses')
    .option('-r, --reporters <reporters list>', 'Comma separated list of Karma reporters. Can be empty.')
    .option('-b, --browsers <browsers list>', 'Comma separated list of browsers to use for the unit tests. Default is Chrome.')
    .parse(process.argv);

program.isProd = true;
program.verbose = program.verbose || false; //Make sure the value is boolean

program.debug = (msg) => {
    if (program.verbose) {
        log(msg);
    }
};
program.error = (msg) => {
    log(msg);
};
program.info = (msg) => {
    log(msg);
};

Object.defineProperty(program, 'serverHost', {
    get: function get() {
        if (!this.server) {
            return 'localhost';

            //Extract ip from http://[ip address]/ of https://[ip address]/
        } else if (_.startsWith(this.server, 'http')) {
            return (/\d+\.\d+\.\d+\.\d+/).exec(this.server)[0];

            //Whatever
        } else {
            return this.server;
        }
    }
});

module.exports = program;
