const lazypipe = require('lazypipe');
const useref = require('gulp-useref');
const gulpif = require('gulp-if');
const uglify = require('gulp-uglify');
const ngAnnotate = require('gulp-ng-annotate');
const injectString = require('gulp-inject-string');
const sourceMaps = require('gulp-sourcemaps');
const gulp = require('gulp');

module.exports = lazypipe()
    //Since we're uglifying, we need to make sure all injections are declared in the strict mode. As recommended in
    //https://github.com/olov/ng-annotate#highly-recommended-enable-ng-strict-di-in-your-minified-builds
    .pipe(injectString.after, 'ng-app="noc"', ' ng-strict-di')

    //Parses the index.html file for 'build:' and 'endbuild' annotations and concatenates all files between them
    //those files are sent downstream to the ngAnnotate and uglify plugins
    //changes files to be the referenced assets & concatenates
    .pipe(useref, {searchPath: './dist'})

    //Copy the original concatenated source to the artifacts dir so they will be available when source maps are used
    .pipe(() => {
        return gulpif('**/*.js', gulp.dest('artifacts/'));
    })

    //Start listening to the changes in files
    .pipe(() => {
        return gulpif('**/*.js', sourceMaps.init());
    })

    //Add inline array annotation (https://docs.angularjs.org/guide/di) to the injected parameters
    //few parameters that are not automatically found are marked with @ngInject comment
    .pipe(() => {
        return gulpif('**/*.js', ngAnnotate());
    })

    //Uglify the concatenated files
    .pipe(() => {
        return gulpif('**/*.js', uglify());
    })

    //Write the mapping between the original and uglified values in files
    .pipe(() => {
        return gulpif('**/*.js', sourceMaps.write('.', {includeContent: false, sourceRoot: '../../'}));
    });
