describe("Login Controller", function test(){
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(inject(function(){
        magnifispec.mockController("LoginController");
    }));

    describe("logging in", function(){
        beforeEach(function(){
            $("body").append("<input id='loginEmail'>");
            $("body").append("<input id='loginPassword'>");
            $("#loginEmail").val("name@gmail.com");
            $("#loginPassword").val("pwd");
        });

        it("should validate the email in the client because server can't handle hebrew", function() {
            $("#loginEmail").val("תמי@netapp.com");
            j.scope.login();
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/auth/login", {email: "", password: "pwd"}).respond(204);
            j.$httpBackend.flush();
        });


        it("should make a post request passing in the username and password", function(){
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/auth/login", {email: "name@gmail.com", password: "pwd"}).respond(204);
            j.scope.login();
            j.$httpBackend.flush();
        });

        it("should reload the page in case of post success", function(){
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/auth/login", {email: "name@gmail.com", password: "pwd"}).respond(204);
            j.scope.login();
            j.$httpBackend.flush();
            expect(j.nocUI.reloadSilently).toHaveBeenCalledWith();
        });

        it("should set failed to be true in case of login failure", function(){
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/auth/login", {email: "name@gmail.com", password: "pwd"}).respond(401, {});
            expect(j.scope.failed).toBeUndefined();
            j.scope.login();
            j.$httpBackend.flush();
            expect(j.scope.failed).toEqual(true);
            expect(j.scope.animate).toBeFalsy();
        });

        it("should animate error message in case or not-first failure", function () {
            j.scope.failed = true;
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/auth/login", {email: "name@gmail.com", password: "pwd"}).respond(401, {});
            j.scope.login();
            j.$httpBackend.flush();
            expect(j.scope.animate).toBeTruthy();
            j.$timeout.flush(1000);
            expect(j.scope.animate).toBeFalsy();
        });

        it("should set failed to be true in case of non-login post failure", function(){
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/auth/login", {email: "name@gmail.com", password: "pwd"}).respond(500, {});
            expect(j.scope.failed).toBeUndefined();
            expect(j.scope.appState).toBeUndefined();
            j.scope.login();
            j.$httpBackend.flush();
            expect(j.scope.failed).toBeUndefined();
            expect(j.scope.appState).toEqual("failed");
        });
    });

    describe("logging out", function(){
        it("should instantly logout", function(){
            j.scope.logout();
            expect(j.nocUI.logout).toHaveBeenCalled();
        });
    });
});
