describe.skip('activateSnapshotComponent', () => {
    let controller;
    const volumes = [
        {
            name: 'vol1',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'default'
        },
        {
            name: 'vol2',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'none'
        },
        {
            name: 'vol3',
            volumeType: 'rw',
            rootVolume: true,
            moving: false,
            state: 'online',
            snapshotPolicy: 'default'
        },
        {
            name: 'vol4',
            volumeType: 'rw',
            rootVolume: false,
            moving: true,
            state: 'online',
            snapshotPolicy: 'default'
        },
        {
            name: 'vol5',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'none'
        },
        {
            name: 'vol6',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'none'
        },
        {
            name: 'vol7',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: '111'
        }];
    const volumesAllProtected = [
        {
            name: 'vol1',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'default'
        },
        {
            name: 'vol2',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'default'
        },
        {
            name: 'vol3',
            volumeType: 'rw',
            rootVolume: true,
            moving: false,
            state: 'online',
            snapshotPolicy: 'none'
        },
        {
            name: 'vol4',
            volumeType: 'rw',
            rootVolume: false,
            moving: true,
            state: 'online',
            snapshotPolicy: 'none'
        },
        {
            name: 'vol5',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'default'
        },
        {
            name: 'vol6',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: 'default'
        },
        {
            name: 'vol7',
            volumeType: 'rw',
            rootVolume: false,
            moving: false,
            state: 'online',
            snapshotPolicy: '111'
        }];
    let changesObj;
    let scope;

    beforeEach(inject(($componentController, $rootScope) => {
        scope = $rootScope.$new();
        controller = $componentController('activateSnapshotComponent', {$scope: scope}, {
            isActiveAction: false,
            volumes
        });
    }));

    describe('init', () => {
        it('should initialize values', () => {
            controller.$onInit();

            expect(controller.isButtonDisabled).toEqual(false);
            expect(controller.countVolumesNoSnapshotBeforeActivate).toEqual(undefined);
            expect(controller.isActivatingFinished).toEqual(false);
            expect(controller.errorMessage).toEqual('');
        });

        it('should set button to disable if is activating', () => {
            controller.isActiveAction = true;
            controller.$onInit();

            expect(controller.isButtonDisabled).toEqual(true);
        });

    });
    describe('volumes changed', () => {
        beforeEach(() => {
            jest.spyOn(controller, 'updateData').mockImplementation(() => {});
            changesObj = {
                isActiveAction: {},
                volumes: {
                    currentValue: volumesAllProtected,
                    previousValue: volumes,
                    isFirstChange: _.constant(false)
                }
            };
            controller.isActiveAction = true;
            controller.$onInit();
        });

        it('should call update data upon changes', () => {
            controller.isActiveAction = false;
            controller.$onChanges(changesObj);

            expect(controller.updateData).toHaveBeenCalled();
        });

        it('should not call update data upon changes if still activating', () => {
            controller.$onChanges(changesObj);

            expect(controller.updateData).not.toHaveBeenCalled();
        });

        it('should set button to enabled if activating is done', () => {
            controller.isActiveAction = false;
            controller.$onChanges(changesObj);

            expect(controller.isButtonDisabled).toEqual(false);
        });

        it('should not set button to enabled if activating is not done', () => {
            controller.$onChanges(changesObj);

            expect(controller.isButtonDisabled).toEqual(true);
        });
    });

    describe('isActiveAction updated', () => {
        beforeEach(() => {
            controller.$onInit();
        });

        it('should set isActivatingFinished is done activating', () => {
            changesObj = {
                isActiveAction: {
                    currentValue: false,
                    previousValue: true,
                    isFirstChange: _.constant(false)
                },
                volumes: {}
            };
            controller.isActiveAction = false;
            controller.$onChanges(changesObj);

            expect(controller.isActivatingFinished).toEqual(true);
        });

        it('should do nothing if activating is starting', () => {
            changesObj = {
                isActiveAction: {
                    currentValue: true,
                    previousValue: false,
                    isFirstChange: _.constant(false)
                },
                volumes: {}
            };
            controller.$onChanges(changesObj);

            expect(controller.isActivatingFinished).toEqual(false);
        });
    });

    describe('activateClicked', () => {
        beforeEach(() => {
            controller.$onInit();
            controller.errorMessage = 'bla';
            controller.volumes = volumes;
            controller.isActivatingFinished = true;
            jest.spyOn(scope, '$emit').mockImplementation(() => {});
        });

        it('should perform actions upon click', inject(($timeout) => {
            controller.isButtonDisabled = false;

            controller.activateClicked();
            $timeout.flush();

            expect(controller.errorMessage).toEqual('');
            expect(controller.countVolumesNoSnapshotBeforeActivate).toEqual(3);
            expect(controller.isActivatingFinished).toEqual(false);
            expect(controller.isButtonDisabled).toEqual(true);
            expect(scope.$emit).toHaveBeenCalledWith('activateSnapshot');
        }));

        it('should do nothing if already clicked', inject(($timeout) => {
            controller.isButtonDisabled = true;

            controller.activateClicked();
            $timeout.flush();

            expect(controller.errorMessage).toEqual('bla');
            expect(controller.isActivatingFinished).toEqual(true);
            expect(controller.isButtonDisabled).toEqual(true);
            expect(scope.$emit).not.toHaveBeenCalled();
        }));
    });

    describe('updateData', () => {
        beforeEach(() => {
            controller.$onInit();
        });

        it('should update data in case filteredVolumes > 0', () => {
            controller.updateData(volumes);

            expect(controller.eligibleVolumes.length).toEqual(5);
            expect(controller.volumesNoSnapshotCount).toEqual(3);
            expect(controller.chartMainTitle).toEqual('40 %');
            expect(controller.chartData).toEqual([{name: '2 protected volumes', y: 40}, {name: '3 unprotected volumes', y: 60}]);
            expect(controller.errorMessage).toEqual('');
        });

        it('should do nothing in case filteredVolumes is 0', inject((ransomwareProtectionResource) => {
            jest.spyOn(ransomwareProtectionResource, 'countVolumesNoSnapshot').mockImplementation(() => {});
            controller.updateData([]);

            expect(ransomwareProtectionResource.countVolumesNoSnapshot).not.toHaveBeenCalled();
        }));

        it('should show error message if no eligible volume was updated', () => {
            controller.isActivatingFinished = true;
            controller.countVolumesNoSnapshotBeforeActivate = 3;
            controller.updateData(volumes);

            expect(controller.errorMessage).toEqual('Unable to activate Snapshot Policy on these volumes. Please try again later.');
        });

        it('should show error message if not all eligible volume was updated', () => {
            controller.isActivatingFinished = true;
            controller.countVolumesNoSnapshotBeforeActivate = 7;
            controller.updateData(volumes);

            expect(controller.errorMessage).toEqual('Unable to activate Snapshot Policy on 3 volumes. Please try again later.');
        });
    });
});
