describe.skip('parseMessageWithUrl', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let scope;
    let element;
    const msgWithUrl = 'This is a message Message line to remove https://blabla.blabla The end';
    const msgWithoutUrl = 'This is a message Message line to remove The end';

    beforeEach(() => {
        scope = j.$rootScope.$new();
    });

    describe('Both textForLink and textToRemove are supplied to the directive', () => {
        beforeEach(() => {
            element = angular.element(`<div ng-model="message" 
                                            parse-message-with-url 
                                            text-for-link="This is link text"
                                            text-to-remove="Message line to remove ">`);
        });

        it('should replace the url with "a" tag and use textForLink, and remove textToRemove, if url exists in the message', () => {
            scope.message = msgWithUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message <a href="https://blabla.blabla" target="_blank" title="https://blabla.blabla">This is link text</a> The end');
        });

        it('should only remove textToRemove, if no url exists in the message', () => {
            scope.message = msgWithoutUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message The end');
        });

        it('should replace the url with "a" tag and use textForLink, if url exists in the message and textToRemove is supplied but does not exist', () => {
            element = angular.element(`<div ng-model="message" 
                                            parse-message-with-url 
                                            text-for-link="This is link text"
                                            text-to-remove="Wrong text to remove ">`);
            scope.message = msgWithUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message Message line to remove <a href="https://blabla.blabla" target="_blank" title="https://blabla.blabla">This is link text</a> The end');
        });

        it('should replace the url with "a" tag and use textForLink, and remove textToRemove, if url exists in the message and url is at the beginning of the message string', () => {
            scope.message = 'https://blabla.blabla This is a message Message line to remove The end';
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('<a href="https://blabla.blabla" target="_blank" title="https://blabla.blabla">This is link text</a> This is a message The end');
        });

        it('should replace the url with "a" tag and use textForLink, and remove textToRemove, if url exists in the message and url is at the end of the message string', () => {
            scope.message = 'This is a message Message line to remove The end https://blabla.blabla';
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message The end <a href="https://blabla.blabla" target="_blank" title="https://blabla.blabla">This is link text</a>');
        });

        it('should replace the url with "a" tag and use textForLink, and remove textToRemove, if url exists in the message and url ends with /', () => {
            scope.message = 'This is a message Message line to remove http://blabla.blabla/ The end';
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message <a href="http://blabla.blabla/" target="_blank" title="http://blabla.blabla/">This is link text</a> The end');
        });
    });



    describe('Only textForLink is supplied to the directive', () => {
        beforeEach(() => {
            element = angular.element(`<div ng-model="message" 
                                            parse-message-with-url 
                                            text-for-link="This is link text">`);
        });

        it('should replace the url with "a" tag and use textForLink, if url exists in the message', () => {
            scope.message = msgWithUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message Message line to remove <a href="https://blabla.blabla" target="_blank" title="https://blabla.blabla">This is link text</a> The end');
        });

        it('should do nothing, if no url exists in the message', () => {
            scope.message = msgWithoutUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe(msgWithoutUrl);
        });
    });

    describe('Only textToRemove is supplied to the directive', () => {
        beforeEach(() => {
            element = angular.element(`<div ng-model="message" 
                                        parse-message-with-url 
                                        text-to-remove="Message line to remove ">`);
        });

        it('should replace the url with "a" tag and use the url text, if url exists in the message', () => {
            scope.message = msgWithUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message <a href="https://blabla.blabla" target="_blank" title="https://blabla.blabla">https://blabla.blabla</a> The end');
        });

        it('should only remove textToRemove, if no url exists in the message', () => {
            scope.message = msgWithoutUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message The end');
        });
    });

    describe('Both textForLink and textToRemove are not supplied', () => {
        beforeEach(() => {
            element = angular.element('<div ng-model="message" parse-message-with-url>');
        });

        it('should replace the url with "a" tag and use the url text, if url exists in the message', () => {
            scope.message = msgWithUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe('This is a message Message line to remove <a href="https://blabla.blabla" target="_blank" title="https://blabla.blabla">https://blabla.blabla</a> The end');
        });

        it('should do nothing, if no url exists in the message', () => {
            scope.message = msgWithoutUrl;
            element = j.$compile(element)(scope);
            scope.$digest();

            expect(element[0].innerHTML).toBe(msgWithoutUrl);
        });
    });
});
