describe.skip('noc policy', () => {
    'use strict';
    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    let scope, tElement, element;
    const policies = [
        {isDefault: false, isSupported: true, reasons: [], _policyInfo: {displayName: 'Mirror latest snapshot copy (legacy)', description: 'Typically used for a one-time data replication for ONTAP 8.2 and earlier'}, name: 'DPDefault', type: 'async_mirror', description: 'Default policy for DP relationship', rules: [{snapMirrorLabel: 'sm_created', retention: 1}]},
        {isDefault: true, isSupported: true, reasons: [], _policyInfo: {displayName: 'Mirror', description: 'Typically used for disaster tecovery'}, name: 'MirrorAllSnapshots', type: 'async_mirror', description: 'Asynchronous SnapMirror policy for mirroring all snapshots and latest active file system', rules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}]},
        {isDefault: false, isSupported: true, reasons: [], _policyInfo: {displayName: 'Mirror and Backup', description: 'Typically used when both disaster recovery and long-term retention are needed'}, name: 'MirrorAndVault', type: 'mirror_vault', description: 'A unified Asynchronous SnapMirror and SnapVault policy for mirroring the latest active file system and daily and weekly snapshots', rules: [{snapMirrorLabel: "sm_created", retention: 1}, {snapMirrorLabel: "daily", retention: 7}]},
        {isDefault: false, isSupported: false, reason: 'unsupported version', _policyInfo: {displayName: 'Backup', description: 'Typically used for long-term retention of backup copies'}, name: 'XDPDefault', type: 'vault', description: 'Default policy for XDP relationship with daily and weekly rules', rules: [{snapMirrorLabel: 'daily', retention: 7}]}
    ];

    beforeEach(() => {
        scope = j.$rootScope.$new();
        tElement = $('<noc-policy shown-policy="policies[0]" selected-policy="" select-policy="angular.noop"></noc-policy>');
    });

    describe('policyTooltip', () => {
        let elementScope;
        beforeEach(() => {
            element = j.$compile(tElement)(scope);
            scope.$digest();
            elementScope = element.isolateScope();
        });
        it('should return empty string in case policy is not defined or null', () => {
            elementScope.ctrlPolicy.shownPolicy = null;
            expect(elementScope.ctrlPolicy.policyTooltip()).toEqual('');
        });

        it('should return the policy display name in case the policy is supported', () => {
            elementScope.ctrlPolicy.shownPolicy = policies[0];
            expect(elementScope.ctrlPolicy.policyTooltip()).toEqual('Mirror latest snapshot copy (legacy)');
        });

        it('should return a list of the reasons in case the policy is not supported', () => {
            elementScope.ctrlPolicy.shownPolicy = policies[3];
            expect(elementScope.ctrlPolicy.policyTooltip()).toEqual('unsupported version');
        });
    });
});
