describe.skip('noc search widget', () => {
    'use strict';

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let scope, element, directivesScope;
    const body = $('body');

    function init(closeable) {
        body.empty();
        scope = j.$rootScope.$new();
        scope.text = 'someText';
        scope.placeholder = 'placeholderHa';
        element = j.$compile($('<noc-search-widget ng-model="model" placeholder="{{placeholder}}" ' + (closeable ? 'closeable' : '') + '/>'))(scope);
        body.append(element);
        scope.$digest();
        directivesScope = element.isolateScope();
    }

    describe('not closeable state', () => {
        beforeEach(_.partial(init, false));

        it('will start out open', () => {
            expect(directivesScope.enableSearchFields).toEqual(true);
        });

        describe('should clear the text, but not close when escape is pressed', () => {
            let input;
            beforeEach(() => {
                j.$httpBackend.expectGET("assets/templates/components/search-widget.html").respond(require('../../../../app/templates/components/search-widget.html'));
                element.find('i.widget-icon').click();
                input = element.find('input');

                expect(directivesScope.enableSearchFields).toEqual(true);
            });

            it('and the field is still empty', () => {
                input.trigger(new $.Event('keydown', {keyCode: 27}));
                expect(directivesScope.enableSearchFields).toEqual(true);
                expect(directivesScope.searchString).toEqual('');
            });

            it('and the field is not empty', () => {
                input.val('vol1').trigger('input');
                expect(directivesScope.searchString).toEqual('vol1');

                input.trigger(new $.Event('keydown', {keyCode: 27}));
                expect(directivesScope.enableSearchFields).toEqual(true);
                expect(directivesScope.searchString).toEqual('');
            });
        });
    });

    describe('closeable state', () => {
        beforeEach(_.partial(init, true));

        beforeEach(() => {
            j.$httpBackend.expectGET("assets/templates/components/search-widget.html").respond(require('../../../../app/templates/components/search-widget.html'));
        });

        it('will start out closed', () => {
            expect(directivesScope.enableSearchFields).toEqual(false);
        });

        describe('losing and gaining focus', () => {
            let distraction, input;
            beforeEach(() => {
                element.find('i.widget-icon').click();
                scope.$digest();
                j.$timeout.flush();
                distraction = $('<button>Distraction</button>');
                body.append(distraction);
                input = element.find('input');
            });

            it('should open and gain focus upon click', () => {
                expect(directivesScope.enableSearchFields).toEqual(true);
                expect(document.activeElement).toEqual(input[0]);
            });

            it('should close upon lost of focus if field is empty', () => {
                distraction.focus();
                input.blur();
                scope.$digest();

                expect(directivesScope.enableSearchFields).toEqual(false);
            });

            it('should not close upon lost of focus if field is dirty', () => {
                scope.model = 'something';
                distraction.focus();
                input.blur();
                scope.$digest();
                expect(document.activeElement).toEqual(distraction[0]);
                expect(directivesScope.enableSearchFields).toEqual(true);
            });
        });

        describe('should close when escape is pressed', () => {
            let input;
            beforeEach(() => {
                element.find('i.widget-icon').click();
                input = element.find('input');

                expect(directivesScope.enableSearchFields).toEqual(true);
            });

            it('and the field is still empty', () => {
                input.trigger(new $.Event('keydown', {keyCode: 27}));
                expect(directivesScope.enableSearchFields).toEqual(false);
                expect(directivesScope.searchString).toEqual('');
            });

            it('and the field is not empty', () => {
                input.val('vol1').trigger('input');
                expect(directivesScope.searchString).toEqual('vol1');

                input.trigger(new $.Event('keydown', {keyCode: 27}));
                expect(directivesScope.enableSearchFields).toEqual(false);
                expect(directivesScope.searchString).toEqual('');
            });
        });
    });
});
