describe.skip('Virgin wizard tree builder', () => {
    'use strict';

    let treeBuilder;
    let stateProvider;

    beforeEach(inject((virginWizardTreeBuilder) => {
        treeBuilder = virginWizardTreeBuilder;
        stateProvider = jasmine.createSpyObj('stateProvider', ['state']);
    }));
    const testSteps = [
        {
            name: 'root',
            controller: 'rootController',
            templateUrl: 'rootTemplate'
        },
        {
            name: 'child1',
            controller: 'cont1',
            wizardTemplateUrl: 'temp1'
        },
        {
            name: 'child2',
            controller: 'cont2',
            wizardTemplateUrl: 'temp2'
        }
    ];

    const testTree = {
        root: {
            child1: true,
            child2: true
        }
    };

    it('should be building a tree', () => {
        treeBuilder(stateProvider, testTree, testSteps);
        expect(stateProvider.state.mock.calls.length).toEqual(3);
        expect(stateProvider.state.mock.calls[0]).toEqual(['root', {
            controller: 'rootController',
            templateUrl: 'rootTemplate',
            url: '/root',
            data: {name: 'root'}
        }]);
        expect(stateProvider.state.mock.calls[1]).toEqual(['root.child1', {
            controller: 'cont1',
            url: '/child1',
            data: {name: 'child1'},
            template: '<noc-super-duper></noc-super-duper><ng-include ng-if="appView === \'storage-system-view\' && currentStateName === \'child1\'" src="\'assets/templates/components/virgin-wizard/previous.html\'"></ng-include><form novalidate><input type="text" style="display:none"><input type="password" style="display:none"><ng-include src="\'temp1\'"></ng-include></form><div><div ui-view></div></div>'
        }]);
        expect(stateProvider.state.mock.calls[2]).toEqual(['root.child2', {
            controller: 'cont2',
            url: '/child2',
            data: {name: 'child2'},
            template: '<noc-super-duper></noc-super-duper><ng-include ng-if="appView === \'storage-system-view\' && currentStateName === \'child2\'" src="\'assets/templates/components/virgin-wizard/previous.html\'"></ng-include><form novalidate><input type="text" style="display:none"><input type="password" style="display:none"><ng-include src="\'temp2\'"></ng-include></form><div><div ui-view></div></div>'
        }]);
    });

    it('should throw an exception when duplicate step names exists', () => {
        testSteps.push({name: 'child1'});
        expect(() => {
            treeBuilder(stateProvider, testTree, testSteps);
        }).toThrowError();
    });
});
