describe("Error Response test", function () {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    var ErrorResponse;

    beforeEach(inject(function (NocErrorResponse) {
        ErrorResponse = NocErrorResponse;
    }));

    it("should set a server fault message in case payload is undefined", function () {
        expect(new ErrorResponse().message).toEqual("Server Fault: Unknown Error Response");
    });

    it("should set a server fault message in case message is undefined", function () {
        expect(new ErrorResponse({}).message).toEqual("Server Fault: Unknown Error Response");
    });

    it("should set a server fault message in case message is empty", function () {
        expect(new ErrorResponse({message: ""}).message).toEqual("Server Fault: Unknown Error Response");
    });

    it("should set a server fault message in case message is not a string", function () {
        expect(new ErrorResponse({message: 2}).message).toEqual("Server Fault: Unknown Error Response");
    });

    it("should set the message of the ErrorResponse to be the message of the payload", function () {
        expect(new ErrorResponse({message: "How are you?"}).message).toEqual("How are you?");
    });

    it("should return the message when calling messageForDisplay()", function () {
        expect(new ErrorResponse({message: "Just fine, thank you."}).messageForDisplay()).toEqual("Just fine, thank you.");
    });

    it("should return the date formatted correctly when calling messageForDisplay()", function () {
        var errorResponse = new ErrorResponse({message: "The day the music died: ", date: -344264700000});
        expect(new ErrorResponse(errorResponse).messageForDisplay()).toEqual('The day the music died: Feb 3, 1959 12:55:00 pm');
    });
});