describe('region format filter spec', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    it('should return html formatted region', () => {
        const result = j.$filter('regionFormat')('us-east-1', true);
        expect(result).toEqual(j.nocUtil.formatSelect2('US East', 'US East', 'N. Virginia', 'us-east-1'));
    });

    it('should return non-html formatted region', () => {
        const result = j.$filter('regionFormat')('us-east-1');
        expect(result).toEqual('US East (N. Virginia)');
    });

    it('should return non-html formatted region for unknown region', () => {
        const result = j.$filter('regionFormat')('us-east-22');
        expect(result).toEqual('us-east-22');
    });

    it('should use the dictionary if provided', () => {
        const result = j.$filter('regionFormat')('us-east-1', false, {'us-east-1': {name: 'someName'}});
        expect(result).toEqual('someName');
    });
});

describe('availability zones format', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let filter;

    beforeEach(() => {
        filter = j.$filter('availabilityZonesFormat');
    });

    it('should handle a single zone', () => {
        const zones = ['us-west-2b'];
        expect(filter(zones)).toEqual('us-west-2b');
    });

    it('should handle a more than one zone and remove duplicates', () => {
        const zones = ['us-west-2b', 'us-east-1a', 'us-west-2a', 'us-west-2b'];
        expect(filter(zones)).toEqual('us-east-1a, us-west-2a, 2b');
    });

    it('should return N/A when no zones are present', () => {
        const zones = [];
        expect(filter(zones)).toEqual('N/A');
    });
});
