describe('usersResource', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let usersResource;

    beforeEach(inject((_usersResource_) => {
        usersResource = _usersResource_;
    }));

    describe('getUser', () => {
        it('should retrieve the user data', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when({name: 'user1'}));
            expect(usersResource.getUser(j.nocHttp, 'id1')).toBeResolvedWith({name: 'user1'});
        });

        it.skip('should mark Cloud Sync license as expired if needed', () => {
            advanceTo(moment('2016-12-01').toDate());
            const licenseExpirationDate = moment('2016-11-01').toDate();
            console.log(new Date().toISOString())

            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when({
                name: 'user1',
                cloudSyncLicenseInformation: {licenseExpirationDate}
            }));

            expect(usersResource.getUser(j.nocHttp, 'id1')).toBeResolvedWith(jasmine.objectContaining({
                cloudSyncLicenseInformation: jasmine.objectContaining({_cloudSyncLicenseExpired: true})
            }));
        });

        it('should not mark Cloud Sync license as expired if not expired', () => {
            advanceTo(moment('2016-12-01').toDate());
            const licenseExpirationDate = moment('2016-12-02').toDate();

            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when({
                name: 'user1',
                cloudSyncLicenseInformation: {licenseExpirationDate}
            }));

            expect(usersResource.getUser(j.nocHttp, 'id1')).toBeResolvedWith({
                name: 'user1',
                cloudSyncLicenseInformation: {
                    licenseExpirationDate,
                    _cloudSyncLicenseExpired: undefined
                }
            });
        });

        it('should fail in case of error', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));
            expect(usersResource.getUser(j.nocHttp, 'id1')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith('/users/id1', 'retrieving user data', j.nocHttp.BLOCK);
        });
    });

    describe('getUsers', () => {
        it('should retrieve the users data', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(['user1', 'user2']));
            expect(usersResource.getUsers(j.nocHttp, 'id1')).toBeResolvedWith(['user1', 'user2']);
        });

        it('should fail in case of error', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));
            expect(usersResource.getUsers(j.nocHttp, 'id1')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith('/users', 'retrieving users information', j.nocHttp.LOADING);
        });
    });
});
