describe("Client Server Logger", function(){
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var logStore;
    beforeEach(inject(function(_logStore_){
        logStore = _logStore_;
    }));

    describe("$log wrapper", function(){
        it("should call logStore.$log on each call to any original $log function", inject(function($log){
            jest.spyOn(logStore, "log").mockImplementation(() => {});
            $log.log("test");
            expect(logStore.log).toHaveBeenCalledWith("info", "test");
        }));
    });

    describe("loggerService", function(){
        var loggerService;
        beforeEach(inject(function(_loggerService_){
            loggerService = _loggerService_;
            jest.spyOn(j.nocPoller, "start").mockImplementation(() => {});
        }));

        it("should start polling upon call to start", function(){
            loggerService.start();
            expect(j.nocPoller.start).toHaveBeenCalled();
            expect(j.nocPoller.start.mock.calls[0][0]).toEqual(loggerService.paramsFn);
        });

        it("should not start polling multiple times upon successive calls to start", function(){
            loggerService.start();
            loggerService.start();
            expect(j.nocPoller.start.mock.calls.length).toEqual(1);
        });

        describe("loggerService.paramsFn", function(){
            it("should return null if there are no current logs in the logStore", function(){
                jest.spyOn(logStore, "popAll").mockReturnValue([]);
                var result = loggerService.paramsFn();
                expect(result).toBeNull();
            });

            it("should return the params object if there are logs in the logStore", function(){
                var logs = ["log1", "log2"];
                jest.spyOn(logStore, "popAll").mockReturnValue(logs);
                var result = loggerService.paramsFn();
                expect(result).toEqual({
                    url: "/occm/client-logs",
                    method: "POST",
                    data: logs
                });
            });
        });
    });

    describe("logStore", function(){
        describe("extractError", function(){
            it("should concat the message and stack if present", function(){
                var result = logStore.extractError({stack: "stack", message: "message"});
                expect(result).toEqual("message\nstack");
            });

            it("should return the arg if there is no message and stack", function(){
                var result = logStore.extractError("message");
                expect(result).toEqual("message");
            });
        });

        describe("popAll", function(){
            it("should return the values of the array and empty the array", function(){
                logStore.popAll();
                logStore.log("test", "test");
                logStore.log("test", "test2");
                expect(logStore.popAll().length).toEqual(2);
                expect(logStore.popAll().length).toEqual(0);
            });
        });

        describe("log", function(){
            it("should push the message with the current time", function(){
                var oldMoment = window.moment;
                window.moment = jest.fn().mockReturnValue({utc: jest.fn().mockReturnValue({format: jest.fn()})});
                logStore.log("info", "message");
                var log = logStore.popAll()[0];
                expect(log.level).toEqual("info");
                expect(log.message).toEqual("message");
                expect(window.moment().utc().format).toHaveBeenCalled();
                window.moment = oldMoment;
            });
        });
    });
});