describe('nocCloudSyncService', function () {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let nocCloudSyncService;

    beforeEach(inject((_nocCloudSyncService_) => {
        nocCloudSyncService = _nocCloudSyncService_;
    }));

    describe('getBuckets', function () {
        it('should work upon success', function () {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(['bucket1', 'bucket2']));

            expect(nocCloudSyncService.getBuckets(j.nocHttp, 'we1'))
                .toBeResolvedWith(['bucket1', 'bucket2']);
        });

        it('should work upon error', function () {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(nocCloudSyncService.getBuckets(j.nocHttp, 'we1'))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.nocHttp.get).toHaveBeenCalledWith({
                url: '/cloudsync/buckets/we1'
            }, 'retrieving S3 buckets', j.nocHttp.LOADING);
        });
    });

    describe('sync', function () {
        it('should work upon success', function () {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.when());

            expect(nocCloudSyncService.sync(j.nocHttp, 'weId1', 'syncRequest'))
                .toBeResolvedWith();
        });

        it('should work upon error', function () {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(nocCloudSyncService.sync(j.nocHttp, 'weId1', 'syncRequest'))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.nocHttp.post).toHaveBeenCalledWith({
                url: '/cloudsync/sync/weId1',
                data: 'syncRequest'
            }, 'creating sync with Cloud Sync', j.nocHttp.BLOCK);
        });
    });

    describe('getBucketSubdirectories', function () {
        it('should work upon success', function () {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when('bucketInformation'));

            expect(nocCloudSyncService.getBucketSubdirectories(j.nocHttp, 'bucket1', 'we1', 'fullPath'))
                .toBeResolvedWith(('bucketInformation'));
        });

        it('should work upon error', function () {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(nocCloudSyncService.getBucketSubdirectories(j.nocHttp, 'bucket1', 'we1', 'fullPath'))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.nocHttp.get).toHaveBeenCalledWith({
                url: '/cloudsync/buckets/bucket1/sub-directories/we1',
                params: {fullPath: 'fullPath'}
            }, 'retrieving sub-directories for S3 bucket bucket1', j.nocHttp.LOADING);
        });
    });


    describe('deleteSync', function () {
        it('should work upon success', function () {
            jest.spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.when());

            expect(nocCloudSyncService.deleteSync(j.nocHttp, 'wePublicId1'))
                .toBeResolved();
        });

        it('should work upon error', function () {
            jest.spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(nocCloudSyncService.deleteSync(j.nocHttp, 'wePublicId1'))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.nocHttp.delete).toHaveBeenCalledWith('/cloudsync/sync/wePublicId1', 'deleting Cloud Sync integration', j.nocHttp.BLOCK);
        });
    });

    describe('syncVolume', function () {
        it('should work upon success', function () {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.when());

            expect(nocCloudSyncService.syncVolume(j.nocHttp, 'weId1', 'volume1'))
                .toBeResolvedWith();
        });

        it('should work upon error', function () {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(nocCloudSyncService.syncVolume(j.nocHttp, 'weId1', 'volume1'))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.nocHttp.post).toHaveBeenCalledWith('/cloudsync/relationship/weId1/volume1',
                'syncing volume volume1 to S3 with cloud sync', j.nocHttp.BLOCK);
        });
    });

    describe('deleteSyncVolume', function () {
        it('should work upon success', function () {
            jest.spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.when());

            expect(nocCloudSyncService.deleteSyncVolume(j.nocHttp, 'weId1', 'volume1'))
                .toBeResolvedWith();
        });

        it('should work upon error', function () {
            jest.spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(nocCloudSyncService.deleteSyncVolume(j.nocHttp, 'weId1', 'volume1'))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.nocHttp.delete).toHaveBeenCalledWith('/cloudsync/relationship/weId1/volume1',
                'deleting Cloud Sync relationship for volume volume1', j.nocHttp.BLOCK);
        });
    });
});
