describe('ColumnsPickerService', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let columnPickerService;
    let columnsService;

    beforeEach(inject((_columnPickerService_) => {
        columnPickerService = _columnPickerService_;
        columnsService = {saveGridState: jest.fn()};
        columnPickerService.columnsService = columnsService;

        columnPickerService.gridOptions = {
            columnDefs: [
                {visible: true, displayName: 'Source', field: 'source.clusterName', disabled: true, pinnedLeft: true, default: true, minWidth: 120, _filterBy: ['source.clusterName', 'source.volumeName', 'source.workingEnvironmentStatus'],
                    cellTemplate: "<div class='ui-grid-cell-contents'><div title='{{COL_FIELD}} ({{::row.entity.source.workingEnvironmentStatus}})'>{{COL_FIELD}} <span style='font-style: italic'>({{::row.entity.source.workingEnvironmentStatus}})</span></div><div><strong title='{{::row.entity.source.volumeName}}'>{{::row.entity.source.volumeName}}</strong></div></div>"},
                {visible: false, displayName: 'Source SVM Name', field: 'source.svmName', minWidth: 120},
                {visible: true, displayName: 'Destination', field: 'destination.clusterName', disabled: false, pinnedLeft: true, default: true, minWidth: 120, _filterBy: ['destination.clusterName', 'destination.volumeName', 'destination.workingEnvironmentStatus'],
                    cellTemplate: "<div class='ui-grid-cell-contents'><div title='{{COL_FIELD}} ({{::row.entity.destination.workingEnvironmentStatus}})'>{{COL_FIELD}} <span style='font-style: italic'>({{::row.entity.destination.workingEnvironmentStatus}})</span></div><div><strong title='{{::row.entity.destination.volumeName}}'>{{::row.entity.destination.volumeName}}</strong></div></div>"},
                {visible: false, displayName: 'Target SVM Name', field: 'destination.svmName', minWidth: 170},
                {visible: true, displayName: 'Mirror State', disabled: true, field: 'mirrorState', default: false, minWidth: 170}
            ]};

        columnPickerService.gridAPI = {core: {notifyDataChange: jest.fn()}};
    }));

    describe('picking visible columns', () => {
        it('should open a dialog', () => {
            jest.spyOn(columnPickerService, 'setShowFunction').mockReturnValue('1');
            magnifispec.mockConfirm();

            columnPickerService.startColumnPicker(columnPickerService.gridOptions, columnPickerService.gridAPI, columnsService, 'Columns Dialog Title');
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Columns Dialog Title', 'assets/templates/dialogs/ui-grid-cols-picker-dialog.html',
                {
                    columns: columnPickerService.gridOptions.columnDefs,
                    width: 500,
                    fromTop: '20%',
                    setShow: columnPickerService.setShowFunction(columnPickerService.gridOptions.columnDefs)
                }
            );
        });

        describe('the set show function', () => {
            let alterFunction;
            beforeEach(() => {
                alterFunction = columnPickerService.setShowFunction(columnPickerService.gridOptions.columnDefs);
            });
            it('gets a state and creates a function that alters it, all none, or defaults', () => {
                alterFunction('all');
                expect(_.every(columnPickerService.gridOptions.columnDefs, 'visible')).toEqual(true);
                alterFunction('none');
                expect(_.every(columnPickerService.gridOptions.columnDefs, (col) => {
                    return !col.visible || (col.disabled && col.visible);
                })).toEqual(true);
                alterFunction();
                expect(_.every(columnPickerService.gridOptions.columnDefs, (col) => {
                    return !col.visible || (col.default && col.visible);
                })).toEqual(true);
            });
        });

        describe('the dialog confirm function', () => {
            it('updates the state of the grid columns and calls update event on the grid, then saves to localstorage', () => {
                const dialogScope = {
                    columns: [
                        {visible: true, displayName: 'Source'},
                        {visible: true, displayName: 'Source SVM Name'},
                        {visible: true, displayName: 'Destination'},
                        {visible: false, displayName: 'Target SVM Name'},
                        {visible: false, displayName: 'Mirror State'}
                    ]
                };
                columnPickerService.columnPickerOnConfirm(dialogScope);
                //They're not the same
                expect(columnPickerService.gridOptions.columnDefs === dialogScope.columns).not.toEqual(true);
                //And yet they are equal
                expect(_.map(columnPickerService.gridOptions.columnDefs, (col) => _.pick(col, 'visible', 'displayName')))
                    .toEqual(dialogScope.columns);
                j.$timeout.flush();

                expect(columnsService.saveGridState).toHaveBeenCalled();
                expect(columnPickerService.gridAPI.core.notifyDataChange).toHaveBeenCalled();
            });
        });
    });
});
