describe.skip('Dialog Service', function () {
    'use strict';
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    var successFunction, failureFunction;

    beforeEach(function () {
        successFunction = jest.fn();
        failureFunction = jest.fn();
    });

    describe('creating an alert', function () {
        it('should call success function after clicking close', function () {
            j.nocDialog.alert('test').then(function () {
                successFunction();
            });
            j.$timeout.flush();
            $('.message button.noc-close').click();
            j.$timeout.flush();
            j.$timeout.flush(1000);
            expect(successFunction).toHaveBeenCalled();
        });
    });

    describe('creating a dialog', function () {
        it('should call success function after clicking close', function () {
            var callbackFunction = jest.fn();
            j.nocDialog.dialog('test', 'assets/templates/dialogs/alert-content.html', {callback: callbackFunction})
                .then(function () {
                    successFunction();
                });
            j.$timeout.flush();
            expect(callbackFunction).toHaveBeenCalled();
            $('.dialog button.noc-close').click();
            j.$timeout.flush();
            j.$timeout.flush(1000);
            expect(successFunction).toHaveBeenCalled();
        });
    });

    describe('creating a confirm', function () {
        it('should call success function after clicking ok', function () {
            j.nocDialog.confirm('test').then(successFunction, failureFunction);
            j.$timeout.flush();
            $('.confirm button.save').click();
            j.$timeout.flush();
            expect(successFunction).toHaveBeenCalled();
            expect(failureFunction).not.toHaveBeenCalled();
        });

        it('should broadcast a \'clicked ok\' event', function () {
            jest.spyOn(j.$rootScope, '$broadcast');
            j.nocDialog.confirm('test', undefined, undefined, 1).then(successFunction, failureFunction);
            j.$timeout.flush();
            $('.confirm button.save').click();
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(j.Constants.broadcastEventNames.MODAL_OK_CLICKED, 1);
        });

        it('should call failure function after clicking cancel', function () {
            j.nocDialog.confirm('test').then(successFunction, failureFunction);
            j.$timeout.flush();
            $('.confirm button.cancel').click();
            j.$timeout.flush();
            expect(successFunction).not.toHaveBeenCalled();
            expect(failureFunction).toHaveBeenCalled();
        });

        describe('moadlOkIfValid', function () {
            beforeEach(function () {
                var templateHtml = '<div><form name="testForm">' +
                    '<input type="text" required="true">' +
                    '<button class="save left" ng-click="modalOkIfValid(\'testForm\')">OK</button>' +
                    '<button class="cancel right" ng-click="modalCancel()" autofocus>Cancel</button>' +
                    '</form></div>';
                j.$templateCache.put('assets/templates/dialogs/default-confirm-content.html', templateHtml);
            });

            it('shouldn\'t validate if form doesn\'t exist', function () {
                j.$templateCache.put('assets/templates/dialogs/default-confirm-content.html', '<div><form name="testForm">' +
                    '<input type="text" required="true">' +
                    '<button class="save left" ng-click="modalOkIfValid(\'doesntExist\')">OK</button>' +
                    '<button class="cancel right" ng-click="modalCancel()" autofocus>Cancel</button>' +
                    '</form></div>');
                j.nocDialog.confirm('test', null, {
                    callback: function () {
                        this['testForm'].$valid = false;
                    }
                }).then(successFunction);
                j.$timeout.flush();
                j.$timeout.flush();
                $('.confirm button.save').click();
                j.$timeout.flush();
                j.$timeout.flush();
                expect(j.nocUIMain.handleFormValidity).not.toHaveBeenCalled();
                expect(successFunction).toHaveBeenCalled();
            });

            it('should validate the form after clicking ok if form is invalid', function () {
                j.nocDialog.confirm('test', null, {
                    callback: function () {
                        this['testForm'].$valid = false;
                    }
                });
                j.$timeout.flush();
                j.$timeout.flush();
                $('.confirm button.save').click();
                expect(j.nocUIMain.handleFormValidity).toHaveBeenCalled();
            });

            it('should call success function after clicking ok only if form is valid', function () {
                j.nocDialog.confirm('test', null, {
                    callback: function () {
                        this['testForm'].$valid = true;
                    }
                })
                    .then(successFunction, failureFunction);
                j.$timeout.flush();
                j.$timeout.flush();
                $('.confirm button.save').click();
                j.$timeout.flush();
                j.$timeout.flush();
                expect(j.nocUIMain.handleFormValidity).not.toHaveBeenCalled();
                expect(successFunction).toHaveBeenCalled();
            });
        });
    });
});