describe("noc local storage", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(function () {
        j.$window.localStorage.clear();
        j.$window.localStorage["bad-key"] = "this is not a jason";
        j.$window.localStorage["good-key"] = '{"name": "name"}';
    });

    describe("isDefined", function () {
        it("should return true iff the key is defined and is a valid json", function () {
            expect(j.nocLocalStorage("undefined-key").isDefined()).toEqual(false);
            expect(j.nocLocalStorage("good-key").isDefined()).toEqual(true);
            expect(j.nocLocalStorage("bad-key").isDefined()).toEqual(true);
        });
    });

    describe("restore", function () {
        it("should return undefined when there is an exception while trying to restore", function () {
            expect(j.nocLocalStorage("bad-key").restore()).toBeUndefined();
        });

        it("should return undefined the key doesn't exist", function () {
            expect(j.nocLocalStorage("no-key").restore()).toBeUndefined();
        });

        it("should return the right key when requested", function () {
            expect(j.nocLocalStorage("good-key").restore()).toEqual({name: "name"});
        });
    });

    describe("clear", function() {
        it("should clean a key", function() {
            var humpty = j.nocLocalStorage('humpty');
            humpty.save('dumpty');
            expect(humpty.restore()).toEqual('dumpty');
            humpty.clear();
            expect(humpty.restore()).toBeUndefined();
        });
    });

    describe("save", function () {
        it("should keep the right jason in string format in the local storage", function () {
            j.nocLocalStorage('json').save({name: "jsonObject"});
            expect(j.$window.localStorage["json"]).toEqual('{"name":"jsonObject"}');
        });

        it("should keep the right value for a boolean in the local storage", function () {
            j.nocLocalStorage('bool').save(false);
            expect(j.$window.localStorage["bool"]).toEqual('false');

        });
    });

    describe('gridStorage', () => {
        const savedGrid ={columns: [
            {name: 'banana', taste: 'sweet'},
            {name: 'lemon', taste:'sour'}
        ]};

        beforeEach(() => {
            j.nocLocalStorage.GALLERY_GRID.save(savedGrid);
        });

        it('should restore like a boss when columns names are not changed', () => {
            expect(j.nocLocalStorage.GALLERY_GRID.restore({grid:{columns:[
                {name: 'banana', taste: 'sweet'},
                {name: 'lemon', taste:'sour'}
            ]}})).toEqual(savedGrid);
        });

        it('should restore null when columns names are changed', () => {
            expect(j.nocLocalStorage.GALLERY_GRID.restore({grid:{columns:[
                {name: 'banana', taste: 'sweet'},
                {name: 'potato', taste:'sour'}
            ]}})).toBeNull();
            expect(j.nocLocalStorage.GALLERY_GRID.isDefined()).toBeFalsy();
        });

        it('should restore null when columns order is changed', () => {
            expect(j.nocLocalStorage.GALLERY_GRID.restore({grid:{columns:[
                {name: 'lemon', taste:'sour'},
                {name: 'banana', taste: 'sweet'}
            ]}})).toBeNull();
            expect(j.nocLocalStorage.GALLERY_GRID.isDefined()).toBeFalsy();
        });
    });

});

