describe('Long Poller Service', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    const responseHeaders = {};
    let defaultGetHeaders,
        requestHeadersWithAsync,
        theActualData = null;

    beforeEach(() => {
        j.$rootScope.appState = 'loaded';
        defaultGetHeaders = {
            'Accept': 'application/json',
            'If-Modified-Since': '0',
            'Cache-Control': 'no-cache',
            'Pragma': 'no-cache'
        };
        requestHeadersWithAsync = _.clone(defaultGetHeaders);
        requestHeadersWithAsync[j.nocLongPollingService.REQUEST_ID_REQUEST_HEADER] = j.nocLongPollingService.getClientId();
    });

    describe('End to End', () => {
        beforeEach(() => {
            responseHeaders[j.nocLongPollingService.REQUEST_ID_RESPONSE_HEADER] = 1;
            j.$httpBackend.expectGET('/occm/api/something', requestHeadersWithAsync).respond(202, {}, responseHeaders);
            j.$httpBackend.expectGET('/occm/api/async/' + j.nocLongPollingService.getClientId() + '?last=null', defaultGetHeaders).respond(200, [{
                requestId: '1',
                statusCode: 200,
                response: {data: 'data1'}
            }]);

            j.$http.get('/something')
                .then((response) => {
                    theActualData = response.data;
                });

            j.$httpBackend.flush();
        });

        it('should request the response from the async API', () => {
            expect(theActualData).toEqual({data: 'data1'});
        });

        it('should send a request to async api with the last request id in the query param', () => {
            j.$httpBackend.expectGET('/occm/api/something', requestHeadersWithAsync).respond(202, {}, responseHeaders);
            j.$httpBackend.expectGET('/occm/api/async/' + j.nocLongPollingService.getClientId() + '?last=1', defaultGetHeaders).respond(200, [{
                requestId: '1',
                statusCode: 200,
                'response': {data: 'data1'}
            }]);

            j.$http.get('/something')
                .then((response) => {
                    theActualData = response.data;
                });

            j.$httpBackend.flush();
            expect(theActualData).toEqual({data: 'data1'});
        });
    });

    describe('racing conditions', () => {
        it('should deal with requests coming not in order', () => {
            responseHeaders[j.nocLongPollingService.REQUEST_ID_RESPONSE_HEADER] = 2;
            j.$httpBackend.expectGET('/occm/api/something', requestHeadersWithAsync).respond(202, {}, responseHeaders);
            j.$httpBackend.expectGET('/occm/api/async/' + j.nocLongPollingService.getClientId() + '?last=null', defaultGetHeaders).respond(200,
                [
                    {requestId: '2', statusCode: 200, 'response': {data: 'someData'}},
                    {requestId: '3', statusCode: 200, 'response': {data: 'out of order'}}
                ]
            );

            j.$http.get('/something');

            j.$httpBackend.flush();

            responseHeaders[j.nocLongPollingService.REQUEST_ID_RESPONSE_HEADER] = 3;
            j.$httpBackend.expectGET('/occm/api/something', requestHeadersWithAsync).respond(202, {}, responseHeaders);

            j.$http.get('/something')
                .then((response) => {
                    theActualData = response.data;
                });

            j.$httpBackend.flush();

            expect(theActualData).toEqual({data: 'out of order'});
        });
    });
});
