describe("Messagable", function () {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(inject(function (Messageable) {
        Messageable.call(j.$rootScope);
    }));

    describe("pushTimedMessage", function () {
        it("should add a success message to the success array and delete it after 5 seconds", function () {
            jest.spyOn(j.$rootScope, "$broadcast");

            j.$rootScope.alertMessages.messageType = [];
            var msg = {message: "I will disappear in 5 seconds", param1: "value1"};
            j.$rootScope.pushTimedMessage(msg, "messageType");

            expect(j.$rootScope.alertMessages.messageType.length).toEqual(1);
            expect(j.$rootScope.alertMessages.messageType[0]).toEqual(msg);
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith("header-change-height");

            j.$timeout.flush(6000);

            expect(j.$rootScope.alertMessages.messageType.length).toEqual(0);
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith("header-change-height");
        });

        it("should clear every message at it's time", function () {
            jest.spyOn(j.$rootScope, "$broadcast");

            j.$rootScope.pushSuccess("it was a success1");
            j.$timeout(function () {
                j.$rootScope.pushSuccess("it was a success2");
            }, 1000);
            expect(j.$rootScope.alertMessages.successes.length).toEqual(1);

            j.$timeout.flush(1000);
            expect(j.$rootScope.alertMessages.successes.length).toEqual(2);
            expect(j.$rootScope.alertMessages.successes[0]).toEqual({message: "it was a success1", duration: 6000});

            j.$timeout.flush(5000);
            expect(j.$rootScope.alertMessages.successes.length).toEqual(1);
            expect(j.$rootScope.alertMessages.successes[0]).toEqual({message: "it was a success2", duration: 6000});

            j.$timeout.flush(1000);
            expect(j.$rootScope.alertMessages.successes.length).toEqual(0);
        });

        it("should respect custom duration type", function () {
            jest.spyOn(j.$rootScope, "$broadcast");

            j.$rootScope.alertMessages.messageType = [];
            var msg = {message: "I will disappear in 1.234 seconds", param1: "value1", duration: 1234};
            j.$rootScope.pushTimedMessage(msg, "messageType");

            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith("header-change-height");

            j.$timeout.flush(6000);

            expect(j.$rootScope.alertMessages.messageType.length).toEqual(0);
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith("header-change-height");
        });
    });

    describe("pushSuccess", function () {
        it("should call pushTimedMessage with appropriate params", function () {
            jest.spyOn(j.$rootScope, "pushTimedMessage").mockImplementation(() => {});
            j.$rootScope.pushSuccess("it was a success");
            expect(j.$rootScope.pushTimedMessage).toHaveBeenCalledWith({message: "it was a success"}, "successes");
        });
    });

    describe("inProgress", function () {
        it("should call pushTimedMessage with appropriate params (default duration)", function () {
            jest.spyOn(j.$rootScope, "pushTimedMessage").mockImplementation(() => {});
            j.$rootScope.pushInProgress("Something is in progress", true);
            expect(j.$rootScope.pushTimedMessage).toHaveBeenCalledWith({message: "Something is in progress", seeTimeline: true, duration: undefined}, "inProgress");
        });

        it("should call pushTimedMessage with appropriate params (custom duration)", function () {
            jest.spyOn(j.$rootScope, "pushTimedMessage").mockImplementation(() => {});
            j.$rootScope.pushInProgress("Something is in progress", false, 1234);
            expect(j.$rootScope.pushTimedMessage).toHaveBeenCalledWith({message: "Something is in progress", seeTimeline: false, duration: 1234}, "inProgress");
        });
    });

    describe("pushError", function () {
        beforeEach(function () {
            jest.spyOn(j.$rootScope, "$broadcast").mockImplementation(() => {});
        });

        it("should change the caller, caller type and message when the error is a network error", function () {
            var errResponse = {status:"NETWORK_FAILURE", message:"network error message"};
            j.$rootScope.pushError("original message", errResponse, "originalCallerKey", "originalCallerType");

            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith("header-change-height");
            expect(j.$rootScope.alertMessages.errors[0]).toEqual(j.$rootScope.errorsMap["NETWORK"].NETWORK);
            expect(j.$rootScope.alertMessages.errors[0].message).toEqual("network error message");
        });

        it("should push an error if doesn't exist yet, and should replace the error message with the new one when the caller and type already exists and the message is not dismissed", function () {
            var errResponse = {message:"first error message"};
            j.$rootScope.pushError("original message", errResponse, "originalCallerKey", "originalCallerType");
            expect(j.$rootScope.alertMessages.errors[0]).toEqual(j.$rootScope.errorsMap["originalCallerKey"].originalCallerType);
            expect(j.$rootScope.alertMessages.errors[0].message).toEqual("original message");

            j.$rootScope.pushError("second error message", errResponse, "originalCallerKey", "originalCallerType");
            expect(j.$rootScope.alertMessages.errors[0]).toEqual(j.$rootScope.errorsMap["originalCallerKey"].originalCallerType);
            expect(j.$rootScope.alertMessages.errors[0].message).toEqual("second error message");
        });

        it("should not replace the error message with the new one when the caller and type already exists and the message is dismissed and should not add the error to the alertMessages", function () {
            var errResponse = {message:"first error message"};
            j.$rootScope.pushError("original message", errResponse, "originalCallerKey", "originalCallerType");
            j.$rootScope.alertMessages.errors[0].dismiss();
            expect(j.$rootScope.errorsMap["originalCallerKey"].originalCallerType.dismissed).toEqual(true);
            expect(j.$rootScope.alertMessages[0]).toBeUndefined();

            j.$rootScope.pushError("second error message", errResponse, "originalCallerKey", "originalCallerType");
            expect(j.$rootScope.alertMessages.errors[0]).toBeUndefined();
            expect(j.$rootScope.errorsMap["originalCallerKey"].originalCallerType.message).toEqual("original message");

        });

        it("should push another message if the callerType is different", function () {
            var errResponse = {message:"first error message"};
            j.$rootScope.pushError("original message", errResponse, "originalCallerKey", "originalCallerType");
            expect(j.$rootScope.alertMessages.errors[0]).toEqual(j.$rootScope.errorsMap["originalCallerKey"].originalCallerType);
            expect(j.$rootScope.alertMessages.errors[0].message).toEqual("original message");

            j.$rootScope.pushError("second error message", errResponse, "originalCallerKey", "secondCallerType");
            expect(j.$rootScope.alertMessages.errors[0]).toEqual(j.$rootScope.errorsMap["originalCallerKey"].originalCallerType);
            expect(j.$rootScope.alertMessages.errors[1]).toEqual(j.$rootScope.errorsMap["originalCallerKey"].secondCallerType);
            expect(j.$rootScope.alertMessages.errors[0].message).toEqual("original message");
            expect(j.$rootScope.alertMessages.errors[1].message).toEqual("second error message");

        });
    });

    describe("dismiss", function () {
        it("should delete the message from the map on force", function () {
            var errResponse = {message:"first error message"};
            j.$rootScope.pushError("first error message", errResponse, "caller", "callerType");
            expect(j.$rootScope.alertMessages.errors.length).toEqual(1);
            expect(j.$rootScope.errorsMap["caller"].callerType).toEqual(j.$rootScope.alertMessages.errors[0]);

            j.$rootScope.alertMessages.errors[0].dismiss(true);

            expect(j.$rootScope.alertMessages.errors.length).toEqual(0);
            expect(j.$rootScope.errorsMap["caller"].callerType).toBeUndefined();
        });

        it("should only set the dismissed flag when force is not defined", function () {
            var errResponse = {message:"first error message"};
            j.$rootScope.pushError("first error message", errResponse, "caller", "callerType");
            expect(j.$rootScope.alertMessages.errors.length).toEqual(1);
            expect(j.$rootScope.errorsMap["caller"].callerType).toEqual(j.$rootScope.alertMessages.errors[0]);
            expect(j.$rootScope.errorsMap["caller"].callerType.dismissed).toEqual(false);

            j.$rootScope.alertMessages.errors[0].dismiss(false);

            expect(j.$rootScope.alertMessages.errors.length).toEqual(0);
            expect(j.$rootScope.errorsMap["caller"].callerType.dismissed).toEqual(true);
        });

        it("should broadcast the 'header-change-height' message on dismiss of a message", function () {
            jest.spyOn(j.$rootScope,"$broadcast").mockImplementation(() => {});

            var errResponse = {message:"first error message"};
            j.$rootScope.pushError("first error message", errResponse, "caller", "callerType");

            j.$rootScope.alertMessages.errors[0].dismiss(false);

            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith("header-change-height");
        });

        it("should set show errors to false when the alertMessages array size becomes 1 after dismissing an error", function () {
            var errResponse = {message:"first error message"};
            j.$rootScope.pushError("first error message", errResponse, "caller", "callerType");
            j.$rootScope.pushError("second error message", errResponse, "caller", "callerType2");


            j.$rootScope.alertMessages.errors[0].dismiss(false);

            expect(j.$rootScope.alertMessages.showErrors).toEqual(false);

        });
    });

    describe("clearing messages", function () {
        it("it should call dismiss on all the errors when dismissAllErros is called on alertMessages", function () {
            var dismissSpy = jest.fn();
            j.$rootScope.alertMessages.errors = [{dismiss: dismissSpy}, {dismiss: dismissSpy}, {dismiss: dismissSpy}];

            j.$rootScope.alertMessages.dismissAllErrors();

            expect(dismissSpy.mock.calls.length).toEqual(3);
        });
    });

    describe("clearMessages", function () {
        it("should always clear all the network caller errors if there is a caller type", function () {
            jest.spyOn(j.$rootScope, "clearMessages");

            j.$rootScope.clearMessages("caller", "callerType");

            expect(j.$rootScope.clearMessages).toHaveBeenCalledWith("caller", "callerType");
            expect(j.$rootScope.clearMessages).toHaveBeenCalledWith("NETWORK");
        });

        it("should call dismiss with force on all the errors of a caller when only a caller is defined", function () {
            var dismissSpy = jest.fn();
            j.$rootScope.errorsMap = {caller: {type1: {dismiss: dismissSpy}, type2: {dismiss: dismissSpy}, type3: {dismiss: dismissSpy}}};

            j.$rootScope.clearMessages("caller");

            expect(dismissSpy.mock.calls.length).toEqual(3);
            expect(dismissSpy.mock.calls[0][0]).toEqual(true);
            expect(dismissSpy.mock.calls[1][0]).toEqual(true);
            expect(dismissSpy.mock.calls[2][0]).toEqual(true);
        });

        it("should call dismiss only on the error of the callerType if callerType is given", function () {
            var dismissSpy1 = jest.fn();
            var dismissSpy2 = jest.fn();
            var dismissSpy3 = jest.fn();
            j.$rootScope.errorsMap = {caller: {type1: {dismiss: dismissSpy1}, type2: {dismiss: dismissSpy2}, type3: {dismiss: dismissSpy3}}};

            j.$rootScope.clearMessages("caller", "type2");

            expect(dismissSpy2).toHaveBeenCalledWith(true);
            expect(dismissSpy1).not.toHaveBeenCalled();
            expect(dismissSpy3).not.toHaveBeenCalled();
        });
    });

});
