describe('Policy Service', () => {
    'use strict';

    let policiesService, policies;
    let j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(inject((nocPoliciesService) => {
        policiesService = nocPoliciesService;
        policies = [
            {name: 'DPDefault', isDefault: false, isSupported: true, reasons: [], type: 'async_mirror', description: 'Default policy for DP relationship', rules: [{snapMirrorLabel: 'sm_created', retention: 1}]},
            {name: 'MirrorAllSnapshots', isDefault: true, isSupported: true, reasons: [], type: 'async_mirror', description: 'Asynchronous SnapMirror policy for mirroring all snapshots and latest active file system', rules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}]},
            {name: 'MirrorAndVault', isDefault: false, isSupported: true, reasons: [], type: 'mirror_vault', description: 'A unified Asynchronous SnapMirror and SnapVault policy for mirroring the latest active file system and daily and weekly snapshots', rules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'daily', retention: 7}]},
            {name: 'XDPDefault', isDefault: false, isSupported: false, reason: 'unsupported version', type: 'vault', description: 'Default policy for XDP relationship with daily and weekly rules', rules: [{snapMirrorLabel: 'daily', retention: 7}]},
            {name: 'NoRules', isDefault: false, isSupported: true, reasons: [], type: 'vault', description: '', rules: []}
        ];
        jest.spyOn(j.$rootScope, '$broadcast');
    }));

    describe('getPolicies', () =>  {
        it('should broadcast the policies upon success ', () =>  {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.when(policies));
            var extendedPolices = [
                _.assignIn(policies[0],{ _policyInfo: {displayName: 'Mirror latest Snapshot copy only (legacy)', description: 'Typically used for a one-time data replication for ONTAP 8.2 and earlier'}, _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC, _policyImage : 'assets/images/wizard/snapmirror-policy.png'}),
                _.assignIn(policies[1],{ _policyInfo: {displayName: 'Mirror', description: 'Typically used for disaster recovery'}, _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}], _defaultRulesDescription: policiesService.COMBINED_DEFAULT_RULES_DESC,_policyImage : 'assets/images/wizard/snapmirror-policy.png'}),
                _.assignIn(policies[2],{ _policyInfo: {displayName: 'Mirror and Backup', description: 'Typically used when both disaster recovery and long-term retention are needed'},_otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC, _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2, _labelAndRetentionSum: 'daily (7)', _policyImage : 'assets/images/wizard/snapmirror-policy.png'}),
                _.assignIn(policies[3],{ _policyInfo:{displayName: 'Backup', description: 'Typically used for long-term retention of backup copies'}, _otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [], _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2, _labelAndRetentionSum: 'daily (7)',_policyImage : 'assets/images/wizard/snapmirror-policy-disabled.png'}),
                _.assignIn(policies[4],{ _policyInfo:{displayName: 'NoRules', description: undefined}, _otherRules : [], _defaultRules: [],_policyImage : 'assets/images/wizard/snapmirror-policy.png'})
            ];
            expect(policiesService.getPolicies(j.nocHttp, 'sourceWEId', 'targetWEId')).toBeResolvedWith(extendedPolices);
        });

        it('should broadcast an error message with an error in case of an error in the http request', () =>  {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.reject(j.errorResponse));

            expect(policiesService.getPolicies(j.nocHttp, 'sourceWEId', 'targetWEId')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith('/replication/metadata/snapmirror-policies/sourceWEId/targetWEId', 'retrieving replication policies', j.nocHttp.LOADING);
        });
    });

    describe('getSnapshotLabels', () =>  {
        it('should broadcast the snapshot labels message upon success ', () =>  {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.when(['daily']));
             var policies1 = [
                 _.assignIn(policies[0],{_displayName: 'DPDefault', _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC}),
                 _.assignIn(policies[1],{_displayName: 'Mirror all Snapshot copies', _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}], _defaultRulesDescription: policiesService.COMBINED_DEFAULT_RULES_DESC}),
                 _.assignIn(policies[2],{_displayName: 'MirrorAndVault', _otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC, _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2}),
                 _.assignIn(policies[3],{_displayName: 'XDPDefault', _otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [], _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2})
             ];
            var policiesWithSnapshots = [
                _.assignIn(policies[0],{_displayName: 'DPDefault', _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC, _snapshotWarning: false}),
                _.assignIn(policies[1],{_displayName: 'Mirror all snapshots', _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}], _defaultRulesDescription: policiesService.COMBINED_DEFAULT_RULES_DESC, _snapshotWarning: false }),
                _.assignIn(policies[2],{_displayName: 'MirrorAndVault', _otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC, _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2, _snapshotWarning: false}),
                _.assignIn(policies[3],{_displayName: 'XDPDefault', _otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [], _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2, _snapshotWarning: false})
            ];

            expect(policiesService.getSnapshotLabels(j.nocHttp, policies1, 'sourceWEId', 'svmName', 'volumeName'))
                .toBeResolvedWith(policiesWithSnapshots);
        });

        it('should broadcast a snapshot labels error message with an error in case of an error in the http request', () =>  {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.reject(j.errorResponse));
            var policies1 = [
                _.assignIn(policies[0],{_displayName: 'DPDefault', _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC}),
                _.assignIn(policies[1],{_displayName: 'Mirror all Snapshot copies', _otherRules : [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}], _defaultRulesDescription: policiesService.COMBINED_DEFAULT_RULES_DESC}),
                _.assignIn(policies[2],{_displayName: 'MirrorAndVault', _otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC, _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2}),
                _.assignIn(policies[3],{_displayName: 'XDPDefault', _otherRules : [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [], _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2})
            ];

            expect(policiesService.getSnapshotLabels(j.nocHttp, policies1, 'sourceWEId', 'svmName', 'volumeName'))
                .toBeRejectedWith(j.errorResponse);
        });
        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith({
                url: '/replication/metadata/snapshot-labels/sourceWEId/svmName/volumeName',
                params: {labels: ['daily']}
            }, 'retrieving Snapshots labels', j.nocHttp.LOADING);
        });
    });

    describe('build labels list', () => {
        it('should build a list of unique labels split by , for each of the other rules in policies', () => {
            expect(policiesService.buildLabelsList([
                _.assignIn(policies[0], {_displayName: 'DPDefault', _otherRules: [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC}),
                _.assignIn(policies[1], {_displayName: 'Mirror all snapshots', _otherRules: [], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}], _defaultRulesDescription: policiesService.COMBINED_DEFAULT_RULES_DESC}),
                _.assignIn(policies[2], {_displayName: 'MirrorAndVault', _otherRules: [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [{snapMirrorLabel: 'sm_created', retention: 1}], _defaultRulesDescription: policiesService.SM_CREATED_DESC, _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2}),
                _.assignIn(policies[3], {_displayName: 'XDPDefault', _otherRules: [{snapMirrorLabel: 'daily', retention: 7}], _defaultRules: [], _otherRulesDescription1: policiesService.OTHER_RULES_DESC1, _otherRulesDescription2: policiesService.OTHER_RULES_DESC2})
            ])).toEqual(['daily']);
        });
    });

    describe('findExistingLabels', () => {
        it('should return true if any of the mirror labels in the rules exist in the snapshot labels', () => {
            expect(policiesService.findExistingLabels(policies[2].rules, ['daily'])).toEqual(true);
        });

        it('should return false if none of the mirror labels in the rules exist in the snapshot labels', () => {
            expect(policiesService.findExistingLabels(policies[2].rules, ['weekly'])).toEqual(false);
        });
    });

    describe('getPolicyInfo', () => {
        it('should take the name from the map if the name exists there', () => {
            expect(policiesService.getPolicyInfo(policies[1].name)).toEqual({displayName: 'Mirror', description: 'Typically used for disaster recovery'});
        });

        it('should use return the original name if the name is not found in the map', () => {
            expect(policiesService.getPolicyInfo(policies[2].name)).toEqual({displayName: 'Mirror and Backup (1 year retention, weekly)', description: 'Configures disaster recovery and long-term retention of backups on the same destination volume'});
        });

    });

    describe('snapshotWarningIfNeeded', () => {
        it('be undefined for policy with default rules', () => {
            expect(policiesService.snapshotWarningIfNeeded({_defaultRules: [{}]})).toBeUndefined();
        });
        it('be undefined for policy with other rules with matching labels', () => {
            expect(policiesService.snapshotWarningIfNeeded({_otherRules: [{snapMirrorLabel: 'something'}]}, ['something'])).toBeUndefined();
        });
        it('be "no matching labels" for policy with other rules with no matching label', () => {
            expect(policiesService.snapshotWarningIfNeeded({_otherRules: [{snapMirrorLabel: 'something'}]}, ['else'])).toMatch(/.*snapshot.*match.*label.*/i);
        });
        it('be "no rules" for policy with no other rules', () => {
            expect(policiesService.snapshotWarningIfNeeded({}, ['doesnt matter'])).toMatch(/.*no.*rules.*/i);
        });
    });
});
