describe("nocSchedulesService", function () {
    "use strict";

    var schedulesService, response;
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(inject(function (nocSchedulesService) {
        schedulesService = nocSchedulesService;
        response = [
            {"name": "5min", "description": "@:00,:05,:10,:15,:20,:25,:30,:35,:40,:45,:50,:55", "cronJobSchedule": {"months": [], "days": [], "weekDays": [], "hours": [], "minutes": [0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55]}},
            {"name": "8hour", "description": "@2:15,10:15,18:15", "cronJobSchedule": {"months": [], "days": [], "weekDays": [], "hours": [2, 10, 18], "minutes": [15]}},
            {"name": "daily", "description": "@0:10", "cronJobSchedule": {"months": [], "days": [], "weekDays": [], "hours": [0], "minutes": [10]}},
            {"name": "hourly", "description": "@:05", "cronJobSchedule": {"months": [], "days": [], "weekDays": [], "hours": [], "minutes": [5]}},
            {"name": "weekly", "description": "Sun@0:15", "cronJobSchedule": {"months": [], "days": [], "weekDays": [0], "hours": [0], "minutes": [15]}},
            {"name": "weird", "description": "description", "cronJobSchedule": {"months": [0, 3, 6], "days": [], "weekDays": [0, 5, 6], "hours": [0], "minutes": [22]}},
            {"name": "monthly", "description": "description", "cronJobSchedule": {"months": [], "days": [0], "weekDays": [], "hours": [15], "minutes": [31]}},
            {"name": "Example 1", "description": "In January only, every hour at @:05", "cronJobSchedule": {"months": [0], "days": [], "weekDays": [], "hours": [], "minutes": [5]}}
        ];
    }));

    describe("getSchedules", function () {
        var targetWorkingEnvironment;
        beforeEach(function () {
            targetWorkingEnvironment = {_pathPrefix: "/vsa/", publicId: "value2", name: "workingEnv1"};
        });

        it("should work upon success", function () {
            jest.spyOn(j.replicationResource, "getSchedules").mockReturnValue(j.$q.resolve(response));
            jest.spyOn(schedulesService, "stringifyCron").mockReturnValue(["Array", "of", "Strings"]);

            var extendedResponse = _.cloneDeep(response);
            _.forEach(extendedResponse, function (response) {
                _.assignIn(response, {descArr: ["Array", "of", "Strings"]});
            });

            expect(schedulesService.getSchedules(j.nocHttp, targetWorkingEnvironment.publicId)).toBeResolvedWith(extendedResponse);

            expect(schedulesService.stringifyCron.mock.calls.length).toEqual(response.length);
        });

        it("should work upon error", function () {
            jest.spyOn(j.replicationResource, "getSchedules").mockReturnValue(j.$q.reject(j.errorResponse));

            expect(schedulesService.getSchedules(j.nocHttp, targetWorkingEnvironment.publicId)).toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.replicationResource.getSchedules).toHaveBeenCalledWith(j.nocHttp, 'value2');
        });
    });

    describe("stringifyCron", function () {
        var strings = [
            ['Every hour', 'Minutes: 0th, 5th, 10th, 15th, 20th, 25th, 30th, 35th, 40th, 45th, 50th and 55th minute'],
            ['Every day', 'Hours: 2 AM, 10 AM and 6 PM', 'Minutes: 15th minute'],
            ['Every day', 'Hours: 12 AM', 'Minutes: 10th minute'],
            ['Every hour', 'Minutes: 5th minute'],
            ['Every week', 'Days: Sun', 'Hours: 12 AM', 'Minutes: 15th minute'],
            ['Months: Jan, Apr and Jul', 'Every week', 'Days: Sun, Fri and Sat', 'Hours: 12 AM', 'Minutes: 22nd minute'],
            ['Every month', 'Days: 1st', 'Hours: 3 PM', 'Minutes: 31st minute'],
            ['Months: Jan', 'Every day', 'Every hour', 'Minutes: 5th minute']
        ];

        it("Should stringify the crons correctly", function () {
            for (var i = 0; i < response.length; ++i) {
                expect(schedulesService.stringifyCron(response[i].cronJobSchedule)).toEqual(strings[i]);
            }
        });
    });

});

