describe('tenants service', function () {
    'use strict';

    let nocTenantsService, nocCurrentUserService, nocSelectedWorkspaceOrTenantService;
    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    var tenants = [{_name: 'the we admin', name: 'the we admin', publicId: 1}, {
        _name: 'the oncloud admin',
        name: 'the oncloud admin',
        publicId: 2
    }];

    beforeEach(inject(function (_nocTenantsService_, _nocCurrentUserService_, _nocSelectedWorkspaceOrTenantService_) {
        j.$rootScope.currentUser = {};
        j.$rootScope.checkAndSetState = jest.fn();
        j.$httpBackend.whenGET(j.nocBaseUrl + '/tenants').respond(200, tenants);

        nocTenantsService = _nocTenantsService_;
        nocTenantsService.reloadTenants();
        j.$httpBackend.flush();
        j.$rootScope.selectedTenant = undefined;

        nocCurrentUserService = _nocCurrentUserService_;
        nocSelectedWorkspaceOrTenantService = _nocSelectedWorkspaceOrTenantService_;
    }));

    describe('getTenantsFromServer', function () {
        it('should update the tenants', function () {
            j.$httpBackend.expectGET('/occm/api/tenants?fields=workingEnvironmentsInformation').respond(200, tenants);

            nocTenantsService.getTenantsFromServer(j.nocHttp, {workingEnvironmentsInformation: true});

            j.$httpBackend.flush();

            expect(nocTenantsService.getTenants()).toEqual(tenants);
        });
    });

    describe('setSelectedTenant', function () {
        it('should set the tenant on the rootScope', function () {
            jest.spyOn(j.nocLocalStorage.SELECTED_TENANT, 'save').mockImplementation(() => {
            });
            nocTenantsService.setSelectedTenant({_pathPrefix: '/vsa/', publicId: 'kuki1'});
            expect(j.$rootScope.selectedTenant).toEqual({_pathPrefix: '/vsa/', publicId: 'kuki1'});
        });
    });

    describe('resolveTenantFromCurrentUserOrUrl', function () {
        it('should fetch the tenant from WE and Tenant Admins', function () {
            jest.spyOn(nocCurrentUserService, 'getCurrentUser').mockReturnValue({tenantId: 1});
            expect(nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl()).toEqual(tenants[0]);
        });
        it('should fetch the tenant from the url', function () {
            jest.spyOn(nocCurrentUserService, 'getCurrentUser').mockReturnValue({});//no tenant id === onCloud admin
            expect(nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl({slTenantId: 2})).toEqual(tenants[1]);

            expect(nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl({slTenantId: 1})).toEqual(tenants[0]);
        });
        it('should fetch the first tenant as last option', function () {
            j.$rootScope.currentUser = {};//no tenant id === onCloud admin
            expect(nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl({})).toEqual(tenants[0]);
        });
        it('should fetch the first tenant as last option if the user tenant is not found', function () {
            j.$rootScope.currentUser = {tenantId: -1};
            expect(nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl({tenantId: 1})).toEqual(tenants[0]);
        });
        it('should fetch the first tenant as last option if the url tenant is not found', function () {
            j.$rootScope.currentUser = {};
            expect(nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl({slTenantId: -1})).toEqual(tenants[0]);
        });

        it('should use the locally stored user if exists', function () {
            jest.spyOn(j.nocLocalStorage.SELECTED_TENANT, 'restore').mockReturnValue(2);

            j.$rootScope.currentUser = {};
            expect(nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl({})).toEqual(tenants[1]);
        });
    });

    describe('getTenantNssData', function () {

        it('should return all tenant nss data in a promise', function () {
            // mock http responses
            j.$httpBackend.expectGET(j.nocBaseUrl + '/tenants/' + tenants[0].publicId)
                .respond(200, {nssUserName: 'invalidUserName'});
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/tenants/' + tenants[0].publicId + '/validate')
                .respond(200, {isValid: false, reason: 'invalid credentials'});

            // mock promise and spy and success callback
            var onSuccess = jest.fn();
            nocTenantsService.getTenantNssData(j.nocHttp, tenants[0].publicId, true)
                .then(onSuccess);

            j.$httpBackend.flush();

            expect(onSuccess).toHaveBeenCalledWith({
                nssUserName: 'invalidUserName',
                isNssValid: false,
                nssInvalidReason: 'invalid credentials'
            });
        });

        it('should only return tenant user name in a promise if not validated', function () {
            // mock http response
            j.$httpBackend.expectGET(j.nocBaseUrl + '/tenants/' + tenants[0].publicId)
                .respond(200, {nssUserName: 'userName'});

            // mock promise and spy and success callback
            var onSuccess = jest.fn();
            nocTenantsService.getTenantNssData(j.nocHttp, tenants[0].publicId, false)
                .then(onSuccess);

            j.$httpBackend.flush();

            expect(onSuccess).toHaveBeenCalledWith({
                nssUserName: 'userName',
                isNssValid: false, // default return value
                nssInvalidReason: null // default return value
            });
        });
    });
});
