describe("Users Controller", function () {
    "use strict";


    var changePasswordUrl;

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(function(){
        changePasswordUrl = j.nocBaseUrl + "/auth/current-user/change-password";

        magnifispec.mockDialogController("ChangePasswordController",{});
    });

    describe("Is change password valid", function () {
        it("should return true if the changePasswordForm is valid", function () {
            j.scope.changePasswordForm = {$valid:true};
            expect(j.scope.isChangePasswordValid()).toEqual(true);
        });

        it("should return false if the changePasswordForm is invalid", function () {
            j.scope.changePasswordForm = {$valid:false};
            expect(j.scope.isChangePasswordValid()).toEqual(false);
        });
    });

    describe("Password handling", function () {

        describe("Saving password", function () {
            it("should call handle form validity if invalid", function () {
                jest.spyOn(j.scope, "isChangePasswordValid").mockReturnValue(false);

                j.scope.save();

                expect(j.scope.isChangePasswordValid).toHaveBeenCalled();
                expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
            });

            it("should make a put request with the new password and the old password, and if successful call backTo", function () {
                jest.spyOn(j.scope, "isChangePasswordValid").mockReturnValue(true);

                j.scope.changePassword = {newPassword:"new", oldPassword: "old"};

                j.$httpBackend.expectPUT(changePasswordUrl, {newPassword:"new", oldPassword: "old"}).respond(200);

                j.scope.save();
                j.$httpBackend.flush();

                expect(j.nocUI.block.mock.calls.length).toEqual(1);
                expect(j.nocUI.unblock.mock.calls.length).toEqual(1);

                expect(j.parentScope.closeDialog).toHaveBeenCalled();
                expect(j.pidgeon.clear).toHaveBeenCalledWith("changing password");
            });

            it("should make a put request with the new password, and if failure push an alert", function () {
                jest.spyOn(j.scope, "isChangePasswordValid").mockReturnValue(true);

                j.scope.changePassword = {newPassword:"new", oldPassword: "old"};

                j.$httpBackend.expectPUT(changePasswordUrl, {newPassword:"new", oldPassword: "old"}).respond(400, j.errorResponse);

                j.scope.save();
                j.$httpBackend.flush();
                expect(j.nocUI.block.mock.calls.length).toEqual(1);
                expect(j.nocUI.unblock.mock.calls.length).toEqual(1);

                expect(j.parentScope.closeDialog).not.toHaveBeenCalled();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });
        });

        describe("Cancelling change password", function () {
            it("should call back to", function () {
                j.scope.cancel();
                expect(j.parentScope.closeDialog).toHaveBeenCalled();
            });
        });
    });
});