describe("Update Controller", function () {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(function(){
        j.$rootScope.externalConfig = {};
        j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/system/available-update-versions").respond(200, ["version1"]);
        magnifispec.mockDialogController("UpdateController",{});
        j.$httpBackend.flush();
    });

    describe("initialize", function () {
        it("should work upon success", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/system/available-update-versions").respond(200, ["version3", "version4"]);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.versions).toEqual(["version3", "version4"]);
        });

        it("should work upon error", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/system/available-update-versions").respond(400, j.errorResponse);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.versions).toEqual(["version1"]);
        });
    });

    describe("versionNumber", function () {
        it("should extract the number from normal occm version name", function () {
            expect(j.scope.versionNumber('occm-1')).toBe('1');
            expect(j.scope.versionNumber('occm-2.1')).toBe('2.1');
        });

        it("should use the name if it is not in the expected format", function () {
            expect(j.scope.versionNumber('1.1')).toBe('1.1');
            expect(j.scope.versionNumber('patch-2.4.1')).toBe('patch-2.4.1');
        });
    });

    describe("network failures", function() {
        it("should stop expecting network failures after destroyed", function () {
            j.$rootScope.expectNetworkFailures = true;
            j.scope.$broadcast('$destroy');
            expect(j.$rootScope.expectNetworkFailures).toEqual(false);
        });
    });

    describe("show patch dialog", function () {
        beforeEach(function () {
            magnifispec.fakeConfirmNew(true);
            j.$rootScope.about = {};
        });
        it("should clear the pidgen and call confirm with the right eula for a non beta version", function () {
            j.$rootScope.about.beta = false;
            j.scope.showPatchDialog();
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocDialog.confirm).toHaveBeenCalledWith("",
                "assets/templates/dialogs/update-occm-confirm.html",
                {
                    downloadBackupToClient: true,
                    eulaUrl: "https://s3-us-west-1.amazonaws.com/cloud.support.netapp.com/occm/docs/eula_license.rtf",
                    width: 500
                },
                "UPLOAD_DIALOG");
        });

        it("should clear the pidgen and call confirm with the right eula for a beta version", function () {
            j.$rootScope.about.beta = true;
            j.scope.showPatchDialog();
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocDialog.confirm).toHaveBeenCalledWith("",
                "assets/templates/dialogs/update-occm-confirm.html",
                {
                    downloadBackupToClient: true,
                    eulaUrl: "https://s3-us-west-1.amazonaws.com/cloud.support.netapp.com/occm/docs/eula_license_beta.pdf",
                    width: 500
                },
                "UPLOAD_DIALOG");
        });
    });
});