describe('AddOrEditKeyManagerController', () => {
    'use strict';

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    beforeEach(() => {
        magnifispec.mockDialogController('AddOrEditKeyManagerController');
    });

    describe('saveAddKeyManager', () => {
        it('should handle form validity when not valid', () => {
            j.scope.addKeyManagerForm = {$valid: false};
            j.scope.saveAddKeyManager();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        describe('when form is valid', () => {
            beforeEach(() => {
                j.scope.addKeyManagerForm = {$valid: true};
                j.scope.gridOptionsKeyManagers = {};
                j.scope.name = 'a';
                j.scope.address = '2.3.4.5';
                j.scope.usernameObject.usernameField = 'Country';
                j.scope.usernameObject.usernameValue = 'Nepal';
            });

            it('should work upon success', () => {
                j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/encryption/key-managers', {
                    name: 'a',
                    address: '2.3.4.5',
                    usernameField: 'Country',
                    usernameValue: 'Nepal'
                }).respond(200, 'value1');

                j.scope.saveAddKeyManager();
                j.$httpBackend.flush();

                expect(j.scope.closeDialog).toHaveBeenCalled();
            });

            it('should replace empty usernameField or usernameValue with undefined.', () => {
                j.scope.usernameObject.usernameField = '';
                j.scope.usernameObject.usernameValue = '';
                j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/encryption/key-managers', {
                    name: 'a',
                    address: '2.3.4.5',
                    usernameField: undefined,
                    usernameValue: undefined
                }).respond(200, 'value1');

                j.scope.saveAddKeyManager();
                j.$httpBackend.flush();

                expect(j.scope.closeDialog).toHaveBeenCalled();
            });

            it('should work upon error', () => {
                j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/encryption/key-managers', {
                    name: 'a',
                    address: '2.3.4.5',
                    usernameField: 'Country',
                    usernameValue: 'Nepal'
                }).respond(400, j.errorResponse);

                j.scope.saveAddKeyManager();
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            afterEach(() => {
                expect(j.pidgeon.clear).toHaveBeenCalled();
                expect(j.nocUI.block).toHaveBeenCalled();
                expect(j.nocUI.unblock).toHaveBeenCalled();
            });
        });
    });

    describe('saveEditKeyManager', () => {
        it('should handle form validity when not valid', () => {
            j.scope.addKeyManagerForm = {$valid: false};
            j.scope.saveEditKeyManager();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        describe('when form is valid', () => {
            beforeEach(() => {
                j.scope.addKeyManagerForm = {$valid: true};
                j.scope.gridOptionsKeyManagers = {};
                j.scope.name = 'a';
                j.scope.address = '1.2.3.4';
                j.scope.publicId = 'pid1';
                j.scope.usernameObject.usernameField = 'Country';
                j.scope.usernameObject.usernameValue = 'Tibet';
            });

            it('should work upon success', () => {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/encryption/key-managers/pid1', {
                    name: 'a',
                    address: '1.2.3.4',
                    usernameField: 'Country',
                    usernameValue: 'Tibet'
                }).respond(200, 'value1');

                j.scope.saveEditKeyManager();
                j.$httpBackend.flush();

                expect(j.scope.closeDialog).toHaveBeenCalled();
            });

            it('should replace empty string usernameField or usernameValue with undefined', () => {
                j.scope.usernameObject.usernameField = '';
                j.scope.usernameObject.usernameValue = '';
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/encryption/key-managers/pid1', {
                    name: 'a',
                    address: '1.2.3.4',
                    usernameField: undefined,
                    usernameValue: undefined
                }).respond(200, 'value1');

                j.scope.saveEditKeyManager();
                j.$httpBackend.flush();

                expect(j.scope.closeDialog).toHaveBeenCalled();
            });

            it('should work upon error', () => {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/encryption/key-managers/pid1', {
                    name: 'a',
                    address: '1.2.3.4',
                    usernameField: 'Country',
                    usernameValue: 'Tibet'
                }).respond(400, j.errorResponse);

                j.scope.saveEditKeyManager();
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            afterEach(() => {
                expect(j.pidgeon.clear).toHaveBeenCalled();
                expect(j.nocUI.block).toHaveBeenCalled();
                expect(j.nocUI.unblock).toHaveBeenCalled();
            });
        });
    });
});
