describe("Delegate CA Controller", function () {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    var constants, nocEncryptionResourcesService, nocEncryptionResourcesServiceSpy;

    beforeEach(inject(function (Constants, _nocEncryptionResourcesService_) {
        constants = Constants;
        nocEncryptionResourcesService = _nocEncryptionResourcesService_;
        nocEncryptionResourcesServiceSpy = jest.spyOn(nocEncryptionResourcesService, "getOccmCertificate").mockImplementation(
            magnifispec.mockPromise({_pathPrefix:"/vsa/",publicId: 'initialized_value1', _expirationState: 'mocked_state'}));
        j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/encryption/csr").respond(200, {csr: "initialized_value2"});
        magnifispec.mockTabsController("DelegateCaController");
        j.$httpBackend.flush();
        magnifispec.processPromise();

        jest.spyOn(document, "getElementById").mockReturnValue({select: jest.fn()});
    }));

    describe("getCertificate", function () {
        beforeEach(function () {
            j.scope.certificateLoaded = undefined;
        });
        it("should work upon success with certificate", function () {
            nocEncryptionResourcesServiceSpy.mockImplementation(
                magnifispec.mockPromise({_pathPrefix:"/vsa/",publicId: "certificate_object1", _expirationState: 'mocked_state'}));

            j.scope.getCertificate();
            magnifispec.processPromise();

            expect(j.scope.certificate).toEqual({_pathPrefix:"/vsa/",publicId: 'certificate_object1', _expirationState: 'mocked_state'});
            expect(j.scope.certificateLoaded).toBeTruthy();
        });

        it("should work upon success without certificate", function () {
            nocEncryptionResourcesServiceSpy.mockImplementation(
                magnifispec.mockPromise(null, true)
            );

            j.scope.getCertificate();
            magnifispec.processPromise();

            expect(j.scope.certificate).toBeNull();
            expect(j.scope.certificateLoaded).toBeTruthy();
        });


        it("should work upon error", function () {
            nocEncryptionResourcesServiceSpy.mockImplementation(
                magnifispec.mockPromise()
            );

            j.scope.getCertificate();
            magnifispec.processPromise();

            expect(j.scope.certificate).toEqual({_pathPrefix:"/vsa/",publicId: 'initialized_value1', _expirationState: 'mocked_state'});
            expect(j.scope.certificateLoaded).toBeFalsy();
        });

        afterEach(function () {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe("getCsr", function () {
        it("should work upon success with actual csr", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/encryption/csr").respond(200, {csr: "csr_request1"});

            j.scope.getCsr();
            j.$httpBackend.flush();

            expect(j.scope.csr).toEqual("csr_request1");
            expect(j.scope.occmCertificate).toBeNull();
        });

        it("should work upon success with no csr", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/encryption/csr").respond(200, null);

            j.scope.getCsr();
            j.$httpBackend.flush();

            expect(j.scope.csr).toBeNull();
            expect(j.scope.occmCertificate).toBeNull();
        });

        it("should work upon error", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/encryption/csr").respond(400, j.errorResponse);

            j.scope.getCsr();
            j.$httpBackend.flush();

            expect(j.scope.csr).toBeNull();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(function () {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe("initialize", function () {
        it("should call getCertificate", function () {
            jest.spyOn(j.scope, "getCertificate").mockImplementation(() => {});
            j.scope.initialize();
            expect(j.scope.getCertificate).toHaveBeenCalled();
        });
    });

    describe("view certificate", function () {
        it("should open the proper dialog", function () {
            j.scope.certificate = {a: 'b', certificate: 'value333'};
            magnifispec.fakeConfirmNew(true);

            j.scope.viewCertificate();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Your Cloud Manager Certificate', 'assets/templates/tabs/encryption/show-certificate.html', {
                certificate: 'value333',
                width: constants.dialogWidthForDisplayingCertificate,
                callback: j.scope.selectCertificateText
            });
        });
    });

    describe("generateCsr", function () {
        it("should work upon success", function () {

            j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/encryption/csr').respond(200, {csr: "csr_request2"});

            j.scope.generateCsr();
            j.$httpBackend.flush();

            expect(j.scope.csr).toEqual("csr_request2");
            j.$timeout.flush();

            expect(document.getElementById).toHaveBeenCalledWith("csrTextArea");
            expect(document.getElementById("csrTextArea").select).toHaveBeenCalled();
            expect(j.scope.occmCertificate).toEqual("");
        });

        it("should work upon error", function () {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/encryption/csr').respond(400, j.errorResponse);
            jest.spyOn(j.scope, "getCsr").mockImplementation(() => {});

            j.scope.generateCsr();
            j.$httpBackend.flush();

            expect(j.scope.csr).toEqual('initialized_value2');
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.scope.getCsr).toHaveBeenCalled();
        });

        afterEach(function () {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe("regenerateCsr", function () {
        it("should call generateCsr upon ok", function () {
            jest.spyOn(j.scope, "generateCsr").mockImplementation(() => {});
            magnifispec.fakeConfirmNew(true);
            j.scope.regenerateCsr();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Regenerate CSR?', 'assets/templates/tabs/encryption/delete-current-csr-dialog.html');
            expect(j.scope.generateCsr).toHaveBeenCalled();
        });

        it("should not call generateCsr upon cancel", function () {
            jest.spyOn(j.scope, "generateCsr").mockImplementation(() => {});
            magnifispec.fakeConfirmNew(false);
            j.scope.regenerateCsr();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Regenerate CSR?', 'assets/templates/tabs/encryption/delete-current-csr-dialog.html');
            expect(j.scope.generateCsr).not.toHaveBeenCalled();
        });
    });

    describe("installOccmCertificate", function () {
        it("should work upon success", function () {
            jest.spyOn(nocEncryptionResourcesService, "formatCertificate").mockImplementation(() => {});
            jest.spyOn(j.nocDialog, "dialog").mockImplementation(() => {});
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/encryption/certificate').respond(200, {name: 'certificate1', endDate:'someDate'});

            j.scope.installOccmCertificate();
            j.$httpBackend.flush();

            expect(nocEncryptionResourcesService.formatCertificate).toHaveBeenCalledWith({name: 'certificate1', endDate:'someDate'});
            expect(j.scope.certificate).toEqual({name: 'certificate1', endDate:'someDate'});
            expect(j.scope.csr).toBeNull();
            expect(j.scope.occmCertificate).toBeNull();
            expect(j.scope.renew).toEqual(false);
            expect(j.nocDialog.dialog).toHaveBeenCalledWith("Certificate installed successfully", "assets/templates/tabs/encryption/occm-certificate-installed.html");
        });

        it("should work upon error", function () {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/encryption/certificate').respond(400, j.errorResponse);

            j.scope.installOccmCertificate();
            j.$httpBackend.flush();

            expect(j.scope.certificate).toEqual({_pathPrefix:"/vsa/",publicId: 'initialized_value1', _expirationState: 'mocked_state'});
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(function () {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });
});