describe("HTTPS Certificate controller spec", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(function () {
        j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/key-store/signed-certificate").respond(200);
        j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/key-store/self-certificate").respond(200);
        magnifispec.mockStandaloneController("HttpsCertificateController", {});
        j.$httpBackend.flush();
    });

    describe("switchToCertificateInstalledStep", function () {
        it("should set the certificate to the provided certificate after formatting, and set the step to 3", function(){
            jest.spyOn(j.nocEncryptionResourcesService, "formatCertificate").mockImplementation(() => {});
            j.scope.switchToCertificateInstalledStep("certificate");

            expect(j.nocEncryptionResourcesService.formatCertificate).toHaveBeenCalledWith("certificate");
            expect(j.scope.step).toEqual(3);
            expect(j.scope.certificate).toEqual("certificate");
        });
    });

    describe("isCsrFormValid", function(){
        it("should return form validity flag", function(){
            j.scope.csrForm = {$valid: true};
            expect(j.scope.isCsrFormValid()).toEqual(true);
        });
    });
    
    describe("csrCommonNameChanged", function(){
        beforeEach(function(){
           jest.spyOn(j.nocDialog, "alert").mockImplementation(() => {});
        });
        it("should show an alert if called the first time and step is 2", function(){
            j.scope.changeCsrCommonNameWarningGiven = false;
            j.scope.step = 2;
            j.scope.csrCommonNameChanged();
            expect(j.nocDialog.alert).toHaveBeenCalled();
        });

        it("should not show an alert if not called the first time", function(){
            j.scope.changeCsrCommonNameWarningGiven = true;
            j.scope.step = 2;
            j.scope.csrCommonNameChanged();
            expect(j.nocDialog.alert).not.toHaveBeenCalled();
        });

        it("should not show an alert if not step 2", function(){
            j.scope.changeCsrCommonNameWarningGiven = false;
            j.scope.step = 3;
            j.scope.csrCommonNameChanged();
            expect(j.nocDialog.alert).not.toHaveBeenCalled();
        });
    });

    describe("changeCertificate", function(){
       it("should set step to 1 and changeCsrCommonNameWarningGiven to false", function(){
           j.scope.step = 3;
           j.scope.changeCsrCommonNameWarningGiven = true;

           j.scope.changeCertificate();
           expect(j.scope.step).toEqual(1);
           expect(j.scope.changeCsrCommonNameWarningGiven).toEqual(false);
       });
    });

    describe("viewCsr", function(){
        it("should show a confirm", function(){
            jest.spyOn(j.nocDialog, "confirm").mockImplementation(() => {});
            j.scope.viewCsr("text");
            expect(j.nocDialog.confirm).toHaveBeenCalled();
        });
    });

    describe("viewCertificate", function(){
        it("should show a confirm", function(){
            jest.spyOn(j.nocDialog, "confirm").mockImplementation(() => {});
            j.scope.certificate = {certificate: "text"};
            j.scope.viewCertificate();
            expect(j.nocDialog.confirm).toHaveBeenCalled();
        });
    });

    describe('uploadAndInstallCertificateFile', function(){
        it('should call external-certificate API when certificate is external', function () {
            j.scope.certificateFile = {name: 'rootCA.txt'};
            j.scope.privateKeyFile = {name: 'rootCAkey.txt'};
            j.scope.externalCertificate = true;
            jest.spyOn(j.scope, 'switchToCertificateInstalledStep').mockImplementation(() => {});
            j.scope.upload = jest.fn().mockReturnValue(j.$q.when());
            j.scope.uploadAndInstallCertificateFile();
            magnifispec.processPromise(); // initiate .then()
            expect(j.scope.switchToCertificateInstalledStep).toHaveBeenCalled();
            expect(j.scope.upload).toHaveBeenCalledWith('uploading certificate file', [
                    {
                        fieldName: 'certificateFilename',
                        file: j.scope.certificateFile,
                        uploadFormField: 'certificate'
                    },
                    {
                        fieldName: 'privateKeyFilename',
                        file: j.scope.privateKeyFile,
                        uploadFormField: 'privateKey'
                    }],
                    '/occm/key-store/external-certificate-file'

            );
        });

        it('should call signed-certificate API when certificate is generated', function () {
            j.scope.certificateFile = {name: 'rootCA.txt'};
            j.scope.externalCertificate = false;
            jest.spyOn(j.scope, 'switchToCertificateInstalledStep').mockImplementation(() => {});
            j.scope.upload = jest.fn().mockReturnValue(j.$q.when());
            j.scope.uploadAndInstallCertificateFile();
            magnifispec.processPromise(); // initiate .then()
            expect(j.scope.switchToCertificateInstalledStep).toHaveBeenCalled();
            expect(j.scope.upload).toHaveBeenCalledWith('uploading certificate file', [
                    {
                        fieldName: 'filename',
                        file: j.scope.certificateFile,
                        uploadFormField: 'certificate'
                    }],
                    '/occm/key-store/signed-certificate-file'

            );
        });

    });

    describe("generateCsr", function(){
        it("should formValidate and call generate CSR API", function(){
            jest.spyOn(j.scope, "viewCsr").mockImplementation(() => {});
            j.scope.certificate = "cert";
            expect(j.scope.alreadyGenerated).toEqual(false);
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/occm/key-store", {commonName: "commonName"}).respond({csr: "text"});
            j.scope.generateCsr("commonName ");
            j.$httpBackend.flush();
            expect(j.nocUI.formValidate).toHaveBeenCalled();
            expect(j.scope.viewCsr).toHaveBeenCalledWith("text");
            expect(j.scope.step).toEqual(1);
            expect(j.scope.changeCsrCommonNameWarningGiven).toEqual(false);
            expect(j.scope.alreadyGenerated).toEqual(true);
            expect(j.scope.certificate).not.toBeDefined();
        });

        it("should trim the commonName", function () {
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/occm/key-store", {commonName: "commonName"}).respond({csr: "text"});
            j.$httpBackend.expectGET("assets/templates/tabs/encryption/show-certificate.html").respond("<html></html>");
            j.$httpBackend.expectGET("assets/templates/common/messages-container.html").respond("<html></html>");
            j.scope.generateCsr("  commonName ");
            j.$httpBackend.flush();
        });
    });

    describe("regenerateCsr", function(){
        it("should call generateCsr after dialog", function(){
            jest.spyOn(j.scope, "generateCsr").mockImplementation(() => {});
            j.scope.step = 2;
            expect(j.scope.step).toEqual(2);
            magnifispec.fakeConfirmNew(true);
            j.scope.regenerateCsr("text");
            j.$timeout.flush();
            expect(j.nocUI.formValidate).toHaveBeenCalled();
            expect(j.nocDialog.confirm).toHaveBeenCalled();
            expect(j.scope.step).toEqual(1);
            expect(j.scope.generateCsr).toHaveBeenCalledWith("text");
        });
    });

    describe("initialize", function(){
        it("should call switchToCertificateInstalledStep if there is a signed certificate and set alreadyGenerated to true", function(){
            var cert = {subjectName: "certificate CN=bla"};
            jest.spyOn(j.scope, "switchToCertificateInstalledStep").mockImplementation(() => {});
            expect(j.scope.alreadyGenerated).toEqual(false);
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/key-store/signed-certificate").respond(cert);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.alreadyGenerated).toEqual(true);
            expect(j.scope.switchToCertificateInstalledStep).toHaveBeenCalledWith(cert);
        });

        describe("with no signed certificate", function(){
            beforeEach(function(){
                j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/key-store/signed-certificate").respond(204);
            });
            it("set step to 1 if there is a self certificate", function(){
                j.scope.initialize();
                j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/key-store/self-certificate").respond({subjectName: "CN=commonName", alternativeNames: ""});
                j.$httpBackend.flush();
                expect(j.scope.step).toEqual(1);
                expect(j.scope.commonName).toEqual("commonName");
            });

            it("should set to step 1 if there is no self certificate", function(){
                j.scope.initialize();
                j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/key-store/self-certificate").respond(204);
                j.$httpBackend.flush();
                expect(j.scope.step).toEqual(0);
            });

            it("should append the alternative names to the commonName", function () {
                j.scope.initialize();
                j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/key-store/self-certificate").respond({subjectName: "CN=commonName", alternativeNames: "b;c"});
                j.$httpBackend.flush();
                expect(j.scope.commonName).toEqual("commonName;b;c");
            });
        });
    });

    describe("extractCommonNameFromCertificate", function(){

        it("should call extract a complicated common name from a certificate's subject", function(){
            var certificate = {subjectName: 'CN=www.1-complicated*common_name.com'};
            expect(j.scope.extractCommonNameFromCertificate(certificate)).toEqual('www.1-complicated*common_name.com');
        });

        it("should call extract a common name from a full certificate's subject name", function(){
            var certificate = {subjectName: 'EMAILADDRESS=a@a.a,  CN=www.commonName.com,  OU=a,  O=a,  L=a,  ST=a,  C=BS'};
            expect(j.scope.extractCommonNameFromCertificate(certificate)).toEqual('www.commonName.com');
        });

        it("should call extract a common name from a full certificate's subject name plus its alternative names", function(){
            var certificate = {subjectName: 'EMAILADDRESS=a@a.a,  CN=www.commonName.com,  OU=a,  O=a,  L=a,  ST=a,  C=BS', alternativeNames: 'www.alternativeName1.net;alternativeName2.org'};
            expect(j.scope.extractCommonNameFromCertificate(certificate)).toEqual('www.commonName.com;www.alternativeName1.net;alternativeName2.org');
        });

    });


});