describe('ChangeOntapImageController Spec', function () {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var workingEnvironment = {_pathPrefix:"/vsa/",publicId: "1", workingEnvironmentType: "VSA"};
    beforeEach(function () {
        magnifispec.mockTabsController('ChangeOntapImageController', {}, {
            selectedWorkingEnvironment: workingEnvironment,
            changeOntapImageForm: {},
            showMonitoring: jest.fn()
        });
    });

    describe('handleLocalImageChange', () => {
        it('should set selected local image to be the one with the given name', () => {
            j.scope.localImages = [{name: 'not-test'}, {name: 'test'}];
            j.scope.handleLocalImageChange('test');
            expect(j.scope.selectedImage).toEqual({name: 'test'});
        });
    });

    describe("isValid", function(){
        it("should return true if the form is $valid", function(){
            j.scope.changeOntapImageForm.$valid = true;
            expect(j.scope.isValid()).toEqual(true);
        });

        it("should return false if the form is not $valid", function(){
            j.scope.changeOntapImageForm.$valid = false;
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe("changeOntapImageConfirm", function(){
        var isValidSpy, fakePayload;
        beforeEach(function () {
            isValidSpy = jest.spyOn(j.scope, "isValid").mockReturnValue(true);
            fakePayload = {updateType: "a", updateParameter: "b", workingEnvironment: workingEnvironment, width: 600};
            jest.spyOn(j.scope, "constructPayload").mockReturnValue(fakePayload);
            jest.spyOn(j.scope, "changeOntapImage").mockImplementation(() => {});
        });
        it("should call handleForm validity if not valid()", function(){
            isValidSpy.mockReturnValue(false);
            j.scope.changeOntapImageConfirm();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalledWith(j.scope.changeOntapImageForm, j.scope);
        });

        it("should fail in external mode if the given url is not .tgz file", function () {
            j.scope.values.externalUrl = "http://www.lala.com/filename.exe";
            j.scope.values.mode = "external";
            j.scope.changeOntapImageConfirm(workingEnvironment, undefined);
            expect(j.pidgeon.error).toHaveBeenCalledWith("Please enter valid URL of file with .tgz extension", null, "url");
        });

        it("should not fail in local mode about extention", function () {
            j.scope.values.mode = "local";
            j.scope.changeOntapImageConfirm(workingEnvironment, undefined);
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        it("should not fail with the right extention in exteranl mode", function () {
            j.scope.values.externalUrl = "http://www.lala.com/filename.tgz";
            j.scope.values.mode = "external";
            j.scope.changeOntapImageConfirm(workingEnvironment, undefined);
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        it("should call confirm if valid() and pass in a constructed payload", function(){
            magnifispec.fakeConfirmNew(true);
            j.scope.changeOntapImageConfirm(workingEnvironment);

            expect(j.nocDialog.confirm).toHaveBeenCalledWith("Confirm the Cloud Volumes ONTAP image change", "assets/templates/dialogs/change-ontap-version-confirm-content.html", fakePayload);
            expect(j.scope.changeOntapImage).toHaveBeenCalledWith(workingEnvironment, fakePayload);
        });
    });

    describe("constructPayload", function(){
       it("should return a correct EXTERNAL_FILE payload mode is external", function(){
           expect(j.scope.constructPayload(null, "http://localhost", "external", {})).toEqual({updateType: "EXTERNAL_FILE", updateParameter: "http://localhost"});
       });

        it("should return a correct LOCAL_IMAGE payload when mode is local", function(){
            expect(j.scope.constructPayload({name: "image1"}, null, "local", {})).toEqual({updateType: "LOCAL_IMAGE", updateParameter: "image1"});
        });
    });

    describe("changeOntapImage", function(){
        var fakePayload = {updateType: "a", updateParameter: "b"};

        beforeEach(function(){
            j.$httpBackend.expectGET(j.nocBaseUrl + "/vsa/working-environments/1/ontap-available-images").respond([]);
        });

        it("should pidgeon an error in case of HTTP failure", function(){
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/vsa/working-environments/1/update-image", fakePayload).respond(500, j.errorResponse);
            j.scope.changeOntapImage(workingEnvironment, fakePayload);
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        it("should go back to monitoring and update working environment status in case of success", function(){
            jest.spyOn(j.workingEnvironmentResource, "updateWorkingEnvironmentStatus").mockImplementation(() => {});
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/vsa/working-environments/1/update-image", fakePayload).respond(204);
            j.scope.changeOntapImage(workingEnvironment, fakePayload);
            j.$httpBackend.flush();
            expect(j.scope.showMonitoring).toHaveBeenCalled();
            expect(j.workingEnvironmentResource.updateWorkingEnvironmentStatus).toHaveBeenCalledWith(workingEnvironment, "updating");
        });
    });

    describe("changeMode", function () {
        it("should change the mode for 'local'", function () {
            j.scope.changeMode("local");
            expect(j.scope.values.mode).toEqual("local");
        });
        it("should change the mode and clear selectedImageName for 'external'", function () {
            j.scope.changeMode("external");
            expect(j.scope.values.mode).toEqual("external");
            expect(j.scope.selectedImageName).toBeNull();
        });
    });

    describe("extensionMatch", function () {
        it("should be legal filename with '.tgz' extension", function () {
            expect(j.scope.extensionMatch.test("http://www.example.com/a.tgz")).toBeTruthy();
            expect(j.scope.extensionMatch.test("http://www.example.com/6.tgz")).toBeTruthy();
            expect(j.scope.extensionMatch.test("http://www.example.com/a-b-c.tgz")).toBeTruthy();
            expect(j.scope.extensionMatch.test("http://www.example.com/filename.tgz")).toBeTruthy();

            expect(j.scope.extensionMatch.test("http://www.example.com/a?.tgz")).toBeFalsy();
            expect(j.scope.extensionMatch.test("http://www.example.com/filename.exe")).toBeFalsy();
            expect(j.scope.extensionMatch.test("http://www.example.com/filenametgz")).toBeFalsy();
            expect(j.scope.extensionMatch.test("http://www.example.com/.tgz")).toBeFalsy();
        });
        it("must have a url and slash before the filename", function () {
            expect(j.scope.extensionMatch.test("http://1/filename.tgz")).toBeTruthy();
            expect(j.scope.extensionMatch.test("http://a/filename.tgz")).toBeTruthy();
            expect(j.scope.extensionMatch.test("https://a/filename.tgz")).toBeTruthy();

            expect(j.scope.extensionMatch.test("http://filename.tgz")).toBeFalsy();
            expect(j.scope.extensionMatch.test("http://?/filename.tgz")).toBeFalsy();
            expect(j.scope.extensionMatch.test("filename.tgz")).toBeFalsy();
            expect(j.scope.extensionMatch.test("http://./filename.tgz")).toBeFalsy();

        });
    });
});


