describe('ChangeTierController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    const workingEnvironment = {
        _pathPrefix: '/vsa/',
        _cloudProviderName: 'AWS',
        isHA: false,
        publicId: '1',
        workingEnvironmentType: 'VSA',
        status: {status: 'ON'},
        _providerVolumeTypes: {
            'gp2': {name: 'gp2', diskName: 'gp2', display: 'high performance'},
            'gp2+S3': {name: 'gp2+S3', diskName: 'gp2+S3', display: 'high performance and S3'},
            'st1': {name: 'st1', diskName: 'st1', display: 'low performance'}
        },
        capacityFeatures: {
            supportedCapacityTiers: {
                supportedCapacityTiersPerVolumeType: [
                    {availableTieringPolicies: [], supportedCapacityTiers: [], volumeType: 'standard'},
                    {
                        availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                        supportedCapacityTiers: ['S3'],
                        volumeType: 'io1'
                    },
                    {
                        availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                        supportedCapacityTiers: ['S3'],
                        volumeType: 'gp2'
                    },
                    {
                        availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                        supportedCapacityTiers: ['S3'],
                        volumeType: 'st1'
                    },
                    {availableTieringPolicies: [], supportedCapacityTiers: [], volumeType: 'sc1'}
                ]
            }
        }
    };
    const volumes = [
        {name: 'vol1', svmName: 'svm1', providerVolumeType: 'gp2', _capacityTier: 'gp2'},
        {name: 'vol2', svmName: 'svm1', providerVolumeType: 'st1', _capacityTier: 'st1'}
    ];
    let getAggregatesFromResourceSpy;

    beforeEach(inject((aggregatesResource) => {
        magnifispec.mockTabsController('ChangeTierController', {}, {
            selectedWorkingEnvironment: workingEnvironment,
            updateSelectedWorkingEnvironment: j.$q.when
        });
        j.$stateParams.selectedVolumeName = 'vol1';
        j.$stateParams.selectedVolumeSVM = 'svm1';
        magnifispec.processPromise();
        getAggregatesFromResourceSpy = jest.spyOn(aggregatesResource, 'getAggregates');
    }));

    describe('initializeController', () => {
        it('should wait for the volumes before initializing', () => {
            getAggregatesFromResourceSpy.mockReturnValue(j.$q.when());
            expect(j.scope.loaded).toEqual(false);
            expect(j.scope.selectedVolume).not.toBeDefined();

            j.scope.volumes = volumes;
            j.$rootScope.$apply();

            expect(j.scope.selectedVolume).toEqual(j.scope.volumes[0]);
            expect(j.scope.loaded).toBeTruthy();
        });

        it('should create possibleTiers correctly', () => {
            getAggregatesFromResourceSpy.mockReturnValue(j.$q.when());
            j.scope.volumes = volumes;
            j.$rootScope.$apply();

            expect(j.scope.possibleTiers).toEqual({
                'gp2': {name: 'gp2', diskName: 'gp2', display: 'high performance', tieringEnabled: false},
                'gp2+S3': {name: 'gp2+S3', diskName: 'gp2+S3', display: 'high performance and S3'},
                'st1': {name: 'st1', diskName: 'st1', display: 'low performance'}
            });
        });

        describe('should preselect the current tier', () => {
            beforeEach(() => {
                jest.spyOn(j.scope, 'tierSelected').mockImplementation(() => {});
                getAggregatesFromResourceSpy.mockReturnValue(j.$q.when());
            });

            it('case 1: not composite volume', () => {
                j.scope.volumes = volumes;
                j.$rootScope.$apply();

                expect(j.scope.tierSelected).toHaveBeenCalledWith({
                    name: 'gp2',
                    diskName: 'gp2',
                    display: 'high performance',
                    tieringEnabled: false
                });
            });

            it('case 2: composite', () => {
                j.scope.volumes = [
                    {name: 'vol1', svmName: 'svm1', providerVolumeType: 'gp2', _capacityTier: 'gp2+S3', capacityTier: 'S3'},
                    {name: 'vol2', svmName: 'svm1', providerVolumeType: 'st1', _capacityTier: 'st1'}
                ];
                j.scope.selectedVolume = volumes[1];
                j.$rootScope.$apply();

                expect(j.scope.tierSelected).toHaveBeenCalledWith({
                    name: 'gp2',
                    diskName: 'gp2',
                    display: 'high performance',
                    tieringEnabled: true
                });
            });
        });

        it('should initialize wizardController object (for tier selection component', () => {
            getAggregatesFromResourceSpy.mockReturnValue(j.$q.when());
            j.scope.volumes = volumes;
            j.$rootScope.$apply();

            expect(j.scope.wizardController).toBeDefined();
        });
    });

    describe('disabledTooltipForTier', () => {
        it('should return tier disable message if exists', () => {
            expect(j.scope.disabledTooltipForTier({disabledMessage: 'Denied!'})).toEqual('Denied!');
        });

        it('should return correct message for thick volume + composite tier', () => {
            j.scope.selectedVolume = {thinProvisioning: false};

            expect(j.scope.disabledTooltipForTier({name: 'dt1+Blob'})).toEqual('Blob tiering is not available with "No Storage Efficiency" usage profile.');
        });
    });

    describe('tierSelected', () => {
        beforeEach(() => {
            j.scope.wizardController = {tieringEnabled: true};
            j.scope.changeDiskQuoteRequest = {};
        });

        it('should set the selected tier and remove the checkbox if tier does not support S3', () => {
            j.scope.tierSelected({name: 'dt1', diskName: 'dt2'});

            expect(j.scope.wizardController.selectedTier).toEqual({name: 'dt1', diskName: 'dt2'});
            expect(j.scope.wizardController.tieringEnabled).toEqual(false);
        });

        it('should set the selected tier and leave the checkbox as is, if tier supports S3', () => {
            j.scope.tierSelected({name: 'dt1', diskName: 'dt2', supportedCapacityTiers: ['S3']});

            expect(j.scope.wizardController.selectedTier).toEqual({name: 'dt1', diskName: 'dt2', supportedCapacityTiers: ['S3']});
            expect(j.scope.wizardController.tieringEnabled).toEqual(true);
        });
    });

    describe('startChangeTierProcess', () => {
        let askForQuoteSpy, getUserApprovalForNewVolumeIfNeededSpy;

        beforeEach(inject((volumesResource) => {
            getAggregatesFromResourceSpy.mockReturnValue(j.$q.when());
            j.scope.possibleTiers = {st1: {name: 'st1', display: 'low performance'}};
            j.scope.selectedWorkingEnvironment = {
                _isAWS: true,
                _pathPrefix: '/vsa/',
                _cloudProviderName: 'AWS',
                isHA: false,
                _providerVolumeTypes: {
                    'gp2': {name: 'gp2', diskName: 'gp2', display: 'high performance'},
                    'gp2+S3': {name: 'gp2+S3', diskName: 'gp2+S3', display: 'high performance and S3'},
                    'st1': {name: 'st1', diskName: 'st1', display: 'low performance'}
                },
                capacityFeatures: {
                    supportedCapacityTiers: {
                        supportedCapacityTiersPerVolumeType: [
                            {availableTieringPolicies: [], supportedCapacityTiers: [], volumeType: 'standard'},
                            {
                                availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                                supportedCapacityTiers: ['S3'],
                                volumeType: 'io1'
                            },
                            {
                                availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                                supportedCapacityTiers: ['S3'],
                                volumeType: 'gp2'
                            },
                            {
                                availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                                supportedCapacityTiers: ['S3'],
                                volumeType: 'st1'
                            },
                            {availableTieringPolicies: [], supportedCapacityTiers: [], volumeType: 'sc1'}
                        ]
                    }
                }

            };
            j.scope.selectedVolume = {name: 'vol1', thinProvisioning: true};
            jest.spyOn(j.scope, 'changeTier').mockImplementation(() => {});
            askForQuoteSpy = jest.spyOn(volumesResource, 'askForQuote').mockReturnValue(j.$q.when('quoteResponse'));
            getUserApprovalForNewVolumeIfNeededSpy = jest.spyOn(volumesResource, 'getUserApprovalForNewVolumeIfNeeded').mockImplementation((...args) => j.$q.when(args[0]));
            j.scope.currentTier = {name: 'gp2', supportedCapacityTiers: ['S3']};
            j.scope.wizardController = {
                selectedTier: {name: 'st1', supportedCapacityTiers: ['S3']},
                tieringEnabled: false,
                forceCompositeVersion: true,
                compositeSupported: true
            };
            j.scope.changeDiskQuoteRequest = {name: 'vol1'};
        }));

        it('should update changeDiskQuoteRequest correctly with new tier info', () => {
            j.scope.startChangeTierProcess();

            expect(j.scope.changeDiskQuoteRequest.providerVolumeType).toEqual('st1');
            expect(j.scope.changeDiskQuoteRequest.capacityTier).toEqual('S3');
        });

        it('should update changeDiskQuoteRequest correctly with new composite tier info', () => {
            j.scope.wizardController.tieringEnabled = true;

            j.scope.startChangeTierProcess();

            expect(j.scope.changeDiskQuoteRequest.providerVolumeType).toEqual('st1');
            expect(j.scope.changeDiskQuoteRequest.capacityTier).toEqual('S3');
        });

        it('should ask for quote, then ask for approval if needed, then change tier', () => {
            j.scope.startChangeTierProcess();
            magnifispec.processPromise();

            expect(askForQuoteSpy).toHaveBeenCalledWith(jasmine.any(Object), '/vsa/', {
                name: 'vol1',
                providerVolumeType: 'st1',
                capacityTier: 'S3',
                tieringPolicy: undefined
            }, 'changing volume tier');
            expect(getUserApprovalForNewVolumeIfNeededSpy).toHaveBeenCalledWith('quoteResponse', 'AWS', 'low performance', false, 'vol1');
            expect(j.scope.changeTier).toHaveBeenCalledWith('quoteResponse');
        });
    });

    describe('changeTier', () => {
        it('should call the right API', () => {
            j.scope.boxGoBack = jest.fn();
            j.scope.selectedVolume = {name: 'vol1', svmName: 'svm1'};
            j.scope.changeDiskQuoteRequest = {
                providerVolumeType: 'st1',
                capacityTier: 'S3'
            };
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/volumes/1/svm1/vol1/change-tier', {
                aggregateName: 'aggr3',
                numOfDisks: 2,
                newAggregate: false,
                newDiskTypeName: 'st1',
                newCapacityTier: 'S3'
            }).respond(200);

            j.scope.changeTier({
                aggregateName: 'aggr3',
                numOfDisks: 2,
                newAggregate: false,
                autoVsaCapacityManagement: true
            });
            j.$httpBackend.flush();

            expect(j.scope.boxGoBack).toHaveBeenCalled();
            expect(j.pidgeon.inProgress).toHaveBeenCalledWith('Changing tier started.', true);
        });
    });
});
