describe('CloneVolumeController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let workingEnvironment;
    const volume = {name: 'vol1', svmName: 'svm1'};
    const volumes = [volume];
    let getSnapshotsSpy;

    beforeEach(inject((_volumesResource_) => {
        workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1, workingEnvironmentType: 'VSA'};
        getSnapshotsSpy = jest.spyOn(_volumesResource_, 'getSnapshots').mockReturnValue(j.$q.when(['snap1', 'snap2']));
        magnifispec.mockTabsController('CloneVolumeController', {}, {
            volumes,
            selectedWorkingEnvironment: workingEnvironment,
            showMonitoring: jest.fn()
        });
        j.$stateParams.selectedVolumeName = 'vol1';
        j.$stateParams.selectedVolumeSVM = 'svm1';
        magnifispec.processPromise();
    }));

    it('initializes properly', () => {
        expect(j.scope.cloneVolumeRequest).toEqual({'newVolumeName': 'vol1_clone'});
        expect(j.scope.snapshots).toEqual(['snap1', 'snap2']);
        expect(j.scope.latestSnapShot).toEqual('snap1');
    });

    describe('cloneVolume', () => {
        it('should check validity before sending any request', () => {
            j.scope.cloneVolumeForm = {$valid: false};
            j.nocUI.handleFormValidity = jest.fn();
            j.scope.cloneVolume();

            expect(j.nocUI.handleFormValidity).toHaveBeenCalledWith(j.scope.cloneVolumeForm, j.scope);
        });

        it('should sent the request properly', () => {
            j.$httpBackend.expectPOST(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix(workingEnvironment.workingEnvironmentType, 'AWS') + 'volumes/' + workingEnvironment.publicId + '/' + volume.svmName + '/' + volume.name + '/clone', {'newVolumeName': 'vol1_clone'}).respond(200);
            j.scope.cloneVolumeForm = {$valid: true};
            j.scope.cloneVolume();
            j.$httpBackend.flush();
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.scope.showMonitoring).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should sent the request properly for ha', () => {
            workingEnvironment.isHA = true;
            workingEnvironment._pathPrefix = '/aws/ha/';
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/aws/ha/' + 'volumes/' + workingEnvironment.publicId + '/' + volume.svmName + '/' + volume.name + '/clone', {'newVolumeName': 'vol1_clone'}).respond(200);
            j.scope.cloneVolumeForm = {$valid: true};
            j.scope.cloneVolume();
            j.$httpBackend.flush();
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.scope.showMonitoring).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should fail gracefully', () => {
            j.$httpBackend.expectPOST('/occm/api' + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix(workingEnvironment.workingEnvironmentType, 'AWS') + 'volumes/' + workingEnvironment.publicId + '/' + volume.svmName + '/' + volume.name + '/clone', {'newVolumeName': 'vol1_clone'}).respond(400, j.errorResponse);
            j.scope.cloneVolumeForm = {$valid: true};
            j.scope.cloneVolume();
            j.$httpBackend.flush();
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });
    });

    describe('getSnapshots', () => {
        it('should call the resource properly', () => {
            getSnapshotsSpy.mockReturnValue(j.$q.when(['snap1', 'snap2']));

            j.scope.getSnapshots();
            magnifispec.processPromise();

            expect(getSnapshotsSpy).toHaveBeenCalledWith(jasmine.any(Object), '/vsa/', 1, 'svm1', 'vol1');
            expect(j.scope.snapshots).toEqual(['snap1', 'snap2']);
            expect(j.scope.latestSnapShot).toEqual('snap1');
            expect(j.scope.restore.selectedSnapshot).toEqual('');
        });

        it('Should auto select snapshot if there is only one', () => {
            getSnapshotsSpy.mockReturnValue(j.$q.when(['snap1']));

            j.scope.getSnapshots();
            magnifispec.processPromise();

            expect(j.scope.restore.selectedSnapshot).toEqual('snap1');
        });
    });
});
