describe('LicenseController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let getAggregatesFromResourceSpy;

    beforeEach(inject((aggregatesResource) => {
        magnifispec.mockTabsController('LicenseController', {}, {vsaResponse: {}});

        getAggregatesFromResourceSpy = jest.spyOn(aggregatesResource, 'getAggregates');
    }));

    describe('getAggregates', () => {
        const aggregatesResponse = [{
            providerVolumes: [
                {size: {size: 10, unit: 'TB'}},
                {size: {size: 10, unit: 'TB'}}
            ]
        }, {
            providerVolumes: [
                {size: {size: 10, unit: 'TB'}},
                {size: {size: 10, unit: 'TB'}}
            ]
        }];

        it('should get aggregates correctly for single node', () => {
            getAggregatesFromResourceSpy.mockReturnValue(j.$q.when(aggregatesResponse));

            j.scope.getAggregates();
            magnifispec.processPromise();

            expect(j.scope.licenseAggregates).toEqual(aggregatesResponse);
            expect(j.$filter('normalizedCapacity')(j.scope.attachedProviderCapacity, 2, true)).toEqual('40 TB');
        });

        it('should get aggregates correctly for HA', () => {
            j.scope.selectedWorkingEnvironment = {isHA: true};
            getAggregatesFromResourceSpy.mockReturnValue(j.$q.when(aggregatesResponse));

            j.scope.getAggregates();
            magnifispec.processPromise();

            expect(j.scope.licenseAggregates).toEqual(aggregatesResponse);
            expect(j.$filter('normalizedCapacity')(j.scope.attachedProviderCapacity, 2, true)).toEqual('20 TB');
        });
    });

    describe('PAYGO initialization', () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment = {
                _pathPrefix: '/vsa/',
                publicId: '1',
                workingEnvironmentType: 'VSA',
                _isAWS: true,
                _ontapVersion: 'ONTAP-8.4.1',
                status: {status: 'ON'}
            };
            j.scope.vsaResponse = {
                instanceType: 'm3.2xlarge',
                cluster: {licenseType: {name: 'Cloud Volumes ONTAP Standard'}, nodes: [{name: 'node1'}, {name: 'node2'}]},
                vsaEncryptionResponse: {ontapEncryption: false}
            };

            j.scope.initialize();
        });

        it('should initialized correctly', () => {
            expect(j.scope.licenseAggregates).not.toBeDefined();
            expect(j.scope.registrationDetails).not.toBeDefined();
            expect(j.scope.isVersionPost84).toEqual(true);
        });

        it('should succeed on PAYGO license', () => {
            j.scope.vsaResponse.cluster.licenseType.name = 'Cloud Volumes ONTAP Standard';
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/aggregates?workingEnvironmentId=1').respond(200, {});
            j.$httpBackend.flush();

            expect(j.scope.licenseAggregates).toEqual({});
            expect(j.scope.registrationDetails).not.toBeDefined();
        });
    });

    describe('BYOL initialization for single node', () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment = {
                _pathPrefix: '/vsa/',
                publicId: '1',
                workingEnvironmentType: 'VSA',
                _isAWS: true,
                status: {status: 'ON'}
            };
            j.scope.vsaResponse = {
                instanceType: 'm3.2xlarge',
                cluster: {licenseType: {name: 'Cloud Volumes ONTAP BYOL'}, nodes: [{name: 'node1'}]},
                vsaEncryptionResponse: {ontapEncryption: false}
            };
        });

        it('should succeed on BYOL license', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/aggregates?workingEnvironmentId=1').respond(200, {});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=supportRegistrationInformation').respond(200, {
                supportRegistrationInformation: [
                    {
                        supportRegistrationStatus: 'registered',
                        serialNumber: '90120140000000000125',
                        licenseExpiryDate: 1463518800000
                    }
                ]
            });

            j.scope.initialize();
            j.$httpBackend.flush();

            expect(j.scope.licenseAggregates).toEqual({});
            expect(j.scope.registrationDetails.licenseExpiryDates).toEqual(['May 18, 2016']);
            expect(j.scope.registrationDetails.serialNumbers).toEqual(['90120140000000000125']);
        });

        it('should correctly build the license expiry for Evaluation license', () => {
            j.scope.selectedWorkingEnvironment.ontapClusterProperties = {evaluation: true};
            j.scope.evalExpiration = 'April 7, 2017';
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/aggregates?workingEnvironmentId=1').respond(200, {});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=supportRegistrationInformation').respond(200, {});

            j.scope.initialize();
            j.$httpBackend.flush();

            expect(j.scope.formattedEvalExpiration).toEqual(['April 7, 2017']);
        });

        it('should fail like a boss', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/aggregates?workingEnvironmentId=1').respond(400, j.errorResponse);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=supportRegistrationInformation').respond(400, j.errorResponse);

            j.scope.initialize();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.scope.licenseAggregates).not.toBeDefined();
            expect(j.scope.registrationDetails).toEqual({serialNumbers: [''], licenseExpiryDates: ['']});
        });
    });

    describe('BYOL initialization for HA', () => {
        beforeEach(() => {
            magnifispec.mockTabsController('LicenseController', {}, {
                selectedWorkingEnvironment: {
                    _pathPrefix: '/aws/ha/',
                    publicId: '1',
                    workingEnvironmentType: 'VSA',
                    _isAWS: true,
                    status: {status: 'ON'}
                },
                vsaResponse: {
                    instanceType: 'm3.2xlarge',
                    cluster: {licenseType: {name: 'Cloud Volumes ONTAP BYOL'}, nodes: [{name: 'node1'}, {name: 'node2'}]},
                    vsaEncryptionResponse: {ontapEncryption: false}
                }
            });
            j.scope.vsaResponse.cluster.licenseType.name = 'Cloud Volumes ONTAP BYOL';
        });

        it('should succeed on BYOL license', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/aggregates?workingEnvironmentId=1').respond(200, {});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/working-environments/1?fields=supportRegistrationInformation').respond(200, {
                supportRegistrationInformation: [
                    {
                        supportRegistrationStatus: 'registered',
                        serialNumber: '90120140000000000125',
                        licenseExpiryDate: 1463518800000
                    }, {
                        supportRegistrationStatus: 'registered',
                        serialNumber: '90120140000000000126',
                        licenseExpiryDate: 1463518800000
                    }
                ]
            });
            j.$httpBackend.flush();
            expect(j.scope.licenseAggregates).toEqual({});
            expect(j.scope.registrationDetails.licenseExpiryDates).toEqual(['node1 - May 18, 2016', 'node2 - May 18, 2016']);
            expect(j.scope.registrationDetails.serialNumbers).toEqual(['node1 - 90120140000000000125', 'node2 - 90120140000000000126']);
        });

        it('should correctly build the license expiry for Evaluation license', () => {
            j.scope.evalLicense = true;
            j.scope.evalExpiration = 'April 7, 2017';
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/aggregates?workingEnvironmentId=1').respond(200, {});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/working-environments/1?fields=supportRegistrationInformation').respond(200, {});

            j.$httpBackend.flush();

            expect(j.scope.formattedEvalExpiration).toEqual(['node1 - April 7, 2017', 'node2 - April 7, 2017']);
        });
    });

    describe('showUploadLicenseFileDialog', () => {
        it('open the dialog and update registration details on success', () => {
            magnifispec.mockConfirm(true);
            jest.spyOn(j.scope, 'getRegistrationDetails').mockImplementation(() => {});

            j.scope.showUploadLicenseFileDialog();
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Update License', 'assets/templates/dialogs/upload-license-file.html', jasmine.anything());
            expect(j.scope.getRegistrationDetails).toHaveBeenCalled();
        });

        it('open the dialog without updating registration details on failure', () => {
            magnifispec.mockConfirm(false);
            jest.spyOn(j.scope, 'getRegistrationDetails').mockImplementation(() => {});

            j.scope.showUploadLicenseFileDialog();
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Update License', 'assets/templates/dialogs/upload-license-file.html', jasmine.anything());
            expect(j.scope.getRegistrationDetails).not.toHaveBeenCalled();
        });
    });
});
