describe("SetVSAPasswordController", function () {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var workingEnvironment = {_pathPrefix: "/vsa/", publicId: "instance1"};

    beforeEach(function () {
        magnifispec.mockTabsController("SetVSAPasswordController", {}, {
            selectedWorkingEnvironment: workingEnvironment,
            showMonitoring: jest.fn()
        });
    });

    it("should add changeVSAPassword object to scope on construction", function () {
        expect(j.scope.changeVSAPassword).toEqual({});
    });

    describe("isValid", function () {
        beforeEach(function () {
            j.scope.changeVSAPasswordForm = {};
        });

        it("should return true if the form is valid", function () {
            j.scope.changeVSAPasswordForm.$valid = true;
            expect(j.scope.isValid()).toEqual(true);
        });

        it("should return false if the form is invalid", function () {
            j.scope.changeVSAPasswordForm.$valid = false;
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe("savePassword", function () {
        it("should call handleFormValidity if invalid", function () {
            jest.spyOn(j.scope, "isValid").mockReturnValue(false);

            j.scope.saveVSAPassword({});

            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        it("should clear set-password messages, add a success message, go to monitoring in case of success", function () {
            jest.spyOn(j.scope, "isValid").mockReturnValue(true);
            jest.spyOn(j.nocPoller, "pause").mockImplementation(() => {});
            jest.spyOn(j.nocPoller, "unpause").mockImplementation(() => {});

            j.$httpBackend.expectPUT("/occm/api/vsa/working-environments/instance1/set-password", {password: "newPassword"}).respond(200);

            j.scope.saveVSAPassword({newPassword: "newPassword"});
            expect(j.nocPoller.pause).toHaveBeenCalled();
            j.$httpBackend.flush();

            expect(j.nocPoller.unpause).toHaveBeenCalled();
            expect(j.scope.showMonitoring).toHaveBeenCalled();
            expect(j.pidgeon.success).toHaveBeenCalledWith("Cloud Volumes ONTAP password successfully set.");
        });
    });

    it("should call clearFormValidity on destruction", function () {
        j.scope.$broadcast("$destroy");
        expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
    });
});