describe('UpdateCredentialsController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        magnifispec.mockTabsController('UpdateCredentialsController', {}, {
            selectedWorkingEnvironment: {
                publicId: 'wepid',
                name: 'weName',
                workingEnvironmentType: 'ON_PREM',
                _typeLongName: 'ONTAP'
            }
        });
    });

    describe('initialize', () => {
        it('should add updateCredentials object to scope on construction', () => {
            expect(j.scope.updateCredentials).toEqual({username: 'admin'});
        });

        it('should generate the correct password regexp for ON_PREM', () => {
            // eslint-disable-next-line no-control-regex
            expect(j.scope.passwordRegexp).toEqual(/^.{0,255}$/);
        });

        it('should generate the correct password regexp for VSA', () => {
            j.scope.selectedWorkingEnvironment.workingEnvironmentType = 'VSA';

            j.scope.initialize();

            expect(j.scope.passwordRegexp).toEqual(/^(?=.{0,50}$)(?=[\x21-\x7E]*[0-9])(?=[\x21-\x7E]*[a-zA-Z])((?!admin)[\x21-\x7E]){8,}$/);
        });
    });

    describe('isValid', () => {
        beforeEach(() => {
            j.scope.updateCredentialsForm = {};
        });
        it('should return true if the form is valid', () => {
            j.scope.updateCredentialsForm.$valid = true;
            expect(j.scope.isValid()).toEqual(true);
        });

        it('should return false if the form is invalid', () => {
            j.scope.updateCredentialsForm.$valid = false;
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe('updatePassword', () => {
        it('should call handleFormValidity if invalid', () => {
            jest.spyOn(j.scope, 'isValid').mockReturnValue(false);

            j.scope.saveUpdatedCredentials({});

            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        describe('should call API', () => {
            beforeEach(() => {
                jest.spyOn(j.scope, 'isValid').mockReturnValue(true);
                jest.spyOn(j.nocPoller, 'pause').mockImplementation(() => {});
                jest.spyOn(j.nocPoller, 'unpause').mockImplementation(() => {});
                j.scope.closeDialog = jest.fn();
            });

            describe('case when ON_PREM', () => {
                it('should call pidgeon and exit in case of success', () => {
                    j.$httpBackend.expectPUT('/occm/api/onprem/working-environments/' + j.scope.workingEnvironment.publicId + '/update-credentials',
                        {username: 'newUserName', password: 'newPassword'}).respond(200);

                    j.scope.saveUpdatedCredentials({username: 'newUserName', password: 'newPassword'});

                    expect(j.nocPoller.pause).toHaveBeenCalled();
                    j.$httpBackend.flush();
                    expect(j.scope.$parent.$parent.pushSuccess).toHaveBeenCalledWith('Successfully updated credentials for ONTAP working environment weName.');
                });
            });

            describe('case when VSA', () => {
                beforeEach(() => {
                    j.scope.workingEnvironment.workingEnvironmentType = 'VSA';
                    j.scope.workingEnvironment._typeLongName = 'Cloud Volumes ONTAP';
                    j.scope.workingEnvironment._pathPrefix = '/vsa/';
                });

                it('should call pidgeon and exit in case of success', () => {
                    j.scope.$parent.$parent.pushSuccess = jest.fn();
                    j.$httpBackend.expectPUT('/occm/api/vsa/working-environments/' + j.scope.workingEnvironment.publicId + '/set-password?occmOnly=true',
                        {password: 'newPassword'}).respond(200);

                    j.scope.saveUpdatedCredentials({username: 'newUserName', password: 'newPassword'});
                    expect(j.nocPoller.pause).toHaveBeenCalled();
                    expect(j.nocUI.block).toHaveBeenCalled();

                    j.$httpBackend.flush();

                    expect(j.scope.$parent.$parent.pushSuccess).toHaveBeenCalledWith('Successfully updated credentials for Cloud Volumes ONTAP working environment weName.');
                    expect(j.scope.closeDialog).toHaveBeenCalled();
                });
            });

            afterEach(() => {
                expect(j.nocPoller.unpause).toHaveBeenCalled();
            });
        });
    });

    it('should call clearFormValidity on destruction', () => {
        j.scope.$broadcast('$destroy');

        expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
    });
});
