describe('MonitoringSVMController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    const workingEnvironment = {_allowedOperations: {resources: true}, status: {status: 'on'}};
    let volumesResource;
    let stopVolumesPolling;

    beforeEach(inject((_volumesResource_) => {
        volumesResource = _volumesResource_;
        stopVolumesPolling = jest.fn();
        jest.spyOn(volumesResource, 'startVolumesPolling').mockReturnValue(stopVolumesPolling);
        magnifispec.mockTabsController('MonitoringSVMController', {}, {
            monitoringInitialized: true,
            setComputingCapacity: jest.fn(),
            selectedWorkingEnvironment: workingEnvironment
        });
    }));

    it('should construct properly', () => {
        expect(volumesResource.startVolumesPolling).toHaveBeenCalledWith(workingEnvironment, jasmine.any(Function));
    });

    describe('calculateVolumesState', () => {
        it('should return volumes if vsaStartingStoppingState is running and volumes has returned', () => {
            j.scope.selectedWorkingEnvironment = {workingEnvironmentType: 'VSA', status: {status: 'on'}};
            j.scope.volumes = [];
            j.scope.vsaResponse = {};
            expect(j.scope.calculateVolumesState()).toEqual('volumes');
        });
    });

    describe('action required behavior', () => {
        beforeEach(() => {
            jest.spyOn(j.actionRequiredResource, 'startActionRequiredPolling');
            j.$rootScope.setActionRequiredFor = jest.fn();
            magnifispec.mockTabsController('MonitoringSVMController', {}, {
                monitoringInitialized: true,
                setComputingCapacity: jest.fn(),
                selectedWorkingEnvironment: {
                    _pathPrefix: '/vsa/',
                    publicId: '1',
                    activeActions: null,
                    vpcId: '1',
                    workingEnvironmentType: 'VSA',
                    _allowedOperations: {resources: true},
                    status: {status: 'on'}
                }
            });
            jest.spyOn(j.workingEnvironmentResource, 'initWorkingEnvironment').mockImplementation(() => {});
            j.scope.actionsRequired = {};
        });

        it('should start polling when gets first VSA response', () => {
            j.$rootScope.$broadcast(j.workingEnvironmentResource.VSA_GET_EVENT2, {workingEnvironment: {}, currentActiveActions: {}});
            expect(j.actionRequiredResource.startActionRequiredPolling).toHaveBeenCalled();
        });

        it('should restart polling when active become inactive', () => {
            j.scope.selectedWorkingEnvironment = {_pathPrefix: '/vsa/', publicId: '1', activeActions: ['something1'], vpcId: '1', workingEnvironmentType: 'VSA'};
            j.scope.vsaResponse = {_pathPrefix: '/vsa/', publicId: '1', activeActions: ['something2'], workingEnvironmentType: 'VSA', vpcId: '1'};
            j.$rootScope.$broadcast(1 + j.workingEnvironmentResource.VSA_ACTIVE_ACTIONS_GET_EVENT, []);
            j.$rootScope.$broadcast(1 + j.workingEnvironmentResource.VSA_ACTIVE_ACTIONS_GET_EVENT, ['something3']);
            j.$rootScope.$broadcast(1 + j.workingEnvironmentResource.VSA_ACTIVE_ACTIONS_GET_EVENT, []);

            expect(j.actionRequiredResource.startActionRequiredPolling.mock.calls.length).toEqual(2);
        });

        it('should update on action required polling', () => {
            _.set(j.$rootScope, 'supportServices.portalService', {});
            j.scope.selectedWorkingEnvironment = {_pathPrefix: '/vsa/', publicId: '1', activeActions: [], vpcId: '1', workingEnvironmentType: 'VSA'};
            j.scope.vsaResponse = {_pathPrefix: '/vsa/', publicId: '1', activeActions: [], workingEnvironmentType: 'VSA', vpcId: '1'};
            j.$rootScope.$broadcast(j.actionRequiredResource.ACTION_REQUIRED_GET_EVENT, [{'something': 'something'}]);
            expect(j.scope.setActionRequiredFor).toHaveBeenCalledWith('1', false, {'1': [{'something': 'something'}]});
            expect(j.scope.setComputingCapacity).toHaveBeenCalledWith(false);
        });

        it('should fail gracefully', () => {
            j.$rootScope.$broadcast(j.actionRequiredResource.ACTION_REQUIRED_ERROR_EVENT, j.errorResponse);
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.scope.setComputingCapacity).toHaveBeenCalledWith(false);
        });
    });

    describe('on destroy', () => {
        it('should clean up nicely', () => {
            j.$rootScope.$broadcast('$destroy');
            expect(stopVolumesPolling).toHaveBeenCalled();
        });
    });

    describe('volume polling', () => {
        beforeEach(() => {
            jest.spyOn(j.workingEnvironmentResource, 'initWorkingEnvironment').mockImplementation(() => {});
        });

        it('should stop polling when we status is changed to stopping', () => {
            j.workingEnvironmentResource.updateWorkingEnvironmentStatus(j.scope.selectedWorkingEnvironment, 'stopping');
            expect(stopVolumesPolling).toHaveBeenCalled();
        });

        it('should start polling when working environment is on', () => {
            j.workingEnvironmentResource.updateWorkingEnvironmentStatus(j.scope.selectedWorkingEnvironment, 'on');
            expect(volumesResource.startVolumesPolling).toHaveBeenCalled();
        });

        it('should stop the polling when the status is not defined', () => {
            j.workingEnvironmentResource.updateWorkingEnvironmentStatus(j.scope.selectedWorkingEnvironment);
            expect(stopVolumesPolling).toHaveBeenCalled();
        });
    });

    describe('filter volumes', () => {
        it('should filter the volumes list according to the search phrase', () => {
            j.scope.volumes = [{name: 'vol1', _displaySize: '10 GB'}, {name: 'vol2', _displaySize: '11 GB'}];
            j.scope.search = 'vol2';

            j.scope.$digest(); //Triggers scope.$watch

            expect(j.scope.filteredVolumes).toEqual([{name: 'vol2', _displaySize: '11 GB'}]);
        });
    });

    describe('handleActiveActionsUpdate', () => {
        let actionRequiredResource;
        beforeEach(inject((_actionRequiredResource_) => {
            actionRequiredResource = _actionRequiredResource_;
            j.scope.setComputingCapacity = jest.fn();
        }));

        it('should set computing capacity', () => {
            j.scope.handleActiveActionsUpdate(['a', 'b'], []);

            expect(j.scope.setComputingCapacity).toHaveBeenCalledWith(true);
        });

        it('should restart actionsRequired poller if all activeActions halted', () => {
            j.scope.selectedWorkingEnvironment.publicId = 'pid1';
            j.scope.actionsRequired = {'pid1': ['actions', 'required']};
            jest.spyOn(actionRequiredResource, 'startActionRequiredPolling').mockReturnValue(_.noop);

            j.scope.handleActiveActionsUpdate([], ['a', 'b']);

            expect(j.scope.actionsRequired.pid1).not.toBeDefined();
            expect(actionRequiredResource.startActionRequiredPolling).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment, jasmine.any(Function));
        });
    });

    describe('disableVolumeActionsForProblematicSync', () => {
        let _nocCurrentUserService;

        beforeEach(inject((nocCurrentUserService) => {
            j.scope.selectedWorkingEnvironment.cloudSyncProperties = {status: {status: 'COMPLETE'}};
            j.$rootScope.ownerUser = {cloudSyncLicenseInformation: {}};
            _nocCurrentUserService = nocCurrentUserService;

        }));

        it('should return message in case data broker status is not "completed"', () => {
            j.scope.selectedWorkingEnvironment.cloudSyncProperties.status.status = 'FAILED';

            expect(j.scope.disableVolumeActionsForProblematicSync({name: 'vol1'}))
                .toEqual('Volume cannot be synced to S3, Sync is not available');
        });

        it('should return message in case user Cloud Sync license expired', () => {
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseExpired').mockReturnValue(true);

            expect(j.scope.disableVolumeActionsForProblematicSync({name: 'vol1'}))
                .toEqual('Cloud Sync license expired');
        });
    });

    describe('disabledTooltipForSyncVolume', () => {
        it('should call disableVolumeActionsForProblematicSync to handle cloud sync issues', () => {
            jest.spyOn(j.scope, 'disableVolumeActionsForProblematicSync').mockReturnValue('Cloud Sync problem');

            expect(j.scope.disabledTooltipForSyncVolume({name: 'vol1'})).toEqual('Cloud Sync problem');

            expect(j.scope.disableVolumeActionsForProblematicSync).toHaveBeenCalled();
        });

        it('should call disabledTooltipForVolumeAction for handling other cases', () => {
            jest.spyOn(j.scope, 'disableVolumeActionsForProblematicSync').mockReturnValue(undefined);
            jest.spyOn(j.scope, 'disabledTooltipForVolumeAction').mockImplementation(() => {});

            expect(j.scope.disabledTooltipForSyncVolume({name: 'vol1'})).toBeFalsy();

            expect(j.scope.disabledTooltipForVolumeAction).toHaveBeenCalledWith({name: 'vol1'}, 'synced to S3', {
                dp: true,
                cifs: true,
                offline: true,
                undefined: true,
                moving: true,
                noJunctionPath: true,
                noMountPoint: true
            });
        });
    });

    describe('onclick', () => {
        it('should set selectedCardIndex to null if it is number', () => {
            j.scope.selectedCardIndex = 8;

            j.$window.onclick();
            j.$timeout.flush();

            expect(j.scope.selectedCardIndex).toBeNull();
        });

        it('should set selectedCardIndex to null if it is 0', () => {
            j.scope.selectedCardIndex = 0;

            j.$window.onclick();
            j.$timeout.flush();

            expect(j.scope.selectedCardIndex).toBeNull();
        });

        it('should do nothing if selectedCardIndex is not number', () => {
            j.scope.selectedCardIndex = undefined;

            j.$window.onclick();
            j.$timeout.flush();

            expect(j.scope.selectedCardIndex).toBe(undefined);
        });
    });

    describe('toggleCardMenu', () => {
        it('should set the selectedCardIndex', () => {
            j.scope.toggleCardMenu(7);

            expect(j.scope.selectedCardIndex).toEqual(7);
        });

        it('should stop stopPropagation', () => {
            const stopPropagationSpy = jest.fn();

            j.scope.toggleCardMenu(7, {stopPropagation: stopPropagationSpy});

            expect(stopPropagationSpy).toHaveBeenCalled();
        });
    });
});
