describe('vpcControllerSpec', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    const workingEnvironment = _.cloneDeep({publicId: '1', vpcId: '1', _pathPrefix: '/vsa/', _cloudProviderName: 'AWS'});

    beforeEach(() => {
        jest.spyOn(j.workingEnvironmentResource, 'startInstancesPolling').mockImplementation(() => {});
        jest.spyOn(j.workingEnvironmentResource, 'getVpcOfWorkingEnvironment');
        magnifispec.mockTabsController('VpcController', {}, {selectedWorkingEnvironment: workingEnvironment});
    });

    it('should call instances polling upon initialize', () => {
        expect(j.workingEnvironmentResource.startInstancesPolling).toHaveBeenCalledWith(workingEnvironment);
    });

    it('should call getVpcOfWorkingEnvironment upon initialize', () => {
        expect(j.workingEnvironmentResource.getVpcOfWorkingEnvironment).toHaveBeenCalled();
    });

    describe('loaded?', () => {
        it('should return true if there is both a vpc and instances array', () => {
            j.scope.instances = [];
            j.scope.vpc = {};
            expect(j.scope.loaded()).toEqual(true);
        });

        it('should return false if there is no vpc', () => {
            j.scope.instances = [];
            expect(j.scope.loaded()).toEqual(false);
        });

        it('should return false if there is no instance array', () => {
            j.scope.vpc = {};
            expect(j.scope.loaded()).toEqual(false);
        });
    });

    describe('getting vpc of working environment', () => {
        it('should set the vpc to the retrieved value in success case', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=awsProperties').respond({awsProperties: {vpc: {id: 2, cidrBlock: 2}}});
            j.$httpBackend.flush();
            expect(j.scope.vpc).toEqual({id: 2, cidrBlock: 2});
            expect(j.scope.selectedWorkingEnvironment.vpcCidrBlock).toEqual(2);
        });
    });

    describe('filtering instances', () => {
        const payload = {
            workingEnvironment: {publicId: 1},
            instances: [{id: 2, cidrBlock: 2, name: 'bla'}, {id: 3, cidrBlock: 3}]
        };

        beforeEach(() => {
            j.scope.selectedWorkingEnvironment = {publicId: 1};

            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=awsProperties').respond(payload);
            j.$rootScope.$broadcast(j.workingEnvironmentResource.INSTANCES_GET_EVENT, payload);
        });

        it('should provide a filtered instances collection on the scope', () => {
            expect(j.scope.instances).toEqual(payload.instances);
            expect(j.scope.filteredInstances).toEqual(payload.instances);
        });

        it('should filter', () => {
            j.$rootScope.$apply(() => {
                j.scope.search = 'blat';
            });
            expect(j.scope.filteredInstances).toEqual([]);

            j.$rootScope.$apply(() => {
                j.scope.search = 'bla';
            });
            expect(j.scope.filteredInstances).toEqual([{id: 2, cidrBlock: 2, name: 'bla'}]);
        });

        it('should not filter when not filtering', () => {
            j.$rootScope.$apply(() => {
                j.scope.search = '';
            });
            expect(j.scope.filteredInstances).toEqual([{id: 2, cidrBlock: 2, name: 'bla'}, {id: 3, cidrBlock: 3, name: 'Unnamed'}]);
        });
    });
});
