describe('Users Controller', () => {
    'use strict';

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    const mockedUsers = [
        {
            publicId: '1',
            firstName: 'Roy',
            lastName: 'Alon',
            email: 'kuki@puki.com',
            roleId: 'Role-1',
            role: {name: 'Oncloud Admin', publicId: 'Role-1'},
            tenantId: 'Tenant-1234',
            createdWorkingEnvironments: [],
            assignedWorkingEnvironments: ['we-1'],
            isLdap: false
        },
        {
            publicId: '2',
            firstName: 'Daniel',
            lastName: 'QA',
            email: 'daniel@puki.com',
            roleId: 'Role-2',
            tenantId: null,
            createdWorkingEnvironments: [],
            assignedWorkingEnvironments: ['we-2', '3'],
            isLdap: true,
            ldapUserName: 'daniel'
        },
        {
            publicId: '3',
            firstName: 'Anatoly',
            lastName: 'Shaoulov',
            email: 'anatoly@puki.com',
            roleId: 'Role-3',
            tenantId: 'Tenant-2345',
            createdWorkingEnvironments: [],
            assignedWorkingEnvironments: [],
            isLdap: false
        }
    ];

    beforeEach(inject((nocCurrentUserService) => {
        jest.spyOn(nocCurrentUserService, 'getRoles').mockReturnValue([{name: 'OnCloud Admin', publicId: 'Role-1'}, {
            name: 'Tenant Admin',
            publicId: 'Role-2'
        }, {name: 'WE Admin', publicId: 'Role-3'}]);


        j.nocFriendlyStore.roles = [{name: 'OnCloud Admin', publicId: 'Role-1'}, {
            name: 'Tenant Admin',
            publicId: 'Role-2'
        }, {name: 'WE Admin', publicId: 'Role-3'}];

        j.currentUser = _.cloneDeep(mockedUsers[0]);

        magnifispec.mockTabsController('UsersController', undefined, {
            tabs: [{tabName: 'tabs/users'}]
        });

        j.usersUrl = j.nocBaseUrl + '/users';

        j.workingEnvironmentsUrl = j.nocBaseUrl + '/working-environments';

        j.$rootScope.checkAndSetState = jest.fn().mockReturnValue(j.$q.when());
    }));

    describe('Initialize Users', () => {
        beforeEach(inject((nocTenantsService) => {
            jest.spyOn(nocTenantsService, 'getTenants').mockReturnValue([{name: 'Tenant1', publicId: 'Tenant-1234'}]);
            j.scope.initializeUsers();
            const wes = {
                vsaWorkingEnvironments: [{name: 'WE1', publicId: 'we-1'}, {name: 'WE2', publicId: 'we-2'}, {
                    name: 'WE3',
                    publicId: '3'
                }],
                onPremWorkingEnvironments: []
            };
            jest.spyOn(j.workingEnvironmentResource, 'processAggregateWorkingEnvironmentResponse').mockImplementation(() => {
                return [{name: 'WE1', publicId: 'we-1'}, {name: 'WE2', publicId: 'we-2'}, {name: 'WE3', publicId: '3'}];
            });
            j.$httpBackend.expectGET(j.workingEnvironmentsUrl).respond(200, wes);
        }));

        //todo: fix...
        // it('should indicate which user is the current user', () => {
        //     j.$httpBackend.expectGET(j.usersUrl).respond(200, [mockedUsers[0], mockedUsers[1]]);
        //     j.$httpBackend.flush();
        //     expect(j.scope.workingEnvironments).toEqual([{name: 'WE1', publicId: 'we-1'}, {
        //         name: 'WE2',
        //         publicId: 'we-2'
        //     }, {name: 'WE3', publicId: '3'}]);
        //     expect(j.scope.users).toEqual([
        //         {
        //             publicId: '1', firstName: 'Roy',
        //             lastName: 'Alon',
        //             email: 'kuki@puki.com',
        //             roleId: 'Role-1',
        //             role: {name: 'OnCloud Admin', publicId: 'Role-1'},
        //             tenantId: 'Tenant-1234',
        //             createdWorkingEnvironments: [],
        //             workingEnvironments: [{name: 'WE1', publicId: 'we-1'}],
        //             assignedWorkingEnvironments: ['we-1'],
        //             currentUser: true,
        //             name: 'Roy Alon',
        //             tenant: {name: 'Tenant1', publicId: 'Tenant-1234'},
        //             _association: 'Tenant1',
        //             isLdap: false,
        //             _accountType: 'Cloud Manager'
        //         },
        //         {
        //             publicId: '2', firstName: 'Daniel',
        //             lastName: 'QA',
        //             email: 'daniel@puki.com',
        //             roleId: 'Role-2',
        //             tenantId: null,
        //             createdWorkingEnvironments: [],
        //             workingEnvironments: [{name: 'WE2', publicId: 'we-2'}, {name: 'WE3', publicId: '3'}],
        //             assignedWorkingEnvironments: ['we-2', '3'],
        //             currentUser: false,
        //             name: 'Daniel QA',
        //             role: {name: 'Tenant Admin', publicId: 'Role-2'},
        //             _association: '--',
        //             isLdap: true,
        //             ldapUserName: 'daniel',
        //             _accountType: 'Active Directory'
        //         }
        //     ]);
        // });
    });

    describe('Generate working environments map', () => {
        it('should properly generate the working environments map', () => {
            const user = {workingEnvironments: [{name: 'WE1', publicId: 'we-1'}, {name: 'WE2', publicId: 'we-2'}]};
            const map = j.scope.generateGrantedWorkingEnvironmentsMap(user);
            expect(map).toEqual({'we-1': true, 'we-2': true});
        });
    });

    describe('Navigating functions', () => {
        it('should call checkAndSetState with the right parameters when newUser is called', () => {
            j.scope.newUser();
            expect(j.$rootScope.checkAndSetState).toHaveBeenCalledWith('^.new');
        });

        it('should call checkAndSetState with the right parameters when editGivenUser is called', () => {
            j.scope.editGivenUser({publicId: '1'});
            expect(j.$rootScope.checkAndSetState).toHaveBeenCalledWith('^.edit', {userId: '1'});
        });
    });

    describe('transition to main.users.edit', () => {
        it('should prevent call prevent default if the selected user is tenant and the current user is WE', () => {
            const event = j.$rootScope.$broadcast('$stateChangeStart', {name: 'main.users.edit'}, {userId: '1'});
            expect(event.defaultPrevented).toEqual(true);
            expect(j.scope.originalSelectedUser).toBeUndefined();
        });
    });

    describe('Deleting the selected user', () => {
        beforeEach(() => {
            j.scope.users = mockedUsers;
            j.scope.gridOptions.data = mockedUsers;
            j.scope.selectedUser = j.scope.users[1];
            j.scope.selectedUser.name = j.scope.selectedUser.firstName + ' ' + j.scope.selectedUser.lastName;
        });

        it('should do nothing if the user does not confirm (and make sure confirm message is what we want)', () => {
            magnifispec.mockConfirm(false);

            j.scope.deleteUser(j.scope.selectedUser);

            expect(j.nocDialog.confirm).toHaveBeenCalledWith(j.Messages.deleteConfirm({
                type: 'user',
                name: 'Daniel QA'
            }));
        });

        it('should call delete for the user, and on success remove the selected user from the list and gridOptions', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectDELETE(j.usersUrl + '/2').respond(200, {});

            j.scope.deleteUser(j.scope.selectedUser);
            j.$httpBackend.flush();
            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);

            const result = [mockedUsers[0], mockedUsers[2]];

            expect(j.scope.users).toEqual(result);
            expect(j.scope.gridOptions.data).toEqual(result);

            expect(j.pidgeon.clear).toHaveBeenCalledWith('deleting user');
        });

        it('should call delete for the user and on failure push an alert and not call close', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectDELETE(j.usersUrl + '/2').respond(400, j.errorResponse);

            j.scope.deleteUser(j.scope.selectedUser);
            j.$httpBackend.flush();
            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });
});
