describe("UsersResetPassword", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(function () {
        magnifispec.mockDialogController("UsersResetPassword", {}, {selectedUser: {publicId: "1"}});
        j.usersUrl = j.nocBaseUrl + "/users";
    });

    describe("Is valid", function () {
        it("should return true if the changePasswordForm is valid", function () {
            j.scope.changePasswordForm = {$valid:true};
            expect(j.scope.isValid()).toEqual(true);
        });

        it("should return false if the changePasswordForm is invalid", function () {
            j.scope.changePasswordForm = {$valid:false};
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe("Saving password", function () {
        it("should call handle form validity if invalid", function () {
            jest.spyOn(j.scope, "isValid").mockReturnValue(false);

            j.scope.savePasswordOfSelectedUser();

            expect(j.scope.isValid).toHaveBeenCalled();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        it("should make a put request with the new password, and if successful set mode to edit", function () {
            jest.spyOn(j.scope, "isValid").mockReturnValue(true);

            j.scope.changePassword = {password:"myPassword"};
            j.scope.selectedUser = {publicId: "2", firstName: "Daniel",
                lastName: "QA",
                email: "daniel@puki.com",
                roleId: "Role-2",
                tenantId: null,
                role: {name:"Tenant Admin", publicId:"Role-2"},
                workingEnvironments: []};

            j.$httpBackend.expectPUT(j.usersUrl +"/2/reset-password", {password:"myPassword"}).respond(200);

            j.scope.savePasswordOfSelectedUser();
            j.$httpBackend.flush();

            expect(j.parentScope.closeDialog).toHaveBeenCalled();
        });

        it("should make a put request with the new password, and if failure push an alert", function () {
            jest.spyOn(j.scope, "isValid").mockReturnValue(true);

            j.scope.changePassword = {password:"myPassword"};
            j.scope.selectedUser = {publicId: "2", firstName: "Daniel",
                lastName: "QA",
                email: "daniel@puki.com",
                roleId: "Role-2",
                tenantId: null,
                role: {name:"Tenant Admin", publicId:"Role-2"},
                workingEnvironments: []};

            j.$httpBackend.expectPUT(j.usersUrl +"/2/reset-password", {password:"myPassword"}).respond(400,j.errorResponse);

            j.scope.savePasswordOfSelectedUser();
            j.$httpBackend.flush();

            expect(j.parentScope.closeDialog).not.toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();

        });
    });
});