describe('BaseCreateVolume', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let BaseCreateVolume;

    beforeEach(inject((_BaseCreateVolume_) => {
        BaseCreateVolume = _BaseCreateVolume_;
        j.scope = {
            wizardController: {
                exportPolicyModel: {
                    name: 'Custom export policy',
                    value: 'custom'
                }
            },
            volumeRequest: {},
            snapshotPolicies: [
                {
                    name: 'snap1',
                    descrition: 'snap1 desc',
                    schedules: [
                        {frequency: 'weekly', retention: '6'},
                        {frequency: 'monthly', retention: '2'}
                    ]
                },
                {
                    name: 'snap2',
                    description: 'snap2 desc',
                    schedules: [
                        {frequency: 'weekly', retention: '6'},
                        {frequency: 'monthly', retention: '2'}
                    ]
                },
                {
                    name: 'name1111111111111111111111111111111111111111',
                    description: 'snap3 desc',
                    schedules: [
                        {frequency: 'weekly', retention: '6'},
                        {frequency: 'monthly', retention: '2'}
                    ]
                }
            ]
        };
        BaseCreateVolume.call(j.scope);
    }));

    describe('getSnapshotPolicyByName', () => {
        it("should return the snapshot policy with the given name ByName", () => {
            j.scope.snapshotPolicies = [
                {name: "a"},
                {name: "b"}
            ];
            expect(j.scope.getSnapshotPolicyByName("a")).toEqual({name: "a"});
        });
    });

    describe("selectADefaultSnapshotPolicy", function () {
        it("should select the default policy if exists", function () {
            j.scope.snapshotPolicies[1].name = "default";
            j.scope.selectADefaultSnapshotPolicy();
            expect(j.scope.volumeRequest.snapshotPolicyName).toEqual("default");
        });

        it("should select the first policy if the default doesn't exist", function () {
            j.scope.selectADefaultSnapshotPolicy();
            expect(j.scope.volumeRequest.snapshotPolicyName).toEqual("snap1");
        });

        it("should do nothing if there are no policies", function () {
            j.scope.snapshotPolicies = null;
            j.scope.selectADefaultSnapshotPolicy();
            expect(j.scope.volumeRequest.snapshotPolicyName).toBeUndefined();
        });

        //Test for: NOC-6051 - UI - Encryption - Wizard - When going back, Protection Snapshot Policy is not saved
        it("should not override existing selection", function () {
            j.scope.volumeRequest.snapshotPolicyName = "already selected policy";
            j.scope.selectADefaultSnapshotPolicy();
            expect(j.scope.volumeRequest.snapshotPolicyName).toEqual("already selected policy");
        });
    });

    describe('fetchSnapshotPolicy', () => {
        const mockedSnapshots = [
            {
                name: "snap1",
                descrition: "snap1 desc",
                schedules: [
                    {
                        frequency: "weekly",
                        retention: "6"
                    },
                    {
                        frequency: "monthly",
                        retention: "2"
                    }
                ]
            },
            {
                name: "snap2",
                descrition: "snap2 desc",
                schedules: [
                    {
                        frequency: "weekly",
                        retention: "6"
                    },
                    {
                        frequency: "monthly",
                        retention: "2"
                    }
                ]
            },
            {
                name: "default-1weekly",
                descrition: "snap2 desc",
                schedules: [
                    {
                        frequency: "weekly",
                        retention: "6"
                    },
                    {
                        frequency: "monthly",
                        retention: "2"
                    }
                ]
            }
        ];
        const workingEnvironment = {
            publicId: "1",
            _pathPrefix: "/vsa/"
        };

        it("should remove default-1weekly", () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/vsa/working-environments/1?fields=snapshotPolicies").respond({snapshotPolicies: mockedSnapshots});

            j.scope.fetchSnapshotPolicy(workingEnvironment, null, j.nocHttp);
            j.$httpBackend.flush();

            const item = _.find(j.scope.snapshotPolicies, {name: 'default-1weekly'});
            expect(item).not.toBeDefined();
            expect(j.scope.snapshotPolicies.length).toEqual(2);
        });

        it("should fail gracefully", () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/vsa/working-environments/1?fields=snapshotPolicies").respond(400, j.errorResponse);

            j.scope.fetchSnapshotPolicy(workingEnvironment, null, j.nocHttp);
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe("initialization", function () {
        it("controller initialization", function () {
            expect(j.scope.volumeProtocol.type).toEqual("NFS");
        });
    });

    describe("generateRequestBody with NFS", function () {
        beforeEach(function () {
            j.scope.volumeRequest = {
                exportPolicyInfo:
                {
                    policyType: "all",
                    ips: [],
                    _ips: ""

                },
                usageProfile: j.usageProfiles.default
            };
        });

        it("should clone the request from the scope", function () {
            var body = j.scope.generateRequestBody(undefined, "NFS");
            expect(body).not.toEqual(j.scope.volumeRequest);
            expect(body.approvedAllocation).toBeUndefined();
            expect(body.exportPolicy).toEqual(j.scope.volumeRequest.exportPolicy);
        });

        it("should include the efficiency flags but not the usage profile", function () {
            var body = j.scope.generateRequestBody(undefined, "NFS");
            expect(body.enableThinProvisioning).toEqual(true);
            expect(body.enableDeduplication).toEqual(true);
            expect(body.enableCompression).toEqual(true);
            expect(body.usageProfile).toBeUndefined();
        });

        it("should include the quote attributes when passed", function () {
            var body = j.scope.generateRequestBody({aggregateName:'aggr1',numOfDisks:3});
            expect(body.aggregateName).toEqual('aggr1');
            expect(body.maxNumOfDisksApprovedToAdd).toEqual(3);
        });
    });

    describe("generateRequestBody with CIFS", function () {
        beforeEach(function () {
            j.scope.volumeRequest = {
                exportPolicyInfo:{
                    policyType: "all",
                    ips: [],
                    _ips: ""
                },
                shareInfo:{
                    accessControl: {
                        users: ["Everyone"],
                        permission: "full_control"
                    },
                    shareName: ""
                }, usageProfile: j.usageProfiles.default
            };
        });

        it("should remove nfs protocol info and set cifs protocol info for sending", function () {
            j.scope.volumeRequest.shareInfo.accessControl._users = "Everyone;Someone;None;";
            var body = j.scope.generateRequestBody(undefined, "CIFS");
            expect(body.exportPolicy).toBeUndefined();
            expect(body.shareInfo.accessControl.users).toEqual(["Everyone","Someone","None"]);
            expect(body.shareInfo.accessControl.permission).toEqual(j.scope.volumeRequest.shareInfo.accessControl.permission);
            expect(body.shareInfo.shareName).toEqual('');
        });
    });

    describe('policyTypeChanged', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'initSyncToS3Status').mockImplementation(() => {});
            _.set(j.scope, 'exportPolicyWrapper.exportPolicyModel', {value: 'none'});
            _.set(j.scope, 'volumeRequest.exportPolicyInfo.policyType', 'custom');
        });

        it('should update the policy on the volume request', () => {
            j.scope.policyTypeChanged();

            expect(j.scope.volumeRequest.exportPolicyInfo.policyType).toEqual('none');
        });

        it('should disable the syncToS3 with no access volume', () => {
            j.scope.syncToS3Status = 'ENABLED';

            j.scope.policyTypeChanged();

            expect(j.scope.syncToS3Status).toEqual('NO_ACCESS');
            expect(j.scope.volumeRequest.syncToS3).toBeFalsy();
        });

        it('should restore initial syncToS3Status value for volume with access', () => {
            j.scope.syncToS3Status = 'ENABLED';
            _.set(j.scope, 'exportPolicyWrapper.exportPolicyModel.value', 'custom');

            j.scope.policyTypeChanged();

            expect(j.scope.initSyncToS3Status).toHaveBeenCalled();
        });

        it('should do nothing if syncToS3 is not available altogether', () => {
            j.scope.syncToS3Status = 'NOT_AVAILABLE';

            j.scope.policyTypeChanged();

            expect(j.scope.syncToS3Status).toEqual('NOT_AVAILABLE');
            expect(j.scope.volumeRequest.syncToS3).toBeFalsy();
            expect(j.scope.initSyncToS3Status).not.toHaveBeenCalled();
        });

        it('should do nothing if syncToS3 license is expired', () => {
            j.scope.syncToS3Status = 'EXPIRED';

            j.scope.policyTypeChanged();

            expect(j.scope.syncToS3Status).toEqual('EXPIRED');
            expect(j.scope.volumeRequest.syncToS3).toBeFalsy();
            expect(j.scope.initSyncToS3Status).not.toHaveBeenCalled();
        });
    });
});
