describe('edit tags controller', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    const awsKeysData = [
        {
            key: 'instance.group-id',
            values: ['sg-1']
        },
        {
            key: 'a',
            values: ['']
        },
        {
            key: 'Version',
            values: ['1.0']
        },
        {
            key: 'gil',
            values: ['hey']
        },
        {
            key: 'Network',
            values: ['ISCSI-LIF']
        },
        {
            key: 'lian',
            values: ['yoyo']
        },
        {
            key: 'also',
            values: ['go']
        },
        {
            key: 'tag:aws:cloudformation:stack-id',
            values: ['stack-SG-Version10-mSoSyHS9']
        },
        {
            key: 'noam',
            values: ['yo']
        }
    ];
    const azureKeysData = [
        {
            key: 'azureNoam',
            values: ['yo']
        },
        {
            key: 'azureGil',
            values: ['gc']
        },
        {
            key: 'azureAaaaa',
            values: ['a']
        }
    ];
    const vsaTagsResponse = {
        tags: [
            {
                tagKey: 'a',
                tagValue: ''
            },
            {
                tagKey: 'gil',
                tagValue: 'hey'
            },
            {
                tagKey: 'lian',
                tagValue: 'yoyo'
            }
        ]
    };
    const azureVsaTagsResponse = {
        tags: [
            {
                tagKey: 'azureNoam',
                tagValue: 'yo'
            }
        ]
    };
    const haTagsResponse = {
        tags: [
            {
                tagKey: 'also',
                tagValue: 'go'
            }
        ]
    };
    const expectedVsaCustomTags = [{tagKey: 'a', tagValue: ''}, {tagKey: 'gil', tagValue: 'hey'}, {
        tagKey: 'lian',
        tagValue: 'yoyo'
    }];
    const expectedHaCustomTags = [{tagKey: 'also', tagValue: 'go'}];
    const expectedAzureCustomTags = [{tagKey: 'azureNoam', tagValue: 'yo'}];
    const awsVsaWe = {
        cloudProviderName: 'Amazon',
        _pathPrefix: '/vsa/',
        publicId: '1',
        workingEnvironmentType: 'VSA',
        status: {status: 'ON'},
        providerProperties: {numOfNics: 6},
        _cloudProviderName: 'AWS',
        awsProperties: {
            instances: [{
                tags: {
                    a: '',
                    gil: 'hey',
                    lian: 'yoyo',
                    WorkingEnvironment: 'vsa',
                    'CloudONTAP.serial-number': 'serial-vsa',
                    WorkingEnvironmentId: '1'
                }
            }]
        },
        _cloudProviderAccountId: 'cloudProviderAccountId'
    };
    const haWe = {
        cloudProviderName: 'Amazon',
        _pathPrefix: '/aws/ha/',
        publicId: '1',
        workingEnvironmentType: 'VSA',
        status: {status: 'ON'},
        providerProperties: {numOfNics: 6},
        _cloudProviderName: 'AWS',
        isHA: true,
        awsProperties: {
            instances: [
                {
                    name: 'ha-node2',
                    id: 'ha-node2-id',
                    tags: {
                        color: 'green',
                        'CloudONTAP.serial-number': '90120140000000000124'
                    }
                },
                {
                    name: 'ha-node1',
                    id: 'ha-node1-id',
                    tags: {
                        color: 'red',
                        'CloudONTAP.serial-number': '90120140000000000123'
                    }
                }
            ]
        },
        _cloudProviderAccountId: 'cloudProviderAccountIdHA',
        ontapClusterProperties: {
            nodes: [
                {cloudProviderId: 'ha-node1-id'},
                {cloudProviderId: 'ha-node2-id'}
            ]
        }
    };

    const azureWe = {
        _pathPrefix: '/azure/vsa/',
        publicId: '1',
        workingEnvironmentType: 'VSA',
        status: {status: 'ON'},
        providerProperties: {numOfNics: 6, subscriptionId: '123'},
        _cloudProviderName: 'Azure',
        _cloudProviderAccountId: 'cloudProviderAccountId'

    };

    beforeEach(() => {
        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/user-tags').respond(200, vsaTagsResponse);
        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/tag-keys?cloudProviderAccountId=cloudProviderAccountId').respond(200, awsKeysData);
        magnifispec.mockTabsController('EditTagsController', {}, {
            selectedWorkingEnvironment: awsVsaWe
        });
        j.$httpBackend.flush();
    });

    describe('initialize controller AWS', () => {
        it('on normal scenario VSA', () => {
            j.scope.selectedWorkingEnvironment = awsVsaWe;
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/user-tags').respond(200, vsaTagsResponse);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/tag-keys?cloudProviderAccountId=cloudProviderAccountId').respond(200, awsKeysData);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.tags.customTags).toEqual(expectedVsaCustomTags);
            expect(j.scope.existingTagNames).toEqual(['instance.group-id', 'Version', 'Network', 'also', 'tag:aws:cloudformation:stack-id', 'noam']);
            expect(j.scope.existingTags).toEqual({
                'instance.group-id': ['sg-1'],
                Version: ['1.0'],
                Network: ['ISCSI-LIF'],
                also: ['go'],
                'tag:aws:cloudformation:stack-id': ['stack-SG-Version10-mSoSyHS9'],
                noam: ['yo']
            });
        });

        it('on normal scenario HA', () => {
            j.scope.selectedWorkingEnvironment = haWe;
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/working-environments/1/user-tags').respond(200, haTagsResponse);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/metadata/tag-keys?cloudProviderAccountId=cloudProviderAccountIdHA').respond(200, awsKeysData);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.tags.customTags).toEqual(expectedHaCustomTags);
            expect(j.scope.existingTagNames).toEqual(['instance.group-id', 'a', 'Version', 'gil', 'Network', 'lian', 'tag:aws:cloudformation:stack-id', 'noam']);
            expect(j.scope.existingTags).toEqual({
                'instance.group-id': ['sg-1'],
                a: [''],
                Version: ['1.0'],
                gil: ['hey'],
                Network: ['ISCSI-LIF'],
                lian: ['yoyo'],
                'tag:aws:cloudformation:stack-id': ['stack-SG-Version10-mSoSyHS9'],
                noam: ['yo']
            });
        });

        it('on normal scenario Azure', () => {
            j.scope.selectedWorkingEnvironment = azureWe;
            j.$httpBackend.expectGET(j.nocBaseUrl + '/azure/vsa/working-environments/1/user-tags').respond(200, azureVsaTagsResponse);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/azure/vsa/metadata/tag-keys?subscriptionId=' + j.scope.selectedWorkingEnvironment.providerProperties.subscriptionId + '&cloudProviderAccountId=cloudProviderAccountId').respond(200, azureKeysData);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.tags.customTags).toEqual(expectedAzureCustomTags);
            expect(j.scope.existingTagNames).toEqual(['azureGil', 'azureAaaaa']);
            expect(j.scope.existingTags).toEqual({azureGil: ['gc'], azureAaaaa: ['a']});
        });
    });

    describe('tags should return ', () => {
        it('empty for HA WE', () => {
            expect(j.scope.tags({isHA: true, _cloudProviderName: 'AWS'})).toEqual({});
        });

        it('undefined when there are no instances', () => {
            expect(j.scope.tags(_.set({_cloudProviderName: 'AWS'}, 'awsProperties.instances', []))).toBeUndefined();
        });

        it('tags with the right we id', () => {
            const tags = {WorkingEnvironmentId: 'qwerty', anotherTag: 'someValue', 'CloudONTAP.serial-number': 12345};
            const we = {
                _cloudProviderName: 'AWS',
                publicId: 'qwerty',
                awsProperties: {instances: [{tags}]}
            };
            expect(j.scope.tags(we)).toEqual(tags);
        });

        it('tags for working environment and not for data broker with same weId', () => {
            const weTags = {WorkingEnvironmentId: 'qwerty', anotherTag: 'someValue', 'CloudONTAP.serial-number': 12345};
            const dataBrokerTags = {WorkingEnvironmentId: 'qwerty'};
            const we = {
                _cloudProviderName: 'AWS',
                publicId: 'qwerty',
                awsProperties: {instances: [{tags: weTags}, {tags: dataBrokerTags}]}
            };
            expect(j.scope.tags(we)).toEqual(weTags);
        });

        it('undefined when the wrong we id', () => {
            const tags = {WorkingEnvironmentId: 'not.qwerty', anotherTag: 'someValue'};
            const we = {
                _cloudProviderName: 'AWS',
                publicId: 'qwerty',
                awsProperties: {instances: [{tags}]}
            };
            expect(j.scope.tags(we)).toBeUndefined();
        });

        it('undefined when there are no tags', () => {
            const we = {
                _cloudProviderName: 'AWS',
                publicId: 'qwerty',
                awsProperties: {instances: [{}]}
            };
            expect(j.scope.tags(we)).toBeUndefined();
        });

        it('the tags for azure', () => {
            const tags = {WorkingEnvironmentId: 'qwerty', anotherTag: 'someValue'};
            const we = {
                _cloudProviderName: 'Azure',
                providerProperties: {tags}
            };
            expect(j.scope.tags(we)).toEqual(tags);
        });

        it('undefined is providerProperties.tags is not defined', () => {
            const we = {
                _cloudProviderName: 'Azure'
            };
            expect(j.scope.tags(we)).toBeUndefined();
        });
    });

    describe('showAdditionalTagsDialog', () => {
        const dialogText = 'The following tags are applied to some of the AWS resources associated with this working environment. You can manage these tags from the AWS console.';
        const warningText = 'Do not edit or delete system tags from the AWS console. Doing so can cause permanent damage to the working environment.';

        it('should properly construct the list arrays and open the dialog for VSA', () => {
            const lists = {
                leftList: [
                    {text: 'serial-vsa', key: 'CloudONTAP.serial-number', isSystemTag: true, trimmedText: true, trimmedKey: true},
                    {text: 'vsa', key: 'WorkingEnvironment', isSystemTag: true, trimmedText: true, trimmedKey: true},
                    {text: '1', key: 'WorkingEnvironmentId', isSystemTag: true, trimmedText: true, trimmedKey: true}
                ],
                keysHeader: 'Key', valuesHeader: 'Value'
            };
            const finish = magnifispec.fakeDialog();
            j.scope.showAdditionalTagsDialog();
            finish();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Additional Tags [3]', 'assets/templates/dialogs/general-info-dialog.html', {
                dialogText,
                width: 800,
                lists,
                hasWarning: true,
                warningText,
                limitedHeight: true
            });
        });

        it('should properly construct the list arrays and open the dialog for HA', () => {
            j.scope.selectedWorkingEnvironment = haWe;
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/working-environments/1/user-tags').respond(200, haTagsResponse);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/aws/ha/metadata/tag-keys?cloudProviderAccountId=cloudProviderAccountIdHA').respond(200, awsKeysData);
            j.scope.initialize();
            j.$httpBackend.flush();
            const tagsData = {
                data: [
                    {key: 'CloudONTAP.serial-number', isSystemTag: true, value1: '90120140000000000123', value2: '90120140000000000124'},
                    {key: 'color', isSystemTag: false, value1: 'red', value2: 'green'}
                ],
                node1: 'ha-node1',
                node2: 'ha-node2'
            };
            const finish = magnifispec.fakeDialog();
            j.scope.selectedWorkingEnvironment.ontapClusterProperties = {
                nodes: [
                    {cloudProviderId: 'ha-node1-id'},
                    {cloudProviderId: 'ha-node2-id'}
                ]
            };
            //Unsorted on purpose - should sort nodes by name
            j.scope.selectedWorkingEnvironment.awsProperties = {
                instances: [
                    {
                        name: 'ha-node2',
                        id: 'ha-node2-id',
                        tags: {
                            color: 'green',
                            'CloudONTAP.serial-number': '90120140000000000124'
                        }
                    },
                    {
                        name: 'ha-node1',
                        id: 'ha-node1-id',
                        tags: {
                            color: 'red',
                            'CloudONTAP.serial-number': '90120140000000000123'
                        }
                    }
                ]
            };
            j.scope.showAdditionalTagsDialog();
            finish();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Additional Tags [2]', 'assets/templates/tabs/working-environments/ha-tags.html', {
                dialogText,
                width: 1250,
                tags: tagsData,
                hasWarning: true,
                warningText,
                limitedHeight: true
            });
        });
    });

    describe('updateTags', () => {
        it('should call set tags api and return to monitoring uppon success', () => {
            j.scope.tags.customTags.push({tagKey: 'key', tagValue: 'val'});
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/working-environments/1/user-tags');
            _.set(j.scope, 'editTagsForm.$valid', true);
            j.scope.updateTags();
            expect(j.nocUIMain.handleFormValidity).not.toHaveBeenCalled();
        });

        it('should fail if tags are not valid', () => {
            j.scope.addCustomTag();
            _.set(j.scope, 'editTagsForm.$valid', false);
            j.scope.updateTags();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });
    });

    describe('noChanges', () => {
        it('should return true when no changes happened', () => {
            expect(j.scope.noChanges()).toBeTruthy();
        });

        it('should return true when only the keys order has changed', () => {
            j.scope.tags.customTags = [
                {
                    tagKey: 'gil',
                    tagValue: 'hey'
                },
                {
                    tagKey: 'a',
                    tagValue: ''
                },
                {
                    tagKey: 'lian',
                    tagValue: 'yoyo'
                }
            ];
            expect(j.scope.noChanges()).toBeTruthy();
        });

        it('should return false when the whole list has been changed', () => {
            j.scope.tags.customTags = haTagsResponse;
            expect(j.scope.noChanges()).toBeFalsy();
        });

        it('should return false when a key was edited', () => {
            j.scope.tags.customTags = [
                {
                    tagKey: 'changed',
                    tagValue: ''
                },
                {
                    tagKey: 'gil',
                    tagValue: 'hey'
                },
                {
                    tagKey: 'lian',
                    tagValue: 'yoyo'
                }
            ];
            expect(j.scope.noChanges()).toBeFalsy();
        });

        it('should return false when a value was edited', () => {
            j.scope.tags.customTags = [
                {
                    tagKey: 'a',
                    tagValue: 'changed'
                },
                {
                    tagKey: 'gil',
                    tagValue: 'hey'
                },
                {
                    tagKey: 'lian',
                    tagValue: 'yoyo'
                }
            ];
            expect(j.scope.noChanges()).toBeFalsy();
        });
    });

    describe('addCustomTag', () => {
        it('should add empty tag key and value and change addTagsPressed', () => {
            j.scope.addTagsPressed = false;
            j.scope.tags.customTags = _.clone(expectedVsaCustomTags);
            j.scope.addCustomTag();
            expect(j.scope.addTagsPressed).toBeTruthy();
            expect(j.scope.tags.customTags).toEqual(_.concat(expectedVsaCustomTags, [{
                tagKey: '',
                tagValue: ''
            }]));
        });
    });

    describe('removeCustomTag', () => {
        it('should remove the tag it was called with ', () => {
            const tag = {tagKey: 'gil', tagValue: 'hey'};
            j.scope.addTagsPressed = false;
            j.scope.tags.customTags = _.clone(expectedVsaCustomTags);
            j.scope.removeCustomTag(tag);
            expect(j.scope.tags.customTags).toEqual(_.without(j.scope.tags.customTags, tag));
        });
    });
});
