describe('GalleryGridController', function () {
    'use strict';

    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    let workingEnvironmentStatuses, uiGridFilterService, _nocSelectedWorkspaceOrTenantService;
    beforeEach(inject(function (_workingEnvironmentStatuses_, _uiGridFilterService_, nocSelectedWorkspaceOrTenantService) {
        magnifispec.mockTabsController('GalleryGridController', undefined, {});
        workingEnvironmentStatuses = _workingEnvironmentStatuses_;
        uiGridFilterService = _uiGridFilterService_;

        j.scope.workingEnvironments = [
            {
                _pathPrefix: '/vsa/',
                publicId: 1,
                name: 'we1',
                status: {status: 'on'},
                workingEnvironmentType: 'VSA',
                _typeAndProvider: 'VSA_AWS',
                _typeLongName: 'Cloud Volumes ONTAP',
                ontapClusterProperties: {ontapVersion: '8.3'}
            },
            {
                _pathPrefix: '/vsa/',
                publicId: 2,
                name: 'we2',
                status: {status: 'on'},
                workingEnvironmentType: 'VSA',
                _typeAndProvider: 'VSA_AWS',
                _typeLongName: 'Cloud Volumes ONTAP',
                ontapClusterProperties: {ontapVersion: '8.3'}
            }
        ];

        _nocSelectedWorkspaceOrTenantService = nocSelectedWorkspaceOrTenantService;
        jest.spyOn(_nocSelectedWorkspaceOrTenantService, 'getPublicId').mockReturnValue(1);

    }));

    describe('replicate from table', function () {
        it('should open the select working environment dialog with the right data', function () {
            magnifispec.fakeConfirmNew();
            j.scope.replicateFromTable(j.scope.workingEnvironments[0]);
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Select the working environment to which you want to replicate data', 'assets/templates/dialogs/select-target-working-environment-dialog.html',
                {
                    targetWorkingEnvironments: [{
                        _pathPrefix: '/vsa/',
                        publicId: 2,
                        name: 'we2',
                        status: {status: 'on'},
                        workingEnvironmentType: 'VSA',
                        _typeAndProvider: 'VSA_AWS',
                        _typeLongName: 'Cloud Volumes ONTAP',
                        ontapClusterProperties: {ontapVersion: '8.3'}
                    }],
                    targetWorkingEnvironmentsOptions: j.scope.targetWorkingEnvironmentsOptions,
                    target: {
                        _pathPrefix: '/vsa/',
                        publicId: 2,
                        name: 'we2',
                        status: {status: 'on'},
                        workingEnvironmentType: 'VSA',
                        _typeAndProvider: 'VSA_AWS',
                        _typeLongName: 'Cloud Volumes ONTAP',
                        ontapClusterProperties: {ontapVersion: '8.3'}
                    },
                    targetName: 'we2',
                    setTarget: j.scope.setTarget,
                    width: 520,
                    fromTop: '10%'
                }
            );
            expect(j.scope.target).toEqual({
                _pathPrefix: '/vsa/',
                publicId: 2,
                name: 'we2',
                status: {status: 'on'},
                workingEnvironmentType: 'VSA',
                _typeAndProvider: 'VSA_AWS',
                _typeLongName: 'Cloud Volumes ONTAP',
                ontapClusterProperties: {ontapVersion: '8.3'}
            });
            expect(j.scope.targetName).toEqual('we2');
        });

        it('should check if replication is allowed when a target working environment is selected', function () {
            magnifispec.fakeConfirmNew(true);
            jest.spyOn(workingEnvironmentStatuses, 'isReplicationAllowed').mockImplementation(() => {
            });
            j.scope.target = j.scope.workingEnvironments[1];
            j.scope.replicateFromTable(j.scope.workingEnvironments[0]);
            expect(workingEnvironmentStatuses.isReplicationAllowed).toHaveBeenCalledWith(j.scope.workingEnvironments[0], j.scope.workingEnvironments[1]);
        });

        it('should start the snap mirror wizard after selecing the target working environment', function () {
            magnifispec.fakeConfirmNew(true);
            j.scope.selectedTenant = {_pathPrefix: '/vsa/', publicId: 1};
            j.scope.checkAndSetState = jest.fn();
            jest.spyOn(workingEnvironmentStatuses, 'isReplicationAllowed').mockReturnValue(true);
            j.scope.target = j.scope.workingEnvironments[1];
            j.scope.replicateFromTable(j.scope.workingEnvironments[0]);
            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('snap-mirror-wizard', {
                from: 'main.working-environments.index',
                slTenantId: 1,
                sourceId: 1,
                targetId: 2,
                sourceType: undefined,
                sourceProvider: undefined,
                targetType: undefined,
                targetProvider: undefined
            });
        });

    });

    describe('the table', function () {
        beforeEach(function () {
            j.scope.gridOptions = {
                columnDefs: [
                    {visible: true, default: true, displayName: 'Name', field: 'name'},
                    {visible: false, default: false, displayName: 'Status', field: 'status'},
                    {visible: false, default: true, displayName: 'Working Environment Type', field: '_typeLongName'}
                ]
            };

            uiGridFilterService.gridOptions = j.scope.gridOptions;
            uiGridFilterService.scope = j.scope;

        });
    });

    describe('targetWorkingEnvironmentsMatcher', function () {
        it('should find the right region and call genericItemFilter', function () {
            jest.spyOn(j.nocUtil, 'genericItemFilter').mockImplementation(() => {
            });
            j.scope.workingEnvironments = [
                {name: 'we1', _typeLongName: 'NPS'},
                {name: 'we2', _typeLongName: 'On Premises', workingEnvironmentType: 'ON_PREM'}
            ];
            j.scope.targetWorkingEnvironmentsMatcher('searchPhrase', 'we2');
            expect(j.nocUtil.genericItemFilter).toHaveBeenCalledWith('searchPhrase', {
                    name: 'we2',
                    _typeLongName: 'On Premises',
                    workingEnvironmentType: 'ON_PREM',
                    _formattedRegion: ''
                },
                ['name', '_typeLongName', '_formattedRegion']);
        });

        it('should add formatted region field and search in it for no on_prem WEs', function () {
            jest.spyOn(j.nocUtil, 'genericItemFilter').mockImplementation(() => {
            });
            j.scope.workingEnvironments = [
                {name: 'we1', _typeLongName: 'NPS'},
                {
                    name: 'we2',
                    _typeLongName: 'Cloud Volumes ONTAP',
                    workingEnvironmentType: 'VSA',
                    regionName: 'eu-west-1'
                }
            ];
            j.scope.targetWorkingEnvironmentsMatcher('searchPhrase', 'we2');
            expect(j.nocUtil.genericItemFilter).toHaveBeenCalledWith('searchPhrase', {
                    name: 'we2',
                    _typeLongName: 'Cloud Volumes ONTAP',
                    workingEnvironmentType: 'VSA',
                    regionName: 'eu-west-1',
                    _formattedRegion: 'EU (Ireland)'
                },
                ['name', '_typeLongName', '_formattedRegion']);
        });
    });

    describe('formatTargetWorkingEnvironments', function () {
        beforeEach(function () {
            j.scope.workingEnvironments = [
                {
                    id: 'id1',
                    name: 'we1',
                    _typeLongName: 'Cloud Volumes ONTAP',
                    workingEnvironmentType: 'VSA',
                    regionName: 'eu-west-1'
                },
                {id: 'id2', name: 'we2', _typeLongName: 'On Premises', workingEnvironmentType: 'ON_PREM'}
            ];
        });

        it('should build the right html for VSA', function () {
            var item = {id: 'we1', text: 'we1'};
            expect(j.scope.formatTargetWorkingEnvironments(item)).toEqual(j.nocUtil.formatSelect2('we1', 'we1', 'Cloud Volumes ONTAP', 'EU (Ireland)'));
        });

        it('should build the right html for onPrem', function () {
            var item = {id: 'we2', text: 'we2'};
            expect(j.scope.formatTargetWorkingEnvironments(item)).toEqual(j.nocUtil.formatSelect2('we2', 'we2', 'On Premises', ''));
        });

        it('should return the item\'s text in case the connection is not found in the list', function () {
            var item = {id: 'id3', text: 'we3'};
            expect(j.scope.formatTargetWorkingEnvironments(item)).toEqual('we3');
        });
    });

});

