describe('SyncS3StatusMonitoringController', () => {
    'use strict';

    let j, _nocCurrentUserService;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    beforeEach(inject((configService) => {
        jest.spyOn(configService, 'getConfig').mockReturnValue(j.$q.when({
            cloudSyncConstants: {
                warningPeriod: 7,
                reminderPeriod: 12
            }
        }));
        magnifispec.mockTabsController('SyncS3StatusMonitoringController', {}, {
            updateSelectedWorkingEnvironment: jest.fn().mockReturnValue(j.$q.when()),
            selectedWorkingEnvironment: {
                publicId: '1',
                name: 'we1',
                cloudSyncProperties: {}
            }
        });
        magnifispec.processPromise();
    }));

    describe('getFreeTrialNotificationLevel', () => {

        beforeEach(inject((_nocCurrentUserService_) => {
            advanceTo(moment('2017-01-01').toDate());
            _nocCurrentUserService = _nocCurrentUserService_;
            _nocCurrentUserService.initializeCurrentUser({cloudSyncLicenseInformation: {cloudSyncLicenseTypes: ['trial']}});
        }));

        it('should return nothing if ownerUser is not available', () => {
            _.set(j.scope.$rootScope, 'currentUser', null);
            _nocCurrentUserService.initializeCurrentUser({});
            expect(j.scope.getFreeTrialNotificationLevel()).toBeFalsy();
        });

        it('should return nothing if current license is not trial', () => {
            _.set(j.$rootScope, 'currentUser.cloudSyncLicenseInformation', {
                cloudSyncLicenseTypes: ['byol'],
                licenseExpirationDate: moment('2017-01-02').toDate()
            });
            _nocCurrentUserService.initializeCurrentUser({
                cloudSyncLicenseInformation: {
                    cloudSyncLicenseTypes: ['byol'],
                    licenseExpirationDate: moment('2017-01-02').toDate()
                }
            });

            expect(j.scope.getFreeTrialNotificationLevel()).toBeFalsy();
        });

        it('should return nothing for period longer than reminderPeriod', () => {
            _.set(j.$rootScope, 'currentUser.cloudSyncLicenseInformation.licenseExpirationDate', moment('2017-01-27').toDate());
            _nocCurrentUserService.initializeCurrentUser({cloudSyncLicenseInformation: {licenseExpirationDate: moment('2017-01-27').toDate()}});

            expect(j.scope.getFreeTrialNotificationLevel()).toBeFalsy();
        });

        it('should return "warning" for period in the reminderPeriod', () => {
            _nocCurrentUserService.initializeCurrentUser({cloudSyncLicenseInformation: {licenseExpirationDate: moment('2017-01-12').toDate()}});
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseInformation').mockReturnValue({licenseExpirationDate: moment('2017-01-12').toDate()});
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseTypes').mockReturnValue(['trial']);
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseExpirationDate').mockReturnValue(moment('2017-01-12').toDate());

            expect(j.scope.getFreeTrialNotificationLevel()).toEqual('warning');
        });

        it('should return "error" for period of warningPeriod', () => {
            _nocCurrentUserService.initializeCurrentUser({cloudSyncLicenseInformation: {licenseExpirationDate: moment('2017-01-04').toDate()}});
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseInformation').mockReturnValue({licenseExpirationDate: moment('2017-01-04').toDate()});
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseTypes').mockReturnValue(['trial']);
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseExpirationDate').mockReturnValue(moment('2017-01-04').toDate());

            expect(j.scope.getFreeTrialNotificationLevel()).toEqual('error');
        });

        it('should return "error" for period shorter than warningPeriod', () => {
            _nocCurrentUserService.initializeCurrentUser({cloudSyncLicenseInformation: {licenseExpirationDate: moment('2017-01-02').toDate()}});
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseInformation').mockReturnValue({licenseExpirationDate: moment('2017-01-02').toDate()});
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseTypes').mockReturnValue(['trial']);
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserCloudSyncLicenseExpirationDate').mockReturnValue(moment('2017-01-02').toDate());

            expect(j.scope.getFreeTrialNotificationLevel()).toEqual('error');
        });
    });
});
