describe('VirginChooseWizardController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        j.$state.go = jest.fn();
        j.$rootScope.currentUser = {tenantId: '1'};
        jest.spyOn(j.ManifestMapper, 'initialize').mockReturnValue(j.$q.when());
        magnifispec.mockWizardStepController('VirginChooseWizardController', {});
        j.$httpBackend.expectGET('/occm/api/accounts').respond({});
        j.scope.wizardController.instancePlacement = {role: true, providerRoleType: 'AWS'};
    });

    it('should call BaseVirginStep and clear all data', () => {
        expect(j.BaseVirginStep.call).toHaveBeenCalled();
        expect(j.parentScope.clearAllStepData).toHaveBeenCalled();
    });

    describe('startWizard', () => {
        it('should fail on unexpected choices', () => {
            expect(() => {
                j.scope.startWizard('kuku');
            }).toThrowError('Wizard \'kuku\' is not supported. Should be one of VSA, VSA_AZURE, Recover, RecoverAzure, OnPrem, HA, HA_AZURE, SoftLayer');
        });

        it('should clear all the backed up data if the user changes a choice', () => {
            j.scope.choice = 'VSA';
            j.scope.startWizardWrapper('HA');
            j.$timeout.flush();
            expect(j.parentScope.clearAllStepData.mock.calls.length).toBe(2); //1 for the initialization and one for the case the choice has changed
        });

        it('should not clear all the data if the choice hasn\'t changed', () => {
            j.scope.choice = 'VSA';
            j.scope.startWizardWrapper('VSA');
            expect(j.parentScope.clearAllStepData.mock.calls.length).toBe(1); //1 for the initialization only.
        });

        describe('should do the right thing for', () => {
            const createNew = (type) => {
                it(type + ' when manifest is valid', () => {
                    j.scope.startWizardWrapper(type);
                    magnifispec.processPromise(); //Initiate .then() of ManifestMapper.initialize

                    expect(j.scope.choice).toBe(type);
                    expect(j.scope.wizardController.workingEnvironmentType).toBe(type);
                    expect(j.scope.wizardController.title).toBeTruthy();
                });

                it(type + ' when manifest is invalid', () => {
                    j.ManifestMapper.initialize = jest.fn().mockReturnValue(j.$q.reject('Error Message'));
                    j.scope.startWizard(type);
                    magnifispec.processPromise(); //Initiate .catch() of ManifestMapper.initialize

                    expect(j.pidgeon.error).toHaveBeenCalledWith('An error occurred while retrieving manifests. Cannot create a new Cloud Volumes ONTAP. You can still discover an existing working environment', null, 'manifest');
                });
            };

            createNew('VSA');
            createNew('HA');

            it('Recover', () => {
                j.scope.startWizard('Recover');

                expect(j.scope.wizardController.workingEnvironmentType).toEqual('VSA');
                expect(j.$state.go).toHaveBeenCalledWith('wizard.init.choose.region');
            });

            it('OnPrem', () => {
                j.scope.startWizard('OnPrem');

                expect(j.scope.wizardController.workingEnvironmentType).toEqual('ON_PREM');
                expect(j.$state.go).toHaveBeenCalledWith('wizard.init.choose.cluster');
            });
        });
    });

    describe('entering the wizard in recreate mode', () => {
        it('should call getCreateRequestParameters', () => {
            j.$state.params.type = 'VSA';
            jest.spyOn(j.scope, 'getCreateRequestParameters').mockImplementation(() => {});

            j.scope.initialize();

            expect(j.scope.getCreateRequestParameters).toHaveBeenCalled();
        });

        describe('getCreateRequestParameters', () => {
            it('should retrieve parameters used in create working environment request', () => {
                j.$state.params.pathPrefix = '/vsa/';
                j.scope.wizardController.recreateId = 'id1';
                j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/id1/create-request-parameters').respond(200, {parameters: 'abc'});
                jest.spyOn(j.scope, 'startWizard').mockImplementation(() => {});

                j.scope.getCreateRequestParameters();
                j.$httpBackend.flush();

                expect(j.scope.wizardController.failedRequest).toEqual('abc');
                expect(j.scope.startWizard).toHaveBeenCalled();
            });

            it('should fail in case of error', () => {
                j.$state.params.pathPrefix = '/vsa/';
                j.scope.wizardController.recreateId = 'id1';
                j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/id1/create-request-parameters').respond(400, j.errorResponse);

                j.scope.getCreateRequestParameters();
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });
        });
    });

    it('should call clear all step data upon initialization', () => {
        expect(j.parentScope.clearAllStepData).toHaveBeenCalled();
    });
});
