describe('virgin wizard controller', () => {
    const mockedLayoutOptions = {contentUrl: '/content.html'};
    const mockedExitOptions = {exit: 'some exit function'};
    const mockedInitOptions = {firstStep: 'the first step'};
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        j.$state.go = jest.fn();
        magnifispec.mockController('VirginWizardController', {layoutOptions: mockedLayoutOptions, exitOptions: mockedExitOptions, initOptions: mockedInitOptions});
        j.scope.virginWizardForm = {$error: {}};
    });

    it('should extend the scope', () => {
        expect(j.scope.exit).toBeDefined();
        expect(j.scope.firstStep).toBeDefined();
    });

    describe('backupStepData', () => {
        it('should back up the data', () => {
            const data = {value1: 'data1', value2: 'data2'};
            j.scope.backupStepData('step1', data);
            expect(j.scope.wizardController.backupData.step1).toEqual(data);
        });
    });

    describe('clearAllStepData', () => {
        it('should clear up all the data', () => {
            j.scope.clearAllStepData();
            expect(j.scope.wizardController.backupData).toEqual({});
        });
    });

    describe('getStepData', () => {
        it('should return step data', () => {
            j.scope.wizardController.backupData.step1 = 'very important data';
            expect(j.scope.getStepData('step1')).toEqual('very important data');
        });
    });

    describe('clearStepData', () => {
        it('should return step data', () => {
            j.scope.wizardController.backupData.step1 = 'knock knock knocking';
            j.scope.wizardController.backupData.step2 = 'on heavens door';
            j.scope.clearStepData('step1');
            j.$timeout.flush();
            expect(j.scope.wizardController.backupData).toEqual({step1: {}, step2: 'on heavens door'});
        });
    });

    describe('initialization', () => {
        it('should set the state to the first step', () => {
            expect(j.$state.go).toHaveBeenCalledWith('the first step');
        });
    });

    it('should listen to state changes and set the current state name on the scope', () => {
        j.$state.current.data = {name: 'thisStep'};
        j.scope.$broadcast('$stateChangeSuccess');
        expect(j.scope.currentStateName).toEqual('thisStep');
    });

    describe('Is initial step', () => {
        it('should identify if a state is the initial step in case no init step defined', () => {
            j.$state.current.name = 'state1';
            expect(j.scope.isInitialStep()).toEqual(true);
            j.$state.current.name = 'state1.state2';
            expect(j.scope.isInitialStep()).toEqual(true);
            j.$state.current.name = 'state1.state2.state3';
            expect(j.scope.isInitialStep()).toEqual(false);
        });

        it('should identify if a state is the initial step in case init step defined', () => {
            j.scope.hasInitStep = true;
            j.$state.current.name = 'state1';
            expect(j.scope.isInitialStep()).toEqual(true);
            j.$state.current.name = 'state1.state2';
            expect(j.scope.isInitialStep()).toEqual(true);
            j.$state.current.name = 'state1.state2.state3';
            expect(j.scope.isInitialStep()).toEqual(true);
            j.$state.current.name = 'state1.state2.state3.state4';
            expect(j.scope.isInitialStep()).toEqual(false);
        });
    });

    describe('Is valid', () => {
        it('should return true if the connectionsForm is valid', () => {
            j.scope.VirginWizardForm = {$valid: true};
            expect(j.scope.isValid()).toEqual(true);
        });

        it('should return false if the connectionsForm is invalid', () => {
            j.scope.VirginWizardForm = {$valid: false};
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe('back', () => {
        beforeEach(() => {
            jest.spyOn(j.$state, 'transitionTo').mockImplementation(() => {});
        });

        it('should go back if it is not the initial step', () => {
            j.$state.current.name = 'state1.state2.state3';
            j.scope.back();
            expect(j.$state.transitionTo).toHaveBeenCalledWith('^', {}, {location: 'replace', inherit: true, relative: j.$state.$current, notify: true});
        });

        it('should not go back if it is the initial step', () => {
            j.$state.current.name = 'state1';
            j.scope.back();
            expect(j.$state.transitionTo).not.toHaveBeenCalled();
        });
    });

    describe('canLeave', () => {
        let proceedFunction;
        beforeEach(() => {
            proceedFunction = jest.fn();
        });

        describe('should call window confirm', () => {
            beforeEach(() => {
                j.scope.wizardController.wizardCompletedSucssesfully = false;
                jest.spyOn(j.scope, 'isInitialStep').mockReturnValue(false);
            });

            it('and proceed if confirmed', () => {
                magnifispec.mockConfirm(true);

                j.scope.canLeave(proceedFunction);
                magnifispec.processPromise();

                expect(j.nocDialog.confirm).toHaveBeenCalled();
                j.$timeout.flush(249);
                expect(j.scope.canceled).toEqual(false);
                expect(proceedFunction).not.toHaveBeenCalled();
                j.$timeout.flush(1);
                expect(j.scope.canceled).toEqual(true);
                expect(proceedFunction).toHaveBeenCalled();
            });

            it('and do nothing if not confirmed', () => {
                magnifispec.mockConfirm(false);

                j.scope.canLeave(proceedFunction);
                magnifispec.processPromise();

                expect(j.nocDialog.confirm).toHaveBeenCalled();
                expect(proceedFunction).not.toHaveBeenCalled();
                expect(j.scope.canceled).toEqual(false);
            });
        });

        describe('should call the proceed function directly', () => {
            beforeEach(() => {
                magnifispec.mockConfirm(true);
            });

            it('in the first step', () => {
                j.scope.wizardController.wizardCompletedSucssesfully = false;
                jest.spyOn(j.scope, 'isInitialStep').mockReturnValue(true);
                j.scope.canLeave(proceedFunction);
            });

            it('should call the proceed function directly in the first step', () => {
                j.scope.wizardController.wizardCompletedSucssesfully = true;
                jest.spyOn(j.scope, 'isInitialStep').mockReturnValue(false);
                j.scope.canLeave(proceedFunction);
            });

            afterEach(() => {
                magnifispec.processPromise();
                expect(j.nocDialog.confirm).not.toHaveBeenCalled();
                expect(proceedFunction).toHaveBeenCalled();
                expect(j.scope.canceled).toEqual(true);
            });
        });
    });

    describe('next', () => {
        beforeEach(() => {
            j.$state.go = jest.fn();
        });

        it('should continue if input is valid', () => {
            jest.spyOn(j.scope, 'isValid').mockReturnValue(true);
            j.scope.next('targetString');
            expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
            expect(j.$state.go).toHaveBeenCalledWith('targetString');
        });

        it('should not continue if input is invalid', () => {
            jest.spyOn(j.scope, 'isValid').mockReturnValue(false);
            j.scope.next('targetString');
            expect(j.$state.go).not.toHaveBeenCalled();
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
        });
    });

    it('should call handleNavigate away on navigate away request with the right parameters', () => {
        const payload = {};
        jest.spyOn(j.nocNavigateAway, 'handleNavigateAway').mockImplementation(() => {});
        jest.spyOn(j.scope, 'canLeave').mockImplementation(() => {});
        j.scope.$emit(j.nocNavigateAway.eventName, payload);
        expect(j.nocNavigateAway.handleNavigateAway).toHaveBeenCalledWith(payload, j.scope.canLeave);
    });
});
