describe('CloudSyncActivationDataBrokerStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let nocCloudSyncService;
    let getKeyPairsSpy, getRegionsSpy;

    beforeEach(inject((_nocCloudSyncService_, _MetadataResource_) => {
        nocCloudSyncService = _nocCloudSyncService_;
        getKeyPairsSpy = jest.spyOn(_nocCloudSyncService_, 'getKetPairsByWorkingEnvironment').mockReturnValue(j.$q.when(['keyPair1', 'keyPair2']));
        getRegionsSpy = jest.spyOn(_MetadataResource_, 'getRegions').mockReturnValue(j.$q.when([]));
        j.$rootScope.instancePlacement = {};
        j.$state.params = {type: 'VSA'};

        magnifispec.mockWizardStepController('CloudSyncActivationDataBrokerStepController', {}, {
            exit: jest.fn(),
            workingEnvironmentId: 'weId1',
            selectedVolumes: ['vol1', 'vol2'],
            s3Location: 'bucket1/subdirectory1'
        });
        magnifispec.processPromise();
    }));

    describe('getKeyPairs', () => {
        beforeEach(() => {
            j.scope.wizardController.selectedRegion = {code: 'ab-west-1'};
            j.scope.wizardController.type = 'VSA';
        });

        it('should call MetadataResource correctly and update the scope with key pairs', () => {
            j.scope.getKeyPairs();
            magnifispec.processPromise();

            expect(getKeyPairsSpy).toHaveBeenCalledWith(jasmine.any(Object), 'weId1');
            expect(j.scope.keyPairs).toEqual(['keyPair1', 'keyPair2']);
        });

        it('should not change the selection of existing keyPair', () => {
            j.scope.wizardController.keyPair = 'keyPair3';

            j.scope.getKeyPairs();
            magnifispec.processPromise();

            expect(j.scope.wizardController.keyPair).toEqual('keyPair3');
        });
    });

    describe('getRegions', () => {
        it('should get the regions and update the scope', () => {
            getRegionsSpy.mockReturnValue(j.$q.when([
                {code: 'region1', location: 'aza', name: 'gaza'},
                {code: 'region2', location: 'gazaza', name: 'gziza'}
            ]));

            j.scope.getRegions();
            magnifispec.processPromise();

            expect(j.scope.regions).toEqual([
                {code: 'region1', location: 'aza', name: 'gaza'},
                {code: 'region2', location: 'gazaza', name: 'gziza'}
            ]);
            expect(getRegionsSpy).toHaveBeenCalledWith(jasmine.any(Object), '/vsa/');
        });
    });

    describe('initialize', () => {
        beforeEach(() => {
            j.$rootScope.instancePlacement = {
                vpcId: 'vpc-1',
                subnetId: 'subnet-2',
                keyPair: 'keyPair1'
            };
            jest.spyOn(j.scope, 'handleRegionChange').mockReturnValue(j.$q.when());
            j.scope.vpcsWithSubnets = [
                {vpcId: 'vpc-1', subnets: [{subnetId: 'subnet-2'}, {subnetId: 'subnet-3'}]},
                {vpcId: 'vpc-33', subnets: [{subnetId: 'subnet-55'}, {subnetId: 'subnet-44'}]},
                {vpcId: 'vpc-4'}
            ];
        });

        it('should prefer params supplied by the working environment to the wizard', () => {
            j.scope.wizardController.vpcId = 'vpc-33';
            j.scope.wizardController.subnetId = 'subnet-44';
            j.scope.wizardController.keyPairName = 'keyPair-55';

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.wizardController.selectedVpc).toEqual({vpcId: 'vpc-33', subnets: [{subnetId: 'subnet-55'}, {subnetId: 'subnet-44'}]});
            expect(j.scope.wizardController.selectedSubnet).toEqual({subnetId: 'subnet-44'});
            expect(j.scope.wizardController.keyPair).toEqual('keyPair-55');
        });

        it('should fallback to instance placement info if not working environment info supplied', () => {
            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.wizardController.selectedVpc).toEqual({vpcId: 'vpc-1', subnets: [{subnetId: 'subnet-2'}, {subnetId: 'subnet-3'}]});
            expect(j.scope.wizardController.selectedSubnet).toEqual({subnetId: 'subnet-2'});
            expect(j.scope.wizardController.keyPair).toEqual('keyPair1');
        });
    });

    describe('finish', () => {
        beforeEach(() => {
            j.scope.isValid = jest.fn();
        });

        it('should call sync service with appropriate parameters', () => {
            j.scope.isValid.mockReturnValue(true);
            jest.spyOn(nocCloudSyncService, 'sync').mockReturnValue(j.$q.when());
            j.scope.wizardController.selectedRegion = {code: 'ab-west-1'};
            j.scope.wizardController.selectedVpc = {vpcId: 'vpc1'};
            j.scope.wizardController.selectedSubnet = {subnetId: 'subnet1'};
            j.scope.wizardController.keyPair = 'keyPair1';

            j.scope.finish();
            magnifispec.processPromise();

            expect(nocCloudSyncService.sync).toHaveBeenCalledWith(jasmine.any(Object), 'weId1', {
                volumeNames: ['vol1', 'vol2'],
                s3Location: 's3://bucket1/subdirectory1',
                dataBrokerNetworkInformation: {
                    region: 'ab-west-1',
                    vpcId: 'vpc1',
                    subnetId: 'subnet1',
                    keyPair: 'keyPair1'
                }
            }, 'VSA');
            expect(j.scope.wizardController.wizardCompletedSucssesfully).toBeTruthy();
            expect(j.scope.wizardController.exit).toHaveBeenCalled();
        });

        it('should handle errors for invalid form', () => {
            j.scope.isValid.mockReturnValue(false);

            j.scope.finish();

            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
        });
    });
});
