describe('CreateNewAggregateDetailsStepController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let getWorkingEnvironmentByPublicIdSpy;

    beforeEach(() => {
        getWorkingEnvironmentByPublicIdSpy = jest.spyOn(j.workingEnvironmentResource, 'getWorkingEnvironmentByPublicId').mockReturnValue(j.$q.when({
            name: 'we1',
            publicId: 1
        }));

        magnifispec.mockWizardStepController('CreateNewAggregateDetailsStepController', {}, {});
        magnifispec.processPromise();
    });

    describe('getSelectedWorkingEnvironment', () => {
        it('should get the correct working environment from the server', () => {
            j.nocUI.loading.mockClear();
            j.nocUI.unloading.mockClear();
            getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when('we'));
            j.$state.params.type = 'VSA';
            j.$state.params.provider = 'AWS';

            expect(j.scope.getSelectedWorkingEnvironment('id1')).toBeResolved();

            expect(j.scope.wizardController.selectedWorkingEnvironment).toEqual('we');
            expect(getWorkingEnvironmentByPublicIdSpy).toHaveBeenCalledWith(jasmine.any(Object), 'id1', '/vsa/', 'ontapClusterProperties,capacityFeatures');
            expect(j.nocUI.loading.mock.calls.length).toEqual(1);
            expect(j.nocUI.unloading.mock.calls.length).toEqual(1);
        });
    });

    describe('initialize', () => {
        it('should initialize correctly', () => {
            j.scope.wizardController.title = 'base wizard title';

            j.scope.initializeController();

            expect(j.scope.wizardController.addAggregateRequest).toEqual({
                workingEnvironmentId: 1
            });
            expect(j.scope.wizardController.title).toEqual('base wizard title in we1');
        });

        it('should initialize correctly for HA', () => {
            j.scope.wizardController.selectedWorkingEnvironment = {
                isHA: true,
                publicId: 1,
                ontapClusterProperties: {
                    nodes: [{name: 'node-1'}, {name: 'node-2'}]
                }
            };

            j.scope.initializeController();

            expect(j.scope.wizardController.addAggregateRequest).toEqual({
                workingEnvironmentId: 1,
                homeNode: 'node-1'
            });
        });
    });
});
