describe('CreateNewVolumeUsageProfileStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        magnifispec.mockWizardStepController('CreateNewVolumeUsageProfileStepController', {}, {
            setWizardStep: jest.fn(),
            selectedTier: {name: 'gp2', diskName: 'gp2'},
            selectedWorkingEnvironment: {
                publicId: 1,
                capacityFeatures: {
                    supportedCapacityTiers: {
                        supportedCapacityTiersPerVolumeType: [
                            {availableTieringPolicies: [], supportedCapacityTiers: [], volumeType: 'standard'},
                            {
                                availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                                supportedCapacityTiers: ['S3'],
                                volumeType: 'io1'
                            },
                            {
                                availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                                supportedCapacityTiers: ['S3'],
                                volumeType: 'gp2'
                            },
                            {
                                availableTieringPolicies: ['none', 'snapshot_only', 'auto'],
                                supportedCapacityTiers: ['S3'],
                                volumeType: 'st1'
                            },
                            {availableTieringPolicies: [], supportedCapacityTiers: [], volumeType: 'sc1'}
                        ]
                    }
                },
                name: 'we1',
                _providerVolumeTypes: {
                    'gp2': {
                        name: 'gp2',
                        shortName: 'gp2',
                        diskName: 'gp2',
                        display: 'General Purpose SSD (gp2)',
                        description: 'General purpose SSD volume that balances price and performance.',
                        supportedInHa: true,
                        'default': true,
                        supportedSizes: [{maxDisksAllow: {numOfDisks: 4}}]
                    },
                    'gp2+S2': {
                        name: 'gp2+S2',
                        shortName: 'gp2+S2',
                        diskName: 'gp2',
                        display: 'General Purpose SSD (gp2) + S3',
                        description: 'General purpose SSD volume that balances price and performance. + S3',
                        supportedInHa: true,
                        supportedSizes: [{maxDisksAllow: {numOfDisks: 6}}]
                    }
                }
            }
        });
    });

    describe('initialize controller', () => {
        it('should initialize possibleTiers', () => {
            expect(j.scope.possibleTiers).toEqual([
                {
                    name: 'gp2',
                    shortName: 'gp2',
                    diskName: 'gp2',
                    display: 'General Purpose SSD (gp2)',
                    description: 'General purpose SSD volume that balances price and performance.',
                    supportedInHa: true,
                    'default': true,
                    supportedSizes: [{maxDisksAllow: {numOfDisks: 4}}]
                }, {
                    name: 'gp2+S2',
                    shortName: 'gp2+S2',
                    diskName: 'gp2',
                    display: 'General Purpose SSD (gp2) + S3',
                    description: 'General purpose SSD volume that balances price and performance. + S3',
                    supportedInHa: true,
                    supportedSizes: [{maxDisksAllow: {numOfDisks: 6}}]
                }
            ]);
        });
    });

    describe('usageProfileChanged', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'tierSelected').mockImplementation(() => {});
        });

        it('should change selectedTier if it is now disabled, and "No Storage Efficiency" was selected', () => {
            j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
            j.scope.wizardController.tieringEnabled = true;

            j.scope.usageProfileChanged({display: 'No Storage Efficiency'});

            expect(j.scope.tierSelected).toHaveBeenCalledWith(jasmine.objectContaining({name: 'gp2'}));
            expect(j.scope.wizardController.tieringEnabled).toEqual(false);
        });

        it('should do nothing for selection which is not "No Storage Efficiency"', () => {
            j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
            j.scope.wizardController.tieringEnabled = true;

            j.scope.usageProfileChanged({display: 'Storage Efficiency'});

            expect(j.scope.tierSelected).not.toHaveBeenCalled();
            expect(j.scope.wizardController.tieringEnabled).toEqual(true);
        });
    });

    describe('disabledTooltipForTier', () => {
        it('should return tier disabledMessage if exists', () => {
            _.set(j.scope, 'wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning', true);

            expect(j.scope.disabledTooltipForTier({name: 'gp2', disabledMessage: 'reason1', supportedCapacityTiers: ['S3']})).toEqual('reason1');
        });

        it('should return tooltip in case selected disk type is tiered AND selected usage profile is "No Storage Efficiency"', () => {
            _.set(j.scope, 'wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning', false);

            expect(j.scope.disabledTooltipForTier({name: 'gp2', supportedCapacityTiers: ['S3']}))
                .toEqual('If you want to use storage tiers, you must enable the Storage Efficiency usage profile.');
        });

        it('should return correct message in case selected disk type is not tiered', () => {
            _.set(j.scope, 'wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning', true);
            j.scope.selectedWorkingEnvironment._isAWS = true;

            expect(j.scope.disabledTooltipForTier({name: 'io1'})).toEqual('S3 tiering is not available for this disk type.');
        });

        it('should return correct message in case selected disk type is not tiered', () => {
            _.set(j.scope, 'wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning', true);
            j.scope.selectedWorkingEnvironment._isAWS = false;

            expect(j.scope.disabledTooltipForTier({name: 'Premium_LRS'})).toEqual('Blob tiering is not available for this disk type.');
        });

        it('should return undefined in case selected usage profile is not "No Storage Efficiency"', () => {
            _.set(j.scope, 'wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning', true);

            expect(j.scope.disabledTooltipForTier({name: 'gp2', supportedCapacityTiers: ['S3']})).not.toBeDefined();
        });
    });

    describe('tierSelected', () => {
        it('should set the tier and selectedDiskTypeDisplay', () => {
            jest.spyOn(j.scope, 'disabledTooltipForTier').mockReturnValue(undefined);
            j.scope.wizardController.path = {usageProfile: 'path.to.usageProfile.step'};
            const tier = {name: 'gp2', diskName: 'gp2'};

            j.scope.tierSelected(tier);

            expect(j.scope.wizardController.selectedTier).toEqual(tier);
            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.usageProfile.step');
            expect(j.scope.wizardController.selectedDiskTypeDisplay).toEqual('General Purpose SSD (gp2)');
        });

        it('should set selectedDiskTypeDisplay according to diskName, not tier itself', () => {
            jest.spyOn(j.scope, 'disabledTooltipForTier').mockReturnValue(undefined);
            j.scope.wizardController.path = {usageProfile: 'path.to.usageProfile.step'};
            j.scope.tierSelected({name: 'gp2', diskName: 'gp2'});

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.usageProfile.step');
            expect(j.scope.wizardController.selectedDiskTypeDisplay).toEqual('General Purpose SSD (gp2)');
        });
    });

    describe('save add volume', () => {
        beforeEach(() => {
            j.scope.createVolumeOnVsa = jest.fn();
        });

        it('should validate the form', () => {
            j.scope.isValid = jest.fn().mockReturnValue(false);

            j.scope.saveAddVolume();

            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
            expect(j.scope.createVolumeOnVsa).not.toHaveBeenCalled();
        });

        it('should create the volume', () => {
            j.scope.wizardController.selectedTier = {name: 'gp2+S3', diskName: 'gp2'};
            j.scope.isValid = jest.fn().mockReturnValue(true);

            j.scope.saveAddVolume();

            expect(j.scope.createVolumeOnVsa).toHaveBeenCalled();
        });
    });
});
