describe("Lifs Step Controller", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(function () {
        j.$state.go = jest.fn();

        magnifispec.mockWizardStepController("SnapMirrorLifsController", {config: {value: "value2"}}, {
            selectedLifsAddresses: {},
            currentStateName: "value1Lifs",
            currentStepPath: "some.path.to.this.step",
            lifs: {value1Lifs: [
                {name: "cluster01-01_icl01", address: "12.125.14.1", status: "up", port: "8080", node: "e0e"},
                {name: "cluster01-01_icl02", address: "12.125.14.2", status: "up", port: "8080", node: "e0f"}
            ]}
        });
    });

    describe("initialize", function () {
        it("should call staticFeedback", function () {
            expect(j.scope.staticFeedback).toHaveBeenCalledWith("assets/templates/wizards/virgin-snap-mirror/snapMirrorStaticFeedback.html");
        });

        it("should initialize selectedLifsAddresses", function () {
            expect(j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState]).toBeDefined();
            expect(j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState].length).toEqual(0);
        });

        it("should get lifsState right", function () {
            expect(j.scope.lifsState).toEqual("value1Lifs");
        });

        it("should save the path to this step", function () {
            expect(j.scope.pathToThisStep).toEqual("some.path.to.this.step");
        });

        it("should make sure all the lifs are unselected", function () {
            expect(j.scope.lifs["value1Lifs"][0].selected).toEqual(false);
            expect(j.scope.lifs["value1Lifs"][1].selected).toEqual(false);
        });
    });

    describe("lifClicked", function () {
        it("should do it's thing when selecting a volume", function () {
            j.scope.lifClicked(j.scope.lifs[j.scope.lifsState][0]);
            expect(j.scope.lifs[j.scope.lifsState][0].selected).toEqual(true);
            expect(j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState]).toEqual(["12.125.14.1"]);
        });

        it("should do it's thing when de-selecting a volume", function () {
            j.scope.lifClicked(j.scope.lifs[j.scope.lifsState][0]);
            j.scope.lifClicked(j.scope.lifs[j.scope.lifsState][0]);
            expect(j.scope.lifs[j.scope.lifsState][0].selected).toEqual(false);
            expect(j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState]).toEqual([]);
        });

        it("should set the wizard back to current step when emptying the list when scrolling up from further step", function () {
            j.scope.wizardController.setWizardStep = jest.fn();
            j.scope.wizardController.currentStateName = "advanced-step";
            j.scope.lifClicked(j.scope.lifs[j.scope.lifsState][0]);
            j.scope.lifClicked(j.scope.lifs[j.scope.lifsState][0]);
            expect(j.scope.lifs[j.scope.lifsState][0].selected).toEqual(false);
            expect(j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState]).toEqual([]);
            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith("some.path.to.this.step");
        });
    });

    describe("next", function () {
        describe("Should move to next step if Lifs were selected", function () {
            it("should go to the next step according to nextStep map: case 'source'", function () {
                j.scope.lifsState = "sourceLifs";
                j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState] = ["ip1"]; //Selecting 1 lif
                j.scope.wizardController.nextStep = {sourceLifs: ".nextStepAfterSourceLifs"};
                j.scope.next();
                expect(j.$state.go).toHaveBeenCalledWith(".nextStepAfterSourceLifs");
            });

            it("should go to the next step according to nextStep map: case 'destination", function () {
                j.scope.lifsState = "destinationLifs";
                j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState] = ["ip1"]; //Selecting 1 lif
                j.scope.wizardController.nextStep = {destinationLifs: ".nextStepAfterDestinationLifs"};
                j.scope.next();
                expect(j.$state.go).toHaveBeenCalledWith(".nextStepAfterDestinationLifs");
            });

        });

        it("Should not move to next step if lifs were not selected", function () {
            j.nocDialog.alert = jest.fn();
            j.scope.wizardController.selectedLifsAddresses[j.scope.lifsState] = []; //None lifs was selected.
            j.scope.next();
            expect(j.$state.go).not.toHaveBeenCalledWith(".volumes");
            expect(j.nocDialog.alert).toHaveBeenCalledWith("You must select at least one LIF before you can proceed.");
        });
    });
});