describe('SnapMirrorVolumesController', () => {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    const volumesResponse = [
        {"name": "csg_ag_quorum", "svmName": "sv5-csg-01", "size": {"size": 5.0, "unit": "GB"}, "usedSize": {"size": 6.3323974609375E-4, "unit": "GB"}, "junctionPath": "/csg_ag_quorum", "compressionSpaceSaved": {"size": 0.0, "unit": "GB"}, "deduplicationSpaceSaved": {"size": 0.0, "unit": "GB"}, "thinProvisioning": false, "snapshotPolicy": "default", "parentVolumeName": "", "state": "online"},
        {"name": "sdw_test", "svmName": "sv5-csg-01", "size": {"size": 5.0, "unit": "GB"}, "usedSize": {"size": 1.41143798828125E-4, "unit": "GB"}, "junctionPath": "", "compressionSpaceSaved": {"size": 0.0, "unit": "GB"}, "deduplicationSpaceSaved": {"size": 0.0, "unit": "GB"}, "thinProvisioning": false, "snapshotPolicy": "none", "parentVolumeName": "", "state": "online"},
        {"name": "shuki", "svmName": "sv5-csg-02", "size": {"size": 0.029296875, "unit": "GB"}, "usedSize": {"size": 1.4495849609375E-4, "unit": "GB"}, "junctionPath": "/shuki", "compressionSpaceSaved": {"size": 0.0, "unit": "GB"}, "deduplicationSpaceSaved": {"size": 0.0, "unit": "GB"}, "thinProvisioning": false, "snapshotPolicy": "default", "parentVolumeName": "", "state": "online"}
    ];

    beforeEach(inject((volumesResource) => {
        jest.spyOn(volumesResource, 'getVolumes').mockReturnValue(j.$q.when(volumesResponse));

        magnifispec.mockWizardStepController("SnapMirrorVolumesController", {}, {
            sourceWorkingEnvironment: {
                name: "value1",
                publicId: "value2",
                workingEnvironmentType: "VSA",
                _pathPrefix: "/vsa/"
            }
        });
        magnifispec.processPromise();
    }));

    describe('initialize', function () {
        it('should get the volumes', function () {
            expect(j.scope.volumes.length).toEqual(3);
            expect(j.scope.svms).toEqual([
                    {name: 'All SVMs', value: ''},
                    {name: 'sv5-csg-01', value: 'sv5-csg-01'},
                    {name: 'sv5-csg-02', value: 'sv5-csg-02'}
                ]
            );
        });
    });

    describe('volumeSelected', () => {
        beforeEach(() => {
            j.scope.setWizardStep = jest.fn();
            j.scope.wizardController.path = {'volumes': 'path.to.volumes.step'};
            jest.spyOn(j.scope, 'next').mockImplementation(() => {});
        });

        it('Should select the volume', () => {
            const vol = volumesResponse[0];

            j.scope.volumeSelected(vol);

            expect(j.scope.wizardController.selectedVolume).toEqual(vol);
            expect(j.scope.wizardController.targetVolumeName).toEqual(vol.name + '_copy');
            j.$timeout.flush();
            expect(j.scope.next).toHaveBeenCalled();
        });

        it('should set the wizard step to volumes if necessary', () => {
            const vol = volumesResponse[0];

            j.scope.volumeSelected(vol);

            expect(j.scope.setWizardStep).toHaveBeenCalledWith('path.to.volumes.step');
        });
    });

    describe("next", function () {
        beforeEach(function () {
            j.$state.go = jest.fn();
        });

        it('Should move to next step if volume was selected', () => {
            j.scope.next();
            expect(j.$state.go).toHaveBeenCalledWith('.destinationVolume');
        });

        afterEach(function () {
            expect(j.scope.feedback).toHaveBeenCalledWith("assets/templates/wizards/virgin-snap-mirror/volumes-feedback.html");
        });
    });
});