describe('VirginCreateWorkingEnvironmentNssStepController', function () {
    'use strict';

    let j, _nocSelectedWorkspaceOrTenantService;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    beforeEach(inject((nocSelectedWorkspaceOrTenantService) => {
        j.$rootScope.currentUser = {roleId: 'Role-1'};
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentNssStepController', {}, {selectedRegion: 'us-west-1'});
        j.$state.go = jest.fn();
        j.scope.selectedTenant = {publicId: 1};

        _nocSelectedWorkspaceOrTenantService = nocSelectedWorkspaceOrTenantService;
        jest.spyOn(_nocSelectedWorkspaceOrTenantService, 'getPublicId').mockReturnValue(1);
    }));

    describe('initialize', function () {
        it('should clear the pidgeon for invalid NSS credentials', function () {
            j.scope.initialize();
            expect(j.pidgeon.clear).toHaveBeenCalledWith('nss-invalid-reason');
        });

        it('should pidgeon nss invalid error if exists', function () {
            j.scope.wizardController.nssInvalidReason = 'Something is wrong with your NSS credentials';
            j.scope.initialize();
            expect(j.pidgeon.error).toHaveBeenCalledWith('Something is wrong with your NSS credentials', null, 'nss-invalid-reason');
        });
    });

    describe('next', function () {
        it('should check validity', function () {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            jest.spyOn(j.scope, 'saveChangeNssOfTenant').mockImplementation(() => {
            });
            j.scope.next();
            expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
            expect(j.pidgeon.clear).toHaveBeenCalledWith('saving NSS credentials');
            expect(j.scope.saveChangeNssOfTenant).toHaveBeenCalled();
        });

        it('should check invalidity', function () {
            j.scope.isValid = jest.fn().mockReturnValue(false);
            j.scope.next();
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
        });
    });

    describe('saveChangeNssOfTenant', function () {
        it('should go to tiering step if no nss credentials were entered', function () {
            j.scope.wizardController = {nssUserName: '', nssPassword: '', selectedPackage: {name: 'aws_custom'}};
            j.scope.saveChangeNssOfTenant();
            expect(j.$state.go).toHaveBeenCalledWith('.tiering');
        });

        it('should go to volume step if no nss credentials were entered (package flow)', () => {
            j.scope.wizardController = {nssUserName: '', nssPassword: '', selectedPackage: {name: 'aws_standard'}};
            j.scope.saveChangeNssOfTenant();
            expect(j.$state.go).toHaveBeenCalledWith('.volume');
        });

        describe('when calling api', function () {
            beforeEach(() => {
                j.scope.wizardController = {
                    nssUserName: 'newName',
                    nssPassword: 'newPassword',
                    selectedPackage: {name: 'aws_custom'}
                };
            });

            it('should work upon success', function () {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/tenants/1/nss-keys', {
                    nssUserName: 'newName',
                    nssPassword: 'newPassword'
                }).respond(200);
                j.scope.saveChangeNssOfTenant();
                j.$httpBackend.flush();
                expect(j.pidgeon.clear).toHaveBeenCalledWith('nss-invalid-reason');
                expect(j.scope.selectedTenant.nssUserName).toEqual('newName');
                expect(j.$state.go).toHaveBeenCalledWith('.tiering');
            });

            it('should work upon error', function () {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/tenants/1/nss-keys').respond(400, j.errorResponse);
                j.scope.saveChangeNssOfTenant();
                j.$httpBackend.flush();
                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            afterEach(function () {
                expect(j.nocUI.loading).toHaveBeenCalled();
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });
        });
    });
});