describe('VirginCreateWorkingEnvironmentReviewStepController', () => {
    'use strict';

    let j, _nocSelectedWorkspaceOrTenantService;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    beforeEach(inject((nocSelectedWorkspaceOrTenantService) => {
        j.$rootScope.isFeatureOn = jest.fn().mockReturnValue(true);
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentReviewStepController', {}, {
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS', combined: 'VSA_AWS'}
        });
        j.scope.wizardController.selectedOntapVersion = 'v1';
        j.$state.go = jest.fn();

        _nocSelectedWorkspaceOrTenantService = nocSelectedWorkspaceOrTenantService;
        jest.spyOn(_nocSelectedWorkspaceOrTenantService, 'getPublicId').mockReturnValue(33);
    }));

    describe('getVpcName', () => {
        it('should return the vpcId()', () => {
            _.set(j.parentScope.wizardController, 'createVpcSelectObjects.vpc', {
                vpcId: '222',
                cidrBlock: '444'
            });
            expect(j.scope.getVPCName()).toEqual('222 (444)');
        });
    });

    describe('createWorkingEnvironment', () => {
        let baseData;
        beforeEach(() => {
            j.scope.setPathToPreviousOrRoot = jest.fn();
            j.scope.wizardController.encryptionType = 'NONE';
            baseData = {
                workingEnvironmentName: 'fred',
                svmUserName: 'user',
                svmPassword: 'pass',
                volumeSkipped: true,
                createVpcSelectObjects: {
                    region: 'us-east',
                    vpc: {vpcId: 1},
                    subnet: {subnetId: 21},
                    securityGroup: {securityGroupId: 'someid'},
                    access: {requestValue: 'key'},
                    key: {name: 'key1'}
                },
                selectedRegion: 'us-east',
                iops: null,
                tenancy: {requestName: 'default'},
                selectedTenant: {publicId: 33},
                selectedCapacitySize: 8,
                selectedLicense: {
                    type: 'explore',
                    instanceType: 'xlarge-12-4e',
                    ontapVersion: 'v1'
                },
                selectedInstanceType: 'xlarge-12-4e'
            };
        });

        describe('vsa flow', () => {
            let vsaURL;
            beforeEach(() => {
                _.assignIn(baseData, {workingEnvironmentType: 'VSA'});
                _.assignIn(j.parentScope, baseData);
                vsaURL = j.nocBaseUrl + '/vsa/working-environments';
                j.scope.wizardController.awsVolumeEncryption = false;
                j.scope.wizardController.ontapEncryption = false;
                j.scope.wizardController.defaultSecurityGroup = {
                    securityGroupId: 'Use automatic Security Group',
                    description: 'Default Cloud Volumes ONTAP Firewall Rules',
                    name: ''
                };
                j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
                j.scope.wizardController.providerVolumeSize = {size: 500, unit: 'GB'};
                j.scope.wizardController.iops = null;
                j.scope.wizardController.selectedPackage = {name: 'package_custom'};
                j.scope.isValid = jest.fn().mockReturnValue(true);
            });

            it('should validate the form', () => {
                j.scope.isValid.mockReturnValue(false);
                j.scope.createWorkingEnvironment();
                expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
            });

            it('should send a vpc descriptor with an id', () => {
                j.scope.wizardController.createVpcSelectObjects.securityGroup = {securityGroupId: 'Use automatic Security Group'};
                j.$httpBackend.expectPOST(vsaURL, {
                    name: 'fred',
                    svmPassword: 'pass',
                    vpcId: 1,
                    volume: undefined,
                    securityGroupId: undefined,
                    region: 'us-east',
                    tenantId: 33,
                    subnetId: 21,
                    vsaMetadata: {licenseType: 'explore', instanceType: 'xlarge-12-4e', ontapVersion: 'v1'},
                    dataEncryptionType: 'NONE',
                    ontapEncryptionParameters: null,
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    capacityTier: undefined,
                    awsTags: [],
                    optimizedNetworkUtilization: false,
                    clusterKeyPairName: 'key1',
                    instanceTenancy: 'default',
                    iops: null,
                    packageName: 'package_custom'
                }).respond(200);

                j.scope.createWorkingEnvironment();
                j.$httpBackend.flush();
                j.expectUItoBlockAndUnblock();
                expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
                expect(j.pidgeon.clear).toHaveBeenCalledWith('create');
            });

            it('should not send security group id if its the default "use automatic bla bla..."', () => {
                j.$httpBackend.expectPOST(vsaURL, {
                    name: 'fred',
                    svmPassword: 'pass',
                    vpcId: 1,
                    volume: undefined,
                    securityGroupId: undefined,
                    region: 'us-east',
                    tenantId: 33,
                    subnetId: 21,
                    vsaMetadata: {licenseType: 'explore', instanceType: 'xlarge-12-4e', ontapVersion: 'v1'},
                    dataEncryptionType: 'NONE',
                    ontapEncryptionParameters: false,
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    clusterKeyPairName: 'key1',
                    instanceTenancy: 'default',
                    iops: null
                });
            });

            it('should add the WE id to the request if in recreate mode', () => {
                j.scope.wizardController.failedRequestType = 'VSA';
                j.scope.wizardController.recreateId = 'id1';

                expect(j.scope.createWERequest()).toEqual(jasmine.objectContaining({failedToCreateWorkingEnvironmentId: 'id1'}));
            });

            it('should add serialNumber if needed', () => {
                j.scope.wizardController.selectedLicense.platformLicenseRequired = true;
                j.scope.wizardController.platformSerialNumber = 123123123;

                expect(j.scope.createWERequest().vsaMetadata.platformSerialNumber).toEqual(123123123);
            });

            it('should pass the security group id if its not the default "use automatic bla bla..."', () => {
                j.$httpBackend.expectPOST(vsaURL, {
                    name: 'fred',
                    svmPassword: 'pass',
                    vpcId: 1,
                    volume: undefined,
                    securityGroupId: 'someid',
                    region: 'us-east',
                    tenantId: 33,
                    subnetId: 21,
                    vsaMetadata: {licenseType: 'explore', instanceType: 'xlarge-12-4e', ontapVersion: 'v1'},
                    dataEncryptionType: 'NONE',
                    ontapEncryptionParameters: null,
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    awsTags: [],
                    optimizedNetworkUtilization: false,
                    clusterKeyPairName: 'key1',
                    instanceTenancy: 'default',
                    iops: null,
                    packageName: 'package_custom'
                }).respond(200, {});

                j.scope.createWorkingEnvironment();

                j.$httpBackend.flush();
                expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
                j.expectUItoBlockAndUnblock();
            });

            it('should pass ontap encryption data like a boss', () => {
                j.scope.wizardController.createVpcSelectObjects.securityGroup = {securityGroupId: 'Use automatic Security Group'};
                j.scope.wizardController.ontapEncryption = {
                    keyManagers: [{publicId: 'pid1', name: 'name1'}, {publicId: 'pid2', name: 'name2'}],
                    keyManagerCaCertificateId: 'certId1'
                };
                j.$httpBackend.expectPOST(vsaURL, {
                    name: 'fred',
                    svmPassword: 'pass',
                    vpcId: 1,
                    volume: undefined,
                    securityGroupId: undefined,
                    region: 'us-east',
                    tenantId: 33,
                    subnetId: 21,
                    vsaMetadata: {licenseType: 'explore', instanceType: 'xlarge-12-4e', ontapVersion: 'v1'},
                    dataEncryptionType: 'NONE',
                    ontapEncryptionParameters: {
                        keyManagerIds: ['pid1', 'pid2'],
                        keyManagerCaCertificateId: 'certId1'
                    },
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    awsTags: [],
                    optimizedNetworkUtilization: false,
                    clusterKeyPairName: 'key1',
                    instanceTenancy: 'default',
                    iops: null,
                    packageName: 'package_custom'
                }).respond(200, {});

                j.scope.createWorkingEnvironment();

                j.$httpBackend.flush();
                expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
                j.expectUItoBlockAndUnblock();
            });

            it('should push error alert when post errors', () => {
                j.$httpBackend.expectPOST(vsaURL, {
                    name: 'fred',
                    svmPassword: 'pass',
                    vpcId: 1,
                    volume: undefined,
                    region: 'us-east',
                    tenantId: 33,
                    subnetId: 21,
                    vsaMetadata: {licenseType: 'explore', instanceType: 'xlarge-12-4e', ontapVersion: 'v1'},
                    dataEncryptionType: 'NONE',
                    ontapEncryptionParameters: null,
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    awsTags: [],
                    optimizedNetworkUtilization: false,
                    clusterKeyPairName: 'key1',
                    instanceTenancy: 'default',
                    iops: null,
                    securityGroupId: 'someid',
                    packageName: 'package_custom'
                }).respond(400, j.errorResponse);

                j.scope.createWorkingEnvironment();

                j.$httpBackend.flush();
                expect(j.scope.setPathToPreviousOrRoot).not.toHaveBeenCalled();
                j.expectUItoBlockAndUnblock();
                expect(j.pidgeon.clear).toHaveBeenCalledWith('create');
                expect(j.pidgeon.error).toHaveBeenCalledWith('An error occurred while creating working environment', j.errorResponse, 'create');
            });

            it('display the right dialog if creation fails due to lack of AMI subscription', () => {
                const amazonError = {
                    message: 'In order to use this AWS Marketplace product you need to accept terms and subscribe. To do so please visit http://aws.amazon.com/marketplace/pp?sku=bk9ifwvjadm70j0qym0ui6msi',
                    failureInfo: 'In order to use this AWS Marketplace product you need to accept terms and subscribe. To do so please visit http://aws.amazon.com/marketplace/pp?sku=bk9ifwvjadm70j0qym0ui6msi',
                    causeMessage: 'NoLicenseForProductException: In order to use this AWS Marketplace product you need to accept terms and subscribe. To do so please visit http://aws.amazon.com/marketplace/pp?sku=bk9ifwvjadm70j0qym0ui6msi'
                };

                j.$httpBackend.expectPOST(vsaURL, {
                    name: 'fred',
                    svmPassword: 'pass',
                    vpcId: 1,
                    volume: undefined,
                    region: 'us-east',
                    tenantId: 33,
                    subnetId: 21,
                    vsaMetadata: {licenseType: 'explore', instanceType: 'xlarge-12-4e', ontapVersion: 'v1'},
                    dataEncryptionType: 'NONE',
                    ontapEncryptionParameters: null,
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    awsTags: [],
                    optimizedNetworkUtilization: false,
                    clusterKeyPairName: 'key1',
                    instanceTenancy: 'default',
                    iops: null,
                    securityGroupId: 'someid',
                    packageName: 'package_custom'
                }).respond(400, amazonError);
                jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {
                });

                j.scope.createWorkingEnvironment();

                j.$httpBackend.flush();

                expect(j.nocDialog.dialog).toHaveBeenCalledWith('Subscribe in Marketplace',
                    'assets/templates/dialogs/include-linky-dialog.html', {bindText: amazonError.failureInfo});
                expect(j.pidgeon.error).not.toHaveBeenCalled();
            });

            describe('volume tests', () => {
                const expectedRequest = {
                    name: 'fred',
                    svmPassword: 'pass',
                    vpcId: 1,
                    volume: {
                        enableThinProvisioning: true,
                        enableDeduplication: true,
                        enableCompression: true,
                        volumeData1: 1,
                        volumeData2: 2,
                        size: {size: 5, unit: 'GB'},
                        exportPolicyInfo: {policyType: 'custom', ips: ['1', '2']}
                    },
                    region: 'us-east',
                    tenantId: 33,
                    subnetId: 21,
                    vsaMetadata: {licenseType: 'explore', instanceType: 'xlarge-12-4e', ontapVersion: 'v1'},
                    dataEncryptionType: 'NONE',
                    ontapEncryptionParameters: null,
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    awsTags: [],
                    optimizedNetworkUtilization: false,
                    securityGroupId: 'someid',
                    instanceTenancy: 'default',
                    iops: null,
                    clusterKeyPairName: 'key1',
                    packageName: 'package_custom'
                };

                beforeEach(() => {
                    j.parentScope.volumeSkipped = false;
                    j.parentScope.volumeProtocol = {type: 'NFS'};
                    j.parentScope.volumeRequest = {
                        sizeGb: 5,
                        volumeData1: 1,
                        volumeData2: 2,
                        usageProfile: j.usageProfiles.default,
                        exportPolicyInfo: {policyType: 'custom', _ips: '1,2'}
                    };
                });

                it('should never send a volume object even if volumeSkipped is false for VSA', () => {
                    j.parentScope.volumeSkipped = false;

                    j.$httpBackend.expectPOST(vsaURL, expectedRequest).respond(200);

                    j.scope.createWorkingEnvironment();

                    j.$httpBackend.flush();
                    expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
                    j.expectUItoBlockAndUnblock();
                });

                it('should send an empty array of ips property if volume export policy type is none', () => {
                    j.parentScope.volumeRequest.exportPolicyInfo = {policyType: 'none', _ips: '1,2'}; // <- filling a non-empty array

                    expectedRequest.volume.exportPolicyInfo = {policyType: 'none', ips: []}; // <- should send an empty array

                    j.$httpBackend.expectPOST(vsaURL, expectedRequest).respond(200);

                    j.scope.createWorkingEnvironment();

                    j.$httpBackend.flush();
                });
            });
        });

        describe('HA AWS Creation', () => {
            let haUrl;
            beforeEach(() => {
                _.assignIn(baseData, {workingEnvironmentType: 'HA', choice: 'HA'});
                _.assignIn(j.parentScope, baseData);
                haUrl = j.nocBaseUrl + '/aws/ha/working-environments';
                j.scope.wizardController.awsVolumeEncryption = false;
                j.scope.wizardController.ontapEncryption = false;
                j.scope.wizardController.defaultSecurityGroup = {
                    securityGroupId: 'Use automatic Security Group',
                    description: 'Default Cloud Volumes ONTAP Firewall Rules',
                    name: ''
                };
                j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
                j.scope.wizardController.providerVolumeSize = {size: 500, unit: 'GB'};
                j.scope.wizardController.iops = null;
                j.scope.wizardController.haParameters = {gp2: {size: 500, unit: 'GB'}};
                j.scope.wizardController.failoverMode = 'FloatingIP';
                j.scope.wizardController.createVpcSelectObjects = {
                    region: {name: 'r1'},
                    vpc: {vpcId: 'v1'},
                    subnet: {subnetId: '21'},
                    node2Subnet: {subnetId: '22'},
                    mediatorSubnet: {subnetId: '23'},
                    securityGroup: {securityGroupId: 'Use automatic Security Group'}
                };
                j.scope.wizardController.haKeySelectObjects = {
                    mediatorKey: {name: 'mediatorKey'},
                    access: {requestValue: 'key'},
                    nodesKey: {name: 'key1'}
                };
                j.scope.wizardController.haParameters.haMediatorSecurityGroup = {
                    securityGroupId: 'Use automatic Security Group'
                };
                j.scope.wizardController.selectedPackage = {name: 'package_ha_custom'};
                j.scope.wizardController.mediatorAccess = {access: {name: 'Public IP address', requestValue: 'public'}};

                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.scope.wizardController.instanceProfileName = 'instanceProfileName';
            });

            it('should use the right tools to build the response', () => {
                const expectedResponse = {
                    svmPassword: 'pass',
                    vpcId: 'v1',
                    ontapEncryptionParameters: null,
                    securityGroupId: undefined,
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {
                        size: 500,
                        unit: 'GB'
                    },
                    haParams: {
                        gp2: {
                            size: 500,
                            unit: 'GB'
                        },
                        node1SubnetId: '21',
                        node2SubnetId: '22',
                        mediatorSubnetId: '23',
                        routeTableIds: ['1', '2'],
                        platformSerialNumberNode1: '1',
                        platformSerialNumberNode2: '2',
                        providedLicenseNode1: 'ABC1',
                        providedLicenseNode2: 'ABC2',
                        failoverMode: 'FloatingIP',
                        mediatorKeyPairName: 'mediatorKey',
                        mediatorAssignPublicIP: true,
                        mediatorSecurityGroupId: undefined

                    },
                    awsTags: [],
                    optimizedNetworkUtilization: false,
                    clusterKeyPairName: 'key1',
                    instanceTenancy: 'default',
                    iops: null,
                    instanceProfileName: 'instanceProfileName',
                    cloudProviderAccount: undefined
                };

                j.scope.wizardController.selectedLicense.platformLicenseRequired = true;
                j.scope.wizardController.platformSerialNumberNode1 = '1';
                j.scope.wizardController.platformSerialNumberNode2 = '2';
                j.scope.wizardController.providedLicenseNode1 = 'ABC1';
                j.scope.wizardController.providedLicenseNode2 = 'ABC2';

                j.scope.wizardController.routeTables = [{id: '1', selected: true}, {id: '2', selected: true}, {
                    id: '3',
                    selected: false
                }];
                jest.spyOn(j.scope, 'buildHAVSAWorkingEnvironmentRequest');
                j.scope.createWorkingEnvironment();
                expect(j.scope.buildHAVSAWorkingEnvironmentRequest).toHaveBeenCalled();
                expect(j.scope.buildHAVSAWorkingEnvironmentRequest()).toEqual(expectedResponse);
            });

            it('should send the request to the right api', () => {
                j.$httpBackend.expectPOST(haUrl).respond(202);
                j.scope.createWorkingEnvironment();
                j.$httpBackend.flush();
            });
        });
    });

    describe('backing up data', () => {
        it('should backup data and clear fields upon destroy', () => {
            j.scope.wizardController.agreeRegister = true;
            j.scope.wizardController.agreeResources = true;
            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('review', {
                agreeRegister: true,
                agreeResources: true
            });
            expect(j.scope.agreeRegister).toEqual(false);
            expect(j.scope.agreeResources).toEqual(false);
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('review');
        });
    });

    describe('getKeyManagersDisplayList', () => {
        it('should return a list of names separated by commas', () => {
            j.scope.wizardController.ontapEncryption = {};
            j.scope.wizardController.ontapEncryption.keyManagers = [];
            expect(j.scope.getKeyManagersDisplayList()).toEqual('');

            j.scope.wizardController.ontapEncryption.keyManagers = [{name: 'name1'}];
            expect(j.scope.getKeyManagersDisplayList()).toEqual('name1');

            j.scope.wizardController.ontapEncryption.keyManagers = [{name: 'name1'}, {name: 'name2'}];
            expect(j.scope.getKeyManagersDisplayList()).toEqual('name1, name2');
        });
    });

    describe('encryptionTypeString', () => {
        it('should return the right string for ontap encryption', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';
            expect(j.scope.encryptionTypeString()).toEqual('ONTAP Managed');
        });

        it('should return the right string for aws encryption', () => {
            j.scope.wizardController.encryptionType = 'AWS';
            expect(j.scope.encryptionTypeString()).toEqual('AWS Managed');
        });

        it('should return the right string for Azure encryption', () => {
            j.scope.wizardController.encryptionType = 'AZURE';
            expect(j.scope.encryptionTypeString()).toEqual('Azure Managed');
        });

        it('should return the right string for no encryption', () => {
            j.scope.wizardController.encryptionType = 'NONE';
            expect(j.scope.encryptionTypeString()).toEqual('None');
        });
    });

    describe('getRegionName', () => {
        it('should return the original name for AWS', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'AWS';
            expect(j.scope.getRegionName('southeastasia')).toEqual('southeastasia');
        });

        it('should convert the name to the display name for Azure', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';
            expect(j.scope.getRegionName('southeastasia')).toEqual('Southeast Asia');
        });

        it('should return the original for a not found azure region', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';
            expect(j.scope.getRegionName('unknown region')).toEqual('unknown region');
        });
    });

    describe('getVnetName', () => {
        it('should return empty string if there is no selected vnet', () => {
            j.scope.wizardController.selectedVnet = null;
            expect(j.scope.getVnetName()).toEqual('');
        });

        it('should build the right name of the vnet', () => {
            j.scope.wizardController.selectedVnet = {name: 'vnet', resourceGroup: 'rg1', cidr: {cidr: '1.1.1.1/24'}};
            expect(j.scope.getVnetName()).toEqual('vnet | rg1 (1.1.1.1/24)');
        });
    });

    describe('getUnderlyingDisks', () => {
        beforeEach(() => {
            j.scope.wizardController.selectedTier = {name: 'gp2', display: 'General Purpose SSD (gp2)'};
            j.scope.wizardController.providerVolumeSize = {size: 1, unit: 'TB', _identifier: '1 TB'};
        });

        it('should return the right value if all defined', () => {
            expect(j.scope.getUnderlyingDisks()).toEqual('General Purpose SSD (gp2), 1 TB');
        });
    });
});
