describe('VirginCreateWorkingEnvironmentAzureVnetStepController', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    const vnets = {
        virtualNetworks: [{
            name: 'blabla',
            id: 'blabla',
            resourceGroup: 'bla',
            cidrs: [
                {
                    cidr: '10.0.0.0/32',
                    subnets: [{
                        id: 'bla',
                        cidr: '10.0.0.1/16',
                        name: 'bla',
                        availableIps: 20
                    }]
                }, {
                    cidr: '11.0.0.0/32',
                    subnets: [{
                        id: 'bla1',
                        cidr: '11.0.0.1/16',
                        name: 'bla1',
                        availableIps: 20
                    }]
                }]
        }],
        securityGroups: [{
            id: 'securityGroup1',
            name: 'securityGroup1',
            resourceGroup: 'bla'
        }]
    };

    const virtualNetworks = [
        {name: 'vnet1', identifier: 'vnet1_1', securityGroups: [{id: 1, name: 'secG1', resourceGroup: 'rg1'}, {id: 2, name: 'secG2', resourceGroup: 'rg2'}], cidr: {cidr: '1', subnets: [{cidr: '1', availableIps: 7}]}},
        {name: 'vnet2', identifier: 'vnet2_2', securityGroups: [{id: 1, name: 'secG1', resourceGroup: 'rg1'}, {id: 2, name: 'secG2', resourceGroup: 'rg2'}], cidr: {cidr: '2', subnets: [{cidr: '2', availableIps: 1}]}},
        {name: 'vnet3', identifier: 'vnet3_3', securityGroups: [{id: 1, name: 'secG1', resourceGroup: 'rg1'}, {id: 2, name: 'secG2', resourceGroup: 'rg2'}], cidr: {cidr: '3', subnets: [{cidr: '1', availableIps: 1}, {cidr: '2', availableIps: 2}]}}
    ];

    const regions = [
        {name: 'region2', displayName: 'region 2', vnets: []},
        {name: 'region1', displayName: 'region 1', vnets}
    ];

    beforeEach(() => {
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentAzureVnetStepController', {}, {
            currentUser: {},
            pathPrefix: '/azure/vsa/',
            selectedAzureSubscription: {name: 'dev', subscriptionId: '123'},
            cloudProviderAccountId: 'cloudProviderAccountId'
        });
    });

    describe('initialize', () => {
        beforeEach(() => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider = 'first';
            jest.spyOn(j.scope, 'clearAllModelValues').mockImplementation(() => {});
            jest.spyOn(j.scope, 'getNetworkRequirements').mockReturnValue(j.$q.when());
            jest.spyOn(j.scope, 'getAzureRegions').mockImplementation(() => {});
        });

        it('should not really do anything if we initialized with this kind of working environment', () => {
            j.scope.wizardController.vnetBeenHereBefore = 'first';
            j.scope.initialize();
            expect(j.scope.clearAllModelValues).not.toHaveBeenCalled();
            expect(j.scope.getAzureRegions).not.toHaveBeenCalled();
            expect(j.scope.getNetworkRequirements).not.toHaveBeenCalled();
        });

        it('should continue if this working environment was not selected before', () => {
            j.scope.wizardController.vpcBeenHereBefore = 'second';
            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.clearAllModelValues).toHaveBeenCalled();
            expect(j.scope.getNetworkRequirements).toHaveBeenCalled();
            expect(j.scope.getAzureRegions).toHaveBeenCalled();
        });
    });

    describe('selectSubnetAndSecurityGroup', () => {
        beforeEach(() => {
            j.scope.wizardController.selectedRegionObject = {vnets: virtualNetworks};
            j.scope.wizardController.securityGroups = [{id: 1}, {id: 2}];
            j.scope.wizardController.networkRequirements = {node: 6};
            j.scope.wizardController.useExistingSecurityGroup = false;
        });

        it('should select the first subnet if there is only one with enough ips', () => {
            j.scope.wizardController.selectedVnet = _.find(virtualNetworks, {identifier: 'vnet1_1'});
            j.scope.wizardController.useExistingSecurityGroup = true;
            j.scope.selectSubnetAndSecurityGroup();
            expect(j.scope.wizardController.selectedVnet).toEqual(j.scope.wizardController.selectedRegionObject.vnets[0]);
            expect(j.scope.wizardController.selectedSubnet).toEqual({cidr: '1', availableIps: 7});
            expect(j.scope.wizardController.selectedSecurityGroup).toEqual({id: 1, name: 'secG1', resourceGroup: 'rg1'});
        });

        it('should set subnet to null if there is only one subnet with not enough ips', () => {
            j.scope.wizardController.selectedVnet = _.find(virtualNetworks, {identifier: 'vnet2_2'});
            j.scope.selectSubnetAndSecurityGroup();
            expect(j.scope.wizardController.selectedSubnet).toBeNull();
        });

        it('should set subnet to null if there is more than one subnet', () => {
            j.scope.wizardController.selectedVnet = _.find(virtualNetworks, {identifier: 'vnet3_3'});
            j.scope.selectSubnetAndSecurityGroup();
            expect(j.scope.wizardController.selectedSubnet).toBeNull();
        });

        it('should use a generated security group by default', () => {
            j.scope.wizardController.defaultSecurityGroup = {
                id: 'Use a generated security group',
                name: 'Use a generated security group',
                resourceGroup: ''
            };
            j.scope.wizardController.selectedVnet = _.find(virtualNetworks, {identifier: 'vnet1_1'});
            j.scope.selectSubnetAndSecurityGroup();
            expect(j.scope.wizardController.selectedSecurityGroup).toEqual({id: 'Use a generated security group', name: 'Use a generated security group', resourceGroup: ''});
        });
    });

    describe('handleRegionChange', () => {
        it('should do nothing if the region is not set', () => {
            j.scope.lastSelectedRegion = 'not null';

            j.scope.handleRegionChange();

            expect(j.scope.lastSelectedRegion).toEqual('not null');
        });

        it('should do nothing if the region was not changed', () => {
            j.scope.lastSelectedRegion = 'region1';
            j.scope.wizardController.selectedVnet = 'not null';

            j.scope.handleRegionChange({name: 'region1'});

            expect(j.scope.wizardController.selectedRegion).toEqual('region1');
            expect(j.scope.wizardController.selectedVnet).toEqual('not null');
        });

        it('update the last selected region, clear all the vnets related information and get vents again', () => {
            j.scope.lastSelectedRegion = 'region2';
            j.scope.wizardController.selectedVnet = 'not null';
            j.scope.wizardController.selectedSubnet = 'not null';
            j.scope.wizardController.selectedSecurityGroup = 'not null';

            j.scope.handleRegionChange({name: 'region1'});

            expect(j.scope.selectedRegion).toEqual('region1');
            expect(j.scope.lastSelectedRegion).toEqual('region1');
            expect(j.scope.wizardController.selectedVnet).toBeNull();
            expect(j.scope.wizardController.selectedSubnet).toBeNull();
            expect(j.scope.wizardController.selectedSecurityGroup).toBeNull();
        });

        it('should automatically select VNet if only one exists and has subnets', () => {
            jest.spyOn(j.scope, 'selectSubnetAndSecurityGroup').mockImplementation(() => {});

            j.scope.handleRegionChange({name: 'region1', vnets: [{identifier: 'Vnet1', cidr: {subnets: [{id: 'subnet1'}]}}]});

            expect(j.scope.wizardController.selectedVnet).toEqual({identifier: 'Vnet1', cidr: {subnets: [{id: 'subnet1'}]}});
            expect(j.scope.selectSubnetAndSecurityGroup).toHaveBeenCalled();
        });

        it('should not automatically select VNet if only one exists but has no subnets', () => {
            jest.spyOn(j.scope, 'selectSubnetAndSecurityGroup').mockImplementation(() => {});

            j.scope.handleRegionChange({name: 'region1', vnets: [{identifier: 'Vnet1', cidr: {subnets: []}}]});

            expect(j.scope.wizardController.selectedVnet).toEqual(null);
            expect(j.scope.selectSubnetAndSecurityGroup).not.toHaveBeenCalled();
        });

        it('should not automatically select VNet if there is more than one', () => {
            jest.spyOn(j.scope, 'selectSubnetAndSecurityGroup').mockImplementation(() => {});

            j.scope.handleRegionChange({name: 'region1', vnets: [{identifier: 'Vnet1'}, {identifier: 'Vnet2'}]});

            expect(j.scope.wizardController.selectedVnet).toEqual(null);
            expect(j.scope.selectSubnetAndSecurityGroup).not.toHaveBeenCalled();
        });
    });

    describe('backing up data', () => {
        it('should backup data and clear fields upon destroy', () => {
            j.scope.wizardController.vnetBeenHereBefore = 'aa';
            j.scope.wizardController.selectedRegionObject = 'a';
            j.scope.wizardController.selectedRegion = 'c';
            j.scope.regions = ['region-code-1', 'region-code-2', 'region-code-3'];
            j.scope.wizardController.selectedVnet = 'd';
            j.scope.wizardController.selectedSecurityGroup = 'f';
            j.scope.wizardController.selectedSubnet = 'h';

            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('vnet', {
                vnetBeenHereBefore: 'aa',
                selectedRegionObject: 'a',
                selectedRegion: 'c',
                regions: ['region-code-1', 'region-code-2', 'region-code-3'],
                selectedVnet: 'd',
                selectedSecurityGroup: 'f',
                selectedSubnet: 'h'
            });
            expect(j.scope.wizardController.selectedRegionObject).toBeNull();
            expect(j.scope.wizardController.selectedRegion).toBeNull();
            expect(j.scope.wizardController.selectedVnet).toBeNull();
            expect(j.scope.wizardController.selectedSecurityGroup).toBeNull();
            expect(j.scope.wizardController.selectedSubnet).toBeNull();
        });

        it('should restore backed up data upon creation', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('vnet');
        });
    });

    describe('reconstruct vnets', () => {
        it('should reconstruct the vnets correctly so that each vnet that has more than one cidr will be duplicated for that cidr', () => {
            expect(j.scope.reconstructVnets(vnets)).toEqual([
                {
                    name: 'blabla',
                    identifier: 'blabla_10.0.0.0/32',
                    vnetCidr: '10.0.0.0/32',
                    id: 'blabla',
                    resourceGroup: 'bla',
                    cidr: {
                        cidr: '10.0.0.0/32',
                        subnets: [
                            {
                                id: 'bla',
                                cidr: '10.0.0.1/16',
                                name: 'bla',
                                'availableIps': 20
                            }
                        ]
                    },
                    subnets: 1,
                    securityGroups: [
                        {
                            id: 'securityGroup1',
                            name: 'securityGroup1',
                            resourceGroup: 'bla'
                        }
                    ]
                },
                {
                    name: 'blabla',
                    identifier: 'blabla_11.0.0.0/32',
                    vnetCidr: '11.0.0.0/32',
                    id: 'blabla',
                    resourceGroup: 'bla',
                    cidr: {
                        cidr: '11.0.0.0/32',
                        subnets: [
                            {
                                id: 'bla1',
                                cidr: '11.0.0.1/16',
                                name: 'bla1',
                                'availableIps': 20
                            }
                        ]
                    },
                    subnets: 1,
                    securityGroups: [
                        {
                            id: 'securityGroup1',
                            name: 'securityGroup1',
                            resourceGroup: 'bla'
                        }
                    ]
                }
            ]);
        });
    });

    describe('getAzureRegions', () => {
        let getAzureRegionsSpy;

        beforeEach(inject((MetadataResource) => {
            getAzureRegionsSpy = jest.spyOn(MetadataResource, 'getRegions').mockReturnValue(j.$q.when(regions));
            jest.spyOn(j.scope, 'handleRegionChange').mockImplementation(() => {});
            jest.spyOn(j.scope, 'reconstructVnets').mockImplementation((vnets) => {
                if (_.isEmpty(vnets)) return [];
                else return ['VNet1', 'VNet2'];
            });
        }));

        it('should call handleRegionChange and update the regions upon success, and select the first region that has vnets', () => {
            j.scope.getAzureRegions();
            magnifispec.processPromise();

            expect(j.scope.reconstructVnets).toHaveBeenCalled();
            expect(j.scope.handleRegionChange).toHaveBeenCalledWith(jasmine.objectContaining({name: 'region1'}));

            expect(j.scope.regions).toEqual([
                {
                    name: 'region2',
                    displayName: 'region 2',
                    vnets: []
                },
                {
                    name: 'region1',
                    displayName: 'region 1',
                    vnets: ['VNet1', 'VNet2']
                }
            ]);
        });

        it('should not update the selected region if the list of regions is empty', () => {
            getAzureRegionsSpy.mockReturnValue(j.$q.when([]));

            j.scope.getAzureRegions();
            magnifispec.processPromise();

            expect(j.scope.handleRegionChange).toHaveBeenCalledWith(undefined);
            expect(j.scope.reconstructVnets).not.toHaveBeenCalled();
            expect(j.scope.regions).toEqual([]);
            expect(j.scope.wizardController.selectedRegion).not.toBeDefined();
        });

        afterEach(() => {
            expect(getAzureRegionsSpy).toHaveBeenCalledWith(jasmine.any(Object), '/azure/vsa/', true, 'subscriptionId=123', 'cloudProviderAccountId');
        });
    });

    describe('pluralize string', () => {
        it('should return no subnets when the length is 0', () => {
            expect(j.scope.pluralizeString(0, ' subnet')).toEqual('No subnets');
        });

        it('should return the singular string when length is 1', () => {
            expect(j.scope.pluralizeString(1, ' subnet')).toEqual('1 subnet');
        });

        it('should return the plural string when length is 1', () => {
            expect(j.scope.pluralizeString(5, ' subnet')).toEqual('5 subnets');
        });
    });

    describe('subnetTitle', () => {
        beforeEach(() => {
            j.scope.wizardController.networkRequirements = {node: 10};
        });

        it('should return correct title when has available IPs but not enough', () => {
            expect(j.scope.subnetTitle({cidr: '1.2.3.4/16', availableIps: 7})).toEqual('The working environment requires 10 IPs, but the subnet only has 7 available.');
        });

        it('should return correct title when does not have available IPs at all', () => {
            expect(j.scope.subnetTitle({cidr: '1.2.3.4/16', availableIps: 0})).toEqual('The working environment requires 10 IPs, but the subnet has none available.');
        });

        it('should return the cidr if has enough available IPs', () => {
            expect(j.scope.subnetTitle({cidr: '1.2.3.4/16', availableIps: 20})).toEqual('1.2.3.4/16');
            expect(j.scope.subnetTitle({cidr: '1.2.3.4/16', availableIps: 10})).toEqual('1.2.3.4/16');
        });
    });
});
