describe("Cluster Step Controller", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(function () {
        magnifispec.mockWizardStepController("VirginOnPremClusterStepController");
        j.scope.wizardController.clusterAddress = "1.1.1.1";

        j.scope.wizardController.clusterPassword = "pass";
        j.scope.wizardController.workingEnvironmentName = "workingEnvName";
        j.scope.wizardController.dataONTAPLocation = "Azure";
    });

    describe("verifyCredentials", function () {

        beforeEach(function () {
            j.$state.go = jest.fn();
        });

        it("initialization", function () {
            expect(j.BaseVirginStep.call).toHaveBeenCalled();
            expect(j.scope.wizardController.title).toEqual("Discover ONTAP Cluster");
            expect(j.scope.wizardController.clusterUserName).toEqual("admin");
        });

        it("should clear discovery error and verified state, and when form is not valid call handle form validity", function () {
            j.scope.isValid = jest.fn().mockReturnValue(false);
            j.scope.verifyCredentials();
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
        });

        it("should call verify with the right parameters and call pidgeon success and updated verified upon success response", function () {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.$httpBackend.expectGET(j.nocBaseUrl + "/onprem/metadata/cluster-info", {"clusterAddress":"1.1.1.1","clusterUsername":"admin","clusterPassword":"pass","Accept":"application/json","If-Modified-Since":"0","Cache-Control":"no-cache","Pragma":"no-cache"}).respond(200, {clusterName: "cluster1", serialNumber: "1"});
            j.scope.verifyCredentials();
            j.$httpBackend.flush();

            expect(j.scope.wizardController.workingEnvironmentName).toEqual("cluster1");
            expect(j.$state.go).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it("should call pidgeon error upon error response", function () {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.$httpBackend.expectGET(j.nocBaseUrl + "/onprem/metadata/cluster-info", {"clusterAddress":"1.1.1.1","clusterUsername":"admin","clusterPassword":"pass","Accept":"application/json","If-Modified-Since":"0","Cache-Control":"no-cache","Pragma":"no-cache"}).respond(400, j.errorResponse);
            j.scope.verifyCredentials();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
            expect(j.$state.go).not.toHaveBeenCalled();
        });

    });

    it("should backup data and clear fields upon destroy", function () {
        j.$rootScope.$broadcast('$destroy');
        expect(j.parentScope.backupStepData).toHaveBeenCalledWith("cluster", {
            clusterAddress: "1.1.1.1",
            clusterUserName: "admin",
            clusterPassword: "pass",
            workingEnvironmentName: "workingEnvName",
            dataONTAPLocation: "Azure"
        });
        expect(j.scope.clusterAddress).toEqual("");
        expect(j.scope.clusterUserName).toEqual("");
        expect(j.scope.clusterPassword).toEqual("");
        expect(j.scope.workingEnvironmentName).toEqual("");
        expect(j.scope.dataONTAPLocation).toEqual("");

    });

    it("should restore backed up data upon creation ", function () {
        expect(j.parentScope.getStepData).toHaveBeenCalledWith("cluster");
    });

    it("should initialize the locations properly", function () {
        expect(j.scope.dataONTAPLocations).toEqual([
            {value: "ON_PREM", displayName: "On Premises", providerName:"", location:"On Premises", description:"", shortName: "On-Prem"},
            {value: "AWS", displayName: "AWS", providerName: "AWS", location:"AWS", description:"Web Services", shortName: "AWS"},
            {value: "AZURE", displayName: "Azure", providerName: "Azure", location:"Azure", description:"Microsoft", shortName: "Azure"},
            {value: "SOFTLAYER", displayName: "IBM Cloud", providerName: "IBM", location:"IBM", description:"Cloud", shortName: "IBM"},
            {value: "GOOGLE", displayName: "Google", providerName: "Google", location:"Google", description:"Cloud Platform", shortName: "Google"}
        ]);
    });
});