describe('Details Step Controller', function () {
    'use strict';

    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    let request, _nocSelectedWorkspaceOrTenantService;

    beforeEach(inject((nocSelectedWorkspaceOrTenantService) => {
        magnifispec.mockWizardStepController('VirginOnPremDetailsStepController');
        j.scope.detailsForm = {$error: {}};
        request = {
            tenantId: 'User1',
            name: 'value2',
            clusterAddress: 'value3',
            clusterUserName: 'value4',
            clusterPassword: 'value5',
            location: 'value6'
        };

        _nocSelectedWorkspaceOrTenantService = nocSelectedWorkspaceOrTenantService;
        jest.spyOn(_nocSelectedWorkspaceOrTenantService, 'getPublicId').mockReturnValue('User1');
    }));

    it('should initialize correctly', function () {
        expect(j.scope.weNameReadonly).toEqual(true);
        expect(j.scope.weNameRegex).toBeNull();
    });


    describe('buildDiscoverRequest', function () {
        it('should return the right values', function () {
            j.scope.selectedTenant = {_pathPrefix: '/vsa/', publicId: 'User1'};
            j.scope.wizardController.workingEnvironmentName = 'value2';
            j.scope.wizardController.clusterAddress = 'value3';
            j.scope.wizardController.clusterUserName = 'value4';
            j.scope.wizardController.clusterPassword = 'value5';
            j.scope.wizardController.dataONTAPLocation = 'value6';

            expect(j.scope.buildDiscoverRequest()).toEqual(request);
        });

    });

    describe('discoverOnPrem', function () {
        beforeEach(function () {
            jest.spyOn(j.scope, 'buildDiscoverRequest').mockReturnValue(request);
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.scope.setPathToPreviousOrRoot = jest.fn();
        });

        it('should work upon success', function () {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/onprem/working-environments', request).respond(200, {});
            j.scope.discoverOnPrem();
            j.$httpBackend.flush();

            expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
        });

        it('should work upon error', function () {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/onprem/working-environments', request).respond(400, j.errorResponse);
            j.scope.discoverOnPrem();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.scope.setPathToPreviousOrRoot).not.toHaveBeenCalled();
        });

        afterEach(function () {
            expect(j.scope.buildDiscoverRequest).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });
    });

    describe('validity check', function () {
        it('should call handle form validity if the form is not valid', function () {
            j.scope.isValid = jest.fn().mockReturnValue(false);
            jest.spyOn(j.scope, 'buildDiscoverRequest').mockImplementation(() => {
            });

            j.scope.discoverOnPrem();
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
            expect(j.scope.buildDiscoverRequest).not.toHaveBeenCalled();
        });
    });

    describe('backing up data', function () {
        it('should backup data and clear fields upon destroy', function () {
            j.$rootScope.$broadcast('$destroy');
        });

        it('should restore backed up data upon creation ', function () {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('details');
        });

    });

});