angular.module('noc').component('activateFpolicyComponent', {
    bindings: {
        isActiveAction: '<',
        workingEnvironment: '<',
        isAsyncOperationError: '<'
    },
    templateUrl: 'assets/templates/components/ransomware-protection/activate-fpolicy-component.html',
    controllerAs: 'ctrlActivateFpolicy',
    controller: function ($scope, nocDialog) {
        this.$onInit = () => {
            this.isButtonDisabled = this.isActiveAction || false;
            this.isActivatingFinished = false;
            this.fileExtensionBL = _.join(_.get(this.workingEnvironment, 'fpolicyProperties.fileExtensions'), ', ');

            // FpolicyProperties.fPolicyProtocolStatus.cifs === 1 -> fpolicy for cifs is activated, fpolicyProperties.fPolicyProtocolStatus.cifs === -1 fpolicy for cifs is N/A
            this.needCifsActivation = _.get(this.workingEnvironment, 'fpolicyProperties.fPolicyProtocolStatus.cifs') === 0;
            // FpolicyProperties.status === 1 -> fpolicy for nfs is activated, fpolicyProperties.status === -1 fpolicy for nfs is N/A
            this.needNfsActivation = _.get(this.workingEnvironment, 'fpolicyProperties.status') === 0;

            this.showText = !this.needNfsActivation && !this.needCifsActivation;

            this.objectForPopupReadMore = {
                fileExtensionBL: this.fileExtensionBL,
                showText: this.showText
            };

            this.buttonTitle = 'Activate FPolicy' + (this.needCifsActivation && !this.needNfsActivation ? ' on CIFS Server' : '');
        };


        this.$onChanges = (changesObj) => {
            if (changesObj.isAsyncOperationError && changesObj.isAsyncOperationError.currentValue) {
                this.isActivatingFinished = true;
                this.isButtonDisabled = false;
            }

            if (changesObj.isActiveAction && changesObj.isActiveAction.previousValue && !changesObj.isActiveAction.currentValue) {
                this.isActivatingFinished = true;
            }

            if (this.isButtonDisabled && !this.isActiveAction) {
                this.isButtonDisabled = false;
            }

            if (this.deactivate) {
                this.deactivate = false;
            }
        };

        this.$doCheck = () => {
            this.needCifsActivation = _.get(this.workingEnvironment, 'fpolicyProperties.fPolicyProtocolStatus.cifs') === 0;
            this.needNfsActivation = _.get(this.workingEnvironment, 'fpolicyProperties.status') === 0;
            this.buttonTitle = 'Activate FPolicy' + (this.needCifsActivation && !this.needNfsActivation ? ' on CIFS Server' : '');

            this.objectForPopupReadMore.fileExtensionBL = _.join(_.get(this.workingEnvironment, 'fpolicyProperties.fileExtensions'), ', ');
            this.objectForPopupReadMore.showText = !this.needNfsActivation && !this.needCifsActivation;
        };

        this.activateClicked = () => {
            if (this.isButtonDisabled) return;
            this.isButtonDisabled = true;
            setTimeout(() => {// To allow minimum time for the user to see the progress circle
                $scope.$emit('activateFpolicy');
            }, 2000);
        };

        this.openDeactivateDialog = () => {
            nocDialog.confirm('', 'assets/templates/dialogs/deactivate-fpolicy-dialog.html', {
                width: 650
            }).then(() => {
                $scope.$emit('disableFpolicy');
                this.deactivate = true;
                this.isButtonDisabled = this.isActiveAction || false;
                this.isActivatingFinished = false;
            });
        };
    }
});
