angular.module('noc').component('activateSnapshotComponent', {
    bindings: {
        isActiveAction: '<',
        volumes: '<',
        isAsyncOperationError: '<'
    },
    templateUrl: 'assets/templates/components/ransomware-protection/activate-snapshot-component.html',
    controllerAs: 'ctrlActivateSnapshots',
    controller: function ($scope, $timeout, ransomwareProtectionResource) {
        this.$onInit = () => {
            this.isButtonDisabled = this.isActiveAction || false;
            this.countVolumesNoSnapshotBeforeActivate = undefined;
            this.isActivatingFinished = false;
            this.errorMessage = '';
        };

        this.$onChanges = (changesObj) => {
            if (changesObj.isAsyncOperationError && changesObj.isAsyncOperationError.currentValue) {
                this.isActivatingFinished = true;
                this.isButtonDisabled = false;
            }

            if (changesObj.isActiveAction && changesObj.isActiveAction.previousValue && !changesObj.isActiveAction.currentValue) {
                this.isActivatingFinished = true;
            }
            if (changesObj.volumes) {
                if (!this.isActiveAction) {
                    this.updateData(this.volumes);
                }
                if (this.isButtonDisabled && !this.isActiveAction) {
                    this.isButtonDisabled = false;
                }
            }
        };

        this.activateClicked = () => {
            if (this.isButtonDisabled) return;
            this.errorMessage = '';
            this.countVolumesNoSnapshotBeforeActivate = ransomwareProtectionResource.countVolumesNoSnapshot(ransomwareProtectionResource.getEligibleVolumes(this.volumes));
            this.isActivatingFinished = false;
            this.isButtonDisabled = true;
            $timeout(() => {// To allow minimum time for the user to see the progress circle
                $scope.$emit('activateSnapshot');
            }, 2000);
        };

        this.updateData = (volumes) => {
            this.eligibleVolumes = ransomwareProtectionResource.getEligibleVolumes(volumes);
            if (this.eligibleVolumes && this.eligibleVolumes.length > 0) {
                this.volumesNoSnapshotCount = ransomwareProtectionResource.countVolumesNoSnapshot(this.eligibleVolumes);
                const percentageVolumesProtected = Math.floor((this.eligibleVolumes.length - this.volumesNoSnapshotCount) / this.eligibleVolumes.length * 100);
                this.chartMainTitle = `${percentageVolumesProtected} %`;
                this.chartData = [
                    {name: (this.eligibleVolumes.length - this.volumesNoSnapshotCount) + ' protected volumes', y: percentageVolumesProtected},
                    {name: this.volumesNoSnapshotCount + ' unprotected volumes', y: (100 - percentageVolumesProtected)}
                ];
                if (this.volumesNoSnapshotCount !== 0 && this.isActivatingFinished) {
                    this.errorMessage = 'Unable to activate Snapshot Policy on ' +
                                        ((this.countVolumesNoSnapshotBeforeActivate === this.volumesNoSnapshotCount) ? 'these' : this.volumesNoSnapshotCount) +
                                        ' volumes. Please try again later.';
                }
            }
        };
    }
});
