describe('Ransomware Protection Resource', () => {
    let ransomwareProtectionResource;
    let volumes;

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(inject((_ransomwareProtectionResource_) => {
        ransomwareProtectionResource = _ransomwareProtectionResource_;
        volumes = [
            {
                name: 'vol1',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: 'default'
            },
            {
                name: 'vol2',
                volumeType: 'r',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: 'none'
            },
            {
                name: 'vol3',
                volumeType: 'rw',
                rootVolume: true,
                moving: false,
                state: 'online',
                snapshotPolicy: 'default'
            },
            {
                name: 'vol4',
                volumeType: 'rw',
                rootVolume: false,
                moving: true,
                state: 'online',
                snapshotPolicy: 'default'
            },
            {
                name: 'vol5',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'off',
                snapshotPolicy: 'none'
            },
            {
                name: 'vol6',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: 'none'
            },
            {
                name: 'vol7',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: 'none'
            },
            {
                name: 'vol8',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: '111'
            }];
    }));

    describe('getEligibleVolumes', () => {
        it('should filter out non-eligible volumes', () => {
            const eligibleVolumes = ransomwareProtectionResource.getEligibleVolumes(volumes);

            expect(eligibleVolumes).toEqual([
                {
                    name: 'vol1',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'default'
                },
                {
                    name: 'vol6',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'none'
                },
                {
                    name: 'vol7',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'none'
                },
                {
                    name: 'vol8',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: '111'
                }
            ]);
        });

        it('should return empty array if the input is empty array', () => {
            const eligibleVolumes = ransomwareProtectionResource.getEligibleVolumes([]);

            expect(eligibleVolumes).toEqual([]);
        });

        it('should return empty array if all input are non-eligible volumes', () => {
            volumes = [
                {
                    name: 'vol2',
                    volumeType: 'r',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'none'
                },
                {
                    name: 'vol3',
                    volumeType: 'rw',
                    rootVolume: true,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'default'
                },
                {
                    name: 'vol4',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: true,
                    state: 'online',
                    snapshotPolicy: 'default'
                },
                {
                    name: 'vol5',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'offline',
                    snapshotPolicy: 'default'
                }];
            const eligibleVolumes = ransomwareProtectionResource.getEligibleVolumes(volumes);

            expect(eligibleVolumes).toEqual([]);
        });
    });

    describe('countVolumesNoSnapshot', () => {
        it('should count the volumes with snapshot policy of none', () => {
            const count = ransomwareProtectionResource.countVolumesNoSnapshot(volumes);

            expect(count).toEqual(4);
        });

        it('should return 0 for empty volumes array', () => {
            const count = ransomwareProtectionResource.countVolumesNoSnapshot([]);

            expect(count).toEqual(0);
        });
    });
});
